// Koka generated module: "std/text/regex", koka version: 2.4.2, platform: 64-bit
#include "std_text_regex.h"
/*---------------------------------------------------------------------------
  Copyright 2021, Microsoft Research, Daan Leijen.

  This is free software; you can redistribute it and/or modify it under the
  terms of the Apache License, Version 2.0. A copy of the License can be
  found in the LICENSE file at the root of this distribution.
---------------------------------------------------------------------------*/

#pragma clang diagnostic ignored "-Wdisabled-macro-expansion"
#define PCRE2_CODE_UNIT_WIDTH 8
#define PCRE2_STATIC
#include <pcre2.h>

/* -----------------------------------------------------------------------
  Init/Done
------------------------------------------------------------------------*/
#define KK_CUSTOM_INIT  kk_regex_custom_init
#define KK_CUSTOM_DONE  kk_regex_custom_done

static pcre2_general_context* gen_ctx;
static pcre2_compile_context* cmp_ctx;
static pcre2_match_context*   match_ctx; 

static void* kk_pcre2_malloc( PCRE2_SIZE size, void* data ) {
  return kk_malloc( kk_to_ssize_t(size), kk_get_context() );  
}
static void kk_pcre2_free( void* p, void* data ) {
  kk_free(p,kk_get_context());
}

static void kk_regex_custom_init( kk_context_t* ctx ) {
  gen_ctx = pcre2_general_context_create( &kk_pcre2_malloc, &kk_pcre2_free, NULL );
  if (gen_ctx != NULL) {
    match_ctx = pcre2_match_context_create( gen_ctx );
    cmp_ctx = pcre2_compile_context_create( gen_ctx );
    if (cmp_ctx != NULL) {
      pcre2_set_newline( cmp_ctx, PCRE2_NEWLINE_ANYCRLF );
      pcre2_set_bsr( cmp_ctx, PCRE2_BSR_ANYCRLF );
    }
  }
}

static void kk_regex_custom_done( kk_context_t* ctx ) {
  if (cmp_ctx != NULL) {
    pcre2_compile_context_free(cmp_ctx);
    cmp_ctx = NULL;
  }
  if (match_ctx != NULL) {
    pcre2_match_context_free(match_ctx);
    match_ctx = NULL;
  }
  if (gen_ctx != NULL) {
    pcre2_general_context_free(gen_ctx);
    gen_ctx = NULL;
  }
}


/* -----------------------------------------------------------------------
  Compile
------------------------------------------------------------------------*/

static void kk_regex_free( void* pre, kk_block_t* b, kk_context_t* ctx ) {
  kk_unused(ctx);
  pcre2_code* re = (pcre2_code*)pre;
  //kk_info_message( "free regex at %p\n", re );
  if (re != NULL) { pcre2_code_free(re); }
}

#define KK_REGEX_OPTIONS  (PCRE2_ALT_BSUX | PCRE2_EXTRA_ALT_BSUX | PCRE2_MATCH_UNSET_BACKREF /* javascript compat */ \
                          | PCRE2_NEVER_BACKSLASH_C | PCRE2_NEVER_UCP | PCRE2_UTF /* utf-8 safety */ \
                          )

static kk_box_t kk_regex_create( kk_string_t pat, bool ignore_case, bool multi_line, kk_context_t* ctx ) {
  kk_ssize_t len;
  const uint8_t* cpat = kk_string_buf_borrow( pat, &len, ctx );
  PCRE2_SIZE errofs = 0;
  int        errnum = 0;
  uint32_t   options = KK_REGEX_OPTIONS;
  if (ignore_case) options |= PCRE2_CASELESS;
  if (multi_line)  options |= PCRE2_MULTILINE;
  pcre2_code* re = pcre2_compile( cpat, PCRE2_ZERO_TERMINATED, options, &errnum, &errofs, cmp_ctx);
  //kk_info_message( "create regex: err:%i, at %p\n", (re==NULL ? 0 : errnum), re );
  kk_string_drop(pat,ctx);
  return kk_cptr_raw_box( &kk_regex_free, re, ctx );
}


/* -----------------------------------------------------------------------
  Match
------------------------------------------------------------------------*/

/*
static void kk_match_data_free( void* pmd, kk_block_t* b ) {
  kk_info_message( "free match_data\n" );
  pcre2_match_data* md = (pcre2_match_data*)pmd;
  if (md != NULL) pcre2_match_data_free(md);
}
*/

static kk_std_core__list kk_regex_exec_ex( pcre2_code* re, pcre2_match_data* match_data, 
                                           kk_string_t str_borrow, const uint8_t* cstr, kk_ssize_t len, bool allow_empty, 
                                           kk_ssize_t start, kk_ssize_t* mstart, kk_ssize_t* end, int* res, kk_context_t* ctx ) 
{
  // match
  kk_std_core__list hd  = kk_std_core__new_Nil(ctx);
  uint32_t options = 0;
  if (!allow_empty) options |= (PCRE2_NOTEMPTY_ATSTART | PCRE2_ANCHORED);
  int rc = pcre2_match( re, cstr, len, start, options, match_data, match_ctx );
  if (res != NULL) *res = rc;    
  if (rc > 0) {    
    // extract captures
    uint32_t    gcount = pcre2_get_ovector_count(match_data);
    PCRE2_SIZE* groups = pcre2_get_ovector_pointer(match_data);
    for( uint32_t i = gcount; i > 0; ) {
      i--;
      kk_ssize_t sstart = groups[i*2];       // on no-match, sstart and send == -1.
      kk_ssize_t send   = groups[i*2 + 1];
      kk_assert(send >= sstart);
      kk_std_core__sslice sslice = kk_std_core__new_Sslice( kk_string_dup(str_borrow,ctx), kk_integer_from_ssize_t(sstart,ctx), kk_integer_from_ssize_t(send - sstart,ctx), ctx ); 
      hd = kk_std_core__new_Cons(kk_reuse_null,0,kk_std_core__sslice_box(sslice,ctx), hd, ctx);
      if (i == 0) {
        if (mstart != NULL) { *mstart = sstart; }
        if (end    != NULL) { *end = send; }
      }
    }
  }
  return hd;
}


static kk_std_core__list kk_regex_exec( kk_box_t bre, kk_string_t str, kk_ssize_t start, kk_context_t* ctx ) 
{
  // unpack
  pcre2_match_data* match_data = NULL;
  kk_std_core__list res = kk_std_core__new_Nil(ctx);
  pcre2_code* re = (pcre2_code*)kk_cptr_raw_unbox_borrowed(bre,ctx);
  kk_ssize_t len = 0;
  const uint8_t* cstr = NULL;
  if (re == NULL) goto done;    
  match_data = pcre2_match_data_create_from_pattern(re, gen_ctx);
  if (match_data==NULL) goto done;  
  cstr = kk_string_buf_borrow(str, &len, ctx );  

  // and match
  res = kk_regex_exec_ex( re, match_data, str, cstr, len, true, start, NULL, NULL, NULL, ctx );

done:  
  if (match_data != NULL) {
    //kk_info_message( "free match data: %p\n", match_data );
    pcre2_match_data_free(match_data);
  }
  kk_string_drop(str,ctx);
  kk_box_drop(bre,ctx);
  return res;
}

static kk_std_core__list kk_regex_exec_all( kk_box_t bre, kk_string_t str, kk_ssize_t start, kk_ssize_t atmost, kk_context_t* ctx ) {
  // unpack
  if (atmost < 0) atmost = KK_SSIZE_MAX;
  pcre2_match_data* match_data = NULL;
  kk_std_core__list res = kk_std_core__new_Nil(ctx);
  pcre2_code* re = (pcre2_code*)kk_cptr_raw_unbox_borrowed(bre,ctx);
  if (re == NULL) goto done;    
  match_data = pcre2_match_data_create_from_pattern(re, gen_ctx);
  if (match_data==NULL) goto done;  
  {
    kk_ssize_t len;
    const uint8_t* cstr = kk_string_buf_borrow(str, &len, ctx );  

    // and match
    kk_std_core__list* tail = NULL;
    bool allow_empty = true;
    int rc = 1;    
    kk_ssize_t next = start;
    while( rc > 0 && start < len && atmost > 0) {
      atmost--;
      rc = 0;
      kk_ssize_t mstart = start;
      kk_std_core__list cap = kk_regex_exec_ex( re, match_data, str, cstr, len, allow_empty, start, &mstart, &next, &rc, ctx );
      if (rc > 0) {
        // found a match; 
        // push string up to match, and the actual matched regex
        kk_std_core__sslice pre = kk_std_core__new_Sslice( kk_string_dup(str,ctx), kk_integer_from_ssize_t(start,ctx), kk_integer_from_ssize_t(mstart - start,ctx), ctx ); 
        kk_std_core__list   prelist = kk_std_core__new_Cons( kk_reuse_null, 0, kk_std_core__sslice_box(pre,ctx), kk_std_core__new_Nil(ctx), ctx );
        kk_std_core__list   capcons = kk_std_core__new_Cons( kk_reuse_null, 0, kk_std_core__list_box(cap,ctx), kk_std_core__new_Nil(ctx) /*tail*/, ctx );
        kk_std_core__list   cons = kk_std_core__new_Cons( kk_reuse_null, 0, kk_std_core__list_box(prelist,ctx), capcons, ctx );
        if (tail==NULL) res = cons;
                  else *tail = cons;
        tail = &kk_std_core__as_Cons(capcons,ctx)->tail;
        allow_empty = (next > start);
        start = next;
      }
      else if (rc <= 0 && !allow_empty) {
        // skip one character and try again
        // todo: handle cr/lf pairs better?
        const uint8_t* p = kk_utf8_next( cstr + start );
        start = (p - cstr);
        allow_empty = true;
        rc = 1;
      }
    }
    
    // push final string part as well and end the list
    kk_std_core__sslice post    = kk_std_core__new_Sslice( kk_string_dup(str,ctx), kk_integer_from_ssize_t(next,ctx), kk_integer_from_ssize_t(len - next,ctx), ctx ); 
    kk_std_core__list   postlist= kk_std_core__new_Cons( kk_reuse_null, 0, kk_std_core__sslice_box(post,ctx), kk_std_core__new_Nil(ctx), ctx );
    kk_std_core__list   cons    = kk_std_core__new_Cons( kk_reuse_null, 0, kk_std_core__list_box(postlist,ctx), kk_std_core__new_Nil(ctx), ctx );
    if (tail==NULL) res = cons;
              else *tail = cons;  
  }

done:  
  if (match_data != NULL) {
    //kk_info_message( "free match data: %p\n", match_data );
    pcre2_match_data_free(match_data);
  }
  kk_string_drop(str,ctx);
  kk_box_drop(bre,ctx);
  return res;
}

/*
kk_std_core__sslice kk_slice_upto( struct kk_std_core_Sslice slice1, struct kk_std_core_Sslice slice2, kk_context_t* ctx ) {
  kk_ssize_t start = slice1.start;
  kk_ssize_t len   = (slice1.start <= slice2.start ? slice2.start - slice1.start : -1);
  return kk_std_core__new_Sslice(slice1.str, start, len, ctx);
}
*/

kk_std_text_regex__regex kk_std_text_regex__copy(kk_std_text_regex__regex _this, kk_std_core_types__optional obj0, kk_std_core_types__optional src0, kk_context_t* _ctx) { /* (regex, obj : optional<any>, src : optional<string>) -> regex */ 
  kk_box_t _x1259;
  if (kk_std_core_types__is_Optional(obj0, _ctx)) {
    kk_box_t _box_x1083 = obj0._cons.Optional.value;
    kk_box_t _obj_104 = kk_box_unbox(_box_x1083, _ctx);
    kk_box_dup(_obj_104, _ctx);
    kk_std_core_types__optional_drop(obj0, _ctx);
    _x1259 = _obj_104; /*any*/
  }
  else {
    kk_std_core_types__optional_drop(obj0, _ctx);
    {
      kk_box_t _x = _this.obj;
      kk_box_dup(_x, _ctx);
      _x1259 = _x; /*any*/
    }
  }
  kk_string_t _x1260;
  if (kk_std_core_types__is_Optional(src0, _ctx)) {
    kk_box_t _box_x1084 = src0._cons.Optional.value;
    kk_string_t _src_110 = kk_string_unbox(_box_x1084);
    kk_string_dup(_src_110, _ctx);
    kk_std_core_types__optional_drop(src0, _ctx);
    kk_std_text_regex__regex_drop(_this, _ctx);
    _x1260 = _src_110; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(src0, _ctx);
    {
      kk_string_t _x0 = _this.src;
      kk_string_dup(_x0, _ctx);
      kk_std_text_regex__regex_drop(_this, _ctx);
      _x1260 = _x0; /*string*/
    }
  }
  return kk_std_text_regex__new_Regex(_x1259, _x1260, _ctx);
}
 
// Return the full matched string of a capture group

kk_string_t kk_std_text_regex_captured(kk_std_core__list matched0, kk_context_t* _ctx) { /* (matched : list<sslice>) -> string */ 
  if (kk_std_core__is_Cons(matched0, _ctx)) {
    struct kk_std_core_Cons* _con1261 = kk_std_core__as_Cons(matched0, _ctx);
    kk_box_t _box_x1085 = _con1261->head;
    kk_std_core__sslice s = kk_std_core__sslice_unbox(_box_x1085, KK_BORROWED, _ctx);
    kk_std_core__list _pat3 = _con1261->tail;
    if kk_likely(kk_datatype_ptr_is_unique(matched0, _ctx)) {
      kk_std_core__list_drop(_pat3, _ctx);
      kk_std_core__sslice_dup(s, _ctx);
      kk_box_drop(_box_x1085, _ctx);
      kk_datatype_ptr_free(matched0, _ctx);
    }
    else {
      kk_std_core__sslice_dup(s, _ctx);
      kk_datatype_ptr_decref(matched0, _ctx);
    }
    return kk_std_core_string_3(s, _ctx);
  }
  {
    return kk_string_empty();
  }
}

kk_std_core__list kk_std_text_regex_regex_exec(kk_box_t regex0, kk_string_t str, kk_ssize_t start, kk_context_t* _ctx) { /* (regex : any, str : string, start : ssize_t) -> list<sslice> */ 
  return kk_regex_exec(regex0,str,start,kk_context());
}

kk_std_core__list kk_std_text_regex_regex_exec_all(kk_box_t regex0, kk_string_t str, kk_ssize_t start, kk_ssize_t atmost, kk_context_t* _ctx) { /* (regex : any, str : string, start : ssize_t, atmost : ssize_t) -> list<list<sslice>> */ 
  return kk_regex_exec_all(regex0,str,start,atmost,kk_context());
}

kk_box_t kk_std_text_regex_regex_create(kk_string_t source0, bool ignore_case, bool multi_line, kk_context_t* _ctx) { /* (source : string, ignore-case : bool, multi-line : bool) -> any */ 
  return kk_regex_create(source0,ignore_case,multi_line,kk_context());
}
extern kk_box_t kk_std_text_regex_captures_fun1263(kk_function_t _fself, kk_box_t _b_1088, kk_context_t* _ctx) {
  kk_unused(_fself);
  kk_string_t _x1264;
  kk_std_core__list _x1265 = kk_std_core__list_unbox(_b_1088, KK_OWNED, _ctx); /*list<sslice>*/
  _x1264 = kk_std_text_regex_captured(_x1265, _ctx); /*string*/
  return kk_string_box(_x1264);
}
 
// monadic lift

kk_string_t kk_std_text_regex__mlift1046_concat_replace(kk_std_core__list acc, kk_std_core__list mm, kk_std_core__list pre, kk_function_t repl, kk_string_t _y_1034, kk_context_t* _ctx) { /* forall<e> (acc : list<string>, mm : list<list<sslice>>, pre : list<sslice>, repl : (list<sslice>) -> e string, string) -> e string */ 
  kk_string_t _b_1098_1092;
  if (kk_std_core__is_Cons(pre, _ctx)) {
    struct kk_std_core_Cons* _con1266 = kk_std_core__as_Cons(pre, _ctx);
    kk_box_t _box_x1091 = _con1266->head;
    kk_std_core__sslice s = kk_std_core__sslice_unbox(_box_x1091, KK_BORROWED, _ctx);
    kk_std_core__list _pat3 = _con1266->tail;
    if kk_likely(kk_datatype_ptr_is_unique(pre, _ctx)) {
      kk_std_core__list_drop(_pat3, _ctx);
      kk_std_core__sslice_dup(s, _ctx);
      kk_box_drop(_box_x1091, _ctx);
      kk_datatype_ptr_free(pre, _ctx);
    }
    else {
      kk_std_core__sslice_dup(s, _ctx);
      kk_datatype_ptr_decref(pre, _ctx);
    }
    _b_1098_1092 = kk_std_core_string_3(s, _ctx); /*string*/
  }
  else {
    _b_1098_1092 = kk_string_empty(); /*string*/
  }
  kk_std_core__list _x1268;
  kk_std_core__list _x1269 = kk_std_core__new_Cons(kk_reuse_null, 0, kk_string_box(_b_1098_1092), acc, _ctx); /*list<61>*/
  _x1268 = kk_std_core__new_Cons(kk_reuse_null, 0, kk_string_box(_y_1034), _x1269, _ctx); /*list<61>*/
  return kk_std_text_regex_concat_replace(mm, repl, _x1268, _ctx);
}


// lift anonymous function
struct kk_std_text_regex_concat_replace_fun1274__t {
  struct kk_function_s _base;
  kk_std_core__list acc0;
  kk_std_core__list mm0;
  kk_std_core__list pre0;
  kk_function_t repl0;
};
static kk_box_t kk_std_text_regex_concat_replace_fun1274(kk_function_t _fself, kk_box_t _b_1103, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_concat_replace_fun1274(kk_std_core__list acc0, kk_std_core__list mm0, kk_std_core__list pre0, kk_function_t repl0, kk_context_t* _ctx) {
  struct kk_std_text_regex_concat_replace_fun1274__t* _self = kk_function_alloc_as(struct kk_std_text_regex_concat_replace_fun1274__t, 5, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_text_regex_concat_replace_fun1274, kk_context());
  _self->acc0 = acc0;
  _self->mm0 = mm0;
  _self->pre0 = pre0;
  _self->repl0 = repl0;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_box_t kk_std_text_regex_concat_replace_fun1274(kk_function_t _fself, kk_box_t _b_1103, kk_context_t* _ctx) {
  struct kk_std_text_regex_concat_replace_fun1274__t* _self = kk_function_as(struct kk_std_text_regex_concat_replace_fun1274__t*, _fself, _ctx);
  kk_std_core__list acc0 = _self->acc0; /* list<string> */
  kk_std_core__list mm0 = _self->mm0; /* list<list<sslice>> */
  kk_std_core__list pre0 = _self->pre0; /* list<sslice> */
  kk_function_t repl0 = _self->repl0; /* (list<sslice>) -> 321 string */
  kk_drop_match(_self, {kk_std_core__list_dup(acc0, _ctx);kk_std_core__list_dup(mm0, _ctx);kk_std_core__list_dup(pre0, _ctx);kk_function_dup(repl0, _ctx);}, {}, _ctx)
  kk_string_t _y_1122_10340 = kk_string_unbox(_b_1103); /*string*/;
  kk_string_t _x1275 = kk_std_text_regex__mlift1046_concat_replace(acc0, mm0, pre0, repl0, _y_1122_10340, _ctx); /*string*/
  return kk_string_box(_x1275);
}

kk_string_t kk_std_text_regex_concat_replace(kk_std_core__list matches, kk_function_t repl0, kk_std_core__list acc0, kk_context_t* _ctx) { /* forall<e> (matches : list<list<sslice>>, repl : (list<sslice>) -> e string, acc : list<string>) -> e string */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(matches, _ctx)) {
    struct kk_std_core_Cons* _con1270 = kk_std_core__as_Cons(matches, _ctx);
    kk_box_t _box_x1100 = _con1270->head;
    kk_std_core__list _pat0 = _con1270->tail;
    if (kk_std_core__is_Cons(_pat0, _ctx)) {
      struct kk_std_core_Cons* _con1271 = kk_std_core__as_Cons(_pat0, _ctx);
      kk_box_t _box_x1101 = _con1271->head;
      kk_std_core__list pre0 = kk_std_core__list_unbox(_box_x1100, KK_BORROWED, _ctx);
      kk_std_core__list mm0 = _con1271->tail;
      kk_std_core__list m = kk_std_core__list_unbox(_box_x1101, KK_BORROWED, _ctx);
      kk_reuse_t _ru_1241 = kk_reuse_null; /*reuse*/;
      kk_reuse_t _ru_1240 = kk_reuse_null; /*reuse*/;
      if kk_likely(kk_datatype_ptr_is_unique(matches, _ctx)) {
        if kk_likely(kk_datatype_ptr_is_unique(_pat0, _ctx)) {
          _ru_1240 = (kk_datatype_ptr_reuse(_pat0, _ctx));
        }
        else {
          kk_std_core__list_dup(m, _ctx);
          kk_std_core__list_dup(mm0, _ctx);
          kk_datatype_ptr_decref(_pat0, _ctx);
        }
        _ru_1241 = (kk_datatype_ptr_reuse(matches, _ctx));
      }
      else {
        kk_std_core__list_dup(m, _ctx);
        kk_std_core__list_dup(mm0, _ctx);
        kk_std_core__list_dup(pre0, _ctx);
        kk_datatype_ptr_decref(matches, _ctx);
      }
      kk_string_t x_1047;
      kk_function_t _x1272 = kk_function_dup(repl0, _ctx); /*(list<sslice>) -> 321 string*/
      x_1047 = kk_function_call(kk_string_t, (kk_function_t, kk_std_core__list, kk_context_t*), _x1272, (_x1272, m, _ctx), _ctx); /*string*/
      if (kk_yielding(kk_context())) {
        kk_reuse_drop(_ru_1240,kk_context());
        kk_reuse_drop(_ru_1241,kk_context());
        kk_string_drop(x_1047, _ctx);
        kk_box_t _x1273 = kk_std_core_hnd_yield_extend(kk_std_text_regex_new_concat_replace_fun1274(acc0, mm0, pre0, repl0, _ctx), _ctx); /*3926*/
        return kk_string_unbox(_x1273);
      }
      {
        kk_string_t _b_1120_1105;
        if (kk_std_core__is_Cons(pre0, _ctx)) {
          struct kk_std_core_Cons* _con1276 = kk_std_core__as_Cons(pre0, _ctx);
          kk_box_t _box_x1104 = _con1276->head;
          kk_std_core__sslice s0 = kk_std_core__sslice_unbox(_box_x1104, KK_BORROWED, _ctx);
          kk_std_core__list _pat30 = _con1276->tail;
          if kk_likely(kk_datatype_ptr_is_unique(pre0, _ctx)) {
            kk_std_core__list_drop(_pat30, _ctx);
            kk_std_core__sslice_dup(s0, _ctx);
            kk_box_drop(_box_x1104, _ctx);
            kk_datatype_ptr_free(pre0, _ctx);
          }
          else {
            kk_std_core__sslice_dup(s0, _ctx);
            kk_datatype_ptr_decref(pre0, _ctx);
          }
          _b_1120_1105 = kk_std_core_string_3(s0, _ctx); /*string*/
        }
        else {
          _b_1120_1105 = kk_string_empty(); /*string*/
        }
        { // tailcall
          kk_std_core__list _x1278;
          kk_std_core__list _x1279 = kk_std_core__new_Cons(_ru_1241, 0, kk_string_box(_b_1120_1105), acc0, _ctx); /*list<61>*/
          _x1278 = kk_std_core__new_Cons(_ru_1240, 0, kk_string_box(x_1047), _x1279, _ctx); /*list<61>*/
          matches = mm0;
          acc0 = _x1278;
          goto kk__tailcall;
        }
      }
    }
  }
  if (kk_std_core__is_Cons(matches, _ctx)) {
    struct kk_std_core_Cons* _con1280 = kk_std_core__as_Cons(matches, _ctx);
    kk_box_t _box_x1109 = _con1280->head;
    kk_std_core__list _pat200 = _con1280->tail;
    kk_std_core__list post = kk_std_core__list_unbox(_box_x1109, KK_BORROWED, _ctx);
    kk_function_drop(repl0, _ctx);
    kk_reuse_t _ru_1243 = kk_reuse_null; /*reuse*/;
    if kk_likely(kk_datatype_ptr_is_unique(matches, _ctx)) {
      _ru_1243 = (kk_datatype_ptr_reuse(matches, _ctx));
    }
    else {
      kk_std_core__list_dup(post, _ctx);
      kk_datatype_ptr_decref(matches, _ctx);
    }
    kk_string_t _b_1113_1111;
    if (kk_std_core__is_Cons(post, _ctx)) {
      struct kk_std_core_Cons* _con1281 = kk_std_core__as_Cons(post, _ctx);
      kk_box_t _box_x1110 = _con1281->head;
      kk_std_core__sslice s00 = kk_std_core__sslice_unbox(_box_x1110, KK_BORROWED, _ctx);
      kk_std_core__list _pat300 = _con1281->tail;
      if kk_likely(kk_datatype_ptr_is_unique(post, _ctx)) {
        kk_std_core__list_drop(_pat300, _ctx);
        kk_std_core__sslice_dup(s00, _ctx);
        kk_box_drop(_box_x1110, _ctx);
        kk_datatype_ptr_free(post, _ctx);
      }
      else {
        kk_std_core__sslice_dup(s00, _ctx);
        kk_datatype_ptr_decref(post, _ctx);
      }
      _b_1113_1111 = kk_std_core_string_3(s00, _ctx); /*string*/
    }
    else {
      _b_1113_1111 = kk_string_empty(); /*string*/
    }
    kk_std_core__list xs0_17603;
    kk_std_core__list _x1283 = kk_std_core__new_Cons(_ru_1243, 0, kk_string_box(_b_1113_1111), acc0, _ctx); /*list<61>*/
    xs0_17603 = kk_std_core__lift17313_reverse_append(kk_std_core__new_Nil(_ctx), _x1283, _ctx); /*list<string>*/
    if (kk_std_core__is_Nil(xs0_17603, _ctx)) {
      return kk_string_empty();
    }
    {
      struct kk_std_core_Cons* _con1285 = kk_std_core__as_Cons(xs0_17603, _ctx);
      kk_box_t _box_x1115 = _con1285->head;
      kk_std_core__list xx = _con1285->tail;
      kk_string_t x0 = kk_string_unbox(_box_x1115);
      if kk_likely(kk_datatype_ptr_is_unique(xs0_17603, _ctx)) {
        kk_datatype_ptr_free(xs0_17603, _ctx);
      }
      else {
        kk_string_dup(x0, _ctx);
        kk_std_core__list_dup(xx, _ctx);
        kk_datatype_ptr_decref(xs0_17603, _ctx);
      }
      kk_string_t _x1286 = kk_string_empty(); /*string*/
      return kk_std_core__lift17316_joinsep(_x1286, xx, x0, _ctx);
    }
  }
  {
    kk_function_drop(repl0, _ctx);
    kk_std_core__list xs0_176030 = kk_std_core__lift17313_reverse_append(kk_std_core__new_Nil(_ctx), acc0, _ctx); /*list<string>*/;
    if (kk_std_core__is_Nil(xs0_176030, _ctx)) {
      return kk_string_empty();
    }
    {
      struct kk_std_core_Cons* _con1289 = kk_std_core__as_Cons(xs0_176030, _ctx);
      kk_box_t _box_x1116 = _con1289->head;
      kk_std_core__list xx0 = _con1289->tail;
      kk_string_t x00 = kk_string_unbox(_box_x1116);
      if kk_likely(kk_datatype_ptr_is_unique(xs0_176030, _ctx)) {
        kk_datatype_ptr_free(xs0_176030, _ctx);
      }
      else {
        kk_string_dup(x00, _ctx);
        kk_std_core__list_dup(xx0, _ctx);
        kk_datatype_ptr_decref(xs0_176030, _ctx);
      }
      kk_string_t _x1290 = kk_string_empty(); /*string*/
      return kk_std_core__lift17316_joinsep(_x1290, xx0, x00, _ctx);
    }
  }
}
 
// Does a regular expression pattern occur in a string `s`?
// (note: called `test` in javascript)

bool kk_std_text_regex_contains(kk_string_t s, kk_std_text_regex__regex r, kk_context_t* _ctx) { /* (s : string, r : regex) -> bool */ 
  kk_std_core__list list0_984;
  kk_box_t _x1294;
  {
    kk_box_t _x = r.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(r, _ctx);
    _x1294 = _x; /*any*/
  }
  kk_ssize_t _x1295 = (KK_IZ(0)); /*ssize_t*/
  list0_984 = kk_std_text_regex_regex_exec(_x1294, s, _x1295, _ctx); /*list<sslice>*/
  if (kk_std_core__is_Cons(list0_984, _ctx)) {
    struct kk_std_core_Cons* _con1296 = kk_std_core__as_Cons(list0_984, _ctx);
    kk_box_t _box_x1123 = _con1296->head;
    kk_std_core__list _pat1 = _con1296->tail;
    if kk_likely(kk_datatype_ptr_is_unique(list0_984, _ctx)) {
      kk_std_core__list_drop(_pat1, _ctx);
      kk_box_drop(_box_x1123, _ctx);
      kk_datatype_ptr_free(list0_984, _ctx);
    }
    else {
      kk_datatype_ptr_decref(list0_984, _ctx);
    }
    return true;
  }
  {
    return false;
  }
}
 
// Match a regular expression `regex` over a string `s`.
// Matches at most `atmost` times (and matches all by default).
// Returns always an odd number of elements where every even
// element is a match and the odd ones the string parts between the
// matches.
// See also `find-all` and `strings`.

kk_std_core__list kk_std_text_regex_exec_all(kk_std_text_regex__regex regex0, kk_string_t s, kk_std_core_types__optional atmost, kk_context_t* _ctx) { /* (regex : regex, s : string, atmost : optional<int>) -> list<list<sslice>> */ 
  kk_box_t _x1297;
  {
    kk_box_t _x = regex0.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(regex0, _ctx);
    _x1297 = _x; /*any*/
  }
  kk_ssize_t _x1298 = (KK_IZ(0)); /*ssize_t*/
  kk_ssize_t _x1299;
  kk_integer_t _x1300;
  if (kk_std_core_types__is_Optional(atmost, _ctx)) {
    kk_box_t _box_x1124 = atmost._cons.Optional.value;
    kk_integer_t _atmost_345 = kk_integer_unbox(_box_x1124, _ctx);
    kk_integer_dup(_atmost_345, _ctx);
    kk_std_core_types__optional_drop(atmost, _ctx);
    _x1300 = _atmost_345; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(atmost, _ctx);
    _x1300 = kk_integer_from_small(-1); /*int*/
  }
  _x1299 = kk_std_core_ssize__t(_x1300, _ctx); /*ssize_t*/
  return kk_std_text_regex_regex_exec_all(_x1297, s, _x1298, _x1299, _ctx);
}
 
// Filter only for the matched parts.

kk_std_core__list kk_std_text_regex__trmc_filter_matches(kk_std_core__list xs, kk_std_core_types__cctx _acc, kk_context_t* _ctx) { /* (xs : list<list<sslice>>, ctx<list<list<sslice>>>) -> list<list<sslice>> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs, _ctx)) {
    struct kk_std_core_Cons* _con1301 = kk_std_core__as_Cons(xs, _ctx);
    kk_box_t _box_x1125 = _con1301->head;
    kk_std_core__list _pat1 = _con1301->tail;
    if (kk_std_core__is_Cons(_pat1, _ctx)) {
      struct kk_std_core_Cons* _con1302 = kk_std_core__as_Cons(_pat1, _ctx);
      kk_box_t _box_x1126 = _con1302->head;
      kk_std_core__list mm = _con1302->tail;
      kk_std_core__list m = kk_std_core__list_unbox(_box_x1126, KK_BORROWED, _ctx);
      kk_reuse_t _ru_1249 = kk_reuse_null; /*reuse*/;
      if kk_likely(kk_datatype_ptr_is_unique(xs, _ctx)) {
        if kk_likely(kk_datatype_ptr_is_unique(_pat1, _ctx)) {
          kk_datatype_ptr_free(_pat1, _ctx);
        }
        else {
          kk_std_core__list_dup(m, _ctx);
          kk_std_core__list_dup(mm, _ctx);
          kk_datatype_ptr_decref(_pat1, _ctx);
        }
        kk_box_drop(_box_x1125, _ctx);
        _ru_1249 = (kk_datatype_ptr_reuse(xs, _ctx));
      }
      else {
        kk_std_core__list_dup(m, _ctx);
        kk_std_core__list_dup(mm, _ctx);
        kk_datatype_ptr_decref(xs, _ctx);
      }
      kk_std_core__list _trmc_1030 = kk_datatype_null(); /*list<list<sslice>>*/;
      kk_std_core__list _trmc_1031 = kk_std_core__new_Cons(_ru_1249, 0, kk_std_core__list_box(m, _ctx), _trmc_1030, _ctx); /*list<list<sslice>>*/;
      kk_box_t* _b_1141_1136 = (kk_box_t*)((&kk_std_core__as_Cons(_trmc_1031, _ctx)->tail)); /*field-addr<list<list<sslice>>>*/;
      { // tailcall
        kk_std_core_types__cctx _x1303 = kk_cctx_extend(_acc,(kk_std_core__list_box(_trmc_1031, _ctx)),_b_1141_1136,true,kk_context()); /*ctx<0>*/
        xs = mm;
        _acc = _x1303;
        goto kk__tailcall;
      }
    }
  }
  {
    kk_std_core__list_drop(xs, _ctx);
    kk_box_t _x1304 = kk_cctx_apply(_acc,(kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx)),true,kk_context()); /*-1*/
    return kk_std_core__list_unbox(_x1304, KK_OWNED, _ctx);
  }
}
 
// Filter only for the matched parts.

kk_std_core__list kk_std_text_regex_filter_matches(kk_std_core__list xs0, kk_context_t* _ctx) { /* (xs : list<list<sslice>>) -> list<list<sslice>> */ 
  kk_std_core_types__cctx _x1305 = kk_cctx_empty(kk_context()); /*ctx<0>*/
  return kk_std_text_regex__trmc_filter_matches(xs0, _x1305, _ctx);
}
 
// Filter only for the non-matched parts.

kk_std_core__list kk_std_text_regex__trmc_filter_non_matches(kk_std_core__list xs, kk_std_core_types__cctx _acc, kk_context_t* _ctx) { /* (xs : list<list<sslice>>, ctx<list<list<sslice>>>) -> list<list<sslice>> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs, _ctx)) {
    struct kk_std_core_Cons* _con1306 = kk_std_core__as_Cons(xs, _ctx);
    kk_box_t _box_x1147 = _con1306->head;
    kk_std_core__list _pat0 = _con1306->tail;
    if (kk_std_core__is_Cons(_pat0, _ctx)) {
      struct kk_std_core_Cons* _con1307 = kk_std_core__as_Cons(_pat0, _ctx);
      kk_box_t _box_x1148 = _con1307->head;
      kk_std_core__list s = kk_std_core__list_unbox(_box_x1147, KK_BORROWED, _ctx);
      kk_std_core__list xx = _con1307->tail;
      kk_reuse_t _ru_1251 = kk_reuse_null; /*reuse*/;
      if kk_likely(kk_datatype_ptr_is_unique(xs, _ctx)) {
        if kk_likely(kk_datatype_ptr_is_unique(_pat0, _ctx)) {
          kk_box_drop(_box_x1148, _ctx);
          kk_datatype_ptr_free(_pat0, _ctx);
        }
        else {
          kk_std_core__list_dup(xx, _ctx);
          kk_datatype_ptr_decref(_pat0, _ctx);
        }
        _ru_1251 = (kk_datatype_ptr_reuse(xs, _ctx));
      }
      else {
        kk_std_core__list_dup(s, _ctx);
        kk_std_core__list_dup(xx, _ctx);
        kk_datatype_ptr_decref(xs, _ctx);
      }
      kk_std_core__list _trmc_1032 = kk_datatype_null(); /*list<list<sslice>>*/;
      kk_std_core__list _trmc_1033 = kk_std_core__new_Cons(_ru_1251, 0, kk_std_core__list_box(s, _ctx), _trmc_1032, _ctx); /*list<list<sslice>>*/;
      kk_box_t* _b_1168_1158 = (kk_box_t*)((&kk_std_core__as_Cons(_trmc_1033, _ctx)->tail)); /*field-addr<list<list<sslice>>>*/;
      { // tailcall
        kk_std_core_types__cctx _x1308 = kk_cctx_extend(_acc,(kk_std_core__list_box(_trmc_1033, _ctx)),_b_1168_1158,true,kk_context()); /*ctx<0>*/
        xs = xx;
        _acc = _x1308;
        goto kk__tailcall;
      }
    }
  }
  if (kk_std_core__is_Cons(xs, _ctx)) {
    struct kk_std_core_Cons* _con1309 = kk_std_core__as_Cons(xs, _ctx);
    kk_box_t _box_x1159 = _con1309->head;
    kk_std_core__list _pat3 = _con1309->tail;
    kk_std_core__list s0 = kk_std_core__list_unbox(_box_x1159, KK_BORROWED, _ctx);
    kk_reuse_t _ru_1252 = kk_reuse_null; /*reuse*/;
    if kk_likely(kk_datatype_ptr_is_unique(xs, _ctx)) {
      _ru_1252 = (kk_datatype_ptr_reuse(xs, _ctx));
    }
    else {
      kk_std_core__list_dup(s0, _ctx);
      kk_datatype_ptr_decref(xs, _ctx);
    }
    kk_box_t _x1310;
    kk_box_t _x1311;
    kk_std_core__list _x1312 = kk_std_core__new_Cons(_ru_1252, 0, kk_std_core__list_box(s0, _ctx), kk_std_core__new_Nil(_ctx), _ctx); /*list<61>*/
    _x1311 = kk_std_core__list_box(_x1312, _ctx); /*-1*/
    _x1310 = kk_cctx_apply(_acc,_x1311,true,kk_context()); /*-1*/
    return kk_std_core__list_unbox(_x1310, KK_OWNED, _ctx);
  }
  {
    kk_box_t _x1313 = kk_cctx_apply(_acc,(kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx)),true,kk_context()); /*-1*/
    return kk_std_core__list_unbox(_x1313, KK_OWNED, _ctx);
  }
}
 
// Filter only for the non-matched parts.

kk_std_core__list kk_std_text_regex_filter_non_matches(kk_std_core__list xs0, kk_context_t* _ctx) { /* (xs : list<list<sslice>>) -> list<list<sslice>> */ 
  kk_std_core_types__cctx _x1314 = kk_cctx_empty(kk_context()); /*ctx<0>*/
  return kk_std_text_regex__trmc_filter_non_matches(xs0, _x1314, _ctx);
}
 
// Find a match for a regular expression.
// See also `exec`

kk_std_core_types__maybe kk_std_text_regex_find(kk_string_t s, kk_std_text_regex__regex r, kk_context_t* _ctx) { /* (s : string, r : regex) -> maybe<string> */ 
  kk_std_core__list _match_1234;
  kk_box_t _x1315;
  {
    kk_box_t _x = r.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(r, _ctx);
    _x1315 = _x; /*any*/
  }
  kk_ssize_t _x1316 = (KK_IZ(0)); /*ssize_t*/
  _match_1234 = kk_std_text_regex_regex_exec(_x1315, s, _x1316, _ctx); /*list<sslice>*/
  if (kk_std_core__is_Cons(_match_1234, _ctx)) {
    struct kk_std_core_Cons* _con1317 = kk_std_core__as_Cons(_match_1234, _ctx);
    kk_box_t _box_x1178 = _con1317->head;
    kk_std_core__sslice m = kk_std_core__sslice_unbox(_box_x1178, KK_BORROWED, _ctx);
    kk_std_core__list _pat3 = _con1317->tail;
    if kk_likely(kk_datatype_ptr_is_unique(_match_1234, _ctx)) {
      kk_std_core__list_drop(_pat3, _ctx);
      kk_std_core__sslice_dup(m, _ctx);
      kk_box_drop(_box_x1178, _ctx);
      kk_datatype_ptr_free(_match_1234, _ctx);
    }
    else {
      kk_std_core__sslice_dup(m, _ctx);
      kk_datatype_ptr_decref(_match_1234, _ctx);
    }
    kk_string_t _b_1180_1179 = kk_std_core_string_3(m, _ctx); /*string*/;
    return kk_std_core_types__new_Just(kk_string_box(_b_1180_1179), _ctx);
  }
  {
    return kk_std_core_types__new_Nothing(_ctx);
  }
}
 
// Find all matches for a regular expression in a string.


// lift anonymous function
struct kk_std_text_regex_find_all_fun1323__t {
  struct kk_function_s _base;
};
static kk_box_t kk_std_text_regex_find_all_fun1323(kk_function_t _fself, kk_box_t _b_1184, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_find_all_fun1323(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_text_regex_find_all_fun1323, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_box_t kk_std_text_regex_find_all_fun1323(kk_function_t _fself, kk_box_t _b_1184, kk_context_t* _ctx) {
  kk_unused(_fself);
  kk_string_t _x1324;
  kk_std_core__list _x1325 = kk_std_core__list_unbox(_b_1184, KK_OWNED, _ctx); /*list<sslice>*/
  _x1324 = kk_std_text_regex_captured(_x1325, _ctx); /*string*/
  return kk_string_box(_x1324);
}

kk_std_core__list kk_std_text_regex_find_all(kk_string_t s, kk_std_text_regex__regex r, kk_std_core_types__optional atmost, kk_context_t* _ctx) { /* (s : string, r : regex, atmost : optional<int>) -> list<string> */ 
  kk_std_core__list xs_992;
  kk_std_core__list _x1318;
  kk_box_t _x1319;
  {
    kk_box_t _x = r.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(r, _ctx);
    _x1319 = _x; /*any*/
  }
  kk_ssize_t _x1320 = (KK_IZ(0)); /*ssize_t*/
  kk_ssize_t _x1321;
  kk_integer_t _x1322;
  if (kk_std_core_types__is_Optional(atmost, _ctx)) {
    kk_box_t _box_x1181 = atmost._cons.Optional.value;
    kk_integer_t _atmost_437 = kk_integer_unbox(_box_x1181, _ctx);
    kk_integer_dup(_atmost_437, _ctx);
    kk_std_core_types__optional_drop(atmost, _ctx);
    _x1322 = _atmost_437; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(atmost, _ctx);
    _x1322 = kk_integer_from_small(-1); /*int*/
  }
  _x1321 = kk_std_core_ssize__t(_x1322, _ctx); /*ssize_t*/
  _x1318 = kk_std_text_regex_regex_exec_all(_x1319, s, _x1320, _x1321, _ctx); /*list<list<sslice>>*/
  xs_992 = kk_std_text_regex_filter_matches(_x1318, _ctx); /*list<list<sslice>>*/
  return kk_std_core_map_5(xs_992, kk_std_text_regex_new_find_all_fun1323(_ctx), _ctx);
}
 
// Create a new regular expression. Takes two optional parameters. Set `ignoreCase` to `True`
// to ignore uppercase/lowercase distinction. If  `multiline` is set to `True`, then `^` and `$`
// match also the beginning and end of every line (instead of the entire input).

kk_std_text_regex__regex kk_std_text_regex_regex(kk_string_t regex0, kk_std_core_types__optional ignorecase, kk_std_core_types__optional multiline, kk_context_t* _ctx) { /* (regex : string, ignorecase : optional<bool>, multiline : optional<bool>) -> regex */ 
  kk_box_t _x1326;
  kk_string_t _x1327 = kk_string_dup(regex0, _ctx); /*string*/
  bool _x1328;
  if (kk_std_core_types__is_Optional(ignorecase, _ctx)) {
    kk_box_t _box_x1187 = ignorecase._cons.Optional.value;
    bool _ignorecase_455 = kk_bool_unbox(_box_x1187);
    kk_std_core_types__optional_drop(ignorecase, _ctx);
    _x1328 = _ignorecase_455; /*bool*/
  }
  else {
    kk_std_core_types__optional_drop(ignorecase, _ctx);
    _x1328 = false; /*bool*/
  }
  bool _x1329;
  if (kk_std_core_types__is_Optional(multiline, _ctx)) {
    kk_box_t _box_x1188 = multiline._cons.Optional.value;
    bool _multiline_459 = kk_bool_unbox(_box_x1188);
    kk_std_core_types__optional_drop(multiline, _ctx);
    _x1329 = _multiline_459; /*bool*/
  }
  else {
    kk_std_core_types__optional_drop(multiline, _ctx);
    _x1329 = false; /*bool*/
  }
  _x1326 = kk_std_text_regex_regex_create(_x1327, _x1328, _x1329, _ctx); /*any*/
  return kk_std_text_regex__new_Regex(_x1326, regex0, _ctx);
}
 
// Replace the all occurrences of `regex` by the result of the replacement fun `repl` in a string `s`.

kk_string_t kk_std_text_regex_replace_all(kk_string_t s, kk_std_text_regex__regex r, kk_function_t repl, kk_std_core_types__optional atmost, kk_context_t* _ctx) { /* forall<e> (s : string, r : regex, repl : (list<sslice>) -> e string, atmost : optional<int>) -> e string */ 
  kk_std_core__list _x1330;
  kk_box_t _x1331;
  {
    kk_box_t _x = r.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(r, _ctx);
    _x1331 = _x; /*any*/
  }
  kk_ssize_t _x1332 = (KK_IZ(0)); /*ssize_t*/
  kk_ssize_t _x1333;
  kk_integer_t _x1334;
  if (kk_std_core_types__is_Optional(atmost, _ctx)) {
    kk_box_t _box_x1189 = atmost._cons.Optional.value;
    kk_integer_t _atmost_475 = kk_integer_unbox(_box_x1189, _ctx);
    kk_integer_dup(_atmost_475, _ctx);
    kk_std_core_types__optional_drop(atmost, _ctx);
    _x1334 = _atmost_475; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(atmost, _ctx);
    _x1334 = kk_integer_from_small(-1); /*int*/
  }
  _x1333 = kk_std_core_ssize__t(_x1334, _ctx); /*ssize_t*/
  _x1330 = kk_std_text_regex_regex_exec_all(_x1331, s, _x1332, _x1333, _ctx); /*list<list<sslice>>*/
  return kk_std_text_regex_concat_replace(_x1330, repl, kk_std_core__new_Nil(_ctx), _ctx);
}
 
// Replace using a replacement string that can contain `$$` for a `$` sign, `$n` for a capture group,
// `$&` for the entire match `==$0`.


// lift anonymous function
struct kk_std_text_regex_replace_captures_fun1347__t {
  struct kk_function_s _base;
  kk_std_core__list caps;
};
static kk_string_t kk_std_text_regex_replace_captures_fun1347(kk_function_t _fself, kk_std_core__list cap, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_replace_captures_fun1347(kk_std_core__list caps, kk_context_t* _ctx) {
  struct kk_std_text_regex_replace_captures_fun1347__t* _self = kk_function_alloc_as(struct kk_std_text_regex_replace_captures_fun1347__t, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_text_regex_replace_captures_fun1347, kk_context());
  _self->caps = caps;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_string_t kk_std_text_regex_replace_captures_fun1347(kk_function_t _fself, kk_std_core__list cap, kk_context_t* _ctx) {
  struct kk_std_text_regex_replace_captures_fun1347__t* _self = kk_function_as(struct kk_std_text_regex_replace_captures_fun1347__t*, _fself, _ctx);
  kk_std_core__list caps = _self->caps; /* list<sslice> */
  kk_drop_match(_self, {kk_std_core__list_dup(caps, _ctx);}, {}, _ctx)
  if (kk_std_core__is_Cons(cap, _ctx)) {
    struct kk_std_core_Cons* _con1348 = kk_std_core__as_Cons(cap, _ctx);
    kk_box_t _box_x1193 = _con1348->head;
    kk_std_core__list _pat4 = _con1348->tail;
    if (kk_std_core__is_Cons(_pat4, _ctx)) {
      struct kk_std_core_Cons* _con1349 = kk_std_core__as_Cons(_pat4, _ctx);
      kk_std_core__sslice _pat000 = kk_std_core__sslice_unbox(_box_x1193, KK_BORROWED, _ctx);
      kk_box_t _box_x1194 = _con1349->head;
      kk_std_core__list _pat8 = _con1349->tail;
      if (kk_std_core__is_Cons(_pat8, _ctx)) {
        struct kk_std_core_Cons* _con1350 = kk_std_core__as_Cons(_pat8, _ctx);
        kk_std_core__sslice digit = kk_std_core__sslice_unbox(_box_x1194, KK_BORROWED, _ctx);
        kk_box_t _box_x1195 = _con1350->head;
        kk_std_core__list _pat12 = _con1350->tail;
        if (kk_std_core__is_Cons(_pat12, _ctx)) {
          struct kk_std_core_Cons* _con1351 = kk_std_core__as_Cons(_pat12, _ctx);
          kk_std_core__sslice amp = kk_std_core__sslice_unbox(_box_x1195, KK_BORROWED, _ctx);
          kk_box_t _box_x1196 = _con1351->head;
          kk_std_core__list _pat16 = _con1351->tail;
          if (kk_std_core__is_Nil(_pat16, _ctx)) {
            kk_std_core__sslice dollar = kk_std_core__sslice_unbox(_box_x1196, KK_BORROWED, _ctx);
            if kk_likely(kk_datatype_ptr_is_unique(cap, _ctx)) {
              if kk_likely(kk_datatype_ptr_is_unique(_pat4, _ctx)) {
                if kk_likely(kk_datatype_ptr_is_unique(_pat8, _ctx)) {
                  if kk_likely(kk_datatype_ptr_is_unique(_pat12, _ctx)) {
                    kk_std_core__sslice_dup(dollar, _ctx);
                    kk_box_drop(_box_x1196, _ctx);
                    kk_datatype_ptr_free(_pat12, _ctx);
                  }
                  else {
                    kk_std_core__sslice_dup(dollar, _ctx);
                    kk_datatype_ptr_decref(_pat12, _ctx);
                  }
                  kk_std_core__sslice_dup(amp, _ctx);
                  kk_box_drop(_box_x1195, _ctx);
                  kk_datatype_ptr_free(_pat8, _ctx);
                }
                else {
                  kk_std_core__sslice_dup(amp, _ctx);
                  kk_std_core__sslice_dup(dollar, _ctx);
                  kk_datatype_ptr_decref(_pat8, _ctx);
                }
                kk_std_core__sslice_dup(digit, _ctx);
                kk_box_drop(_box_x1194, _ctx);
                kk_datatype_ptr_free(_pat4, _ctx);
              }
              else {
                kk_std_core__sslice_dup(amp, _ctx);
                kk_std_core__sslice_dup(digit, _ctx);
                kk_std_core__sslice_dup(dollar, _ctx);
                kk_datatype_ptr_decref(_pat4, _ctx);
              }
              kk_box_drop(_box_x1193, _ctx);
              kk_datatype_ptr_free(cap, _ctx);
            }
            else {
              kk_std_core__sslice_dup(amp, _ctx);
              kk_std_core__sslice_dup(digit, _ctx);
              kk_std_core__sslice_dup(dollar, _ctx);
              kk_datatype_ptr_decref(cap, _ctx);
            }
            bool _match_1224;
            kk_integer_t _brw_1229;
            {
              kk_integer_t _x0 = dollar.start;
              kk_integer_dup(_x0, _ctx);
              kk_std_core__sslice_drop(dollar, _ctx);
              _brw_1229 = _x0; /*int*/
            }
            bool _brw_1230 = kk_integer_gte_borrow(_brw_1229,(kk_integer_from_small(0)),kk_context()); /*bool*/;
            kk_integer_drop(_brw_1229, _ctx);
            _match_1224 = _brw_1230; /*bool*/
            if (_match_1224) {
              kk_std_core__sslice_drop(digit, _ctx);
              kk_std_core__list_drop(caps, _ctx);
              kk_std_core__sslice_drop(amp, _ctx);
              kk_define_string_literal(, _s1352, 1, "$", _ctx)
              return kk_string_dup(_s1352, _ctx);
            }
            {
              kk_integer_t grp;
              bool _match_1226;
              kk_integer_t _brw_1227;
              {
                kk_integer_t _x00 = amp.start;
                kk_integer_dup(_x00, _ctx);
                kk_std_core__sslice_drop(amp, _ctx);
                _brw_1227 = _x00; /*int*/
              }
              bool _brw_1228 = kk_integer_gte_borrow(_brw_1227,(kk_integer_from_small(0)),kk_context()); /*bool*/;
              kk_integer_drop(_brw_1227, _ctx);
              _match_1226 = _brw_1228; /*bool*/
              if (_match_1226) {
                kk_std_core__sslice_drop(digit, _ctx);
                grp = kk_integer_from_small(0); /*int*/
              }
              else {
                kk_string_t _x1353 = kk_std_core_string_3(digit, _ctx); /*string*/
                kk_std_core_types__optional _x1354 = kk_std_core_types__new_Optional(kk_integer_box(kk_integer_from_small(0), _ctx), _ctx); /*optional<116>*/
                grp = kk_std_core_parse_int_default(_x1353, _x1354, kk_std_core_types__new_None(_ctx), _ctx); /*int*/
              }
              kk_std_core_types__maybe _match_1225 = kk_std_core__lp__lb__rb__2_rp_(caps, grp, _ctx); /*maybe<10073>*/;
              if (kk_std_core_types__is_Nothing(_match_1225, _ctx)) {
                return kk_string_empty();
              }
              {
                kk_box_t _box_x1199 = _match_1225._cons.Just.value;
                kk_std_core__sslice s0 = kk_std_core__sslice_unbox(_box_x1199, KK_BORROWED, _ctx);
                kk_std_core__sslice_dup(s0, _ctx);
                kk_std_core_types__maybe_drop(_match_1225, _ctx);
                return kk_std_core_string_3(s0, _ctx);
              }
            }
          }
        }
      }
    }
  }
  {
    kk_std_core__list_drop(caps, _ctx);
    kk_std_core__list_drop(cap, _ctx);
    kk_define_string_literal(, _s1356, 1, "$", _ctx)
    return kk_string_dup(_s1356, _ctx);
  }
}

kk_string_t kk_std_text_regex_replace_captures(kk_std_core__list caps, kk_string_t repl, kk_context_t* _ctx) { /* (caps : list<sslice>, repl : string) -> string */ 
  kk_std_text_regex__regex r_1051;
  kk_box_t _x1335;
  kk_string_t _x1336;
  kk_define_string_literal(, _s1337, 20, "\\$(\?:(\\d)|(\\&)|(\\$))", _ctx)
  _x1336 = kk_string_dup(_s1337, _ctx); /*string*/
  bool _x1338;
  kk_std_core_types__optional _match_1233 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_1233, _ctx)) {
    kk_box_t _box_x1190 = _match_1233._cons.Optional.value;
    bool _ignorecase_455 = kk_bool_unbox(_box_x1190);
    kk_std_core_types__optional_drop(_match_1233, _ctx);
    _x1338 = _ignorecase_455; /*bool*/
  }
  else {
    kk_std_core_types__optional_drop(_match_1233, _ctx);
    _x1338 = false; /*bool*/
  }
  bool _x1339;
  kk_std_core_types__optional _match_1232 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_1232, _ctx)) {
    kk_box_t _box_x1191 = _match_1232._cons.Optional.value;
    bool _multiline_459 = kk_bool_unbox(_box_x1191);
    kk_std_core_types__optional_drop(_match_1232, _ctx);
    _x1339 = _multiline_459; /*bool*/
  }
  else {
    kk_std_core_types__optional_drop(_match_1232, _ctx);
    _x1339 = false; /*bool*/
  }
  _x1335 = kk_std_text_regex_regex_create(_x1336, _x1338, _x1339, _ctx); /*any*/
  kk_string_t _x1340;
  kk_define_string_literal(, _s1341, 20, "\\$(\?:(\\d)|(\\&)|(\\$))", _ctx)
  _x1340 = kk_string_dup(_s1341, _ctx); /*string*/
  r_1051 = kk_std_text_regex__new_Regex(_x1335, _x1340, _ctx); /*std/text/regex/regex*/
  kk_std_core__list _x1342;
  kk_box_t _x1343;
  {
    kk_box_t _x = r_1051.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(r_1051, _ctx);
    _x1343 = _x; /*any*/
  }
  kk_ssize_t _x1344 = (KK_IZ(0)); /*ssize_t*/
  kk_ssize_t _x1345;
  kk_integer_t _x1346;
  kk_std_core_types__optional _match_1231 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_1231, _ctx)) {
    kk_box_t _box_x1192 = _match_1231._cons.Optional.value;
    kk_integer_t _atmost_475 = kk_integer_unbox(_box_x1192, _ctx);
    kk_integer_dup(_atmost_475, _ctx);
    kk_std_core_types__optional_drop(_match_1231, _ctx);
    _x1346 = _atmost_475; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(_match_1231, _ctx);
    _x1346 = kk_integer_from_small(-1); /*int*/
  }
  _x1345 = kk_std_core_ssize__t(_x1346, _ctx); /*ssize_t*/
  _x1342 = kk_std_text_regex_regex_exec_all(_x1343, repl, _x1344, _x1345, _ctx); /*list<list<sslice>>*/
  return kk_std_text_regex_concat_replace(_x1342, kk_std_text_regex_new_replace_captures_fun1347(caps, _ctx), kk_std_core__new_Nil(_ctx), _ctx);
}
 
// Replace all occurrences of `regex` with the replacement string `repl` in a string `s`.
// The replacement string can contain `$$` for a `$` sign, `$n` for a capture group,
// `$&` for the entire match `==$0`.


// lift anonymous function
struct kk_std_text_regex_replace_all_fun1365__t_1 {
  struct kk_function_s _base;
  kk_string_t repl;
};
static kk_string_t kk_std_text_regex_replace_all_fun1365_1(kk_function_t _fself, kk_std_core__list caps, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_replace_all_fun1365_1(kk_string_t repl, kk_context_t* _ctx) {
  struct kk_std_text_regex_replace_all_fun1365__t_1* _self = kk_function_alloc_as(struct kk_std_text_regex_replace_all_fun1365__t_1, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_text_regex_replace_all_fun1365_1, kk_context());
  _self->repl = repl;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_string_t kk_std_text_regex_replace_all_fun1365_1(kk_function_t _fself, kk_std_core__list caps, kk_context_t* _ctx) {
  struct kk_std_text_regex_replace_all_fun1365__t_1* _self = kk_function_as(struct kk_std_text_regex_replace_all_fun1365__t_1*, _fself, _ctx);
  kk_string_t repl = _self->repl; /* string */
  kk_drop_match(_self, {kk_string_dup(repl, _ctx);}, {}, _ctx)
  return kk_std_text_regex_replace_captures(caps, repl, _ctx);
}


// lift anonymous function
struct kk_std_text_regex_replace_all_fun1371__t_1 {
  struct kk_function_s _base;
  kk_string_t repl;
};
static kk_string_t kk_std_text_regex_replace_all_fun1371_1(kk_function_t _fself, kk_std_core__list ___wildcard__158__35, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_replace_all_fun1371_1(kk_string_t repl, kk_context_t* _ctx) {
  struct kk_std_text_regex_replace_all_fun1371__t_1* _self = kk_function_alloc_as(struct kk_std_text_regex_replace_all_fun1371__t_1, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_text_regex_replace_all_fun1371_1, kk_context());
  _self->repl = repl;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_string_t kk_std_text_regex_replace_all_fun1371_1(kk_function_t _fself, kk_std_core__list ___wildcard__158__35, kk_context_t* _ctx) {
  struct kk_std_text_regex_replace_all_fun1371__t_1* _self = kk_function_as(struct kk_std_text_regex_replace_all_fun1371__t_1*, _fself, _ctx);
  kk_string_t repl = _self->repl; /* string */
  kk_drop_match(_self, {kk_string_dup(repl, _ctx);}, {}, _ctx)
  kk_std_core__list_drop(___wildcard__158__35, _ctx);
  return repl;
}

kk_string_t kk_std_text_regex_replace_all_1(kk_string_t s, kk_std_text_regex__regex regex0, kk_string_t repl, kk_std_core_types__optional atmost, kk_context_t* _ctx) { /* (s : string, regex : regex, repl : string, atmost : optional<int>) -> string */ 
  bool _match_1223;
  kk_string_t _x1357 = kk_string_dup(repl, _ctx); /*string*/
  kk_string_t _x1358;
  kk_define_string_literal(, _s1359, 1, "$", _ctx)
  _x1358 = kk_string_dup(_s1359, _ctx); /*string*/
  _match_1223 = kk_string_contains(_x1357,_x1358,kk_context()); /*bool*/
  if (_match_1223) {
    kk_std_core__list _x1360;
    kk_box_t _x1361;
    {
      kk_box_t _x = regex0.obj;
      kk_box_dup(_x, _ctx);
      kk_std_text_regex__regex_drop(regex0, _ctx);
      _x1361 = _x; /*any*/
    }
    kk_ssize_t _x1362 = (KK_IZ(0)); /*ssize_t*/
    kk_ssize_t _x1363;
    kk_integer_t _x1364;
    if (kk_std_core_types__is_Optional(atmost, _ctx)) {
      kk_box_t _box_x1200 = atmost._cons.Optional.value;
      kk_integer_t _atmost_496 = kk_integer_unbox(_box_x1200, _ctx);
      kk_integer_dup(_atmost_496, _ctx);
      kk_std_core_types__optional_drop(atmost, _ctx);
      _x1364 = _atmost_496; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(atmost, _ctx);
      _x1364 = kk_integer_from_small(-1); /*int*/
    }
    _x1363 = kk_std_core_ssize__t(_x1364, _ctx); /*ssize_t*/
    _x1360 = kk_std_text_regex_regex_exec_all(_x1361, s, _x1362, _x1363, _ctx); /*list<list<sslice>>*/
    return kk_std_text_regex_concat_replace(_x1360, kk_std_text_regex_new_replace_all_fun1365_1(repl, _ctx), kk_std_core__new_Nil(_ctx), _ctx);
  }
  {
    kk_std_core__list _x1366;
    kk_box_t _x1367;
    {
      kk_box_t _x0 = regex0.obj;
      kk_box_dup(_x0, _ctx);
      kk_std_text_regex__regex_drop(regex0, _ctx);
      _x1367 = _x0; /*any*/
    }
    kk_ssize_t _x1368 = (KK_IZ(0)); /*ssize_t*/
    kk_ssize_t _x1369;
    kk_integer_t _x1370;
    if (kk_std_core_types__is_Optional(atmost, _ctx)) {
      kk_box_t _box_x1201 = atmost._cons.Optional.value;
      kk_integer_t _atmost_4960 = kk_integer_unbox(_box_x1201, _ctx);
      kk_integer_dup(_atmost_4960, _ctx);
      kk_std_core_types__optional_drop(atmost, _ctx);
      _x1370 = _atmost_4960; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(atmost, _ctx);
      _x1370 = kk_integer_from_small(-1); /*int*/
    }
    _x1369 = kk_std_core_ssize__t(_x1370, _ctx); /*ssize_t*/
    _x1366 = kk_std_text_regex_regex_exec_all(_x1367, s, _x1368, _x1369, _ctx); /*list<list<sslice>>*/
    return kk_std_text_regex_concat_replace(_x1366, kk_std_text_regex_new_replace_all_fun1371_1(repl, _ctx), kk_std_core__new_Nil(_ctx), _ctx);
  }
}

kk_std_text_regex__regex kk_std_text_regex_rx_nongroup;
 
// How many groups are captured by this regex?


// lift anonymous function
struct kk_std_text_regex_groups_count_fun1391__t {
  struct kk_function_s _base;
};
static kk_string_t kk_std_text_regex_groups_count_fun1391(kk_function_t _fself, kk_std_core__list caps, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_groups_count_fun1391(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_text_regex_groups_count_fun1391, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_string_t kk_std_text_regex_groups_count_fun1391(kk_function_t _fself, kk_std_core__list caps, kk_context_t* _ctx) {
  kk_unused(_fself);
  kk_string_t _x1392 = kk_string_empty(); /*string*/
  return kk_std_text_regex_replace_captures(caps, _x1392, _ctx);
}


// lift anonymous function
struct kk_std_text_regex_groups_count_fun1401__t {
  struct kk_function_s _base;
};
static kk_string_t kk_std_text_regex_groups_count_fun1401(kk_function_t _fself, kk_std_core__list ___wildcard__158__35, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_groups_count_fun1401(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_text_regex_groups_count_fun1401, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_string_t kk_std_text_regex_groups_count_fun1401(kk_function_t _fself, kk_std_core__list ___wildcard__158__35, kk_context_t* _ctx) {
  kk_unused(_fself);
  kk_std_core__list_drop(___wildcard__158__35, _ctx);
  return kk_string_empty();
}

kk_integer_t kk_std_text_regex_groups_count(kk_std_text_regex__regex r, kk_context_t* _ctx) { /* (r : regex) -> int */ 
  kk_string_t _x1379;
  bool _match_1218;
  kk_string_t _x1380 = kk_string_empty(); /*string*/
  kk_string_t _x1382;
  kk_define_string_literal(, _s1383, 1, "$", _ctx)
  _x1382 = kk_string_dup(_s1383, _ctx); /*string*/
  _match_1218 = kk_string_contains(_x1380,_x1382,kk_context()); /*bool*/
  if (_match_1218) {
    kk_std_core__list _x1384;
    kk_box_t _x1385;
    kk_std_text_regex__regex _x1386 = kk_std_text_regex_rx_nongroup; /*std/text/regex/regex*/
    {
      kk_box_t _x = _x1386.obj;
      _x1385 = kk_box_dup(_x, _ctx); /*any*/
    }
    kk_string_t _x1387;
    {
      kk_string_t _x0 = r.src;
      kk_string_dup(_x0, _ctx);
      kk_std_text_regex__regex_drop(r, _ctx);
      _x1387 = _x0; /*string*/
    }
    kk_ssize_t _x1388 = (KK_IZ(0)); /*ssize_t*/
    kk_ssize_t _x1389;
    kk_integer_t _x1390;
    kk_std_core_types__optional _match_1220 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_1220, _ctx)) {
      kk_box_t _box_x1204 = _match_1220._cons.Optional.value;
      kk_integer_t _atmost_496 = kk_integer_unbox(_box_x1204, _ctx);
      kk_integer_dup(_atmost_496, _ctx);
      kk_std_core_types__optional_drop(_match_1220, _ctx);
      _x1390 = _atmost_496; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(_match_1220, _ctx);
      _x1390 = kk_integer_from_small(-1); /*int*/
    }
    _x1389 = kk_std_core_ssize__t(_x1390, _ctx); /*ssize_t*/
    _x1384 = kk_std_text_regex_regex_exec_all(_x1385, _x1387, _x1388, _x1389, _ctx); /*list<list<sslice>>*/
    _x1379 = kk_std_text_regex_concat_replace(_x1384, kk_std_text_regex_new_groups_count_fun1391(_ctx), kk_std_core__new_Nil(_ctx), _ctx); /*string*/
  }
  else {
    kk_std_core__list _x1394;
    kk_box_t _x1395;
    kk_std_text_regex__regex _x1396 = kk_std_text_regex_rx_nongroup; /*std/text/regex/regex*/
    {
      kk_box_t _x1 = _x1396.obj;
      _x1395 = kk_box_dup(_x1, _ctx); /*any*/
    }
    kk_string_t _x1397;
    {
      kk_string_t _x00 = r.src;
      kk_string_dup(_x00, _ctx);
      kk_std_text_regex__regex_drop(r, _ctx);
      _x1397 = _x00; /*string*/
    }
    kk_ssize_t _x1398 = (KK_IZ(0)); /*ssize_t*/
    kk_ssize_t _x1399;
    kk_integer_t _x1400;
    kk_std_core_types__optional _match_1219 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_1219, _ctx)) {
      kk_box_t _box_x1205 = _match_1219._cons.Optional.value;
      kk_integer_t _atmost_4960 = kk_integer_unbox(_box_x1205, _ctx);
      kk_integer_dup(_atmost_4960, _ctx);
      kk_std_core_types__optional_drop(_match_1219, _ctx);
      _x1400 = _atmost_4960; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(_match_1219, _ctx);
      _x1400 = kk_integer_from_small(-1); /*int*/
    }
    _x1399 = kk_std_core_ssize__t(_x1400, _ctx); /*ssize_t*/
    _x1394 = kk_std_text_regex_regex_exec_all(_x1395, _x1397, _x1398, _x1399, _ctx); /*list<list<sslice>>*/
    _x1379 = kk_std_text_regex_concat_replace(_x1394, kk_std_text_regex_new_groups_count_fun1401(_ctx), kk_std_core__new_Nil(_ctx), _ctx); /*string*/
  }
  return kk_std_core_count_1(_x1379, _ctx);
}
 
// Replace the first occurrence of `regex` by the result of the replacement fun `repl` in a string `s`.

kk_string_t kk_std_text_regex_replace(kk_string_t s, kk_std_text_regex__regex r, kk_function_t repl, kk_context_t* _ctx) { /* forall<e> (s : string, r : regex, repl : (list<sslice>) -> e string) -> e string */ 
  kk_std_core__list _x1403;
  kk_box_t _x1404;
  {
    kk_box_t _x = r.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(r, _ctx);
    _x1404 = _x; /*any*/
  }
  kk_ssize_t _x1405 = (KK_IZ(0)); /*ssize_t*/
  kk_ssize_t _x1406 = (KK_IZ(1)); /*ssize_t*/
  _x1403 = kk_std_text_regex_regex_exec_all(_x1404, s, _x1405, _x1406, _ctx); /*list<list<sslice>>*/
  return kk_std_text_regex_concat_replace(_x1403, repl, kk_std_core__new_Nil(_ctx), _ctx);
}
 
// Replace the first occurrence of `regex` with a replacement string `repl` in a string `s`.
// The replacement string can contain `$$` for a `$` sign, `$n` for a capture group,
// `$&` for the entire match `==$0`.


// lift anonymous function
struct kk_std_text_regex_replace_fun1414__t_1 {
  struct kk_function_s _base;
  kk_string_t repl;
};
static kk_string_t kk_std_text_regex_replace_fun1414_1(kk_function_t _fself, kk_std_core__list caps, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_replace_fun1414_1(kk_string_t repl, kk_context_t* _ctx) {
  struct kk_std_text_regex_replace_fun1414__t_1* _self = kk_function_alloc_as(struct kk_std_text_regex_replace_fun1414__t_1, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_text_regex_replace_fun1414_1, kk_context());
  _self->repl = repl;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_string_t kk_std_text_regex_replace_fun1414_1(kk_function_t _fself, kk_std_core__list caps, kk_context_t* _ctx) {
  struct kk_std_text_regex_replace_fun1414__t_1* _self = kk_function_as(struct kk_std_text_regex_replace_fun1414__t_1*, _fself, _ctx);
  kk_string_t repl = _self->repl; /* string */
  kk_drop_match(_self, {kk_string_dup(repl, _ctx);}, {}, _ctx)
  return kk_std_text_regex_replace_captures(caps, repl, _ctx);
}


// lift anonymous function
struct kk_std_text_regex_replace_fun1419__t_1 {
  struct kk_function_s _base;
  kk_string_t repl;
};
static kk_string_t kk_std_text_regex_replace_fun1419_1(kk_function_t _fself, kk_std_core__list ___wildcard__158__35, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_replace_fun1419_1(kk_string_t repl, kk_context_t* _ctx) {
  struct kk_std_text_regex_replace_fun1419__t_1* _self = kk_function_alloc_as(struct kk_std_text_regex_replace_fun1419__t_1, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_text_regex_replace_fun1419_1, kk_context());
  _self->repl = repl;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_string_t kk_std_text_regex_replace_fun1419_1(kk_function_t _fself, kk_std_core__list ___wildcard__158__35, kk_context_t* _ctx) {
  struct kk_std_text_regex_replace_fun1419__t_1* _self = kk_function_as(struct kk_std_text_regex_replace_fun1419__t_1*, _fself, _ctx);
  kk_string_t repl = _self->repl; /* string */
  kk_drop_match(_self, {kk_string_dup(repl, _ctx);}, {}, _ctx)
  kk_std_core__list_drop(___wildcard__158__35, _ctx);
  return repl;
}

kk_string_t kk_std_text_regex_replace_1(kk_string_t s, kk_std_text_regex__regex regex0, kk_string_t repl, kk_context_t* _ctx) { /* (s : string, regex : regex, repl : string) -> string */ 
  bool _match_1217;
  kk_string_t _x1407 = kk_string_dup(repl, _ctx); /*string*/
  kk_string_t _x1408;
  kk_define_string_literal(, _s1409, 1, "$", _ctx)
  _x1408 = kk_string_dup(_s1409, _ctx); /*string*/
  _match_1217 = kk_string_contains(_x1407,_x1408,kk_context()); /*bool*/
  if (_match_1217) {
    kk_std_core__list _x1410;
    kk_box_t _x1411;
    {
      kk_box_t _x = regex0.obj;
      kk_box_dup(_x, _ctx);
      kk_std_text_regex__regex_drop(regex0, _ctx);
      _x1411 = _x; /*any*/
    }
    kk_ssize_t _x1412 = (KK_IZ(0)); /*ssize_t*/
    kk_ssize_t _x1413 = (KK_IZ(1)); /*ssize_t*/
    _x1410 = kk_std_text_regex_regex_exec_all(_x1411, s, _x1412, _x1413, _ctx); /*list<list<sslice>>*/
    return kk_std_text_regex_concat_replace(_x1410, kk_std_text_regex_new_replace_fun1414_1(repl, _ctx), kk_std_core__new_Nil(_ctx), _ctx);
  }
  {
    kk_std_core__list _x1415;
    kk_box_t _x1416;
    {
      kk_box_t _x0 = regex0.obj;
      kk_box_dup(_x0, _ctx);
      kk_std_text_regex__regex_drop(regex0, _ctx);
      _x1416 = _x0; /*any*/
    }
    kk_ssize_t _x1417 = (KK_IZ(0)); /*ssize_t*/
    kk_ssize_t _x1418 = (KK_IZ(1)); /*ssize_t*/
    _x1415 = kk_std_text_regex_regex_exec_all(_x1416, s, _x1417, _x1418, _ctx); /*list<list<sslice>>*/
    return kk_std_text_regex_concat_replace(_x1415, kk_std_text_regex_new_replace_fun1419_1(repl, _ctx), kk_std_core__new_Nil(_ctx), _ctx);
  }
}
 
// Split a string `s` in at most `atmost` parts using a regular expression `r` as separator.


// lift anonymous function
struct kk_std_text_regex_split_fun1425__t {
  struct kk_function_s _base;
};
static kk_box_t kk_std_text_regex_split_fun1425(kk_function_t _fself, kk_box_t _b_1209, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_split_fun1425(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_text_regex_split_fun1425, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_box_t kk_std_text_regex_split_fun1425(kk_function_t _fself, kk_box_t _b_1209, kk_context_t* _ctx) {
  kk_unused(_fself);
  kk_string_t _x1426;
  kk_std_core__list _x1427 = kk_std_core__list_unbox(_b_1209, KK_OWNED, _ctx); /*list<sslice>*/
  _x1426 = kk_std_text_regex_captured(_x1427, _ctx); /*string*/
  return kk_string_box(_x1426);
}

kk_std_core__list kk_std_text_regex_split(kk_string_t s, kk_std_text_regex__regex r, kk_std_core_types__optional atmost, kk_context_t* _ctx) { /* (s : string, r : regex, atmost : optional<int>) -> list<string> */ 
  kk_std_core__list xs_1016;
  kk_std_core__list _x1420;
  kk_box_t _x1421;
  {
    kk_box_t _x = r.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(r, _ctx);
    _x1421 = _x; /*any*/
  }
  kk_ssize_t _x1422 = (KK_IZ(0)); /*ssize_t*/
  kk_ssize_t _x1423;
  kk_integer_t _x1424;
  if (kk_std_core_types__is_Optional(atmost, _ctx)) {
    kk_box_t _box_x1206 = atmost._cons.Optional.value;
    kk_integer_t _atmost_938 = kk_integer_unbox(_box_x1206, _ctx);
    kk_integer_dup(_atmost_938, _ctx);
    kk_std_core_types__optional_drop(atmost, _ctx);
    _x1424 = _atmost_938; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(atmost, _ctx);
    _x1424 = kk_integer_from_small(-1); /*int*/
  }
  _x1423 = kk_std_core_ssize__t(_x1424, _ctx); /*ssize_t*/
  _x1420 = kk_std_text_regex_regex_exec_all(_x1421, s, _x1422, _x1423, _ctx); /*list<list<sslice>>*/
  xs_1016 = kk_std_text_regex_filter_non_matches(_x1420, _ctx); /*list<list<sslice>>*/
  return kk_std_core_map_5(xs_1016, kk_std_text_regex_new_split_fun1425(_ctx), _ctx);
}

bool kk_std_text_regex_testabc(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> bool */ 
  kk_std_text_regex__regex r_1022;
  kk_box_t _x1428;
  kk_string_t _x1429;
  kk_define_string_literal(, _s1430, 6, "[ab]+c", _ctx)
  _x1429 = kk_string_dup(_s1430, _ctx); /*string*/
  bool _x1431;
  kk_std_core_types__optional _match_1216 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_1216, _ctx)) {
    kk_box_t _box_x1212 = _match_1216._cons.Optional.value;
    bool _ignorecase_455 = kk_bool_unbox(_box_x1212);
    kk_std_core_types__optional_drop(_match_1216, _ctx);
    _x1431 = _ignorecase_455; /*bool*/
  }
  else {
    kk_std_core_types__optional_drop(_match_1216, _ctx);
    _x1431 = false; /*bool*/
  }
  bool _x1432;
  kk_std_core_types__optional _match_1215 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_1215, _ctx)) {
    kk_box_t _box_x1213 = _match_1215._cons.Optional.value;
    bool _multiline_459 = kk_bool_unbox(_box_x1213);
    kk_std_core_types__optional_drop(_match_1215, _ctx);
    _x1432 = _multiline_459; /*bool*/
  }
  else {
    kk_std_core_types__optional_drop(_match_1215, _ctx);
    _x1432 = false; /*bool*/
  }
  _x1428 = kk_std_text_regex_regex_create(_x1429, _x1431, _x1432, _ctx); /*any*/
  kk_string_t _x1433;
  kk_define_string_literal(, _s1434, 6, "[ab]+c", _ctx)
  _x1433 = kk_string_dup(_s1434, _ctx); /*string*/
  r_1022 = kk_std_text_regex__new_Regex(_x1428, _x1433, _ctx); /*std/text/regex/regex*/
  kk_std_core__list list0_1026;
  kk_box_t _x1435;
  {
    kk_box_t _x = r_1022.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(r_1022, _ctx);
    _x1435 = _x; /*any*/
  }
  kk_ssize_t _x1436 = (KK_IZ(0)); /*ssize_t*/
  list0_1026 = kk_std_text_regex_regex_exec(_x1435, s, _x1436, _ctx); /*list<sslice>*/
  if (kk_std_core__is_Cons(list0_1026, _ctx)) {
    struct kk_std_core_Cons* _con1437 = kk_std_core__as_Cons(list0_1026, _ctx);
    kk_box_t _box_x1214 = _con1437->head;
    kk_std_core__list _pat1 = _con1437->tail;
    if kk_likely(kk_datatype_ptr_is_unique(list0_1026, _ctx)) {
      kk_std_core__list_drop(_pat1, _ctx);
      kk_box_drop(_box_x1214, _ctx);
      kk_datatype_ptr_free(list0_1026, _ctx);
    }
    else {
      kk_datatype_ptr_decref(list0_1026, _ctx);
    }
    return true;
  }
  {
    return false;
  }
}

// initialization
void kk_std_text_regex__init(kk_context_t* _ctx){
  static bool _kk_initialized = false;
  if (_kk_initialized) return;
  _kk_initialized = true;
  kk_std_core_types__init(_ctx);
  kk_std_core_hnd__init(_ctx);
  kk_std_core__init(_ctx);
  #if defined(KK_CUSTOM_INIT)
    KK_CUSTOM_INIT (_ctx);
  #endif
  {
    kk_box_t _x1372;
    kk_string_t _x1373;
    kk_define_string_literal(, _s1374, 45, "[^\\\\\\[(]+|\\\\[\\s\\S]\?|\\(\\\?|\\[(\?:[^\\\\\\]]|\\\\.)*\\]", _ctx)
    _x1373 = kk_string_dup(_s1374, _ctx); /*string*/
    bool _x1375;
    kk_std_core_types__optional _match_1222 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_1222, _ctx)) {
      kk_box_t _box_x1202 = _match_1222._cons.Optional.value;
      bool _ignorecase_455 = kk_bool_unbox(_box_x1202);
      kk_std_core_types__optional_drop(_match_1222, _ctx);
      _x1375 = _ignorecase_455; /*bool*/
    }
    else {
      kk_std_core_types__optional_drop(_match_1222, _ctx);
      _x1375 = false; /*bool*/
    }
    bool _x1376;
    kk_std_core_types__optional _match_1221 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_1221, _ctx)) {
      kk_box_t _box_x1203 = _match_1221._cons.Optional.value;
      bool _multiline_459 = kk_bool_unbox(_box_x1203);
      kk_std_core_types__optional_drop(_match_1221, _ctx);
      _x1376 = _multiline_459; /*bool*/
    }
    else {
      kk_std_core_types__optional_drop(_match_1221, _ctx);
      _x1376 = false; /*bool*/
    }
    _x1372 = kk_std_text_regex_regex_create(_x1373, _x1375, _x1376, _ctx); /*any*/
    kk_string_t _x1377;
    kk_define_string_literal(, _s1378, 45, "[^\\\\\\[(]+|\\\\[\\s\\S]\?|\\(\\\?|\\[(\?:[^\\\\\\]]|\\\\.)*\\]", _ctx)
    _x1377 = kk_string_dup(_s1378, _ctx); /*string*/
    kk_std_text_regex_rx_nongroup = kk_std_text_regex__new_Regex(_x1372, _x1377, _ctx); /*std/text/regex/regex*/
  }
}

// termination
void kk_std_text_regex__done(kk_context_t* _ctx){
  static bool _kk_done = false;
  if (_kk_done) return;
  _kk_done = true;
  #if defined(KK_CUSTOM_DONE)
    KK_CUSTOM_DONE (_ctx);
  #endif
  kk_std_text_regex__regex_drop(kk_std_text_regex_rx_nongroup, _ctx);
  kk_std_core__done(_ctx);
  kk_std_core_hnd__done(_ctx);
  kk_std_core_types__done(_ctx);
}
