// Koka generated module: "std/text/regex", koka version: 2.4.2, platform: 64-bit
#include "std_text_regex.h"
/*---------------------------------------------------------------------------
  Copyright 2021, Microsoft Research, Daan Leijen.

  This is free software; you can redistribute it and/or modify it under the
  terms of the Apache License, Version 2.0. A copy of the License can be
  found in the LICENSE file at the root of this distribution.
---------------------------------------------------------------------------*/

#pragma clang diagnostic ignored "-Wdisabled-macro-expansion"
#define PCRE2_CODE_UNIT_WIDTH 8
#define PCRE2_STATIC
#include <pcre2.h>

/* -----------------------------------------------------------------------
  Init/Done
------------------------------------------------------------------------*/
#define KK_CUSTOM_INIT  kk_regex_custom_init
#define KK_CUSTOM_DONE  kk_regex_custom_done

static pcre2_general_context* gen_ctx;
static pcre2_compile_context* cmp_ctx;
static pcre2_match_context*   match_ctx; 

static void* kk_pcre2_malloc( PCRE2_SIZE size, void* data ) {
  return kk_malloc( kk_to_ssize_t(size), kk_get_context() );  
}
static void kk_pcre2_free( void* p, void* data ) {
  kk_free(p,kk_get_context());
}

static void kk_regex_custom_init( kk_context_t* ctx ) {
  gen_ctx = pcre2_general_context_create( &kk_pcre2_malloc, &kk_pcre2_free, NULL );
  if (gen_ctx != NULL) {
    match_ctx = pcre2_match_context_create( gen_ctx );
    cmp_ctx = pcre2_compile_context_create( gen_ctx );
    if (cmp_ctx != NULL) {
      pcre2_set_newline( cmp_ctx, PCRE2_NEWLINE_ANYCRLF );
      pcre2_set_bsr( cmp_ctx, PCRE2_BSR_ANYCRLF );
    }
  }
}

static void kk_regex_custom_done( kk_context_t* ctx ) {
  if (cmp_ctx != NULL) {
    pcre2_compile_context_free(cmp_ctx);
    cmp_ctx = NULL;
  }
  if (match_ctx != NULL) {
    pcre2_match_context_free(match_ctx);
    match_ctx = NULL;
  }
  if (gen_ctx != NULL) {
    pcre2_general_context_free(gen_ctx);
    gen_ctx = NULL;
  }
}


/* -----------------------------------------------------------------------
  Compile
------------------------------------------------------------------------*/

static void kk_regex_free( void* pre, kk_block_t* b, kk_context_t* ctx ) {
  kk_unused(ctx);
  pcre2_code* re = (pcre2_code*)pre;
  //kk_info_message( "free regex at %p\n", re );
  if (re != NULL) { pcre2_code_free(re); }
}

#define KK_REGEX_OPTIONS  (PCRE2_ALT_BSUX | PCRE2_EXTRA_ALT_BSUX | PCRE2_MATCH_UNSET_BACKREF /* javascript compat */ \
                          | PCRE2_NEVER_BACKSLASH_C | PCRE2_NEVER_UCP | PCRE2_UTF /* utf-8 safety */ \
                          )

static kk_box_t kk_regex_create( kk_string_t pat, bool ignore_case, bool multi_line, kk_context_t* ctx ) {
  kk_ssize_t len;
  const uint8_t* cpat = kk_string_buf_borrow( pat, &len, ctx );
  PCRE2_SIZE errofs = 0;
  int        errnum = 0;
  uint32_t   options = KK_REGEX_OPTIONS;
  if (ignore_case) options |= PCRE2_CASELESS;
  if (multi_line)  options |= PCRE2_MULTILINE;
  pcre2_code* re = pcre2_compile( cpat, PCRE2_ZERO_TERMINATED, options, &errnum, &errofs, cmp_ctx);
  //kk_info_message( "create regex: err:%i, at %p\n", (re==NULL ? 0 : errnum), re );
  kk_string_drop(pat,ctx);
  return kk_cptr_raw_box( &kk_regex_free, re, ctx );
}


/* -----------------------------------------------------------------------
  Match
------------------------------------------------------------------------*/

/*
static void kk_match_data_free( void* pmd, kk_block_t* b ) {
  kk_info_message( "free match_data\n" );
  pcre2_match_data* md = (pcre2_match_data*)pmd;
  if (md != NULL) pcre2_match_data_free(md);
}
*/

static kk_std_core__list kk_regex_exec_ex( pcre2_code* re, pcre2_match_data* match_data, 
                                           kk_string_t str_borrow, const uint8_t* cstr, kk_ssize_t len, bool allow_empty, 
                                           kk_ssize_t start, kk_ssize_t* mstart, kk_ssize_t* end, int* res, kk_context_t* ctx ) 
{
  // match
  kk_std_core__list hd  = kk_std_core__new_Nil(ctx);
  uint32_t options = 0;
  if (!allow_empty) options |= (PCRE2_NOTEMPTY_ATSTART | PCRE2_ANCHORED);
  int rc = pcre2_match( re, cstr, len, start, options, match_data, match_ctx );
  if (res != NULL) *res = rc;    
  if (rc > 0) {    
    // extract captures
    uint32_t    gcount = pcre2_get_ovector_count(match_data);
    PCRE2_SIZE* groups = pcre2_get_ovector_pointer(match_data);
    for( uint32_t i = gcount; i > 0; ) {
      i--;
      kk_ssize_t sstart = groups[i*2];       // on no-match, sstart and send == -1.
      kk_ssize_t send   = groups[i*2 + 1];
      kk_assert(send >= sstart);
      kk_std_core__sslice sslice = kk_std_core__new_Sslice( kk_string_dup(str_borrow,ctx), kk_integer_from_ssize_t(sstart,ctx), kk_integer_from_ssize_t(send - sstart,ctx), ctx ); 
      hd = kk_std_core__new_Cons(kk_reuse_null,0,kk_std_core__sslice_box(sslice,ctx), hd, ctx);
      if (i == 0) {
        if (mstart != NULL) { *mstart = sstart; }
        if (end    != NULL) { *end = send; }
      }
    }
  }
  return hd;
}


static kk_std_core__list kk_regex_exec( kk_box_t bre, kk_string_t str, kk_ssize_t start, kk_context_t* ctx ) 
{
  // unpack
  pcre2_match_data* match_data = NULL;
  kk_std_core__list res = kk_std_core__new_Nil(ctx);
  pcre2_code* re = (pcre2_code*)kk_cptr_raw_unbox_borrowed(bre,ctx);
  kk_ssize_t len = 0;
  const uint8_t* cstr = NULL;
  if (re == NULL) goto done;    
  match_data = pcre2_match_data_create_from_pattern(re, gen_ctx);
  if (match_data==NULL) goto done;  
  cstr = kk_string_buf_borrow(str, &len, ctx );  

  // and match
  res = kk_regex_exec_ex( re, match_data, str, cstr, len, true, start, NULL, NULL, NULL, ctx );

done:  
  if (match_data != NULL) {
    //kk_info_message( "free match data: %p\n", match_data );
    pcre2_match_data_free(match_data);
  }
  kk_string_drop(str,ctx);
  kk_box_drop(bre,ctx);
  return res;
}

static kk_std_core__list kk_regex_exec_all( kk_box_t bre, kk_string_t str, kk_ssize_t start, kk_ssize_t atmost, kk_context_t* ctx ) {
  // unpack
  if (atmost < 0) atmost = KK_SSIZE_MAX;
  pcre2_match_data* match_data = NULL;
  kk_std_core__list res = kk_std_core__new_Nil(ctx);
  pcre2_code* re = (pcre2_code*)kk_cptr_raw_unbox_borrowed(bre,ctx);
  if (re == NULL) goto done;    
  match_data = pcre2_match_data_create_from_pattern(re, gen_ctx);
  if (match_data==NULL) goto done;  
  {
    kk_ssize_t len;
    const uint8_t* cstr = kk_string_buf_borrow(str, &len, ctx );  

    // and match
    kk_std_core__list* tail = NULL;
    bool allow_empty = true;
    int rc = 1;    
    kk_ssize_t next = start;
    while( rc > 0 && start < len && atmost > 0) {
      atmost--;
      rc = 0;
      kk_ssize_t mstart = start;
      kk_std_core__list cap = kk_regex_exec_ex( re, match_data, str, cstr, len, allow_empty, start, &mstart, &next, &rc, ctx );
      if (rc > 0) {
        // found a match; 
        // push string up to match, and the actual matched regex
        kk_std_core__sslice pre = kk_std_core__new_Sslice( kk_string_dup(str,ctx), kk_integer_from_ssize_t(start,ctx), kk_integer_from_ssize_t(mstart - start,ctx), ctx ); 
        kk_std_core__list   prelist = kk_std_core__new_Cons( kk_reuse_null, 0, kk_std_core__sslice_box(pre,ctx), kk_std_core__new_Nil(ctx), ctx );
        kk_std_core__list   capcons = kk_std_core__new_Cons( kk_reuse_null, 0, kk_std_core__list_box(cap,ctx), kk_std_core__new_Nil(ctx) /*tail*/, ctx );
        kk_std_core__list   cons = kk_std_core__new_Cons( kk_reuse_null, 0, kk_std_core__list_box(prelist,ctx), capcons, ctx );
        if (tail==NULL) res = cons;
                  else *tail = cons;
        tail = &kk_std_core__as_Cons(capcons,ctx)->tail;
        allow_empty = (next > start);
        start = next;
      }
      else if (rc <= 0 && !allow_empty) {
        // skip one character and try again
        // todo: handle cr/lf pairs better?
        const uint8_t* p = kk_utf8_next( cstr + start );
        start = (p - cstr);
        allow_empty = true;
        rc = 1;
      }
    }
    
    // push final string part as well and end the list
    kk_std_core__sslice post    = kk_std_core__new_Sslice( kk_string_dup(str,ctx), kk_integer_from_ssize_t(next,ctx), kk_integer_from_ssize_t(len - next,ctx), ctx ); 
    kk_std_core__list   postlist= kk_std_core__new_Cons( kk_reuse_null, 0, kk_std_core__sslice_box(post,ctx), kk_std_core__new_Nil(ctx), ctx );
    kk_std_core__list   cons    = kk_std_core__new_Cons( kk_reuse_null, 0, kk_std_core__list_box(postlist,ctx), kk_std_core__new_Nil(ctx), ctx );
    if (tail==NULL) res = cons;
              else *tail = cons;  
  }

done:  
  if (match_data != NULL) {
    //kk_info_message( "free match data: %p\n", match_data );
    pcre2_match_data_free(match_data);
  }
  kk_string_drop(str,ctx);
  kk_box_drop(bre,ctx);
  return res;
}

/*
kk_std_core__sslice kk_slice_upto( struct kk_std_core_Sslice slice1, struct kk_std_core_Sslice slice2, kk_context_t* ctx ) {
  kk_ssize_t start = slice1.start;
  kk_ssize_t len   = (slice1.start <= slice2.start ? slice2.start - slice1.start : -1);
  return kk_std_core__new_Sslice(slice1.str, start, len, ctx);
}
*/

kk_std_text_regex__regex kk_std_text_regex__copy(kk_std_text_regex__regex _this, kk_std_core_types__optional obj0, kk_std_core_types__optional src0, kk_context_t* _ctx) { /* (regex, obj : optional<any>, src : optional<string>) -> regex */ 
  kk_box_t _x1944;
  if (kk_std_core_types__is_Optional(obj0, _ctx)) {
    kk_box_t _box_x1533 = obj0._cons.Optional.value;
    kk_box_t _obj_104 = kk_box_unbox(_box_x1533, _ctx);
    kk_box_dup(_obj_104, _ctx);
    kk_std_core_types__optional_drop(obj0, _ctx);
    _x1944 = _obj_104; /*any*/
  }
  else {
    kk_std_core_types__optional_drop(obj0, _ctx);
    {
      kk_box_t _x = _this.obj;
      kk_box_dup(_x, _ctx);
      _x1944 = _x; /*any*/
    }
  }
  kk_string_t _x1945;
  if (kk_std_core_types__is_Optional(src0, _ctx)) {
    kk_box_t _box_x1534 = src0._cons.Optional.value;
    kk_string_t _src_110 = kk_string_unbox(_box_x1534);
    kk_string_dup(_src_110, _ctx);
    kk_std_core_types__optional_drop(src0, _ctx);
    kk_std_text_regex__regex_drop(_this, _ctx);
    _x1945 = _src_110; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(src0, _ctx);
    {
      kk_string_t _x0 = _this.src;
      kk_string_dup(_x0, _ctx);
      kk_std_text_regex__regex_drop(_this, _ctx);
      _x1945 = _x0; /*string*/
    }
  }
  return kk_std_text_regex__new_Regex(_x1944, _x1945, _ctx);
}
 
// Return the full matched string of a capture group

kk_string_t kk_std_text_regex_captured(kk_std_core__list matched0, kk_context_t* _ctx) { /* (matched : list<sslice>) -> string */ 
  if (kk_std_core__is_Cons(matched0, _ctx)) {
    struct kk_std_core_Cons* _con1946 = kk_std_core__as_Cons(matched0, _ctx);
    kk_box_t _box_x1535 = _con1946->head;
    kk_std_core__sslice s = kk_std_core__sslice_unbox(_box_x1535, KK_BORROWED, _ctx);
    kk_std_core__list _pat3 = _con1946->tail;
    if kk_likely(kk_datatype_ptr_is_unique(matched0, _ctx)) {
      kk_std_core__list_drop(_pat3, _ctx);
      kk_std_core__sslice_dup(s, _ctx);
      kk_box_drop(_box_x1535, _ctx);
      kk_datatype_ptr_free(matched0, _ctx);
    }
    else {
      kk_std_core__sslice_dup(s, _ctx);
      kk_datatype_ptr_decref(matched0, _ctx);
    }
    return kk_std_core_string_3(s, _ctx);
  }
  {
    return kk_string_empty();
  }
}

kk_std_core__list kk_std_text_regex_regex_exec(kk_box_t regex0, kk_string_t str, kk_ssize_t start, kk_context_t* _ctx) { /* (regex : any, str : string, start : ssize_t) -> list<sslice> */ 
  return kk_regex_exec(regex0,str,start,kk_context());
}

kk_std_core__list kk_std_text_regex_regex_exec_all(kk_box_t regex0, kk_string_t str, kk_ssize_t start, kk_ssize_t atmost, kk_context_t* _ctx) { /* (regex : any, str : string, start : ssize_t, atmost : ssize_t) -> list<list<sslice>> */ 
  return kk_regex_exec_all(regex0,str,start,atmost,kk_context());
}

kk_box_t kk_std_text_regex_regex_create(kk_string_t source0, bool ignore_case, bool multi_line, kk_context_t* _ctx) { /* (source : string, ignore-case : bool, multi-line : bool) -> any */ 
  return kk_regex_create(source0,ignore_case,multi_line,kk_context());
}
 
// lifted local: .lift1471-captures, captures, .spec.1166
// specialized: std/core/.unroll17333-map.5, on parameters f1161, using:
// f1161 = std/text/regex/captured

kk_std_core__list kk_std_text_regex__trmc_lift1472_captures(kk_std_core__list xs1160, kk_std_core_types__cctx _acc, kk_context_t* _ctx) { /* (xs1160 : list<list<sslice>>, ctx<list<string>>) -> list<string> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs1160, _ctx)) {
    struct kk_std_core_Cons* _con1948 = kk_std_core__as_Cons(xs1160, _ctx);
    kk_box_t _box_x1536 = _con1948->head;
    kk_std_core__list xx1164 = _con1948->tail;
    kk_std_core__list x1163 = kk_std_core__list_unbox(_box_x1536, KK_BORROWED, _ctx);
    kk_reuse_t _ru_1877 = kk_reuse_null; /*reuse*/;
    if kk_likely(kk_datatype_ptr_is_unique(xs1160, _ctx)) {
      _ru_1877 = (kk_datatype_ptr_reuse(xs1160, _ctx));
    }
    else {
      kk_std_core__list_dup(x1163, _ctx);
      kk_std_core__list_dup(xx1164, _ctx);
      kk_datatype_ptr_decref(xs1160, _ctx);
    }
    kk_string_t _trmc_1496;
    if (kk_std_core__is_Cons(x1163, _ctx)) {
      struct kk_std_core_Cons* _con1949 = kk_std_core__as_Cons(x1163, _ctx);
      kk_box_t _box_x1537 = _con1949->head;
      kk_std_core__sslice s = kk_std_core__sslice_unbox(_box_x1537, KK_BORROWED, _ctx);
      kk_std_core__list _pat3 = _con1949->tail;
      if kk_likely(kk_datatype_ptr_is_unique(x1163, _ctx)) {
        kk_std_core__list_drop(_pat3, _ctx);
        kk_std_core__sslice_dup(s, _ctx);
        kk_box_drop(_box_x1537, _ctx);
        kk_datatype_ptr_free(x1163, _ctx);
      }
      else {
        kk_std_core__sslice_dup(s, _ctx);
        kk_datatype_ptr_decref(x1163, _ctx);
      }
      _trmc_1496 = kk_std_core_string_3(s, _ctx); /*string*/
    }
    else {
      _trmc_1496 = kk_string_empty(); /*string*/
    }
    kk_std_core__list _trmc_1497 = kk_datatype_null(); /*list<string>*/;
    kk_std_core__list _trmc_1498 = kk_std_core__new_Cons(_ru_1877, 0, kk_string_box(_trmc_1496), _trmc_1497, _ctx); /*list<string>*/;
    kk_box_t* _b_1552_1547 = (kk_box_t*)((&kk_std_core__as_Cons(_trmc_1498, _ctx)->tail)); /*field-addr<list<string>>*/;
    { // tailcall
      kk_std_core_types__cctx _x1951 = kk_cctx_extend(_acc,(kk_std_core__list_box(_trmc_1498, _ctx)),_b_1552_1547,true,kk_context()); /*ctx<0>*/
      xs1160 = xx1164;
      _acc = _x1951;
      goto kk__tailcall;
    }
  }
  {
    kk_box_t _x1952 = kk_cctx_apply(_acc,(kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx)),true,kk_context()); /*-1*/
    return kk_std_core__list_unbox(_x1952, KK_OWNED, _ctx);
  }
}
 
// lifted local: .lift1471-captures, captures, .spec.1166
// specialized: std/core/.unroll17333-map.5, on parameters f1161, using:
// f1161 = std/text/regex/captured

kk_std_core__list kk_std_text_regex__lift1472_captures(kk_std_core__list xs11600, kk_context_t* _ctx) { /* (xs1160 : list<list<sslice>>) -> list<string> */ 
  kk_std_core_types__cctx _x1953 = kk_cctx_empty(kk_context()); /*ctx<0>*/
  return kk_std_text_regex__trmc_lift1472_captures(xs11600, _x1953, _ctx);
}
 
// lifted local: captures, .spec.1159
// specialized: std/core/map.5, on parameters f1156, using:
// f1156 = std/text/regex/captured

kk_std_core__list kk_std_text_regex__lift1471_captures(kk_std_core__list xs1155, kk_context_t* _ctx) { /* (xs1155 : list<list<sslice>>) -> list<string> */ 
  if (kk_std_core__is_Nil(xs1155, _ctx)) {
    return kk_std_core__new_Nil(_ctx);
  }
  {
    return kk_std_text_regex__lift1472_captures(xs1155, _ctx);
  }
}
 
// monadic lift

kk_string_t kk_std_text_regex__mlift1523_concat_replace(kk_std_core__list acc, kk_std_core__list mm, kk_std_core__list pre, kk_function_t repl, kk_string_t _y_1509, kk_context_t* _ctx) { /* forall<e> (acc : list<string>, mm : list<list<sslice>>, pre : list<sslice>, repl : (list<sslice>) -> e string, string) -> e string */ 
  kk_string_t _b_1565_1559;
  if (kk_std_core__is_Cons(pre, _ctx)) {
    struct kk_std_core_Cons* _con1954 = kk_std_core__as_Cons(pre, _ctx);
    kk_box_t _box_x1558 = _con1954->head;
    kk_std_core__sslice s = kk_std_core__sslice_unbox(_box_x1558, KK_BORROWED, _ctx);
    kk_std_core__list _pat3 = _con1954->tail;
    if kk_likely(kk_datatype_ptr_is_unique(pre, _ctx)) {
      kk_std_core__list_drop(_pat3, _ctx);
      kk_std_core__sslice_dup(s, _ctx);
      kk_box_drop(_box_x1558, _ctx);
      kk_datatype_ptr_free(pre, _ctx);
    }
    else {
      kk_std_core__sslice_dup(s, _ctx);
      kk_datatype_ptr_decref(pre, _ctx);
    }
    _b_1565_1559 = kk_std_core_string_3(s, _ctx); /*string*/
  }
  else {
    _b_1565_1559 = kk_string_empty(); /*string*/
  }
  kk_std_core__list _x1956;
  kk_std_core__list _x1957 = kk_std_core__new_Cons(kk_reuse_null, 0, kk_string_box(_b_1565_1559), acc, _ctx); /*list<61>*/
  _x1956 = kk_std_core__new_Cons(kk_reuse_null, 0, kk_string_box(_y_1509), _x1957, _ctx); /*list<61>*/
  return kk_std_text_regex_concat_replace(mm, repl, _x1956, _ctx);
}


// lift anonymous function
struct kk_std_text_regex_concat_replace_fun1962__t {
  struct kk_function_s _base;
  kk_std_core__list acc0;
  kk_std_core__list mm0;
  kk_std_core__list pre0;
  kk_function_t repl0;
};
static kk_box_t kk_std_text_regex_concat_replace_fun1962(kk_function_t _fself, kk_box_t _b_1570, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_concat_replace_fun1962(kk_std_core__list acc0, kk_std_core__list mm0, kk_std_core__list pre0, kk_function_t repl0, kk_context_t* _ctx) {
  struct kk_std_text_regex_concat_replace_fun1962__t* _self = kk_function_alloc_as(struct kk_std_text_regex_concat_replace_fun1962__t, 5, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_text_regex_concat_replace_fun1962, kk_context());
  _self->acc0 = acc0;
  _self->mm0 = mm0;
  _self->pre0 = pre0;
  _self->repl0 = repl0;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_box_t kk_std_text_regex_concat_replace_fun1962(kk_function_t _fself, kk_box_t _b_1570, kk_context_t* _ctx) {
  struct kk_std_text_regex_concat_replace_fun1962__t* _self = kk_function_as(struct kk_std_text_regex_concat_replace_fun1962__t*, _fself, _ctx);
  kk_std_core__list acc0 = _self->acc0; /* list<string> */
  kk_std_core__list mm0 = _self->mm0; /* list<list<sslice>> */
  kk_std_core__list pre0 = _self->pre0; /* list<sslice> */
  kk_function_t repl0 = _self->repl0; /* (list<sslice>) -> 321 string */
  kk_drop_match(_self, {kk_std_core__list_dup(acc0, _ctx);kk_std_core__list_dup(mm0, _ctx);kk_std_core__list_dup(pre0, _ctx);kk_function_dup(repl0, _ctx);}, {}, _ctx)
  kk_string_t _y_1588_15090 = kk_string_unbox(_b_1570); /*string*/;
  kk_string_t _x1963 = kk_std_text_regex__mlift1523_concat_replace(acc0, mm0, pre0, repl0, _y_1588_15090, _ctx); /*string*/
  return kk_string_box(_x1963);
}

kk_string_t kk_std_text_regex_concat_replace(kk_std_core__list matches, kk_function_t repl0, kk_std_core__list acc0, kk_context_t* _ctx) { /* forall<e> (matches : list<list<sslice>>, repl : (list<sslice>) -> e string, acc : list<string>) -> e string */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(matches, _ctx)) {
    struct kk_std_core_Cons* _con1958 = kk_std_core__as_Cons(matches, _ctx);
    kk_box_t _box_x1567 = _con1958->head;
    kk_std_core__list _pat0 = _con1958->tail;
    if (kk_std_core__is_Cons(_pat0, _ctx)) {
      struct kk_std_core_Cons* _con1959 = kk_std_core__as_Cons(_pat0, _ctx);
      kk_box_t _box_x1568 = _con1959->head;
      kk_std_core__list pre0 = kk_std_core__list_unbox(_box_x1567, KK_BORROWED, _ctx);
      kk_std_core__list mm0 = _con1959->tail;
      kk_std_core__list m = kk_std_core__list_unbox(_box_x1568, KK_BORROWED, _ctx);
      kk_reuse_t _ru_1881 = kk_reuse_null; /*reuse*/;
      kk_reuse_t _ru_1880 = kk_reuse_null; /*reuse*/;
      if kk_likely(kk_datatype_ptr_is_unique(matches, _ctx)) {
        if kk_likely(kk_datatype_ptr_is_unique(_pat0, _ctx)) {
          _ru_1880 = (kk_datatype_ptr_reuse(_pat0, _ctx));
        }
        else {
          kk_std_core__list_dup(m, _ctx);
          kk_std_core__list_dup(mm0, _ctx);
          kk_datatype_ptr_decref(_pat0, _ctx);
        }
        _ru_1881 = (kk_datatype_ptr_reuse(matches, _ctx));
      }
      else {
        kk_std_core__list_dup(m, _ctx);
        kk_std_core__list_dup(mm0, _ctx);
        kk_std_core__list_dup(pre0, _ctx);
        kk_datatype_ptr_decref(matches, _ctx);
      }
      kk_string_t x_1525;
      kk_function_t _x1960 = kk_function_dup(repl0, _ctx); /*(list<sslice>) -> 321 string*/
      x_1525 = kk_function_call(kk_string_t, (kk_function_t, kk_std_core__list, kk_context_t*), _x1960, (_x1960, m, _ctx), _ctx); /*string*/
      if (kk_yielding(kk_context())) {
        kk_reuse_drop(_ru_1880,kk_context());
        kk_reuse_drop(_ru_1881,kk_context());
        kk_string_drop(x_1525, _ctx);
        kk_box_t _x1961 = kk_std_core_hnd_yield_extend(kk_std_text_regex_new_concat_replace_fun1962(acc0, mm0, pre0, repl0, _ctx), _ctx); /*3926*/
        return kk_string_unbox(_x1961);
      }
      {
        kk_string_t _b_1586_1572;
        if (kk_std_core__is_Cons(pre0, _ctx)) {
          struct kk_std_core_Cons* _con1964 = kk_std_core__as_Cons(pre0, _ctx);
          kk_box_t _box_x1571 = _con1964->head;
          kk_std_core__sslice s0 = kk_std_core__sslice_unbox(_box_x1571, KK_BORROWED, _ctx);
          kk_std_core__list _pat30 = _con1964->tail;
          if kk_likely(kk_datatype_ptr_is_unique(pre0, _ctx)) {
            kk_std_core__list_drop(_pat30, _ctx);
            kk_std_core__sslice_dup(s0, _ctx);
            kk_box_drop(_box_x1571, _ctx);
            kk_datatype_ptr_free(pre0, _ctx);
          }
          else {
            kk_std_core__sslice_dup(s0, _ctx);
            kk_datatype_ptr_decref(pre0, _ctx);
          }
          _b_1586_1572 = kk_std_core_string_3(s0, _ctx); /*string*/
        }
        else {
          _b_1586_1572 = kk_string_empty(); /*string*/
        }
        { // tailcall
          kk_std_core__list _x1966;
          kk_std_core__list _x1967 = kk_std_core__new_Cons(_ru_1881, 0, kk_string_box(_b_1586_1572), acc0, _ctx); /*list<61>*/
          _x1966 = kk_std_core__new_Cons(_ru_1880, 0, kk_string_box(x_1525), _x1967, _ctx); /*list<61>*/
          matches = mm0;
          acc0 = _x1966;
          goto kk__tailcall;
        }
      }
    }
  }
  if (kk_std_core__is_Cons(matches, _ctx)) {
    struct kk_std_core_Cons* _con1968 = kk_std_core__as_Cons(matches, _ctx);
    kk_box_t _box_x1576 = _con1968->head;
    kk_std_core__list _pat200 = _con1968->tail;
    kk_std_core__list post = kk_std_core__list_unbox(_box_x1576, KK_BORROWED, _ctx);
    kk_function_drop(repl0, _ctx);
    kk_reuse_t _ru_1883 = kk_reuse_null; /*reuse*/;
    if kk_likely(kk_datatype_ptr_is_unique(matches, _ctx)) {
      _ru_1883 = (kk_datatype_ptr_reuse(matches, _ctx));
    }
    else {
      kk_std_core__list_dup(post, _ctx);
      kk_datatype_ptr_decref(matches, _ctx);
    }
    kk_string_t _b_1580_1578;
    if (kk_std_core__is_Cons(post, _ctx)) {
      struct kk_std_core_Cons* _con1969 = kk_std_core__as_Cons(post, _ctx);
      kk_box_t _box_x1577 = _con1969->head;
      kk_std_core__sslice s00 = kk_std_core__sslice_unbox(_box_x1577, KK_BORROWED, _ctx);
      kk_std_core__list _pat31 = _con1969->tail;
      if kk_likely(kk_datatype_ptr_is_unique(post, _ctx)) {
        kk_std_core__list_drop(_pat31, _ctx);
        kk_std_core__sslice_dup(s00, _ctx);
        kk_box_drop(_box_x1577, _ctx);
        kk_datatype_ptr_free(post, _ctx);
      }
      else {
        kk_std_core__sslice_dup(s00, _ctx);
        kk_datatype_ptr_decref(post, _ctx);
      }
      _b_1580_1578 = kk_std_core_string_3(s00, _ctx); /*string*/
    }
    else {
      _b_1580_1578 = kk_string_empty(); /*string*/
    }
    kk_std_core__list xs_1515 = kk_std_core__new_Cons(_ru_1883, 0, kk_string_box(_b_1580_1578), acc0, _ctx); /*list<string>*/;
    kk_std_core__list xs0_17798;
    if (kk_std_core__is_Nil(xs_1515, _ctx)) {
      xs0_17798 = kk_std_core__new_Nil(_ctx); /*list<string>*/
    }
    else {
      xs0_17798 = kk_std_core__unroll17327_lift17313_reverse_append(kk_std_core__new_Nil(_ctx), xs_1515, _ctx); /*list<string>*/
    }
    if (kk_std_core__is_Nil(xs0_17798, _ctx)) {
      return kk_string_empty();
    }
    {
      struct kk_std_core_Cons* _con1972 = kk_std_core__as_Cons(xs0_17798, _ctx);
      kk_box_t _box_x1582 = _con1972->head;
      kk_std_core__list xx = _con1972->tail;
      kk_string_t x0 = kk_string_unbox(_box_x1582);
      if kk_likely(kk_datatype_ptr_is_unique(xs0_17798, _ctx)) {
        kk_datatype_ptr_free(xs0_17798, _ctx);
      }
      else {
        kk_string_dup(x0, _ctx);
        kk_std_core__list_dup(xx, _ctx);
        kk_datatype_ptr_decref(xs0_17798, _ctx);
      }
      if (kk_std_core__is_Nil(xx, _ctx)) {
        return x0;
      }
      {
        kk_string_t _x1973 = kk_string_empty(); /*string*/
        return kk_std_core__unroll17331_lift17316_joinsep(_x1973, xx, x0, _ctx);
      }
    }
  }
  {
    kk_function_drop(repl0, _ctx);
    return kk_std_core_reverse_join(acc0, _ctx);
  }
}
 
// Does a regular expression pattern occur in a string `s`?
// (note: called `test` in javascript)

bool kk_std_text_regex_contains(kk_string_t s, kk_std_text_regex__regex r, kk_context_t* _ctx) { /* (s : string, r : regex) -> bool */ 
  kk_std_core__list list0_984;
  kk_box_t _x1977;
  {
    kk_box_t _x = r.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(r, _ctx);
    _x1977 = _x; /*any*/
  }
  kk_ssize_t _x1978 = (KK_IZ(0)); /*ssize_t*/
  list0_984 = kk_std_text_regex_regex_exec(_x1977, s, _x1978, _ctx); /*list<sslice>*/
  if (kk_std_core__is_Cons(list0_984, _ctx)) {
    struct kk_std_core_Cons* _con1979 = kk_std_core__as_Cons(list0_984, _ctx);
    kk_box_t _box_x1589 = _con1979->head;
    kk_std_core__list _pat1 = _con1979->tail;
    if kk_likely(kk_datatype_ptr_is_unique(list0_984, _ctx)) {
      kk_std_core__list_drop(_pat1, _ctx);
      kk_box_drop(_box_x1589, _ctx);
      kk_datatype_ptr_free(list0_984, _ctx);
    }
    else {
      kk_datatype_ptr_decref(list0_984, _ctx);
    }
    return true;
  }
  {
    return false;
  }
}
 
// Match a regular expression `regex` over a string `s`.
// Matches at most `atmost` times (and matches all by default).
// Returns always an odd number of elements where every even
// element is a match and the odd ones the string parts between the
// matches.
// See also `find-all` and `strings`.

kk_std_core__list kk_std_text_regex_exec_all(kk_std_text_regex__regex regex0, kk_string_t s, kk_std_core_types__optional atmost, kk_context_t* _ctx) { /* (regex : regex, s : string, atmost : optional<int>) -> list<list<sslice>> */ 
  kk_box_t _x1980;
  {
    kk_box_t _x = regex0.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(regex0, _ctx);
    _x1980 = _x; /*any*/
  }
  kk_ssize_t _x1981 = (KK_IZ(0)); /*ssize_t*/
  kk_ssize_t _x1982;
  kk_integer_t _x1983;
  if (kk_std_core_types__is_Optional(atmost, _ctx)) {
    kk_box_t _box_x1590 = atmost._cons.Optional.value;
    kk_integer_t _atmost_345 = kk_integer_unbox(_box_x1590, _ctx);
    kk_integer_dup(_atmost_345, _ctx);
    kk_std_core_types__optional_drop(atmost, _ctx);
    _x1983 = _atmost_345; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(atmost, _ctx);
    _x1983 = kk_integer_from_small(-1); /*int*/
  }
  _x1982 = kk_std_core_ssize__t(_x1983, _ctx); /*ssize_t*/
  return kk_std_text_regex_regex_exec_all(_x1980, s, _x1981, _x1982, _ctx);
}
 
// Filter only for the matched parts.

kk_std_core__list kk_std_text_regex__trmc_filter_matches(kk_std_core__list xs, kk_std_core_types__cctx _acc, kk_context_t* _ctx) { /* (xs : list<list<sslice>>, ctx<list<list<sslice>>>) -> list<list<sslice>> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs, _ctx)) {
    struct kk_std_core_Cons* _con1984 = kk_std_core__as_Cons(xs, _ctx);
    kk_box_t _box_x1591 = _con1984->head;
    kk_std_core__list _pat1 = _con1984->tail;
    if (kk_std_core__is_Cons(_pat1, _ctx)) {
      struct kk_std_core_Cons* _con1985 = kk_std_core__as_Cons(_pat1, _ctx);
      kk_box_t _box_x1592 = _con1985->head;
      kk_std_core__list mm = _con1985->tail;
      kk_std_core__list m = kk_std_core__list_unbox(_box_x1592, KK_BORROWED, _ctx);
      kk_reuse_t _ru_1888 = kk_reuse_null; /*reuse*/;
      if kk_likely(kk_datatype_ptr_is_unique(xs, _ctx)) {
        if kk_likely(kk_datatype_ptr_is_unique(_pat1, _ctx)) {
          kk_datatype_ptr_free(_pat1, _ctx);
        }
        else {
          kk_std_core__list_dup(m, _ctx);
          kk_std_core__list_dup(mm, _ctx);
          kk_datatype_ptr_decref(_pat1, _ctx);
        }
        kk_box_drop(_box_x1591, _ctx);
        _ru_1888 = (kk_datatype_ptr_reuse(xs, _ctx));
      }
      else {
        kk_std_core__list_dup(m, _ctx);
        kk_std_core__list_dup(mm, _ctx);
        kk_datatype_ptr_decref(xs, _ctx);
      }
      kk_std_core__list _trmc_1499 = kk_datatype_null(); /*list<list<sslice>>*/;
      kk_std_core__list _trmc_1500 = kk_std_core__new_Cons(_ru_1888, 0, kk_std_core__list_box(m, _ctx), _trmc_1499, _ctx); /*list<list<sslice>>*/;
      kk_box_t* _b_1607_1602 = (kk_box_t*)((&kk_std_core__as_Cons(_trmc_1500, _ctx)->tail)); /*field-addr<list<list<sslice>>>*/;
      { // tailcall
        kk_std_core_types__cctx _x1986 = kk_cctx_extend(_acc,(kk_std_core__list_box(_trmc_1500, _ctx)),_b_1607_1602,true,kk_context()); /*ctx<0>*/
        xs = mm;
        _acc = _x1986;
        goto kk__tailcall;
      }
    }
  }
  {
    kk_std_core__list_drop(xs, _ctx);
    kk_box_t _x1987 = kk_cctx_apply(_acc,(kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx)),true,kk_context()); /*-1*/
    return kk_std_core__list_unbox(_x1987, KK_OWNED, _ctx);
  }
}
 
// Filter only for the matched parts.

kk_std_core__list kk_std_text_regex_filter_matches(kk_std_core__list xs0, kk_context_t* _ctx) { /* (xs : list<list<sslice>>) -> list<list<sslice>> */ 
  kk_std_core_types__cctx _x1988 = kk_cctx_empty(kk_context()); /*ctx<0>*/
  return kk_std_text_regex__trmc_filter_matches(xs0, _x1988, _ctx);
}
 
// Filter only for the non-matched parts.

kk_std_core__list kk_std_text_regex__trmc_filter_non_matches(kk_std_core__list xs, kk_std_core_types__cctx _acc, kk_context_t* _ctx) { /* (xs : list<list<sslice>>, ctx<list<list<sslice>>>) -> list<list<sslice>> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs, _ctx)) {
    struct kk_std_core_Cons* _con1989 = kk_std_core__as_Cons(xs, _ctx);
    kk_box_t _box_x1613 = _con1989->head;
    kk_std_core__list _pat0 = _con1989->tail;
    if (kk_std_core__is_Cons(_pat0, _ctx)) {
      struct kk_std_core_Cons* _con1990 = kk_std_core__as_Cons(_pat0, _ctx);
      kk_box_t _box_x1614 = _con1990->head;
      kk_std_core__list s = kk_std_core__list_unbox(_box_x1613, KK_BORROWED, _ctx);
      kk_std_core__list xx = _con1990->tail;
      kk_reuse_t _ru_1890 = kk_reuse_null; /*reuse*/;
      if kk_likely(kk_datatype_ptr_is_unique(xs, _ctx)) {
        if kk_likely(kk_datatype_ptr_is_unique(_pat0, _ctx)) {
          kk_box_drop(_box_x1614, _ctx);
          kk_datatype_ptr_free(_pat0, _ctx);
        }
        else {
          kk_std_core__list_dup(xx, _ctx);
          kk_datatype_ptr_decref(_pat0, _ctx);
        }
        _ru_1890 = (kk_datatype_ptr_reuse(xs, _ctx));
      }
      else {
        kk_std_core__list_dup(s, _ctx);
        kk_std_core__list_dup(xx, _ctx);
        kk_datatype_ptr_decref(xs, _ctx);
      }
      kk_std_core__list _trmc_1501 = kk_datatype_null(); /*list<list<sslice>>*/;
      kk_std_core__list _trmc_1502 = kk_std_core__new_Cons(_ru_1890, 0, kk_std_core__list_box(s, _ctx), _trmc_1501, _ctx); /*list<list<sslice>>*/;
      kk_box_t* _b_1634_1624 = (kk_box_t*)((&kk_std_core__as_Cons(_trmc_1502, _ctx)->tail)); /*field-addr<list<list<sslice>>>*/;
      { // tailcall
        kk_std_core_types__cctx _x1991 = kk_cctx_extend(_acc,(kk_std_core__list_box(_trmc_1502, _ctx)),_b_1634_1624,true,kk_context()); /*ctx<0>*/
        xs = xx;
        _acc = _x1991;
        goto kk__tailcall;
      }
    }
  }
  if (kk_std_core__is_Cons(xs, _ctx)) {
    struct kk_std_core_Cons* _con1992 = kk_std_core__as_Cons(xs, _ctx);
    kk_box_t _box_x1625 = _con1992->head;
    kk_std_core__list _pat3 = _con1992->tail;
    kk_std_core__list s0 = kk_std_core__list_unbox(_box_x1625, KK_BORROWED, _ctx);
    kk_reuse_t _ru_1891 = kk_reuse_null; /*reuse*/;
    if kk_likely(kk_datatype_ptr_is_unique(xs, _ctx)) {
      _ru_1891 = (kk_datatype_ptr_reuse(xs, _ctx));
    }
    else {
      kk_std_core__list_dup(s0, _ctx);
      kk_datatype_ptr_decref(xs, _ctx);
    }
    kk_box_t _x1993;
    kk_box_t _x1994;
    kk_std_core__list _x1995 = kk_std_core__new_Cons(_ru_1891, 0, kk_std_core__list_box(s0, _ctx), kk_std_core__new_Nil(_ctx), _ctx); /*list<61>*/
    _x1994 = kk_std_core__list_box(_x1995, _ctx); /*-1*/
    _x1993 = kk_cctx_apply(_acc,_x1994,true,kk_context()); /*-1*/
    return kk_std_core__list_unbox(_x1993, KK_OWNED, _ctx);
  }
  {
    kk_box_t _x1996 = kk_cctx_apply(_acc,(kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx)),true,kk_context()); /*-1*/
    return kk_std_core__list_unbox(_x1996, KK_OWNED, _ctx);
  }
}
 
// Filter only for the non-matched parts.

kk_std_core__list kk_std_text_regex_filter_non_matches(kk_std_core__list xs0, kk_context_t* _ctx) { /* (xs : list<list<sslice>>) -> list<list<sslice>> */ 
  kk_std_core_types__cctx _x1997 = kk_cctx_empty(kk_context()); /*ctx<0>*/
  return kk_std_text_regex__trmc_filter_non_matches(xs0, _x1997, _ctx);
}
 
// Find a match for a regular expression.
// See also `exec`

kk_std_core_types__maybe kk_std_text_regex_find(kk_string_t s, kk_std_text_regex__regex r, kk_context_t* _ctx) { /* (s : string, r : regex) -> maybe<string> */ 
  kk_std_core__list _match_1872;
  kk_box_t _x1998;
  {
    kk_box_t _x = r.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(r, _ctx);
    _x1998 = _x; /*any*/
  }
  kk_ssize_t _x1999 = (KK_IZ(0)); /*ssize_t*/
  _match_1872 = kk_std_text_regex_regex_exec(_x1998, s, _x1999, _ctx); /*list<sslice>*/
  if (kk_std_core__is_Cons(_match_1872, _ctx)) {
    struct kk_std_core_Cons* _con2000 = kk_std_core__as_Cons(_match_1872, _ctx);
    kk_box_t _box_x1644 = _con2000->head;
    kk_std_core__sslice m = kk_std_core__sslice_unbox(_box_x1644, KK_BORROWED, _ctx);
    kk_std_core__list _pat3 = _con2000->tail;
    if kk_likely(kk_datatype_ptr_is_unique(_match_1872, _ctx)) {
      kk_std_core__list_drop(_pat3, _ctx);
      kk_std_core__sslice_dup(m, _ctx);
      kk_box_drop(_box_x1644, _ctx);
      kk_datatype_ptr_free(_match_1872, _ctx);
    }
    else {
      kk_std_core__sslice_dup(m, _ctx);
      kk_datatype_ptr_decref(_match_1872, _ctx);
    }
    kk_string_t _b_1646_1645 = kk_std_core_string_3(m, _ctx); /*string*/;
    return kk_std_core_types__new_Just(kk_string_box(_b_1646_1645), _ctx);
  }
  {
    return kk_std_core_types__new_Nothing(_ctx);
  }
}
 
// lifted local: .lift1473-find-all, find-all, .spec.1178
// specialized: std/core/.unroll17333-map.5, on parameters f1173, using:
// f1173 = std/text/regex/captured

kk_std_core__list kk_std_text_regex__trmc_lift1474_find_all(kk_std_core__list xs1172, kk_std_core_types__cctx _acc, kk_context_t* _ctx) { /* (xs1172 : list<list<sslice>>, ctx<list<string>>) -> list<string> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs1172, _ctx)) {
    struct kk_std_core_Cons* _con2001 = kk_std_core__as_Cons(xs1172, _ctx);
    kk_box_t _box_x1647 = _con2001->head;
    kk_std_core__list xx1176 = _con2001->tail;
    kk_std_core__list x1175 = kk_std_core__list_unbox(_box_x1647, KK_BORROWED, _ctx);
    kk_reuse_t _ru_1893 = kk_reuse_null; /*reuse*/;
    if kk_likely(kk_datatype_ptr_is_unique(xs1172, _ctx)) {
      _ru_1893 = (kk_datatype_ptr_reuse(xs1172, _ctx));
    }
    else {
      kk_std_core__list_dup(x1175, _ctx);
      kk_std_core__list_dup(xx1176, _ctx);
      kk_datatype_ptr_decref(xs1172, _ctx);
    }
    kk_string_t _trmc_1503;
    if (kk_std_core__is_Cons(x1175, _ctx)) {
      struct kk_std_core_Cons* _con2002 = kk_std_core__as_Cons(x1175, _ctx);
      kk_box_t _box_x1648 = _con2002->head;
      kk_std_core__sslice s = kk_std_core__sslice_unbox(_box_x1648, KK_BORROWED, _ctx);
      kk_std_core__list _pat3 = _con2002->tail;
      if kk_likely(kk_datatype_ptr_is_unique(x1175, _ctx)) {
        kk_std_core__list_drop(_pat3, _ctx);
        kk_std_core__sslice_dup(s, _ctx);
        kk_box_drop(_box_x1648, _ctx);
        kk_datatype_ptr_free(x1175, _ctx);
      }
      else {
        kk_std_core__sslice_dup(s, _ctx);
        kk_datatype_ptr_decref(x1175, _ctx);
      }
      _trmc_1503 = kk_std_core_string_3(s, _ctx); /*string*/
    }
    else {
      _trmc_1503 = kk_string_empty(); /*string*/
    }
    kk_std_core__list _trmc_1504 = kk_datatype_null(); /*list<string>*/;
    kk_std_core__list _trmc_1505 = kk_std_core__new_Cons(_ru_1893, 0, kk_string_box(_trmc_1503), _trmc_1504, _ctx); /*list<string>*/;
    kk_box_t* _b_1663_1658 = (kk_box_t*)((&kk_std_core__as_Cons(_trmc_1505, _ctx)->tail)); /*field-addr<list<string>>*/;
    { // tailcall
      kk_std_core_types__cctx _x2004 = kk_cctx_extend(_acc,(kk_std_core__list_box(_trmc_1505, _ctx)),_b_1663_1658,true,kk_context()); /*ctx<0>*/
      xs1172 = xx1176;
      _acc = _x2004;
      goto kk__tailcall;
    }
  }
  {
    kk_box_t _x2005 = kk_cctx_apply(_acc,(kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx)),true,kk_context()); /*-1*/
    return kk_std_core__list_unbox(_x2005, KK_OWNED, _ctx);
  }
}
 
// lifted local: .lift1473-find-all, find-all, .spec.1178
// specialized: std/core/.unroll17333-map.5, on parameters f1173, using:
// f1173 = std/text/regex/captured

kk_std_core__list kk_std_text_regex__lift1474_find_all(kk_std_core__list xs11720, kk_context_t* _ctx) { /* (xs1172 : list<list<sslice>>) -> list<string> */ 
  kk_std_core_types__cctx _x2006 = kk_cctx_empty(kk_context()); /*ctx<0>*/
  return kk_std_text_regex__trmc_lift1474_find_all(xs11720, _x2006, _ctx);
}
 
// lifted local: find-all, .spec.1171
// specialized: std/core/map.5, on parameters f1168, using:
// f1168 = std/text/regex/captured

kk_std_core__list kk_std_text_regex__lift1473_find_all(kk_std_core__list xs1167, kk_context_t* _ctx) { /* (xs1167 : list<list<sslice>>) -> list<string> */ 
  if (kk_std_core__is_Nil(xs1167, _ctx)) {
    return kk_std_core__new_Nil(_ctx);
  }
  {
    return kk_std_text_regex__lift1474_find_all(xs1167, _ctx);
  }
}
 
// Find all matches for a regular expression in a string.

kk_std_core__list kk_std_text_regex_find_all(kk_string_t s, kk_std_text_regex__regex r, kk_std_core_types__optional atmost, kk_context_t* _ctx) { /* (s : string, r : regex, atmost : optional<int>) -> list<string> */ 
  kk_std_core__list xs_992;
  kk_std_core__list _x2007;
  kk_box_t _x2008;
  {
    kk_box_t _x = r.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(r, _ctx);
    _x2008 = _x; /*any*/
  }
  kk_ssize_t _x2009 = (KK_IZ(0)); /*ssize_t*/
  kk_ssize_t _x2010;
  kk_integer_t _x2011;
  if (kk_std_core_types__is_Optional(atmost, _ctx)) {
    kk_box_t _box_x1669 = atmost._cons.Optional.value;
    kk_integer_t _atmost_437 = kk_integer_unbox(_box_x1669, _ctx);
    kk_integer_dup(_atmost_437, _ctx);
    kk_std_core_types__optional_drop(atmost, _ctx);
    _x2011 = _atmost_437; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(atmost, _ctx);
    _x2011 = kk_integer_from_small(-1); /*int*/
  }
  _x2010 = kk_std_core_ssize__t(_x2011, _ctx); /*ssize_t*/
  _x2007 = kk_std_text_regex_regex_exec_all(_x2008, s, _x2009, _x2010, _ctx); /*list<list<sslice>>*/
  xs_992 = kk_std_text_regex_filter_matches(_x2007, _ctx); /*list<list<sslice>>*/
  return kk_std_text_regex__lift1473_find_all(xs_992, _ctx);
}
 
// Create a new regular expression. Takes two optional parameters. Set `ignoreCase` to `True`
// to ignore uppercase/lowercase distinction. If  `multiline` is set to `True`, then `^` and `$`
// match also the beginning and end of every line (instead of the entire input).

kk_std_text_regex__regex kk_std_text_regex_regex(kk_string_t regex0, kk_std_core_types__optional ignorecase, kk_std_core_types__optional multiline, kk_context_t* _ctx) { /* (regex : string, ignorecase : optional<bool>, multiline : optional<bool>) -> regex */ 
  kk_box_t _x2012;
  kk_string_t _x2013 = kk_string_dup(regex0, _ctx); /*string*/
  bool _x2014;
  if (kk_std_core_types__is_Optional(ignorecase, _ctx)) {
    kk_box_t _box_x1670 = ignorecase._cons.Optional.value;
    bool _ignorecase_455 = kk_bool_unbox(_box_x1670);
    kk_std_core_types__optional_drop(ignorecase, _ctx);
    _x2014 = _ignorecase_455; /*bool*/
  }
  else {
    kk_std_core_types__optional_drop(ignorecase, _ctx);
    _x2014 = false; /*bool*/
  }
  bool _x2015;
  if (kk_std_core_types__is_Optional(multiline, _ctx)) {
    kk_box_t _box_x1671 = multiline._cons.Optional.value;
    bool _multiline_459 = kk_bool_unbox(_box_x1671);
    kk_std_core_types__optional_drop(multiline, _ctx);
    _x2015 = _multiline_459; /*bool*/
  }
  else {
    kk_std_core_types__optional_drop(multiline, _ctx);
    _x2015 = false; /*bool*/
  }
  _x2012 = kk_std_text_regex_regex_create(_x2013, _x2014, _x2015, _ctx); /*any*/
  return kk_std_text_regex__new_Regex(_x2012, regex0, _ctx);
}
 
// Replace the all occurrences of `regex` by the result of the replacement fun `repl` in a string `s`.

kk_string_t kk_std_text_regex_replace_all(kk_string_t s, kk_std_text_regex__regex r, kk_function_t repl, kk_std_core_types__optional atmost, kk_context_t* _ctx) { /* forall<e> (s : string, r : regex, repl : (list<sslice>) -> e string, atmost : optional<int>) -> e string */ 
  kk_std_core__list _x2016;
  kk_box_t _x2017;
  {
    kk_box_t _x = r.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(r, _ctx);
    _x2017 = _x; /*any*/
  }
  kk_ssize_t _x2018 = (KK_IZ(0)); /*ssize_t*/
  kk_ssize_t _x2019;
  kk_integer_t _x2020;
  if (kk_std_core_types__is_Optional(atmost, _ctx)) {
    kk_box_t _box_x1672 = atmost._cons.Optional.value;
    kk_integer_t _atmost_475 = kk_integer_unbox(_box_x1672, _ctx);
    kk_integer_dup(_atmost_475, _ctx);
    kk_std_core_types__optional_drop(atmost, _ctx);
    _x2020 = _atmost_475; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(atmost, _ctx);
    _x2020 = kk_integer_from_small(-1); /*int*/
  }
  _x2019 = kk_std_core_ssize__t(_x2020, _ctx); /*ssize_t*/
  _x2016 = kk_std_text_regex_regex_exec_all(_x2017, s, _x2018, _x2019, _ctx); /*list<list<sslice>>*/
  return kk_std_text_regex_concat_replace(_x2016, repl, kk_std_core__new_Nil(_ctx), _ctx);
}
 
// lifted local: replace-captures, .spec.1217
// specialized: std/text/regex/concat-replace, on parameters repl1180, using:
// repl1180 = fn(cap: (list :: V -> V)<sslice>){
//   match (cap) {
//     ((std/core/Cons(((.skip std/core/Sslice((.pat11: string) : string, (.pat21: int) : int, (.pat30: int) : int) : sslice ) as .pat03: sslice) : sslice, ((std/core/Cons(((.skip std/core/Sslice((.pat5: string) : string, (.pat6: int) : int, (.pat7: int) : int) : sslice ) as digit: sslice) : sslice, ((std/core/Cons(((.skip std/core/Sslice((.pat9: string) : string, (.pat100: int) : int, (.pat110: int) : int) : sslice ) as amp: sslice) : sslice, ((std/core/Cons(((.skip std/core/Sslice((.pat13: string) : string, (.pat14: int) : int, (.pat15: int) : int) : sslice ) as dollar: sslice) : sslice, ((std/core/Nil() : (list :: V -> V)<sslice> ) as .pat16: (list :: V -> V)<sslice>) : (list :: V -> V)<sslice>) : (list :: V -> V)<sslice> ) as .pat12: (list :: V -> V)<sslice>) : (list :: V -> V)<sslice>) : (list :: V -> V)<sslice> ) as .pat8: (list :: V -> V)<sslice>) : (list :: V -> V)<sslice>) : (list :: V -> V)<sslice> ) as .pat40: (list :: V -> V)<sslice>) : (list :: V -> V)<sslice>) : (list :: V -> V)<sslice> ) as .pat4: ((list :: V -> V)<sslice>))
//        -> (match ((std/core/(>=.1)((match (dollar) {
//           ((.skip std/core/Sslice((.pat04: string) : string, (.x0: int) : int, (.pat18: int) : int) : sslice ) as .pat17: sslice)
//              -> .x0;
//         }), 0))) {
//         ((std/core/types/True() : bool ) as .pat170: bool)
//            -> "$";
//         ((.skip std/core/types/False() : bool ) as .pat180: bool)
//            -> val grp : int
//                     = (match ((std/core/(>=.1)((match (amp) {
//                         ((.skip std/core/Sslice((.pat05: string) : string, (.x1: int) : int, (.pat111: int) : int) : sslice ) as .pat19: sslice)
//                            -> .x1;
//                       }), 0))) {
//                       ((std/core/types/True() : bool ) as .pat190: bool)
//                          -> 0;
//                       ((.skip std/core/types/False() : bool ) as .pat200: bool)
//                          -> val s1.1007 : string;
//                             val default0.1008 : (optional :: V -> V)<int>;
//                         (match ((std/core/(==.3)(s1.1007, ""))) {
//                           ((std/core/types/True() : bool ) as .pat31: bool)
//                              -> (match (default0.1008) {
//                               ((std/core/types/Optional((.default.15874: int) : int) : (optional :: V -> V)<int> ) as .pat22: ((optional :: V -> V)<int>))
//                                  -> .default.15874;
//                               ((.skip std/core/types/None() : (optional :: V -> V)<int> ) as .pat06: ((optional :: V -> V)<int>))
//                                  -> 0;
//                             });
//                           ((.skip std/core/types/False() : bool ) as .pat41: bool)
//                              -> val m.17769 : (maybe :: V -> V)<int>
//                                       = std/core/xparse-int((std/core/trim-right((std/core/trim-left(s1.1007)))), (match ((std/core/types/None<bool>)) {
//                                           ((std/core/types/Optional((.hex.15878: bool) : bool) : (optional :: V -> V)<bool> ) as .pat101: ((optional :: V -> V)<bool>))
//                                              -> .hex.15878;
//                                           ((.skip std/core/types/None() : (optional :: V -> V)<bool> ) as .pat201: ((optional :: V -> V)<bool>))
//                                              -> std/core/types/False;
//                                         }));
//                             (match (m.17769) {
//                               ((std/core/types/Nothing() : (maybe :: V -> V)<int> ) as .pat112: ((maybe :: V -> V)<int>))
//                                  -> (match (default0.1008) {
//                                   ((std/core/types/Optional((.default.158740: int) : int) : (optional :: V -> V)<int> ) as .pat23: ((optional :: V -> V)<int>))
//                                      -> .default.158740;
//                                   ((.skip std/core/types/None() : (optional :: V -> V)<int> ) as .pat000: ((optional :: V -> V)<int>))
//                                      -> 0;
//                                 });
//                               ((.skip std/core/types/Just((x: int) : int) : (maybe :: V -> V)<int> ) as .pat0000: ((maybe :: V -> V)<int>))
//                                  -> x;
//                             });
//                         });
//                     });
//           (match ((match (caps) {
//             ((std/core/Nil() : (list :: V -> V)<sslice> ) as .pat42: ((list :: V -> V)<sslice>))
//                -> std/core/types/Nothing<sslice>;
//             (.pat24: ((list :: V -> V)<sslice>))
//                -> std/core/.unroll17334-op<sslice>(caps, grp);
//           })) {
//             ((std/core/types/Nothing() : (maybe :: V -> V)<sslice> ) as .pat210: ((maybe :: V -> V)<sslice>))
//                -> "";
//             ((.skip std/core/types/Just(((.skip std/core/Sslice((.pat230: string) : string, (.pat240: int) : int, (.pat25: int) : int) : sslice ) as s2: sslice) : sslice) : (maybe :: V -> V)<sslice> ) as .pat220: ((maybe :: V -> V)<sslice>))
//                -> std/core/string.3(s2);
//           });
//       });
//     (.pat26: ((list :: V -> V)<sslice>))
//        -> "$";
//   };
// }

kk_string_t kk_std_text_regex__lift1475_replace_captures(kk_std_core__list caps, kk_std_core__list matches1179, kk_std_core__list acc1181, kk_context_t* _ctx) { /* (caps : list<sslice>, matches1179 : list<list<sslice>>, acc1181 : list<string>) -> string */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(matches1179, _ctx)) {
    struct kk_std_core_Cons* _con2021 = kk_std_core__as_Cons(matches1179, _ctx);
    kk_box_t _box_x1673 = _con2021->head;
    kk_std_core__list _pat01184 = _con2021->tail;
    if (kk_std_core__is_Cons(_pat01184, _ctx)) {
      struct kk_std_core_Cons* _con2022 = kk_std_core__as_Cons(_pat01184, _ctx);
      kk_box_t _box_x1674 = _con2022->head;
      kk_std_core__list pre1183 = kk_std_core__list_unbox(_box_x1673, KK_BORROWED, _ctx);
      kk_std_core__list mm1186 = _con2022->tail;
      kk_std_core__list m1185 = kk_std_core__list_unbox(_box_x1674, KK_BORROWED, _ctx);
      kk_reuse_t _ru_1896 = kk_reuse_null; /*reuse*/;
      kk_reuse_t _ru_1895 = kk_reuse_null; /*reuse*/;
      if kk_likely(kk_datatype_ptr_is_unique(matches1179, _ctx)) {
        if kk_likely(kk_datatype_ptr_is_unique(_pat01184, _ctx)) {
          _ru_1895 = (kk_datatype_ptr_reuse(_pat01184, _ctx));
        }
        else {
          kk_std_core__list_dup(m1185, _ctx);
          kk_std_core__list_dup(mm1186, _ctx);
          kk_datatype_ptr_decref(_pat01184, _ctx);
        }
        _ru_1896 = (kk_datatype_ptr_reuse(matches1179, _ctx));
      }
      else {
        kk_std_core__list_dup(m1185, _ctx);
        kk_std_core__list_dup(mm1186, _ctx);
        kk_std_core__list_dup(pre1183, _ctx);
        kk_datatype_ptr_decref(matches1179, _ctx);
      }
      kk_string_t _b_1691_1689;
      if (kk_std_core__is_Cons(m1185, _ctx)) {
        struct kk_std_core_Cons* _con2024 = kk_std_core__as_Cons(m1185, _ctx);
        kk_box_t _box_x1675 = _con2024->head;
        kk_std_core__list _pat40 = _con2024->tail;
        if (kk_std_core__is_Cons(_pat40, _ctx)) {
          struct kk_std_core_Cons* _con2025 = kk_std_core__as_Cons(_pat40, _ctx);
          kk_std_core__sslice _pat03 = kk_std_core__sslice_unbox(_box_x1675, KK_BORROWED, _ctx);
          kk_box_t _box_x1676 = _con2025->head;
          kk_std_core__list _pat8 = _con2025->tail;
          if (kk_std_core__is_Cons(_pat8, _ctx)) {
            struct kk_std_core_Cons* _con2026 = kk_std_core__as_Cons(_pat8, _ctx);
            kk_std_core__sslice digit = kk_std_core__sslice_unbox(_box_x1676, KK_BORROWED, _ctx);
            kk_box_t _box_x1677 = _con2026->head;
            kk_std_core__list _pat12 = _con2026->tail;
            if (kk_std_core__is_Cons(_pat12, _ctx)) {
              struct kk_std_core_Cons* _con2027 = kk_std_core__as_Cons(_pat12, _ctx);
              kk_std_core__sslice amp = kk_std_core__sslice_unbox(_box_x1677, KK_BORROWED, _ctx);
              kk_box_t _box_x1678 = _con2027->head;
              kk_std_core__list _pat16 = _con2027->tail;
              if (kk_std_core__is_Nil(_pat16, _ctx)) {
                kk_std_core__sslice dollar = kk_std_core__sslice_unbox(_box_x1678, KK_BORROWED, _ctx);
                if kk_likely(kk_datatype_ptr_is_unique(m1185, _ctx)) {
                  if kk_likely(kk_datatype_ptr_is_unique(_pat40, _ctx)) {
                    if kk_likely(kk_datatype_ptr_is_unique(_pat8, _ctx)) {
                      if kk_likely(kk_datatype_ptr_is_unique(_pat12, _ctx)) {
                        kk_std_core__sslice_dup(dollar, _ctx);
                        kk_box_drop(_box_x1678, _ctx);
                        kk_datatype_ptr_free(_pat12, _ctx);
                      }
                      else {
                        kk_std_core__sslice_dup(dollar, _ctx);
                        kk_datatype_ptr_decref(_pat12, _ctx);
                      }
                      kk_std_core__sslice_dup(amp, _ctx);
                      kk_box_drop(_box_x1677, _ctx);
                      kk_datatype_ptr_free(_pat8, _ctx);
                    }
                    else {
                      kk_std_core__sslice_dup(amp, _ctx);
                      kk_std_core__sslice_dup(dollar, _ctx);
                      kk_datatype_ptr_decref(_pat8, _ctx);
                    }
                    kk_std_core__sslice_dup(digit, _ctx);
                    kk_box_drop(_box_x1676, _ctx);
                    kk_datatype_ptr_free(_pat40, _ctx);
                  }
                  else {
                    kk_std_core__sslice_dup(amp, _ctx);
                    kk_std_core__sslice_dup(digit, _ctx);
                    kk_std_core__sslice_dup(dollar, _ctx);
                    kk_datatype_ptr_decref(_pat40, _ctx);
                  }
                  kk_box_drop(_box_x1675, _ctx);
                  kk_datatype_ptr_free(m1185, _ctx);
                }
                else {
                  kk_std_core__sslice_dup(amp, _ctx);
                  kk_std_core__sslice_dup(digit, _ctx);
                  kk_std_core__sslice_dup(dollar, _ctx);
                  kk_datatype_ptr_decref(m1185, _ctx);
                }
                bool _match_1863;
                kk_integer_t _brw_1870;
                {
                  kk_integer_t _x0 = dollar.start;
                  kk_integer_dup(_x0, _ctx);
                  kk_std_core__sslice_drop(dollar, _ctx);
                  _brw_1870 = _x0; /*int*/
                }
                bool _brw_1871 = kk_integer_gte_borrow(_brw_1870,(kk_integer_from_small(0)),kk_context()); /*bool*/;
                kk_integer_drop(_brw_1870, _ctx);
                _match_1863 = _brw_1871; /*bool*/
                if (_match_1863) {
                  kk_std_core__sslice_drop(digit, _ctx);
                  kk_std_core__sslice_drop(amp, _ctx);
                  kk_define_string_literal(, _s2028, 1, "$", _ctx)
                  _b_1691_1689 = kk_string_dup(_s2028, _ctx); /*string*/
                  goto _match2023;
                }
                {
                  kk_integer_t grp;
                  bool _match_1865;
                  kk_integer_t _brw_1868;
                  {
                    kk_integer_t _x1 = amp.start;
                    kk_integer_dup(_x1, _ctx);
                    kk_std_core__sslice_drop(amp, _ctx);
                    _brw_1868 = _x1; /*int*/
                  }
                  bool _brw_1869 = kk_integer_gte_borrow(_brw_1868,(kk_integer_from_small(0)),kk_context()); /*bool*/;
                  kk_integer_drop(_brw_1868, _ctx);
                  _match_1865 = _brw_1869; /*bool*/
                  if (_match_1865) {
                    kk_std_core__sslice_drop(digit, _ctx);
                    grp = kk_integer_from_small(0); /*int*/
                  }
                  else {
                    kk_string_t s1_1007 = kk_std_core_string_3(digit, _ctx); /*string*/;
                    kk_std_core_types__optional default0_1008 = kk_std_core_types__new_Optional(kk_integer_box(kk_integer_from_small(0), _ctx), _ctx); /*optional<int>*/;
                    bool _match_1866;
                    kk_string_t _x2029 = kk_string_dup(s1_1007, _ctx); /*string*/
                    kk_string_t _x2030 = kk_string_empty(); /*string*/
                    _match_1866 = kk_string_is_eq(_x2029,_x2030,kk_context()); /*bool*/
                    if (_match_1866) {
                      kk_string_drop(s1_1007, _ctx);
                      if (kk_std_core_types__is_Optional(default0_1008, _ctx)) {
                        kk_box_t _box_x1681 = default0_1008._cons.Optional.value;
                        kk_integer_t _default_15874 = kk_integer_unbox(_box_x1681, _ctx);
                        kk_integer_dup(_default_15874, _ctx);
                        kk_std_core_types__optional_drop(default0_1008, _ctx);
                        grp = _default_15874; /*int*/
                      }
                      else {
                        kk_std_core_types__optional_drop(default0_1008, _ctx);
                        grp = kk_integer_from_small(0); /*int*/
                      }
                    }
                    else {
                      kk_std_core_types__maybe m_17769;
                      kk_string_t _x2032;
                      kk_string_t _x2033 = kk_string_trim_left(s1_1007,kk_context()); /*string*/
                      _x2032 = kk_string_trim_right(_x2033,kk_context()); /*string*/
                      bool _x2034;
                      kk_std_core_types__optional _match_1867 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
                      if (kk_std_core_types__is_Optional(_match_1867, _ctx)) {
                        kk_box_t _box_x1682 = _match_1867._cons.Optional.value;
                        bool _hex_15878 = kk_bool_unbox(_box_x1682);
                        kk_std_core_types__optional_drop(_match_1867, _ctx);
                        _x2034 = _hex_15878; /*bool*/
                      }
                      else {
                        kk_std_core_types__optional_drop(_match_1867, _ctx);
                        _x2034 = false; /*bool*/
                      }
                      m_17769 = kk_std_core_xparse_int(_x2032, _x2034, _ctx); /*maybe<int>*/
                      if (kk_std_core_types__is_Nothing(m_17769, _ctx)) {
                        if (kk_std_core_types__is_Optional(default0_1008, _ctx)) {
                          kk_box_t _box_x1683 = default0_1008._cons.Optional.value;
                          kk_integer_t _default_158740 = kk_integer_unbox(_box_x1683, _ctx);
                          kk_integer_dup(_default_158740, _ctx);
                          kk_std_core_types__optional_drop(default0_1008, _ctx);
                          grp = _default_158740; /*int*/
                        }
                        else {
                          kk_std_core_types__optional_drop(default0_1008, _ctx);
                          grp = kk_integer_from_small(0); /*int*/
                        }
                      }
                      else {
                        kk_box_t _box_x1684 = m_17769._cons.Just.value;
                        kk_integer_t x = kk_integer_unbox(_box_x1684, _ctx);
                        kk_integer_dup(x, _ctx);
                        kk_std_core_types__maybe_drop(m_17769, _ctx);
                        kk_std_core_types__optional_drop(default0_1008, _ctx);
                        grp = x; /*int*/
                      }
                    }
                  }
                  if (kk_std_core__is_Nil(caps, _ctx)) {
                    kk_integer_drop(grp, _ctx);
                    _b_1691_1689 = kk_string_empty(); /*string*/
                    goto _match2023;
                  }
                  {
                    kk_std_core_types__maybe _match_1864;
                    kk_std_core__list _x2036 = kk_std_core__list_dup(caps, _ctx); /*list<sslice>*/
                    _match_1864 = kk_std_core__unroll17334_op(_x2036, grp, _ctx); /*maybe<10073>*/
                    if (kk_std_core_types__is_Nothing(_match_1864, _ctx)) {
                      _b_1691_1689 = kk_string_empty(); /*string*/
                      goto _match2023;
                    }
                    {
                      kk_box_t _box_x1685 = _match_1864._cons.Just.value;
                      kk_std_core__sslice s2 = kk_std_core__sslice_unbox(_box_x1685, KK_BORROWED, _ctx);
                      kk_std_core__sslice_dup(s2, _ctx);
                      kk_std_core_types__maybe_drop(_match_1864, _ctx);
                      _b_1691_1689 = kk_std_core_string_3(s2, _ctx); /*string*/
                      goto _match2023;
                    }
                  }
                }
              }
            }
          }
        }
      }
      {
        kk_std_core__list_drop(m1185, _ctx);
        kk_define_string_literal(, _s2038, 1, "$", _ctx)
        _b_1691_1689 = kk_string_dup(_s2038, _ctx); /*string*/
      }
      _match2023: ;
      kk_string_t _b_1693_1687;
      if (kk_std_core__is_Cons(pre1183, _ctx)) {
        struct kk_std_core_Cons* _con2039 = kk_std_core__as_Cons(pre1183, _ctx);
        kk_box_t _box_x1686 = _con2039->head;
        kk_std_core__sslice s1189 = kk_std_core__sslice_unbox(_box_x1686, KK_BORROWED, _ctx);
        kk_std_core__list _pat31193 = _con2039->tail;
        if kk_likely(kk_datatype_ptr_is_unique(pre1183, _ctx)) {
          kk_std_core__list_drop(_pat31193, _ctx);
          kk_std_core__sslice_dup(s1189, _ctx);
          kk_box_drop(_box_x1686, _ctx);
          kk_datatype_ptr_free(pre1183, _ctx);
        }
        else {
          kk_std_core__sslice_dup(s1189, _ctx);
          kk_datatype_ptr_decref(pre1183, _ctx);
        }
        _b_1693_1687 = kk_std_core_string_3(s1189, _ctx); /*string*/
      }
      else {
        _b_1693_1687 = kk_string_empty(); /*string*/
      }
      { // tailcall
        kk_std_core__list _x2041;
        kk_std_core__list _x2042 = kk_std_core__new_Cons(_ru_1896, 0, kk_string_box(_b_1693_1687), acc1181, _ctx); /*list<61>*/
        _x2041 = kk_std_core__new_Cons(_ru_1895, 0, kk_string_box(_b_1691_1689), _x2042, _ctx); /*list<61>*/
        matches1179 = mm1186;
        acc1181 = _x2041;
        goto kk__tailcall;
      }
    }
  }
  if (kk_std_core__is_Cons(matches1179, _ctx)) {
    struct kk_std_core_Cons* _con2043 = kk_std_core__as_Cons(matches1179, _ctx);
    kk_box_t _box_x1695 = _con2043->head;
    kk_std_core__list _pat201197 = _con2043->tail;
    kk_std_core__list post1196 = kk_std_core__list_unbox(_box_x1695, KK_BORROWED, _ctx);
    kk_reuse_t _ru_1902 = kk_reuse_null; /*reuse*/;
    if kk_likely(kk_datatype_ptr_is_unique(matches1179, _ctx)) {
      _ru_1902 = (kk_datatype_ptr_reuse(matches1179, _ctx));
    }
    else {
      kk_std_core__list_dup(post1196, _ctx);
      kk_datatype_ptr_decref(matches1179, _ctx);
    }
    kk_std_core__list_drop(caps, _ctx);
    kk_string_t _b_1699_1697;
    if (kk_std_core__is_Cons(post1196, _ctx)) {
      struct kk_std_core_Cons* _con2044 = kk_std_core__as_Cons(post1196, _ctx);
      kk_box_t _box_x1696 = _con2044->head;
      kk_std_core__sslice s01210 = kk_std_core__sslice_unbox(_box_x1696, KK_BORROWED, _ctx);
      kk_std_core__list _pat311214 = _con2044->tail;
      if kk_likely(kk_datatype_ptr_is_unique(post1196, _ctx)) {
        kk_std_core__list_drop(_pat311214, _ctx);
        kk_std_core__sslice_dup(s01210, _ctx);
        kk_box_drop(_box_x1696, _ctx);
        kk_datatype_ptr_free(post1196, _ctx);
      }
      else {
        kk_std_core__sslice_dup(s01210, _ctx);
        kk_datatype_ptr_decref(post1196, _ctx);
      }
      _b_1699_1697 = kk_std_core_string_3(s01210, _ctx); /*string*/
    }
    else {
      _b_1699_1697 = kk_string_empty(); /*string*/
    }
    kk_std_core__list xs1198 = kk_std_core__new_Cons(_ru_1902, 0, kk_string_box(_b_1699_1697), acc1181, _ctx); /*list<string>*/;
    kk_std_core__list xs0_177981201;
    if (kk_std_core__is_Nil(xs1198, _ctx)) {
      xs0_177981201 = kk_std_core__new_Nil(_ctx); /*list<string>*/
    }
    else {
      xs0_177981201 = kk_std_core__unroll17327_lift17313_reverse_append(kk_std_core__new_Nil(_ctx), xs1198, _ctx); /*list<string>*/
    }
    if (kk_std_core__is_Nil(xs0_177981201, _ctx)) {
      return kk_string_empty();
    }
    {
      struct kk_std_core_Cons* _con2047 = kk_std_core__as_Cons(xs0_177981201, _ctx);
      kk_box_t _box_x1701 = _con2047->head;
      kk_std_core__list xx1205 = _con2047->tail;
      kk_string_t x1204 = kk_string_unbox(_box_x1701);
      if kk_likely(kk_datatype_ptr_is_unique(xs0_177981201, _ctx)) {
        kk_datatype_ptr_free(xs0_177981201, _ctx);
      }
      else {
        kk_string_dup(x1204, _ctx);
        kk_std_core__list_dup(xx1205, _ctx);
        kk_datatype_ptr_decref(xs0_177981201, _ctx);
      }
      if (kk_std_core__is_Nil(xx1205, _ctx)) {
        return x1204;
      }
      {
        kk_string_t _x2048 = kk_string_empty(); /*string*/
        return kk_std_core__unroll17331_lift17316_joinsep(_x2048, xx1205, x1204, _ctx);
      }
    }
  }
  {
    kk_std_core__list_drop(caps, _ctx);
    return kk_std_core_reverse_join(acc1181, _ctx);
  }
}
 
// Replace using a replacement string that can contain `$$` for a `$` sign, `$n` for a capture group,
// `$&` for the entire match `==$0`.

kk_string_t kk_std_text_regex_replace_captures(kk_std_core__list caps, kk_string_t repl, kk_context_t* _ctx) { /* (caps : list<sslice>, repl : string) -> string */ 
  kk_std_text_regex__regex r_999;
  kk_box_t _x2050;
  kk_string_t _x2051;
  kk_define_string_literal(, _s2052, 20, "\\$(\?:(\\d)|(\\&)|(\\$))", _ctx)
  _x2051 = kk_string_dup(_s2052, _ctx); /*string*/
  bool _x2053;
  kk_std_core_types__optional _match_1862 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_1862, _ctx)) {
    kk_box_t _box_x1702 = _match_1862._cons.Optional.value;
    bool _ignorecase_455 = kk_bool_unbox(_box_x1702);
    kk_std_core_types__optional_drop(_match_1862, _ctx);
    _x2053 = _ignorecase_455; /*bool*/
  }
  else {
    kk_std_core_types__optional_drop(_match_1862, _ctx);
    _x2053 = false; /*bool*/
  }
  bool _x2054;
  kk_std_core_types__optional _match_1861 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_1861, _ctx)) {
    kk_box_t _box_x1703 = _match_1861._cons.Optional.value;
    bool _multiline_459 = kk_bool_unbox(_box_x1703);
    kk_std_core_types__optional_drop(_match_1861, _ctx);
    _x2054 = _multiline_459; /*bool*/
  }
  else {
    kk_std_core_types__optional_drop(_match_1861, _ctx);
    _x2054 = false; /*bool*/
  }
  _x2050 = kk_std_text_regex_regex_create(_x2051, _x2053, _x2054, _ctx); /*any*/
  kk_string_t _x2055;
  kk_define_string_literal(, _s2056, 20, "\\$(\?:(\\d)|(\\&)|(\\$))", _ctx)
  _x2055 = kk_string_dup(_s2056, _ctx); /*string*/
  r_999 = kk_std_text_regex__new_Regex(_x2050, _x2055, _ctx); /*std/text/regex/regex*/
  kk_std_core__list matches1179_1486;
  kk_box_t _x2057;
  {
    kk_box_t _x = r_999.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(r_999, _ctx);
    _x2057 = _x; /*any*/
  }
  kk_ssize_t _x2058 = (KK_IZ(0)); /*ssize_t*/
  kk_ssize_t _x2059;
  kk_integer_t _x2060;
  kk_std_core_types__optional _match_1860 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_1860, _ctx)) {
    kk_box_t _box_x1704 = _match_1860._cons.Optional.value;
    kk_integer_t _atmost_475 = kk_integer_unbox(_box_x1704, _ctx);
    kk_integer_dup(_atmost_475, _ctx);
    kk_std_core_types__optional_drop(_match_1860, _ctx);
    _x2060 = _atmost_475; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(_match_1860, _ctx);
    _x2060 = kk_integer_from_small(-1); /*int*/
  }
  _x2059 = kk_std_core_ssize__t(_x2060, _ctx); /*ssize_t*/
  matches1179_1486 = kk_std_text_regex_regex_exec_all(_x2057, repl, _x2058, _x2059, _ctx); /*list<list<sslice>>*/
  return kk_std_text_regex__lift1475_replace_captures(caps, matches1179_1486, kk_std_core__new_Nil(_ctx), _ctx);
}
 
// lifted local: replace-all.1, .spec.1257
// specialized: std/text/regex/concat-replace, on parameters repl1220, using:
// repl1220 = fn(caps: (list :: V -> V)<sslice>){
//   std/text/regex/replace-captures(caps, repl);
// }

kk_string_t kk_std_text_regex__lift1476_replace_all_1(kk_string_t repl, kk_std_core__list matches1219, kk_std_core__list acc1221, kk_context_t* _ctx) { /* (repl : string, matches1219 : list<list<sslice>>, acc1221 : list<string>) -> string */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(matches1219, _ctx)) {
    struct kk_std_core_Cons* _con2061 = kk_std_core__as_Cons(matches1219, _ctx);
    kk_box_t _box_x1705 = _con2061->head;
    kk_std_core__list _pat01224 = _con2061->tail;
    if (kk_std_core__is_Cons(_pat01224, _ctx)) {
      struct kk_std_core_Cons* _con2062 = kk_std_core__as_Cons(_pat01224, _ctx);
      kk_box_t _box_x1706 = _con2062->head;
      kk_std_core__list pre1223 = kk_std_core__list_unbox(_box_x1705, KK_BORROWED, _ctx);
      kk_std_core__list mm1226 = _con2062->tail;
      kk_std_core__list m1225 = kk_std_core__list_unbox(_box_x1706, KK_BORROWED, _ctx);
      kk_reuse_t _ru_1906 = kk_reuse_null; /*reuse*/;
      kk_reuse_t _ru_1905 = kk_reuse_null; /*reuse*/;
      if kk_likely(kk_datatype_ptr_is_unique(matches1219, _ctx)) {
        if kk_likely(kk_datatype_ptr_is_unique(_pat01224, _ctx)) {
          _ru_1905 = (kk_datatype_ptr_reuse(_pat01224, _ctx));
        }
        else {
          kk_std_core__list_dup(m1225, _ctx);
          kk_std_core__list_dup(mm1226, _ctx);
          kk_datatype_ptr_decref(_pat01224, _ctx);
        }
        _ru_1906 = (kk_datatype_ptr_reuse(matches1219, _ctx));
      }
      else {
        kk_std_core__list_dup(m1225, _ctx);
        kk_std_core__list_dup(mm1226, _ctx);
        kk_std_core__list_dup(pre1223, _ctx);
        kk_datatype_ptr_decref(matches1219, _ctx);
      }
      kk_string_t _b_1712_1710;
      kk_string_t _x2063 = kk_string_dup(repl, _ctx); /*string*/
      _b_1712_1710 = kk_std_text_regex_replace_captures(m1225, _x2063, _ctx); /*string*/
      kk_string_t _b_1714_1708;
      if (kk_std_core__is_Cons(pre1223, _ctx)) {
        struct kk_std_core_Cons* _con2064 = kk_std_core__as_Cons(pre1223, _ctx);
        kk_box_t _box_x1707 = _con2064->head;
        kk_std_core__sslice s1229 = kk_std_core__sslice_unbox(_box_x1707, KK_BORROWED, _ctx);
        kk_std_core__list _pat31233 = _con2064->tail;
        if kk_likely(kk_datatype_ptr_is_unique(pre1223, _ctx)) {
          kk_std_core__list_drop(_pat31233, _ctx);
          kk_std_core__sslice_dup(s1229, _ctx);
          kk_box_drop(_box_x1707, _ctx);
          kk_datatype_ptr_free(pre1223, _ctx);
        }
        else {
          kk_std_core__sslice_dup(s1229, _ctx);
          kk_datatype_ptr_decref(pre1223, _ctx);
        }
        _b_1714_1708 = kk_std_core_string_3(s1229, _ctx); /*string*/
      }
      else {
        _b_1714_1708 = kk_string_empty(); /*string*/
      }
      { // tailcall
        kk_std_core__list _x2066;
        kk_std_core__list _x2067 = kk_std_core__new_Cons(_ru_1906, 0, kk_string_box(_b_1714_1708), acc1221, _ctx); /*list<61>*/
        _x2066 = kk_std_core__new_Cons(_ru_1905, 0, kk_string_box(_b_1712_1710), _x2067, _ctx); /*list<61>*/
        matches1219 = mm1226;
        acc1221 = _x2066;
        goto kk__tailcall;
      }
    }
  }
  if (kk_std_core__is_Cons(matches1219, _ctx)) {
    struct kk_std_core_Cons* _con2068 = kk_std_core__as_Cons(matches1219, _ctx);
    kk_box_t _box_x1716 = _con2068->head;
    kk_std_core__list _pat201237 = _con2068->tail;
    kk_std_core__list post1236 = kk_std_core__list_unbox(_box_x1716, KK_BORROWED, _ctx);
    kk_string_drop(repl, _ctx);
    kk_reuse_t _ru_1908 = kk_reuse_null; /*reuse*/;
    if kk_likely(kk_datatype_ptr_is_unique(matches1219, _ctx)) {
      _ru_1908 = (kk_datatype_ptr_reuse(matches1219, _ctx));
    }
    else {
      kk_std_core__list_dup(post1236, _ctx);
      kk_datatype_ptr_decref(matches1219, _ctx);
    }
    kk_string_t _b_1720_1718;
    if (kk_std_core__is_Cons(post1236, _ctx)) {
      struct kk_std_core_Cons* _con2069 = kk_std_core__as_Cons(post1236, _ctx);
      kk_box_t _box_x1717 = _con2069->head;
      kk_std_core__sslice s01250 = kk_std_core__sslice_unbox(_box_x1717, KK_BORROWED, _ctx);
      kk_std_core__list _pat311254 = _con2069->tail;
      if kk_likely(kk_datatype_ptr_is_unique(post1236, _ctx)) {
        kk_std_core__list_drop(_pat311254, _ctx);
        kk_std_core__sslice_dup(s01250, _ctx);
        kk_box_drop(_box_x1717, _ctx);
        kk_datatype_ptr_free(post1236, _ctx);
      }
      else {
        kk_std_core__sslice_dup(s01250, _ctx);
        kk_datatype_ptr_decref(post1236, _ctx);
      }
      _b_1720_1718 = kk_std_core_string_3(s01250, _ctx); /*string*/
    }
    else {
      _b_1720_1718 = kk_string_empty(); /*string*/
    }
    kk_std_core__list xs1238 = kk_std_core__new_Cons(_ru_1908, 0, kk_string_box(_b_1720_1718), acc1221, _ctx); /*list<string>*/;
    kk_std_core__list xs0_177981241;
    if (kk_std_core__is_Nil(xs1238, _ctx)) {
      xs0_177981241 = kk_std_core__new_Nil(_ctx); /*list<string>*/
    }
    else {
      xs0_177981241 = kk_std_core__unroll17327_lift17313_reverse_append(kk_std_core__new_Nil(_ctx), xs1238, _ctx); /*list<string>*/
    }
    if (kk_std_core__is_Nil(xs0_177981241, _ctx)) {
      return kk_string_empty();
    }
    {
      struct kk_std_core_Cons* _con2072 = kk_std_core__as_Cons(xs0_177981241, _ctx);
      kk_box_t _box_x1722 = _con2072->head;
      kk_std_core__list xx1245 = _con2072->tail;
      kk_string_t x1244 = kk_string_unbox(_box_x1722);
      if kk_likely(kk_datatype_ptr_is_unique(xs0_177981241, _ctx)) {
        kk_datatype_ptr_free(xs0_177981241, _ctx);
      }
      else {
        kk_string_dup(x1244, _ctx);
        kk_std_core__list_dup(xx1245, _ctx);
        kk_datatype_ptr_decref(xs0_177981241, _ctx);
      }
      if (kk_std_core__is_Nil(xx1245, _ctx)) {
        return x1244;
      }
      {
        kk_string_t _x2073 = kk_string_empty(); /*string*/
        return kk_std_core__unroll17331_lift17316_joinsep(_x2073, xx1245, x1244, _ctx);
      }
    }
  }
  {
    kk_string_drop(repl, _ctx);
    return kk_std_core_reverse_join(acc1221, _ctx);
  }
}
 
// lifted local: replace-all.1, .spec.1297
// specialized: std/text/regex/concat-replace, on parameters repl1260, using:
// repl1260 = fn(._wildcard_158_35: (list :: V -> V)<sslice>){
//   repl;
// }

kk_string_t kk_std_text_regex__lift1477_replace_all_1(kk_string_t repl, kk_std_core__list matches1259, kk_std_core__list acc1261, kk_context_t* _ctx) { /* (repl : string, matches1259 : list<list<sslice>>, acc1261 : list<string>) -> string */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(matches1259, _ctx)) {
    struct kk_std_core_Cons* _con2075 = kk_std_core__as_Cons(matches1259, _ctx);
    kk_box_t _box_x1723 = _con2075->head;
    kk_std_core__list _pat01264 = _con2075->tail;
    if (kk_std_core__is_Cons(_pat01264, _ctx)) {
      struct kk_std_core_Cons* _con2076 = kk_std_core__as_Cons(_pat01264, _ctx);
      kk_box_t _box_x1724 = _con2076->head;
      kk_std_core__list pre1263 = kk_std_core__list_unbox(_box_x1723, KK_BORROWED, _ctx);
      kk_std_core__list mm1266 = _con2076->tail;
      kk_reuse_t _ru_1912 = kk_reuse_null; /*reuse*/;
      kk_reuse_t _ru_1911 = kk_reuse_null; /*reuse*/;
      if kk_likely(kk_datatype_ptr_is_unique(matches1259, _ctx)) {
        if kk_likely(kk_datatype_ptr_is_unique(_pat01264, _ctx)) {
          kk_box_drop(_box_x1724, _ctx);
          _ru_1911 = (kk_datatype_ptr_reuse(_pat01264, _ctx));
        }
        else {
          kk_std_core__list_dup(mm1266, _ctx);
          kk_datatype_ptr_decref(_pat01264, _ctx);
        }
        _ru_1912 = (kk_datatype_ptr_reuse(matches1259, _ctx));
      }
      else {
        kk_std_core__list_dup(mm1266, _ctx);
        kk_std_core__list_dup(pre1263, _ctx);
        kk_datatype_ptr_decref(matches1259, _ctx);
      }
      kk_string_t _b_1732_1726;
      if (kk_std_core__is_Cons(pre1263, _ctx)) {
        struct kk_std_core_Cons* _con2077 = kk_std_core__as_Cons(pre1263, _ctx);
        kk_box_t _box_x1725 = _con2077->head;
        kk_std_core__sslice s1269 = kk_std_core__sslice_unbox(_box_x1725, KK_BORROWED, _ctx);
        kk_std_core__list _pat31273 = _con2077->tail;
        if kk_likely(kk_datatype_ptr_is_unique(pre1263, _ctx)) {
          kk_std_core__list_drop(_pat31273, _ctx);
          kk_std_core__sslice_dup(s1269, _ctx);
          kk_box_drop(_box_x1725, _ctx);
          kk_datatype_ptr_free(pre1263, _ctx);
        }
        else {
          kk_std_core__sslice_dup(s1269, _ctx);
          kk_datatype_ptr_decref(pre1263, _ctx);
        }
        _b_1732_1726 = kk_std_core_string_3(s1269, _ctx); /*string*/
      }
      else {
        _b_1732_1726 = kk_string_empty(); /*string*/
      }
      { // tailcall
        kk_string_t _x2079 = kk_string_dup(repl, _ctx); /*string*/
        kk_std_core__list _x2080;
        kk_std_core__list _x2081 = kk_std_core__new_Cons(_ru_1912, 0, kk_string_box(_b_1732_1726), acc1261, _ctx); /*list<61>*/
        _x2080 = kk_std_core__new_Cons(_ru_1911, 0, kk_string_box(repl), _x2081, _ctx); /*list<61>*/
        repl = _x2079;
        matches1259 = mm1266;
        acc1261 = _x2080;
        goto kk__tailcall;
      }
    }
  }
  if (kk_std_core__is_Cons(matches1259, _ctx)) {
    struct kk_std_core_Cons* _con2082 = kk_std_core__as_Cons(matches1259, _ctx);
    kk_box_t _box_x1734 = _con2082->head;
    kk_std_core__list _pat201277 = _con2082->tail;
    kk_std_core__list post1276 = kk_std_core__list_unbox(_box_x1734, KK_BORROWED, _ctx);
    kk_string_drop(repl, _ctx);
    kk_reuse_t _ru_1914 = kk_reuse_null; /*reuse*/;
    if kk_likely(kk_datatype_ptr_is_unique(matches1259, _ctx)) {
      _ru_1914 = (kk_datatype_ptr_reuse(matches1259, _ctx));
    }
    else {
      kk_std_core__list_dup(post1276, _ctx);
      kk_datatype_ptr_decref(matches1259, _ctx);
    }
    kk_string_t _b_1738_1736;
    if (kk_std_core__is_Cons(post1276, _ctx)) {
      struct kk_std_core_Cons* _con2083 = kk_std_core__as_Cons(post1276, _ctx);
      kk_box_t _box_x1735 = _con2083->head;
      kk_std_core__sslice s01290 = kk_std_core__sslice_unbox(_box_x1735, KK_BORROWED, _ctx);
      kk_std_core__list _pat311294 = _con2083->tail;
      if kk_likely(kk_datatype_ptr_is_unique(post1276, _ctx)) {
        kk_std_core__list_drop(_pat311294, _ctx);
        kk_std_core__sslice_dup(s01290, _ctx);
        kk_box_drop(_box_x1735, _ctx);
        kk_datatype_ptr_free(post1276, _ctx);
      }
      else {
        kk_std_core__sslice_dup(s01290, _ctx);
        kk_datatype_ptr_decref(post1276, _ctx);
      }
      _b_1738_1736 = kk_std_core_string_3(s01290, _ctx); /*string*/
    }
    else {
      _b_1738_1736 = kk_string_empty(); /*string*/
    }
    kk_std_core__list xs1278 = kk_std_core__new_Cons(_ru_1914, 0, kk_string_box(_b_1738_1736), acc1261, _ctx); /*list<string>*/;
    kk_std_core__list xs0_177981281;
    if (kk_std_core__is_Nil(xs1278, _ctx)) {
      xs0_177981281 = kk_std_core__new_Nil(_ctx); /*list<string>*/
    }
    else {
      xs0_177981281 = kk_std_core__unroll17327_lift17313_reverse_append(kk_std_core__new_Nil(_ctx), xs1278, _ctx); /*list<string>*/
    }
    if (kk_std_core__is_Nil(xs0_177981281, _ctx)) {
      return kk_string_empty();
    }
    {
      struct kk_std_core_Cons* _con2086 = kk_std_core__as_Cons(xs0_177981281, _ctx);
      kk_box_t _box_x1740 = _con2086->head;
      kk_std_core__list xx1285 = _con2086->tail;
      kk_string_t x1284 = kk_string_unbox(_box_x1740);
      if kk_likely(kk_datatype_ptr_is_unique(xs0_177981281, _ctx)) {
        kk_datatype_ptr_free(xs0_177981281, _ctx);
      }
      else {
        kk_string_dup(x1284, _ctx);
        kk_std_core__list_dup(xx1285, _ctx);
        kk_datatype_ptr_decref(xs0_177981281, _ctx);
      }
      if (kk_std_core__is_Nil(xx1285, _ctx)) {
        return x1284;
      }
      {
        kk_string_t _x2087 = kk_string_empty(); /*string*/
        return kk_std_core__unroll17331_lift17316_joinsep(_x2087, xx1285, x1284, _ctx);
      }
    }
  }
  {
    kk_string_drop(repl, _ctx);
    return kk_std_core_reverse_join(acc1261, _ctx);
  }
}
 
// Replace all occurrences of `regex` with the replacement string `repl` in a string `s`.
// The replacement string can contain `$$` for a `$` sign, `$n` for a capture group,
// `$&` for the entire match `==$0`.

kk_string_t kk_std_text_regex_replace_all_1(kk_string_t s, kk_std_text_regex__regex regex0, kk_string_t repl, kk_std_core_types__optional atmost, kk_context_t* _ctx) { /* (s : string, regex : regex, repl : string, atmost : optional<int>) -> string */ 
  bool _match_1859;
  kk_string_t _x2089 = kk_string_dup(repl, _ctx); /*string*/
  kk_string_t _x2090;
  kk_define_string_literal(, _s2091, 1, "$", _ctx)
  _x2090 = kk_string_dup(_s2091, _ctx); /*string*/
  _match_1859 = kk_string_contains(_x2089,_x2090,kk_context()); /*bool*/
  if (_match_1859) {
    kk_std_core__list matches1219;
    kk_box_t _x2092;
    {
      kk_box_t _x = regex0.obj;
      kk_box_dup(_x, _ctx);
      kk_std_text_regex__regex_drop(regex0, _ctx);
      _x2092 = _x; /*any*/
    }
    kk_ssize_t _x2093 = (KK_IZ(0)); /*ssize_t*/
    kk_ssize_t _x2094;
    kk_integer_t _x2095;
    if (kk_std_core_types__is_Optional(atmost, _ctx)) {
      kk_box_t _box_x1741 = atmost._cons.Optional.value;
      kk_integer_t _atmost_496 = kk_integer_unbox(_box_x1741, _ctx);
      kk_integer_dup(_atmost_496, _ctx);
      kk_std_core_types__optional_drop(atmost, _ctx);
      _x2095 = _atmost_496; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(atmost, _ctx);
      _x2095 = kk_integer_from_small(-1); /*int*/
    }
    _x2094 = kk_std_core_ssize__t(_x2095, _ctx); /*ssize_t*/
    matches1219 = kk_std_text_regex_regex_exec_all(_x2092, s, _x2093, _x2094, _ctx); /*list<list<sslice>>*/
    return kk_std_text_regex__lift1476_replace_all_1(repl, matches1219, kk_std_core__new_Nil(_ctx), _ctx);
  }
  {
    kk_std_core__list matches1259;
    kk_box_t _x2096;
    {
      kk_box_t _x0 = regex0.obj;
      kk_box_dup(_x0, _ctx);
      kk_std_text_regex__regex_drop(regex0, _ctx);
      _x2096 = _x0; /*any*/
    }
    kk_ssize_t _x2097 = (KK_IZ(0)); /*ssize_t*/
    kk_ssize_t _x2098;
    kk_integer_t _x2099;
    if (kk_std_core_types__is_Optional(atmost, _ctx)) {
      kk_box_t _box_x1742 = atmost._cons.Optional.value;
      kk_integer_t _atmost_4960 = kk_integer_unbox(_box_x1742, _ctx);
      kk_integer_dup(_atmost_4960, _ctx);
      kk_std_core_types__optional_drop(atmost, _ctx);
      _x2099 = _atmost_4960; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(atmost, _ctx);
      _x2099 = kk_integer_from_small(-1); /*int*/
    }
    _x2098 = kk_std_core_ssize__t(_x2099, _ctx); /*ssize_t*/
    matches1259 = kk_std_text_regex_regex_exec_all(_x2096, s, _x2097, _x2098, _ctx); /*list<list<sslice>>*/
    return kk_std_text_regex__lift1477_replace_all_1(repl, matches1259, kk_std_core__new_Nil(_ctx), _ctx);
  }
}

kk_std_text_regex__regex kk_std_text_regex_rx_nongroup;
 
// lifted local: groups-count, .spec.1337
// specialized: std/text/regex/concat-replace, on parameters repl1300, using:
// repl1300 = fn(caps: (list :: V -> V)<sslice>){
//   std/text/regex/replace-captures(caps, "");
// }

kk_string_t kk_std_text_regex__lift1478_groups_count(kk_std_core__list matches1299, kk_std_core__list acc1301, kk_context_t* _ctx) { /* (matches1299 : list<list<sslice>>, acc1301 : list<string>) -> string */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(matches1299, _ctx)) {
    struct kk_std_core_Cons* _con2107 = kk_std_core__as_Cons(matches1299, _ctx);
    kk_box_t _box_x1745 = _con2107->head;
    kk_std_core__list _pat01304 = _con2107->tail;
    if (kk_std_core__is_Cons(_pat01304, _ctx)) {
      struct kk_std_core_Cons* _con2108 = kk_std_core__as_Cons(_pat01304, _ctx);
      kk_box_t _box_x1746 = _con2108->head;
      kk_std_core__list pre1303 = kk_std_core__list_unbox(_box_x1745, KK_BORROWED, _ctx);
      kk_std_core__list mm1306 = _con2108->tail;
      kk_std_core__list m1305 = kk_std_core__list_unbox(_box_x1746, KK_BORROWED, _ctx);
      kk_reuse_t _ru_1918 = kk_reuse_null; /*reuse*/;
      kk_reuse_t _ru_1917 = kk_reuse_null; /*reuse*/;
      if kk_likely(kk_datatype_ptr_is_unique(matches1299, _ctx)) {
        if kk_likely(kk_datatype_ptr_is_unique(_pat01304, _ctx)) {
          _ru_1917 = (kk_datatype_ptr_reuse(_pat01304, _ctx));
        }
        else {
          kk_std_core__list_dup(m1305, _ctx);
          kk_std_core__list_dup(mm1306, _ctx);
          kk_datatype_ptr_decref(_pat01304, _ctx);
        }
        _ru_1918 = (kk_datatype_ptr_reuse(matches1299, _ctx));
      }
      else {
        kk_std_core__list_dup(m1305, _ctx);
        kk_std_core__list_dup(mm1306, _ctx);
        kk_std_core__list_dup(pre1303, _ctx);
        kk_datatype_ptr_decref(matches1299, _ctx);
      }
      kk_std_text_regex__regex r_999;
      kk_box_t _x2109;
      kk_string_t _x2110;
      kk_define_string_literal(, _s2111, 20, "\\$(\?:(\\d)|(\\&)|(\\$))", _ctx)
      _x2110 = kk_string_dup(_s2111, _ctx); /*string*/
      bool _x2112;
      kk_std_core_types__optional _match_1856 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
      if (kk_std_core_types__is_Optional(_match_1856, _ctx)) {
        kk_box_t _box_x1747 = _match_1856._cons.Optional.value;
        bool _ignorecase_455 = kk_bool_unbox(_box_x1747);
        kk_std_core_types__optional_drop(_match_1856, _ctx);
        _x2112 = _ignorecase_455; /*bool*/
      }
      else {
        kk_std_core_types__optional_drop(_match_1856, _ctx);
        _x2112 = false; /*bool*/
      }
      bool _x2113;
      kk_std_core_types__optional _match_1855 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
      if (kk_std_core_types__is_Optional(_match_1855, _ctx)) {
        kk_box_t _box_x1748 = _match_1855._cons.Optional.value;
        bool _multiline_459 = kk_bool_unbox(_box_x1748);
        kk_std_core_types__optional_drop(_match_1855, _ctx);
        _x2113 = _multiline_459; /*bool*/
      }
      else {
        kk_std_core_types__optional_drop(_match_1855, _ctx);
        _x2113 = false; /*bool*/
      }
      _x2109 = kk_std_text_regex_regex_create(_x2110, _x2112, _x2113, _ctx); /*any*/
      kk_string_t _x2114;
      kk_define_string_literal(, _s2115, 20, "\\$(\?:(\\d)|(\\&)|(\\$))", _ctx)
      _x2114 = kk_string_dup(_s2115, _ctx); /*string*/
      r_999 = kk_std_text_regex__new_Regex(_x2109, _x2114, _ctx); /*std/text/regex/regex*/
      kk_std_core__list matches1179_1486;
      kk_box_t _x2116;
      {
        kk_box_t _x = r_999.obj;
        kk_box_dup(_x, _ctx);
        kk_std_text_regex__regex_drop(r_999, _ctx);
        _x2116 = _x; /*any*/
      }
      kk_string_t _x2117 = kk_string_empty(); /*string*/
      kk_ssize_t _x2119 = (KK_IZ(0)); /*ssize_t*/
      kk_ssize_t _x2120;
      kk_integer_t _x2121;
      kk_std_core_types__optional _match_1854 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
      if (kk_std_core_types__is_Optional(_match_1854, _ctx)) {
        kk_box_t _box_x1749 = _match_1854._cons.Optional.value;
        kk_integer_t _atmost_475 = kk_integer_unbox(_box_x1749, _ctx);
        kk_integer_dup(_atmost_475, _ctx);
        kk_std_core_types__optional_drop(_match_1854, _ctx);
        _x2121 = _atmost_475; /*int*/
      }
      else {
        kk_std_core_types__optional_drop(_match_1854, _ctx);
        _x2121 = kk_integer_from_small(-1); /*int*/
      }
      _x2120 = kk_std_core_ssize__t(_x2121, _ctx); /*ssize_t*/
      matches1179_1486 = kk_std_text_regex_regex_exec_all(_x2116, _x2117, _x2119, _x2120, _ctx); /*list<list<sslice>>*/
      kk_string_t _b_1762_1753 = kk_std_text_regex__lift1475_replace_captures(m1305, matches1179_1486, kk_std_core__new_Nil(_ctx), _ctx); /*string*/;
      kk_string_t _b_1764_1751;
      if (kk_std_core__is_Cons(pre1303, _ctx)) {
        struct kk_std_core_Cons* _con2122 = kk_std_core__as_Cons(pre1303, _ctx);
        kk_box_t _box_x1750 = _con2122->head;
        kk_std_core__sslice s1309 = kk_std_core__sslice_unbox(_box_x1750, KK_BORROWED, _ctx);
        kk_std_core__list _pat31313 = _con2122->tail;
        if kk_likely(kk_datatype_ptr_is_unique(pre1303, _ctx)) {
          kk_std_core__list_drop(_pat31313, _ctx);
          kk_std_core__sslice_dup(s1309, _ctx);
          kk_box_drop(_box_x1750, _ctx);
          kk_datatype_ptr_free(pre1303, _ctx);
        }
        else {
          kk_std_core__sslice_dup(s1309, _ctx);
          kk_datatype_ptr_decref(pre1303, _ctx);
        }
        _b_1764_1751 = kk_std_core_string_3(s1309, _ctx); /*string*/
      }
      else {
        _b_1764_1751 = kk_string_empty(); /*string*/
      }
      { // tailcall
        kk_std_core__list _x2124;
        kk_std_core__list _x2125 = kk_std_core__new_Cons(_ru_1918, 0, kk_string_box(_b_1764_1751), acc1301, _ctx); /*list<61>*/
        _x2124 = kk_std_core__new_Cons(_ru_1917, 0, kk_string_box(_b_1762_1753), _x2125, _ctx); /*list<61>*/
        matches1299 = mm1306;
        acc1301 = _x2124;
        goto kk__tailcall;
      }
    }
  }
  if (kk_std_core__is_Cons(matches1299, _ctx)) {
    struct kk_std_core_Cons* _con2126 = kk_std_core__as_Cons(matches1299, _ctx);
    kk_box_t _box_x1755 = _con2126->head;
    kk_std_core__list _pat201317 = _con2126->tail;
    kk_std_core__list post1316 = kk_std_core__list_unbox(_box_x1755, KK_BORROWED, _ctx);
    kk_reuse_t _ru_1920 = kk_reuse_null; /*reuse*/;
    if kk_likely(kk_datatype_ptr_is_unique(matches1299, _ctx)) {
      _ru_1920 = (kk_datatype_ptr_reuse(matches1299, _ctx));
    }
    else {
      kk_std_core__list_dup(post1316, _ctx);
      kk_datatype_ptr_decref(matches1299, _ctx);
    }
    kk_string_t _b_1759_1757;
    if (kk_std_core__is_Cons(post1316, _ctx)) {
      struct kk_std_core_Cons* _con2127 = kk_std_core__as_Cons(post1316, _ctx);
      kk_box_t _box_x1756 = _con2127->head;
      kk_std_core__sslice s01330 = kk_std_core__sslice_unbox(_box_x1756, KK_BORROWED, _ctx);
      kk_std_core__list _pat311334 = _con2127->tail;
      if kk_likely(kk_datatype_ptr_is_unique(post1316, _ctx)) {
        kk_std_core__list_drop(_pat311334, _ctx);
        kk_std_core__sslice_dup(s01330, _ctx);
        kk_box_drop(_box_x1756, _ctx);
        kk_datatype_ptr_free(post1316, _ctx);
      }
      else {
        kk_std_core__sslice_dup(s01330, _ctx);
        kk_datatype_ptr_decref(post1316, _ctx);
      }
      _b_1759_1757 = kk_std_core_string_3(s01330, _ctx); /*string*/
    }
    else {
      _b_1759_1757 = kk_string_empty(); /*string*/
    }
    kk_std_core__list xs1318 = kk_std_core__new_Cons(_ru_1920, 0, kk_string_box(_b_1759_1757), acc1301, _ctx); /*list<string>*/;
    kk_std_core__list xs0_177981321;
    if (kk_std_core__is_Nil(xs1318, _ctx)) {
      xs0_177981321 = kk_std_core__new_Nil(_ctx); /*list<string>*/
    }
    else {
      xs0_177981321 = kk_std_core__unroll17327_lift17313_reverse_append(kk_std_core__new_Nil(_ctx), xs1318, _ctx); /*list<string>*/
    }
    if (kk_std_core__is_Nil(xs0_177981321, _ctx)) {
      return kk_string_empty();
    }
    {
      struct kk_std_core_Cons* _con2130 = kk_std_core__as_Cons(xs0_177981321, _ctx);
      kk_box_t _box_x1761 = _con2130->head;
      kk_std_core__list xx1325 = _con2130->tail;
      kk_string_t x1324 = kk_string_unbox(_box_x1761);
      if kk_likely(kk_datatype_ptr_is_unique(xs0_177981321, _ctx)) {
        kk_datatype_ptr_free(xs0_177981321, _ctx);
      }
      else {
        kk_string_dup(x1324, _ctx);
        kk_std_core__list_dup(xx1325, _ctx);
        kk_datatype_ptr_decref(xs0_177981321, _ctx);
      }
      if (kk_std_core__is_Nil(xx1325, _ctx)) {
        return x1324;
      }
      {
        kk_string_t _x2131 = kk_string_empty(); /*string*/
        return kk_std_core__unroll17331_lift17316_joinsep(_x2131, xx1325, x1324, _ctx);
      }
    }
  }
  {
    return kk_std_core_reverse_join(acc1301, _ctx);
  }
}
 
// lifted local: groups-count, .spec.1377
// specialized: std/text/regex/concat-replace, on parameters repl1340, using:
// repl1340 = fn(._wildcard_158_35: (list :: V -> V)<sslice>){
//   "";
// }

kk_string_t kk_std_text_regex__lift1479_groups_count(kk_std_core__list matches1339, kk_std_core__list acc1341, kk_context_t* _ctx) { /* (matches1339 : list<list<sslice>>, acc1341 : list<string>) -> string */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(matches1339, _ctx)) {
    struct kk_std_core_Cons* _con2133 = kk_std_core__as_Cons(matches1339, _ctx);
    kk_box_t _box_x1766 = _con2133->head;
    kk_std_core__list _pat01344 = _con2133->tail;
    if (kk_std_core__is_Cons(_pat01344, _ctx)) {
      struct kk_std_core_Cons* _con2134 = kk_std_core__as_Cons(_pat01344, _ctx);
      kk_box_t _box_x1767 = _con2134->head;
      kk_std_core__list pre1343 = kk_std_core__list_unbox(_box_x1766, KK_BORROWED, _ctx);
      kk_std_core__list mm1346 = _con2134->tail;
      kk_reuse_t _ru_1924 = kk_reuse_null; /*reuse*/;
      kk_reuse_t _ru_1923 = kk_reuse_null; /*reuse*/;
      if kk_likely(kk_datatype_ptr_is_unique(matches1339, _ctx)) {
        if kk_likely(kk_datatype_ptr_is_unique(_pat01344, _ctx)) {
          kk_box_drop(_box_x1767, _ctx);
          _ru_1923 = (kk_datatype_ptr_reuse(_pat01344, _ctx));
        }
        else {
          kk_std_core__list_dup(mm1346, _ctx);
          kk_datatype_ptr_decref(_pat01344, _ctx);
        }
        _ru_1924 = (kk_datatype_ptr_reuse(matches1339, _ctx));
      }
      else {
        kk_std_core__list_dup(mm1346, _ctx);
        kk_std_core__list_dup(pre1343, _ctx);
        kk_datatype_ptr_decref(matches1339, _ctx);
      }
      kk_string_t _b_1782_1769;
      if (kk_std_core__is_Cons(pre1343, _ctx)) {
        struct kk_std_core_Cons* _con2135 = kk_std_core__as_Cons(pre1343, _ctx);
        kk_box_t _box_x1768 = _con2135->head;
        kk_std_core__sslice s1349 = kk_std_core__sslice_unbox(_box_x1768, KK_BORROWED, _ctx);
        kk_std_core__list _pat31353 = _con2135->tail;
        if kk_likely(kk_datatype_ptr_is_unique(pre1343, _ctx)) {
          kk_std_core__list_drop(_pat31353, _ctx);
          kk_std_core__sslice_dup(s1349, _ctx);
          kk_box_drop(_box_x1768, _ctx);
          kk_datatype_ptr_free(pre1343, _ctx);
        }
        else {
          kk_std_core__sslice_dup(s1349, _ctx);
          kk_datatype_ptr_decref(pre1343, _ctx);
        }
        _b_1782_1769 = kk_std_core_string_3(s1349, _ctx); /*string*/
      }
      else {
        _b_1782_1769 = kk_string_empty(); /*string*/
      }
      { // tailcall
        kk_std_core__list _x2137;
        kk_box_t _x2138;
        kk_string_t _x2139 = kk_string_empty(); /*string*/
        _x2138 = kk_string_box(_x2139); /*61*/
        kk_std_core__list _x2141 = kk_std_core__new_Cons(_ru_1924, 0, kk_string_box(_b_1782_1769), acc1341, _ctx); /*list<61>*/
        _x2137 = kk_std_core__new_Cons(_ru_1923, 0, _x2138, _x2141, _ctx); /*list<61>*/
        matches1339 = mm1346;
        acc1341 = _x2137;
        goto kk__tailcall;
      }
    }
  }
  if (kk_std_core__is_Cons(matches1339, _ctx)) {
    struct kk_std_core_Cons* _con2142 = kk_std_core__as_Cons(matches1339, _ctx);
    kk_box_t _box_x1773 = _con2142->head;
    kk_std_core__list _pat201357 = _con2142->tail;
    kk_std_core__list post1356 = kk_std_core__list_unbox(_box_x1773, KK_BORROWED, _ctx);
    kk_reuse_t _ru_1926 = kk_reuse_null; /*reuse*/;
    if kk_likely(kk_datatype_ptr_is_unique(matches1339, _ctx)) {
      _ru_1926 = (kk_datatype_ptr_reuse(matches1339, _ctx));
    }
    else {
      kk_std_core__list_dup(post1356, _ctx);
      kk_datatype_ptr_decref(matches1339, _ctx);
    }
    kk_string_t _b_1777_1775;
    if (kk_std_core__is_Cons(post1356, _ctx)) {
      struct kk_std_core_Cons* _con2143 = kk_std_core__as_Cons(post1356, _ctx);
      kk_box_t _box_x1774 = _con2143->head;
      kk_std_core__sslice s01370 = kk_std_core__sslice_unbox(_box_x1774, KK_BORROWED, _ctx);
      kk_std_core__list _pat311374 = _con2143->tail;
      if kk_likely(kk_datatype_ptr_is_unique(post1356, _ctx)) {
        kk_std_core__list_drop(_pat311374, _ctx);
        kk_std_core__sslice_dup(s01370, _ctx);
        kk_box_drop(_box_x1774, _ctx);
        kk_datatype_ptr_free(post1356, _ctx);
      }
      else {
        kk_std_core__sslice_dup(s01370, _ctx);
        kk_datatype_ptr_decref(post1356, _ctx);
      }
      _b_1777_1775 = kk_std_core_string_3(s01370, _ctx); /*string*/
    }
    else {
      _b_1777_1775 = kk_string_empty(); /*string*/
    }
    kk_std_core__list xs1358 = kk_std_core__new_Cons(_ru_1926, 0, kk_string_box(_b_1777_1775), acc1341, _ctx); /*list<string>*/;
    kk_std_core__list xs0_177981361;
    if (kk_std_core__is_Nil(xs1358, _ctx)) {
      xs0_177981361 = kk_std_core__new_Nil(_ctx); /*list<string>*/
    }
    else {
      xs0_177981361 = kk_std_core__unroll17327_lift17313_reverse_append(kk_std_core__new_Nil(_ctx), xs1358, _ctx); /*list<string>*/
    }
    if (kk_std_core__is_Nil(xs0_177981361, _ctx)) {
      return kk_string_empty();
    }
    {
      struct kk_std_core_Cons* _con2146 = kk_std_core__as_Cons(xs0_177981361, _ctx);
      kk_box_t _box_x1779 = _con2146->head;
      kk_std_core__list xx1365 = _con2146->tail;
      kk_string_t x1364 = kk_string_unbox(_box_x1779);
      if kk_likely(kk_datatype_ptr_is_unique(xs0_177981361, _ctx)) {
        kk_datatype_ptr_free(xs0_177981361, _ctx);
      }
      else {
        kk_string_dup(x1364, _ctx);
        kk_std_core__list_dup(xx1365, _ctx);
        kk_datatype_ptr_decref(xs0_177981361, _ctx);
      }
      if (kk_std_core__is_Nil(xx1365, _ctx)) {
        return x1364;
      }
      {
        kk_string_t _x2147 = kk_string_empty(); /*string*/
        return kk_std_core__unroll17331_lift17316_joinsep(_x2147, xx1365, x1364, _ctx);
      }
    }
  }
  {
    return kk_std_core_reverse_join(acc1341, _ctx);
  }
}
 
// How many groups are captured by this regex?

kk_integer_t kk_std_text_regex_groups_count(kk_std_text_regex__regex r, kk_context_t* _ctx) { /* (r : regex) -> int */ 
  kk_string_t _x2149;
  bool _match_1851;
  kk_string_t _x2150 = kk_string_empty(); /*string*/
  kk_string_t _x2152;
  kk_define_string_literal(, _s2153, 1, "$", _ctx)
  _x2152 = kk_string_dup(_s2153, _ctx); /*string*/
  _match_1851 = kk_string_contains(_x2150,_x2152,kk_context()); /*bool*/
  if (_match_1851) {
    kk_std_core__list _x2154;
    kk_box_t _x2155;
    kk_std_text_regex__regex _x2156 = kk_std_text_regex_rx_nongroup; /*std/text/regex/regex*/
    {
      kk_box_t _x = _x2156.obj;
      _x2155 = kk_box_dup(_x, _ctx); /*any*/
    }
    kk_string_t _x2157;
    {
      kk_string_t _x1 = r.src;
      kk_string_dup(_x1, _ctx);
      kk_std_text_regex__regex_drop(r, _ctx);
      _x2157 = _x1; /*string*/
    }
    kk_ssize_t _x2158 = (KK_IZ(0)); /*ssize_t*/
    kk_ssize_t _x2159;
    kk_integer_t _x2160;
    kk_std_core_types__optional _match_1853 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_1853, _ctx)) {
      kk_box_t _box_x1784 = _match_1853._cons.Optional.value;
      kk_integer_t _atmost_496 = kk_integer_unbox(_box_x1784, _ctx);
      kk_integer_dup(_atmost_496, _ctx);
      kk_std_core_types__optional_drop(_match_1853, _ctx);
      _x2160 = _atmost_496; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(_match_1853, _ctx);
      _x2160 = kk_integer_from_small(-1); /*int*/
    }
    _x2159 = kk_std_core_ssize__t(_x2160, _ctx); /*ssize_t*/
    _x2154 = kk_std_text_regex_regex_exec_all(_x2155, _x2157, _x2158, _x2159, _ctx); /*list<list<sslice>>*/
    _x2149 = kk_std_text_regex__lift1478_groups_count(_x2154, kk_std_core__new_Nil(_ctx), _ctx); /*string*/
  }
  else {
    kk_std_core__list _x2161;
    kk_box_t _x2162;
    kk_std_text_regex__regex _x2163 = kk_std_text_regex_rx_nongroup; /*std/text/regex/regex*/
    {
      kk_box_t _x0 = _x2163.obj;
      _x2162 = kk_box_dup(_x0, _ctx); /*any*/
    }
    kk_string_t _x2164;
    {
      kk_string_t _x10 = r.src;
      kk_string_dup(_x10, _ctx);
      kk_std_text_regex__regex_drop(r, _ctx);
      _x2164 = _x10; /*string*/
    }
    kk_ssize_t _x2165 = (KK_IZ(0)); /*ssize_t*/
    kk_ssize_t _x2166;
    kk_integer_t _x2167;
    kk_std_core_types__optional _match_1852 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_1852, _ctx)) {
      kk_box_t _box_x1785 = _match_1852._cons.Optional.value;
      kk_integer_t _atmost_4960 = kk_integer_unbox(_box_x1785, _ctx);
      kk_integer_dup(_atmost_4960, _ctx);
      kk_std_core_types__optional_drop(_match_1852, _ctx);
      _x2167 = _atmost_4960; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(_match_1852, _ctx);
      _x2167 = kk_integer_from_small(-1); /*int*/
    }
    _x2166 = kk_std_core_ssize__t(_x2167, _ctx); /*ssize_t*/
    _x2161 = kk_std_text_regex_regex_exec_all(_x2162, _x2164, _x2165, _x2166, _ctx); /*list<list<sslice>>*/
    _x2149 = kk_std_text_regex__lift1479_groups_count(_x2161, kk_std_core__new_Nil(_ctx), _ctx); /*string*/
  }
  return kk_std_core_count_1(_x2149, _ctx);
}
 
// Replace the first occurrence of `regex` by the result of the replacement fun `repl` in a string `s`.

kk_string_t kk_std_text_regex_replace(kk_string_t s, kk_std_text_regex__regex r, kk_function_t repl, kk_context_t* _ctx) { /* forall<e> (s : string, r : regex, repl : (list<sslice>) -> e string) -> e string */ 
  kk_std_core__list _x2168;
  kk_box_t _x2169;
  {
    kk_box_t _x = r.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(r, _ctx);
    _x2169 = _x; /*any*/
  }
  kk_ssize_t _x2170 = (KK_IZ(0)); /*ssize_t*/
  kk_ssize_t _x2171 = (KK_IZ(1)); /*ssize_t*/
  _x2168 = kk_std_text_regex_regex_exec_all(_x2169, s, _x2170, _x2171, _ctx); /*list<list<sslice>>*/
  return kk_std_text_regex_concat_replace(_x2168, repl, kk_std_core__new_Nil(_ctx), _ctx);
}
 
// lifted local: replace.1, .spec.1417
// specialized: std/text/regex/concat-replace, on parameters repl1380, using:
// repl1380 = fn(caps: (list :: V -> V)<sslice>){
//   std/text/regex/replace-captures(caps, repl);
// }

kk_string_t kk_std_text_regex__lift1480_replace_1(kk_string_t repl, kk_std_core__list matches1379, kk_std_core__list acc1381, kk_context_t* _ctx) { /* (repl : string, matches1379 : list<list<sslice>>, acc1381 : list<string>) -> string */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(matches1379, _ctx)) {
    struct kk_std_core_Cons* _con2172 = kk_std_core__as_Cons(matches1379, _ctx);
    kk_box_t _box_x1786 = _con2172->head;
    kk_std_core__list _pat01384 = _con2172->tail;
    if (kk_std_core__is_Cons(_pat01384, _ctx)) {
      struct kk_std_core_Cons* _con2173 = kk_std_core__as_Cons(_pat01384, _ctx);
      kk_box_t _box_x1787 = _con2173->head;
      kk_std_core__list pre1383 = kk_std_core__list_unbox(_box_x1786, KK_BORROWED, _ctx);
      kk_std_core__list mm1386 = _con2173->tail;
      kk_std_core__list m1385 = kk_std_core__list_unbox(_box_x1787, KK_BORROWED, _ctx);
      kk_reuse_t _ru_1930 = kk_reuse_null; /*reuse*/;
      kk_reuse_t _ru_1929 = kk_reuse_null; /*reuse*/;
      if kk_likely(kk_datatype_ptr_is_unique(matches1379, _ctx)) {
        if kk_likely(kk_datatype_ptr_is_unique(_pat01384, _ctx)) {
          _ru_1929 = (kk_datatype_ptr_reuse(_pat01384, _ctx));
        }
        else {
          kk_std_core__list_dup(m1385, _ctx);
          kk_std_core__list_dup(mm1386, _ctx);
          kk_datatype_ptr_decref(_pat01384, _ctx);
        }
        _ru_1930 = (kk_datatype_ptr_reuse(matches1379, _ctx));
      }
      else {
        kk_std_core__list_dup(m1385, _ctx);
        kk_std_core__list_dup(mm1386, _ctx);
        kk_std_core__list_dup(pre1383, _ctx);
        kk_datatype_ptr_decref(matches1379, _ctx);
      }
      kk_string_t _b_1793_1791;
      kk_string_t _x2174 = kk_string_dup(repl, _ctx); /*string*/
      _b_1793_1791 = kk_std_text_regex_replace_captures(m1385, _x2174, _ctx); /*string*/
      kk_string_t _b_1795_1789;
      if (kk_std_core__is_Cons(pre1383, _ctx)) {
        struct kk_std_core_Cons* _con2175 = kk_std_core__as_Cons(pre1383, _ctx);
        kk_box_t _box_x1788 = _con2175->head;
        kk_std_core__sslice s1389 = kk_std_core__sslice_unbox(_box_x1788, KK_BORROWED, _ctx);
        kk_std_core__list _pat31393 = _con2175->tail;
        if kk_likely(kk_datatype_ptr_is_unique(pre1383, _ctx)) {
          kk_std_core__list_drop(_pat31393, _ctx);
          kk_std_core__sslice_dup(s1389, _ctx);
          kk_box_drop(_box_x1788, _ctx);
          kk_datatype_ptr_free(pre1383, _ctx);
        }
        else {
          kk_std_core__sslice_dup(s1389, _ctx);
          kk_datatype_ptr_decref(pre1383, _ctx);
        }
        _b_1795_1789 = kk_std_core_string_3(s1389, _ctx); /*string*/
      }
      else {
        _b_1795_1789 = kk_string_empty(); /*string*/
      }
      { // tailcall
        kk_std_core__list _x2177;
        kk_std_core__list _x2178 = kk_std_core__new_Cons(_ru_1930, 0, kk_string_box(_b_1795_1789), acc1381, _ctx); /*list<61>*/
        _x2177 = kk_std_core__new_Cons(_ru_1929, 0, kk_string_box(_b_1793_1791), _x2178, _ctx); /*list<61>*/
        matches1379 = mm1386;
        acc1381 = _x2177;
        goto kk__tailcall;
      }
    }
  }
  if (kk_std_core__is_Cons(matches1379, _ctx)) {
    struct kk_std_core_Cons* _con2179 = kk_std_core__as_Cons(matches1379, _ctx);
    kk_box_t _box_x1797 = _con2179->head;
    kk_std_core__list _pat201397 = _con2179->tail;
    kk_std_core__list post1396 = kk_std_core__list_unbox(_box_x1797, KK_BORROWED, _ctx);
    kk_string_drop(repl, _ctx);
    kk_reuse_t _ru_1932 = kk_reuse_null; /*reuse*/;
    if kk_likely(kk_datatype_ptr_is_unique(matches1379, _ctx)) {
      _ru_1932 = (kk_datatype_ptr_reuse(matches1379, _ctx));
    }
    else {
      kk_std_core__list_dup(post1396, _ctx);
      kk_datatype_ptr_decref(matches1379, _ctx);
    }
    kk_string_t _b_1801_1799;
    if (kk_std_core__is_Cons(post1396, _ctx)) {
      struct kk_std_core_Cons* _con2180 = kk_std_core__as_Cons(post1396, _ctx);
      kk_box_t _box_x1798 = _con2180->head;
      kk_std_core__sslice s01410 = kk_std_core__sslice_unbox(_box_x1798, KK_BORROWED, _ctx);
      kk_std_core__list _pat311414 = _con2180->tail;
      if kk_likely(kk_datatype_ptr_is_unique(post1396, _ctx)) {
        kk_std_core__list_drop(_pat311414, _ctx);
        kk_std_core__sslice_dup(s01410, _ctx);
        kk_box_drop(_box_x1798, _ctx);
        kk_datatype_ptr_free(post1396, _ctx);
      }
      else {
        kk_std_core__sslice_dup(s01410, _ctx);
        kk_datatype_ptr_decref(post1396, _ctx);
      }
      _b_1801_1799 = kk_std_core_string_3(s01410, _ctx); /*string*/
    }
    else {
      _b_1801_1799 = kk_string_empty(); /*string*/
    }
    kk_std_core__list xs1398 = kk_std_core__new_Cons(_ru_1932, 0, kk_string_box(_b_1801_1799), acc1381, _ctx); /*list<string>*/;
    kk_std_core__list xs0_177981401;
    if (kk_std_core__is_Nil(xs1398, _ctx)) {
      xs0_177981401 = kk_std_core__new_Nil(_ctx); /*list<string>*/
    }
    else {
      xs0_177981401 = kk_std_core__unroll17327_lift17313_reverse_append(kk_std_core__new_Nil(_ctx), xs1398, _ctx); /*list<string>*/
    }
    if (kk_std_core__is_Nil(xs0_177981401, _ctx)) {
      return kk_string_empty();
    }
    {
      struct kk_std_core_Cons* _con2183 = kk_std_core__as_Cons(xs0_177981401, _ctx);
      kk_box_t _box_x1803 = _con2183->head;
      kk_std_core__list xx1405 = _con2183->tail;
      kk_string_t x1404 = kk_string_unbox(_box_x1803);
      if kk_likely(kk_datatype_ptr_is_unique(xs0_177981401, _ctx)) {
        kk_datatype_ptr_free(xs0_177981401, _ctx);
      }
      else {
        kk_string_dup(x1404, _ctx);
        kk_std_core__list_dup(xx1405, _ctx);
        kk_datatype_ptr_decref(xs0_177981401, _ctx);
      }
      if (kk_std_core__is_Nil(xx1405, _ctx)) {
        return x1404;
      }
      {
        kk_string_t _x2184 = kk_string_empty(); /*string*/
        return kk_std_core__unroll17331_lift17316_joinsep(_x2184, xx1405, x1404, _ctx);
      }
    }
  }
  {
    kk_string_drop(repl, _ctx);
    return kk_std_core_reverse_join(acc1381, _ctx);
  }
}
 
// lifted local: replace.1, .spec.1457
// specialized: std/text/regex/concat-replace, on parameters repl1420, using:
// repl1420 = fn(._wildcard_158_35: (list :: V -> V)<sslice>){
//   repl;
// }

kk_string_t kk_std_text_regex__lift1481_replace_1(kk_string_t repl, kk_std_core__list matches1419, kk_std_core__list acc1421, kk_context_t* _ctx) { /* (repl : string, matches1419 : list<list<sslice>>, acc1421 : list<string>) -> string */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(matches1419, _ctx)) {
    struct kk_std_core_Cons* _con2186 = kk_std_core__as_Cons(matches1419, _ctx);
    kk_box_t _box_x1804 = _con2186->head;
    kk_std_core__list _pat01424 = _con2186->tail;
    if (kk_std_core__is_Cons(_pat01424, _ctx)) {
      struct kk_std_core_Cons* _con2187 = kk_std_core__as_Cons(_pat01424, _ctx);
      kk_box_t _box_x1805 = _con2187->head;
      kk_std_core__list pre1423 = kk_std_core__list_unbox(_box_x1804, KK_BORROWED, _ctx);
      kk_std_core__list mm1426 = _con2187->tail;
      kk_reuse_t _ru_1936 = kk_reuse_null; /*reuse*/;
      kk_reuse_t _ru_1935 = kk_reuse_null; /*reuse*/;
      if kk_likely(kk_datatype_ptr_is_unique(matches1419, _ctx)) {
        if kk_likely(kk_datatype_ptr_is_unique(_pat01424, _ctx)) {
          kk_box_drop(_box_x1805, _ctx);
          _ru_1935 = (kk_datatype_ptr_reuse(_pat01424, _ctx));
        }
        else {
          kk_std_core__list_dup(mm1426, _ctx);
          kk_datatype_ptr_decref(_pat01424, _ctx);
        }
        _ru_1936 = (kk_datatype_ptr_reuse(matches1419, _ctx));
      }
      else {
        kk_std_core__list_dup(mm1426, _ctx);
        kk_std_core__list_dup(pre1423, _ctx);
        kk_datatype_ptr_decref(matches1419, _ctx);
      }
      kk_string_t _b_1813_1807;
      if (kk_std_core__is_Cons(pre1423, _ctx)) {
        struct kk_std_core_Cons* _con2188 = kk_std_core__as_Cons(pre1423, _ctx);
        kk_box_t _box_x1806 = _con2188->head;
        kk_std_core__sslice s1429 = kk_std_core__sslice_unbox(_box_x1806, KK_BORROWED, _ctx);
        kk_std_core__list _pat31433 = _con2188->tail;
        if kk_likely(kk_datatype_ptr_is_unique(pre1423, _ctx)) {
          kk_std_core__list_drop(_pat31433, _ctx);
          kk_std_core__sslice_dup(s1429, _ctx);
          kk_box_drop(_box_x1806, _ctx);
          kk_datatype_ptr_free(pre1423, _ctx);
        }
        else {
          kk_std_core__sslice_dup(s1429, _ctx);
          kk_datatype_ptr_decref(pre1423, _ctx);
        }
        _b_1813_1807 = kk_std_core_string_3(s1429, _ctx); /*string*/
      }
      else {
        _b_1813_1807 = kk_string_empty(); /*string*/
      }
      { // tailcall
        kk_string_t _x2190 = kk_string_dup(repl, _ctx); /*string*/
        kk_std_core__list _x2191;
        kk_std_core__list _x2192 = kk_std_core__new_Cons(_ru_1936, 0, kk_string_box(_b_1813_1807), acc1421, _ctx); /*list<61>*/
        _x2191 = kk_std_core__new_Cons(_ru_1935, 0, kk_string_box(repl), _x2192, _ctx); /*list<61>*/
        repl = _x2190;
        matches1419 = mm1426;
        acc1421 = _x2191;
        goto kk__tailcall;
      }
    }
  }
  if (kk_std_core__is_Cons(matches1419, _ctx)) {
    struct kk_std_core_Cons* _con2193 = kk_std_core__as_Cons(matches1419, _ctx);
    kk_box_t _box_x1815 = _con2193->head;
    kk_std_core__list _pat201437 = _con2193->tail;
    kk_std_core__list post1436 = kk_std_core__list_unbox(_box_x1815, KK_BORROWED, _ctx);
    kk_string_drop(repl, _ctx);
    kk_reuse_t _ru_1938 = kk_reuse_null; /*reuse*/;
    if kk_likely(kk_datatype_ptr_is_unique(matches1419, _ctx)) {
      _ru_1938 = (kk_datatype_ptr_reuse(matches1419, _ctx));
    }
    else {
      kk_std_core__list_dup(post1436, _ctx);
      kk_datatype_ptr_decref(matches1419, _ctx);
    }
    kk_string_t _b_1819_1817;
    if (kk_std_core__is_Cons(post1436, _ctx)) {
      struct kk_std_core_Cons* _con2194 = kk_std_core__as_Cons(post1436, _ctx);
      kk_box_t _box_x1816 = _con2194->head;
      kk_std_core__sslice s01450 = kk_std_core__sslice_unbox(_box_x1816, KK_BORROWED, _ctx);
      kk_std_core__list _pat311454 = _con2194->tail;
      if kk_likely(kk_datatype_ptr_is_unique(post1436, _ctx)) {
        kk_std_core__list_drop(_pat311454, _ctx);
        kk_std_core__sslice_dup(s01450, _ctx);
        kk_box_drop(_box_x1816, _ctx);
        kk_datatype_ptr_free(post1436, _ctx);
      }
      else {
        kk_std_core__sslice_dup(s01450, _ctx);
        kk_datatype_ptr_decref(post1436, _ctx);
      }
      _b_1819_1817 = kk_std_core_string_3(s01450, _ctx); /*string*/
    }
    else {
      _b_1819_1817 = kk_string_empty(); /*string*/
    }
    kk_std_core__list xs1438 = kk_std_core__new_Cons(_ru_1938, 0, kk_string_box(_b_1819_1817), acc1421, _ctx); /*list<string>*/;
    kk_std_core__list xs0_177981441;
    if (kk_std_core__is_Nil(xs1438, _ctx)) {
      xs0_177981441 = kk_std_core__new_Nil(_ctx); /*list<string>*/
    }
    else {
      xs0_177981441 = kk_std_core__unroll17327_lift17313_reverse_append(kk_std_core__new_Nil(_ctx), xs1438, _ctx); /*list<string>*/
    }
    if (kk_std_core__is_Nil(xs0_177981441, _ctx)) {
      return kk_string_empty();
    }
    {
      struct kk_std_core_Cons* _con2197 = kk_std_core__as_Cons(xs0_177981441, _ctx);
      kk_box_t _box_x1821 = _con2197->head;
      kk_std_core__list xx1445 = _con2197->tail;
      kk_string_t x1444 = kk_string_unbox(_box_x1821);
      if kk_likely(kk_datatype_ptr_is_unique(xs0_177981441, _ctx)) {
        kk_datatype_ptr_free(xs0_177981441, _ctx);
      }
      else {
        kk_string_dup(x1444, _ctx);
        kk_std_core__list_dup(xx1445, _ctx);
        kk_datatype_ptr_decref(xs0_177981441, _ctx);
      }
      if (kk_std_core__is_Nil(xx1445, _ctx)) {
        return x1444;
      }
      {
        kk_string_t _x2198 = kk_string_empty(); /*string*/
        return kk_std_core__unroll17331_lift17316_joinsep(_x2198, xx1445, x1444, _ctx);
      }
    }
  }
  {
    kk_string_drop(repl, _ctx);
    return kk_std_core_reverse_join(acc1421, _ctx);
  }
}
 
// Replace the first occurrence of `regex` with a replacement string `repl` in a string `s`.
// The replacement string can contain `$$` for a `$` sign, `$n` for a capture group,
// `$&` for the entire match `==$0`.

kk_string_t kk_std_text_regex_replace_1(kk_string_t s, kk_std_text_regex__regex regex0, kk_string_t repl, kk_context_t* _ctx) { /* (s : string, regex : regex, repl : string) -> string */ 
  bool _match_1850;
  kk_string_t _x2200 = kk_string_dup(repl, _ctx); /*string*/
  kk_string_t _x2201;
  kk_define_string_literal(, _s2202, 1, "$", _ctx)
  _x2201 = kk_string_dup(_s2202, _ctx); /*string*/
  _match_1850 = kk_string_contains(_x2200,_x2201,kk_context()); /*bool*/
  if (_match_1850) {
    kk_std_core__list matches1379;
    kk_box_t _x2203;
    {
      kk_box_t _x = regex0.obj;
      kk_box_dup(_x, _ctx);
      kk_std_text_regex__regex_drop(regex0, _ctx);
      _x2203 = _x; /*any*/
    }
    kk_ssize_t _x2204 = (KK_IZ(0)); /*ssize_t*/
    kk_ssize_t _x2205 = (KK_IZ(1)); /*ssize_t*/
    matches1379 = kk_std_text_regex_regex_exec_all(_x2203, s, _x2204, _x2205, _ctx); /*list<list<sslice>>*/
    return kk_std_text_regex__lift1480_replace_1(repl, matches1379, kk_std_core__new_Nil(_ctx), _ctx);
  }
  {
    kk_std_core__list matches1419;
    kk_box_t _x2206;
    {
      kk_box_t _x0 = regex0.obj;
      kk_box_dup(_x0, _ctx);
      kk_std_text_regex__regex_drop(regex0, _ctx);
      _x2206 = _x0; /*any*/
    }
    kk_ssize_t _x2207 = (KK_IZ(0)); /*ssize_t*/
    kk_ssize_t _x2208 = (KK_IZ(1)); /*ssize_t*/
    matches1419 = kk_std_text_regex_regex_exec_all(_x2206, s, _x2207, _x2208, _ctx); /*list<list<sslice>>*/
    return kk_std_text_regex__lift1481_replace_1(repl, matches1419, kk_std_core__new_Nil(_ctx), _ctx);
  }
}
 
// lifted local: .lift1482-split, split, .spec.1470
// specialized: std/core/.unroll17333-map.5, on parameters f1465, using:
// f1465 = std/text/regex/captured

kk_std_core__list kk_std_text_regex__trmc_lift1483_split(kk_std_core__list xs1464, kk_std_core_types__cctx _acc, kk_context_t* _ctx) { /* (xs1464 : list<list<sslice>>, ctx<list<string>>) -> list<string> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs1464, _ctx)) {
    struct kk_std_core_Cons* _con2209 = kk_std_core__as_Cons(xs1464, _ctx);
    kk_box_t _box_x1822 = _con2209->head;
    kk_std_core__list xx1468 = _con2209->tail;
    kk_std_core__list x1467 = kk_std_core__list_unbox(_box_x1822, KK_BORROWED, _ctx);
    kk_reuse_t _ru_1941 = kk_reuse_null; /*reuse*/;
    if kk_likely(kk_datatype_ptr_is_unique(xs1464, _ctx)) {
      _ru_1941 = (kk_datatype_ptr_reuse(xs1464, _ctx));
    }
    else {
      kk_std_core__list_dup(x1467, _ctx);
      kk_std_core__list_dup(xx1468, _ctx);
      kk_datatype_ptr_decref(xs1464, _ctx);
    }
    kk_string_t _trmc_1506;
    if (kk_std_core__is_Cons(x1467, _ctx)) {
      struct kk_std_core_Cons* _con2210 = kk_std_core__as_Cons(x1467, _ctx);
      kk_box_t _box_x1823 = _con2210->head;
      kk_std_core__sslice s = kk_std_core__sslice_unbox(_box_x1823, KK_BORROWED, _ctx);
      kk_std_core__list _pat3 = _con2210->tail;
      if kk_likely(kk_datatype_ptr_is_unique(x1467, _ctx)) {
        kk_std_core__list_drop(_pat3, _ctx);
        kk_std_core__sslice_dup(s, _ctx);
        kk_box_drop(_box_x1823, _ctx);
        kk_datatype_ptr_free(x1467, _ctx);
      }
      else {
        kk_std_core__sslice_dup(s, _ctx);
        kk_datatype_ptr_decref(x1467, _ctx);
      }
      _trmc_1506 = kk_std_core_string_3(s, _ctx); /*string*/
    }
    else {
      _trmc_1506 = kk_string_empty(); /*string*/
    }
    kk_std_core__list _trmc_1507 = kk_datatype_null(); /*list<string>*/;
    kk_std_core__list _trmc_1508 = kk_std_core__new_Cons(_ru_1941, 0, kk_string_box(_trmc_1506), _trmc_1507, _ctx); /*list<string>*/;
    kk_box_t* _b_1838_1833 = (kk_box_t*)((&kk_std_core__as_Cons(_trmc_1508, _ctx)->tail)); /*field-addr<list<string>>*/;
    { // tailcall
      kk_std_core_types__cctx _x2212 = kk_cctx_extend(_acc,(kk_std_core__list_box(_trmc_1508, _ctx)),_b_1838_1833,true,kk_context()); /*ctx<0>*/
      xs1464 = xx1468;
      _acc = _x2212;
      goto kk__tailcall;
    }
  }
  {
    kk_box_t _x2213 = kk_cctx_apply(_acc,(kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx)),true,kk_context()); /*-1*/
    return kk_std_core__list_unbox(_x2213, KK_OWNED, _ctx);
  }
}
 
// lifted local: .lift1482-split, split, .spec.1470
// specialized: std/core/.unroll17333-map.5, on parameters f1465, using:
// f1465 = std/text/regex/captured

kk_std_core__list kk_std_text_regex__lift1483_split(kk_std_core__list xs14640, kk_context_t* _ctx) { /* (xs1464 : list<list<sslice>>) -> list<string> */ 
  kk_std_core_types__cctx _x2214 = kk_cctx_empty(kk_context()); /*ctx<0>*/
  return kk_std_text_regex__trmc_lift1483_split(xs14640, _x2214, _ctx);
}
 
// lifted local: split, .spec.1463
// specialized: std/core/map.5, on parameters f1460, using:
// f1460 = std/text/regex/captured

kk_std_core__list kk_std_text_regex__lift1482_split(kk_std_core__list xs1459, kk_context_t* _ctx) { /* (xs1459 : list<list<sslice>>) -> list<string> */ 
  if (kk_std_core__is_Nil(xs1459, _ctx)) {
    return kk_std_core__new_Nil(_ctx);
  }
  {
    return kk_std_text_regex__lift1483_split(xs1459, _ctx);
  }
}
 
// Split a string `s` in at most `atmost` parts using a regular expression `r` as separator.

kk_std_core__list kk_std_text_regex_split(kk_string_t s, kk_std_text_regex__regex r, kk_std_core_types__optional atmost, kk_context_t* _ctx) { /* (s : string, r : regex, atmost : optional<int>) -> list<string> */ 
  kk_std_core__list xs_1033;
  kk_std_core__list _x2215;
  kk_box_t _x2216;
  {
    kk_box_t _x = r.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(r, _ctx);
    _x2216 = _x; /*any*/
  }
  kk_ssize_t _x2217 = (KK_IZ(0)); /*ssize_t*/
  kk_ssize_t _x2218;
  kk_integer_t _x2219;
  if (kk_std_core_types__is_Optional(atmost, _ctx)) {
    kk_box_t _box_x1844 = atmost._cons.Optional.value;
    kk_integer_t _atmost_938 = kk_integer_unbox(_box_x1844, _ctx);
    kk_integer_dup(_atmost_938, _ctx);
    kk_std_core_types__optional_drop(atmost, _ctx);
    _x2219 = _atmost_938; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(atmost, _ctx);
    _x2219 = kk_integer_from_small(-1); /*int*/
  }
  _x2218 = kk_std_core_ssize__t(_x2219, _ctx); /*ssize_t*/
  _x2215 = kk_std_text_regex_regex_exec_all(_x2216, s, _x2217, _x2218, _ctx); /*list<list<sslice>>*/
  xs_1033 = kk_std_text_regex_filter_non_matches(_x2215, _ctx); /*list<list<sslice>>*/
  return kk_std_text_regex__lift1482_split(xs_1033, _ctx);
}

bool kk_std_text_regex_testabc(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> bool */ 
  kk_std_text_regex__regex r_1039;
  kk_box_t _x2220;
  kk_string_t _x2221;
  kk_define_string_literal(, _s2222, 6, "[ab]+c", _ctx)
  _x2221 = kk_string_dup(_s2222, _ctx); /*string*/
  bool _x2223;
  kk_std_core_types__optional _match_1849 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_1849, _ctx)) {
    kk_box_t _box_x1845 = _match_1849._cons.Optional.value;
    bool _ignorecase_455 = kk_bool_unbox(_box_x1845);
    kk_std_core_types__optional_drop(_match_1849, _ctx);
    _x2223 = _ignorecase_455; /*bool*/
  }
  else {
    kk_std_core_types__optional_drop(_match_1849, _ctx);
    _x2223 = false; /*bool*/
  }
  bool _x2224;
  kk_std_core_types__optional _match_1848 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_1848, _ctx)) {
    kk_box_t _box_x1846 = _match_1848._cons.Optional.value;
    bool _multiline_459 = kk_bool_unbox(_box_x1846);
    kk_std_core_types__optional_drop(_match_1848, _ctx);
    _x2224 = _multiline_459; /*bool*/
  }
  else {
    kk_std_core_types__optional_drop(_match_1848, _ctx);
    _x2224 = false; /*bool*/
  }
  _x2220 = kk_std_text_regex_regex_create(_x2221, _x2223, _x2224, _ctx); /*any*/
  kk_string_t _x2225;
  kk_define_string_literal(, _s2226, 6, "[ab]+c", _ctx)
  _x2225 = kk_string_dup(_s2226, _ctx); /*string*/
  r_1039 = kk_std_text_regex__new_Regex(_x2220, _x2225, _ctx); /*std/text/regex/regex*/
  kk_std_core__list list0_1043;
  kk_box_t _x2227;
  {
    kk_box_t _x = r_1039.obj;
    kk_box_dup(_x, _ctx);
    kk_std_text_regex__regex_drop(r_1039, _ctx);
    _x2227 = _x; /*any*/
  }
  kk_ssize_t _x2228 = (KK_IZ(0)); /*ssize_t*/
  list0_1043 = kk_std_text_regex_regex_exec(_x2227, s, _x2228, _ctx); /*list<sslice>*/
  if (kk_std_core__is_Cons(list0_1043, _ctx)) {
    struct kk_std_core_Cons* _con2229 = kk_std_core__as_Cons(list0_1043, _ctx);
    kk_box_t _box_x1847 = _con2229->head;
    kk_std_core__list _pat1 = _con2229->tail;
    if kk_likely(kk_datatype_ptr_is_unique(list0_1043, _ctx)) {
      kk_std_core__list_drop(_pat1, _ctx);
      kk_box_drop(_box_x1847, _ctx);
      kk_datatype_ptr_free(list0_1043, _ctx);
    }
    else {
      kk_datatype_ptr_decref(list0_1043, _ctx);
    }
    return true;
  }
  {
    return false;
  }
}

// initialization
void kk_std_text_regex__init(kk_context_t* _ctx){
  static bool _kk_initialized = false;
  if (_kk_initialized) return;
  _kk_initialized = true;
  kk_std_core_types__init(_ctx);
  kk_std_core_hnd__init(_ctx);
  kk_std_core__init(_ctx);
  #if defined(KK_CUSTOM_INIT)
    KK_CUSTOM_INIT (_ctx);
  #endif
  {
    kk_box_t _x2100;
    kk_string_t _x2101;
    kk_define_string_literal(, _s2102, 45, "[^\\\\\\[(]+|\\\\[\\s\\S]\?|\\(\\\?|\\[(\?:[^\\\\\\]]|\\\\.)*\\]", _ctx)
    _x2101 = kk_string_dup(_s2102, _ctx); /*string*/
    bool _x2103;
    kk_std_core_types__optional _match_1858 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_1858, _ctx)) {
      kk_box_t _box_x1743 = _match_1858._cons.Optional.value;
      bool _ignorecase_455 = kk_bool_unbox(_box_x1743);
      kk_std_core_types__optional_drop(_match_1858, _ctx);
      _x2103 = _ignorecase_455; /*bool*/
    }
    else {
      kk_std_core_types__optional_drop(_match_1858, _ctx);
      _x2103 = false; /*bool*/
    }
    bool _x2104;
    kk_std_core_types__optional _match_1857 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_1857, _ctx)) {
      kk_box_t _box_x1744 = _match_1857._cons.Optional.value;
      bool _multiline_459 = kk_bool_unbox(_box_x1744);
      kk_std_core_types__optional_drop(_match_1857, _ctx);
      _x2104 = _multiline_459; /*bool*/
    }
    else {
      kk_std_core_types__optional_drop(_match_1857, _ctx);
      _x2104 = false; /*bool*/
    }
    _x2100 = kk_std_text_regex_regex_create(_x2101, _x2103, _x2104, _ctx); /*any*/
    kk_string_t _x2105;
    kk_define_string_literal(, _s2106, 45, "[^\\\\\\[(]+|\\\\[\\s\\S]\?|\\(\\\?|\\[(\?:[^\\\\\\]]|\\\\.)*\\]", _ctx)
    _x2105 = kk_string_dup(_s2106, _ctx); /*string*/
    kk_std_text_regex_rx_nongroup = kk_std_text_regex__new_Regex(_x2100, _x2105, _ctx); /*std/text/regex/regex*/
  }
}

// termination
void kk_std_text_regex__done(kk_context_t* _ctx){
  static bool _kk_done = false;
  if (_kk_done) return;
  _kk_done = true;
  #if defined(KK_CUSTOM_DONE)
    KK_CUSTOM_DONE (_ctx);
  #endif
  kk_std_text_regex__regex_drop(kk_std_text_regex_rx_nongroup, _ctx);
  kk_std_core__done(_ctx);
  kk_std_core_hnd__done(_ctx);
  kk_std_core_types__done(_ctx);
}
