// Koka generated module: "std/time/calendar", koka version: 2.4.2, platform: 64-bit
#include "std_time_calendar.h"
/*---------------------------------------------------------------------------
  Copyright 2020-2021, Microsoft Research, Daan Leijen.

  This is free software; you can redistribute it and/or modify it under the
  terms of the Apache License, Version 2.0. A copy of the License can be
  found in the LICENSE file at the root of this distribution.
---------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------------------------------
  Local time zone UTC delta
--------------------------------------------------------------------------------------------------*/

#if !defined(__STDC_WANT_LIB_EXT1__)
#define __STDC_WANT_LIB_EXT1__    // for gmtime_s
#endif
#if defined(__GLIBC__) && !defined(_GNU_SOURCE)
#define _GNU_SOURCE               // for tm_zone and tm_gmtoff
#endif
#include <time.h>

static long kk_local_utc_delta(double unix_secs, kk_string_t* ptzname, kk_context_t* ctx) {
  // get the UTC delta in a somewhat portable way...
  bool isdst = false;
  time_t t = (time_t)unix_secs;
  #if (_WIN32 && KK_INTPTR_SIZE==8)
  if (t < 0) { t += 3155673600; } // try to avoid errors for negative times on some platforms by adding 100 years..
  #endif
  time_t loct = t;    
  #if defined(_GNU_SOURCE)
    // GNU libc has the tm_zone and tm_gmtoff fields
    struct tm loctm;
    if (localtime_r(&t, &loctm) != NULL) {
      isdst = (loctm.tm_isdst != 0);
      loct  = t - loctm.tm_gmtoff + (isdst ? 3600 : 0);
    }
    else {
      loctm.tm_zone = "";
    }
  #elif defined(_WIN32) && !defined(__MINGW32__)
    struct tm gmtm;
    if (gmtime_s(&gmtm, &t) == 0) {      // switched parameters :-(      
      loct = mktime(&gmtm);              // interpret gmt as local time
      struct tm loctm;
      localtime_s(&loctm, &t);           // switched parameters :-(
      isdst = (loctm.tm_isdst != 0);
    }
  #elif defined(__STDC_LIB_EXT1__)
    struct tm gmtm;
    if (gmtime_s(&t, &gmtm) != NULL) {
      loct = mktime(&gmtm);              // interpret gmt as local time
      struct tm loctm;
      localtime_s(&t, &loctm);
      isdst = (loctm.tm_isdst != 0);
    }
  #else
    struct tm* pgmtm = gmtime(&t);
    if (pgmtm != NULL) {
      loct = mktime(pgmtm);                // interpret gmt as local time
      struct tm* ploctm = localtime(&t);
      isdst = (ploctm->tm_isdst != 0);
    }
  #endif
  const time_t utc_delta = t - loct + (isdst ? 3600 : 0);   // the difference is the utc offset at that time
  if (ptzname != NULL) {
    // getting the timezone name
    #if defined(_WIN32) && !defined(__MINGW32__)
      char tzonename[256];
      size_t tznamelen;
      _get_tzname(&tznamelen, tzonename, 255, isdst ? 1 : 0); tzonename[255] = 0;
      *ptzname = kk_string_alloc_from_qutf8(tzonename, ctx);
    #elif defined(_GNU_SOURCE)
      *ptzname = kk_string_alloc_from_qutf8(loctm.tm_zone, ctx);
    #elif (_POSIX_C_SOURCE >= 1) || _XOPEN_SOURCE || _POSIX_SOURCE || __MINGW32__ // tzname
      *ptzname = kk_string_alloc_from_qutf8(tzname[isdst ? 1 : 0], ctx);
    #else
      // give up :-(
      * ptzname = kk_string_empty();
    #endif
  }
  return (long)utc_delta;
}


static kk_std_time_calendar__local_timezone kk_local_get_timezone(kk_context_t* ctx) {
  return kk_datatype_from_tag((kk_tag_t)1); // dummy value; we cannot store the local timezone as it is a global :-(
}

static kk_std_core_types__tuple2_ kk_local_get_utc_delta_tuple(kk_std_time_calendar__local_timezone tz, double unix_secs, kk_context_t* ctx) {
  kk_string_t tzonename;
  long utc_delta = kk_local_utc_delta(unix_secs, &tzonename, ctx);
  return kk_std_core_types__new_dash__lp__comma__rp_( kk_double_box((double)utc_delta,ctx), kk_string_box(tzonename), ctx );
}



// lift anonymous function
struct kk_std_time_calendar__copy_fun5490__t {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x4927;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar__copy_fun5490(kk_function_t _fself, kk_std_time_instant__instant _b_4930, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_copy_fun5490(kk_box_t _fun_unbox_x4927, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5490__t* _self = kk_function_alloc_as(struct kk_std_time_calendar__copy_fun5490__t, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar__copy_fun5490, kk_context());
  _self->_fun_unbox_x4927 = _fun_unbox_x4927;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_core_types__tuple2_ kk_std_time_calendar__copy_fun5490(kk_function_t _fself, kk_std_time_instant__instant _b_4930, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5490__t* _self = kk_function_as(struct kk_std_time_calendar__copy_fun5490__t*, _fself, _ctx);
  kk_box_t _fun_unbox_x4927 = _self->_fun_unbox_x4927; /* 116 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x4927, _ctx);}, {}, _ctx)
  kk_box_t _x5491;
  kk_function_t _x5492 = kk_function_unbox(_fun_unbox_x4927, _ctx); /*(4928) -> 4929*/
  _x5491 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x5492, (_x5492, kk_std_time_instant__instant_box(_b_4930, _ctx), _ctx), _ctx); /*4929*/
  return kk_std_core_types__tuple2__unbox(_x5491, KK_OWNED, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar__copy_fun5495__t {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x4934;
};
static kk_std_core_types__maybe kk_std_time_calendar__copy_fun5495(kk_function_t _fself, kk_std_time_instant__instant _b_4937, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_copy_fun5495(kk_box_t _fun_unbox_x4934, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5495__t* _self = kk_function_alloc_as(struct kk_std_time_calendar__copy_fun5495__t, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar__copy_fun5495, kk_context());
  _self->_fun_unbox_x4934 = _fun_unbox_x4934;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_core_types__maybe kk_std_time_calendar__copy_fun5495(kk_function_t _fself, kk_std_time_instant__instant _b_4937, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5495__t* _self = kk_function_as(struct kk_std_time_calendar__copy_fun5495__t*, _fself, _ctx);
  kk_box_t _fun_unbox_x4934 = _self->_fun_unbox_x4934; /* 116 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x4934, _ctx);}, {}, _ctx)
  kk_box_t _x5496;
  kk_function_t _x5497 = kk_function_unbox(_fun_unbox_x4934, _ctx); /*(4935) -> 4936*/
  _x5496 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x5497, (_x5497, kk_std_time_instant__instant_box(_b_4937, _ctx), _ctx), _ctx); /*4936*/
  return kk_std_core_types__maybe_unbox(_x5496, KK_OWNED, _ctx);
}

kk_std_time_calendar__timezone kk_std_time_calendar__copy(kk_std_time_calendar__timezone _this, kk_std_core_types__optional name0, kk_std_core_types__optional utc_delta0, kk_std_core_types__optional utc_inverse0, kk_context_t* _ctx) { /* (timezone, name : optional<string>, utc-delta : optional<(std/time/instant/instant) -> (std/time/duration/duration, string)>, utc-inverse : optional<(std/time/instant/instant) -> maybe<std/time/instant/instant>>) -> timezone */ 
  kk_string_t _x5487;
  if (kk_std_core_types__is_Optional(name0, _ctx)) {
    kk_box_t _box_x4923 = name0._cons.Optional.value;
    kk_string_t _name_217 = kk_string_unbox(_box_x4923);
    kk_string_dup(_name_217, _ctx);
    kk_std_core_types__optional_drop(name0, _ctx);
    _x5487 = _name_217; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(name0, _ctx);
    {
      struct kk_std_time_calendar_Timezone* _con5488 = kk_std_time_calendar__as_Timezone(_this, _ctx);
      kk_string_t _x = _con5488->name;
      kk_string_dup(_x, _ctx);
      _x5487 = _x; /*string*/
    }
  }
  kk_function_t _x5489;
  if (kk_std_core_types__is_Optional(utc_delta0, _ctx)) {
    kk_box_t _fun_unbox_x4927 = utc_delta0._cons.Optional.value;
    kk_box_dup(_fun_unbox_x4927, _ctx);
    kk_std_core_types__optional_drop(utc_delta0, _ctx);
    _x5489 = kk_std_time_calendar__new_copy_fun5490(_fun_unbox_x4927, _ctx); /*(std/time/instant/instant) -> (std/time/duration/duration, string)*/
  }
  else {
    kk_std_core_types__optional_drop(utc_delta0, _ctx);
    {
      struct kk_std_time_calendar_Timezone* _con5493 = kk_std_time_calendar__as_Timezone(_this, _ctx);
      kk_function_t _x0 = _con5493->utc_delta;
      kk_function_dup(_x0, _ctx);
      _x5489 = _x0; /*(std/time/instant/instant) -> (std/time/duration/duration, string)*/
    }
  }
  kk_function_t _x5494;
  if (kk_std_core_types__is_Optional(utc_inverse0, _ctx)) {
    kk_box_t _fun_unbox_x4934 = utc_inverse0._cons.Optional.value;
    kk_box_dup(_fun_unbox_x4934, _ctx);
    kk_std_core_types__optional_drop(utc_inverse0, _ctx);
    kk_datatype_ptr_dropn(_this, (KK_I32(3)), _ctx);
    _x5494 = kk_std_time_calendar__new_copy_fun5495(_fun_unbox_x4934, _ctx); /*(std/time/instant/instant) -> maybe<std/time/instant/instant>*/
  }
  else {
    kk_std_core_types__optional_drop(utc_inverse0, _ctx);
    {
      struct kk_std_time_calendar_Timezone* _con5498 = kk_std_time_calendar__as_Timezone(_this, _ctx);
      kk_string_t _pat02 = _con5498->name;
      kk_function_t _pat13 = _con5498->utc_delta;
      kk_function_t _x1 = _con5498->utc_inverse;
      if kk_likely(kk_datatype_ptr_is_unique(_this, _ctx)) {
        kk_function_drop(_pat13, _ctx);
        kk_string_drop(_pat02, _ctx);
        kk_datatype_ptr_free(_this, _ctx);
      }
      else {
        kk_function_dup(_x1, _ctx);
        kk_datatype_ptr_decref(_this, _ctx);
      }
      _x5494 = _x1; /*(std/time/instant/instant) -> maybe<std/time/instant/instant>*/
    }
  }
  return kk_std_time_calendar__new_Timezone(kk_reuse_null, 0, _x5487, _x5489, _x5494, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar__copy_fun5514__t_1 {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x4944;
};
static kk_string_t kk_std_time_calendar__copy_fun5514_1(kk_function_t _fself, kk_std_time_date__date _b_4947, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_copy_fun5514_1(kk_box_t _fun_unbox_x4944, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5514__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar__copy_fun5514__t_1, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar__copy_fun5514_1, kk_context());
  _self->_fun_unbox_x4944 = _fun_unbox_x4944;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_string_t kk_std_time_calendar__copy_fun5514_1(kk_function_t _fself, kk_std_time_date__date _b_4947, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5514__t_1* _self = kk_function_as(struct kk_std_time_calendar__copy_fun5514__t_1*, _fself, _ctx);
  kk_box_t _fun_unbox_x4944 = _self->_fun_unbox_x4944; /* 116 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x4944, _ctx);}, {}, _ctx)
  kk_box_t _x5515;
  kk_function_t _x5516 = kk_function_unbox(_fun_unbox_x4944, _ctx); /*(4945) -> 4946*/
  _x5515 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x5516, (_x5516, kk_std_time_date__date_box(_b_4947, _ctx), _ctx), _ctx); /*4946*/
  return kk_string_unbox(_x5515);
}


// lift anonymous function
struct kk_std_time_calendar__copy_fun5519__t_1 {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x4953;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar__copy_fun5519_1(kk_function_t _fself, kk_std_time_instant__instant _b_4957, kk_std_time_duration__duration _b_4958, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_copy_fun5519_1(kk_box_t _fun_unbox_x4953, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5519__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar__copy_fun5519__t_1, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar__copy_fun5519_1, kk_context());
  _self->_fun_unbox_x4953 = _fun_unbox_x4953;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_core_types__tuple2_ kk_std_time_calendar__copy_fun5519_1(kk_function_t _fself, kk_std_time_instant__instant _b_4957, kk_std_time_duration__duration _b_4958, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5519__t_1* _self = kk_function_as(struct kk_std_time_calendar__copy_fun5519__t_1*, _fself, _ctx);
  kk_box_t _fun_unbox_x4953 = _self->_fun_unbox_x4953; /* 116 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x4953, _ctx);}, {}, _ctx)
  kk_box_t _x5520;
  kk_function_t _x5521 = kk_function_unbox(_fun_unbox_x4953, _ctx); /*(i : 4954, tzdelta : 4955) -> 4956*/
  _x5520 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_box_t, kk_context_t*), _x5521, (_x5521, kk_std_time_instant__instant_box(_b_4957, _ctx), kk_std_time_duration__duration_box(_b_4958, _ctx), _ctx), _ctx); /*4956*/
  return kk_std_core_types__tuple2__unbox(_x5520, KK_OWNED, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar__copy_fun5524__t_1 {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x4968;
};
static kk_std_time_instant__instant kk_std_time_calendar__copy_fun5524_1(kk_function_t _fself, kk_std_time_date__date _b_4974, kk_std_time_date__clock _b_4975, kk_std_time_calendar__timezone _b_4976, kk_std_time_instant__timescale _b_4977, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_copy_fun5524_1(kk_box_t _fun_unbox_x4968, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5524__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar__copy_fun5524__t_1, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar__copy_fun5524_1, kk_context());
  _self->_fun_unbox_x4968 = _fun_unbox_x4968;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_time_instant__instant kk_std_time_calendar__copy_fun5524_1(kk_function_t _fself, kk_std_time_date__date _b_4974, kk_std_time_date__clock _b_4975, kk_std_time_calendar__timezone _b_4976, kk_std_time_instant__timescale _b_4977, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5524__t_1* _self = kk_function_as(struct kk_std_time_calendar__copy_fun5524__t_1*, _fself, _ctx);
  kk_box_t _fun_unbox_x4968 = _self->_fun_unbox_x4968; /* 116 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x4968, _ctx);}, {}, _ctx)
  kk_box_t _x5525;
  kk_function_t _x5526 = kk_function_unbox(_fun_unbox_x4968, _ctx); /*(4969, 4970, 4971, 4972) -> 4973*/
  _x5525 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_box_t, kk_box_t, kk_box_t, kk_context_t*), _x5526, (_x5526, kk_std_time_date__date_box(_b_4974, _ctx), kk_std_time_date__clock_box(_b_4975, _ctx), kk_std_time_calendar__timezone_box(_b_4976, _ctx), kk_std_time_instant__timescale_box(_b_4977, _ctx), _ctx), _ctx); /*4973*/
  return kk_std_time_instant__instant_unbox(_x5525, KK_OWNED, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar__copy_fun5529__t_1 {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x4981;
};
static kk_std_time_date__date kk_std_time_calendar__copy_fun5529_1(kk_function_t _fself, kk_integer_t _b_4984, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_copy_fun5529_1(kk_box_t _fun_unbox_x4981, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5529__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar__copy_fun5529__t_1, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar__copy_fun5529_1, kk_context());
  _self->_fun_unbox_x4981 = _fun_unbox_x4981;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_time_date__date kk_std_time_calendar__copy_fun5529_1(kk_function_t _fself, kk_integer_t _b_4984, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5529__t_1* _self = kk_function_as(struct kk_std_time_calendar__copy_fun5529__t_1*, _fself, _ctx);
  kk_box_t _fun_unbox_x4981 = _self->_fun_unbox_x4981; /* 116 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x4981, _ctx);}, {}, _ctx)
  kk_box_t _x5530;
  kk_function_t _x5531 = kk_function_unbox(_fun_unbox_x4981, _ctx); /*(days : 4982) -> 4983*/
  _x5530 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x5531, (_x5531, kk_integer_box(_b_4984, _ctx), _ctx), _ctx); /*4983*/
  return kk_std_time_date__date_unbox(_x5530, KK_OWNED, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar__copy_fun5534__t_1 {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x4988;
};
static kk_integer_t kk_std_time_calendar__copy_fun5534_1(kk_function_t _fself, kk_std_time_date__date _b_4991, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_copy_fun5534_1(kk_box_t _fun_unbox_x4988, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5534__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar__copy_fun5534__t_1, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar__copy_fun5534_1, kk_context());
  _self->_fun_unbox_x4988 = _fun_unbox_x4988;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_integer_t kk_std_time_calendar__copy_fun5534_1(kk_function_t _fself, kk_std_time_date__date _b_4991, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5534__t_1* _self = kk_function_as(struct kk_std_time_calendar__copy_fun5534__t_1*, _fself, _ctx);
  kk_box_t _fun_unbox_x4988 = _self->_fun_unbox_x4988; /* 116 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x4988, _ctx);}, {}, _ctx)
  kk_box_t _x5535;
  kk_function_t _x5536 = kk_function_unbox(_fun_unbox_x4988, _ctx); /*(date : 4989) -> 4990*/
  _x5535 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x5536, (_x5536, kk_std_time_date__date_box(_b_4991, _ctx), _ctx), _ctx); /*4990*/
  return kk_integer_unbox(_x5535, _ctx);
}

kk_std_time_calendar__calendar kk_std_time_calendar__copy_1(kk_std_time_calendar__calendar _this, kk_std_core_types__optional name0, kk_std_core_types__optional long_name0, kk_std_core_types__optional month_prefix0, kk_std_core_types__optional show_era0, kk_std_core_types__optional instant_to_dc0, kk_std_core_types__optional dc_to_instant0, kk_std_core_types__optional days_to_date0, kk_std_core_types__optional date_to_days0, kk_context_t* _ctx) { /* (calendar, name : optional<string>, long-name : optional<string>, month-prefix : optional<string>, show-era : optional<(std/time/date/date) -> string>, instant-to-dc : optional<(i : std/time/instant/instant, tzdelta : std/time/duration/duration) -> (std/time/date/date, std/time/date/clock)>, dc-to-instant : optional<(std/time/date/date, std/time/date/clock, timezone, std/time/instant/timescale) -> std/time/instant/instant>, days-to-date : optional<(days : int) -> std/time/date/date>, date-to-days : optional<(date : std/time/date/date) -> int>) -> calendar */ 
  kk_string_t _x5507;
  if (kk_std_core_types__is_Optional(name0, _ctx)) {
    kk_box_t _box_x4938 = name0._cons.Optional.value;
    kk_string_t _name_314 = kk_string_unbox(_box_x4938);
    kk_string_dup(_name_314, _ctx);
    kk_std_core_types__optional_drop(name0, _ctx);
    _x5507 = _name_314; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(name0, _ctx);
    {
      struct kk_std_time_calendar_Calendar* _con5508 = kk_std_time_calendar__as_Calendar(_this, _ctx);
      kk_string_t _x = _con5508->name;
      kk_string_dup(_x, _ctx);
      _x5507 = _x; /*string*/
    }
  }
  kk_string_t _x5509;
  if (kk_std_core_types__is_Optional(long_name0, _ctx)) {
    kk_box_t _box_x4939 = long_name0._cons.Optional.value;
    kk_string_t _long_name_320 = kk_string_unbox(_box_x4939);
    kk_string_dup(_long_name_320, _ctx);
    kk_std_core_types__optional_drop(long_name0, _ctx);
    _x5509 = _long_name_320; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(long_name0, _ctx);
    {
      struct kk_std_time_calendar_Calendar* _con5510 = kk_std_time_calendar__as_Calendar(_this, _ctx);
      kk_string_t _x0 = _con5510->long_name;
      kk_string_dup(_x0, _ctx);
      _x5509 = _x0; /*string*/
    }
  }
  kk_string_t _x5511;
  if (kk_std_core_types__is_Optional(month_prefix0, _ctx)) {
    kk_box_t _box_x4940 = month_prefix0._cons.Optional.value;
    kk_string_t _month_prefix_326 = kk_string_unbox(_box_x4940);
    kk_string_dup(_month_prefix_326, _ctx);
    kk_std_core_types__optional_drop(month_prefix0, _ctx);
    _x5511 = _month_prefix_326; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(month_prefix0, _ctx);
    {
      struct kk_std_time_calendar_Calendar* _con5512 = kk_std_time_calendar__as_Calendar(_this, _ctx);
      kk_string_t _x1 = _con5512->month_prefix;
      kk_string_dup(_x1, _ctx);
      _x5511 = _x1; /*string*/
    }
  }
  kk_function_t _x5513;
  if (kk_std_core_types__is_Optional(show_era0, _ctx)) {
    kk_box_t _fun_unbox_x4944 = show_era0._cons.Optional.value;
    kk_box_dup(_fun_unbox_x4944, _ctx);
    kk_std_core_types__optional_drop(show_era0, _ctx);
    _x5513 = kk_std_time_calendar__new_copy_fun5514_1(_fun_unbox_x4944, _ctx); /*(std/time/date/date) -> string*/
  }
  else {
    kk_std_core_types__optional_drop(show_era0, _ctx);
    {
      struct kk_std_time_calendar_Calendar* _con5517 = kk_std_time_calendar__as_Calendar(_this, _ctx);
      kk_function_t _x2 = _con5517->show_era;
      kk_function_dup(_x2, _ctx);
      _x5513 = _x2; /*(std/time/date/date) -> string*/
    }
  }
  kk_function_t _x5518;
  if (kk_std_core_types__is_Optional(instant_to_dc0, _ctx)) {
    kk_box_t _fun_unbox_x4953 = instant_to_dc0._cons.Optional.value;
    kk_box_dup(_fun_unbox_x4953, _ctx);
    kk_std_core_types__optional_drop(instant_to_dc0, _ctx);
    _x5518 = kk_std_time_calendar__new_copy_fun5519_1(_fun_unbox_x4953, _ctx); /*(std/time/instant/instant, std/time/duration/duration) -> (std/time/date/date, std/time/date/clock)*/
  }
  else {
    kk_std_core_types__optional_drop(instant_to_dc0, _ctx);
    {
      struct kk_std_time_calendar_Calendar* _con5522 = kk_std_time_calendar__as_Calendar(_this, _ctx);
      kk_function_t _x3 = _con5522->instant_to_dc;
      kk_function_dup(_x3, _ctx);
      _x5518 = _x3; /*(std/time/instant/instant, std/time/duration/duration) -> (std/time/date/date, std/time/date/clock)*/
    }
  }
  kk_function_t _x5523;
  if (kk_std_core_types__is_Optional(dc_to_instant0, _ctx)) {
    kk_box_t _fun_unbox_x4968 = dc_to_instant0._cons.Optional.value;
    kk_box_dup(_fun_unbox_x4968, _ctx);
    kk_std_core_types__optional_drop(dc_to_instant0, _ctx);
    _x5523 = kk_std_time_calendar__new_copy_fun5524_1(_fun_unbox_x4968, _ctx); /*(std/time/date/date, std/time/date/clock, std/time/calendar/timezone, std/time/instant/timescale) -> std/time/instant/instant*/
  }
  else {
    kk_std_core_types__optional_drop(dc_to_instant0, _ctx);
    {
      struct kk_std_time_calendar_Calendar* _con5527 = kk_std_time_calendar__as_Calendar(_this, _ctx);
      kk_function_t _x4 = _con5527->dc_to_instant;
      kk_function_dup(_x4, _ctx);
      _x5523 = _x4; /*(std/time/date/date, std/time/date/clock, std/time/calendar/timezone, std/time/instant/timescale) -> std/time/instant/instant*/
    }
  }
  kk_function_t _x5528;
  if (kk_std_core_types__is_Optional(days_to_date0, _ctx)) {
    kk_box_t _fun_unbox_x4981 = days_to_date0._cons.Optional.value;
    kk_box_dup(_fun_unbox_x4981, _ctx);
    kk_std_core_types__optional_drop(days_to_date0, _ctx);
    _x5528 = kk_std_time_calendar__new_copy_fun5529_1(_fun_unbox_x4981, _ctx); /*(int) -> std/time/date/date*/
  }
  else {
    kk_std_core_types__optional_drop(days_to_date0, _ctx);
    {
      struct kk_std_time_calendar_Calendar* _con5532 = kk_std_time_calendar__as_Calendar(_this, _ctx);
      kk_function_t _x5 = _con5532->days_to_date;
      kk_function_dup(_x5, _ctx);
      _x5528 = _x5; /*(int) -> std/time/date/date*/
    }
  }
  kk_function_t _x5533;
  if (kk_std_core_types__is_Optional(date_to_days0, _ctx)) {
    kk_box_t _fun_unbox_x4988 = date_to_days0._cons.Optional.value;
    kk_box_dup(_fun_unbox_x4988, _ctx);
    kk_std_core_types__optional_drop(date_to_days0, _ctx);
    kk_datatype_ptr_dropn(_this, (KK_I32(8)), _ctx);
    _x5533 = kk_std_time_calendar__new_copy_fun5534_1(_fun_unbox_x4988, _ctx); /*(std/time/date/date) -> int*/
  }
  else {
    kk_std_core_types__optional_drop(date_to_days0, _ctx);
    {
      struct kk_std_time_calendar_Calendar* _con5537 = kk_std_time_calendar__as_Calendar(_this, _ctx);
      kk_string_t _pat07 = _con5537->name;
      kk_string_t _pat112 = _con5537->long_name;
      kk_string_t _pat28 = _con5537->month_prefix;
      kk_function_t _pat37 = _con5537->show_era;
      kk_function_t _pat47 = _con5537->instant_to_dc;
      kk_function_t _pat57 = _con5537->dc_to_instant;
      kk_function_t _pat67 = _con5537->days_to_date;
      kk_function_t _x6 = _con5537->date_to_days;
      if kk_likely(kk_datatype_ptr_is_unique(_this, _ctx)) {
        kk_function_drop(_pat67, _ctx);
        kk_function_drop(_pat57, _ctx);
        kk_function_drop(_pat47, _ctx);
        kk_function_drop(_pat37, _ctx);
        kk_string_drop(_pat28, _ctx);
        kk_string_drop(_pat112, _ctx);
        kk_string_drop(_pat07, _ctx);
        kk_datatype_ptr_free(_this, _ctx);
      }
      else {
        kk_function_dup(_x6, _ctx);
        kk_datatype_ptr_decref(_this, _ctx);
      }
      _x5533 = _x6; /*(std/time/date/date) -> int*/
    }
  }
  return kk_std_time_calendar__new_Calendar(kk_reuse_null, 0, _x5507, _x5509, _x5511, _x5513, _x5518, _x5523, _x5528, _x5533, _ctx);
}

kk_integer_t kk_std_time_calendar_iso_days_before_year(kk_integer_t year, kk_context_t* _ctx) { /* (year : int) -> int */ 
  kk_integer_t y = kk_integer_add_small_const(year, -1, _ctx); /*int*/;
  kk_integer_t x0_4671;
  kk_integer_t _x5538 = kk_integer_dup(y, _ctx); /*int*/
  x0_4671 = kk_integer_div(_x5538,(kk_integer_from_small(4)),kk_context()); /*int*/
  kk_integer_t y1_4672;
  kk_integer_t _x5539 = kk_integer_dup(y, _ctx); /*int*/
  y1_4672 = kk_integer_div(_x5539,(kk_integer_from_small(100)),kk_context()); /*int*/
  kk_integer_t x_4669 = kk_integer_sub(x0_4671,y1_4672,kk_context()); /*int*/;
  kk_integer_t y0_4670;
  kk_integer_t _x5540 = kk_integer_dup(y, _ctx); /*int*/
  y0_4670 = kk_integer_div(_x5540,(kk_integer_from_small(400)),kk_context()); /*int*/
  kk_integer_t leapdays = kk_integer_add(x_4669,y0_4670,kk_context()); /*int*/;
  kk_integer_t x1_4673 = kk_integer_mul((kk_integer_from_small(365)),y,kk_context()); /*int*/;
  return kk_integer_add(x1_4673,leapdays,kk_context());
}

kk_std_core_types__tuple2_ kk_std_time_calendar_iso_estimate_year(kk_integer_t days, kk_context_t* _ctx) { /* (days : int) -> (int, maybe<int>) */ 
  kk_std_core_types__tuple2_ _match_5442 = kk_integer_div_mod_tuple(days,(kk_integer_from_int(146097, _ctx)),kk_context()); /*(int, int)*/;
  {
    kk_box_t _box_x4992 = _match_5442.fst;
    kk_box_t _box_x4993 = _match_5442.snd;
    kk_integer_t era = kk_integer_unbox(_box_x4992, _ctx);
    kk_integer_t yoe = kk_integer_unbox(_box_x4993, _ctx);
    kk_integer_dup(era, _ctx);
    kk_integer_dup(yoe, _ctx);
    kk_std_core_types__tuple2__drop(_match_5442, _ctx);
    kk_integer_t y0_4678 = kk_integer_mul((kk_integer_from_small(400)),era,kk_context()); /*int*/;
    kk_integer_t x_4675 = kk_integer_add_small_const(y0_4678, 1, _ctx); /*int*/;
    kk_integer_t y_4676;
    kk_integer_t _x5541 = kk_integer_mul((kk_integer_from_small(100)),yoe,kk_context()); /*int*/
    y_4676 = kk_integer_div(_x5541,(kk_integer_from_int(36525, _ctx)),kk_context()); /*int*/
    kk_integer_t _b_4997_4995 = kk_integer_add(x_4675,y_4676,kk_context()); /*int*/;
    kk_box_t _x5542;
    kk_std_core_types__maybe _x5543 = kk_std_core_types__new_Just(kk_integer_box(kk_integer_from_small(363), _ctx), _ctx); /*maybe<111>*/
    _x5542 = kk_std_core_types__maybe_box(_x5543, _ctx); /*7*/
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_4997_4995, _ctx), _x5542, _ctx);
  }
}

kk_std_time_timestamp__timestamp kk_std_time_calendar_dc_timestamp(kk_std_time_date__date d, kk_std_time_date__clock c, kk_function_t date_to_days0, kk_std_core_types__optional has_leap_seconds, kk_context_t* _ctx) { /* (d : std/time/date/date, c : std/time/date/clock, date-to-days : (std/time/date/date) -> int, has-leap-seconds : optional<bool>) -> std/time/timestamp/timestamp */ 
  kk_std_core_types__tuple2_ _match_5439;
  if (kk_std_core_types__is_Optional(has_leap_seconds, _ctx)) {
    kk_box_t _box_x5000 = has_leap_seconds._cons.Optional.value;
    bool _has_leap_seconds_652 = kk_bool_unbox(_box_x5000);
    kk_std_core_types__optional_drop(has_leap_seconds, _ctx);
    if (_has_leap_seconds_652) {
      kk_std_num_ddouble__ddouble y_4682 = kk_std_num_ddouble_ddouble_int_exp(kk_integer_from_small(60), kk_integer_from_small(0), _ctx); /*std/num/ddouble/ddouble*/;
      bool _match_5440;
      kk_std_core_types__order _x5544;
      kk_std_core_types__order _match_5441;
      double _x5545;
      {
        struct kk_std_time_date_Clock* _con5546 = kk_std_time_date__as_Clock(c, _ctx);
        kk_std_num_ddouble__ddouble _x3 = _con5546->seconds;
        {
          double _x0 = _x3.hi;
          _x5545 = _x0; /*float64*/
        }
      }
      double _x5547;
      {
        double _x00 = y_4682.hi;
        _x5547 = _x00; /*float64*/
      }
      _match_5441 = kk_std_num_float64_compare(_x5545, _x5547, _ctx); /*order*/
      if (kk_std_core_types__is_Eq(_match_5441, _ctx)) {
        double _x5548;
        {
          struct kk_std_time_date_Clock* _con5549 = kk_std_time_date__as_Clock(c, _ctx);
          kk_std_num_ddouble__ddouble _x30 = _con5549->seconds;
          {
            double _x1 = _x30.lo;
            _x5548 = _x1; /*float64*/
          }
        }
        double _x5550;
        {
          double _x2 = y_4682.lo;
          _x5550 = _x2; /*float64*/
        }
        _x5544 = kk_std_num_float64_compare(_x5548, _x5550, _ctx); /*order*/
      }
      else {
        _x5544 = _match_5441; /*order*/
      }
      _match_5440 = kk_std_core__lp__eq__eq__4_rp_(_x5544, kk_std_core_types__new_Lt(_ctx), _ctx); /*bool*/
      if (_match_5440) {
        kk_box_t _x5551;
        kk_std_num_ddouble__ddouble _x5552;
        {
          struct kk_std_time_date_Clock* _con5553 = kk_std_time_date__as_Clock(c, _ctx);
          kk_std_num_ddouble__ddouble _x4 = _con5553->seconds;
          _x5552 = _x4; /*std/num/ddouble/ddouble*/
        }
        _x5551 = kk_std_num_ddouble__ddouble_box(_x5552, _ctx); /*6*/
        _match_5439 = kk_std_core_types__new_dash__lp__comma__rp_(_x5551, kk_integer_box(kk_integer_from_small(0), _ctx), _ctx); /*(6, 7)*/
      }
      else {
        kk_integer_t x0_4686;
        kk_std_num_ddouble__ddouble _x5554;
        kk_std_num_ddouble__ddouble _x5555;
        {
          struct kk_std_time_date_Clock* _con5556 = kk_std_time_date__as_Clock(c, _ctx);
          kk_std_num_ddouble__ddouble _x5 = _con5556->seconds;
          _x5555 = _x5; /*std/num/ddouble/ddouble*/
        }
        _x5554 = kk_std_num_ddouble_floor(_x5555, _ctx); /*std/num/ddouble/ddouble*/
        x0_4686 = kk_std_num_ddouble_int(_x5554, kk_std_core_types__new_None(_ctx), _ctx); /*int*/
        kk_integer_t leap = kk_integer_add_small_const(x0_4686, -59, _ctx); /*int*/;
        kk_std_num_ddouble__ddouble y1_4690;
        kk_integer_t _x5557 = kk_integer_dup(leap, _ctx); /*int*/
        y1_4690 = kk_std_num_ddouble_ddouble_int_exp(_x5557, kk_integer_from_small(0), _ctx); /*std/num/ddouble/ddouble*/
        kk_std_num_ddouble__ddouble _b_5015_5003;
        kk_std_num_ddouble__ddouble _x5558;
        {
          struct kk_std_time_date_Clock* _con5559 = kk_std_time_date__as_Clock(c, _ctx);
          kk_std_num_ddouble__ddouble _x7 = _con5559->seconds;
          _x5558 = _x7; /*std/num/ddouble/ddouble*/
        }
        kk_std_num_ddouble__ddouble _x5560;
        double _x5561;
        double _x5562;
        {
          double _x6 = y1_4690.hi;
          _x5562 = _x6; /*float64*/
        }
        _x5561 = (-_x5562); /*float64*/
        double _x5563;
        double _x5564;
        {
          double _x01 = y1_4690.lo;
          _x5564 = _x01; /*float64*/
        }
        _x5563 = (-_x5564); /*float64*/
        _x5560 = kk_std_num_ddouble__new_Ddouble(_x5561, _x5563, _ctx); /*std/num/ddouble/ddouble*/
        _b_5015_5003 = kk_std_num_ddouble__lp__plus__rp_(_x5558, _x5560, _ctx); /*std/num/ddouble/ddouble*/
        _match_5439 = kk_std_core_types__new_dash__lp__comma__rp_(kk_std_num_ddouble__ddouble_box(_b_5015_5003, _ctx), kk_integer_box(leap, _ctx), _ctx); /*(6, 7)*/
      }
    }
    else {
      kk_box_t _x5565;
      kk_std_num_ddouble__ddouble _x5566;
      {
        struct kk_std_time_date_Clock* _con5567 = kk_std_time_date__as_Clock(c, _ctx);
        kk_std_num_ddouble__ddouble _x = _con5567->seconds;
        _x5566 = _x; /*std/num/ddouble/ddouble*/
      }
      _x5565 = kk_std_num_ddouble__ddouble_box(_x5566, _ctx); /*6*/
      _match_5439 = kk_std_core_types__new_dash__lp__comma__rp_(_x5565, kk_integer_box(kk_integer_from_small(0), _ctx), _ctx); /*(6, 7)*/
    }
  }
  else {
    kk_std_core_types__optional_drop(has_leap_seconds, _ctx);
    kk_box_t _x5568;
    kk_std_num_ddouble__ddouble _x5569;
    {
      struct kk_std_time_date_Clock* _con5570 = kk_std_time_date__as_Clock(c, _ctx);
      kk_std_num_ddouble__ddouble _x8 = _con5570->seconds;
      _x5569 = _x8; /*std/num/ddouble/ddouble*/
    }
    _x5568 = kk_std_num_ddouble__ddouble_box(_x5569, _ctx); /*6*/
    _match_5439 = kk_std_core_types__new_dash__lp__comma__rp_(_x5568, kk_integer_box(kk_integer_from_small(0), _ctx), _ctx); /*(6, 7)*/
  }
  {
    kk_box_t _box_x5009 = _match_5439.fst;
    kk_box_t _box_x5010 = _match_5439.snd;
    kk_std_num_ddouble__ddouble xsecs = kk_std_num_ddouble__ddouble_unbox(_box_x5009, KK_BORROWED, _ctx);
    kk_integer_t leap0 = kk_integer_unbox(_box_x5010, _ctx);
    kk_integer_dup(leap0, _ctx);
    kk_std_core_types__tuple2__drop(_match_5439, _ctx);
    kk_integer_t x2_4694;
    kk_integer_t _x5571;
    {
      struct kk_std_time_date_Clock* _con5572 = kk_std_time_date__as_Clock(c, _ctx);
      kk_std_num_ddouble__ddouble _pat19 = _con5572->seconds;
      kk_integer_t _x80 = _con5572->hours;
      kk_integer_dup(_x80, _ctx);
      _x5571 = _x80; /*int*/
    }
    x2_4694 = kk_integer_mul(_x5571,(kk_integer_from_small(60)),kk_context()); /*int*/
    kk_integer_t i1_4693;
    kk_integer_t _x5573;
    kk_integer_t _x5574;
    {
      struct kk_std_time_date_Clock* _con5575 = kk_std_time_date__as_Clock(c, _ctx);
      kk_std_num_ddouble__ddouble _pat1100 = _con5575->seconds;
      kk_integer_t _pat09 = _con5575->hours;
      kk_integer_t _x9 = _con5575->minutes;
      if kk_likely(kk_datatype_ptr_is_unique(c, _ctx)) {
        kk_integer_drop(_pat09, _ctx);
        kk_datatype_ptr_free(c, _ctx);
      }
      else {
        kk_integer_dup(_x9, _ctx);
        kk_datatype_ptr_decref(c, _ctx);
      }
      _x5574 = _x9; /*int*/
    }
    _x5573 = kk_integer_add(x2_4694,_x5574,kk_context()); /*int*/
    i1_4693 = kk_integer_mul(_x5573,(kk_integer_from_small(60)),kk_context()); /*int*/
    kk_std_num_ddouble__ddouble secs;
    kk_std_num_ddouble__ddouble _x5576 = kk_std_num_ddouble_ddouble_int_exp(i1_4693, kk_integer_from_small(0), _ctx); /*std/num/ddouble/ddouble*/
    secs = kk_std_num_ddouble__lp__plus__rp_(_x5576, xsecs, _ctx); /*std/num/ddouble/ddouble*/
    kk_integer_t days = kk_function_call(kk_integer_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), date_to_days0, (date_to_days0, d, _ctx), _ctx); /*int*/;
    kk_std_core_types__optional _x5577 = kk_std_core_types__new_Optional(kk_std_num_ddouble__ddouble_box(secs, _ctx), _ctx); /*optional<116>*/
    kk_std_core_types__optional _x5578 = kk_std_core_types__new_Optional(kk_integer_box(leap0, _ctx), _ctx); /*optional<116>*/
    return kk_std_time_timestamp_timestamp_days(days, _x5577, _x5578, _ctx);
  }
}

kk_std_core_types__tuple2_ kk_std_time_calendar_earth_timestamp_to_dc(kk_std_time_timestamp__timestamp t, kk_std_num_ddouble__ddouble tzdelta, kk_function_t days_to_date0, kk_context_t* _ctx) { /* (t : std/time/timestamp/timestamp, tzdelta : std/time/timestamp/timespan, days-to-date : (int) -> std/time/date/date) -> (std/time/date/date, std/time/date/clock) */ 
  kk_std_time_timestamp__timestamp ts_4698;
  kk_std_num_ddouble__ddouble _x5579;
  kk_std_num_ddouble__ddouble _x5580;
  {
    struct kk_std_time_timestamp_Timestamp* _con5581 = kk_std_time_timestamp__as_Timestamp(t, _ctx);
    kk_std_num_ddouble__ddouble _x0 = _con5581->since;
    _x5580 = _x0; /*std/time/timestamp/timespan*/
  }
  _x5579 = kk_std_num_ddouble__lp__plus__rp_(_x5580, tzdelta, _ctx); /*std/num/ddouble/ddouble*/
  int32_t _x5582;
  {
    struct kk_std_time_timestamp_Timestamp* _con5583 = kk_std_time_timestamp__as_Timestamp(t, _ctx);
    kk_std_num_ddouble__ddouble _pat000 = _con5583->since;
    int32_t _x00 = _con5583->leap32;
    if kk_likely(kk_datatype_ptr_is_unique(t, _ctx)) {
      kk_datatype_ptr_free(t, _ctx);
    }
    else {
      kk_datatype_ptr_decref(t, _ctx);
    }
    _x5582 = _x00; /*int32*/
  }
  ts_4698 = kk_std_time_timestamp__new_Timestamp(kk_reuse_null, 0, _x5579, _x5582, _ctx); /*std/time/timestamp/timestamp*/
  kk_std_core_types__tuple2_ _match_5437;
  kk_std_time_timestamp__timestamp _x5584 = kk_std_time_timestamp__timestamp_dup(ts_4698, _ctx); /*std/time/timestamp/timestamp*/
  _match_5437 = kk_std_time_timestamp_days_seconds(_x5584, _ctx); /*(int, std/num/ddouble/ddouble)*/
  {
    kk_box_t _box_x5023 = _match_5437.fst;
    kk_box_t _box_x5024 = _match_5437.snd;
    kk_std_num_ddouble__ddouble secs = kk_std_num_ddouble__ddouble_unbox(_box_x5024, KK_BORROWED, _ctx);
    kk_integer_t days0 = kk_integer_unbox(_box_x5023, _ctx);
    kk_integer_dup(days0, _ctx);
    kk_std_core_types__tuple2__drop(_match_5437, _ctx);
    kk_std_time_date__clock _match_5438;
    kk_integer_t _x5585;
    int32_t _x5586;
    {
      struct kk_std_time_timestamp_Timestamp* _con5587 = kk_std_time_timestamp__as_Timestamp(ts_4698, _ctx);
      kk_std_num_ddouble__ddouble _pat00 = _con5587->since;
      int32_t _x = _con5587->leap32;
      if kk_likely(kk_datatype_ptr_is_unique(ts_4698, _ctx)) {
        kk_datatype_ptr_free(ts_4698, _ctx);
      }
      else {
        kk_datatype_ptr_decref(ts_4698, _ctx);
      }
      _x5586 = _x; /*int32*/
    }
    _x5585 = kk_integer_from_int(_x5586,kk_context()); /*int*/
    _match_5438 = kk_std_time_date_clock_3(secs, _x5585, _ctx); /*std/time/date/clock*/
    {
      struct kk_std_time_date_Clock* _con5588 = kk_std_time_date__as_Clock(_match_5438, _ctx);
      kk_std_time_date__date _b_5027_5025 = kk_function_call(kk_std_time_date__date, (kk_function_t, kk_integer_t, kk_context_t*), days_to_date0, (days_to_date0, days0, _ctx), _ctx); /*std/time/date/date*/;
      return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_date__date_box(_b_5027_5025, _ctx), kk_std_time_date__clock_box(_match_5438, _ctx), _ctx);
    }
  }
}
 
// Return the days between two dates interpreted by calendar `cal`.

kk_integer_t kk_std_time_calendar_days_until(kk_std_time_calendar__calendar cal, kk_std_time_date__date d1, kk_std_time_date__date d2, kk_context_t* _ctx) { /* (cal : calendar, d1 : std/time/date/date, d2 : std/time/date/date) -> int */ 
  kk_integer_t x;
  {
    struct kk_std_time_calendar_Calendar* _con5589 = kk_std_time_calendar__as_Calendar(cal, _ctx);
    kk_function_t _x0 = _con5589->date_to_days;
    kk_function_dup(_x0, _ctx);
    x = kk_function_call(kk_integer_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _x0, (_x0, d2, _ctx), _ctx); /*int*/
  }
  kk_integer_t y;
  {
    struct kk_std_time_calendar_Calendar* _con5590 = kk_std_time_calendar__as_Calendar(cal, _ctx);
    kk_string_t _pat0 = _con5590->name;
    kk_string_t _pat1 = _con5590->long_name;
    kk_string_t _pat2 = _con5590->month_prefix;
    kk_function_t _pat3 = _con5590->show_era;
    kk_function_t _pat4 = _con5590->instant_to_dc;
    kk_function_t _pat5 = _con5590->dc_to_instant;
    kk_function_t _pat6 = _con5590->days_to_date;
    kk_function_t _x = _con5590->date_to_days;
    if kk_likely(kk_datatype_ptr_is_unique(cal, _ctx)) {
      kk_function_drop(_pat6, _ctx);
      kk_function_drop(_pat5, _ctx);
      kk_function_drop(_pat4, _ctx);
      kk_function_drop(_pat3, _ctx);
      kk_string_drop(_pat2, _ctx);
      kk_string_drop(_pat1, _ctx);
      kk_string_drop(_pat0, _ctx);
      kk_datatype_ptr_free(cal, _ctx);
    }
    else {
      kk_function_dup(_x, _ctx);
      kk_datatype_ptr_decref(cal, _ctx);
    }
    y = kk_function_call(kk_integer_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _x, (_x, d1, _ctx), _ctx); /*int*/
  }
  return kk_integer_sub(x,y,kk_context());
}
 
// Get the current local timezone structure.
// The timezone name is allowed to be the empty string.

kk_std_time_calendar__local_timezone kk_std_time_calendar_local_get_timezone(kk_context_t* _ctx) { /* () -> ndet local-timezone */ 
  return kk_local_get_timezone(kk_context());
}
 
// Return the utc delta in fractional seconds given a local timezone structure
// and time `i` in fractional seconds since the UNIX epoch (1970-01-01).
// Also returns time zone abbreviation.

kk_std_core_types__tuple2_ kk_std_time_calendar_local_utc_delta(kk_std_time_calendar__local_timezone tz, double i, kk_context_t* _ctx) { /* (tz : local-timezone, i : float64) -> (float64, string) */ 
  return kk_local_get_utc_delta_tuple(tz,i,kk_context());
}
extern kk_std_core_types__maybe kk_std_time_calendar__create_Timezone_fun5592(kk_function_t _fself, kk_std_time_instant__instant _b_5035, kk_context_t* _ctx) {
  struct kk_std_time_calendar__create_Timezone_fun5592__t* _self = kk_function_as(struct kk_std_time_calendar__create_Timezone_fun5592__t*, _fself, _ctx);
  kk_box_t _fun_unbox_x5032 = _self->_fun_unbox_x5032; /* 116 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x5032, _ctx);}, {}, _ctx)
  kk_box_t _x5593;
  kk_function_t _x5594 = kk_function_unbox(_fun_unbox_x5032, _ctx); /*(5033) -> 5034*/
  _x5593 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x5594, (_x5594, kk_std_time_instant__instant_box(_b_5035, _ctx), _ctx), _ctx); /*5034*/
  return kk_std_core_types__maybe_unbox(_x5593, KK_OWNED, _ctx);
}
extern kk_std_core_types__maybe kk_std_time_calendar__create_Timezone_fun5595(kk_function_t _fself, kk_std_time_instant__instant i, kk_context_t* _ctx) {
  kk_unused(_fself);
  kk_std_time_instant__instant_drop(i, _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}
 
// Check if two calendars use the same date calculations. (Display of era names etc. may differ)

bool kk_std_time_calendar__lp__eq__eq__rp_(kk_std_time_calendar__calendar c1, kk_std_time_calendar__calendar c2, kk_context_t* _ctx) { /* (c1 : calendar, c2 : calendar) -> bool */ 
  kk_string_t _x5596;
  {
    struct kk_std_time_calendar_Calendar* _con5597 = kk_std_time_calendar__as_Calendar(c1, _ctx);
    kk_string_t _pat0 = _con5597->name;
    kk_string_t _x = _con5597->long_name;
    kk_string_t _pat1 = _con5597->month_prefix;
    kk_function_t _pat2 = _con5597->show_era;
    kk_function_t _pat3 = _con5597->instant_to_dc;
    kk_function_t _pat4 = _con5597->dc_to_instant;
    kk_function_t _pat5 = _con5597->days_to_date;
    kk_function_t _pat6 = _con5597->date_to_days;
    if kk_likely(kk_datatype_ptr_is_unique(c1, _ctx)) {
      kk_function_drop(_pat6, _ctx);
      kk_function_drop(_pat5, _ctx);
      kk_function_drop(_pat4, _ctx);
      kk_function_drop(_pat3, _ctx);
      kk_function_drop(_pat2, _ctx);
      kk_string_drop(_pat1, _ctx);
      kk_string_drop(_pat0, _ctx);
      kk_datatype_ptr_free(c1, _ctx);
    }
    else {
      kk_string_dup(_x, _ctx);
      kk_datatype_ptr_decref(c1, _ctx);
    }
    _x5596 = _x; /*string*/
  }
  kk_string_t _x5598;
  {
    struct kk_std_time_calendar_Calendar* _con5599 = kk_std_time_calendar__as_Calendar(c2, _ctx);
    kk_string_t _pat00 = _con5599->name;
    kk_string_t _x0 = _con5599->long_name;
    kk_string_t _pat10 = _con5599->month_prefix;
    kk_function_t _pat20 = _con5599->show_era;
    kk_function_t _pat30 = _con5599->instant_to_dc;
    kk_function_t _pat40 = _con5599->dc_to_instant;
    kk_function_t _pat50 = _con5599->days_to_date;
    kk_function_t _pat60 = _con5599->date_to_days;
    if kk_likely(kk_datatype_ptr_is_unique(c2, _ctx)) {
      kk_function_drop(_pat60, _ctx);
      kk_function_drop(_pat50, _ctx);
      kk_function_drop(_pat40, _ctx);
      kk_function_drop(_pat30, _ctx);
      kk_function_drop(_pat20, _ctx);
      kk_string_drop(_pat10, _ctx);
      kk_string_drop(_pat00, _ctx);
      kk_datatype_ptr_free(c2, _ctx);
    }
    else {
      kk_string_dup(_x0, _ctx);
      kk_datatype_ptr_decref(c2, _ctx);
    }
    _x5598 = _x0; /*string*/
  }
  return kk_string_is_eq(_x5596,_x5598,kk_context());
}
 
// Same timezone?

bool kk_std_time_calendar__lp__eq__eq__1_rp_(kk_std_time_calendar__timezone tz1, kk_std_time_calendar__timezone tz2, kk_context_t* _ctx) { /* (tz1 : timezone, tz2 : timezone) -> bool */ 
  kk_string_t _x5600;
  {
    struct kk_std_time_calendar_Timezone* _con5601 = kk_std_time_calendar__as_Timezone(tz1, _ctx);
    kk_string_t _x = _con5601->name;
    kk_function_t _pat0 = _con5601->utc_delta;
    kk_function_t _pat1 = _con5601->utc_inverse;
    if kk_likely(kk_datatype_ptr_is_unique(tz1, _ctx)) {
      kk_function_drop(_pat1, _ctx);
      kk_function_drop(_pat0, _ctx);
      kk_datatype_ptr_free(tz1, _ctx);
    }
    else {
      kk_string_dup(_x, _ctx);
      kk_datatype_ptr_decref(tz1, _ctx);
    }
    _x5600 = _x; /*string*/
  }
  kk_string_t _x5602;
  {
    struct kk_std_time_calendar_Timezone* _con5603 = kk_std_time_calendar__as_Timezone(tz2, _ctx);
    kk_string_t _x0 = _con5603->name;
    kk_function_t _pat00 = _con5603->utc_delta;
    kk_function_t _pat10 = _con5603->utc_inverse;
    if kk_likely(kk_datatype_ptr_is_unique(tz2, _ctx)) {
      kk_function_drop(_pat10, _ctx);
      kk_function_drop(_pat00, _ctx);
      kk_datatype_ptr_free(tz2, _ctx);
    }
    else {
      kk_string_dup(_x0, _ctx);
      kk_datatype_ptr_decref(tz2, _ctx);
    }
    _x5602 = _x0; /*string*/
  }
  return kk_string_is_eq(_x5600,_x5602,kk_context());
}

bool kk_std_time_calendar_iso_is_leap(kk_integer_t year, kk_context_t* _ctx) { /* (year : int) -> bool */ 
  bool _match_5429;
  kk_integer_t _brw_5435;
  kk_integer_t _x5604 = kk_integer_dup(year, _ctx); /*int*/
  _brw_5435 = kk_integer_mod(_x5604,(kk_integer_from_small(4)),kk_context()); /*int*/
  bool _brw_5436 = kk_integer_eq_borrow(_brw_5435,(kk_integer_from_small(0)),kk_context()); /*bool*/;
  kk_integer_drop(_brw_5435, _ctx);
  _match_5429 = _brw_5436; /*bool*/
  if (_match_5429) {
    bool _match_5430;
    kk_integer_t _brw_5433;
    kk_integer_t _x5605 = kk_integer_dup(year, _ctx); /*int*/
    _brw_5433 = kk_integer_mod(_x5605,(kk_integer_from_small(100)),kk_context()); /*int*/
    bool _brw_5434 = kk_integer_neq_borrow(_brw_5433,(kk_integer_from_small(0)),kk_context()); /*bool*/;
    kk_integer_drop(_brw_5433, _ctx);
    _match_5430 = _brw_5434; /*bool*/
    if (_match_5430) {
      kk_integer_drop(year, _ctx);
      return true;
    }
    {
      kk_integer_t _brw_5431 = kk_integer_mod(year,(kk_integer_from_small(400)),kk_context()); /*int*/;
      bool _brw_5432 = kk_integer_eq_borrow(_brw_5431,(kk_integer_from_small(0)),kk_context()); /*bool*/;
      kk_integer_drop(_brw_5431, _ctx);
      return _brw_5432;
    }
  }
  {
    kk_integer_drop(year, _ctx);
    return false;
  }
}

kk_integer_t kk_std_time_calendar_iso_days_before_month(kk_integer_t year, kk_integer_t month, kk_context_t* _ctx) { /* (year : int, month : int) -> int */ 
  bool is_before_march_4707 = kk_integer_lte_borrow(month,(kk_integer_from_small(2)),kk_context()); /*bool*/;
  kk_integer_t adj;
  if (is_before_march_4707) {
    kk_integer_drop(year, _ctx);
    adj = kk_integer_from_small(0); /*int*/
  }
  else {
    bool _match_5427 = kk_std_time_calendar_iso_is_leap(year, _ctx); /*bool*/;
    if (_match_5427) {
      adj = kk_integer_from_small(1); /*int*/
    }
    else {
      adj = kk_integer_from_small(2); /*int*/
    }
  }
  kk_integer_t x0_4711 = kk_integer_mul((kk_integer_from_small(367)),month,kk_context()); /*int*/;
  kk_integer_t x_4709;
  kk_integer_t _x5606 = kk_integer_add_small_const(x0_4711, -362, _ctx); /*int*/
  x_4709 = kk_integer_div(_x5606,(kk_integer_from_small(12)),kk_context()); /*int*/
  return kk_integer_sub(x_4709,adj,kk_context());
}

kk_integer_t kk_std_time_calendar_iso_doy_to_month(kk_integer_t year, kk_integer_t doy, kk_context_t* _ctx) { /* (year : int, doy : int) -> int */ 
  bool is_before_march_4713 = kk_integer_lte_borrow(doy,(kk_integer_from_small(58)),kk_context()); /*bool*/;
  kk_integer_t adj;
  if (is_before_march_4713) {
    kk_integer_drop(year, _ctx);
    adj = kk_integer_from_small(0); /*int*/
  }
  else {
    bool _match_5426 = kk_std_time_calendar_iso_is_leap(year, _ctx); /*bool*/;
    if (_match_5426) {
      adj = kk_integer_from_small(1); /*int*/
    }
    else {
      adj = kk_integer_from_small(2); /*int*/
    }
  }
  kk_integer_t x_4715;
  kk_integer_t _x5607 = kk_integer_add(doy,adj,kk_context()); /*int*/
  x_4715 = kk_integer_mul((kk_integer_from_small(12)),_x5607,kk_context()); /*int*/
  kk_integer_t _x5608 = kk_integer_add_small_const(x_4715, 373, _ctx); /*int*/
  return kk_integer_div(_x5608,(kk_integer_from_small(367)),kk_context());
}


// lift anonymous function
struct kk_std_time_calendar_tz_fixed_fun5609__t_1 {
  struct kk_function_s _base;
  kk_std_core_types__optional abbrv;
  kk_std_time_duration__duration delta;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_fixed_fun5609_1(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_fixed_fun5609_1(kk_std_core_types__optional abbrv, kk_std_time_duration__duration delta, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun5609__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar_tz_fixed_fun5609__t_1, 3, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_tz_fixed_fun5609_1, kk_context());
  _self->abbrv = abbrv;
  _self->delta = delta;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_fixed_fun5609_1(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun5609__t_1* _self = kk_function_as(struct kk_std_time_calendar_tz_fixed_fun5609__t_1*, _fself, _ctx);
  kk_std_core_types__optional abbrv = _self->abbrv; /* optional<string> */
  kk_std_time_duration__duration delta = _self->delta; /* std/time/duration/duration */
  kk_drop_match(_self, {kk_std_core_types__optional_dup(abbrv, _ctx);kk_std_time_duration__duration_dup(delta, _ctx);}, {}, _ctx)
  kk_std_time_instant__instant_drop(i0, _ctx);
  kk_box_t _x5610;
  kk_string_t _x5611;
  if (kk_std_core_types__is_Optional(abbrv, _ctx)) {
    kk_box_t _box_x5036 = abbrv._cons.Optional.value;
    kk_string_t _abbrv_2704 = kk_string_unbox(_box_x5036);
    kk_string_dup(_abbrv_2704, _ctx);
    kk_std_core_types__optional_drop(abbrv, _ctx);
    _x5611 = _abbrv_2704; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(abbrv, _ctx);
    _x5611 = kk_string_empty(); /*string*/
  }
  _x5610 = kk_string_box(_x5611); /*7*/
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_duration__duration_box(delta, _ctx), _x5610, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_tz_fixed_fun5613__t_1 {
  struct kk_function_s _base;
  kk_std_time_duration__duration delta;
};
static kk_std_core_types__maybe kk_std_time_calendar_tz_fixed_fun5613_1(kk_function_t _fself, kk_std_time_instant__instant i00, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_fixed_fun5613_1(kk_std_time_duration__duration delta, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun5613__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar_tz_fixed_fun5613__t_1, 1, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_tz_fixed_fun5613_1, kk_context());
  _self->delta = delta;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_core_types__maybe kk_std_time_calendar_tz_fixed_fun5613_1(kk_function_t _fself, kk_std_time_instant__instant i00, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun5613__t_1* _self = kk_function_as(struct kk_std_time_calendar_tz_fixed_fun5613__t_1*, _fself, _ctx);
  kk_std_time_duration__duration delta = _self->delta; /* std/time/duration/duration */
  kk_drop_match(_self, {kk_std_time_duration__duration_dup(delta, _ctx);}, {}, _ctx)
  bool _match_5425;
  double _x5614;
  {
    kk_std_num_ddouble__ddouble _x0 = delta.secs;
    {
      double _x = _x0.hi;
      _x5614 = _x; /*float64*/
    }
  }
  _match_5425 = (_x5614 == (0x0p+0)); /*bool*/
  if (_match_5425) {
    return kk_std_core_types__new_Just(kk_std_time_instant__instant_box(i00, _ctx), _ctx);
  }
  {
    kk_std_time_instant__instant_drop(i00, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
}

kk_std_time_calendar__timezone kk_std_time_calendar_tz_fixed_1(kk_string_t name0, kk_std_time_duration__duration delta, kk_std_core_types__optional abbrv, kk_context_t* _ctx) { /* (name : string, delta : std/time/duration/duration, abbrv : optional<string>) -> timezone */ 
  return kk_std_time_calendar__new_Timezone(kk_reuse_null, 0, name0, kk_std_time_calendar_new_tz_fixed_fun5609_1(abbrv, delta, _ctx), kk_std_time_calendar_new_tz_fixed_fun5613_1(delta, _ctx), _ctx);
}
 
// The standard UTC time zone with a 0 delta.


// lift anonymous function
struct kk_std_time_calendar_tz_utc_fun5617__t {
  struct kk_function_s _base;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_utc_fun5617(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_utc_fun5617(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_tz_utc_fun5617, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_utc_fun5617(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx) {
  kk_unused(_fself);
  kk_std_time_instant__instant_drop(i0, _ctx);
  kk_box_t _x5618;
  kk_string_t _x5619;
  kk_std_core_types__optional _match_5424 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5424, _ctx)) {
    kk_box_t _box_x5043 = _match_5424._cons.Optional.value;
    kk_string_t _abbrv_2704 = kk_string_unbox(_box_x5043);
    kk_string_dup(_abbrv_2704, _ctx);
    kk_std_core_types__optional_drop(_match_5424, _ctx);
    _x5619 = _abbrv_2704; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(_match_5424, _ctx);
    _x5619 = kk_string_empty(); /*string*/
  }
  _x5618 = kk_string_box(_x5619); /*7*/
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_duration__duration_box(kk_std_time_duration_zero, _ctx), _x5618, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_tz_utc_fun5621__t {
  struct kk_function_s _base;
};
static kk_std_core_types__maybe kk_std_time_calendar_tz_utc_fun5621(kk_function_t _fself, kk_std_time_instant__instant i00, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_utc_fun5621(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_tz_utc_fun5621, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_std_core_types__maybe kk_std_time_calendar_tz_utc_fun5621(kk_function_t _fself, kk_std_time_instant__instant i00, kk_context_t* _ctx) {
  kk_unused(_fself);
  bool _match_5423;
  double _x5622;
  kk_std_time_duration__duration _x5623 = kk_std_time_duration_zero; /*std/time/duration/duration*/
  {
    kk_std_num_ddouble__ddouble _x0 = _x5623.secs;
    {
      double _x = _x0.hi;
      _x5622 = _x; /*float64*/
    }
  }
  _match_5423 = (_x5622 == (0x0p+0)); /*bool*/
  if (_match_5423) {
    return kk_std_core_types__new_Just(kk_std_time_instant__instant_box(i00, _ctx), _ctx);
  }
  {
    kk_std_time_instant__instant_drop(i00, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
}

kk_std_time_calendar__timezone kk_std_time_calendar_tz_utc;
 
// Create a time zone with a fixed number of hours and optional minutes (=`0`) difference from UTC/GMT.
// The time zone name and abbreviation is optional, and the default name
// will reflect the offset from UTC, for example `"UTC+1:20"`, or `"UTC-8"`.
// The default abbreviation is the empty string.
// The sign of the minutes will be matched to the hours (unless `hours` is `0`).
// For example, `tz-fixed(-1,30)` gives a time zone offset of -90 minutes, "UTC-1:30".
// Returns `tz-utc` if both `hours` and `mins` are zero.


// lift anonymous function
struct kk_std_time_calendar_tz_fixed_fun5653__t {
  struct kk_function_s _base;
  kk_std_core_types__optional abbrv;
  kk_std_time_duration__duration delta;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_fixed_fun5653(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_fixed_fun5653(kk_std_core_types__optional abbrv, kk_std_time_duration__duration delta, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun5653__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_tz_fixed_fun5653__t, 3, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_tz_fixed_fun5653, kk_context());
  _self->abbrv = abbrv;
  _self->delta = delta;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_fixed_fun5653(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun5653__t* _self = kk_function_as(struct kk_std_time_calendar_tz_fixed_fun5653__t*, _fself, _ctx);
  kk_std_core_types__optional abbrv = _self->abbrv; /* optional<string> */
  kk_std_time_duration__duration delta = _self->delta; /* std/time/duration/duration */
  kk_drop_match(_self, {kk_std_core_types__optional_dup(abbrv, _ctx);kk_std_time_duration__duration_dup(delta, _ctx);}, {}, _ctx)
  kk_std_time_instant__instant_drop(i0, _ctx);
  kk_box_t _x5654;
  kk_string_t _x5655;
  if (kk_std_core_types__is_Optional(abbrv, _ctx)) {
    kk_box_t _box_x5061 = abbrv._cons.Optional.value;
    kk_string_t _abbrv_2281 = kk_string_unbox(_box_x5061);
    kk_string_dup(_abbrv_2281, _ctx);
    kk_std_core_types__optional_drop(abbrv, _ctx);
    _x5655 = _abbrv_2281; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(abbrv, _ctx);
    _x5655 = kk_string_empty(); /*string*/
  }
  _x5654 = kk_string_box(_x5655); /*7*/
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_duration__duration_box(delta, _ctx), _x5654, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_tz_fixed_fun5657__t {
  struct kk_function_s _base;
  kk_std_time_duration__duration delta;
};
static kk_std_core_types__maybe kk_std_time_calendar_tz_fixed_fun5657(kk_function_t _fself, kk_std_time_instant__instant i00, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_fixed_fun5657(kk_std_time_duration__duration delta, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun5657__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_tz_fixed_fun5657__t, 1, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_tz_fixed_fun5657, kk_context());
  _self->delta = delta;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_core_types__maybe kk_std_time_calendar_tz_fixed_fun5657(kk_function_t _fself, kk_std_time_instant__instant i00, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun5657__t* _self = kk_function_as(struct kk_std_time_calendar_tz_fixed_fun5657__t*, _fself, _ctx);
  kk_std_time_duration__duration delta = _self->delta; /* std/time/duration/duration */
  kk_drop_match(_self, {kk_std_time_duration__duration_dup(delta, _ctx);}, {}, _ctx)
  bool _match_5412;
  double _x5658;
  {
    kk_std_num_ddouble__ddouble _x0 = delta.secs;
    {
      double _x = _x0.hi;
      _x5658 = _x; /*float64*/
    }
  }
  _match_5412 = (_x5658 == (0x0p+0)); /*bool*/
  if (_match_5412) {
    return kk_std_core_types__new_Just(kk_std_time_instant__instant_box(i00, _ctx), _ctx);
  }
  {
    kk_std_time_instant__instant_drop(i00, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
}


// lift anonymous function
struct kk_std_time_calendar_tz_fixed_fun5688__t {
  struct kk_function_s _base;
  kk_std_core_types__optional abbrv;
  kk_std_time_duration__duration delta00;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_fixed_fun5688(kk_function_t _fself, kk_std_time_instant__instant i01, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_fixed_fun5688(kk_std_core_types__optional abbrv, kk_std_time_duration__duration delta00, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun5688__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_tz_fixed_fun5688__t, 3, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_tz_fixed_fun5688, kk_context());
  _self->abbrv = abbrv;
  _self->delta00 = delta00;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_fixed_fun5688(kk_function_t _fself, kk_std_time_instant__instant i01, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun5688__t* _self = kk_function_as(struct kk_std_time_calendar_tz_fixed_fun5688__t*, _fself, _ctx);
  kk_std_core_types__optional abbrv = _self->abbrv; /* optional<string> */
  kk_std_time_duration__duration delta00 = _self->delta00; /* std/time/duration/duration */
  kk_drop_match(_self, {kk_std_core_types__optional_dup(abbrv, _ctx);kk_std_time_duration__duration_dup(delta00, _ctx);}, {}, _ctx)
  kk_std_time_instant__instant_drop(i01, _ctx);
  kk_box_t _x5689;
  kk_string_t _x5690;
  if (kk_std_core_types__is_Optional(abbrv, _ctx)) {
    kk_box_t _box_x5075 = abbrv._cons.Optional.value;
    kk_string_t _abbrv_22810 = kk_string_unbox(_box_x5075);
    kk_string_dup(_abbrv_22810, _ctx);
    kk_std_core_types__optional_drop(abbrv, _ctx);
    _x5690 = _abbrv_22810; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(abbrv, _ctx);
    _x5690 = kk_string_empty(); /*string*/
  }
  _x5689 = kk_string_box(_x5690); /*7*/
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_duration__duration_box(delta00, _ctx), _x5689, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_tz_fixed_fun5692__t {
  struct kk_function_s _base;
  kk_std_time_duration__duration delta00;
};
static kk_std_core_types__maybe kk_std_time_calendar_tz_fixed_fun5692(kk_function_t _fself, kk_std_time_instant__instant i000, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_fixed_fun5692(kk_std_time_duration__duration delta00, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun5692__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_tz_fixed_fun5692__t, 1, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_tz_fixed_fun5692, kk_context());
  _self->delta00 = delta00;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_core_types__maybe kk_std_time_calendar_tz_fixed_fun5692(kk_function_t _fself, kk_std_time_instant__instant i000, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun5692__t* _self = kk_function_as(struct kk_std_time_calendar_tz_fixed_fun5692__t*, _fself, _ctx);
  kk_std_time_duration__duration delta00 = _self->delta00; /* std/time/duration/duration */
  kk_drop_match(_self, {kk_std_time_duration__duration_dup(delta00, _ctx);}, {}, _ctx)
  bool _match_5400;
  double _x5693;
  {
    kk_std_num_ddouble__ddouble _x00 = delta00.secs;
    {
      double _x1 = _x00.hi;
      _x5693 = _x1; /*float64*/
    }
  }
  _match_5400 = (_x5693 == (0x0p+0)); /*bool*/
  if (_match_5400) {
    return kk_std_core_types__new_Just(kk_std_time_instant__instant_box(i000, _ctx), _ctx);
  }
  {
    kk_std_time_instant__instant_drop(i000, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
}

kk_std_time_calendar__timezone kk_std_time_calendar_tz_fixed(kk_integer_t hours, kk_std_core_types__optional mins, kk_std_core_types__optional name0, kk_std_core_types__optional abbrv, kk_std_core_types__optional hourwidth, kk_context_t* _ctx) { /* (hours : int, mins : optional<int>, name : optional<string>, abbrv : optional<string>, hourwidth : optional<int>) -> timezone */ 
  kk_integer_t _mins_2272;
  if (kk_std_core_types__is_Optional(mins, _ctx)) {
    kk_box_t _box_x5050 = mins._cons.Optional.value;
    kk_integer_t _mins_2273 = kk_integer_unbox(_box_x5050, _ctx);
    kk_integer_dup(_mins_2273, _ctx);
    kk_std_core_types__optional_drop(mins, _ctx);
    _mins_2272 = _mins_2273; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(mins, _ctx);
    _mins_2272 = kk_integer_from_small(0); /*int*/
  }
  bool _match_5399 = kk_integer_is_zero_borrow(hours); /*bool*/;
  if (_match_5399) {
    bool _match_5411 = kk_integer_is_zero_borrow(_mins_2272); /*bool*/;
    if (_match_5411) {
      kk_std_core_types__optional_drop(name0, _ctx);
      kk_std_core_types__optional_drop(hourwidth, _ctx);
      kk_integer_drop(hours, _ctx);
      kk_std_core_types__optional_drop(abbrv, _ctx);
      kk_integer_drop(_mins_2272, _ctx);
      return kk_std_time_calendar__timezone_dup(kk_std_time_calendar_tz_utc, _ctx);
    }
    {
      kk_integer_t xmins;
      bool _match_5421 = kk_integer_gt_borrow(hours,(kk_integer_from_small(0)),kk_context()); /*bool*/;
      if (_match_5421) {
        kk_integer_t _x5624 = kk_integer_dup(_mins_2272, _ctx); /*int*/
        xmins = kk_integer_abs(_x5624,kk_context()); /*int*/
      }
      else {
        bool _match_5422 = kk_integer_lt_borrow(hours,(kk_integer_from_small(0)),kk_context()); /*bool*/;
        if (_match_5422) {
          kk_integer_t i1_4725;
          kk_integer_t _x5625 = kk_integer_dup(_mins_2272, _ctx); /*int*/
          i1_4725 = kk_integer_abs(_x5625,kk_context()); /*int*/
          xmins = kk_integer_neg(i1_4725,kk_context()); /*int*/
        }
        else {
          xmins = kk_integer_dup(_mins_2272, _ctx); /*int*/
        }
      }
      kk_integer_t x_4726;
      kk_integer_t _x5626 = kk_integer_dup(hours, _ctx); /*int*/
      x_4726 = kk_integer_mul(_x5626,(kk_integer_from_small(3600)),kk_context()); /*int*/
      kk_integer_t y_4727 = kk_integer_mul(xmins,(kk_integer_from_small(60)),kk_context()); /*int*/;
      kk_integer_t secs = kk_integer_add(x_4726,y_4727,kk_context()); /*int*/;
      kk_string_t y0_4729;
      kk_string_t _x5627;
      kk_define_string_literal(, _s5628, 3, "UTC", _ctx)
      _x5627 = kk_string_dup(_s5628, _ctx); /*string*/
      kk_string_t _x5629;
      kk_string_t _x5630;
      bool _match_5420 = kk_integer_lt_borrow(secs,(kk_integer_from_small(0)),kk_context()); /*bool*/;
      if (_match_5420) {
        kk_define_string_literal(, _s5631, 1, "-", _ctx)
        _x5630 = kk_string_dup(_s5631, _ctx); /*string*/
      }
      else {
        kk_define_string_literal(, _s5632, 1, "+", _ctx)
        _x5630 = kk_string_dup(_s5632, _ctx); /*string*/
      }
      kk_string_t _x5633;
      kk_string_t _x5634;
      kk_string_t _own_5418;
      kk_integer_t _x5635 = kk_integer_abs(hours,kk_context()); /*int*/
      _own_5418 = kk_std_core_show(_x5635, _ctx); /*string*/
      kk_integer_t _brw_5417;
      if (kk_std_core_types__is_Optional(hourwidth, _ctx)) {
        kk_box_t _box_x5051 = hourwidth._cons.Optional.value;
        kk_integer_t _hourwidth_2285 = kk_integer_unbox(_box_x5051, _ctx);
        kk_integer_dup(_hourwidth_2285, _ctx);
        kk_std_core_types__optional_drop(hourwidth, _ctx);
        _brw_5417 = _hourwidth_2285; /*int*/
      }
      else {
        kk_std_core_types__optional_drop(hourwidth, _ctx);
        _brw_5417 = kk_integer_from_small(1); /*int*/
      }
      kk_string_t _brw_5419;
      kk_std_core_types__optional _x5636 = kk_std_core_types__new_Optional(kk_char_box('0', _ctx), _ctx); /*optional<116>*/
      _brw_5419 = kk_std_core_pad_left(_own_5418, _brw_5417, _x5636, _ctx); /*string*/
      kk_integer_drop(_brw_5417, _ctx);
      _x5634 = _brw_5419; /*string*/
      kk_string_t _x5637;
      bool _match_5415 = kk_integer_is_zero_borrow(_mins_2272); /*bool*/;
      if (_match_5415) {
        kk_integer_drop(_mins_2272, _ctx);
        _x5637 = kk_string_empty(); /*string*/
      }
      else {
        kk_string_t _x5639;
        kk_define_string_literal(, _s5640, 1, ":", _ctx)
        _x5639 = kk_string_dup(_s5640, _ctx); /*string*/
        kk_string_t _x5641;
        kk_string_t _own_5416;
        kk_integer_t _x5642 = kk_integer_abs(_mins_2272,kk_context()); /*int*/
        _own_5416 = kk_std_core_show(_x5642, _ctx); /*string*/
        kk_std_core_types__optional _x5643 = kk_std_core_types__new_Optional(kk_char_box('0', _ctx), _ctx); /*optional<116>*/
        _x5641 = kk_std_core_pad_left(_own_5416, kk_integer_from_small(2), _x5643, _ctx); /*string*/
        _x5637 = kk_std_core__lp__plus__plus__1_rp_(_x5639, _x5641, _ctx); /*string*/
      }
      _x5633 = kk_std_core__lp__plus__plus__1_rp_(_x5634, _x5637, _ctx); /*string*/
      _x5629 = kk_std_core__lp__plus__plus__1_rp_(_x5630, _x5633, _ctx); /*string*/
      y0_4729 = kk_std_core__lp__plus__plus__1_rp_(_x5627, _x5629, _ctx); /*string*/
      kk_string_t tzonename;
      bool _match_5414;
      kk_string_t _x5644;
      if (kk_std_core_types__is_Optional(name0, _ctx)) {
        kk_box_t _box_x5056 = name0._cons.Optional.value;
        kk_string_t _name_2277 = kk_string_unbox(_box_x5056);
        kk_string_dup(_name_2277, _ctx);
        _x5644 = _name_2277; /*string*/
      }
      else {
        _x5644 = kk_string_empty(); /*string*/
      }
      kk_string_t _x5646 = kk_string_empty(); /*string*/
      _match_5414 = kk_string_is_eq(_x5644,_x5646,kk_context()); /*bool*/
      if (_match_5414) {
        kk_std_core_types__optional_drop(name0, _ctx);
        tzonename = y0_4729; /*string*/
      }
      else {
        kk_string_drop(y0_4729, _ctx);
        if (kk_std_core_types__is_Optional(name0, _ctx)) {
          kk_box_t _box_x5057 = name0._cons.Optional.value;
          kk_string_t _name_22770 = kk_string_unbox(_box_x5057);
          kk_string_dup(_name_22770, _ctx);
          kk_std_core_types__optional_drop(name0, _ctx);
          tzonename = _name_22770; /*string*/
        }
        else {
          kk_std_core_types__optional_drop(name0, _ctx);
          tzonename = kk_string_empty(); /*string*/
        }
      }
      kk_std_time_duration__duration delta;
      kk_std_num_ddouble__ddouble _x5649;
      kk_std_core_types__optional _x5650;
      kk_box_t _x5651;
      double _x5652;
      kk_std_core_types__optional _match_5413 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
      if (kk_std_core_types__is_Optional(_match_5413, _ctx)) {
        kk_box_t _box_x5058 = _match_5413._cons.Optional.value;
        double _frac_159 = kk_double_unbox(_box_x5058, KK_BORROWED, _ctx);
        kk_std_core_types__optional_drop(_match_5413, _ctx);
        _x5652 = _frac_159; /*float64*/
      }
      else {
        kk_std_core_types__optional_drop(_match_5413, _ctx);
        _x5652 = 0x0p+0; /*float64*/
      }
      _x5651 = kk_double_box(_x5652, _ctx); /*116*/
      _x5650 = kk_std_core_types__new_Optional(_x5651, _ctx); /*optional<116>*/
      _x5649 = kk_std_time_timestamp_timespan(secs, _x5650, _ctx); /*std/time/timestamp/timespan*/
      delta = kk_std_time_duration__new_Duration(_x5649, _ctx); /*std/time/duration/duration*/
      return kk_std_time_calendar__new_Timezone(kk_reuse_null, 0, tzonename, kk_std_time_calendar_new_tz_fixed_fun5653(abbrv, delta, _ctx), kk_std_time_calendar_new_tz_fixed_fun5657(delta, _ctx), _ctx);
    }
  }
  {
    kk_integer_t xmins0;
    bool _match_5409 = kk_integer_gt_borrow(hours,(kk_integer_from_small(0)),kk_context()); /*bool*/;
    if (_match_5409) {
      kk_integer_t _x5659 = kk_integer_dup(_mins_2272, _ctx); /*int*/
      xmins0 = kk_integer_abs(_x5659,kk_context()); /*int*/
    }
    else {
      bool _match_5410 = kk_integer_lt_borrow(hours,(kk_integer_from_small(0)),kk_context()); /*bool*/;
      if (_match_5410) {
        kk_integer_t i5_4735;
        kk_integer_t _x5660 = kk_integer_dup(_mins_2272, _ctx); /*int*/
        i5_4735 = kk_integer_abs(_x5660,kk_context()); /*int*/
        xmins0 = kk_integer_neg(i5_4735,kk_context()); /*int*/
      }
      else {
        xmins0 = kk_integer_dup(_mins_2272, _ctx); /*int*/
      }
    }
    kk_integer_t x1_4736;
    kk_integer_t _x5661 = kk_integer_dup(hours, _ctx); /*int*/
    x1_4736 = kk_integer_mul(_x5661,(kk_integer_from_small(3600)),kk_context()); /*int*/
    kk_integer_t y1_4737 = kk_integer_mul(xmins0,(kk_integer_from_small(60)),kk_context()); /*int*/;
    kk_integer_t secs1 = kk_integer_add(x1_4736,y1_4737,kk_context()); /*int*/;
    kk_string_t y2_4739;
    kk_string_t _x5662;
    kk_define_string_literal(, _s5663, 3, "UTC", _ctx)
    _x5662 = kk_string_dup(_s5663, _ctx); /*string*/
    kk_string_t _x5664;
    kk_string_t _x5665;
    bool _match_5408 = kk_integer_lt_borrow(secs1,(kk_integer_from_small(0)),kk_context()); /*bool*/;
    if (_match_5408) {
      kk_define_string_literal(, _s5666, 1, "-", _ctx)
      _x5665 = kk_string_dup(_s5666, _ctx); /*string*/
    }
    else {
      kk_define_string_literal(, _s5667, 1, "+", _ctx)
      _x5665 = kk_string_dup(_s5667, _ctx); /*string*/
    }
    kk_string_t _x5668;
    kk_string_t _x5669;
    kk_string_t _own_5406;
    kk_integer_t _x5670 = kk_integer_abs(hours,kk_context()); /*int*/
    _own_5406 = kk_std_core_show(_x5670, _ctx); /*string*/
    kk_integer_t _brw_5405;
    if (kk_std_core_types__is_Optional(hourwidth, _ctx)) {
      kk_box_t _box_x5065 = hourwidth._cons.Optional.value;
      kk_integer_t _hourwidth_22850 = kk_integer_unbox(_box_x5065, _ctx);
      kk_integer_dup(_hourwidth_22850, _ctx);
      kk_std_core_types__optional_drop(hourwidth, _ctx);
      _brw_5405 = _hourwidth_22850; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(hourwidth, _ctx);
      _brw_5405 = kk_integer_from_small(1); /*int*/
    }
    kk_string_t _brw_5407;
    kk_std_core_types__optional _x5671 = kk_std_core_types__new_Optional(kk_char_box('0', _ctx), _ctx); /*optional<116>*/
    _brw_5407 = kk_std_core_pad_left(_own_5406, _brw_5405, _x5671, _ctx); /*string*/
    kk_integer_drop(_brw_5405, _ctx);
    _x5669 = _brw_5407; /*string*/
    kk_string_t _x5672;
    bool _match_5403 = kk_integer_is_zero_borrow(_mins_2272); /*bool*/;
    if (_match_5403) {
      kk_integer_drop(_mins_2272, _ctx);
      _x5672 = kk_string_empty(); /*string*/
    }
    else {
      kk_string_t _x5674;
      kk_define_string_literal(, _s5675, 1, ":", _ctx)
      _x5674 = kk_string_dup(_s5675, _ctx); /*string*/
      kk_string_t _x5676;
      kk_string_t _own_5404;
      kk_integer_t _x5677 = kk_integer_abs(_mins_2272,kk_context()); /*int*/
      _own_5404 = kk_std_core_show(_x5677, _ctx); /*string*/
      kk_std_core_types__optional _x5678 = kk_std_core_types__new_Optional(kk_char_box('0', _ctx), _ctx); /*optional<116>*/
      _x5676 = kk_std_core_pad_left(_own_5404, kk_integer_from_small(2), _x5678, _ctx); /*string*/
      _x5672 = kk_std_core__lp__plus__plus__1_rp_(_x5674, _x5676, _ctx); /*string*/
    }
    _x5668 = kk_std_core__lp__plus__plus__1_rp_(_x5669, _x5672, _ctx); /*string*/
    _x5664 = kk_std_core__lp__plus__plus__1_rp_(_x5665, _x5668, _ctx); /*string*/
    y2_4739 = kk_std_core__lp__plus__plus__1_rp_(_x5662, _x5664, _ctx); /*string*/
    kk_string_t tzonename0;
    bool _match_5402;
    kk_string_t _x5679;
    if (kk_std_core_types__is_Optional(name0, _ctx)) {
      kk_box_t _box_x5070 = name0._cons.Optional.value;
      kk_string_t _name_227700 = kk_string_unbox(_box_x5070);
      kk_string_dup(_name_227700, _ctx);
      _x5679 = _name_227700; /*string*/
    }
    else {
      _x5679 = kk_string_empty(); /*string*/
    }
    kk_string_t _x5681 = kk_string_empty(); /*string*/
    _match_5402 = kk_string_is_eq(_x5679,_x5681,kk_context()); /*bool*/
    if (_match_5402) {
      kk_std_core_types__optional_drop(name0, _ctx);
      tzonename0 = y2_4739; /*string*/
    }
    else {
      kk_string_drop(y2_4739, _ctx);
      if (kk_std_core_types__is_Optional(name0, _ctx)) {
        kk_box_t _box_x5071 = name0._cons.Optional.value;
        kk_string_t _name_227701 = kk_string_unbox(_box_x5071);
        kk_string_dup(_name_227701, _ctx);
        kk_std_core_types__optional_drop(name0, _ctx);
        tzonename0 = _name_227701; /*string*/
      }
      else {
        kk_std_core_types__optional_drop(name0, _ctx);
        tzonename0 = kk_string_empty(); /*string*/
      }
    }
    kk_std_time_duration__duration delta00;
    kk_std_num_ddouble__ddouble _x5684;
    kk_std_core_types__optional _x5685;
    kk_box_t _x5686;
    double _x5687;
    kk_std_core_types__optional _match_5401 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_5401, _ctx)) {
      kk_box_t _box_x5072 = _match_5401._cons.Optional.value;
      double _frac_1590 = kk_double_unbox(_box_x5072, KK_BORROWED, _ctx);
      kk_std_core_types__optional_drop(_match_5401, _ctx);
      _x5687 = _frac_1590; /*float64*/
    }
    else {
      kk_std_core_types__optional_drop(_match_5401, _ctx);
      _x5687 = 0x0p+0; /*float64*/
    }
    _x5686 = kk_double_box(_x5687, _ctx); /*116*/
    _x5685 = kk_std_core_types__new_Optional(_x5686, _ctx); /*optional<116>*/
    _x5684 = kk_std_time_timestamp_timespan(secs1, _x5685, _ctx); /*std/time/timestamp/timespan*/
    delta00 = kk_std_time_duration__new_Duration(_x5684, _ctx); /*std/time/duration/duration*/
    return kk_std_time_calendar__new_Timezone(kk_reuse_null, 0, tzonename0, kk_std_time_calendar_new_tz_fixed_fun5688(abbrv, delta00, _ctx), kk_std_time_calendar_new_tz_fixed_fun5692(delta00, _ctx), _ctx);
  }
}
 
// Is this the UTC timezone?

bool kk_std_time_calendar_is_tz_utc(kk_std_time_calendar__timezone tz, kk_context_t* _ctx) { /* (tz : timezone) -> bool */ 
  kk_string_t _x5694;
  {
    struct kk_std_time_calendar_Timezone* _con5695 = kk_std_time_calendar__as_Timezone(tz, _ctx);
    kk_string_t _x = _con5695->name;
    kk_function_t _pat0 = _con5695->utc_delta;
    kk_function_t _pat1 = _con5695->utc_inverse;
    if kk_likely(kk_datatype_ptr_is_unique(tz, _ctx)) {
      kk_function_drop(_pat1, _ctx);
      kk_function_drop(_pat0, _ctx);
      kk_datatype_ptr_free(tz, _ctx);
    }
    else {
      kk_string_dup(_x, _ctx);
      kk_datatype_ptr_decref(tz, _ctx);
    }
    _x5694 = _x; /*string*/
  }
  kk_string_t _x5696;
  kk_std_time_calendar__timezone _x5697 = kk_std_time_calendar_tz_utc; /*std/time/calendar/timezone*/
  {
    struct kk_std_time_calendar_Timezone* _con5698 = kk_std_time_calendar__as_Timezone(_x5697, _ctx);
    kk_string_t _x0 = _con5698->name;
    _x5696 = kk_string_dup(_x0, _ctx); /*string*/
  }
  return kk_string_is_eq(_x5694,_x5696,kk_context());
}

kk_std_time_instant__instant kk_std_time_calendar_earth_timestamp_to_instant(kk_std_time_timestamp__timestamp t, kk_std_time_calendar__timezone tz, kk_std_time_instant__timescale ts, kk_context_t* _ctx) { /* (t : std/time/timestamp/timestamp, tz : timezone, ts : std/time/instant/timescale) -> std/time/instant/instant */ 
  kk_std_time_instant__instant i;
  kk_std_time_timestamp__timestamp _x5699 = kk_std_time_timestamp__timestamp_dup(t, _ctx); /*std/time/timestamp/timestamp*/
  kk_std_time_instant__timescale _x5700 = kk_std_time_instant__timescale_dup(ts, _ctx); /*std/time/instant/timescale*/
  i = kk_std_time_instant__new_Instant(_x5699, _x5700, _ctx); /*std/time/instant/instant*/
  bool _match_5395;
  kk_string_t _x5701;
  {
    struct kk_std_time_calendar_Timezone* _con5702 = kk_std_time_calendar__as_Timezone(tz, _ctx);
    kk_string_t _x = _con5702->name;
    kk_string_dup(_x, _ctx);
    _x5701 = _x; /*string*/
  }
  kk_string_t _x5703;
  kk_std_time_calendar__timezone _x5704 = kk_std_time_calendar_tz_utc; /*std/time/calendar/timezone*/
  {
    struct kk_std_time_calendar_Timezone* _con5705 = kk_std_time_calendar__as_Timezone(_x5704, _ctx);
    kk_string_t _x0 = _con5705->name;
    _x5703 = kk_string_dup(_x0, _ctx); /*string*/
  }
  _match_5395 = kk_string_is_eq(_x5701,_x5703,kk_context()); /*bool*/
  if (_match_5395) {
    kk_datatype_ptr_dropn(tz, (KK_I32(3)), _ctx);
    kk_datatype_ptr_dropn(ts, (KK_I32(10)), _ctx);
    kk_datatype_ptr_dropn(t, (KK_I32(0)), _ctx);
    return i;
  }
  {
    struct kk_std_time_calendar_Timezone* _con5706 = kk_std_time_calendar__as_Timezone(tz, _ctx);
    kk_string_t _pat02 = _con5706->name;
    kk_function_t _pat11 = _con5706->utc_delta;
    kk_function_t _x1 = _con5706->utc_inverse;
    kk_function_dup(_x1, _ctx);
    kk_std_core_types__maybe _match_5396;
    kk_std_time_instant__instant _x5707 = kk_std_time_instant__instant_dup(i, _ctx); /*std/time/instant/instant*/
    _match_5396 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x1, (_x1, _x5707, _ctx), _ctx); /*maybe<std/time/instant/instant>*/
    if (kk_std_core_types__is_Just(_match_5396, _ctx)) {
      kk_box_t _box_x5085 = _match_5396._cons.Just.value;
      kk_std_time_instant__instant inv = kk_std_time_instant__instant_unbox(_box_x5085, KK_BORROWED, _ctx);
      if kk_likely(kk_datatype_ptr_is_unique(tz, _ctx)) {
        kk_function_drop(_x1, _ctx);
        kk_function_drop(_pat11, _ctx);
        kk_string_drop(_pat02, _ctx);
        kk_datatype_ptr_free(tz, _ctx);
      }
      else {
        kk_datatype_ptr_decref(tz, _ctx);
      }
      kk_datatype_ptr_dropn(ts, (KK_I32(10)), _ctx);
      kk_datatype_ptr_dropn(t, (KK_I32(0)), _ctx);
      kk_std_time_instant__instant_drop(i, _ctx);
      kk_std_time_instant__instant_dup(inv, _ctx);
      kk_std_core_types__maybe_drop(_match_5396, _ctx);
      return inv;
    }
    {
      kk_std_core_types__tuple2_ _this_4755;
      {
        struct kk_std_time_calendar_Timezone* _con5708 = kk_std_time_calendar__as_Timezone(tz, _ctx);
        kk_function_t _x3 = _con5708->utc_delta;
        kk_function_dup(_x3, _ctx);
        _this_4755 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x3, (_x3, i, _ctx), _ctx); /*(std/time/duration/duration, string)*/
      }
      kk_std_time_timestamp__timestamp t1_4758;
      kk_std_time_timestamp__timestamp _x5709 = kk_std_time_timestamp__timestamp_dup(t, _ctx); /*std/time/timestamp/timestamp*/
      kk_std_num_ddouble__ddouble _x5710;
      {
        kk_box_t _box_x5086 = _this_4755.fst;
        kk_box_t _box_x5087 = _this_4755.snd;
        kk_std_time_duration__duration _x2 = kk_std_time_duration__duration_unbox(_box_x5086, KK_BORROWED, _ctx);
        {
          kk_std_num_ddouble__ddouble _x4 = _x2.secs;
          _x5710 = _x4; /*std/time/timestamp/timespan*/
        }
      }
      t1_4758 = kk_std_time_timestamp__lp__dash__rp_(_x5709, _x5710, _ctx); /*std/time/timestamp/timestamp*/
      kk_std_time_instant__instant tzi1;
      kk_std_time_instant__timescale _x5711 = kk_std_time_instant__timescale_dup(ts, _ctx); /*std/time/instant/timescale*/
      tzi1 = kk_std_time_instant__new_Instant(t1_4758, _x5711, _ctx); /*std/time/instant/instant*/
      kk_std_core_types__tuple2_ _this0_4760;
      {
        struct kk_std_time_calendar_Timezone* _con5712 = kk_std_time_calendar__as_Timezone(tz, _ctx);
        kk_string_t _pat07 = _con5712->name;
        kk_function_t _x6 = _con5712->utc_delta;
        kk_function_t _pat15 = _con5712->utc_inverse;
        if kk_likely(kk_datatype_ptr_is_unique(tz, _ctx)) {
          kk_function_drop(_pat15, _ctx);
          kk_string_drop(_pat07, _ctx);
          kk_datatype_ptr_free(tz, _ctx);
        }
        else {
          kk_function_dup(_x6, _ctx);
          kk_datatype_ptr_decref(tz, _ctx);
        }
        kk_std_time_instant__instant _x5713 = kk_std_time_instant__instant_dup(tzi1, _ctx); /*std/time/instant/instant*/
        _this0_4760 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x6, (_x6, _x5713, _ctx), _ctx); /*(std/time/duration/duration, string)*/
      }
      bool _match_5397;
      kk_std_core_types__order _x5714;
      kk_std_core_types__order _match_5398;
      double _x5715;
      {
        kk_box_t _box_x5088 = _this_4755.fst;
        kk_box_t _box_x5089 = _this_4755.snd;
        kk_std_time_duration__duration _x20 = kk_std_time_duration__duration_unbox(_box_x5088, KK_BORROWED, _ctx);
        {
          kk_std_num_ddouble__ddouble _x30 = _x20.secs;
          {
            double _x7 = _x30.hi;
            _x5715 = _x7; /*float64*/
          }
        }
      }
      double _x5716;
      {
        kk_box_t _box_x5090 = _this0_4760.fst;
        kk_box_t _box_x5091 = _this0_4760.snd;
        kk_std_time_duration__duration _x5 = kk_std_time_duration__duration_unbox(_box_x5090, KK_BORROWED, _ctx);
        {
          kk_std_num_ddouble__ddouble _x40 = _x5.secs;
          {
            double _x00 = _x40.hi;
            _x5716 = _x00; /*float64*/
          }
        }
      }
      _match_5398 = kk_std_num_float64_compare(_x5715, _x5716, _ctx); /*order*/
      if (kk_std_core_types__is_Eq(_match_5398, _ctx)) {
        double _x5717;
        {
          kk_box_t _box_x5092 = _this_4755.fst;
          kk_box_t _box_x5093 = _this_4755.snd;
          kk_std_time_duration__duration _x21 = kk_std_time_duration__duration_unbox(_box_x5092, KK_BORROWED, _ctx);
          kk_std_core_types__tuple2__drop(_this_4755, _ctx);
          {
            kk_std_num_ddouble__ddouble _x300 = _x21.secs;
            {
              double _x10 = _x300.lo;
              _x5717 = _x10; /*float64*/
            }
          }
        }
        double _x5718;
        {
          kk_box_t _box_x5094 = _this0_4760.fst;
          kk_box_t _box_x5095 = _this0_4760.snd;
          kk_std_time_duration__duration _x50 = kk_std_time_duration__duration_unbox(_box_x5094, KK_BORROWED, _ctx);
          {
            kk_std_num_ddouble__ddouble _x400 = _x50.secs;
            {
              double _x200 = _x400.lo;
              _x5718 = _x200; /*float64*/
            }
          }
        }
        _x5714 = kk_std_num_float64_compare(_x5717, _x5718, _ctx); /*order*/
      }
      else {
        kk_std_core_types__tuple2__drop(_this_4755, _ctx);
        _x5714 = _match_5398; /*order*/
      }
      _match_5397 = kk_std_core__lp__eq__eq__4_rp_(_x5714, kk_std_core_types__new_Eq(_ctx), _ctx); /*bool*/
      if (_match_5397) {
        kk_datatype_ptr_dropn(ts, (KK_I32(10)), _ctx);
        kk_datatype_ptr_dropn(t, (KK_I32(0)), _ctx);
        kk_std_core_types__tuple2__drop(_this0_4760, _ctx);
        return tzi1;
      }
      {
        kk_std_time_instant__instant_drop(tzi1, _ctx);
        kk_std_time_timestamp__timestamp t2_4765;
        kk_std_num_ddouble__ddouble _x5719;
        {
          kk_box_t _box_x5096 = _this0_4760.fst;
          kk_box_t _box_x5097 = _this0_4760.snd;
          kk_std_time_duration__duration _x51 = kk_std_time_duration__duration_unbox(_box_x5096, KK_BORROWED, _ctx);
          kk_std_core_types__tuple2__drop(_this0_4760, _ctx);
          {
            kk_std_num_ddouble__ddouble _x8 = _x51.secs;
            _x5719 = _x8; /*std/time/timestamp/timespan*/
          }
        }
        t2_4765 = kk_std_time_timestamp__lp__dash__rp_(t, _x5719, _ctx); /*std/time/timestamp/timestamp*/
        return kk_std_time_instant__new_Instant(t2_4765, ts, _ctx);
      }
    }
  }
}
 
// Create a calendar where we assume that each
// day has 24 hours with 60 minutes, and where minutes are 60 seconds
// (with possibly leap seconds). The function takes care of timezones,
// epoch shifts, and leap seconds, and only needs a calendar `name`,
// a function `days-to-date` to calculate a date given a number of days since the `epoch`,
// a function `date-to-days` to calculate days since the `epoch` for a given date,
// and an optional time scale to be used (=`ts-utc`).
// `month-prefix` (=`""`) is used when displaying numeric months and is set for
// the ISO week dates to `"W"` for example. The `show-era` function can be given
// to display the era of a date.


// lift anonymous function
struct kk_std_time_calendar_earth_calendar_fun5723__t {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x5102;
};
static kk_string_t kk_std_time_calendar_earth_calendar_fun5723(kk_function_t _fself, kk_std_time_date__date _b_5105, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_earth_calendar_fun5723(kk_box_t _fun_unbox_x5102, kk_context_t* _ctx) {
  struct kk_std_time_calendar_earth_calendar_fun5723__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_earth_calendar_fun5723__t, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_earth_calendar_fun5723, kk_context());
  _self->_fun_unbox_x5102 = _fun_unbox_x5102;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_string_t kk_std_time_calendar_earth_calendar_fun5723(kk_function_t _fself, kk_std_time_date__date _b_5105, kk_context_t* _ctx) {
  struct kk_std_time_calendar_earth_calendar_fun5723__t* _self = kk_function_as(struct kk_std_time_calendar_earth_calendar_fun5723__t*, _fself, _ctx);
  kk_box_t _fun_unbox_x5102 = _self->_fun_unbox_x5102; /* 116 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x5102, _ctx);}, {}, _ctx)
  kk_box_t _x5724;
  kk_function_t _x5725 = kk_function_unbox(_fun_unbox_x5102, _ctx); /*(5103) -> 5104*/
  _x5724 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x5725, (_x5725, kk_std_time_date__date_box(_b_5105, _ctx), _ctx), _ctx); /*5104*/
  return kk_string_unbox(_x5724);
}


// lift anonymous function
struct kk_std_time_calendar_earth_calendar_fun5726__t {
  struct kk_function_s _base;
};
static kk_string_t kk_std_time_calendar_earth_calendar_fun5726(kk_function_t _fself, kk_std_time_date__date d, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_earth_calendar_fun5726(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_earth_calendar_fun5726, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_string_t kk_std_time_calendar_earth_calendar_fun5726(kk_function_t _fself, kk_std_time_date__date d, kk_context_t* _ctx) {
  kk_unused(_fself);
  kk_std_time_date__date_drop(d, _ctx);
  return kk_string_empty();
}


// lift anonymous function
struct kk_std_time_calendar_earth_calendar_fun5729__t {
  struct kk_function_s _base;
  kk_function_t days_to_date0;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_earth_calendar_fun5729(kk_function_t _fself, kk_std_time_instant__instant i, kk_std_time_duration__duration tzdelta, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_earth_calendar_fun5729(kk_function_t days_to_date0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_earth_calendar_fun5729__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_earth_calendar_fun5729__t, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_earth_calendar_fun5729, kk_context());
  _self->days_to_date0 = days_to_date0;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_earth_calendar_fun5729(kk_function_t _fself, kk_std_time_instant__instant i, kk_std_time_duration__duration tzdelta, kk_context_t* _ctx) {
  struct kk_std_time_calendar_earth_calendar_fun5729__t* _self = kk_function_as(struct kk_std_time_calendar_earth_calendar_fun5729__t*, _fself, _ctx);
  kk_function_t days_to_date0 = _self->days_to_date0; /* (days : int) -> std/time/date/date */
  kk_drop_match(_self, {kk_function_dup(days_to_date0, _ctx);}, {}, _ctx)
  kk_std_time_timestamp__timestamp ts_4698;
  kk_std_num_ddouble__ddouble _x5730;
  kk_std_num_ddouble__ddouble _x5731;
  {
    kk_std_time_timestamp__timestamp _x1 = i.since;
    kk_std_time_instant__timescale _pat202 = i.ts;
    struct kk_std_time_timestamp_Timestamp* _con5732 = kk_std_time_timestamp__as_Timestamp(_x1, _ctx);
    struct kk_std_time_instant_Timescale* _con5733 = kk_std_time_instant__as_Timescale(_pat202, _ctx);
    kk_std_time_timestamp__timestamp_dup(_x1, _ctx);
    {
      struct kk_std_time_timestamp_Timestamp* _con5734 = kk_std_time_timestamp__as_Timestamp(_x1, _ctx);
      kk_std_num_ddouble__ddouble _x0 = _con5734->since;
      if kk_likely(kk_datatype_ptr_is_unique(_x1, _ctx)) {
        kk_datatype_ptr_free(_x1, _ctx);
      }
      else {
        kk_datatype_ptr_decref(_x1, _ctx);
      }
      _x5731 = _x0; /*std/time/timestamp/timespan*/
    }
  }
  kk_std_num_ddouble__ddouble _x5735;
  {
    kk_std_num_ddouble__ddouble _x01 = tzdelta.secs;
    _x5735 = _x01; /*std/time/timestamp/timespan*/
  }
  _x5730 = kk_std_num_ddouble__lp__plus__rp_(_x5731, _x5735, _ctx); /*std/num/ddouble/ddouble*/
  int32_t _x5736;
  {
    kk_std_time_timestamp__timestamp _x10 = i.since;
    kk_std_time_instant__timescale _pat2020 = i.ts;
    struct kk_std_time_timestamp_Timestamp* _con5737 = kk_std_time_timestamp__as_Timestamp(_x10, _ctx);
    struct kk_std_time_instant_Timescale* _con5738 = kk_std_time_instant__as_Timescale(_pat2020, _ctx);
    kk_std_time_timestamp__timestamp_dup(_x10, _ctx);
    kk_std_time_instant__instant_drop(i, _ctx);
    {
      struct kk_std_time_timestamp_Timestamp* _con5739 = kk_std_time_timestamp__as_Timestamp(_x10, _ctx);
      kk_std_num_ddouble__ddouble _pat000 = _con5739->since;
      int32_t _x00 = _con5739->leap32;
      if kk_likely(kk_datatype_ptr_is_unique(_x10, _ctx)) {
        kk_datatype_ptr_free(_x10, _ctx);
      }
      else {
        kk_datatype_ptr_decref(_x10, _ctx);
      }
      _x5736 = _x00; /*int32*/
    }
  }
  ts_4698 = kk_std_time_timestamp__new_Timestamp(kk_reuse_null, 0, _x5730, _x5736, _ctx); /*std/time/timestamp/timestamp*/
  kk_std_core_types__tuple2_ _match_5393;
  kk_std_time_timestamp__timestamp _x5740 = kk_std_time_timestamp__timestamp_dup(ts_4698, _ctx); /*std/time/timestamp/timestamp*/
  _match_5393 = kk_std_time_timestamp_days_seconds(_x5740, _ctx); /*(int, std/num/ddouble/ddouble)*/
  {
    kk_box_t _box_x5106 = _match_5393.fst;
    kk_box_t _box_x5107 = _match_5393.snd;
    kk_std_num_ddouble__ddouble secs = kk_std_num_ddouble__ddouble_unbox(_box_x5107, KK_BORROWED, _ctx);
    kk_integer_t days0 = kk_integer_unbox(_box_x5106, _ctx);
    kk_integer_dup(days0, _ctx);
    kk_std_core_types__tuple2__drop(_match_5393, _ctx);
    kk_std_time_date__clock _match_5394;
    kk_integer_t _x5741;
    int32_t _x5742;
    {
      struct kk_std_time_timestamp_Timestamp* _con5743 = kk_std_time_timestamp__as_Timestamp(ts_4698, _ctx);
      kk_std_num_ddouble__ddouble _pat001 = _con5743->since;
      int32_t _x = _con5743->leap32;
      if kk_likely(kk_datatype_ptr_is_unique(ts_4698, _ctx)) {
        kk_datatype_ptr_free(ts_4698, _ctx);
      }
      else {
        kk_datatype_ptr_decref(ts_4698, _ctx);
      }
      _x5742 = _x; /*int32*/
    }
    _x5741 = kk_integer_from_int(_x5742,kk_context()); /*int*/
    _match_5394 = kk_std_time_date_clock_3(secs, _x5741, _ctx); /*std/time/date/clock*/
    {
      struct kk_std_time_date_Clock* _con5744 = kk_std_time_date__as_Clock(_match_5394, _ctx);
      kk_std_time_date__date _b_5111_5108 = kk_function_call(kk_std_time_date__date, (kk_function_t, kk_integer_t, kk_context_t*), days_to_date0, (days_to_date0, days0, _ctx), _ctx); /*std/time/date/date*/;
      return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_date__date_box(_b_5111_5108, _ctx), kk_std_time_date__clock_box(_match_5394, _ctx), _ctx);
    }
  }
}


// lift anonymous function
struct kk_std_time_calendar_earth_calendar_fun5746__t {
  struct kk_function_s _base;
  kk_function_t date_to_days0;
};
static kk_std_time_instant__instant kk_std_time_calendar_earth_calendar_fun5746(kk_function_t _fself, kk_std_time_date__date d00, kk_std_time_date__clock c, kk_std_time_calendar__timezone tz, kk_std_time_instant__timescale ts, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_earth_calendar_fun5746(kk_function_t date_to_days0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_earth_calendar_fun5746__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_earth_calendar_fun5746__t, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_earth_calendar_fun5746, kk_context());
  _self->date_to_days0 = date_to_days0;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_time_instant__instant kk_std_time_calendar_earth_calendar_fun5746(kk_function_t _fself, kk_std_time_date__date d00, kk_std_time_date__clock c, kk_std_time_calendar__timezone tz, kk_std_time_instant__timescale ts, kk_context_t* _ctx) {
  struct kk_std_time_calendar_earth_calendar_fun5746__t* _self = kk_function_as(struct kk_std_time_calendar_earth_calendar_fun5746__t*, _fself, _ctx);
  kk_function_t date_to_days0 = _self->date_to_days0; /* (std/time/date/date) -> int */
  kk_drop_match(_self, {kk_function_dup(date_to_days0, _ctx);}, {}, _ctx)
  bool _b_5113_5110;
  {
    struct kk_std_time_instant_Timescale* _con5747 = kk_std_time_instant__as_Timescale(ts, _ctx);
    kk_std_core_types__maybe _x100 = _con5747->mb_seconds_in_day;
    kk_std_core_types__maybe_dup(_x100, _ctx);
    if (kk_std_core_types__is_Nothing(_x100, _ctx)) {
      _b_5113_5110 = false; /*bool*/
    }
    else {
      kk_std_core_types__maybe_drop(_x100, _ctx);
      _b_5113_5110 = true; /*bool*/
    }
  }
  kk_std_time_timestamp__timestamp _x5748;
  kk_std_core_types__optional _x5749 = kk_std_core_types__new_Optional(kk_bool_box(_b_5113_5110), _ctx); /*optional<116>*/
  _x5748 = kk_std_time_calendar_dc_timestamp(d00, c, date_to_days0, _x5749, _ctx); /*std/time/timestamp/timestamp*/
  return kk_std_time_calendar_earth_timestamp_to_instant(_x5748, tz, ts, _ctx);
}

kk_std_time_calendar__calendar kk_std_time_calendar_earth_calendar(kk_string_t name0, kk_string_t long_name0, kk_function_t days_to_date0, kk_function_t date_to_days0, kk_std_core_types__optional month_prefix0, kk_std_core_types__optional show_era0, kk_context_t* _ctx) { /* (name : string, long-name : string, days-to-date : (days : int) -> std/time/date/date, date-to-days : (std/time/date/date) -> int, month-prefix : optional<string>, show-era : optional<(std/time/date/date) -> string>) -> calendar */ 
  kk_string_t _x5720;
  if (kk_std_core_types__is_Optional(month_prefix0, _ctx)) {
    kk_box_t _box_x5098 = month_prefix0._cons.Optional.value;
    kk_string_t _month_prefix_3624 = kk_string_unbox(_box_x5098);
    kk_string_dup(_month_prefix_3624, _ctx);
    kk_std_core_types__optional_drop(month_prefix0, _ctx);
    _x5720 = _month_prefix_3624; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(month_prefix0, _ctx);
    _x5720 = kk_string_empty(); /*string*/
  }
  kk_function_t _x5722;
  if (kk_std_core_types__is_Optional(show_era0, _ctx)) {
    kk_box_t _fun_unbox_x5102 = show_era0._cons.Optional.value;
    kk_box_dup(_fun_unbox_x5102, _ctx);
    kk_std_core_types__optional_drop(show_era0, _ctx);
    _x5722 = kk_std_time_calendar_new_earth_calendar_fun5723(_fun_unbox_x5102, _ctx); /*(std/time/date/date) -> string*/
  }
  else {
    kk_std_core_types__optional_drop(show_era0, _ctx);
    _x5722 = kk_std_time_calendar_new_earth_calendar_fun5726(_ctx); /*(std/time/date/date) -> string*/
  }
  kk_function_t _x5728;
  kk_function_dup(days_to_date0, _ctx);
  _x5728 = kk_std_time_calendar_new_earth_calendar_fun5729(days_to_date0, _ctx); /*(i : std/time/instant/instant, tzdelta : std/time/duration/duration) -> (6, 7)*/
  kk_function_t _x5745;
  kk_function_dup(date_to_days0, _ctx);
  _x5745 = kk_std_time_calendar_new_earth_calendar_fun5746(date_to_days0, _ctx); /*(d00 : std/time/date/date, c : std/time/date/clock, tz : std/time/calendar/timezone, ts : std/time/instant/timescale) -> std/time/instant/instant*/
  return kk_std_time_calendar__new_Calendar(kk_reuse_null, 0, name0, long_name0, _x5720, _x5722, _x5728, _x5745, days_to_date0, date_to_days0, _ctx);
}
 
// An abstraction over solar calendars. Almost all calendars can be defined
// in terms of this function (and `solar-ecalendar`). Given functions to
// convert days to and from a year, and day-of-the-year to and from month
// and month day, this constructs a full calendar.
// Also needs an `epoch-shift` that gives the number of days from the
// calendar epoch to the date ``2000-01-01``. By default this is `730119`
// which corresponds to a ``0001-01-01`` ISO calendar epoch. The
// `has-year-zero` (=`True`) parameter is used when the calendar has a
// year zero. The `month-prefix` (=`""`) is used for displaying numeric
// months (and is `"W"` for `cal-iso-week` for example). The `show-era` function
// can be given to display an era for a given date. Finally the `ts` (=`ts-ti`)
// gives the time scale for the calendar.


// lift anonymous function
struct kk_std_time_calendar_solar_calendar_fun5751__t {
  struct kk_function_s _base;
  kk_function_t days_before_month;
  kk_function_t days_to_month;
  kk_function_t days_to_yeardoy;
  kk_std_core_types__optional epoch_shift;
  kk_std_core_types__optional has_year_zero;
};
static kk_std_time_date__date kk_std_time_calendar_solar_calendar_fun5751(kk_function_t _fself, kk_integer_t days0, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_solar_calendar_fun5751(kk_function_t days_before_month, kk_function_t days_to_month, kk_function_t days_to_yeardoy, kk_std_core_types__optional epoch_shift, kk_std_core_types__optional has_year_zero, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun5751__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_solar_calendar_fun5751__t, 8, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_solar_calendar_fun5751, kk_context());
  _self->days_before_month = days_before_month;
  _self->days_to_month = days_to_month;
  _self->days_to_yeardoy = days_to_yeardoy;
  _self->epoch_shift = epoch_shift;
  _self->has_year_zero = has_year_zero;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_time_date__date kk_std_time_calendar_solar_calendar_fun5751(kk_function_t _fself, kk_integer_t days0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun5751__t* _self = kk_function_as(struct kk_std_time_calendar_solar_calendar_fun5751__t*, _fself, _ctx);
  kk_function_t days_before_month = _self->days_before_month; /* (year : int, month : int) -> int */
  kk_function_t days_to_month = _self->days_to_month; /* (year : int, doy : int) -> int */
  kk_function_t days_to_yeardoy = _self->days_to_yeardoy; /* (days : int) -> (int, int) */
  kk_std_core_types__optional epoch_shift = _self->epoch_shift; /* optional<int> */
  kk_std_core_types__optional has_year_zero = _self->has_year_zero; /* optional<bool> */
  kk_drop_match(_self, {kk_function_dup(days_before_month, _ctx);kk_function_dup(days_to_month, _ctx);kk_function_dup(days_to_yeardoy, _ctx);kk_std_core_types__optional_dup(epoch_shift, _ctx);kk_std_core_types__optional_dup(has_year_zero, _ctx);}, {}, _ctx)
  kk_integer_t days;
  kk_integer_t _x5752;
  if (kk_std_core_types__is_Optional(epoch_shift, _ctx)) {
    kk_box_t _box_x5114 = epoch_shift._cons.Optional.value;
    kk_integer_t _epoch_shift_3691 = kk_integer_unbox(_box_x5114, _ctx);
    kk_integer_dup(_epoch_shift_3691, _ctx);
    kk_std_core_types__optional_drop(epoch_shift, _ctx);
    _x5752 = _epoch_shift_3691; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(epoch_shift, _ctx);
    _x5752 = kk_integer_from_int(730119, _ctx); /*int*/
  }
  days = kk_integer_add(days0,_x5752,kk_context()); /*int*/
  kk_std_core_types__tuple2_ _match_5391 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_integer_t, kk_context_t*), days_to_yeardoy, (days_to_yeardoy, days, _ctx), _ctx); /*(int, int)*/;
  {
    kk_box_t _box_x5115 = _match_5391.fst;
    kk_box_t _box_x5116 = _match_5391.snd;
    kk_integer_t y0 = kk_integer_unbox(_box_x5115, _ctx);
    kk_integer_t doy = kk_integer_unbox(_box_x5116, _ctx);
    kk_integer_dup(doy, _ctx);
    kk_integer_dup(y0, _ctx);
    kk_std_core_types__tuple2__drop(_match_5391, _ctx);
    kk_integer_t month;
    kk_integer_t _x5753 = kk_integer_dup(y0, _ctx); /*int*/
    kk_integer_t _x5754 = kk_integer_dup(doy, _ctx); /*int*/
    month = kk_function_call(kk_integer_t, (kk_function_t, kk_integer_t, kk_integer_t, kk_context_t*), days_to_month, (days_to_month, _x5753, _x5754, _ctx), _ctx); /*int*/
    kk_integer_t y2_4775;
    kk_integer_t _x5755 = kk_integer_dup(y0, _ctx); /*int*/
    kk_integer_t _x5756 = kk_integer_dup(month, _ctx); /*int*/
    y2_4775 = kk_function_call(kk_integer_t, (kk_function_t, kk_integer_t, kk_integer_t, kk_context_t*), days_before_month, (days_before_month, _x5755, _x5756, _ctx), _ctx); /*int*/
    kk_integer_t x0_4772 = kk_integer_sub(doy,y2_4775,kk_context()); /*int*/;
    kk_integer_t day = kk_integer_add_small_const(x0_4772, 1, _ctx); /*int*/;
    kk_integer_t year;
    if (kk_std_core_types__is_Optional(has_year_zero, _ctx)) {
      kk_box_t _box_x5117 = has_year_zero._cons.Optional.value;
      bool _has_year_zero_3695 = kk_bool_unbox(_box_x5117);
      kk_std_core_types__optional_drop(has_year_zero, _ctx);
      if (_has_year_zero_3695) {
        year = y0; /*int*/
      }
      else {
        bool _match_5392 = kk_integer_gt_borrow(y0,(kk_integer_from_small(0)),kk_context()); /*bool*/;
        if (_match_5392) {
          year = y0; /*int*/
        }
        else {
          year = kk_integer_add_small_const(y0, -1, _ctx); /*int*/
        }
      }
    }
    else {
      kk_std_core_types__optional_drop(has_year_zero, _ctx);
      year = y0; /*int*/
    }
    return kk_std_time_date__new_Date(year, month, day, _ctx);
  }
}


// lift anonymous function
struct kk_std_time_calendar_solar_calendar_fun5757__t {
  struct kk_function_s _base;
  kk_function_t days_before_month;
  kk_function_t days_before_year;
  kk_std_core_types__optional epoch_shift;
  kk_std_core_types__optional has_year_zero;
};
static kk_integer_t kk_std_time_calendar_solar_calendar_fun5757(kk_function_t _fself, kk_std_time_date__date d0, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_solar_calendar_fun5757(kk_function_t days_before_month, kk_function_t days_before_year, kk_std_core_types__optional epoch_shift, kk_std_core_types__optional has_year_zero, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun5757__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_solar_calendar_fun5757__t, 7, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_solar_calendar_fun5757, kk_context());
  _self->days_before_month = days_before_month;
  _self->days_before_year = days_before_year;
  _self->epoch_shift = epoch_shift;
  _self->has_year_zero = has_year_zero;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_integer_t kk_std_time_calendar_solar_calendar_fun5757(kk_function_t _fself, kk_std_time_date__date d0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun5757__t* _self = kk_function_as(struct kk_std_time_calendar_solar_calendar_fun5757__t*, _fself, _ctx);
  kk_function_t days_before_month = _self->days_before_month; /* (year : int, month : int) -> int */
  kk_function_t days_before_year = _self->days_before_year; /* (year : int) -> int */
  kk_std_core_types__optional epoch_shift = _self->epoch_shift; /* optional<int> */
  kk_std_core_types__optional has_year_zero = _self->has_year_zero; /* optional<bool> */
  kk_drop_match(_self, {kk_function_dup(days_before_month, _ctx);kk_function_dup(days_before_year, _ctx);kk_std_core_types__optional_dup(epoch_shift, _ctx);kk_std_core_types__optional_dup(has_year_zero, _ctx);}, {}, _ctx)
  kk_integer_t y3;
  if (kk_std_core_types__is_Optional(has_year_zero, _ctx)) {
    kk_box_t _box_x5118 = has_year_zero._cons.Optional.value;
    bool _has_year_zero_36950 = kk_bool_unbox(_box_x5118);
    kk_std_core_types__optional_drop(has_year_zero, _ctx);
    if (_has_year_zero_36950) {
      kk_integer_t _x = d0.year;
      kk_integer_dup(_x, _ctx);
      y3 = _x; /*int*/
    }
    else {
      bool _match_5388;
      kk_integer_t _brw_5389;
      {
        kk_integer_t _x0 = d0.year;
        kk_integer_dup(_x0, _ctx);
        _brw_5389 = _x0; /*int*/
      }
      bool _brw_5390 = kk_integer_gt_borrow(_brw_5389,(kk_integer_from_small(0)),kk_context()); /*bool*/;
      kk_integer_drop(_brw_5389, _ctx);
      _match_5388 = _brw_5390; /*bool*/
      if (_match_5388) {
        kk_integer_t _x1 = d0.year;
        kk_integer_dup(_x1, _ctx);
        y3 = _x1; /*int*/
      }
      else {
        kk_integer_t _x5758;
        {
          kk_integer_t _x2 = d0.year;
          kk_integer_dup(_x2, _ctx);
          _x5758 = _x2; /*int*/
        }
        y3 = kk_integer_add_small_const(_x5758, 1, _ctx); /*int*/
      }
    }
  }
  else {
    kk_std_core_types__optional_drop(has_year_zero, _ctx);
    {
      kk_integer_t _x3 = d0.year;
      kk_integer_dup(_x3, _ctx);
      y3 = _x3; /*int*/
    }
  }
  kk_integer_t x4_4788;
  kk_integer_t _x5759 = kk_integer_dup(y3, _ctx); /*int*/
  x4_4788 = kk_function_call(kk_integer_t, (kk_function_t, kk_integer_t, kk_context_t*), days_before_year, (days_before_year, _x5759, _ctx), _ctx); /*int*/
  kk_integer_t y6_4789;
  kk_integer_t _x5760;
  {
    kk_integer_t _x30 = d0.month;
    kk_integer_dup(_x30, _ctx);
    _x5760 = _x30; /*int*/
  }
  y6_4789 = kk_function_call(kk_integer_t, (kk_function_t, kk_integer_t, kk_integer_t, kk_context_t*), days_before_month, (days_before_month, y3, _x5760, _ctx), _ctx); /*int*/
  kk_integer_t x3_4786 = kk_integer_add(x4_4788,y6_4789,kk_context()); /*int*/;
  kk_integer_t y5_4787;
  kk_integer_t _x5761;
  {
    kk_integer_t _x4 = d0.day;
    kk_integer_dup(_x4, _ctx);
    kk_std_time_date__date_drop(d0, _ctx);
    _x5761 = _x4; /*int*/
  }
  y5_4787 = kk_integer_add_small_const(_x5761, -1, _ctx); /*int*/
  kk_integer_t x2_4784 = kk_integer_add(x3_4786,y5_4787,kk_context()); /*int*/;
  kk_integer_t _x5762;
  if (kk_std_core_types__is_Optional(epoch_shift, _ctx)) {
    kk_box_t _box_x5119 = epoch_shift._cons.Optional.value;
    kk_integer_t _epoch_shift_36910 = kk_integer_unbox(_box_x5119, _ctx);
    kk_integer_dup(_epoch_shift_36910, _ctx);
    kk_std_core_types__optional_drop(epoch_shift, _ctx);
    _x5762 = _epoch_shift_36910; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(epoch_shift, _ctx);
    _x5762 = kk_integer_from_int(730119, _ctx); /*int*/
  }
  return kk_integer_sub(x2_4784,_x5762,kk_context());
}


// lift anonymous function
struct kk_std_time_calendar_solar_calendar_fun5768__t {
  struct kk_function_s _base;
  kk_std_core_types__optional show_era0;
};
static kk_box_t kk_std_time_calendar_solar_calendar_fun5768(kk_function_t _fself, kk_box_t _b_5132, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_solar_calendar_fun5768(kk_std_core_types__optional show_era0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun5768__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_solar_calendar_fun5768__t, 3, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_solar_calendar_fun5768, kk_context());
  _self->show_era0 = show_era0;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_box_t kk_std_time_calendar_solar_calendar_fun5768(kk_function_t _fself, kk_box_t _b_5132, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun5768__t* _self = kk_function_as(struct kk_std_time_calendar_solar_calendar_fun5768__t*, _fself, _ctx);
  kk_std_core_types__optional show_era0 = _self->show_era0; /* optional<(std/time/date/date) -> string> */
  kk_drop_match(_self, {kk_std_core_types__optional_dup(show_era0, _ctx);}, {}, _ctx)
  kk_string_t _x5769;
  kk_std_time_date__date _norm_5135 = kk_std_time_date__date_unbox(_b_5132, KK_OWNED, _ctx); /*std/time/date/date*/;
  if (kk_std_core_types__is_Optional(show_era0, _ctx)) {
    kk_box_t _fun_unbox_x5125 = show_era0._cons.Optional.value;
    kk_box_dup(_fun_unbox_x5125, _ctx);
    kk_std_core_types__optional_drop(show_era0, _ctx);
    kk_std_time_date__date _b_5136_5128 = _norm_5135; /*std/time/date/date*/;
    kk_box_t _x5770;
    kk_function_t _x5771 = kk_function_unbox(_fun_unbox_x5125, _ctx); /*(5126) -> 5127*/
    _x5770 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x5771, (_x5771, kk_std_time_date__date_box(_b_5136_5128, _ctx), _ctx), _ctx); /*5127*/
    _x5769 = kk_string_unbox(_x5770); /*string*/
  }
  else {
    kk_std_core_types__optional_drop(show_era0, _ctx);
    kk_std_time_date__date d_5137 = _norm_5135; /*std/time/date/date*/;
    kk_std_time_date__date_drop(d_5137, _ctx);
    _x5769 = kk_string_empty(); /*string*/
  }
  return kk_string_box(_x5769);
}

kk_std_time_calendar__calendar kk_std_time_calendar_solar_calendar(kk_string_t name0, kk_string_t long_name0, kk_function_t days_before_year, kk_function_t days_to_yeardoy, kk_function_t days_before_month, kk_function_t days_to_month, kk_std_core_types__optional epoch_shift, kk_std_core_types__optional has_year_zero, kk_std_core_types__optional month_prefix0, kk_std_core_types__optional show_era0, kk_context_t* _ctx) { /* (name : string, long-name : string, days-before-year : (year : int) -> int, days-to-yeardoy : (days : int) -> (int, int), days-before-month : (year : int, month : int) -> int, days-to-month : (year : int, doy : int) -> int, epoch-shift : optional<int>, has-year-zero : optional<bool>, month-prefix : optional<string>, show-era : optional<(std/time/date/date) -> string>) -> calendar */ 
  kk_function_t _x5750;
  kk_function_dup(days_before_month, _ctx);
  kk_std_core_types__optional_dup(epoch_shift, _ctx);
  kk_std_core_types__optional_dup(has_year_zero, _ctx);
  _x5750 = kk_std_time_calendar_new_solar_calendar_fun5751(days_before_month, days_to_month, days_to_yeardoy, epoch_shift, has_year_zero, _ctx); /*(days0 : int) -> std/time/date/date*/
  kk_std_core_types__optional _x5763;
  kk_box_t _x5764;
  kk_string_t _x5765;
  if (kk_std_core_types__is_Optional(month_prefix0, _ctx)) {
    kk_box_t _box_x5120 = month_prefix0._cons.Optional.value;
    kk_string_t _month_prefix_3699 = kk_string_unbox(_box_x5120);
    kk_string_dup(_month_prefix_3699, _ctx);
    kk_std_core_types__optional_drop(month_prefix0, _ctx);
    _x5765 = _month_prefix_3699; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(month_prefix0, _ctx);
    _x5765 = kk_string_empty(); /*string*/
  }
  _x5764 = kk_string_box(_x5765); /*116*/
  _x5763 = kk_std_core_types__new_Optional(_x5764, _ctx); /*optional<116>*/
  kk_std_core_types__optional _x5767 = kk_std_core_types__new_Optional(kk_function_box(kk_std_time_calendar_new_solar_calendar_fun5768(show_era0, _ctx), _ctx), _ctx); /*optional<116>*/
  return kk_std_time_calendar_earth_calendar(name0, long_name0, _x5750, kk_std_time_calendar_new_solar_calendar_fun5757(days_before_month, days_before_year, epoch_shift, has_year_zero, _ctx), _x5763, _x5767, _ctx);
}
 
// An abstraction over solar calendars. Almost all calendars can be defined
// in terms of this function (and `solar-calendar`). Given functions to
// convert days to and from a year, and day-of-the-year to and from month
// and month day, this constructs a full calendar. In contrast to `solar-calendar`
// this function just needs an estimation function from days to a year -- for most
// calendars this is much more convenient. The estimation function returns a
// conservative (i.e. lowest) estimate for the year and a `maybe<int>` that
// is the `safe-day`: if the day of the year is lower or equal to that the estimate
// is surely correct. Otherwise the wrapper will calculate the start day for the next
// year until the correct year is found.
// Also needs an `epoch-shift` that gives the number of days from the
// calendar epoch to the date ``2000-01-01``. By default this is `730119`
// which corresponds to a ``0001-01-01`` ISO calendar epoch. The
// `has-year-zero` (=`True`) parameter is used when the calendar has a
// year zero. The `month-prefix` (=`""`) is used for displaying numeric
// months (and is `"W"` for `cal-iso-week` for example). The `show-era` function
// can be given to display an era for a given date. Finally the `ts` (=`ts-utc`)
// gives the time scale for the calendar.


// lift anonymous function
struct kk_std_time_calendar_solar_ecalendar_fun5774__t {
  struct kk_function_s _base;
  kk_function_t days_before_year;
  kk_function_t estimate_year;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_solar_ecalendar_fun5774(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_solar_ecalendar_fun5774(kk_function_t days_before_year, kk_function_t estimate_year, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_ecalendar_fun5774__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_solar_ecalendar_fun5774__t, 3, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_solar_ecalendar_fun5774, kk_context());
  _self->days_before_year = days_before_year;
  _self->estimate_year = estimate_year;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_solar_ecalendar_fun5774(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_ecalendar_fun5774__t* _self = kk_function_as(struct kk_std_time_calendar_solar_ecalendar_fun5774__t*, _fself, _ctx);
  kk_function_t days_before_year = _self->days_before_year; /* (year : int) -> int */
  kk_function_t estimate_year = _self->estimate_year; /* (days : int) -> (int, maybe<int>) */
  kk_drop_match(_self, {kk_function_dup(days_before_year, _ctx);kk_function_dup(estimate_year, _ctx);}, {}, _ctx)
  kk_std_core_types__tuple2_ _match_5383;
  kk_integer_t _x5775 = kk_integer_dup(days, _ctx); /*int*/
  _match_5383 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_integer_t, kk_context_t*), estimate_year, (estimate_year, _x5775, _ctx), _ctx); /*(int, maybe<int>)*/
  {
    kk_box_t _box_x5138 = _match_5383.fst;
    kk_box_t _box_x5139 = _match_5383.snd;
    kk_integer_t approx = kk_integer_unbox(_box_x5138, _ctx);
    kk_std_core_types__maybe safe = kk_std_core_types__maybe_unbox(_box_x5139, KK_BORROWED, _ctx);
    kk_integer_dup(approx, _ctx);
    kk_std_core_types__maybe_dup(safe, _ctx);
    kk_std_core_types__tuple2__drop(_match_5383, _ctx);
    kk_integer_t y_4794;
    kk_function_t _x5777 = kk_function_dup(days_before_year, _ctx); /*(year : int) -> int*/
    kk_integer_t _x5776 = kk_integer_dup(approx, _ctx); /*int*/
    y_4794 = kk_function_call(kk_integer_t, (kk_function_t, kk_integer_t, kk_context_t*), _x5777, (_x5777, _x5776, _ctx), _ctx); /*int*/
    kk_integer_t doy1;
    kk_integer_t _x5778 = kk_integer_dup(days, _ctx); /*int*/
    doy1 = kk_integer_sub(_x5778,y_4794,kk_context()); /*int*/
    bool _match_5384;
    kk_integer_t _brw_5386;
    if (kk_std_core_types__is_Nothing(safe, _ctx)) {
      _brw_5386 = kk_integer_from_small(0); /*int*/
    }
    else {
      kk_box_t _box_x5140 = safe._cons.Just.value;
      kk_integer_t x0 = kk_integer_unbox(_box_x5140, _ctx);
      kk_integer_dup(x0, _ctx);
      kk_std_core_types__maybe_drop(safe, _ctx);
      _brw_5386 = x0; /*int*/
    }
    bool _brw_5387 = kk_integer_lte_borrow(doy1,_brw_5386,kk_context()); /*bool*/;
    kk_integer_drop(_brw_5386, _ctx);
    _match_5384 = _brw_5387; /*bool*/
    if (_match_5384) {
      kk_function_drop(days_before_year, _ctx);
      kk_integer_drop(days, _ctx);
      return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(approx, _ctx), kk_integer_box(doy1, _ctx), _ctx);
    }
    {
      kk_integer_t y0_4798;
      kk_integer_t _x5779;
      kk_integer_t _x5780 = kk_integer_dup(approx, _ctx); /*int*/
      _x5779 = kk_integer_add_small_const(_x5780, 1, _ctx); /*int*/
      y0_4798 = kk_function_call(kk_integer_t, (kk_function_t, kk_integer_t, kk_context_t*), days_before_year, (days_before_year, _x5779, _ctx), _ctx); /*int*/
      kk_integer_t doy2 = kk_integer_sub(days,y0_4798,kk_context()); /*int*/;
      bool _match_5385 = kk_integer_lt_borrow(doy2,(kk_integer_from_small(0)),kk_context()); /*bool*/;
      if (_match_5385) {
        kk_integer_drop(doy2, _ctx);
        return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(approx, _ctx), kk_integer_box(doy1, _ctx), _ctx);
      }
      {
        kk_integer_drop(doy1, _ctx);
        kk_integer_t _b_5168_5145 = kk_integer_add_small_const(approx, 1, _ctx); /*int*/;
        return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5168_5145, _ctx), kk_integer_box(doy2, _ctx), _ctx);
      }
    }
  }
}


// lift anonymous function
struct kk_std_time_calendar_solar_ecalendar_fun5792__t {
  struct kk_function_s _base;
  kk_std_core_types__optional show_era0;
};
static kk_box_t kk_std_time_calendar_solar_ecalendar_fun5792(kk_function_t _fself, kk_box_t _b_5163, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_solar_ecalendar_fun5792(kk_std_core_types__optional show_era0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_ecalendar_fun5792__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_solar_ecalendar_fun5792__t, 3, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_solar_ecalendar_fun5792, kk_context());
  _self->show_era0 = show_era0;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_box_t kk_std_time_calendar_solar_ecalendar_fun5792(kk_function_t _fself, kk_box_t _b_5163, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_ecalendar_fun5792__t* _self = kk_function_as(struct kk_std_time_calendar_solar_ecalendar_fun5792__t*, _fself, _ctx);
  kk_std_core_types__optional show_era0 = _self->show_era0; /* optional<(std/time/date/date) -> string> */
  kk_drop_match(_self, {kk_std_core_types__optional_dup(show_era0, _ctx);}, {}, _ctx)
  kk_string_t _x5793;
  kk_std_time_date__date _norm_5174 = kk_std_time_date__date_unbox(_b_5163, KK_OWNED, _ctx); /*std/time/date/date*/;
  if (kk_std_core_types__is_Optional(show_era0, _ctx)) {
    kk_box_t _fun_unbox_x5156 = show_era0._cons.Optional.value;
    kk_box_dup(_fun_unbox_x5156, _ctx);
    kk_std_core_types__optional_drop(show_era0, _ctx);
    kk_std_time_date__date _b_5175_5159 = _norm_5174; /*std/time/date/date*/;
    kk_box_t _x5794;
    kk_function_t _x5795 = kk_function_unbox(_fun_unbox_x5156, _ctx); /*(5157) -> 5158*/
    _x5794 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x5795, (_x5795, kk_std_time_date__date_box(_b_5175_5159, _ctx), _ctx), _ctx); /*5158*/
    _x5793 = kk_string_unbox(_x5794); /*string*/
  }
  else {
    kk_std_core_types__optional_drop(show_era0, _ctx);
    kk_std_time_date__date d_5176 = _norm_5174; /*std/time/date/date*/;
    kk_std_time_date__date_drop(d_5176, _ctx);
    _x5793 = kk_string_empty(); /*string*/
  }
  return kk_string_box(_x5793);
}

kk_std_time_calendar__calendar kk_std_time_calendar_solar_ecalendar(kk_string_t name0, kk_string_t long_name0, kk_function_t days_before_year, kk_function_t estimate_year, kk_function_t days_before_month, kk_function_t days_to_month, kk_std_core_types__optional epoch_shift, kk_std_core_types__optional has_year_zero, kk_std_core_types__optional month_prefix0, kk_std_core_types__optional show_era0, kk_context_t* _ctx) { /* (name : string, long-name : string, days-before-year : (year : int) -> int, estimate-year : (days : int) -> (int, maybe<int>), days-before-month : (year : int, month : int) -> int, days-to-month : (year : int, doy : int) -> int, epoch-shift : optional<int>, has-year-zero : optional<bool>, month-prefix : optional<string>, show-era : optional<(std/time/date/date) -> string>) -> calendar */ 
  kk_function_t _x5773 = kk_function_dup(days_before_year, _ctx); /*(year : int) -> int*/
  kk_std_core_types__optional _x5781;
  kk_box_t _x5782;
  kk_integer_t _x5783;
  if (kk_std_core_types__is_Optional(epoch_shift, _ctx)) {
    kk_box_t _box_x5147 = epoch_shift._cons.Optional.value;
    kk_integer_t _epoch_shift_2777 = kk_integer_unbox(_box_x5147, _ctx);
    kk_integer_dup(_epoch_shift_2777, _ctx);
    kk_std_core_types__optional_drop(epoch_shift, _ctx);
    _x5783 = _epoch_shift_2777; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(epoch_shift, _ctx);
    _x5783 = kk_integer_from_int(730119, _ctx); /*int*/
  }
  _x5782 = kk_integer_box(_x5783, _ctx); /*116*/
  _x5781 = kk_std_core_types__new_Optional(_x5782, _ctx); /*optional<116>*/
  kk_std_core_types__optional _x5784;
  kk_box_t _x5785;
  bool _x5786;
  if (kk_std_core_types__is_Optional(has_year_zero, _ctx)) {
    kk_box_t _box_x5149 = has_year_zero._cons.Optional.value;
    bool _has_year_zero_2781 = kk_bool_unbox(_box_x5149);
    kk_std_core_types__optional_drop(has_year_zero, _ctx);
    _x5786 = _has_year_zero_2781; /*bool*/
  }
  else {
    kk_std_core_types__optional_drop(has_year_zero, _ctx);
    _x5786 = true; /*bool*/
  }
  _x5785 = kk_bool_box(_x5786); /*116*/
  _x5784 = kk_std_core_types__new_Optional(_x5785, _ctx); /*optional<116>*/
  kk_std_core_types__optional _x5787;
  kk_box_t _x5788;
  kk_string_t _x5789;
  if (kk_std_core_types__is_Optional(month_prefix0, _ctx)) {
    kk_box_t _box_x5151 = month_prefix0._cons.Optional.value;
    kk_string_t _month_prefix_2785 = kk_string_unbox(_box_x5151);
    kk_string_dup(_month_prefix_2785, _ctx);
    kk_std_core_types__optional_drop(month_prefix0, _ctx);
    _x5789 = _month_prefix_2785; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(month_prefix0, _ctx);
    _x5789 = kk_string_empty(); /*string*/
  }
  _x5788 = kk_string_box(_x5789); /*116*/
  _x5787 = kk_std_core_types__new_Optional(_x5788, _ctx); /*optional<116>*/
  kk_std_core_types__optional _x5791 = kk_std_core_types__new_Optional(kk_function_box(kk_std_time_calendar_new_solar_ecalendar_fun5792(show_era0, _ctx), _ctx), _ctx); /*optional<116>*/
  return kk_std_time_calendar_solar_calendar(name0, long_name0, _x5773, kk_std_time_calendar_new_solar_ecalendar_fun5774(days_before_year, estimate_year, _ctx), days_before_month, days_to_month, _x5781, _x5784, _x5787, _x5791, _ctx);
}
 
// Create a standard ISO calendar using a particular time scale
// and calendar `name` (=`ts.name`).


// lift anonymous function
struct kk_std_time_calendar_iso_calendar_fun5800__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_iso_calendar_fun5800(kk_function_t _fself, kk_integer_t _x15799, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_iso_calendar_fun5800(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_iso_calendar_fun5800, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_integer_t kk_std_time_calendar_iso_calendar_fun5800(kk_function_t _fself, kk_integer_t _x15799, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_days_before_year(_x15799, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_iso_calendar_fun5802__t {
  struct kk_function_s _base;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_iso_calendar_fun5802(kk_function_t _fself, kk_integer_t _x15801, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_iso_calendar_fun5802(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_iso_calendar_fun5802, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_iso_calendar_fun5802(kk_function_t _fself, kk_integer_t _x15801, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_estimate_year(_x15801, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_iso_calendar_fun5805__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_iso_calendar_fun5805(kk_function_t _fself, kk_integer_t _x15803, kk_integer_t _x25804, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_iso_calendar_fun5805(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_iso_calendar_fun5805, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_integer_t kk_std_time_calendar_iso_calendar_fun5805(kk_function_t _fself, kk_integer_t _x15803, kk_integer_t _x25804, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_days_before_month(_x15803, _x25804, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_iso_calendar_fun5808__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_iso_calendar_fun5808(kk_function_t _fself, kk_integer_t _x15806, kk_integer_t _x25807, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_iso_calendar_fun5808(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_iso_calendar_fun5808, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_integer_t kk_std_time_calendar_iso_calendar_fun5808(kk_function_t _fself, kk_integer_t _x15806, kk_integer_t _x25807, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_doy_to_month(_x15806, _x25807, _ctx);
}

kk_std_time_calendar__calendar kk_std_time_calendar_iso_calendar(kk_std_time_instant__timescale ___wildcard__422__23, kk_string_t name0, kk_std_core_types__optional long_name0, kk_context_t* _ctx) { /* (std/time/instant/timescale, name : string, long-name : optional<string>) -> calendar */ 
  kk_datatype_ptr_dropn(___wildcard__422__23, (KK_I32(10)), _ctx);
  kk_string_t _x5797 = kk_string_dup(name0, _ctx); /*string*/
  kk_string_t _x5798;
  if (kk_std_core_types__is_Optional(long_name0, _ctx)) {
    kk_box_t _box_x5177 = long_name0._cons.Optional.value;
    kk_string_t _long_name_2969 = kk_string_unbox(_box_x5177);
    kk_string_drop(name0, _ctx);
    kk_string_dup(_long_name_2969, _ctx);
    kk_std_core_types__optional_drop(long_name0, _ctx);
    _x5798 = _long_name_2969; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(long_name0, _ctx);
    _x5798 = name0; /*string*/
  }
  return kk_std_time_calendar_solar_ecalendar(_x5797, _x5798, kk_std_time_calendar_new_iso_calendar_fun5800(_ctx), kk_std_time_calendar_new_iso_calendar_fun5802(_ctx), kk_std_time_calendar_new_iso_calendar_fun5805(_ctx), kk_std_time_calendar_new_iso_calendar_fun5808(_ctx), kk_std_core_types__new_None(_ctx), kk_std_core_types__new_None(_ctx), kk_std_core_types__new_None(_ctx), kk_std_core_types__new_None(_ctx), _ctx);
}
 
// The standard [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) calendar
// using TI time (`ts-ti`). This is a proleptic Gregorian
// calendar except that it uses the year 0 for 1 BCE, -1 for 2 BCE etc.
//
// This is the default calendar used in the library as it guarantees deterministic
// date calculations while still taking historical leap seconds into account.


// lift anonymous function
struct kk_std_time_calendar_cal_iso_fun5814__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_iso_fun5814(kk_function_t _fself, kk_integer_t _x15813, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_fun5814(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_iso_fun5814, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_integer_t kk_std_time_calendar_cal_iso_fun5814(kk_function_t _fself, kk_integer_t _x15813, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_days_before_year(_x15813, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_iso_fun5816__t {
  struct kk_function_s _base;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_cal_iso_fun5816(kk_function_t _fself, kk_integer_t _x15815, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_fun5816(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_iso_fun5816, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_cal_iso_fun5816(kk_function_t _fself, kk_integer_t _x15815, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_estimate_year(_x15815, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_iso_fun5819__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_iso_fun5819(kk_function_t _fself, kk_integer_t _x15817, kk_integer_t _x25818, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_fun5819(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_iso_fun5819, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_integer_t kk_std_time_calendar_cal_iso_fun5819(kk_function_t _fself, kk_integer_t _x15817, kk_integer_t _x25818, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_days_before_month(_x15817, _x25818, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_iso_fun5822__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_iso_fun5822(kk_function_t _fself, kk_integer_t _x15820, kk_integer_t _x25821, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_fun5822(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_iso_fun5822, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_integer_t kk_std_time_calendar_cal_iso_fun5822(kk_function_t _fself, kk_integer_t _x15820, kk_integer_t _x25821, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_doy_to_month(_x15820, _x25821, _ctx);
}

kk_std_time_calendar__calendar kk_std_time_calendar_cal_iso;
 
// Return the instant in time for a given `:date` and `:clock` (= `clock0`) interpreted by
// calendar `cal` (=`cal-iso`) in a timezone `tz` (=`tz-utc` by default).

kk_std_time_instant__instant kk_std_time_calendar_instant(kk_std_time_instant__timescale ts, kk_std_time_date__date d, kk_std_core_types__optional c, kk_std_core_types__optional tz, kk_std_core_types__optional cal, kk_context_t* _ctx) { /* (ts : std/time/instant/timescale, d : std/time/date/date, c : optional<std/time/date/clock>, tz : optional<timezone>, cal : optional<calendar>) -> std/time/instant/instant */ 
  kk_std_time_calendar__calendar _match_5382;
  if (kk_std_core_types__is_Optional(cal, _ctx)) {
    kk_box_t _box_x5178 = cal._cons.Optional.value;
    kk_std_time_calendar__calendar _cal_3011 = kk_std_time_calendar__calendar_unbox(_box_x5178, KK_BORROWED, _ctx);
    kk_std_time_calendar__calendar_dup(_cal_3011, _ctx);
    kk_std_core_types__optional_drop(cal, _ctx);
    _match_5382 = _cal_3011; /*std/time/calendar/calendar*/
  }
  else {
    kk_std_core_types__optional_drop(cal, _ctx);
    _match_5382 = kk_std_time_calendar__calendar_dup(kk_std_time_calendar_cal_iso, _ctx); /*std/time/calendar/calendar*/
  }
  {
    struct kk_std_time_calendar_Calendar* _con5823 = kk_std_time_calendar__as_Calendar(_match_5382, _ctx);
    kk_string_t _pat0 = _con5823->name;
    kk_string_t _pat1 = _con5823->long_name;
    kk_string_t _pat2 = _con5823->month_prefix;
    kk_function_t _pat3 = _con5823->show_era;
    kk_function_t _pat4 = _con5823->instant_to_dc;
    kk_function_t _x = _con5823->dc_to_instant;
    kk_function_t _pat5 = _con5823->days_to_date;
    kk_function_t _pat6 = _con5823->date_to_days;
    if kk_likely(kk_datatype_ptr_is_unique(_match_5382, _ctx)) {
      kk_function_drop(_pat6, _ctx);
      kk_function_drop(_pat5, _ctx);
      kk_function_drop(_pat4, _ctx);
      kk_function_drop(_pat3, _ctx);
      kk_string_drop(_pat2, _ctx);
      kk_string_drop(_pat1, _ctx);
      kk_string_drop(_pat0, _ctx);
      kk_datatype_ptr_free(_match_5382, _ctx);
    }
    else {
      kk_function_dup(_x, _ctx);
      kk_datatype_ptr_decref(_match_5382, _ctx);
    }
    kk_std_time_date__clock _x5824;
    if (kk_std_core_types__is_Optional(c, _ctx)) {
      kk_box_t _box_x5179 = c._cons.Optional.value;
      kk_std_time_date__clock _c_3003 = kk_std_time_date__clock_unbox(_box_x5179, KK_BORROWED, _ctx);
      kk_std_time_date__clock_dup(_c_3003, _ctx);
      kk_std_core_types__optional_drop(c, _ctx);
      _x5824 = _c_3003; /*std/time/date/clock*/
    }
    else {
      kk_std_core_types__optional_drop(c, _ctx);
      _x5824 = kk_std_time_date__clock_dup(kk_std_time_date_clock0, _ctx); /*std/time/date/clock*/
    }
    kk_std_time_calendar__timezone _x5825;
    if (kk_std_core_types__is_Optional(tz, _ctx)) {
      kk_box_t _box_x5180 = tz._cons.Optional.value;
      kk_std_time_calendar__timezone _tz_3007 = kk_std_time_calendar__timezone_unbox(_box_x5180, KK_BORROWED, _ctx);
      kk_std_time_calendar__timezone_dup(_tz_3007, _ctx);
      kk_std_core_types__optional_drop(tz, _ctx);
      _x5825 = _tz_3007; /*std/time/calendar/timezone*/
    }
    else {
      kk_std_core_types__optional_drop(tz, _ctx);
      _x5825 = kk_std_time_calendar__timezone_dup(kk_std_time_calendar_tz_utc, _ctx); /*std/time/calendar/timezone*/
    }
    return kk_function_call(kk_std_time_instant__instant, (kk_function_t, kk_std_time_date__date, kk_std_time_date__clock, kk_std_time_calendar__timezone, kk_std_time_instant__timescale, kk_context_t*), _x, (_x, d, _x5824, _x5825, ts, _ctx), _ctx);
  }
}
 
// monadic lift


// lift anonymous function
struct kk_std_time_calendar__mlift4909_instant_fun5827__t_1 {
  struct kk_function_s _base;
  kk_std_time_instant__timescale _c_4884;
  kk_std_core_types__optional c;
  kk_std_core_types__optional cal;
  kk_std_time_date__date d;
  kk_std_core_types__optional tz;
};
static kk_box_t kk_std_time_calendar__mlift4909_instant_fun5827_1(kk_function_t _fself, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_mlift4909_instant_fun5827_1(kk_std_time_instant__timescale _c_4884, kk_std_core_types__optional c, kk_std_core_types__optional cal, kk_std_time_date__date d, kk_std_core_types__optional tz, kk_context_t* _ctx) {
  struct kk_std_time_calendar__mlift4909_instant_fun5827__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar__mlift4909_instant_fun5827__t_1, 11, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar__mlift4909_instant_fun5827_1, kk_context());
  _self->_c_4884 = _c_4884;
  _self->c = c;
  _self->cal = cal;
  _self->d = d;
  _self->tz = tz;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_box_t kk_std_time_calendar__mlift4909_instant_fun5827_1(kk_function_t _fself, kk_context_t* _ctx) {
  struct kk_std_time_calendar__mlift4909_instant_fun5827__t_1* _self = kk_function_as(struct kk_std_time_calendar__mlift4909_instant_fun5827__t_1*, _fself, _ctx);
  kk_std_time_instant__timescale _c_4884 = _self->_c_4884; /* std/time/instant/timescale */
  kk_std_core_types__optional c = _self->c; /* optional<std/time/date/clock> */
  kk_std_core_types__optional cal = _self->cal; /* optional<std/time/calendar/calendar> */
  kk_std_time_date__date d = _self->d; /* std/time/date/date */
  kk_std_core_types__optional tz = _self->tz; /* optional<std/time/calendar/timezone> */
  kk_drop_match(_self, {kk_std_time_instant__timescale_dup(_c_4884, _ctx);kk_std_core_types__optional_dup(c, _ctx);kk_std_core_types__optional_dup(cal, _ctx);kk_std_time_date__date_dup(d, _ctx);kk_std_core_types__optional_dup(tz, _ctx);}, {}, _ctx)
  kk_std_time_instant__instant _x5828;
  kk_std_time_calendar__calendar _match_5381;
  if (kk_std_core_types__is_Optional(cal, _ctx)) {
    kk_box_t _box_x5181 = cal._cons.Optional.value;
    kk_std_time_calendar__calendar _cal_3033 = kk_std_time_calendar__calendar_unbox(_box_x5181, KK_BORROWED, _ctx);
    kk_std_time_calendar__calendar_dup(_cal_3033, _ctx);
    kk_std_core_types__optional_drop(cal, _ctx);
    _match_5381 = _cal_3033; /*std/time/calendar/calendar*/
  }
  else {
    kk_std_core_types__optional_drop(cal, _ctx);
    _match_5381 = kk_std_time_calendar__calendar_dup(kk_std_time_calendar_cal_iso, _ctx); /*std/time/calendar/calendar*/
  }
  {
    struct kk_std_time_calendar_Calendar* _con5829 = kk_std_time_calendar__as_Calendar(_match_5381, _ctx);
    kk_string_t _pat0 = _con5829->name;
    kk_string_t _pat1 = _con5829->long_name;
    kk_string_t _pat2 = _con5829->month_prefix;
    kk_function_t _pat3 = _con5829->show_era;
    kk_function_t _pat4 = _con5829->instant_to_dc;
    kk_function_t _x = _con5829->dc_to_instant;
    kk_function_t _pat50 = _con5829->days_to_date;
    kk_function_t _pat60 = _con5829->date_to_days;
    if kk_likely(kk_datatype_ptr_is_unique(_match_5381, _ctx)) {
      kk_function_drop(_pat60, _ctx);
      kk_function_drop(_pat50, _ctx);
      kk_function_drop(_pat4, _ctx);
      kk_function_drop(_pat3, _ctx);
      kk_string_drop(_pat2, _ctx);
      kk_string_drop(_pat1, _ctx);
      kk_string_drop(_pat0, _ctx);
      kk_datatype_ptr_free(_match_5381, _ctx);
    }
    else {
      kk_function_dup(_x, _ctx);
      kk_datatype_ptr_decref(_match_5381, _ctx);
    }
    kk_std_time_date__clock _x5830;
    if (kk_std_core_types__is_Optional(c, _ctx)) {
      kk_box_t _box_x5182 = c._cons.Optional.value;
      kk_std_time_date__clock _c_3025 = kk_std_time_date__clock_unbox(_box_x5182, KK_BORROWED, _ctx);
      kk_std_time_date__clock_dup(_c_3025, _ctx);
      kk_std_core_types__optional_drop(c, _ctx);
      _x5830 = _c_3025; /*std/time/date/clock*/
    }
    else {
      kk_std_core_types__optional_drop(c, _ctx);
      _x5830 = kk_std_time_date__clock_dup(kk_std_time_date_clock0, _ctx); /*std/time/date/clock*/
    }
    kk_std_time_calendar__timezone _x5831;
    if (kk_std_core_types__is_Optional(tz, _ctx)) {
      kk_box_t _box_x5183 = tz._cons.Optional.value;
      kk_std_time_calendar__timezone _tz_3029 = kk_std_time_calendar__timezone_unbox(_box_x5183, KK_BORROWED, _ctx);
      kk_std_time_calendar__timezone_dup(_tz_3029, _ctx);
      kk_std_core_types__optional_drop(tz, _ctx);
      _x5831 = _tz_3029; /*std/time/calendar/timezone*/
    }
    else {
      kk_std_core_types__optional_drop(tz, _ctx);
      _x5831 = kk_std_time_calendar__timezone_dup(kk_std_time_calendar_tz_utc, _ctx); /*std/time/calendar/timezone*/
    }
    _x5828 = kk_function_call(kk_std_time_instant__instant, (kk_function_t, kk_std_time_date__date, kk_std_time_date__clock, kk_std_time_calendar__timezone, kk_std_time_instant__timescale, kk_context_t*), _x, (_x, d, _x5830, _x5831, _c_4884, _ctx), _ctx); /*std/time/instant/instant*/
  }
  return kk_std_time_instant__instant_box(_x5828, _ctx);
}

kk_std_time_instant__instant kk_std_time_calendar__mlift4909_instant_1(kk_std_core_types__optional c, kk_std_core_types__optional cal, kk_std_time_date__date d, kk_std_core_types__optional tz, kk_std_time_instant__timescale _c_4884, kk_context_t* _ctx) { /* (c : optional<std/time/date/clock>, cal : optional<calendar>, d : std/time/date/date, tz : optional<timezone>, std/time/instant/timescale) -> std/time/instant/instant */ 
  kk_box_t _x5826 = kk_std_core_hnd__open_none0(kk_std_time_calendar__new_mlift4909_instant_fun5827_1(_c_4884, c, cal, d, tz, _ctx), _ctx); /*3280*/
  return kk_std_time_instant__instant_unbox(_x5826, KK_OWNED, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_instant_fun5836__t_1 {
  struct kk_function_s _base;
  kk_std_core_types__optional c;
  kk_std_core_types__optional cal;
  kk_std_time_date__date d;
  kk_std_core_types__optional tz;
};
static kk_box_t kk_std_time_calendar_instant_fun5836_1(kk_function_t _fself, kk_box_t _b_5194, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_instant_fun5836_1(kk_std_core_types__optional c, kk_std_core_types__optional cal, kk_std_time_date__date d, kk_std_core_types__optional tz, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun5836__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar_instant_fun5836__t_1, 10, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_instant_fun5836_1, kk_context());
  _self->c = c;
  _self->cal = cal;
  _self->d = d;
  _self->tz = tz;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_box_t kk_std_time_calendar_instant_fun5836_1(kk_function_t _fself, kk_box_t _b_5194, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun5836__t_1* _self = kk_function_as(struct kk_std_time_calendar_instant_fun5836__t_1*, _fself, _ctx);
  kk_std_core_types__optional c = _self->c; /* optional<std/time/date/clock> */
  kk_std_core_types__optional cal = _self->cal; /* optional<std/time/calendar/calendar> */
  kk_std_time_date__date d = _self->d; /* std/time/date/date */
  kk_std_core_types__optional tz = _self->tz; /* optional<std/time/calendar/timezone> */
  kk_drop_match(_self, {kk_std_core_types__optional_dup(c, _ctx);kk_std_core_types__optional_dup(cal, _ctx);kk_std_time_date__date_dup(d, _ctx);kk_std_core_types__optional_dup(tz, _ctx);}, {}, _ctx)
  kk_std_time_instant__timescale _c_5201_4884 = kk_std_time_instant__timescale_unbox(_b_5194, KK_OWNED, _ctx); /*std/time/instant/timescale*/;
  kk_std_time_instant__instant _x5837 = kk_std_time_calendar__mlift4909_instant_1(c, cal, d, tz, _c_5201_4884, _ctx); /*std/time/instant/instant*/
  return kk_std_time_instant__instant_box(_x5837, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_instant_fun5838__t_1 {
  struct kk_function_s _base;
  kk_std_core_types__optional c;
  kk_std_core_types__optional cal;
  kk_std_time_date__date d;
  kk_std_core_types__optional tz;
  kk_std_time_instant__timescale x_4911;
};
static kk_box_t kk_std_time_calendar_instant_fun5838_1(kk_function_t _fself, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_instant_fun5838_1(kk_std_core_types__optional c, kk_std_core_types__optional cal, kk_std_time_date__date d, kk_std_core_types__optional tz, kk_std_time_instant__timescale x_4911, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun5838__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar_instant_fun5838__t_1, 11, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_instant_fun5838_1, kk_context());
  _self->c = c;
  _self->cal = cal;
  _self->d = d;
  _self->tz = tz;
  _self->x_4911 = x_4911;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_box_t kk_std_time_calendar_instant_fun5838_1(kk_function_t _fself, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun5838__t_1* _self = kk_function_as(struct kk_std_time_calendar_instant_fun5838__t_1*, _fself, _ctx);
  kk_std_core_types__optional c = _self->c; /* optional<std/time/date/clock> */
  kk_std_core_types__optional cal = _self->cal; /* optional<std/time/calendar/calendar> */
  kk_std_time_date__date d = _self->d; /* std/time/date/date */
  kk_std_core_types__optional tz = _self->tz; /* optional<std/time/calendar/timezone> */
  kk_std_time_instant__timescale x_4911 = _self->x_4911; /* std/time/instant/timescale */
  kk_drop_match(_self, {kk_std_core_types__optional_dup(c, _ctx);kk_std_core_types__optional_dup(cal, _ctx);kk_std_time_date__date_dup(d, _ctx);kk_std_core_types__optional_dup(tz, _ctx);kk_std_time_instant__timescale_dup(x_4911, _ctx);}, {}, _ctx)
  kk_std_time_instant__instant _x5839;
  kk_std_time_calendar__calendar _match_5378;
  if (kk_std_core_types__is_Optional(cal, _ctx)) {
    kk_box_t _box_x5195 = cal._cons.Optional.value;
    kk_std_time_calendar__calendar _cal_3033 = kk_std_time_calendar__calendar_unbox(_box_x5195, KK_BORROWED, _ctx);
    kk_std_time_calendar__calendar_dup(_cal_3033, _ctx);
    kk_std_core_types__optional_drop(cal, _ctx);
    _match_5378 = _cal_3033; /*std/time/calendar/calendar*/
  }
  else {
    kk_std_core_types__optional_drop(cal, _ctx);
    _match_5378 = kk_std_time_calendar__calendar_dup(kk_std_time_calendar_cal_iso, _ctx); /*std/time/calendar/calendar*/
  }
  {
    struct kk_std_time_calendar_Calendar* _con5840 = kk_std_time_calendar__as_Calendar(_match_5378, _ctx);
    kk_string_t _pat01 = _con5840->name;
    kk_string_t _pat11 = _con5840->long_name;
    kk_string_t _pat20 = _con5840->month_prefix;
    kk_function_t _pat32 = _con5840->show_era;
    kk_function_t _pat40 = _con5840->instant_to_dc;
    kk_function_t _x = _con5840->dc_to_instant;
    kk_function_t _pat50 = _con5840->days_to_date;
    kk_function_t _pat60 = _con5840->date_to_days;
    if kk_likely(kk_datatype_ptr_is_unique(_match_5378, _ctx)) {
      kk_function_drop(_pat60, _ctx);
      kk_function_drop(_pat50, _ctx);
      kk_function_drop(_pat40, _ctx);
      kk_function_drop(_pat32, _ctx);
      kk_string_drop(_pat20, _ctx);
      kk_string_drop(_pat11, _ctx);
      kk_string_drop(_pat01, _ctx);
      kk_datatype_ptr_free(_match_5378, _ctx);
    }
    else {
      kk_function_dup(_x, _ctx);
      kk_datatype_ptr_decref(_match_5378, _ctx);
    }
    kk_std_time_date__clock _x5841;
    if (kk_std_core_types__is_Optional(c, _ctx)) {
      kk_box_t _box_x5196 = c._cons.Optional.value;
      kk_std_time_date__clock _c_3025 = kk_std_time_date__clock_unbox(_box_x5196, KK_BORROWED, _ctx);
      kk_std_time_date__clock_dup(_c_3025, _ctx);
      kk_std_core_types__optional_drop(c, _ctx);
      _x5841 = _c_3025; /*std/time/date/clock*/
    }
    else {
      kk_std_core_types__optional_drop(c, _ctx);
      _x5841 = kk_std_time_date__clock_dup(kk_std_time_date_clock0, _ctx); /*std/time/date/clock*/
    }
    kk_std_time_calendar__timezone _x5842;
    if (kk_std_core_types__is_Optional(tz, _ctx)) {
      kk_box_t _box_x5197 = tz._cons.Optional.value;
      kk_std_time_calendar__timezone _tz_3029 = kk_std_time_calendar__timezone_unbox(_box_x5197, KK_BORROWED, _ctx);
      kk_std_time_calendar__timezone_dup(_tz_3029, _ctx);
      kk_std_core_types__optional_drop(tz, _ctx);
      _x5842 = _tz_3029; /*std/time/calendar/timezone*/
    }
    else {
      kk_std_core_types__optional_drop(tz, _ctx);
      _x5842 = kk_std_time_calendar__timezone_dup(kk_std_time_calendar_tz_utc, _ctx); /*std/time/calendar/timezone*/
    }
    _x5839 = kk_function_call(kk_std_time_instant__instant, (kk_function_t, kk_std_time_date__date, kk_std_time_date__clock, kk_std_time_calendar__timezone, kk_std_time_instant__timescale, kk_context_t*), _x, (_x, d, _x5841, _x5842, x_4911, _ctx), _ctx); /*std/time/instant/instant*/
  }
  return kk_std_time_instant__instant_box(_x5839, _ctx);
}

kk_std_time_instant__instant kk_std_time_calendar_instant_1(kk_std_time_date__date d, kk_std_core_types__optional c, kk_std_core_types__optional tz, kk_std_core_types__optional cal, kk_std_core_types__optional ts, kk_context_t* _ctx) { /* (d : std/time/date/date, c : optional<std/time/date/clock>, tz : optional<timezone>, cal : optional<calendar>, ts : optional<std/time/instant/timescale>) -> std/time/utc/utc std/time/instant/instant */ 
  kk_std_time_instant__timescale x_4911;
  if (kk_std_core_types__is_Optional(ts, _ctx)) {
    kk_box_t _box_x5186 = ts._cons.Optional.value;
    kk_std_time_instant__timescale _ts_3039 = kk_std_time_instant__timescale_unbox(_box_x5186, KK_BORROWED, _ctx);
    kk_std_time_instant__timescale_dup(_ts_3039, _ctx);
    kk_std_core_types__optional_drop(ts, _ctx);
    x_4911 = _ts_3039; /*std/time/instant/timescale*/
  }
  else {
    kk_std_core_types__optional_drop(ts, _ctx);
    kk_std_core_hnd__ev ev_4914;
    kk_ssize_t _x5832 = (KK_IZ(0)); /*ssize_t*/
    ev_4914 = kk_evv_at(_x5832,kk_context()); /*std/core/hnd/ev<std/time/utc/.hnd-utc>*/
    kk_box_t _x5833;
    {
      struct kk_std_core_hnd_Ev* _con5834 = kk_std_core_hnd__as_Ev(ev_4914, _ctx);
      kk_box_t _box_x5187 = _con5834->hnd;
      kk_std_core_hnd__marker m0 = _con5834->marker;
      kk_std_time_utc__hnd_utc h = kk_std_time_utc__hnd_utc_unbox(_box_x5187, KK_BORROWED, _ctx);
      kk_std_time_utc__hnd_utc_dup(h, _ctx);
      kk_std_core_hnd__clause0 _match_5379;
      kk_std_core_hnd__clause0 _brw_5380 = kk_std_time_utc__select_utc(h, _ctx); /*std/core/hnd/clause0<std/time/instant/timescale,std/time/utc/.hnd-utc,277,278>*/;
      kk_std_time_utc__hnd_utc_drop(h, _ctx);
      _match_5379 = _brw_5380; /*std/core/hnd/clause0<std/time/instant/timescale,std/time/utc/.hnd-utc,277,278>*/
      {
        kk_function_t _fun_unbox_x5190 = _match_5379.clause;
        _x5833 = kk_function_call(kk_box_t, (kk_function_t, kk_std_core_hnd__marker, kk_std_core_hnd__ev, kk_context_t*), _fun_unbox_x5190, (_fun_unbox_x5190, m0, ev_4914, _ctx), _ctx); /*37*/
      }
    }
    x_4911 = kk_std_time_instant__timescale_unbox(_x5833, KK_OWNED, _ctx); /*std/time/instant/timescale*/
  }
  kk_box_t _x5835;
  if (kk_yielding(kk_context())) {
    kk_datatype_ptr_dropn(x_4911, (KK_I32(10)), _ctx);
    _x5835 = kk_std_core_hnd_yield_extend(kk_std_time_calendar_new_instant_fun5836_1(c, cal, d, tz, _ctx), _ctx); /*3926*/
  }
  else {
    _x5835 = kk_std_core_hnd__open_none0(kk_std_time_calendar_new_instant_fun5838_1(c, cal, d, tz, x_4911, _ctx), _ctx); /*3926*/
  }
  return kk_std_time_instant__instant_unbox(_x5835, KK_OWNED, _ctx);
}
 
/* Return the instant in time for a given date and clock interpreted by
   calendar `cal` (=`cal-iso`) in a timezone `tz` (=`tz-utc` by default).
  The `month`, `day`, `hour`, `minutes` may be outside their usual ranges
  and will be normalized during the conversion. For example, January 33 converts to February 2.
  This makes it very easy to add- or subtract days or months to an existing time.
  When the `seconds` or fraction of seconds `frac` add up over 60 those extra seconds are
  interpreted as leap seconds.
  Due to timezone transitions, or leap seconds, it is possible to specify dates that never
  happened (as it was skipped by a timezone change), or ambigious times (as a timezone springs back).
  In such cases, the time is always interpreted in the earlier timezone.
*/

kk_std_time_instant__instant kk_std_time_calendar_instant_2(kk_std_time_instant__timescale ts, kk_integer_t year, kk_std_core_types__optional month, kk_std_core_types__optional day, kk_std_core_types__optional hours, kk_std_core_types__optional minutes, kk_std_core_types__optional seconds, kk_std_core_types__optional frac, kk_std_core_types__optional tz, kk_std_core_types__optional cal, kk_context_t* _ctx) { /* (ts : std/time/instant/timescale, year : int, month : optional<int>, day : optional<int>, hours : optional<int>, minutes : optional<int>, seconds : optional<int>, frac : optional<float64>, tz : optional<timezone>, cal : optional<calendar>) -> std/time/instant/instant */ 
  kk_std_num_ddouble__ddouble fsecs;
  kk_std_num_ddouble__ddouble _x5843;
  kk_integer_t _x5844;
  if (kk_std_core_types__is_Optional(seconds, _ctx)) {
    kk_box_t _box_x5202 = seconds._cons.Optional.value;
    kk_integer_t _seconds_3092 = kk_integer_unbox(_box_x5202, _ctx);
    kk_integer_dup(_seconds_3092, _ctx);
    kk_std_core_types__optional_drop(seconds, _ctx);
    _x5844 = _seconds_3092; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(seconds, _ctx);
    _x5844 = kk_integer_from_small(0); /*int*/
  }
  _x5843 = kk_std_num_ddouble_ddouble_int_exp(_x5844, kk_integer_from_small(0), _ctx); /*std/num/ddouble/ddouble*/
  kk_std_num_ddouble__ddouble _x5845;
  double _x5846;
  if (kk_std_core_types__is_Optional(frac, _ctx)) {
    kk_box_t _box_x5203 = frac._cons.Optional.value;
    double _frac_3096 = kk_double_unbox(_box_x5203, KK_BORROWED, _ctx);
    kk_std_core_types__optional_drop(frac, _ctx);
    _x5846 = _frac_3096; /*float64*/
  }
  else {
    kk_std_core_types__optional_drop(frac, _ctx);
    _x5846 = 0x0p+0; /*float64*/
  }
  _x5845 = kk_std_num_ddouble__new_Ddouble(_x5846, 0x0p+0, _ctx); /*std/num/ddouble/ddouble*/
  fsecs = kk_std_num_ddouble__lp__plus__rp_(_x5843, _x5845, _ctx); /*std/num/ddouble/ddouble*/
  kk_std_time_calendar__calendar _match_5376;
  if (kk_std_core_types__is_Optional(cal, _ctx)) {
    kk_box_t _box_x5204 = cal._cons.Optional.value;
    kk_std_time_calendar__calendar _cal_3104 = kk_std_time_calendar__calendar_unbox(_box_x5204, KK_BORROWED, _ctx);
    kk_std_time_calendar__calendar_dup(_cal_3104, _ctx);
    kk_std_core_types__optional_drop(cal, _ctx);
    _match_5376 = _cal_3104; /*std/time/calendar/calendar*/
  }
  else {
    kk_std_core_types__optional_drop(cal, _ctx);
    _match_5376 = kk_std_time_calendar__calendar_dup(kk_std_time_calendar_cal_iso, _ctx); /*std/time/calendar/calendar*/
  }
  {
    struct kk_std_time_calendar_Calendar* _con5847 = kk_std_time_calendar__as_Calendar(_match_5376, _ctx);
    kk_string_t _pat0 = _con5847->name;
    kk_string_t _pat1 = _con5847->long_name;
    kk_string_t _pat2 = _con5847->month_prefix;
    kk_function_t _pat3 = _con5847->show_era;
    kk_function_t _pat4 = _con5847->instant_to_dc;
    kk_function_t _x = _con5847->dc_to_instant;
    kk_function_t _pat5 = _con5847->days_to_date;
    kk_function_t _pat6 = _con5847->date_to_days;
    if kk_likely(kk_datatype_ptr_is_unique(_match_5376, _ctx)) {
      kk_function_drop(_pat6, _ctx);
      kk_function_drop(_pat5, _ctx);
      kk_function_drop(_pat4, _ctx);
      kk_function_drop(_pat3, _ctx);
      kk_string_drop(_pat2, _ctx);
      kk_string_drop(_pat1, _ctx);
      kk_string_drop(_pat0, _ctx);
      kk_datatype_ptr_free(_match_5376, _ctx);
    }
    else {
      kk_function_dup(_x, _ctx);
      kk_datatype_ptr_decref(_match_5376, _ctx);
    }
    kk_std_time_date__date _x5848;
    kk_integer_t _x5849;
    if (kk_std_core_types__is_Optional(month, _ctx)) {
      kk_box_t _box_x5205 = month._cons.Optional.value;
      kk_integer_t _month_3076 = kk_integer_unbox(_box_x5205, _ctx);
      kk_integer_dup(_month_3076, _ctx);
      kk_std_core_types__optional_drop(month, _ctx);
      _x5849 = _month_3076; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(month, _ctx);
      _x5849 = kk_integer_from_small(1); /*int*/
    }
    kk_integer_t _x5850;
    if (kk_std_core_types__is_Optional(day, _ctx)) {
      kk_box_t _box_x5206 = day._cons.Optional.value;
      kk_integer_t _day_3080 = kk_integer_unbox(_box_x5206, _ctx);
      kk_integer_dup(_day_3080, _ctx);
      kk_std_core_types__optional_drop(day, _ctx);
      _x5850 = _day_3080; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(day, _ctx);
      _x5850 = kk_integer_from_small(1); /*int*/
    }
    _x5848 = kk_std_time_date__new_Date(year, _x5849, _x5850, _ctx); /*std/time/date/date*/
    kk_std_time_date__clock _x5851;
    kk_integer_t _x5852;
    if (kk_std_core_types__is_Optional(hours, _ctx)) {
      kk_box_t _box_x5207 = hours._cons.Optional.value;
      kk_integer_t _hours_3084 = kk_integer_unbox(_box_x5207, _ctx);
      kk_integer_dup(_hours_3084, _ctx);
      kk_std_core_types__optional_drop(hours, _ctx);
      _x5852 = _hours_3084; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(hours, _ctx);
      _x5852 = kk_integer_from_small(0); /*int*/
    }
    kk_integer_t _x5853;
    if (kk_std_core_types__is_Optional(minutes, _ctx)) {
      kk_box_t _box_x5208 = minutes._cons.Optional.value;
      kk_integer_t _minutes_3088 = kk_integer_unbox(_box_x5208, _ctx);
      kk_integer_dup(_minutes_3088, _ctx);
      kk_std_core_types__optional_drop(minutes, _ctx);
      _x5853 = _minutes_3088; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(minutes, _ctx);
      _x5853 = kk_integer_from_small(0); /*int*/
    }
    _x5851 = kk_std_time_date__new_Clock(kk_reuse_null, 0, _x5852, _x5853, fsecs, _ctx); /*std/time/date/clock*/
    kk_std_time_calendar__timezone _x5854;
    if (kk_std_core_types__is_Optional(tz, _ctx)) {
      kk_box_t _box_x5209 = tz._cons.Optional.value;
      kk_std_time_calendar__timezone _tz_3100 = kk_std_time_calendar__timezone_unbox(_box_x5209, KK_BORROWED, _ctx);
      kk_std_time_calendar__timezone_dup(_tz_3100, _ctx);
      kk_std_core_types__optional_drop(tz, _ctx);
      _x5854 = _tz_3100; /*std/time/calendar/timezone*/
    }
    else {
      kk_std_core_types__optional_drop(tz, _ctx);
      _x5854 = kk_std_time_calendar__timezone_dup(kk_std_time_calendar_tz_utc, _ctx); /*std/time/calendar/timezone*/
    }
    return kk_function_call(kk_std_time_instant__instant, (kk_function_t, kk_std_time_date__date, kk_std_time_date__clock, kk_std_time_calendar__timezone, kk_std_time_instant__timescale, kk_context_t*), _x, (_x, _x5848, _x5851, _x5854, ts, _ctx), _ctx);
  }
}
 
// monadic lift


// lift anonymous function
struct kk_std_time_calendar__mlift4910_instant_fun5856__t_3 {
  struct kk_function_s _base;
  kk_std_time_instant__timescale _c_4886;
  kk_std_core_types__optional cal;
  kk_std_core_types__optional day;
  kk_std_core_types__optional frac;
  kk_std_core_types__optional hours;
  kk_std_core_types__optional minutes;
  kk_std_core_types__optional month;
  kk_std_core_types__optional seconds;
  kk_std_core_types__optional tz;
  kk_integer_t year;
};
static kk_box_t kk_std_time_calendar__mlift4910_instant_fun5856_3(kk_function_t _fself, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_mlift4910_instant_fun5856_3(kk_std_time_instant__timescale _c_4886, kk_std_core_types__optional cal, kk_std_core_types__optional day, kk_std_core_types__optional frac, kk_std_core_types__optional hours, kk_std_core_types__optional minutes, kk_std_core_types__optional month, kk_std_core_types__optional seconds, kk_std_core_types__optional tz, kk_integer_t year, kk_context_t* _ctx) {
  struct kk_std_time_calendar__mlift4910_instant_fun5856__t_3* _self = kk_function_alloc_as(struct kk_std_time_calendar__mlift4910_instant_fun5856__t_3, 19, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar__mlift4910_instant_fun5856_3, kk_context());
  _self->_c_4886 = _c_4886;
  _self->cal = cal;
  _self->day = day;
  _self->frac = frac;
  _self->hours = hours;
  _self->minutes = minutes;
  _self->month = month;
  _self->seconds = seconds;
  _self->tz = tz;
  _self->year = year;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_box_t kk_std_time_calendar__mlift4910_instant_fun5856_3(kk_function_t _fself, kk_context_t* _ctx) {
  struct kk_std_time_calendar__mlift4910_instant_fun5856__t_3* _self = kk_function_as(struct kk_std_time_calendar__mlift4910_instant_fun5856__t_3*, _fself, _ctx);
  kk_std_time_instant__timescale _c_4886 = _self->_c_4886; /* std/time/instant/timescale */
  kk_std_core_types__optional cal = _self->cal; /* optional<std/time/calendar/calendar> */
  kk_std_core_types__optional day = _self->day; /* optional<int> */
  kk_std_core_types__optional frac = _self->frac; /* optional<float64> */
  kk_std_core_types__optional hours = _self->hours; /* optional<int> */
  kk_std_core_types__optional minutes = _self->minutes; /* optional<int> */
  kk_std_core_types__optional month = _self->month; /* optional<int> */
  kk_std_core_types__optional seconds = _self->seconds; /* optional<int> */
  kk_std_core_types__optional tz = _self->tz; /* optional<std/time/calendar/timezone> */
  kk_integer_t year = _self->year; /* int */
  kk_drop_match(_self, {kk_std_time_instant__timescale_dup(_c_4886, _ctx);kk_std_core_types__optional_dup(cal, _ctx);kk_std_core_types__optional_dup(day, _ctx);kk_std_core_types__optional_dup(frac, _ctx);kk_std_core_types__optional_dup(hours, _ctx);kk_std_core_types__optional_dup(minutes, _ctx);kk_std_core_types__optional_dup(month, _ctx);kk_std_core_types__optional_dup(seconds, _ctx);kk_std_core_types__optional_dup(tz, _ctx);kk_integer_dup(year, _ctx);}, {}, _ctx)
  kk_std_time_instant__instant _x5857;
  kk_std_core_types__optional _x5858;
  kk_box_t _x5859;
  kk_integer_t _x5860;
  if (kk_std_core_types__is_Optional(month, _ctx)) {
    kk_box_t _box_x5210 = month._cons.Optional.value;
    kk_integer_t _month_3193 = kk_integer_unbox(_box_x5210, _ctx);
    kk_integer_dup(_month_3193, _ctx);
    kk_std_core_types__optional_drop(month, _ctx);
    _x5860 = _month_3193; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(month, _ctx);
    _x5860 = kk_integer_from_small(1); /*int*/
  }
  _x5859 = kk_integer_box(_x5860, _ctx); /*116*/
  _x5858 = kk_std_core_types__new_Optional(_x5859, _ctx); /*optional<116>*/
  kk_std_core_types__optional _x5861;
  kk_box_t _x5862;
  kk_integer_t _x5863;
  if (kk_std_core_types__is_Optional(day, _ctx)) {
    kk_box_t _box_x5212 = day._cons.Optional.value;
    kk_integer_t _day_3197 = kk_integer_unbox(_box_x5212, _ctx);
    kk_integer_dup(_day_3197, _ctx);
    kk_std_core_types__optional_drop(day, _ctx);
    _x5863 = _day_3197; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(day, _ctx);
    _x5863 = kk_integer_from_small(1); /*int*/
  }
  _x5862 = kk_integer_box(_x5863, _ctx); /*116*/
  _x5861 = kk_std_core_types__new_Optional(_x5862, _ctx); /*optional<116>*/
  kk_std_core_types__optional _x5864;
  kk_box_t _x5865;
  kk_integer_t _x5866;
  if (kk_std_core_types__is_Optional(hours, _ctx)) {
    kk_box_t _box_x5214 = hours._cons.Optional.value;
    kk_integer_t _hours_3201 = kk_integer_unbox(_box_x5214, _ctx);
    kk_integer_dup(_hours_3201, _ctx);
    kk_std_core_types__optional_drop(hours, _ctx);
    _x5866 = _hours_3201; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(hours, _ctx);
    _x5866 = kk_integer_from_small(0); /*int*/
  }
  _x5865 = kk_integer_box(_x5866, _ctx); /*116*/
  _x5864 = kk_std_core_types__new_Optional(_x5865, _ctx); /*optional<116>*/
  kk_std_core_types__optional _x5867;
  kk_box_t _x5868;
  kk_integer_t _x5869;
  if (kk_std_core_types__is_Optional(minutes, _ctx)) {
    kk_box_t _box_x5216 = minutes._cons.Optional.value;
    kk_integer_t _minutes_3205 = kk_integer_unbox(_box_x5216, _ctx);
    kk_integer_dup(_minutes_3205, _ctx);
    kk_std_core_types__optional_drop(minutes, _ctx);
    _x5869 = _minutes_3205; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(minutes, _ctx);
    _x5869 = kk_integer_from_small(0); /*int*/
  }
  _x5868 = kk_integer_box(_x5869, _ctx); /*116*/
  _x5867 = kk_std_core_types__new_Optional(_x5868, _ctx); /*optional<116>*/
  kk_std_core_types__optional _x5870;
  kk_box_t _x5871;
  kk_integer_t _x5872;
  if (kk_std_core_types__is_Optional(seconds, _ctx)) {
    kk_box_t _box_x5218 = seconds._cons.Optional.value;
    kk_integer_t _seconds_3209 = kk_integer_unbox(_box_x5218, _ctx);
    kk_integer_dup(_seconds_3209, _ctx);
    kk_std_core_types__optional_drop(seconds, _ctx);
    _x5872 = _seconds_3209; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(seconds, _ctx);
    _x5872 = kk_integer_from_small(0); /*int*/
  }
  _x5871 = kk_integer_box(_x5872, _ctx); /*116*/
  _x5870 = kk_std_core_types__new_Optional(_x5871, _ctx); /*optional<116>*/
  kk_std_core_types__optional _x5873;
  kk_box_t _x5874;
  double _x5875;
  if (kk_std_core_types__is_Optional(frac, _ctx)) {
    kk_box_t _box_x5220 = frac._cons.Optional.value;
    double _frac_3213 = kk_double_unbox(_box_x5220, KK_BORROWED, _ctx);
    kk_std_core_types__optional_drop(frac, _ctx);
    _x5875 = _frac_3213; /*float64*/
  }
  else {
    kk_std_core_types__optional_drop(frac, _ctx);
    _x5875 = 0x0p+0; /*float64*/
  }
  _x5874 = kk_double_box(_x5875, _ctx); /*116*/
  _x5873 = kk_std_core_types__new_Optional(_x5874, _ctx); /*optional<116>*/
  kk_std_core_types__optional _x5876;
  kk_box_t _x5877;
  kk_std_time_calendar__timezone _x5878;
  if (kk_std_core_types__is_Optional(tz, _ctx)) {
    kk_box_t _box_x5222 = tz._cons.Optional.value;
    kk_std_time_calendar__timezone _tz_3217 = kk_std_time_calendar__timezone_unbox(_box_x5222, KK_BORROWED, _ctx);
    kk_std_time_calendar__timezone_dup(_tz_3217, _ctx);
    kk_std_core_types__optional_drop(tz, _ctx);
    _x5878 = _tz_3217; /*std/time/calendar/timezone*/
  }
  else {
    kk_std_core_types__optional_drop(tz, _ctx);
    _x5878 = kk_std_time_calendar__timezone_dup(kk_std_time_calendar_tz_utc, _ctx); /*std/time/calendar/timezone*/
  }
  _x5877 = kk_std_time_calendar__timezone_box(_x5878, _ctx); /*116*/
  _x5876 = kk_std_core_types__new_Optional(_x5877, _ctx); /*optional<116>*/
  kk_std_core_types__optional _x5879;
  kk_box_t _x5880;
  kk_std_time_calendar__calendar _x5881;
  if (kk_std_core_types__is_Optional(cal, _ctx)) {
    kk_box_t _box_x5224 = cal._cons.Optional.value;
    kk_std_time_calendar__calendar _cal_3221 = kk_std_time_calendar__calendar_unbox(_box_x5224, KK_BORROWED, _ctx);
    kk_std_time_calendar__calendar_dup(_cal_3221, _ctx);
    kk_std_core_types__optional_drop(cal, _ctx);
    _x5881 = _cal_3221; /*std/time/calendar/calendar*/
  }
  else {
    kk_std_core_types__optional_drop(cal, _ctx);
    _x5881 = kk_std_time_calendar__calendar_dup(kk_std_time_calendar_cal_iso, _ctx); /*std/time/calendar/calendar*/
  }
  _x5880 = kk_std_time_calendar__calendar_box(_x5881, _ctx); /*116*/
  _x5879 = kk_std_core_types__new_Optional(_x5880, _ctx); /*optional<116>*/
  _x5857 = kk_std_time_calendar_instant_2(_c_4886, year, _x5858, _x5861, _x5864, _x5867, _x5870, _x5873, _x5876, _x5879, _ctx); /*std/time/instant/instant*/
  return kk_std_time_instant__instant_box(_x5857, _ctx);
}

kk_std_time_instant__instant kk_std_time_calendar__mlift4910_instant_3(kk_std_core_types__optional cal, kk_std_core_types__optional day, kk_std_core_types__optional frac, kk_std_core_types__optional hours, kk_std_core_types__optional minutes, kk_std_core_types__optional month, kk_std_core_types__optional seconds, kk_std_core_types__optional tz, kk_integer_t year, kk_std_time_instant__timescale _c_4886, kk_context_t* _ctx) { /* (cal : optional<calendar>, day : optional<int>, frac : optional<float64>, hours : optional<int>, minutes : optional<int>, month : optional<int>, seconds : optional<int>, tz : optional<timezone>, year : int, std/time/instant/timescale) -> std/time/instant/instant */ 
  kk_box_t _x5855 = kk_std_core_hnd__open_none0(kk_std_time_calendar__new_mlift4910_instant_fun5856_3(_c_4886, cal, day, frac, hours, minutes, month, seconds, tz, year, _ctx), _ctx); /*3280*/
  return kk_std_time_instant__instant_unbox(_x5855, KK_OWNED, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_instant_fun5886__t_3 {
  struct kk_function_s _base;
  kk_std_core_types__optional cal;
  kk_std_core_types__optional day;
  kk_std_core_types__optional frac;
  kk_std_core_types__optional hours;
  kk_std_core_types__optional minutes;
  kk_std_core_types__optional month;
  kk_std_core_types__optional seconds;
  kk_std_core_types__optional tz;
  kk_integer_t year;
};
static kk_box_t kk_std_time_calendar_instant_fun5886_3(kk_function_t _fself, kk_box_t _b_5244, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_instant_fun5886_3(kk_std_core_types__optional cal, kk_std_core_types__optional day, kk_std_core_types__optional frac, kk_std_core_types__optional hours, kk_std_core_types__optional minutes, kk_std_core_types__optional month, kk_std_core_types__optional seconds, kk_std_core_types__optional tz, kk_integer_t year, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun5886__t_3* _self = kk_function_alloc_as(struct kk_std_time_calendar_instant_fun5886__t_3, 18, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_instant_fun5886_3, kk_context());
  _self->cal = cal;
  _self->day = day;
  _self->frac = frac;
  _self->hours = hours;
  _self->minutes = minutes;
  _self->month = month;
  _self->seconds = seconds;
  _self->tz = tz;
  _self->year = year;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_box_t kk_std_time_calendar_instant_fun5886_3(kk_function_t _fself, kk_box_t _b_5244, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun5886__t_3* _self = kk_function_as(struct kk_std_time_calendar_instant_fun5886__t_3*, _fself, _ctx);
  kk_std_core_types__optional cal = _self->cal; /* optional<std/time/calendar/calendar> */
  kk_std_core_types__optional day = _self->day; /* optional<int> */
  kk_std_core_types__optional frac = _self->frac; /* optional<float64> */
  kk_std_core_types__optional hours = _self->hours; /* optional<int> */
  kk_std_core_types__optional minutes = _self->minutes; /* optional<int> */
  kk_std_core_types__optional month = _self->month; /* optional<int> */
  kk_std_core_types__optional seconds = _self->seconds; /* optional<int> */
  kk_std_core_types__optional tz = _self->tz; /* optional<std/time/calendar/timezone> */
  kk_integer_t year = _self->year; /* int */
  kk_drop_match(_self, {kk_std_core_types__optional_dup(cal, _ctx);kk_std_core_types__optional_dup(day, _ctx);kk_std_core_types__optional_dup(frac, _ctx);kk_std_core_types__optional_dup(hours, _ctx);kk_std_core_types__optional_dup(minutes, _ctx);kk_std_core_types__optional_dup(month, _ctx);kk_std_core_types__optional_dup(seconds, _ctx);kk_std_core_types__optional_dup(tz, _ctx);kk_integer_dup(year, _ctx);}, {}, _ctx)
  kk_std_time_instant__timescale _c_5272_4886 = kk_std_time_instant__timescale_unbox(_b_5244, KK_OWNED, _ctx); /*std/time/instant/timescale*/;
  kk_std_time_instant__instant _x5887 = kk_std_time_calendar__mlift4910_instant_3(cal, day, frac, hours, minutes, month, seconds, tz, year, _c_5272_4886, _ctx); /*std/time/instant/instant*/
  return kk_std_time_instant__instant_box(_x5887, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_instant_fun5888__t_3 {
  struct kk_function_s _base;
  kk_std_core_types__optional cal;
  kk_std_core_types__optional day;
  kk_std_core_types__optional frac;
  kk_std_core_types__optional hours;
  kk_std_core_types__optional minutes;
  kk_std_core_types__optional month;
  kk_std_core_types__optional seconds;
  kk_std_core_types__optional tz;
  kk_std_time_instant__timescale x_4916;
  kk_integer_t year;
};
static kk_box_t kk_std_time_calendar_instant_fun5888_3(kk_function_t _fself, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_instant_fun5888_3(kk_std_core_types__optional cal, kk_std_core_types__optional day, kk_std_core_types__optional frac, kk_std_core_types__optional hours, kk_std_core_types__optional minutes, kk_std_core_types__optional month, kk_std_core_types__optional seconds, kk_std_core_types__optional tz, kk_std_time_instant__timescale x_4916, kk_integer_t year, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun5888__t_3* _self = kk_function_alloc_as(struct kk_std_time_calendar_instant_fun5888__t_3, 19, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_instant_fun5888_3, kk_context());
  _self->cal = cal;
  _self->day = day;
  _self->frac = frac;
  _self->hours = hours;
  _self->minutes = minutes;
  _self->month = month;
  _self->seconds = seconds;
  _self->tz = tz;
  _self->x_4916 = x_4916;
  _self->year = year;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_box_t kk_std_time_calendar_instant_fun5888_3(kk_function_t _fself, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun5888__t_3* _self = kk_function_as(struct kk_std_time_calendar_instant_fun5888__t_3*, _fself, _ctx);
  kk_std_core_types__optional cal = _self->cal; /* optional<std/time/calendar/calendar> */
  kk_std_core_types__optional day = _self->day; /* optional<int> */
  kk_std_core_types__optional frac = _self->frac; /* optional<float64> */
  kk_std_core_types__optional hours = _self->hours; /* optional<int> */
  kk_std_core_types__optional minutes = _self->minutes; /* optional<int> */
  kk_std_core_types__optional month = _self->month; /* optional<int> */
  kk_std_core_types__optional seconds = _self->seconds; /* optional<int> */
  kk_std_core_types__optional tz = _self->tz; /* optional<std/time/calendar/timezone> */
  kk_std_time_instant__timescale x_4916 = _self->x_4916; /* std/time/instant/timescale */
  kk_integer_t year = _self->year; /* int */
  kk_drop_match(_self, {kk_std_core_types__optional_dup(cal, _ctx);kk_std_core_types__optional_dup(day, _ctx);kk_std_core_types__optional_dup(frac, _ctx);kk_std_core_types__optional_dup(hours, _ctx);kk_std_core_types__optional_dup(minutes, _ctx);kk_std_core_types__optional_dup(month, _ctx);kk_std_core_types__optional_dup(seconds, _ctx);kk_std_core_types__optional_dup(tz, _ctx);kk_std_time_instant__timescale_dup(x_4916, _ctx);kk_integer_dup(year, _ctx);}, {}, _ctx)
  kk_std_time_instant__instant _x5889;
  kk_std_core_types__optional _x5890;
  kk_box_t _x5891;
  kk_integer_t _x5892;
  if (kk_std_core_types__is_Optional(month, _ctx)) {
    kk_box_t _box_x5245 = month._cons.Optional.value;
    kk_integer_t _month_3193 = kk_integer_unbox(_box_x5245, _ctx);
    kk_integer_dup(_month_3193, _ctx);
    kk_std_core_types__optional_drop(month, _ctx);
    _x5892 = _month_3193; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(month, _ctx);
    _x5892 = kk_integer_from_small(1); /*int*/
  }
  _x5891 = kk_integer_box(_x5892, _ctx); /*116*/
  _x5890 = kk_std_core_types__new_Optional(_x5891, _ctx); /*optional<116>*/
  kk_std_core_types__optional _x5893;
  kk_box_t _x5894;
  kk_integer_t _x5895;
  if (kk_std_core_types__is_Optional(day, _ctx)) {
    kk_box_t _box_x5247 = day._cons.Optional.value;
    kk_integer_t _day_3197 = kk_integer_unbox(_box_x5247, _ctx);
    kk_integer_dup(_day_3197, _ctx);
    kk_std_core_types__optional_drop(day, _ctx);
    _x5895 = _day_3197; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(day, _ctx);
    _x5895 = kk_integer_from_small(1); /*int*/
  }
  _x5894 = kk_integer_box(_x5895, _ctx); /*116*/
  _x5893 = kk_std_core_types__new_Optional(_x5894, _ctx); /*optional<116>*/
  kk_std_core_types__optional _x5896;
  kk_box_t _x5897;
  kk_integer_t _x5898;
  if (kk_std_core_types__is_Optional(hours, _ctx)) {
    kk_box_t _box_x5249 = hours._cons.Optional.value;
    kk_integer_t _hours_3201 = kk_integer_unbox(_box_x5249, _ctx);
    kk_integer_dup(_hours_3201, _ctx);
    kk_std_core_types__optional_drop(hours, _ctx);
    _x5898 = _hours_3201; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(hours, _ctx);
    _x5898 = kk_integer_from_small(0); /*int*/
  }
  _x5897 = kk_integer_box(_x5898, _ctx); /*116*/
  _x5896 = kk_std_core_types__new_Optional(_x5897, _ctx); /*optional<116>*/
  kk_std_core_types__optional _x5899;
  kk_box_t _x5900;
  kk_integer_t _x5901;
  if (kk_std_core_types__is_Optional(minutes, _ctx)) {
    kk_box_t _box_x5251 = minutes._cons.Optional.value;
    kk_integer_t _minutes_3205 = kk_integer_unbox(_box_x5251, _ctx);
    kk_integer_dup(_minutes_3205, _ctx);
    kk_std_core_types__optional_drop(minutes, _ctx);
    _x5901 = _minutes_3205; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(minutes, _ctx);
    _x5901 = kk_integer_from_small(0); /*int*/
  }
  _x5900 = kk_integer_box(_x5901, _ctx); /*116*/
  _x5899 = kk_std_core_types__new_Optional(_x5900, _ctx); /*optional<116>*/
  kk_std_core_types__optional _x5902;
  kk_box_t _x5903;
  kk_integer_t _x5904;
  if (kk_std_core_types__is_Optional(seconds, _ctx)) {
    kk_box_t _box_x5253 = seconds._cons.Optional.value;
    kk_integer_t _seconds_3209 = kk_integer_unbox(_box_x5253, _ctx);
    kk_integer_dup(_seconds_3209, _ctx);
    kk_std_core_types__optional_drop(seconds, _ctx);
    _x5904 = _seconds_3209; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(seconds, _ctx);
    _x5904 = kk_integer_from_small(0); /*int*/
  }
  _x5903 = kk_integer_box(_x5904, _ctx); /*116*/
  _x5902 = kk_std_core_types__new_Optional(_x5903, _ctx); /*optional<116>*/
  kk_std_core_types__optional _x5905;
  kk_box_t _x5906;
  double _x5907;
  if (kk_std_core_types__is_Optional(frac, _ctx)) {
    kk_box_t _box_x5255 = frac._cons.Optional.value;
    double _frac_3213 = kk_double_unbox(_box_x5255, KK_BORROWED, _ctx);
    kk_std_core_types__optional_drop(frac, _ctx);
    _x5907 = _frac_3213; /*float64*/
  }
  else {
    kk_std_core_types__optional_drop(frac, _ctx);
    _x5907 = 0x0p+0; /*float64*/
  }
  _x5906 = kk_double_box(_x5907, _ctx); /*116*/
  _x5905 = kk_std_core_types__new_Optional(_x5906, _ctx); /*optional<116>*/
  kk_std_core_types__optional _x5908;
  kk_box_t _x5909;
  kk_std_time_calendar__timezone _x5910;
  if (kk_std_core_types__is_Optional(tz, _ctx)) {
    kk_box_t _box_x5257 = tz._cons.Optional.value;
    kk_std_time_calendar__timezone _tz_3217 = kk_std_time_calendar__timezone_unbox(_box_x5257, KK_BORROWED, _ctx);
    kk_std_time_calendar__timezone_dup(_tz_3217, _ctx);
    kk_std_core_types__optional_drop(tz, _ctx);
    _x5910 = _tz_3217; /*std/time/calendar/timezone*/
  }
  else {
    kk_std_core_types__optional_drop(tz, _ctx);
    _x5910 = kk_std_time_calendar__timezone_dup(kk_std_time_calendar_tz_utc, _ctx); /*std/time/calendar/timezone*/
  }
  _x5909 = kk_std_time_calendar__timezone_box(_x5910, _ctx); /*116*/
  _x5908 = kk_std_core_types__new_Optional(_x5909, _ctx); /*optional<116>*/
  kk_std_core_types__optional _x5911;
  kk_box_t _x5912;
  kk_std_time_calendar__calendar _x5913;
  if (kk_std_core_types__is_Optional(cal, _ctx)) {
    kk_box_t _box_x5259 = cal._cons.Optional.value;
    kk_std_time_calendar__calendar _cal_3221 = kk_std_time_calendar__calendar_unbox(_box_x5259, KK_BORROWED, _ctx);
    kk_std_time_calendar__calendar_dup(_cal_3221, _ctx);
    kk_std_core_types__optional_drop(cal, _ctx);
    _x5913 = _cal_3221; /*std/time/calendar/calendar*/
  }
  else {
    kk_std_core_types__optional_drop(cal, _ctx);
    _x5913 = kk_std_time_calendar__calendar_dup(kk_std_time_calendar_cal_iso, _ctx); /*std/time/calendar/calendar*/
  }
  _x5912 = kk_std_time_calendar__calendar_box(_x5913, _ctx); /*116*/
  _x5911 = kk_std_core_types__new_Optional(_x5912, _ctx); /*optional<116>*/
  _x5889 = kk_std_time_calendar_instant_2(x_4916, year, _x5890, _x5893, _x5896, _x5899, _x5902, _x5905, _x5908, _x5911, _ctx); /*std/time/instant/instant*/
  return kk_std_time_instant__instant_box(_x5889, _ctx);
}

kk_std_time_instant__instant kk_std_time_calendar_instant_3(kk_integer_t year, kk_std_core_types__optional month, kk_std_core_types__optional day, kk_std_core_types__optional hours, kk_std_core_types__optional minutes, kk_std_core_types__optional seconds, kk_std_core_types__optional frac, kk_std_core_types__optional tz, kk_std_core_types__optional cal, kk_std_core_types__optional ts, kk_context_t* _ctx) { /* (year : int, month : optional<int>, day : optional<int>, hours : optional<int>, minutes : optional<int>, seconds : optional<int>, frac : optional<float64>, tz : optional<timezone>, cal : optional<calendar>, ts : optional<std/time/instant/timescale>) -> std/time/utc/utc std/time/instant/instant */ 
  kk_std_time_instant__timescale x_4916;
  if (kk_std_core_types__is_Optional(ts, _ctx)) {
    kk_box_t _box_x5236 = ts._cons.Optional.value;
    kk_std_time_instant__timescale _ts_3227 = kk_std_time_instant__timescale_unbox(_box_x5236, KK_BORROWED, _ctx);
    kk_std_time_instant__timescale_dup(_ts_3227, _ctx);
    kk_std_core_types__optional_drop(ts, _ctx);
    x_4916 = _ts_3227; /*std/time/instant/timescale*/
  }
  else {
    kk_std_core_types__optional_drop(ts, _ctx);
    kk_std_core_hnd__ev ev_4919;
    kk_ssize_t _x5882 = (KK_IZ(0)); /*ssize_t*/
    ev_4919 = kk_evv_at(_x5882,kk_context()); /*std/core/hnd/ev<std/time/utc/.hnd-utc>*/
    kk_box_t _x5883;
    {
      struct kk_std_core_hnd_Ev* _con5884 = kk_std_core_hnd__as_Ev(ev_4919, _ctx);
      kk_box_t _box_x5237 = _con5884->hnd;
      kk_std_core_hnd__marker m0 = _con5884->marker;
      kk_std_time_utc__hnd_utc h = kk_std_time_utc__hnd_utc_unbox(_box_x5237, KK_BORROWED, _ctx);
      kk_std_time_utc__hnd_utc_dup(h, _ctx);
      kk_std_core_hnd__clause0 _match_5374;
      kk_std_core_hnd__clause0 _brw_5375 = kk_std_time_utc__select_utc(h, _ctx); /*std/core/hnd/clause0<std/time/instant/timescale,std/time/utc/.hnd-utc,277,278>*/;
      kk_std_time_utc__hnd_utc_drop(h, _ctx);
      _match_5374 = _brw_5375; /*std/core/hnd/clause0<std/time/instant/timescale,std/time/utc/.hnd-utc,277,278>*/
      {
        kk_function_t _fun_unbox_x5240 = _match_5374.clause;
        _x5883 = kk_function_call(kk_box_t, (kk_function_t, kk_std_core_hnd__marker, kk_std_core_hnd__ev, kk_context_t*), _fun_unbox_x5240, (_fun_unbox_x5240, m0, ev_4919, _ctx), _ctx); /*37*/
      }
    }
    x_4916 = kk_std_time_instant__timescale_unbox(_x5883, KK_OWNED, _ctx); /*std/time/instant/timescale*/
  }
  kk_box_t _x5885;
  if (kk_yielding(kk_context())) {
    kk_datatype_ptr_dropn(x_4916, (KK_I32(10)), _ctx);
    _x5885 = kk_std_core_hnd_yield_extend(kk_std_time_calendar_new_instant_fun5886_3(cal, day, frac, hours, minutes, month, seconds, tz, year, _ctx), _ctx); /*3926*/
  }
  else {
    _x5885 = kk_std_core_hnd__open_none0(kk_std_time_calendar_new_instant_fun5888_3(cal, day, frac, hours, minutes, month, seconds, tz, x_4916, year, _ctx), _ctx); /*3926*/
  }
  return kk_std_time_instant__instant_unbox(_x5885, KK_OWNED, _ctx);
}
 
// The (proleptic) [Gregorian calendar](https://en.wikipedia.org/wiki/Gregorian_calendar).
// Just like the ISO calendar except that the year 0 does not exist, i.e. after 1 BCE (=`~1`)
// we have 1 CE (=`1`).
// The calendar short name is ``GC``.


// lift anonymous function
struct kk_std_time_calendar_cal_gregorian_fun5919__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_gregorian_fun5919(kk_function_t _fself, kk_integer_t _x15918, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_gregorian_fun5919(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_gregorian_fun5919, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_integer_t kk_std_time_calendar_cal_gregorian_fun5919(kk_function_t _fself, kk_integer_t _x15918, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_days_before_year(_x15918, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_gregorian_fun5921__t {
  struct kk_function_s _base;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_cal_gregorian_fun5921(kk_function_t _fself, kk_integer_t _x15920, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_gregorian_fun5921(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_gregorian_fun5921, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_cal_gregorian_fun5921(kk_function_t _fself, kk_integer_t _x15920, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_estimate_year(_x15920, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_gregorian_fun5924__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_gregorian_fun5924(kk_function_t _fself, kk_integer_t _x15922, kk_integer_t _x25923, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_gregorian_fun5924(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_gregorian_fun5924, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_integer_t kk_std_time_calendar_cal_gregorian_fun5924(kk_function_t _fself, kk_integer_t _x15922, kk_integer_t _x25923, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_days_before_month(_x15922, _x25923, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_gregorian_fun5927__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_gregorian_fun5927(kk_function_t _fself, kk_integer_t _x15925, kk_integer_t _x25926, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_gregorian_fun5927(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_gregorian_fun5927, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_integer_t kk_std_time_calendar_cal_gregorian_fun5927(kk_function_t _fself, kk_integer_t _x15925, kk_integer_t _x25926, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_doy_to_month(_x15925, _x25926, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_gregorian_fun5930__t {
  struct kk_function_s _base;
};
static kk_box_t kk_std_time_calendar_cal_gregorian_fun5930(kk_function_t _fself, kk_box_t _b_5277, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_gregorian_fun5930(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_gregorian_fun5930, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_box_t kk_std_time_calendar_cal_gregorian_fun5930(kk_function_t _fself, kk_box_t _b_5277, kk_context_t* _ctx) {
  kk_unused(_fself);
  kk_string_t _x5931;
  kk_std_time_date__date d_5280 = kk_std_time_date__date_unbox(_b_5277, KK_OWNED, _ctx); /*std/time/date/date*/;
  bool _match_5370;
  kk_integer_t _brw_5371;
  {
    kk_integer_t _x = d_5280.year;
    kk_integer_dup(_x, _ctx);
    kk_std_time_date__date_drop(d_5280, _ctx);
    _brw_5371 = _x; /*int*/
  }
  bool _brw_5372 = kk_integer_lt_borrow(_brw_5371,(kk_integer_from_small(0)),kk_context()); /*bool*/;
  kk_integer_drop(_brw_5371, _ctx);
  _match_5370 = _brw_5372; /*bool*/
  if (_match_5370) {
    kk_define_string_literal(, _s5932, 3, "BCE", _ctx)
    _x5931 = kk_string_dup(_s5932, _ctx); /*string*/
  }
  else {
    kk_define_string_literal(, _s5933, 2, "CE", _ctx)
    _x5931 = kk_string_dup(_s5933, _ctx); /*string*/
  }
  return kk_string_box(_x5931);
}

kk_std_time_calendar__calendar kk_std_time_calendar_cal_gregorian;
 
/* The standard [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) calendar
but using TI-SLS time (`ts-ti-sls`), i.e. TI with
[smoothed leap seconds](https://www.cl.cam.ac.uk/~mgk25/time/utc-sls/).
This calendar is equivalent to the ISO calendar except for the last 1000 seconds of a day where
a leap second occurs. On such day, the leap second time step (positive or negative)
is distributed over the last 1000 seconds of the day. On the full hour, ISO and ISO-SLS are equal again.
This is a recommended calendar to use for
time stamps or communication with other services since it avoids any potential trouble
with leap seconds while still being quite precise.
Other good properties include:
- All days have 86400 seconds, and the time 23:59:60 never appears
- The time never differs more than one second from UTC
- Time as always equal to UTC on the full and half hour
*/


// lift anonymous function
struct kk_std_time_calendar_cal_iso_sls_fun5939__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_iso_sls_fun5939(kk_function_t _fself, kk_integer_t _x15938, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_sls_fun5939(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_iso_sls_fun5939, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_integer_t kk_std_time_calendar_cal_iso_sls_fun5939(kk_function_t _fself, kk_integer_t _x15938, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_days_before_year(_x15938, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_iso_sls_fun5941__t {
  struct kk_function_s _base;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_cal_iso_sls_fun5941(kk_function_t _fself, kk_integer_t _x15940, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_sls_fun5941(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_iso_sls_fun5941, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_cal_iso_sls_fun5941(kk_function_t _fself, kk_integer_t _x15940, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_estimate_year(_x15940, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_iso_sls_fun5944__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_iso_sls_fun5944(kk_function_t _fself, kk_integer_t _x15942, kk_integer_t _x25943, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_sls_fun5944(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_iso_sls_fun5944, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_integer_t kk_std_time_calendar_cal_iso_sls_fun5944(kk_function_t _fself, kk_integer_t _x15942, kk_integer_t _x25943, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_days_before_month(_x15942, _x25943, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_iso_sls_fun5947__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_iso_sls_fun5947(kk_function_t _fself, kk_integer_t _x15945, kk_integer_t _x25946, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_sls_fun5947(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_iso_sls_fun5947, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_integer_t kk_std_time_calendar_cal_iso_sls_fun5947(kk_function_t _fself, kk_integer_t _x15945, kk_integer_t _x25946, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_doy_to_month(_x15945, _x25946, _ctx);
}

kk_std_time_calendar__calendar kk_std_time_calendar_cal_iso_sls;

kk_std_time_timestamp__timestamp kk_std_time_calendar_cal_timestamp(kk_std_time_calendar__calendar cal, kk_std_time_instant__timescale ts, kk_std_time_date__date d, kk_std_time_date__clock c, kk_context_t* _ctx) { /* (cal : calendar, ts : std/time/instant/timescale, d : std/time/date/date, c : std/time/date/clock) -> std/time/timestamp/timestamp */ 
  kk_function_t _x5948;
  {
    struct kk_std_time_calendar_Calendar* _con5949 = kk_std_time_calendar__as_Calendar(cal, _ctx);
    kk_string_t _pat0 = _con5949->name;
    kk_string_t _pat1 = _con5949->long_name;
    kk_string_t _pat2 = _con5949->month_prefix;
    kk_function_t _pat3 = _con5949->show_era;
    kk_function_t _pat4 = _con5949->instant_to_dc;
    kk_function_t _pat5 = _con5949->dc_to_instant;
    kk_function_t _pat6 = _con5949->days_to_date;
    kk_function_t _x = _con5949->date_to_days;
    if kk_likely(kk_datatype_ptr_is_unique(cal, _ctx)) {
      kk_function_drop(_pat6, _ctx);
      kk_function_drop(_pat5, _ctx);
      kk_function_drop(_pat4, _ctx);
      kk_function_drop(_pat3, _ctx);
      kk_string_drop(_pat2, _ctx);
      kk_string_drop(_pat1, _ctx);
      kk_string_drop(_pat0, _ctx);
      kk_datatype_ptr_free(cal, _ctx);
    }
    else {
      kk_function_dup(_x, _ctx);
      kk_datatype_ptr_decref(cal, _ctx);
    }
    _x5948 = _x; /*(date : std/time/date/date) -> int*/
  }
  kk_std_core_types__optional _x5950;
  kk_box_t _x5951;
  bool _x5952;
  {
    struct kk_std_time_instant_Timescale* _con5953 = kk_std_time_instant__as_Timescale(ts, _ctx);
    kk_string_t _pat00 = _con5953->name;
    kk_string_t _pat100 = _con5953->unit;
    kk_function_t _pat20 = _con5953->from_tai;
    kk_function_t _pat30 = _con5953->to_tai;
    kk_std_core_types__maybe _x0 = _con5953->mb_seconds_in_day;
    kk_std_core_types__maybe _pat40 = _con5953->mb_to_mjd2000;
    kk_std_core_types__maybe _pat50 = _con5953->mb_from_mjd2000;
    if kk_likely(kk_datatype_ptr_is_unique(ts, _ctx)) {
      kk_std_core_types__maybe_drop(_pat50, _ctx);
      kk_std_core_types__maybe_drop(_pat40, _ctx);
      kk_function_drop(_pat30, _ctx);
      kk_function_drop(_pat20, _ctx);
      kk_string_drop(_pat100, _ctx);
      kk_string_drop(_pat00, _ctx);
      kk_datatype_ptr_free(ts, _ctx);
    }
    else {
      kk_std_core_types__maybe_dup(_x0, _ctx);
      kk_datatype_ptr_decref(ts, _ctx);
    }
    if (kk_std_core_types__is_Nothing(_x0, _ctx)) {
      _x5952 = false; /*bool*/
    }
    else {
      kk_std_core_types__maybe_drop(_x0, _ctx);
      _x5952 = true; /*bool*/
    }
  }
  _x5951 = kk_bool_box(_x5952); /*116*/
  _x5950 = kk_std_core_types__new_Optional(_x5951, _ctx); /*optional<116>*/
  return kk_std_time_calendar_dc_timestamp(d, c, _x5948, _x5950, _ctx);
}
extern kk_integer_t kk_std_time_calendar_cal_utc_create_fun5961(kk_function_t _fself, kk_integer_t _x15960, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_days_before_year(_x15960, _ctx);
}
extern kk_std_core_types__tuple2_ kk_std_time_calendar_cal_utc_create_fun5963(kk_function_t _fself, kk_integer_t _x15962, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_estimate_year(_x15962, _ctx);
}
extern kk_integer_t kk_std_time_calendar_cal_utc_create_fun5966(kk_function_t _fself, kk_integer_t _x15964, kk_integer_t _x25965, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_days_before_month(_x15964, _x25965, _ctx);
}
extern kk_integer_t kk_std_time_calendar_cal_utc_create_fun5969(kk_function_t _fself, kk_integer_t _x15967, kk_integer_t _x25968, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_doy_to_month(_x15967, _x25968, _ctx);
}
extern kk_integer_t kk_std_time_calendar_cal_utc_sls_create_fun5975(kk_function_t _fself, kk_integer_t _x15974, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_days_before_year(_x15974, _ctx);
}
extern kk_std_core_types__tuple2_ kk_std_time_calendar_cal_utc_sls_create_fun5977(kk_function_t _fself, kk_integer_t _x15976, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_estimate_year(_x15976, _ctx);
}
extern kk_integer_t kk_std_time_calendar_cal_utc_sls_create_fun5980(kk_function_t _fself, kk_integer_t _x15978, kk_integer_t _x25979, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_days_before_month(_x15978, _x25979, _ctx);
}
extern kk_integer_t kk_std_time_calendar_cal_utc_sls_create_fun5983(kk_function_t _fself, kk_integer_t _x15981, kk_integer_t _x25982, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_doy_to_month(_x15981, _x25982, _ctx);
}
 
// Create a new calendar from by combining two other calendars. The `switch-date`
// is in terms of the second calendar (`cal2`) and time after (and including) the switch
// date is displayed in `cal2` while times before it in `cal1`. This function is used
// for example for the Julian Gregorian calendar.


// lift anonymous function
struct kk_std_time_calendar_combine_earth_calendars_fun5985__t {
  struct kk_function_s _base;
  kk_std_time_calendar__calendar cal1;
  kk_std_time_calendar__calendar cal2;
  kk_std_time_date__date switch_date;
};
static kk_std_time_date__date kk_std_time_calendar_combine_earth_calendars_fun5985(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_combine_earth_calendars_fun5985(kk_std_time_calendar__calendar cal1, kk_std_time_calendar__calendar cal2, kk_std_time_date__date switch_date, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun5985__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_combine_earth_calendars_fun5985__t, 6, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_combine_earth_calendars_fun5985, kk_context());
  _self->cal1 = cal1;
  _self->cal2 = cal2;
  _self->switch_date = switch_date;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_time_date__date kk_std_time_calendar_combine_earth_calendars_fun5985(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun5985__t* _self = kk_function_as(struct kk_std_time_calendar_combine_earth_calendars_fun5985__t*, _fself, _ctx);
  kk_std_time_calendar__calendar cal1 = _self->cal1; /* std/time/calendar/calendar */
  kk_std_time_calendar__calendar cal2 = _self->cal2; /* std/time/calendar/calendar */
  kk_std_time_date__date switch_date = _self->switch_date; /* std/time/date/date */
  kk_drop_match(_self, {kk_std_time_calendar__calendar_dup(cal1, _ctx);kk_std_time_calendar__calendar_dup(cal2, _ctx);kk_std_time_date__date_dup(switch_date, _ctx);}, {}, _ctx)
  kk_std_time_calendar__calendar calendar0_4828;
  bool _match_5367;
  kk_integer_t _brw_5368;
  {
    struct kk_std_time_calendar_Calendar* _con5986 = kk_std_time_calendar__as_Calendar(cal2, _ctx);
    kk_function_t _x = _con5986->date_to_days;
    kk_function_dup(_x, _ctx);
    _brw_5368 = kk_function_call(kk_integer_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _x, (_x, switch_date, _ctx), _ctx); /*int*/
  }
  bool _brw_5369 = kk_integer_lt_borrow(days,_brw_5368,kk_context()); /*bool*/;
  kk_integer_drop(_brw_5368, _ctx);
  _match_5367 = _brw_5369; /*bool*/
  if (_match_5367) {
    kk_datatype_ptr_dropn(cal2, (KK_I32(8)), _ctx);
    calendar0_4828 = cal1; /*std/time/calendar/calendar*/
  }
  else {
    kk_datatype_ptr_dropn(cal1, (KK_I32(8)), _ctx);
    calendar0_4828 = cal2; /*std/time/calendar/calendar*/
  }
  {
    struct kk_std_time_calendar_Calendar* _con5987 = kk_std_time_calendar__as_Calendar(calendar0_4828, _ctx);
    kk_string_t _pat00 = _con5987->name;
    kk_string_t _pat10 = _con5987->long_name;
    kk_string_t _pat20 = _con5987->month_prefix;
    kk_function_t _pat30 = _con5987->show_era;
    kk_function_t _pat40 = _con5987->instant_to_dc;
    kk_function_t _pat50 = _con5987->dc_to_instant;
    kk_function_t _x0 = _con5987->days_to_date;
    kk_function_t _pat60 = _con5987->date_to_days;
    if kk_likely(kk_datatype_ptr_is_unique(calendar0_4828, _ctx)) {
      kk_function_drop(_pat60, _ctx);
      kk_function_drop(_pat50, _ctx);
      kk_function_drop(_pat40, _ctx);
      kk_function_drop(_pat30, _ctx);
      kk_string_drop(_pat20, _ctx);
      kk_string_drop(_pat10, _ctx);
      kk_string_drop(_pat00, _ctx);
      kk_datatype_ptr_free(calendar0_4828, _ctx);
    }
    else {
      kk_function_dup(_x0, _ctx);
      kk_datatype_ptr_decref(calendar0_4828, _ctx);
    }
    return kk_function_call(kk_std_time_date__date, (kk_function_t, kk_integer_t, kk_context_t*), _x0, (_x0, days, _ctx), _ctx);
  }
}


// lift anonymous function
struct kk_std_time_calendar_combine_earth_calendars_fun5989__t {
  struct kk_function_s _base;
  kk_std_time_calendar__calendar cal1;
  kk_std_time_calendar__calendar cal2;
  kk_std_time_date__date switch_date;
};
static kk_integer_t kk_std_time_calendar_combine_earth_calendars_fun5989(kk_function_t _fself, kk_std_time_date__date d, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_combine_earth_calendars_fun5989(kk_std_time_calendar__calendar cal1, kk_std_time_calendar__calendar cal2, kk_std_time_date__date switch_date, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun5989__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_combine_earth_calendars_fun5989__t, 6, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_combine_earth_calendars_fun5989, kk_context());
  _self->cal1 = cal1;
  _self->cal2 = cal2;
  _self->switch_date = switch_date;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_integer_t kk_std_time_calendar_combine_earth_calendars_fun5989(kk_function_t _fself, kk_std_time_date__date d, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun5989__t* _self = kk_function_as(struct kk_std_time_calendar_combine_earth_calendars_fun5989__t*, _fself, _ctx);
  kk_std_time_calendar__calendar cal1 = _self->cal1; /* std/time/calendar/calendar */
  kk_std_time_calendar__calendar cal2 = _self->cal2; /* std/time/calendar/calendar */
  kk_std_time_date__date switch_date = _self->switch_date; /* std/time/date/date */
  kk_drop_match(_self, {kk_std_time_calendar__calendar_dup(cal1, _ctx);kk_std_time_calendar__calendar_dup(cal2, _ctx);kk_std_time_date__date_dup(switch_date, _ctx);}, {}, _ctx)
  kk_std_time_calendar__calendar calendar1_4830;
  bool _match_5366;
  kk_std_core_types__order _x5990;
  kk_std_time_date__date _x5991 = kk_std_time_date__date_dup(d, _ctx); /*std/time/date/date*/
  _x5990 = kk_std_time_date_compare(_x5991, switch_date, _ctx); /*order*/
  _match_5366 = kk_std_core__lp__eq__eq__4_rp_(_x5990, kk_std_core_types__new_Lt(_ctx), _ctx); /*bool*/
  if (_match_5366) {
    kk_datatype_ptr_dropn(cal2, (KK_I32(8)), _ctx);
    calendar1_4830 = cal1; /*std/time/calendar/calendar*/
  }
  else {
    kk_datatype_ptr_dropn(cal1, (KK_I32(8)), _ctx);
    calendar1_4830 = cal2; /*std/time/calendar/calendar*/
  }
  {
    struct kk_std_time_calendar_Calendar* _con5992 = kk_std_time_calendar__as_Calendar(calendar1_4830, _ctx);
    kk_string_t _pat01 = _con5992->name;
    kk_string_t _pat12 = _con5992->long_name;
    kk_string_t _pat22 = _con5992->month_prefix;
    kk_function_t _pat31 = _con5992->show_era;
    kk_function_t _pat41 = _con5992->instant_to_dc;
    kk_function_t _pat51 = _con5992->dc_to_instant;
    kk_function_t _pat61 = _con5992->days_to_date;
    kk_function_t _x1 = _con5992->date_to_days;
    if kk_likely(kk_datatype_ptr_is_unique(calendar1_4830, _ctx)) {
      kk_function_drop(_pat61, _ctx);
      kk_function_drop(_pat51, _ctx);
      kk_function_drop(_pat41, _ctx);
      kk_function_drop(_pat31, _ctx);
      kk_string_drop(_pat22, _ctx);
      kk_string_drop(_pat12, _ctx);
      kk_string_drop(_pat01, _ctx);
      kk_datatype_ptr_free(calendar1_4830, _ctx);
    }
    else {
      kk_function_dup(_x1, _ctx);
      kk_datatype_ptr_decref(calendar1_4830, _ctx);
    }
    return kk_function_call(kk_integer_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _x1, (_x1, d, _ctx), _ctx);
  }
}


// lift anonymous function
struct kk_std_time_calendar_combine_earth_calendars_fun5994__t {
  struct kk_function_s _base;
  kk_std_time_calendar__calendar cal1;
  kk_std_time_calendar__calendar cal2;
  kk_std_core_types__optional mb_show_era;
  kk_std_time_date__date switch_date;
};
static kk_box_t kk_std_time_calendar_combine_earth_calendars_fun5994(kk_function_t _fself, kk_box_t _b_5294, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_combine_earth_calendars_fun5994(kk_std_time_calendar__calendar cal1, kk_std_time_calendar__calendar cal2, kk_std_core_types__optional mb_show_era, kk_std_time_date__date switch_date, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun5994__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_combine_earth_calendars_fun5994__t, 8, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_combine_earth_calendars_fun5994, kk_context());
  _self->cal1 = cal1;
  _self->cal2 = cal2;
  _self->mb_show_era = mb_show_era;
  _self->switch_date = switch_date;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_box_t kk_std_time_calendar_combine_earth_calendars_fun5994(kk_function_t _fself, kk_box_t _b_5294, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun5994__t* _self = kk_function_as(struct kk_std_time_calendar_combine_earth_calendars_fun5994__t*, _fself, _ctx);
  kk_std_time_calendar__calendar cal1 = _self->cal1; /* std/time/calendar/calendar */
  kk_std_time_calendar__calendar cal2 = _self->cal2; /* std/time/calendar/calendar */
  kk_std_core_types__optional mb_show_era = _self->mb_show_era; /* optional<maybe<(std/time/date/date) -> string>> */
  kk_std_time_date__date switch_date = _self->switch_date; /* std/time/date/date */
  kk_drop_match(_self, {kk_std_time_calendar__calendar_dup(cal1, _ctx);kk_std_time_calendar__calendar_dup(cal2, _ctx);kk_std_core_types__optional_dup(mb_show_era, _ctx);kk_std_time_date__date_dup(switch_date, _ctx);}, {}, _ctx)
  kk_string_t _x5995;
  kk_std_time_date__date d0_5296 = kk_std_time_date__date_unbox(_b_5294, KK_OWNED, _ctx); /*std/time/date/date*/;
  kk_std_core_types__maybe _match_5364;
  if (kk_std_core_types__is_Optional(mb_show_era, _ctx)) {
    kk_box_t _box_x5283 = mb_show_era._cons.Optional.value;
    kk_std_core_types__maybe _mb_show_era_4195 = kk_std_core_types__maybe_unbox(_box_x5283, KK_BORROWED, _ctx);
    kk_std_core_types__maybe_dup(_mb_show_era_4195, _ctx);
    kk_std_core_types__optional_drop(mb_show_era, _ctx);
    _match_5364 = _mb_show_era_4195; /*maybe<(std/time/date/date) -> string>*/
  }
  else {
    kk_std_core_types__optional_drop(mb_show_era, _ctx);
    _match_5364 = kk_std_core_types__new_Nothing(_ctx); /*maybe<(std/time/date/date) -> string>*/
  }
  if (kk_std_core_types__is_Just(_match_5364, _ctx)) {
    kk_box_t _fun_unbox_x5287 = _match_5364._cons.Just.value;
    kk_std_time_date__date_drop(switch_date, _ctx);
    kk_datatype_ptr_dropn(cal2, (KK_I32(8)), _ctx);
    kk_datatype_ptr_dropn(cal1, (KK_I32(8)), _ctx);
    kk_box_t _x5996;
    kk_function_t _x5997 = kk_function_unbox(_fun_unbox_x5287, _ctx); /*(5288) -> 5289*/
    _x5996 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x5997, (_x5997, kk_std_time_date__date_box(d0_5296, _ctx), _ctx), _ctx); /*5289*/
    _x5995 = kk_string_unbox(_x5996); /*string*/
  }
  else {
    kk_std_time_calendar__calendar calendar2_4833;
    bool _match_5365;
    kk_std_core_types__order _x5998;
    kk_std_time_date__date _x5999 = kk_std_time_date__date_dup(d0_5296, _ctx); /*std/time/date/date*/
    _x5998 = kk_std_time_date_compare(_x5999, switch_date, _ctx); /*order*/
    _match_5365 = kk_std_core__lp__eq__eq__4_rp_(_x5998, kk_std_core_types__new_Lt(_ctx), _ctx); /*bool*/
    if (_match_5365) {
      kk_datatype_ptr_dropn(cal2, (KK_I32(8)), _ctx);
      calendar2_4833 = cal1; /*std/time/calendar/calendar*/
    }
    else {
      kk_datatype_ptr_dropn(cal1, (KK_I32(8)), _ctx);
      calendar2_4833 = cal2; /*std/time/calendar/calendar*/
    }
    {
      struct kk_std_time_calendar_Calendar* _con6000 = kk_std_time_calendar__as_Calendar(calendar2_4833, _ctx);
      kk_string_t _pat03 = _con6000->name;
      kk_string_t _pat14 = _con6000->long_name;
      kk_string_t _pat23 = _con6000->month_prefix;
      kk_function_t _x2 = _con6000->show_era;
      kk_function_t _pat33 = _con6000->instant_to_dc;
      kk_function_t _pat43 = _con6000->dc_to_instant;
      kk_function_t _pat53 = _con6000->days_to_date;
      kk_function_t _pat63 = _con6000->date_to_days;
      if kk_likely(kk_datatype_ptr_is_unique(calendar2_4833, _ctx)) {
        kk_function_drop(_pat63, _ctx);
        kk_function_drop(_pat53, _ctx);
        kk_function_drop(_pat43, _ctx);
        kk_function_drop(_pat33, _ctx);
        kk_string_drop(_pat23, _ctx);
        kk_string_drop(_pat14, _ctx);
        kk_string_drop(_pat03, _ctx);
        kk_datatype_ptr_free(calendar2_4833, _ctx);
      }
      else {
        kk_function_dup(_x2, _ctx);
        kk_datatype_ptr_decref(calendar2_4833, _ctx);
      }
      _x5995 = kk_function_call(kk_string_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _x2, (_x2, d0_5296, _ctx), _ctx); /*string*/
    }
  }
  return kk_string_box(_x5995);
}

kk_std_time_calendar__calendar kk_std_time_calendar_combine_earth_calendars(kk_string_t name0, kk_string_t long_name0, kk_std_time_date__date switch_date, kk_std_time_calendar__calendar cal1, kk_std_time_calendar__calendar cal2, kk_std_core_types__optional mb_show_era, kk_context_t* _ctx) { /* (name : string, long-name : string, switch-date : std/time/date/date, cal1 : calendar, cal2 : calendar, mb-show-era : optional<maybe<(std/time/date/date) -> string>>) -> calendar */ 
  kk_function_t _x5984;
  kk_std_time_calendar__calendar_dup(cal1, _ctx);
  kk_std_time_calendar__calendar_dup(cal2, _ctx);
  kk_std_time_date__date_dup(switch_date, _ctx);
  _x5984 = kk_std_time_calendar_new_combine_earth_calendars_fun5985(cal1, cal2, switch_date, _ctx); /*(days : int) -> std/time/date/date*/
  kk_function_t _x5988;
  kk_std_time_calendar__calendar_dup(cal1, _ctx);
  kk_std_time_calendar__calendar_dup(cal2, _ctx);
  kk_std_time_date__date_dup(switch_date, _ctx);
  _x5988 = kk_std_time_calendar_new_combine_earth_calendars_fun5989(cal1, cal2, switch_date, _ctx); /*(d : std/time/date/date) -> int*/
  kk_std_core_types__optional _x5993 = kk_std_core_types__new_Optional(kk_function_box(kk_std_time_calendar_new_combine_earth_calendars_fun5994(cal1, cal2, mb_show_era, switch_date, _ctx), _ctx), _ctx); /*optional<116>*/
  return kk_std_time_calendar_earth_calendar(name0, long_name0, _x5984, _x5988, kk_std_core_types__new_None(_ctx), _x5993, _ctx);
}
 
// Convert an `:instant` to a `:date`, `:clock`, timezone delta and abbreviation,
// for a given timezone `tz` (=`tz-utc` by default) and calendar (=`cal-iso` by default).

kk_std_core_types__tuple4_ kk_std_time_calendar_instant_dc(kk_std_time_instant__instant i, kk_std_core_types__optional tz, kk_std_core_types__optional cal, kk_context_t* _ctx) { /* (i : std/time/instant/instant, tz : optional<timezone>, cal : optional<calendar>) -> (std/time/date/date, std/time/date/clock, std/time/duration/duration, string) */ 
  kk_std_time_calendar__timezone _match_5360;
  if (kk_std_core_types__is_Optional(tz, _ctx)) {
    kk_box_t _box_x5297 = tz._cons.Optional.value;
    kk_std_time_calendar__timezone _tz_4331 = kk_std_time_calendar__timezone_unbox(_box_x5297, KK_BORROWED, _ctx);
    kk_std_time_calendar__timezone_dup(_tz_4331, _ctx);
    kk_std_core_types__optional_drop(tz, _ctx);
    _match_5360 = _tz_4331; /*std/time/calendar/timezone*/
  }
  else {
    kk_std_core_types__optional_drop(tz, _ctx);
    _match_5360 = kk_std_time_calendar__timezone_dup(kk_std_time_calendar_tz_utc, _ctx); /*std/time/calendar/timezone*/
  }
  {
    struct kk_std_time_calendar_Timezone* _con6001 = kk_std_time_calendar__as_Timezone(_match_5360, _ctx);
    kk_string_t _pat0 = _con6001->name;
    kk_function_t _x = _con6001->utc_delta;
    kk_function_t _pat1 = _con6001->utc_inverse;
    if kk_likely(kk_datatype_ptr_is_unique(_match_5360, _ctx)) {
      kk_function_drop(_pat1, _ctx);
      kk_string_drop(_pat0, _ctx);
      kk_datatype_ptr_free(_match_5360, _ctx);
    }
    else {
      kk_function_dup(_x, _ctx);
      kk_datatype_ptr_decref(_match_5360, _ctx);
    }
    kk_std_core_types__tuple2_ _match_5361;
    kk_std_time_instant__instant _x6002 = kk_std_time_instant__instant_dup(i, _ctx); /*std/time/instant/instant*/
    _match_5361 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x, (_x, _x6002, _ctx), _ctx); /*(std/time/duration/duration, string)*/
    {
      kk_box_t _box_x5298 = _match_5361.fst;
      kk_box_t _box_x5299 = _match_5361.snd;
      kk_std_time_duration__duration tzdelta = kk_std_time_duration__duration_unbox(_box_x5298, KK_BORROWED, _ctx);
      kk_string_t tzabbrv = kk_string_unbox(_box_x5299);
      kk_string_dup(tzabbrv, _ctx);
      kk_std_core_types__tuple2__drop(_match_5361, _ctx);
      kk_std_time_calendar__calendar _match_5362;
      if (kk_std_core_types__is_Optional(cal, _ctx)) {
        kk_box_t _box_x5300 = cal._cons.Optional.value;
        kk_std_time_calendar__calendar _cal_4335 = kk_std_time_calendar__calendar_unbox(_box_x5300, KK_BORROWED, _ctx);
        kk_std_time_calendar__calendar_dup(_cal_4335, _ctx);
        kk_std_core_types__optional_drop(cal, _ctx);
        _match_5362 = _cal_4335; /*std/time/calendar/calendar*/
      }
      else {
        kk_std_core_types__optional_drop(cal, _ctx);
        _match_5362 = kk_std_time_calendar__calendar_dup(kk_std_time_calendar_cal_iso, _ctx); /*std/time/calendar/calendar*/
      }
      {
        struct kk_std_time_calendar_Calendar* _con6003 = kk_std_time_calendar__as_Calendar(_match_5362, _ctx);
        kk_string_t _pat01 = _con6003->name;
        kk_string_t _pat10 = _con6003->long_name;
        kk_string_t _pat20 = _con6003->month_prefix;
        kk_function_t _pat30 = _con6003->show_era;
        kk_function_t _x0 = _con6003->instant_to_dc;
        kk_function_t _pat40 = _con6003->dc_to_instant;
        kk_function_t _pat50 = _con6003->days_to_date;
        kk_function_t _pat6 = _con6003->date_to_days;
        if kk_likely(kk_datatype_ptr_is_unique(_match_5362, _ctx)) {
          kk_function_drop(_pat6, _ctx);
          kk_function_drop(_pat50, _ctx);
          kk_function_drop(_pat40, _ctx);
          kk_function_drop(_pat30, _ctx);
          kk_string_drop(_pat20, _ctx);
          kk_string_drop(_pat10, _ctx);
          kk_string_drop(_pat01, _ctx);
          kk_datatype_ptr_free(_match_5362, _ctx);
        }
        else {
          kk_function_dup(_x0, _ctx);
          kk_datatype_ptr_decref(_match_5362, _ctx);
        }
        kk_std_core_types__tuple2_ _match_5363 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_std_time_instant__instant, kk_std_time_duration__duration, kk_context_t*), _x0, (_x0, i, tzdelta, _ctx), _ctx); /*(std/time/date/date, std/time/date/clock)*/;
        {
          kk_box_t _box_x5301 = _match_5363.fst;
          kk_box_t _box_x5302 = _match_5363.snd;
          kk_std_time_date__date d = kk_std_time_date__date_unbox(_box_x5301, KK_BORROWED, _ctx);
          kk_std_time_date__clock c = kk_std_time_date__clock_unbox(_box_x5302, KK_BORROWED, _ctx);
          struct kk_std_time_date_Clock* _con6004 = kk_std_time_date__as_Clock(c, _ctx);
          kk_std_time_date__clock_dup(c, _ctx);
          kk_std_time_date__date_dup(d, _ctx);
          kk_std_core_types__tuple2__drop(_match_5363, _ctx);
          return kk_std_core_types__new_dash__lp__comma__comma__comma__rp_(kk_reuse_null, 0, kk_std_time_date__date_box(d, _ctx), kk_std_time_date__clock_box(c, _ctx), kk_std_time_duration__duration_box(tzdelta, _ctx), kk_string_box(tzabbrv), _ctx);
        }
      }
    }
  }
}
 
// Return the local timezone on the current system.


// lift anonymous function
struct kk_std_time_calendar_tz_local_fun6007__t {
  struct kk_function_s _base;
  kk_std_time_calendar__local_timezone tz;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_local_fun6007(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_local_fun6007(kk_std_time_calendar__local_timezone tz, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_local_fun6007__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_tz_local_fun6007__t, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_tz_local_fun6007, kk_context());
  _self->tz = tz;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_local_fun6007(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_local_fun6007__t* _self = kk_function_as(struct kk_std_time_calendar_tz_local_fun6007__t*, _fself, _ctx);
  kk_std_time_calendar__local_timezone tz = _self->tz; /* std/time/calendar/local-timezone */
  kk_drop_match(_self, {kk_std_time_calendar__local_timezone_dup(tz, _ctx);}, {}, _ctx)
  kk_std_num_ddouble__ddouble x_4841;
  kk_std_num_ddouble__ddouble _x6008;
  kk_std_time_timestamp__timestamp _x6009;
  kk_std_time_timestamp__timestamp _x6010;
  {
    kk_std_time_timestamp__timestamp _x0 = i0.since;
    kk_std_time_instant__timescale _pat20 = i0.ts;
    struct kk_std_time_timestamp_Timestamp* _con6011 = kk_std_time_timestamp__as_Timestamp(_x0, _ctx);
    struct kk_std_time_instant_Timescale* _con6012 = kk_std_time_instant__as_Timescale(_pat20, _ctx);
    kk_std_time_timestamp__timestamp_dup(_x0, _ctx);
    _x6010 = _x0; /*std/time/timestamp/timestamp*/
  }
  kk_std_time_instant__timescale _x6013;
  {
    kk_std_time_timestamp__timestamp _pat000 = i0.since;
    kk_std_time_instant__timescale _x00 = i0.ts;
    struct kk_std_time_timestamp_Timestamp* _con6014 = kk_std_time_timestamp__as_Timestamp(_pat000, _ctx);
    struct kk_std_time_instant_Timescale* _con6015 = kk_std_time_instant__as_Timescale(_x00, _ctx);
    kk_std_time_instant__timescale_dup(_x00, _ctx);
    kk_std_time_instant__instant_drop(i0, _ctx);
    _x6013 = _x00; /*std/time/instant/timescale*/
  }
  kk_std_time_instant__timescale _x6016 = kk_std_time_instant__timescale_dup(kk_std_time_utc_ts_ti, _ctx); /*std/time/instant/timescale*/
  _x6009 = kk_std_time_instant_convert(_x6010, _x6013, _x6016, _ctx); /*std/time/timestamp/timestamp*/
  _x6008 = kk_std_time_timestamp_unsafe_timespan_withleap(_x6009, _ctx); /*std/time/timestamp/timespan*/
  x_4841 = kk_std_num_ddouble__lp__plus__rp_(kk_std_time_utc_unix2000, _x6008, _ctx); /*std/num/ddouble/ddouble*/
  kk_std_core_types__tuple2_ _match_5359;
  double _x6017;
  {
    double _x = x_4841.hi;
    _x6017 = _x; /*float64*/
  }
  _match_5359 = kk_std_time_calendar_local_utc_delta(tz, _x6017, _ctx); /*(float64, string)*/
  {
    kk_box_t _box_x5311 = _match_5359.fst;
    kk_box_t _box_x5312 = _match_5359.snd;
    double ofs = kk_double_unbox(_box_x5311, KK_BORROWED, _ctx);
    kk_string_t abbrv = kk_string_unbox(_box_x5312);
    kk_string_dup(abbrv, _ctx);
    kk_std_core_types__tuple2__drop(_match_5359, _ctx);
    kk_box_t _x6018;
    kk_std_time_duration__duration _x6019;
    kk_std_num_ddouble__ddouble _x6020 = kk_std_num_ddouble__new_Ddouble(ofs, 0x0p+0, _ctx); /*std/num/ddouble/ddouble*/
    _x6019 = kk_std_time_duration__new_Duration(_x6020, _ctx); /*std/time/duration/duration*/
    _x6018 = kk_std_time_duration__duration_box(_x6019, _ctx); /*6*/
    return kk_std_core_types__new_dash__lp__comma__rp_(_x6018, kk_string_box(abbrv), _ctx);
  }
}


// lift anonymous function
struct kk_std_time_calendar_tz_local_fun6022__t {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x5318;
};
static kk_std_core_types__maybe kk_std_time_calendar_tz_local_fun6022(kk_function_t _fself, kk_std_time_instant__instant _b_5321, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_local_fun6022(kk_box_t _fun_unbox_x5318, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_local_fun6022__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_tz_local_fun6022__t, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_tz_local_fun6022, kk_context());
  _self->_fun_unbox_x5318 = _fun_unbox_x5318;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_core_types__maybe kk_std_time_calendar_tz_local_fun6022(kk_function_t _fself, kk_std_time_instant__instant _b_5321, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_local_fun6022__t* _self = kk_function_as(struct kk_std_time_calendar_tz_local_fun6022__t*, _fself, _ctx);
  kk_box_t _fun_unbox_x5318 = _self->_fun_unbox_x5318; /* 116 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x5318, _ctx);}, {}, _ctx)
  kk_box_t _x6023;
  kk_function_t _x6024 = kk_function_unbox(_fun_unbox_x5318, _ctx); /*(5319) -> 5320*/
  _x6023 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6024, (_x6024, kk_std_time_instant__instant_box(_b_5321, _ctx), _ctx), _ctx); /*5320*/
  return kk_std_core_types__maybe_unbox(_x6023, KK_OWNED, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_tz_local_fun6025__t {
  struct kk_function_s _base;
};
static kk_std_core_types__maybe kk_std_time_calendar_tz_local_fun6025(kk_function_t _fself, kk_std_time_instant__instant i, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_local_fun6025(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_tz_local_fun6025, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_std_core_types__maybe kk_std_time_calendar_tz_local_fun6025(kk_function_t _fself, kk_std_time_instant__instant i, kk_context_t* _ctx) {
  kk_unused(_fself);
  kk_std_time_instant__instant_drop(i, _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}

kk_std_time_calendar__timezone kk_std_time_calendar_tz_local(kk_context_t* _ctx) { /* () -> ndet timezone */ 
  kk_std_time_calendar__local_timezone tz = kk_std_time_calendar_local_get_timezone(_ctx); /*std/time/calendar/local-timezone*/;
  kk_string_t _x6005 = kk_string_empty(); /*string*/
  kk_function_t _x6021;
  kk_std_core_types__optional _match_5358 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5358, _ctx)) {
    kk_box_t _fun_unbox_x5318 = _match_5358._cons.Optional.value;
    kk_box_dup(_fun_unbox_x5318, _ctx);
    kk_std_core_types__optional_drop(_match_5358, _ctx);
    _x6021 = kk_std_time_calendar_new_tz_local_fun6022(_fun_unbox_x5318, _ctx); /*(std/time/instant/instant) -> maybe<std/time/instant/instant>*/
  }
  else {
    kk_std_core_types__optional_drop(_match_5358, _ctx);
    _x6021 = kk_std_time_calendar_new_tz_local_fun6025(_ctx); /*(std/time/instant/instant) -> maybe<std/time/instant/instant>*/
  }
  return kk_std_time_calendar__new_Timezone(kk_reuse_null, 0, _x6005, kk_std_time_calendar_new_tz_local_fun6007(tz, _ctx), _x6021, _ctx);
}
 
// Return the day of the week for a calendar `cal` (=`cal-iso`).

kk_std_time_date__weekday kk_std_time_calendar_weekday(kk_std_time_date__date d, kk_std_time_calendar__calendar cal, kk_context_t* _ctx) { /* (d : std/time/date/date, cal : calendar) -> std/time/date/weekday */ 
  kk_integer_t x_4844;
  {
    struct kk_std_time_calendar_Calendar* _con6026 = kk_std_time_calendar__as_Calendar(cal, _ctx);
    kk_string_t _pat0 = _con6026->name;
    kk_string_t _pat1 = _con6026->long_name;
    kk_string_t _pat2 = _con6026->month_prefix;
    kk_function_t _pat3 = _con6026->show_era;
    kk_function_t _pat4 = _con6026->instant_to_dc;
    kk_function_t _pat5 = _con6026->dc_to_instant;
    kk_function_t _pat6 = _con6026->days_to_date;
    kk_function_t _x = _con6026->date_to_days;
    if kk_likely(kk_datatype_ptr_is_unique(cal, _ctx)) {
      kk_function_drop(_pat6, _ctx);
      kk_function_drop(_pat5, _ctx);
      kk_function_drop(_pat4, _ctx);
      kk_function_drop(_pat3, _ctx);
      kk_string_drop(_pat2, _ctx);
      kk_string_drop(_pat1, _ctx);
      kk_string_drop(_pat0, _ctx);
      kk_datatype_ptr_free(cal, _ctx);
    }
    else {
      kk_function_dup(_x, _ctx);
      kk_datatype_ptr_decref(cal, _ctx);
    }
    x_4844 = kk_function_call(kk_integer_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _x, (_x, d, _ctx), _ctx); /*int*/
  }
  kk_integer_t dow;
  kk_integer_t _x6027 = kk_integer_add_small_const(x_4844, 6, _ctx); /*int*/
  dow = kk_integer_mod(_x6027,(kk_integer_from_small(7)),kk_context()); /*int*/
  return kk_std_time_date_weekday(dow, _ctx);
}
 
// Create a new calendar from anoter calendar `cal` by adding an offset to the
// years (`year-shift`). This is used for example to create the Ethiopian calendar
// from the Coptic calendar, by using:
// `year-shift-earth-calendar( "EC", "Ethiopian", ~276, cal-coptic )`


// lift anonymous function
struct kk_std_time_calendar_year_shift_earth_calendar_fun6029__t {
  struct kk_function_s _base;
  kk_std_time_calendar__calendar cal;
  kk_integer_t year_shift;
};
static kk_std_time_date__date kk_std_time_calendar_year_shift_earth_calendar_fun6029(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_year_shift_earth_calendar_fun6029(kk_std_time_calendar__calendar cal, kk_integer_t year_shift, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6029__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6029__t, 3, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_year_shift_earth_calendar_fun6029, kk_context());
  _self->cal = cal;
  _self->year_shift = year_shift;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_time_date__date kk_std_time_calendar_year_shift_earth_calendar_fun6029(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6029__t* _self = kk_function_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6029__t*, _fself, _ctx);
  kk_std_time_calendar__calendar cal = _self->cal; /* std/time/calendar/calendar */
  kk_integer_t year_shift = _self->year_shift; /* int */
  kk_drop_match(_self, {kk_std_time_calendar__calendar_dup(cal, _ctx);kk_integer_dup(year_shift, _ctx);}, {}, _ctx)
  kk_std_time_date__date d0;
  {
    struct kk_std_time_calendar_Calendar* _con6030 = kk_std_time_calendar__as_Calendar(cal, _ctx);
    kk_string_t _pat0 = _con6030->name;
    kk_string_t _pat1 = _con6030->long_name;
    kk_string_t _pat2 = _con6030->month_prefix;
    kk_function_t _pat3 = _con6030->show_era;
    kk_function_t _pat4 = _con6030->instant_to_dc;
    kk_function_t _pat5 = _con6030->dc_to_instant;
    kk_function_t _x = _con6030->days_to_date;
    kk_function_t _pat6 = _con6030->date_to_days;
    if kk_likely(kk_datatype_ptr_is_unique(cal, _ctx)) {
      kk_function_drop(_pat6, _ctx);
      kk_function_drop(_pat5, _ctx);
      kk_function_drop(_pat4, _ctx);
      kk_function_drop(_pat3, _ctx);
      kk_string_drop(_pat2, _ctx);
      kk_string_drop(_pat1, _ctx);
      kk_string_drop(_pat0, _ctx);
      kk_datatype_ptr_free(cal, _ctx);
    }
    else {
      kk_function_dup(_x, _ctx);
      kk_datatype_ptr_decref(cal, _ctx);
    }
    d0 = kk_function_call(kk_std_time_date__date, (kk_function_t, kk_integer_t, kk_context_t*), _x, (_x, days, _ctx), _ctx); /*std/time/date/date*/
  }
  kk_integer_t _b_5325_5324;
  kk_integer_t _x6031;
  {
    kk_integer_t _x2 = d0.year;
    kk_integer_dup(_x2, _ctx);
    _x6031 = _x2; /*int*/
  }
  _b_5325_5324 = kk_integer_sub(_x6031,year_shift,kk_context()); /*int*/
  kk_integer_t _x6032;
  kk_std_core_types__optional _match_5357 = kk_std_core_types__new_Optional(kk_integer_box(_b_5325_5324, _ctx), _ctx); /*optional<116>*/;
  if (kk_std_core_types__is_Optional(_match_5357, _ctx)) {
    kk_box_t _box_x5326 = _match_5357._cons.Optional.value;
    kk_integer_t _year_143 = kk_integer_unbox(_box_x5326, _ctx);
    kk_integer_dup(_year_143, _ctx);
    kk_std_core_types__optional_drop(_match_5357, _ctx);
    _x6032 = _year_143; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(_match_5357, _ctx);
    {
      kk_integer_t _x0 = d0.year;
      kk_integer_dup(_x0, _ctx);
      _x6032 = _x0; /*int*/
    }
  }
  kk_integer_t _x6033;
  kk_std_core_types__optional _match_5356 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5356, _ctx)) {
    kk_box_t _box_x5327 = _match_5356._cons.Optional.value;
    kk_integer_t _month_149 = kk_integer_unbox(_box_x5327, _ctx);
    kk_integer_dup(_month_149, _ctx);
    kk_std_core_types__optional_drop(_match_5356, _ctx);
    _x6033 = _month_149; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(_match_5356, _ctx);
    {
      kk_integer_t _x00 = d0.month;
      kk_integer_dup(_x00, _ctx);
      _x6033 = _x00; /*int*/
    }
  }
  kk_integer_t _x6034;
  kk_std_core_types__optional _match_5355 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5355, _ctx)) {
    kk_box_t _box_x5328 = _match_5355._cons.Optional.value;
    kk_integer_t _day_155 = kk_integer_unbox(_box_x5328, _ctx);
    kk_std_time_date__date_drop(d0, _ctx);
    kk_integer_dup(_day_155, _ctx);
    kk_std_core_types__optional_drop(_match_5355, _ctx);
    _x6034 = _day_155; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(_match_5355, _ctx);
    {
      kk_integer_t _x1 = d0.day;
      kk_integer_dup(_x1, _ctx);
      kk_std_time_date__date_drop(d0, _ctx);
      _x6034 = _x1; /*int*/
    }
  }
  return kk_std_time_date__new_Date(_x6032, _x6033, _x6034, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_year_shift_earth_calendar_fun6035__t {
  struct kk_function_s _base;
  kk_std_time_calendar__calendar cal;
  kk_integer_t year_shift;
};
static kk_integer_t kk_std_time_calendar_year_shift_earth_calendar_fun6035(kk_function_t _fself, kk_std_time_date__date d1, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_year_shift_earth_calendar_fun6035(kk_std_time_calendar__calendar cal, kk_integer_t year_shift, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6035__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6035__t, 3, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_year_shift_earth_calendar_fun6035, kk_context());
  _self->cal = cal;
  _self->year_shift = year_shift;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_integer_t kk_std_time_calendar_year_shift_earth_calendar_fun6035(kk_function_t _fself, kk_std_time_date__date d1, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6035__t* _self = kk_function_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6035__t*, _fself, _ctx);
  kk_std_time_calendar__calendar cal = _self->cal; /* std/time/calendar/calendar */
  kk_integer_t year_shift = _self->year_shift; /* int */
  kk_drop_match(_self, {kk_std_time_calendar__calendar_dup(cal, _ctx);kk_integer_dup(year_shift, _ctx);}, {}, _ctx)
  kk_integer_t _b_5330_5329;
  kk_integer_t _x6036;
  {
    kk_integer_t _x5 = d1.year;
    kk_integer_dup(_x5, _ctx);
    _x6036 = _x5; /*int*/
  }
  _b_5330_5329 = kk_integer_add(_x6036,year_shift,kk_context()); /*int*/
  {
    struct kk_std_time_calendar_Calendar* _con6037 = kk_std_time_calendar__as_Calendar(cal, _ctx);
    kk_string_t _pat04 = _con6037->name;
    kk_string_t _pat15 = _con6037->long_name;
    kk_string_t _pat21 = _con6037->month_prefix;
    kk_function_t _pat31 = _con6037->show_era;
    kk_function_t _pat41 = _con6037->instant_to_dc;
    kk_function_t _pat51 = _con6037->dc_to_instant;
    kk_function_t _pat60 = _con6037->days_to_date;
    kk_function_t _x3 = _con6037->date_to_days;
    if kk_likely(kk_datatype_ptr_is_unique(cal, _ctx)) {
      kk_function_drop(_pat60, _ctx);
      kk_function_drop(_pat51, _ctx);
      kk_function_drop(_pat41, _ctx);
      kk_function_drop(_pat31, _ctx);
      kk_string_drop(_pat21, _ctx);
      kk_string_drop(_pat15, _ctx);
      kk_string_drop(_pat04, _ctx);
      kk_datatype_ptr_free(cal, _ctx);
    }
    else {
      kk_function_dup(_x3, _ctx);
      kk_datatype_ptr_decref(cal, _ctx);
    }
    kk_std_time_date__date _x6038;
    kk_integer_t _x6039;
    kk_std_core_types__optional _match_5354 = kk_std_core_types__new_Optional(kk_integer_box(_b_5330_5329, _ctx), _ctx); /*optional<116>*/;
    if (kk_std_core_types__is_Optional(_match_5354, _ctx)) {
      kk_box_t _box_x5331 = _match_5354._cons.Optional.value;
      kk_integer_t _year_1430 = kk_integer_unbox(_box_x5331, _ctx);
      kk_integer_dup(_year_1430, _ctx);
      kk_std_core_types__optional_drop(_match_5354, _ctx);
      _x6039 = _year_1430; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(_match_5354, _ctx);
      {
        kk_integer_t _x4 = d1.year;
        kk_integer_dup(_x4, _ctx);
        _x6039 = _x4; /*int*/
      }
    }
    kk_integer_t _x6040;
    kk_std_core_types__optional _match_5353 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_5353, _ctx)) {
      kk_box_t _box_x5332 = _match_5353._cons.Optional.value;
      kk_integer_t _month_1490 = kk_integer_unbox(_box_x5332, _ctx);
      kk_integer_dup(_month_1490, _ctx);
      kk_std_core_types__optional_drop(_match_5353, _ctx);
      _x6040 = _month_1490; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(_match_5353, _ctx);
      {
        kk_integer_t _x01 = d1.month;
        kk_integer_dup(_x01, _ctx);
        _x6040 = _x01; /*int*/
      }
    }
    kk_integer_t _x6041;
    kk_std_core_types__optional _match_5352 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_5352, _ctx)) {
      kk_box_t _box_x5333 = _match_5352._cons.Optional.value;
      kk_integer_t _day_1550 = kk_integer_unbox(_box_x5333, _ctx);
      kk_std_time_date__date_drop(d1, _ctx);
      kk_integer_dup(_day_1550, _ctx);
      kk_std_core_types__optional_drop(_match_5352, _ctx);
      _x6041 = _day_1550; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(_match_5352, _ctx);
      {
        kk_integer_t _x10 = d1.day;
        kk_integer_dup(_x10, _ctx);
        kk_std_time_date__date_drop(d1, _ctx);
        _x6041 = _x10; /*int*/
      }
    }
    _x6038 = kk_std_time_date__new_Date(_x6039, _x6040, _x6041, _ctx); /*std/time/date/date*/
    return kk_function_call(kk_integer_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _x3, (_x3, _x6038, _ctx), _ctx);
  }
}


// lift anonymous function
struct kk_std_time_calendar_year_shift_earth_calendar_fun6047__t {
  struct kk_function_s _base;
  kk_std_core_types__optional show_era0;
};
static kk_box_t kk_std_time_calendar_year_shift_earth_calendar_fun6047(kk_function_t _fself, kk_box_t _b_5346, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_year_shift_earth_calendar_fun6047(kk_std_core_types__optional show_era0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6047__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6047__t, 3, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_year_shift_earth_calendar_fun6047, kk_context());
  _self->show_era0 = show_era0;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_box_t kk_std_time_calendar_year_shift_earth_calendar_fun6047(kk_function_t _fself, kk_box_t _b_5346, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6047__t* _self = kk_function_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6047__t*, _fself, _ctx);
  kk_std_core_types__optional show_era0 = _self->show_era0; /* optional<(std/time/date/date) -> string> */
  kk_drop_match(_self, {kk_std_core_types__optional_dup(show_era0, _ctx);}, {}, _ctx)
  kk_string_t _x6048;
  kk_std_time_date__date _norm_5349 = kk_std_time_date__date_unbox(_b_5346, KK_OWNED, _ctx); /*std/time/date/date*/;
  if (kk_std_core_types__is_Optional(show_era0, _ctx)) {
    kk_box_t _fun_unbox_x5339 = show_era0._cons.Optional.value;
    kk_box_dup(_fun_unbox_x5339, _ctx);
    kk_std_core_types__optional_drop(show_era0, _ctx);
    kk_std_time_date__date _b_5350_5342 = _norm_5349; /*std/time/date/date*/;
    kk_box_t _x6049;
    kk_function_t _x6050 = kk_function_unbox(_fun_unbox_x5339, _ctx); /*(5340) -> 5341*/
    _x6049 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6050, (_x6050, kk_std_time_date__date_box(_b_5350_5342, _ctx), _ctx), _ctx); /*5341*/
    _x6048 = kk_string_unbox(_x6049); /*string*/
  }
  else {
    kk_std_core_types__optional_drop(show_era0, _ctx);
    kk_std_time_date__date d_5351 = _norm_5349; /*std/time/date/date*/;
    kk_std_time_date__date_drop(d_5351, _ctx);
    _x6048 = kk_string_empty(); /*string*/
  }
  return kk_string_box(_x6048);
}

kk_std_time_calendar__calendar kk_std_time_calendar_year_shift_earth_calendar(kk_string_t name0, kk_string_t long_name0, kk_integer_t year_shift, kk_std_time_calendar__calendar cal, kk_std_core_types__optional month_prefix0, kk_std_core_types__optional show_era0, kk_context_t* _ctx) { /* (name : string, long-name : string, year-shift : int, cal : calendar, month-prefix : optional<string>, show-era : optional<(std/time/date/date) -> string>) -> calendar */ 
  kk_function_t _x6028;
  kk_std_time_calendar__calendar_dup(cal, _ctx);
  kk_integer_dup(year_shift, _ctx);
  _x6028 = kk_std_time_calendar_new_year_shift_earth_calendar_fun6029(cal, year_shift, _ctx); /*(days : int) -> std/time/date/date*/
  kk_std_core_types__optional _x6042;
  kk_box_t _x6043;
  kk_string_t _x6044;
  if (kk_std_core_types__is_Optional(month_prefix0, _ctx)) {
    kk_box_t _box_x5334 = month_prefix0._cons.Optional.value;
    kk_string_t _month_prefix_4475 = kk_string_unbox(_box_x5334);
    kk_string_dup(_month_prefix_4475, _ctx);
    kk_std_core_types__optional_drop(month_prefix0, _ctx);
    _x6044 = _month_prefix_4475; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(month_prefix0, _ctx);
    _x6044 = kk_string_empty(); /*string*/
  }
  _x6043 = kk_string_box(_x6044); /*116*/
  _x6042 = kk_std_core_types__new_Optional(_x6043, _ctx); /*optional<116>*/
  kk_std_core_types__optional _x6046 = kk_std_core_types__new_Optional(kk_function_box(kk_std_time_calendar_new_year_shift_earth_calendar_fun6047(show_era0, _ctx), _ctx), _ctx); /*optional<116>*/
  return kk_std_time_calendar_earth_calendar(name0, long_name0, _x6028, kk_std_time_calendar_new_year_shift_earth_calendar_fun6035(cal, year_shift, _ctx), _x6042, _x6046, _ctx);
}

// initialization
void kk_std_time_calendar__init(kk_context_t* _ctx){
  static bool _kk_initialized = false;
  if (_kk_initialized) return;
  _kk_initialized = true;
  kk_std_core_types__init(_ctx);
  kk_std_core_hnd__init(_ctx);
  kk_std_core__init(_ctx);
  kk_std_num_float64__init(_ctx);
  kk_std_num_ddouble__init(_ctx);
  kk_std_time_date__init(_ctx);
  kk_std_time_timestamp__init(_ctx);
  kk_std_time_duration__init(_ctx);
  kk_std_time_instant__init(_ctx);
  kk_std_time_utc__init(_ctx);
  #if defined(KK_CUSTOM_INIT)
    KK_CUSTOM_INIT (_ctx);
  #endif
  {
    kk_string_t _x5615;
    kk_define_string_literal(, _s5616, 3, "UTC", _ctx)
    _x5615 = kk_string_dup(_s5616, _ctx); /*string*/
    kk_std_time_calendar_tz_utc = kk_std_time_calendar__new_Timezone(kk_reuse_null, 0, _x5615, kk_std_time_calendar_new_tz_utc_fun5617(_ctx), kk_std_time_calendar_new_tz_utc_fun5621(_ctx), _ctx); /*std/time/calendar/timezone*/
  }
  {
    kk_string_t _x5809 = kk_string_empty(); /*string*/
    kk_string_t _x5811;
    kk_define_string_literal(, _s5812, 3, "ISO", _ctx)
    _x5811 = kk_string_dup(_s5812, _ctx); /*string*/
    kk_std_time_calendar_cal_iso = kk_std_time_calendar_solar_ecalendar(_x5809, _x5811, kk_std_time_calendar_new_cal_iso_fun5814(_ctx), kk_std_time_calendar_new_cal_iso_fun5816(_ctx), kk_std_time_calendar_new_cal_iso_fun5819(_ctx), kk_std_time_calendar_new_cal_iso_fun5822(_ctx), kk_std_core_types__new_None(_ctx), kk_std_core_types__new_None(_ctx), kk_std_core_types__new_None(_ctx), kk_std_core_types__new_None(_ctx), _ctx); /*std/time/calendar/calendar*/
  }
  {
    kk_string_t _x5914;
    kk_define_string_literal(, _s5915, 2, "GC", _ctx)
    _x5914 = kk_string_dup(_s5915, _ctx); /*string*/
    kk_string_t _x5916;
    kk_define_string_literal(, _s5917, 9, "Gregorian", _ctx)
    _x5916 = kk_string_dup(_s5917, _ctx); /*string*/
    kk_std_core_types__optional _x5928 = kk_std_core_types__new_Optional(kk_bool_box(false), _ctx); /*optional<116>*/
    kk_std_core_types__optional _x5929 = kk_std_core_types__new_Optional(kk_function_box(kk_std_time_calendar_new_cal_gregorian_fun5930(_ctx), _ctx), _ctx); /*optional<116>*/
    kk_std_time_calendar_cal_gregorian = kk_std_time_calendar_solar_ecalendar(_x5914, _x5916, kk_std_time_calendar_new_cal_gregorian_fun5919(_ctx), kk_std_time_calendar_new_cal_gregorian_fun5921(_ctx), kk_std_time_calendar_new_cal_gregorian_fun5924(_ctx), kk_std_time_calendar_new_cal_gregorian_fun5927(_ctx), kk_std_core_types__new_None(_ctx), _x5928, kk_std_core_types__new_None(_ctx), _x5929, _ctx); /*std/time/calendar/calendar*/
  }
  {
    kk_string_t _x5934;
    kk_define_string_literal(, _s5935, 3, "SLS", _ctx)
    _x5934 = kk_string_dup(_s5935, _ctx); /*string*/
    kk_string_t _x5936;
    kk_define_string_literal(, _s5937, 7, "ISO-SLS", _ctx)
    _x5936 = kk_string_dup(_s5937, _ctx); /*string*/
    kk_std_time_calendar_cal_iso_sls = kk_std_time_calendar_solar_ecalendar(_x5934, _x5936, kk_std_time_calendar_new_cal_iso_sls_fun5939(_ctx), kk_std_time_calendar_new_cal_iso_sls_fun5941(_ctx), kk_std_time_calendar_new_cal_iso_sls_fun5944(_ctx), kk_std_time_calendar_new_cal_iso_sls_fun5947(_ctx), kk_std_core_types__new_None(_ctx), kk_std_core_types__new_None(_ctx), kk_std_core_types__new_None(_ctx), kk_std_core_types__new_None(_ctx), _ctx); /*std/time/calendar/calendar*/
  }
}

// termination
void kk_std_time_calendar__done(kk_context_t* _ctx){
  static bool _kk_done = false;
  if (_kk_done) return;
  _kk_done = true;
  #if defined(KK_CUSTOM_DONE)
    KK_CUSTOM_DONE (_ctx);
  #endif
  kk_std_time_calendar__calendar_drop(kk_std_time_calendar_cal_iso_sls, _ctx);
  kk_std_time_calendar__calendar_drop(kk_std_time_calendar_cal_gregorian, _ctx);
  kk_std_time_calendar__calendar_drop(kk_std_time_calendar_cal_iso, _ctx);
  kk_std_time_calendar__timezone_drop(kk_std_time_calendar_tz_utc, _ctx);
  kk_std_time_utc__done(_ctx);
  kk_std_time_instant__done(_ctx);
  kk_std_time_duration__done(_ctx);
  kk_std_time_timestamp__done(_ctx);
  kk_std_time_date__done(_ctx);
  kk_std_num_ddouble__done(_ctx);
  kk_std_num_float64__done(_ctx);
  kk_std_core__done(_ctx);
  kk_std_core_hnd__done(_ctx);
  kk_std_core_types__done(_ctx);
}
