// Koka generated module: "std/time/calendar", koka version: 2.4.2, platform: 64-bit
#include "std_time_calendar.h"
/*---------------------------------------------------------------------------
  Copyright 2020-2021, Microsoft Research, Daan Leijen.

  This is free software; you can redistribute it and/or modify it under the
  terms of the Apache License, Version 2.0. A copy of the License can be
  found in the LICENSE file at the root of this distribution.
---------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------------------------------
  Local time zone UTC delta
--------------------------------------------------------------------------------------------------*/

#if !defined(__STDC_WANT_LIB_EXT1__)
#define __STDC_WANT_LIB_EXT1__    // for gmtime_s
#endif
#if defined(__GLIBC__) && !defined(_GNU_SOURCE)
#define _GNU_SOURCE               // for tm_zone and tm_gmtoff
#endif
#include <time.h>

static long kk_local_utc_delta(double unix_secs, kk_string_t* ptzname, kk_context_t* ctx) {
  // get the UTC delta in a somewhat portable way...
  bool isdst = false;
  time_t t = (time_t)unix_secs;
  #if (_WIN32 && KK_INTPTR_SIZE==8)
  if (t < 0) { t += 3155673600; } // try to avoid errors for negative times on some platforms by adding 100 years..
  #endif
  time_t loct = t;    
  #if defined(_GNU_SOURCE)
    // GNU libc has the tm_zone and tm_gmtoff fields
    struct tm loctm;
    if (localtime_r(&t, &loctm) != NULL) {
      isdst = (loctm.tm_isdst != 0);
      loct  = t - loctm.tm_gmtoff + (isdst ? 3600 : 0);
    }
    else {
      loctm.tm_zone = "";
    }
  #elif defined(_WIN32) && !defined(__MINGW32__)
    struct tm gmtm;
    if (gmtime_s(&gmtm, &t) == 0) {      // switched parameters :-(      
      loct = mktime(&gmtm);              // interpret gmt as local time
      struct tm loctm;
      localtime_s(&loctm, &t);           // switched parameters :-(
      isdst = (loctm.tm_isdst != 0);
    }
  #elif defined(__STDC_LIB_EXT1__)
    struct tm gmtm;
    if (gmtime_s(&t, &gmtm) != NULL) {
      loct = mktime(&gmtm);              // interpret gmt as local time
      struct tm loctm;
      localtime_s(&t, &loctm);
      isdst = (loctm.tm_isdst != 0);
    }
  #else
    struct tm* pgmtm = gmtime(&t);
    if (pgmtm != NULL) {
      loct = mktime(pgmtm);                // interpret gmt as local time
      struct tm* ploctm = localtime(&t);
      isdst = (ploctm->tm_isdst != 0);
    }
  #endif
  const time_t utc_delta = t - loct + (isdst ? 3600 : 0);   // the difference is the utc offset at that time
  if (ptzname != NULL) {
    // getting the timezone name
    #if defined(_WIN32) && !defined(__MINGW32__)
      char tzonename[256];
      size_t tznamelen;
      _get_tzname(&tznamelen, tzonename, 255, isdst ? 1 : 0); tzonename[255] = 0;
      *ptzname = kk_string_alloc_from_qutf8(tzonename, ctx);
    #elif defined(_GNU_SOURCE)
      *ptzname = kk_string_alloc_from_qutf8(loctm.tm_zone, ctx);
    #elif (_POSIX_C_SOURCE >= 1) || _XOPEN_SOURCE || _POSIX_SOURCE || __MINGW32__ // tzname
      *ptzname = kk_string_alloc_from_qutf8(tzname[isdst ? 1 : 0], ctx);
    #else
      // give up :-(
      * ptzname = kk_string_empty();
    #endif
  }
  return (long)utc_delta;
}


static kk_std_time_calendar__local_timezone kk_local_get_timezone(kk_context_t* ctx) {
  return kk_datatype_from_tag((kk_tag_t)1); // dummy value; we cannot store the local timezone as it is a global :-(
}

static kk_std_core_types__tuple2_ kk_local_get_utc_delta_tuple(kk_std_time_calendar__local_timezone tz, double unix_secs, kk_context_t* ctx) {
  kk_string_t tzonename;
  long utc_delta = kk_local_utc_delta(unix_secs, &tzonename, ctx);
  return kk_std_core_types__new_dash__lp__comma__rp_( kk_double_box((double)utc_delta,ctx), kk_string_box(tzonename), ctx );
}



// lift anonymous function
struct kk_std_time_calendar__copy_fun6069__t {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x5171;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar__copy_fun6069(kk_function_t _fself, kk_std_time_instant__instant _b_5174, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_copy_fun6069(kk_box_t _fun_unbox_x5171, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun6069__t* _self = kk_function_alloc_as(struct kk_std_time_calendar__copy_fun6069__t, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar__copy_fun6069, kk_context());
  _self->_fun_unbox_x5171 = _fun_unbox_x5171;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_core_types__tuple2_ kk_std_time_calendar__copy_fun6069(kk_function_t _fself, kk_std_time_instant__instant _b_5174, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun6069__t* _self = kk_function_as(struct kk_std_time_calendar__copy_fun6069__t*, _fself, _ctx);
  kk_box_t _fun_unbox_x5171 = _self->_fun_unbox_x5171; /* 116 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x5171, _ctx);}, {}, _ctx)
  kk_box_t _x6070;
  kk_function_t _x6071 = kk_function_unbox(_fun_unbox_x5171, _ctx); /*(5172) -> 5173*/
  _x6070 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6071, (_x6071, kk_std_time_instant__instant_box(_b_5174, _ctx), _ctx), _ctx); /*5173*/
  return kk_std_core_types__tuple2__unbox(_x6070, KK_OWNED, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar__copy_fun6074__t {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x5178;
};
static kk_std_core_types__maybe kk_std_time_calendar__copy_fun6074(kk_function_t _fself, kk_std_time_instant__instant _b_5181, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_copy_fun6074(kk_box_t _fun_unbox_x5178, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun6074__t* _self = kk_function_alloc_as(struct kk_std_time_calendar__copy_fun6074__t, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar__copy_fun6074, kk_context());
  _self->_fun_unbox_x5178 = _fun_unbox_x5178;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_core_types__maybe kk_std_time_calendar__copy_fun6074(kk_function_t _fself, kk_std_time_instant__instant _b_5181, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun6074__t* _self = kk_function_as(struct kk_std_time_calendar__copy_fun6074__t*, _fself, _ctx);
  kk_box_t _fun_unbox_x5178 = _self->_fun_unbox_x5178; /* 116 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x5178, _ctx);}, {}, _ctx)
  kk_box_t _x6075;
  kk_function_t _x6076 = kk_function_unbox(_fun_unbox_x5178, _ctx); /*(5179) -> 5180*/
  _x6075 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6076, (_x6076, kk_std_time_instant__instant_box(_b_5181, _ctx), _ctx), _ctx); /*5180*/
  return kk_std_core_types__maybe_unbox(_x6075, KK_OWNED, _ctx);
}

kk_std_time_calendar__timezone kk_std_time_calendar__copy(kk_std_time_calendar__timezone _this, kk_std_core_types__optional name0, kk_std_core_types__optional utc_delta0, kk_std_core_types__optional utc_inverse0, kk_context_t* _ctx) { /* (timezone, name : optional<string>, utc-delta : optional<(std/time/instant/instant) -> (std/time/duration/duration, string)>, utc-inverse : optional<(std/time/instant/instant) -> maybe<std/time/instant/instant>>) -> timezone */ 
  kk_string_t _x6066;
  if (kk_std_core_types__is_Optional(name0, _ctx)) {
    kk_box_t _box_x5167 = name0._cons.Optional.value;
    kk_string_t _name_217 = kk_string_unbox(_box_x5167);
    kk_string_dup(_name_217, _ctx);
    kk_std_core_types__optional_drop(name0, _ctx);
    _x6066 = _name_217; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(name0, _ctx);
    {
      struct kk_std_time_calendar_Timezone* _con6067 = kk_std_time_calendar__as_Timezone(_this, _ctx);
      kk_string_t _x = _con6067->name;
      kk_string_dup(_x, _ctx);
      _x6066 = _x; /*string*/
    }
  }
  kk_function_t _x6068;
  if (kk_std_core_types__is_Optional(utc_delta0, _ctx)) {
    kk_box_t _fun_unbox_x5171 = utc_delta0._cons.Optional.value;
    kk_box_dup(_fun_unbox_x5171, _ctx);
    kk_std_core_types__optional_drop(utc_delta0, _ctx);
    _x6068 = kk_std_time_calendar__new_copy_fun6069(_fun_unbox_x5171, _ctx); /*(std/time/instant/instant) -> (std/time/duration/duration, string)*/
  }
  else {
    kk_std_core_types__optional_drop(utc_delta0, _ctx);
    {
      struct kk_std_time_calendar_Timezone* _con6072 = kk_std_time_calendar__as_Timezone(_this, _ctx);
      kk_function_t _x0 = _con6072->utc_delta;
      kk_function_dup(_x0, _ctx);
      _x6068 = _x0; /*(std/time/instant/instant) -> (std/time/duration/duration, string)*/
    }
  }
  kk_function_t _x6073;
  if (kk_std_core_types__is_Optional(utc_inverse0, _ctx)) {
    kk_box_t _fun_unbox_x5178 = utc_inverse0._cons.Optional.value;
    kk_box_dup(_fun_unbox_x5178, _ctx);
    kk_std_core_types__optional_drop(utc_inverse0, _ctx);
    kk_datatype_ptr_dropn(_this, (KK_I32(3)), _ctx);
    _x6073 = kk_std_time_calendar__new_copy_fun6074(_fun_unbox_x5178, _ctx); /*(std/time/instant/instant) -> maybe<std/time/instant/instant>*/
  }
  else {
    kk_std_core_types__optional_drop(utc_inverse0, _ctx);
    {
      struct kk_std_time_calendar_Timezone* _con6077 = kk_std_time_calendar__as_Timezone(_this, _ctx);
      kk_string_t _pat02 = _con6077->name;
      kk_function_t _pat13 = _con6077->utc_delta;
      kk_function_t _x1 = _con6077->utc_inverse;
      if kk_likely(kk_datatype_ptr_is_unique(_this, _ctx)) {
        kk_function_drop(_pat13, _ctx);
        kk_string_drop(_pat02, _ctx);
        kk_datatype_ptr_free(_this, _ctx);
      }
      else {
        kk_function_dup(_x1, _ctx);
        kk_datatype_ptr_decref(_this, _ctx);
      }
      _x6073 = _x1; /*(std/time/instant/instant) -> maybe<std/time/instant/instant>*/
    }
  }
  return kk_std_time_calendar__new_Timezone(kk_reuse_null, 0, _x6066, _x6068, _x6073, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar__copy_fun6093__t_1 {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x5188;
};
static kk_string_t kk_std_time_calendar__copy_fun6093_1(kk_function_t _fself, kk_std_time_date__date _b_5191, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_copy_fun6093_1(kk_box_t _fun_unbox_x5188, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun6093__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar__copy_fun6093__t_1, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar__copy_fun6093_1, kk_context());
  _self->_fun_unbox_x5188 = _fun_unbox_x5188;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_string_t kk_std_time_calendar__copy_fun6093_1(kk_function_t _fself, kk_std_time_date__date _b_5191, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun6093__t_1* _self = kk_function_as(struct kk_std_time_calendar__copy_fun6093__t_1*, _fself, _ctx);
  kk_box_t _fun_unbox_x5188 = _self->_fun_unbox_x5188; /* 116 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x5188, _ctx);}, {}, _ctx)
  kk_box_t _x6094;
  kk_function_t _x6095 = kk_function_unbox(_fun_unbox_x5188, _ctx); /*(5189) -> 5190*/
  _x6094 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6095, (_x6095, kk_std_time_date__date_box(_b_5191, _ctx), _ctx), _ctx); /*5190*/
  return kk_string_unbox(_x6094);
}


// lift anonymous function
struct kk_std_time_calendar__copy_fun6098__t_1 {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x5197;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar__copy_fun6098_1(kk_function_t _fself, kk_std_time_instant__instant _b_5201, kk_std_time_duration__duration _b_5202, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_copy_fun6098_1(kk_box_t _fun_unbox_x5197, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun6098__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar__copy_fun6098__t_1, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar__copy_fun6098_1, kk_context());
  _self->_fun_unbox_x5197 = _fun_unbox_x5197;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_core_types__tuple2_ kk_std_time_calendar__copy_fun6098_1(kk_function_t _fself, kk_std_time_instant__instant _b_5201, kk_std_time_duration__duration _b_5202, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun6098__t_1* _self = kk_function_as(struct kk_std_time_calendar__copy_fun6098__t_1*, _fself, _ctx);
  kk_box_t _fun_unbox_x5197 = _self->_fun_unbox_x5197; /* 116 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x5197, _ctx);}, {}, _ctx)
  kk_box_t _x6099;
  kk_function_t _x6100 = kk_function_unbox(_fun_unbox_x5197, _ctx); /*(i : 5198, tzdelta : 5199) -> 5200*/
  _x6099 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_box_t, kk_context_t*), _x6100, (_x6100, kk_std_time_instant__instant_box(_b_5201, _ctx), kk_std_time_duration__duration_box(_b_5202, _ctx), _ctx), _ctx); /*5200*/
  return kk_std_core_types__tuple2__unbox(_x6099, KK_OWNED, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar__copy_fun6103__t_1 {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x5212;
};
static kk_std_time_instant__instant kk_std_time_calendar__copy_fun6103_1(kk_function_t _fself, kk_std_time_date__date _b_5218, kk_std_time_date__clock _b_5219, kk_std_time_calendar__timezone _b_5220, kk_std_time_instant__timescale _b_5221, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_copy_fun6103_1(kk_box_t _fun_unbox_x5212, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun6103__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar__copy_fun6103__t_1, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar__copy_fun6103_1, kk_context());
  _self->_fun_unbox_x5212 = _fun_unbox_x5212;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_time_instant__instant kk_std_time_calendar__copy_fun6103_1(kk_function_t _fself, kk_std_time_date__date _b_5218, kk_std_time_date__clock _b_5219, kk_std_time_calendar__timezone _b_5220, kk_std_time_instant__timescale _b_5221, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun6103__t_1* _self = kk_function_as(struct kk_std_time_calendar__copy_fun6103__t_1*, _fself, _ctx);
  kk_box_t _fun_unbox_x5212 = _self->_fun_unbox_x5212; /* 116 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x5212, _ctx);}, {}, _ctx)
  kk_box_t _x6104;
  kk_function_t _x6105 = kk_function_unbox(_fun_unbox_x5212, _ctx); /*(5213, 5214, 5215, 5216) -> 5217*/
  _x6104 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_box_t, kk_box_t, kk_box_t, kk_context_t*), _x6105, (_x6105, kk_std_time_date__date_box(_b_5218, _ctx), kk_std_time_date__clock_box(_b_5219, _ctx), kk_std_time_calendar__timezone_box(_b_5220, _ctx), kk_std_time_instant__timescale_box(_b_5221, _ctx), _ctx), _ctx); /*5217*/
  return kk_std_time_instant__instant_unbox(_x6104, KK_OWNED, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar__copy_fun6108__t_1 {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x5225;
};
static kk_std_time_date__date kk_std_time_calendar__copy_fun6108_1(kk_function_t _fself, kk_integer_t _b_5228, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_copy_fun6108_1(kk_box_t _fun_unbox_x5225, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun6108__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar__copy_fun6108__t_1, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar__copy_fun6108_1, kk_context());
  _self->_fun_unbox_x5225 = _fun_unbox_x5225;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_time_date__date kk_std_time_calendar__copy_fun6108_1(kk_function_t _fself, kk_integer_t _b_5228, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun6108__t_1* _self = kk_function_as(struct kk_std_time_calendar__copy_fun6108__t_1*, _fself, _ctx);
  kk_box_t _fun_unbox_x5225 = _self->_fun_unbox_x5225; /* 116 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x5225, _ctx);}, {}, _ctx)
  kk_box_t _x6109;
  kk_function_t _x6110 = kk_function_unbox(_fun_unbox_x5225, _ctx); /*(days : 5226) -> 5227*/
  _x6109 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6110, (_x6110, kk_integer_box(_b_5228, _ctx), _ctx), _ctx); /*5227*/
  return kk_std_time_date__date_unbox(_x6109, KK_OWNED, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar__copy_fun6113__t_1 {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x5232;
};
static kk_integer_t kk_std_time_calendar__copy_fun6113_1(kk_function_t _fself, kk_std_time_date__date _b_5235, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_copy_fun6113_1(kk_box_t _fun_unbox_x5232, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun6113__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar__copy_fun6113__t_1, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar__copy_fun6113_1, kk_context());
  _self->_fun_unbox_x5232 = _fun_unbox_x5232;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_integer_t kk_std_time_calendar__copy_fun6113_1(kk_function_t _fself, kk_std_time_date__date _b_5235, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun6113__t_1* _self = kk_function_as(struct kk_std_time_calendar__copy_fun6113__t_1*, _fself, _ctx);
  kk_box_t _fun_unbox_x5232 = _self->_fun_unbox_x5232; /* 116 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x5232, _ctx);}, {}, _ctx)
  kk_box_t _x6114;
  kk_function_t _x6115 = kk_function_unbox(_fun_unbox_x5232, _ctx); /*(date : 5233) -> 5234*/
  _x6114 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6115, (_x6115, kk_std_time_date__date_box(_b_5235, _ctx), _ctx), _ctx); /*5234*/
  return kk_integer_unbox(_x6114, _ctx);
}

kk_std_time_calendar__calendar kk_std_time_calendar__copy_1(kk_std_time_calendar__calendar _this, kk_std_core_types__optional name0, kk_std_core_types__optional long_name0, kk_std_core_types__optional month_prefix0, kk_std_core_types__optional show_era0, kk_std_core_types__optional instant_to_dc0, kk_std_core_types__optional dc_to_instant0, kk_std_core_types__optional days_to_date0, kk_std_core_types__optional date_to_days0, kk_context_t* _ctx) { /* (calendar, name : optional<string>, long-name : optional<string>, month-prefix : optional<string>, show-era : optional<(std/time/date/date) -> string>, instant-to-dc : optional<(i : std/time/instant/instant, tzdelta : std/time/duration/duration) -> (std/time/date/date, std/time/date/clock)>, dc-to-instant : optional<(std/time/date/date, std/time/date/clock, timezone, std/time/instant/timescale) -> std/time/instant/instant>, days-to-date : optional<(days : int) -> std/time/date/date>, date-to-days : optional<(date : std/time/date/date) -> int>) -> calendar */ 
  kk_string_t _x6086;
  if (kk_std_core_types__is_Optional(name0, _ctx)) {
    kk_box_t _box_x5182 = name0._cons.Optional.value;
    kk_string_t _name_314 = kk_string_unbox(_box_x5182);
    kk_string_dup(_name_314, _ctx);
    kk_std_core_types__optional_drop(name0, _ctx);
    _x6086 = _name_314; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(name0, _ctx);
    {
      struct kk_std_time_calendar_Calendar* _con6087 = kk_std_time_calendar__as_Calendar(_this, _ctx);
      kk_string_t _x = _con6087->name;
      kk_string_dup(_x, _ctx);
      _x6086 = _x; /*string*/
    }
  }
  kk_string_t _x6088;
  if (kk_std_core_types__is_Optional(long_name0, _ctx)) {
    kk_box_t _box_x5183 = long_name0._cons.Optional.value;
    kk_string_t _long_name_320 = kk_string_unbox(_box_x5183);
    kk_string_dup(_long_name_320, _ctx);
    kk_std_core_types__optional_drop(long_name0, _ctx);
    _x6088 = _long_name_320; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(long_name0, _ctx);
    {
      struct kk_std_time_calendar_Calendar* _con6089 = kk_std_time_calendar__as_Calendar(_this, _ctx);
      kk_string_t _x0 = _con6089->long_name;
      kk_string_dup(_x0, _ctx);
      _x6088 = _x0; /*string*/
    }
  }
  kk_string_t _x6090;
  if (kk_std_core_types__is_Optional(month_prefix0, _ctx)) {
    kk_box_t _box_x5184 = month_prefix0._cons.Optional.value;
    kk_string_t _month_prefix_326 = kk_string_unbox(_box_x5184);
    kk_string_dup(_month_prefix_326, _ctx);
    kk_std_core_types__optional_drop(month_prefix0, _ctx);
    _x6090 = _month_prefix_326; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(month_prefix0, _ctx);
    {
      struct kk_std_time_calendar_Calendar* _con6091 = kk_std_time_calendar__as_Calendar(_this, _ctx);
      kk_string_t _x1 = _con6091->month_prefix;
      kk_string_dup(_x1, _ctx);
      _x6090 = _x1; /*string*/
    }
  }
  kk_function_t _x6092;
  if (kk_std_core_types__is_Optional(show_era0, _ctx)) {
    kk_box_t _fun_unbox_x5188 = show_era0._cons.Optional.value;
    kk_box_dup(_fun_unbox_x5188, _ctx);
    kk_std_core_types__optional_drop(show_era0, _ctx);
    _x6092 = kk_std_time_calendar__new_copy_fun6093_1(_fun_unbox_x5188, _ctx); /*(std/time/date/date) -> string*/
  }
  else {
    kk_std_core_types__optional_drop(show_era0, _ctx);
    {
      struct kk_std_time_calendar_Calendar* _con6096 = kk_std_time_calendar__as_Calendar(_this, _ctx);
      kk_function_t _x2 = _con6096->show_era;
      kk_function_dup(_x2, _ctx);
      _x6092 = _x2; /*(std/time/date/date) -> string*/
    }
  }
  kk_function_t _x6097;
  if (kk_std_core_types__is_Optional(instant_to_dc0, _ctx)) {
    kk_box_t _fun_unbox_x5197 = instant_to_dc0._cons.Optional.value;
    kk_box_dup(_fun_unbox_x5197, _ctx);
    kk_std_core_types__optional_drop(instant_to_dc0, _ctx);
    _x6097 = kk_std_time_calendar__new_copy_fun6098_1(_fun_unbox_x5197, _ctx); /*(std/time/instant/instant, std/time/duration/duration) -> (std/time/date/date, std/time/date/clock)*/
  }
  else {
    kk_std_core_types__optional_drop(instant_to_dc0, _ctx);
    {
      struct kk_std_time_calendar_Calendar* _con6101 = kk_std_time_calendar__as_Calendar(_this, _ctx);
      kk_function_t _x3 = _con6101->instant_to_dc;
      kk_function_dup(_x3, _ctx);
      _x6097 = _x3; /*(std/time/instant/instant, std/time/duration/duration) -> (std/time/date/date, std/time/date/clock)*/
    }
  }
  kk_function_t _x6102;
  if (kk_std_core_types__is_Optional(dc_to_instant0, _ctx)) {
    kk_box_t _fun_unbox_x5212 = dc_to_instant0._cons.Optional.value;
    kk_box_dup(_fun_unbox_x5212, _ctx);
    kk_std_core_types__optional_drop(dc_to_instant0, _ctx);
    _x6102 = kk_std_time_calendar__new_copy_fun6103_1(_fun_unbox_x5212, _ctx); /*(std/time/date/date, std/time/date/clock, std/time/calendar/timezone, std/time/instant/timescale) -> std/time/instant/instant*/
  }
  else {
    kk_std_core_types__optional_drop(dc_to_instant0, _ctx);
    {
      struct kk_std_time_calendar_Calendar* _con6106 = kk_std_time_calendar__as_Calendar(_this, _ctx);
      kk_function_t _x4 = _con6106->dc_to_instant;
      kk_function_dup(_x4, _ctx);
      _x6102 = _x4; /*(std/time/date/date, std/time/date/clock, std/time/calendar/timezone, std/time/instant/timescale) -> std/time/instant/instant*/
    }
  }
  kk_function_t _x6107;
  if (kk_std_core_types__is_Optional(days_to_date0, _ctx)) {
    kk_box_t _fun_unbox_x5225 = days_to_date0._cons.Optional.value;
    kk_box_dup(_fun_unbox_x5225, _ctx);
    kk_std_core_types__optional_drop(days_to_date0, _ctx);
    _x6107 = kk_std_time_calendar__new_copy_fun6108_1(_fun_unbox_x5225, _ctx); /*(int) -> std/time/date/date*/
  }
  else {
    kk_std_core_types__optional_drop(days_to_date0, _ctx);
    {
      struct kk_std_time_calendar_Calendar* _con6111 = kk_std_time_calendar__as_Calendar(_this, _ctx);
      kk_function_t _x5 = _con6111->days_to_date;
      kk_function_dup(_x5, _ctx);
      _x6107 = _x5; /*(int) -> std/time/date/date*/
    }
  }
  kk_function_t _x6112;
  if (kk_std_core_types__is_Optional(date_to_days0, _ctx)) {
    kk_box_t _fun_unbox_x5232 = date_to_days0._cons.Optional.value;
    kk_box_dup(_fun_unbox_x5232, _ctx);
    kk_std_core_types__optional_drop(date_to_days0, _ctx);
    kk_datatype_ptr_dropn(_this, (KK_I32(8)), _ctx);
    _x6112 = kk_std_time_calendar__new_copy_fun6113_1(_fun_unbox_x5232, _ctx); /*(std/time/date/date) -> int*/
  }
  else {
    kk_std_core_types__optional_drop(date_to_days0, _ctx);
    {
      struct kk_std_time_calendar_Calendar* _con6116 = kk_std_time_calendar__as_Calendar(_this, _ctx);
      kk_string_t _pat07 = _con6116->name;
      kk_string_t _pat112 = _con6116->long_name;
      kk_string_t _pat28 = _con6116->month_prefix;
      kk_function_t _pat37 = _con6116->show_era;
      kk_function_t _pat47 = _con6116->instant_to_dc;
      kk_function_t _pat57 = _con6116->dc_to_instant;
      kk_function_t _pat67 = _con6116->days_to_date;
      kk_function_t _x6 = _con6116->date_to_days;
      if kk_likely(kk_datatype_ptr_is_unique(_this, _ctx)) {
        kk_function_drop(_pat67, _ctx);
        kk_function_drop(_pat57, _ctx);
        kk_function_drop(_pat47, _ctx);
        kk_function_drop(_pat37, _ctx);
        kk_string_drop(_pat28, _ctx);
        kk_string_drop(_pat112, _ctx);
        kk_string_drop(_pat07, _ctx);
        kk_datatype_ptr_free(_this, _ctx);
      }
      else {
        kk_function_dup(_x6, _ctx);
        kk_datatype_ptr_decref(_this, _ctx);
      }
      _x6112 = _x6; /*(std/time/date/date) -> int*/
    }
  }
  return kk_std_time_calendar__new_Calendar(kk_reuse_null, 0, _x6086, _x6088, _x6090, _x6092, _x6097, _x6102, _x6107, _x6112, _ctx);
}

kk_integer_t kk_std_time_calendar_iso_days_before_year(kk_integer_t year, kk_context_t* _ctx) { /* (year : int) -> int */ 
  kk_integer_t y = kk_integer_add_small_const(year, -1, _ctx); /*int*/;
  kk_integer_t x0_4671;
  kk_integer_t _x6117 = kk_integer_dup(y, _ctx); /*int*/
  x0_4671 = kk_integer_div(_x6117,(kk_integer_from_small(4)),kk_context()); /*int*/
  kk_integer_t y1_4672;
  kk_integer_t _x6118 = kk_integer_dup(y, _ctx); /*int*/
  y1_4672 = kk_integer_div(_x6118,(kk_integer_from_small(100)),kk_context()); /*int*/
  kk_integer_t x_4669 = kk_integer_sub(x0_4671,y1_4672,kk_context()); /*int*/;
  kk_integer_t y0_4670;
  kk_integer_t _x6119 = kk_integer_dup(y, _ctx); /*int*/
  y0_4670 = kk_integer_div(_x6119,(kk_integer_from_small(400)),kk_context()); /*int*/
  kk_integer_t leapdays = kk_integer_add(x_4669,y0_4670,kk_context()); /*int*/;
  kk_integer_t x1_4673 = kk_integer_mul((kk_integer_from_small(365)),y,kk_context()); /*int*/;
  return kk_integer_add(x1_4673,leapdays,kk_context());
}

kk_std_core_types__tuple2_ kk_std_time_calendar_iso_estimate_year(kk_integer_t days, kk_context_t* _ctx) { /* (days : int) -> (int, maybe<int>) */ 
  kk_std_core_types__tuple2_ _match_5986 = kk_integer_div_mod_tuple(days,(kk_integer_from_int(146097, _ctx)),kk_context()); /*(int, int)*/;
  {
    kk_box_t _box_x5236 = _match_5986.fst;
    kk_box_t _box_x5237 = _match_5986.snd;
    kk_integer_t era = kk_integer_unbox(_box_x5236, _ctx);
    kk_integer_t yoe = kk_integer_unbox(_box_x5237, _ctx);
    kk_integer_dup(era, _ctx);
    kk_integer_dup(yoe, _ctx);
    kk_std_core_types__tuple2__drop(_match_5986, _ctx);
    kk_integer_t y0_4678 = kk_integer_mul((kk_integer_from_small(400)),era,kk_context()); /*int*/;
    kk_integer_t x_4675 = kk_integer_add_small_const(y0_4678, 1, _ctx); /*int*/;
    kk_integer_t y_4676;
    kk_integer_t _x6120 = kk_integer_mul((kk_integer_from_small(100)),yoe,kk_context()); /*int*/
    y_4676 = kk_integer_div(_x6120,(kk_integer_from_int(36525, _ctx)),kk_context()); /*int*/
    kk_integer_t _b_5241_5239 = kk_integer_add(x_4675,y_4676,kk_context()); /*int*/;
    kk_box_t _x6121;
    kk_std_core_types__maybe _x6122 = kk_std_core_types__new_Just(kk_integer_box(kk_integer_from_small(363), _ctx), _ctx); /*maybe<111>*/
    _x6121 = kk_std_core_types__maybe_box(_x6122, _ctx); /*7*/
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5241_5239, _ctx), _x6121, _ctx);
  }
}

kk_std_time_timestamp__timestamp kk_std_time_calendar_dc_timestamp(kk_std_time_date__date d, kk_std_time_date__clock c, kk_function_t date_to_days0, kk_std_core_types__optional has_leap_seconds, kk_context_t* _ctx) { /* (d : std/time/date/date, c : std/time/date/clock, date-to-days : (std/time/date/date) -> int, has-leap-seconds : optional<bool>) -> std/time/timestamp/timestamp */ 
  kk_std_core_types__tuple2_ _match_5981;
  if (kk_std_core_types__is_Optional(has_leap_seconds, _ctx)) {
    kk_box_t _box_x5244 = has_leap_seconds._cons.Optional.value;
    bool _has_leap_seconds_652 = kk_bool_unbox(_box_x5244);
    kk_std_core_types__optional_drop(has_leap_seconds, _ctx);
    if (_has_leap_seconds_652) {
      kk_std_num_ddouble__ddouble y_4682 = kk_std_num_ddouble_ddouble_int_exp(kk_integer_from_small(60), kk_integer_from_small(0), _ctx); /*std/num/ddouble/ddouble*/;
      kk_std_core_types__order x0_13002;
      kk_std_core_types__order _match_5985;
      double _x6123;
      {
        struct kk_std_time_date_Clock* _con6124 = kk_std_time_date__as_Clock(c, _ctx);
        kk_std_num_ddouble__ddouble _x3 = _con6124->seconds;
        {
          double _x0 = _x3.hi;
          _x6123 = _x0; /*float64*/
        }
      }
      double _x6125;
      {
        double _x00 = y_4682.hi;
        _x6125 = _x00; /*float64*/
      }
      _match_5985 = kk_std_num_float64_compare(_x6123, _x6125, _ctx); /*order*/
      if (kk_std_core_types__is_Eq(_match_5985, _ctx)) {
        double _x6126;
        {
          struct kk_std_time_date_Clock* _con6127 = kk_std_time_date__as_Clock(c, _ctx);
          kk_std_num_ddouble__ddouble _x30 = _con6127->seconds;
          {
            double _x1 = _x30.lo;
            _x6126 = _x1; /*float64*/
          }
        }
        double _x6128;
        {
          double _x2 = y_4682.lo;
          _x6128 = _x2; /*float64*/
        }
        x0_13002 = kk_std_num_float64_compare(_x6126, _x6128, _ctx); /*order*/
      }
      else {
        x0_13002 = _match_5985; /*order*/
      }
      bool _match_5982;
      kk_integer_t _brw_5983;
      if (kk_std_core_types__is_Lt(x0_13002, _ctx)) {
        _brw_5983 = kk_integer_from_small(-1); /*int*/
        goto _match6129;
      }
      if (kk_std_core_types__is_Eq(x0_13002, _ctx)) {
        _brw_5983 = kk_integer_from_small(0); /*int*/
        goto _match6129;
      }
      {
        _brw_5983 = kk_integer_from_small(1); /*int*/
      }
      _match6129: ;
      bool _brw_5984 = kk_integer_eq_borrow(_brw_5983,(kk_integer_from_small(-1)),kk_context()); /*bool*/;
      kk_integer_drop(_brw_5983, _ctx);
      _match_5982 = _brw_5984; /*bool*/
      if (_match_5982) {
        kk_box_t _x6130;
        kk_std_num_ddouble__ddouble _x6131;
        {
          struct kk_std_time_date_Clock* _con6132 = kk_std_time_date__as_Clock(c, _ctx);
          kk_std_num_ddouble__ddouble _x4 = _con6132->seconds;
          _x6131 = _x4; /*std/num/ddouble/ddouble*/
        }
        _x6130 = kk_std_num_ddouble__ddouble_box(_x6131, _ctx); /*6*/
        _match_5981 = kk_std_core_types__new_dash__lp__comma__rp_(_x6130, kk_integer_box(kk_integer_from_small(0), _ctx), _ctx); /*(6, 7)*/
      }
      else {
        kk_integer_t x0_4686;
        kk_std_num_ddouble__ddouble _x6133;
        kk_std_num_ddouble__ddouble _x6134;
        {
          struct kk_std_time_date_Clock* _con6135 = kk_std_time_date__as_Clock(c, _ctx);
          kk_std_num_ddouble__ddouble _x5 = _con6135->seconds;
          _x6134 = _x5; /*std/num/ddouble/ddouble*/
        }
        _x6133 = kk_std_num_ddouble_floor(_x6134, _ctx); /*std/num/ddouble/ddouble*/
        x0_4686 = kk_std_num_ddouble_int(_x6133, kk_std_core_types__new_None(_ctx), _ctx); /*int*/
        kk_integer_t leap = kk_integer_add_small_const(x0_4686, -59, _ctx); /*int*/;
        kk_std_num_ddouble__ddouble y1_4690;
        kk_integer_t _x6136 = kk_integer_dup(leap, _ctx); /*int*/
        y1_4690 = kk_std_num_ddouble_ddouble_int_exp(_x6136, kk_integer_from_small(0), _ctx); /*std/num/ddouble/ddouble*/
        kk_std_num_ddouble__ddouble _b_5259_5247;
        kk_std_num_ddouble__ddouble _x6137;
        {
          struct kk_std_time_date_Clock* _con6138 = kk_std_time_date__as_Clock(c, _ctx);
          kk_std_num_ddouble__ddouble _x7 = _con6138->seconds;
          _x6137 = _x7; /*std/num/ddouble/ddouble*/
        }
        kk_std_num_ddouble__ddouble _x6139;
        double _x6140;
        double _x6141;
        {
          double _x6 = y1_4690.hi;
          _x6141 = _x6; /*float64*/
        }
        _x6140 = (-_x6141); /*float64*/
        double _x6142;
        double _x6143;
        {
          double _x01 = y1_4690.lo;
          _x6143 = _x01; /*float64*/
        }
        _x6142 = (-_x6143); /*float64*/
        _x6139 = kk_std_num_ddouble__new_Ddouble(_x6140, _x6142, _ctx); /*std/num/ddouble/ddouble*/
        _b_5259_5247 = kk_std_num_ddouble__lp__plus__rp_(_x6137, _x6139, _ctx); /*std/num/ddouble/ddouble*/
        _match_5981 = kk_std_core_types__new_dash__lp__comma__rp_(kk_std_num_ddouble__ddouble_box(_b_5259_5247, _ctx), kk_integer_box(leap, _ctx), _ctx); /*(6, 7)*/
      }
    }
    else {
      kk_box_t _x6144;
      kk_std_num_ddouble__ddouble _x6145;
      {
        struct kk_std_time_date_Clock* _con6146 = kk_std_time_date__as_Clock(c, _ctx);
        kk_std_num_ddouble__ddouble _x = _con6146->seconds;
        _x6145 = _x; /*std/num/ddouble/ddouble*/
      }
      _x6144 = kk_std_num_ddouble__ddouble_box(_x6145, _ctx); /*6*/
      _match_5981 = kk_std_core_types__new_dash__lp__comma__rp_(_x6144, kk_integer_box(kk_integer_from_small(0), _ctx), _ctx); /*(6, 7)*/
    }
  }
  else {
    kk_std_core_types__optional_drop(has_leap_seconds, _ctx);
    kk_box_t _x6147;
    kk_std_num_ddouble__ddouble _x6148;
    {
      struct kk_std_time_date_Clock* _con6149 = kk_std_time_date__as_Clock(c, _ctx);
      kk_std_num_ddouble__ddouble _x8 = _con6149->seconds;
      _x6148 = _x8; /*std/num/ddouble/ddouble*/
    }
    _x6147 = kk_std_num_ddouble__ddouble_box(_x6148, _ctx); /*6*/
    _match_5981 = kk_std_core_types__new_dash__lp__comma__rp_(_x6147, kk_integer_box(kk_integer_from_small(0), _ctx), _ctx); /*(6, 7)*/
  }
  {
    kk_box_t _box_x5253 = _match_5981.fst;
    kk_box_t _box_x5254 = _match_5981.snd;
    kk_std_num_ddouble__ddouble xsecs = kk_std_num_ddouble__ddouble_unbox(_box_x5253, KK_BORROWED, _ctx);
    kk_integer_t leap0 = kk_integer_unbox(_box_x5254, _ctx);
    kk_integer_dup(leap0, _ctx);
    kk_std_core_types__tuple2__drop(_match_5981, _ctx);
    kk_integer_t x2_4694;
    kk_integer_t _x6150;
    {
      struct kk_std_time_date_Clock* _con6151 = kk_std_time_date__as_Clock(c, _ctx);
      kk_std_num_ddouble__ddouble _pat1100 = _con6151->seconds;
      kk_integer_t _x80 = _con6151->hours;
      kk_integer_dup(_x80, _ctx);
      _x6150 = _x80; /*int*/
    }
    x2_4694 = kk_integer_mul(_x6150,(kk_integer_from_small(60)),kk_context()); /*int*/
    kk_integer_t i1_4693;
    kk_integer_t _x6152;
    kk_integer_t _x6153;
    {
      struct kk_std_time_date_Clock* _con6154 = kk_std_time_date__as_Clock(c, _ctx);
      kk_std_num_ddouble__ddouble _pat111 = _con6154->seconds;
      kk_integer_t _pat0110 = _con6154->hours;
      kk_integer_t _x9 = _con6154->minutes;
      if kk_likely(kk_datatype_ptr_is_unique(c, _ctx)) {
        kk_integer_drop(_pat0110, _ctx);
        kk_datatype_ptr_free(c, _ctx);
      }
      else {
        kk_integer_dup(_x9, _ctx);
        kk_datatype_ptr_decref(c, _ctx);
      }
      _x6153 = _x9; /*int*/
    }
    _x6152 = kk_integer_add(x2_4694,_x6153,kk_context()); /*int*/
    i1_4693 = kk_integer_mul(_x6152,(kk_integer_from_small(60)),kk_context()); /*int*/
    kk_std_num_ddouble__ddouble secs;
    kk_std_num_ddouble__ddouble _x6155 = kk_std_num_ddouble_ddouble_int_exp(i1_4693, kk_integer_from_small(0), _ctx); /*std/num/ddouble/ddouble*/
    secs = kk_std_num_ddouble__lp__plus__rp_(_x6155, xsecs, _ctx); /*std/num/ddouble/ddouble*/
    kk_integer_t days = kk_function_call(kk_integer_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), date_to_days0, (date_to_days0, d, _ctx), _ctx); /*int*/;
    kk_std_core_types__optional _x6156 = kk_std_core_types__new_Optional(kk_std_num_ddouble__ddouble_box(secs, _ctx), _ctx); /*optional<116>*/
    kk_std_core_types__optional _x6157 = kk_std_core_types__new_Optional(kk_integer_box(leap0, _ctx), _ctx); /*optional<116>*/
    return kk_std_time_timestamp_timestamp_days(days, _x6156, _x6157, _ctx);
  }
}

kk_std_core_types__tuple2_ kk_std_time_calendar_earth_timestamp_to_dc(kk_std_time_timestamp__timestamp t, kk_std_num_ddouble__ddouble tzdelta, kk_function_t days_to_date0, kk_context_t* _ctx) { /* (t : std/time/timestamp/timestamp, tzdelta : std/time/timestamp/timespan, days-to-date : (int) -> std/time/date/date) -> (std/time/date/date, std/time/date/clock) */ 
  kk_std_core_types__tuple2_ _match_5980;
  kk_std_time_timestamp__timestamp _x6158;
  kk_std_num_ddouble__ddouble _x6159;
  kk_std_num_ddouble__ddouble _x6160;
  {
    struct kk_std_time_timestamp_Timestamp* _con6161 = kk_std_time_timestamp__as_Timestamp(t, _ctx);
    kk_std_num_ddouble__ddouble _x = _con6161->since;
    _x6160 = _x; /*std/time/timestamp/timespan*/
  }
  _x6159 = kk_std_num_ddouble__lp__plus__rp_(_x6160, tzdelta, _ctx); /*std/num/ddouble/ddouble*/
  int32_t _x6162;
  {
    struct kk_std_time_timestamp_Timestamp* _con6163 = kk_std_time_timestamp__as_Timestamp(t, _ctx);
    kk_std_num_ddouble__ddouble _pat00 = _con6163->since;
    int32_t _x0 = _con6163->leap32;
    if kk_likely(kk_datatype_ptr_is_unique(t, _ctx)) {
      kk_datatype_ptr_free(t, _ctx);
    }
    else {
      kk_datatype_ptr_decref(t, _ctx);
    }
    _x6162 = _x0; /*int32*/
  }
  _x6158 = kk_std_time_timestamp__new_Timestamp(kk_reuse_null, 0, _x6159, _x6162, _ctx); /*std/time/timestamp/timestamp*/
  _match_5980 = kk_std_time_timestamp_days_clock(_x6158, _ctx); /*(int, std/time/date/clock)*/
  {
    kk_box_t _box_x5267 = _match_5980.fst;
    kk_box_t _box_x5268 = _match_5980.snd;
    kk_std_time_date__clock clock = kk_std_time_date__clock_unbox(_box_x5268, KK_BORROWED, _ctx);
    struct kk_std_time_date_Clock* _con6164 = kk_std_time_date__as_Clock(clock, _ctx);
    kk_integer_t days = kk_integer_unbox(_box_x5267, _ctx);
    kk_std_time_date__clock_dup(clock, _ctx);
    kk_integer_dup(days, _ctx);
    kk_std_core_types__tuple2__drop(_match_5980, _ctx);
    kk_std_time_date__date _b_5271_5269 = kk_function_call(kk_std_time_date__date, (kk_function_t, kk_integer_t, kk_context_t*), days_to_date0, (days_to_date0, days, _ctx), _ctx); /*std/time/date/date*/;
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_date__date_box(_b_5271_5269, _ctx), kk_std_time_date__clock_box(clock, _ctx), _ctx);
  }
}
 
// Return the days between two dates interpreted by calendar `cal`.

kk_integer_t kk_std_time_calendar_days_until(kk_std_time_calendar__calendar cal, kk_std_time_date__date d1, kk_std_time_date__date d2, kk_context_t* _ctx) { /* (cal : calendar, d1 : std/time/date/date, d2 : std/time/date/date) -> int */ 
  kk_integer_t x;
  {
    struct kk_std_time_calendar_Calendar* _con6165 = kk_std_time_calendar__as_Calendar(cal, _ctx);
    kk_function_t _x0 = _con6165->date_to_days;
    kk_function_dup(_x0, _ctx);
    x = kk_function_call(kk_integer_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _x0, (_x0, d2, _ctx), _ctx); /*int*/
  }
  kk_integer_t y;
  {
    struct kk_std_time_calendar_Calendar* _con6166 = kk_std_time_calendar__as_Calendar(cal, _ctx);
    kk_string_t _pat0 = _con6166->name;
    kk_string_t _pat1 = _con6166->long_name;
    kk_string_t _pat2 = _con6166->month_prefix;
    kk_function_t _pat3 = _con6166->show_era;
    kk_function_t _pat4 = _con6166->instant_to_dc;
    kk_function_t _pat5 = _con6166->dc_to_instant;
    kk_function_t _pat6 = _con6166->days_to_date;
    kk_function_t _x = _con6166->date_to_days;
    if kk_likely(kk_datatype_ptr_is_unique(cal, _ctx)) {
      kk_function_drop(_pat6, _ctx);
      kk_function_drop(_pat5, _ctx);
      kk_function_drop(_pat4, _ctx);
      kk_function_drop(_pat3, _ctx);
      kk_string_drop(_pat2, _ctx);
      kk_string_drop(_pat1, _ctx);
      kk_string_drop(_pat0, _ctx);
      kk_datatype_ptr_free(cal, _ctx);
    }
    else {
      kk_function_dup(_x, _ctx);
      kk_datatype_ptr_decref(cal, _ctx);
    }
    y = kk_function_call(kk_integer_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _x, (_x, d1, _ctx), _ctx); /*int*/
  }
  return kk_integer_sub(x,y,kk_context());
}
 
// Get the current local timezone structure.
// The timezone name is allowed to be the empty string.

kk_std_time_calendar__local_timezone kk_std_time_calendar_local_get_timezone(kk_context_t* _ctx) { /* () -> ndet local-timezone */ 
  return kk_local_get_timezone(kk_context());
}
 
// Return the utc delta in fractional seconds given a local timezone structure
// and time `i` in fractional seconds since the UNIX epoch (1970-01-01).
// Also returns time zone abbreviation.

kk_std_core_types__tuple2_ kk_std_time_calendar_local_utc_delta(kk_std_time_calendar__local_timezone tz, double i, kk_context_t* _ctx) { /* (tz : local-timezone, i : float64) -> (float64, string) */ 
  return kk_local_get_utc_delta_tuple(tz,i,kk_context());
}
extern kk_std_core_types__maybe kk_std_time_calendar__create_Timezone_fun6168(kk_function_t _fself, kk_std_time_instant__instant _b_5279, kk_context_t* _ctx) {
  struct kk_std_time_calendar__create_Timezone_fun6168__t* _self = kk_function_as(struct kk_std_time_calendar__create_Timezone_fun6168__t*, _fself, _ctx);
  kk_box_t _fun_unbox_x5276 = _self->_fun_unbox_x5276; /* 116 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x5276, _ctx);}, {}, _ctx)
  kk_box_t _x6169;
  kk_function_t _x6170 = kk_function_unbox(_fun_unbox_x5276, _ctx); /*(5277) -> 5278*/
  _x6169 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6170, (_x6170, kk_std_time_instant__instant_box(_b_5279, _ctx), _ctx), _ctx); /*5278*/
  return kk_std_core_types__maybe_unbox(_x6169, KK_OWNED, _ctx);
}
extern kk_std_core_types__maybe kk_std_time_calendar__create_Timezone_fun6171(kk_function_t _fself, kk_std_time_instant__instant i, kk_context_t* _ctx) {
  kk_unused(_fself);
  kk_std_time_instant__instant_drop(i, _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}
 
// Check if two calendars use the same date calculations. (Display of era names etc. may differ)

bool kk_std_time_calendar__lp__eq__eq__rp_(kk_std_time_calendar__calendar c1, kk_std_time_calendar__calendar c2, kk_context_t* _ctx) { /* (c1 : calendar, c2 : calendar) -> bool */ 
  kk_string_t _x6172;
  {
    struct kk_std_time_calendar_Calendar* _con6173 = kk_std_time_calendar__as_Calendar(c1, _ctx);
    kk_string_t _pat0 = _con6173->name;
    kk_string_t _x = _con6173->long_name;
    kk_string_t _pat1 = _con6173->month_prefix;
    kk_function_t _pat2 = _con6173->show_era;
    kk_function_t _pat3 = _con6173->instant_to_dc;
    kk_function_t _pat4 = _con6173->dc_to_instant;
    kk_function_t _pat5 = _con6173->days_to_date;
    kk_function_t _pat6 = _con6173->date_to_days;
    if kk_likely(kk_datatype_ptr_is_unique(c1, _ctx)) {
      kk_function_drop(_pat6, _ctx);
      kk_function_drop(_pat5, _ctx);
      kk_function_drop(_pat4, _ctx);
      kk_function_drop(_pat3, _ctx);
      kk_function_drop(_pat2, _ctx);
      kk_string_drop(_pat1, _ctx);
      kk_string_drop(_pat0, _ctx);
      kk_datatype_ptr_free(c1, _ctx);
    }
    else {
      kk_string_dup(_x, _ctx);
      kk_datatype_ptr_decref(c1, _ctx);
    }
    _x6172 = _x; /*string*/
  }
  kk_string_t _x6174;
  {
    struct kk_std_time_calendar_Calendar* _con6175 = kk_std_time_calendar__as_Calendar(c2, _ctx);
    kk_string_t _pat00 = _con6175->name;
    kk_string_t _x0 = _con6175->long_name;
    kk_string_t _pat10 = _con6175->month_prefix;
    kk_function_t _pat20 = _con6175->show_era;
    kk_function_t _pat30 = _con6175->instant_to_dc;
    kk_function_t _pat40 = _con6175->dc_to_instant;
    kk_function_t _pat50 = _con6175->days_to_date;
    kk_function_t _pat60 = _con6175->date_to_days;
    if kk_likely(kk_datatype_ptr_is_unique(c2, _ctx)) {
      kk_function_drop(_pat60, _ctx);
      kk_function_drop(_pat50, _ctx);
      kk_function_drop(_pat40, _ctx);
      kk_function_drop(_pat30, _ctx);
      kk_function_drop(_pat20, _ctx);
      kk_string_drop(_pat10, _ctx);
      kk_string_drop(_pat00, _ctx);
      kk_datatype_ptr_free(c2, _ctx);
    }
    else {
      kk_string_dup(_x0, _ctx);
      kk_datatype_ptr_decref(c2, _ctx);
    }
    _x6174 = _x0; /*string*/
  }
  return kk_string_is_eq(_x6172,_x6174,kk_context());
}
 
// Same timezone?

bool kk_std_time_calendar__lp__eq__eq__1_rp_(kk_std_time_calendar__timezone tz1, kk_std_time_calendar__timezone tz2, kk_context_t* _ctx) { /* (tz1 : timezone, tz2 : timezone) -> bool */ 
  kk_string_t _x6176;
  {
    struct kk_std_time_calendar_Timezone* _con6177 = kk_std_time_calendar__as_Timezone(tz1, _ctx);
    kk_string_t _x = _con6177->name;
    kk_function_t _pat0 = _con6177->utc_delta;
    kk_function_t _pat1 = _con6177->utc_inverse;
    if kk_likely(kk_datatype_ptr_is_unique(tz1, _ctx)) {
      kk_function_drop(_pat1, _ctx);
      kk_function_drop(_pat0, _ctx);
      kk_datatype_ptr_free(tz1, _ctx);
    }
    else {
      kk_string_dup(_x, _ctx);
      kk_datatype_ptr_decref(tz1, _ctx);
    }
    _x6176 = _x; /*string*/
  }
  kk_string_t _x6178;
  {
    struct kk_std_time_calendar_Timezone* _con6179 = kk_std_time_calendar__as_Timezone(tz2, _ctx);
    kk_string_t _x0 = _con6179->name;
    kk_function_t _pat00 = _con6179->utc_delta;
    kk_function_t _pat10 = _con6179->utc_inverse;
    if kk_likely(kk_datatype_ptr_is_unique(tz2, _ctx)) {
      kk_function_drop(_pat10, _ctx);
      kk_function_drop(_pat00, _ctx);
      kk_datatype_ptr_free(tz2, _ctx);
    }
    else {
      kk_string_dup(_x0, _ctx);
      kk_datatype_ptr_decref(tz2, _ctx);
    }
    _x6178 = _x0; /*string*/
  }
  return kk_string_is_eq(_x6176,_x6178,kk_context());
}

bool kk_std_time_calendar_iso_is_leap(kk_integer_t year, kk_context_t* _ctx) { /* (year : int) -> bool */ 
  bool _match_5972;
  kk_integer_t _brw_5978;
  kk_integer_t _x6180 = kk_integer_dup(year, _ctx); /*int*/
  _brw_5978 = kk_integer_mod(_x6180,(kk_integer_from_small(4)),kk_context()); /*int*/
  bool _brw_5979 = kk_integer_eq_borrow(_brw_5978,(kk_integer_from_small(0)),kk_context()); /*bool*/;
  kk_integer_drop(_brw_5978, _ctx);
  _match_5972 = _brw_5979; /*bool*/
  if (_match_5972) {
    bool _match_5973;
    kk_integer_t _brw_5976;
    kk_integer_t _x6181 = kk_integer_dup(year, _ctx); /*int*/
    _brw_5976 = kk_integer_mod(_x6181,(kk_integer_from_small(100)),kk_context()); /*int*/
    bool _brw_5977 = kk_integer_neq_borrow(_brw_5976,(kk_integer_from_small(0)),kk_context()); /*bool*/;
    kk_integer_drop(_brw_5976, _ctx);
    _match_5973 = _brw_5977; /*bool*/
    if (_match_5973) {
      kk_integer_drop(year, _ctx);
      return true;
    }
    {
      kk_integer_t _brw_5974 = kk_integer_mod(year,(kk_integer_from_small(400)),kk_context()); /*int*/;
      bool _brw_5975 = kk_integer_eq_borrow(_brw_5974,(kk_integer_from_small(0)),kk_context()); /*bool*/;
      kk_integer_drop(_brw_5974, _ctx);
      return _brw_5975;
    }
  }
  {
    kk_integer_drop(year, _ctx);
    return false;
  }
}

kk_integer_t kk_std_time_calendar_iso_days_before_month(kk_integer_t year, kk_integer_t month, kk_context_t* _ctx) { /* (year : int, month : int) -> int */ 
  bool is_before_march_4706 = kk_integer_lte_borrow(month,(kk_integer_from_small(2)),kk_context()); /*bool*/;
  kk_integer_t adj;
  if (is_before_march_4706) {
    kk_integer_drop(year, _ctx);
    adj = kk_integer_from_small(0); /*int*/
  }
  else {
    bool _match_5970 = kk_std_time_calendar_iso_is_leap(year, _ctx); /*bool*/;
    if (_match_5970) {
      adj = kk_integer_from_small(1); /*int*/
    }
    else {
      adj = kk_integer_from_small(2); /*int*/
    }
  }
  kk_integer_t x0_4710 = kk_integer_mul((kk_integer_from_small(367)),month,kk_context()); /*int*/;
  kk_integer_t x_4708;
  kk_integer_t _x6182 = kk_integer_add_small_const(x0_4710, -362, _ctx); /*int*/
  x_4708 = kk_integer_div(_x6182,(kk_integer_from_small(12)),kk_context()); /*int*/
  return kk_integer_sub(x_4708,adj,kk_context());
}

kk_integer_t kk_std_time_calendar_iso_doy_to_month(kk_integer_t year, kk_integer_t doy, kk_context_t* _ctx) { /* (year : int, doy : int) -> int */ 
  bool is_before_march_4712 = kk_integer_lte_borrow(doy,(kk_integer_from_small(58)),kk_context()); /*bool*/;
  kk_integer_t adj;
  if (is_before_march_4712) {
    kk_integer_drop(year, _ctx);
    adj = kk_integer_from_small(0); /*int*/
  }
  else {
    bool _match_5969 = kk_std_time_calendar_iso_is_leap(year, _ctx); /*bool*/;
    if (_match_5969) {
      adj = kk_integer_from_small(1); /*int*/
    }
    else {
      adj = kk_integer_from_small(2); /*int*/
    }
  }
  kk_integer_t x_4714;
  kk_integer_t _x6183 = kk_integer_add(doy,adj,kk_context()); /*int*/
  x_4714 = kk_integer_mul((kk_integer_from_small(12)),_x6183,kk_context()); /*int*/
  kk_integer_t _x6184 = kk_integer_add_small_const(x_4714, 373, _ctx); /*int*/
  return kk_integer_div(_x6184,(kk_integer_from_small(367)),kk_context());
}


// lift anonymous function
struct kk_std_time_calendar_tz_fixed_fun6185__t_1 {
  struct kk_function_s _base;
  kk_std_core_types__optional abbrv;
  kk_std_time_duration__duration delta;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_fixed_fun6185_1(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_fixed_fun6185_1(kk_std_core_types__optional abbrv, kk_std_time_duration__duration delta, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun6185__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar_tz_fixed_fun6185__t_1, 3, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_tz_fixed_fun6185_1, kk_context());
  _self->abbrv = abbrv;
  _self->delta = delta;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_fixed_fun6185_1(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun6185__t_1* _self = kk_function_as(struct kk_std_time_calendar_tz_fixed_fun6185__t_1*, _fself, _ctx);
  kk_std_core_types__optional abbrv = _self->abbrv; /* optional<string> */
  kk_std_time_duration__duration delta = _self->delta; /* std/time/duration/duration */
  kk_drop_match(_self, {kk_std_core_types__optional_dup(abbrv, _ctx);kk_std_time_duration__duration_dup(delta, _ctx);}, {}, _ctx)
  kk_std_time_instant__instant_drop(i0, _ctx);
  kk_box_t _x6186;
  kk_string_t _x6187;
  if (kk_std_core_types__is_Optional(abbrv, _ctx)) {
    kk_box_t _box_x5280 = abbrv._cons.Optional.value;
    kk_string_t _abbrv_2704 = kk_string_unbox(_box_x5280);
    kk_string_dup(_abbrv_2704, _ctx);
    kk_std_core_types__optional_drop(abbrv, _ctx);
    _x6187 = _abbrv_2704; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(abbrv, _ctx);
    _x6187 = kk_string_empty(); /*string*/
  }
  _x6186 = kk_string_box(_x6187); /*7*/
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_duration__duration_box(delta, _ctx), _x6186, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_tz_fixed_fun6189__t_1 {
  struct kk_function_s _base;
  kk_std_time_duration__duration delta;
};
static kk_std_core_types__maybe kk_std_time_calendar_tz_fixed_fun6189_1(kk_function_t _fself, kk_std_time_instant__instant i00, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_fixed_fun6189_1(kk_std_time_duration__duration delta, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun6189__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar_tz_fixed_fun6189__t_1, 1, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_tz_fixed_fun6189_1, kk_context());
  _self->delta = delta;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_core_types__maybe kk_std_time_calendar_tz_fixed_fun6189_1(kk_function_t _fself, kk_std_time_instant__instant i00, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun6189__t_1* _self = kk_function_as(struct kk_std_time_calendar_tz_fixed_fun6189__t_1*, _fself, _ctx);
  kk_std_time_duration__duration delta = _self->delta; /* std/time/duration/duration */
  kk_drop_match(_self, {kk_std_time_duration__duration_dup(delta, _ctx);}, {}, _ctx)
  bool _match_5968;
  double _x6190;
  {
    kk_std_num_ddouble__ddouble _x0 = delta.secs;
    {
      double _x = _x0.hi;
      _x6190 = _x; /*float64*/
    }
  }
  _match_5968 = (_x6190 == (0x0p+0)); /*bool*/
  if (_match_5968) {
    return kk_std_core_types__new_Just(kk_std_time_instant__instant_box(i00, _ctx), _ctx);
  }
  {
    kk_std_time_instant__instant_drop(i00, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
}

kk_std_time_calendar__timezone kk_std_time_calendar_tz_fixed_1(kk_string_t name0, kk_std_time_duration__duration delta, kk_std_core_types__optional abbrv, kk_context_t* _ctx) { /* (name : string, delta : std/time/duration/duration, abbrv : optional<string>) -> timezone */ 
  return kk_std_time_calendar__new_Timezone(kk_reuse_null, 0, name0, kk_std_time_calendar_new_tz_fixed_fun6185_1(abbrv, delta, _ctx), kk_std_time_calendar_new_tz_fixed_fun6189_1(delta, _ctx), _ctx);
}
 
// The standard UTC time zone with a 0 delta.


// lift anonymous function
struct kk_std_time_calendar_tz_utc_fun6193__t {
  struct kk_function_s _base;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_utc_fun6193(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_utc_fun6193(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_tz_utc_fun6193, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_utc_fun6193(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx) {
  kk_unused(_fself);
  kk_std_time_instant__instant_drop(i0, _ctx);
  kk_box_t _x6194;
  kk_string_t _x6195;
  kk_std_core_types__optional _match_5967 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5967, _ctx)) {
    kk_box_t _box_x5287 = _match_5967._cons.Optional.value;
    kk_string_t _abbrv_2704 = kk_string_unbox(_box_x5287);
    kk_string_dup(_abbrv_2704, _ctx);
    kk_std_core_types__optional_drop(_match_5967, _ctx);
    _x6195 = _abbrv_2704; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(_match_5967, _ctx);
    _x6195 = kk_string_empty(); /*string*/
  }
  _x6194 = kk_string_box(_x6195); /*7*/
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_duration__duration_box(kk_std_time_duration_zero, _ctx), _x6194, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_tz_utc_fun6197__t {
  struct kk_function_s _base;
};
static kk_std_core_types__maybe kk_std_time_calendar_tz_utc_fun6197(kk_function_t _fself, kk_std_time_instant__instant i00, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_utc_fun6197(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_tz_utc_fun6197, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_std_core_types__maybe kk_std_time_calendar_tz_utc_fun6197(kk_function_t _fself, kk_std_time_instant__instant i00, kk_context_t* _ctx) {
  kk_unused(_fself);
  bool _match_5966;
  double _x6198;
  kk_std_time_duration__duration _x6199 = kk_std_time_duration_zero; /*std/time/duration/duration*/
  {
    kk_std_num_ddouble__ddouble _x0 = _x6199.secs;
    {
      double _x = _x0.hi;
      _x6198 = _x; /*float64*/
    }
  }
  _match_5966 = (_x6198 == (0x0p+0)); /*bool*/
  if (_match_5966) {
    return kk_std_core_types__new_Just(kk_std_time_instant__instant_box(i00, _ctx), _ctx);
  }
  {
    kk_std_time_instant__instant_drop(i00, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
}

kk_std_time_calendar__timezone kk_std_time_calendar_tz_utc;
 
// Create a time zone with a fixed number of hours and optional minutes (=`0`) difference from UTC/GMT.
// The time zone name and abbreviation is optional, and the default name
// will reflect the offset from UTC, for example `"UTC+1:20"`, or `"UTC-8"`.
// The default abbreviation is the empty string.
// The sign of the minutes will be matched to the hours (unless `hours` is `0`).
// For example, `tz-fixed(-1,30)` gives a time zone offset of -90 minutes, "UTC-1:30".
// Returns `tz-utc` if both `hours` and `mins` are zero.


// lift anonymous function
struct kk_std_time_calendar_tz_fixed_fun6237__t {
  struct kk_function_s _base;
  kk_std_core_types__optional abbrv;
  kk_std_time_duration__duration delta;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_fixed_fun6237(kk_function_t _fself, kk_std_time_instant__instant i4, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_fixed_fun6237(kk_std_core_types__optional abbrv, kk_std_time_duration__duration delta, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun6237__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_tz_fixed_fun6237__t, 3, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_tz_fixed_fun6237, kk_context());
  _self->abbrv = abbrv;
  _self->delta = delta;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_fixed_fun6237(kk_function_t _fself, kk_std_time_instant__instant i4, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun6237__t* _self = kk_function_as(struct kk_std_time_calendar_tz_fixed_fun6237__t*, _fself, _ctx);
  kk_std_core_types__optional abbrv = _self->abbrv; /* optional<string> */
  kk_std_time_duration__duration delta = _self->delta; /* std/time/duration/duration */
  kk_drop_match(_self, {kk_std_core_types__optional_dup(abbrv, _ctx);kk_std_time_duration__duration_dup(delta, _ctx);}, {}, _ctx)
  kk_std_time_instant__instant_drop(i4, _ctx);
  kk_box_t _x6238;
  kk_string_t _x6239;
  if (kk_std_core_types__is_Optional(abbrv, _ctx)) {
    kk_box_t _box_x5304 = abbrv._cons.Optional.value;
    kk_string_t _abbrv_2281 = kk_string_unbox(_box_x5304);
    kk_string_dup(_abbrv_2281, _ctx);
    kk_std_core_types__optional_drop(abbrv, _ctx);
    _x6239 = _abbrv_2281; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(abbrv, _ctx);
    _x6239 = kk_string_empty(); /*string*/
  }
  _x6238 = kk_string_box(_x6239); /*7*/
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_duration__duration_box(delta, _ctx), _x6238, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_tz_fixed_fun6241__t {
  struct kk_function_s _base;
  kk_std_time_duration__duration delta;
};
static kk_std_core_types__maybe kk_std_time_calendar_tz_fixed_fun6241(kk_function_t _fself, kk_std_time_instant__instant i00, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_fixed_fun6241(kk_std_time_duration__duration delta, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun6241__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_tz_fixed_fun6241__t, 1, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_tz_fixed_fun6241, kk_context());
  _self->delta = delta;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_core_types__maybe kk_std_time_calendar_tz_fixed_fun6241(kk_function_t _fself, kk_std_time_instant__instant i00, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun6241__t* _self = kk_function_as(struct kk_std_time_calendar_tz_fixed_fun6241__t*, _fself, _ctx);
  kk_std_time_duration__duration delta = _self->delta; /* std/time/duration/duration */
  kk_drop_match(_self, {kk_std_time_duration__duration_dup(delta, _ctx);}, {}, _ctx)
  bool _match_5954;
  double _x6242;
  {
    kk_std_num_ddouble__ddouble _x0 = delta.secs;
    {
      double _x = _x0.hi;
      _x6242 = _x; /*float64*/
    }
  }
  _match_5954 = (_x6242 == (0x0p+0)); /*bool*/
  if (_match_5954) {
    return kk_std_core_types__new_Just(kk_std_time_instant__instant_box(i00, _ctx), _ctx);
  }
  {
    kk_std_time_instant__instant_drop(i00, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
}


// lift anonymous function
struct kk_std_time_calendar_tz_fixed_fun6280__t {
  struct kk_function_s _base;
  kk_std_core_types__optional abbrv;
  kk_std_time_duration__duration delta1;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_fixed_fun6280(kk_function_t _fself, kk_std_time_instant__instant i10, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_fixed_fun6280(kk_std_core_types__optional abbrv, kk_std_time_duration__duration delta1, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun6280__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_tz_fixed_fun6280__t, 3, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_tz_fixed_fun6280, kk_context());
  _self->abbrv = abbrv;
  _self->delta1 = delta1;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_fixed_fun6280(kk_function_t _fself, kk_std_time_instant__instant i10, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun6280__t* _self = kk_function_as(struct kk_std_time_calendar_tz_fixed_fun6280__t*, _fself, _ctx);
  kk_std_core_types__optional abbrv = _self->abbrv; /* optional<string> */
  kk_std_time_duration__duration delta1 = _self->delta1; /* std/time/duration/duration */
  kk_drop_match(_self, {kk_std_core_types__optional_dup(abbrv, _ctx);kk_std_time_duration__duration_dup(delta1, _ctx);}, {}, _ctx)
  kk_std_time_instant__instant_drop(i10, _ctx);
  kk_box_t _x6281;
  kk_string_t _x6282;
  if (kk_std_core_types__is_Optional(abbrv, _ctx)) {
    kk_box_t _box_x5317 = abbrv._cons.Optional.value;
    kk_string_t _abbrv_22810 = kk_string_unbox(_box_x5317);
    kk_string_dup(_abbrv_22810, _ctx);
    kk_std_core_types__optional_drop(abbrv, _ctx);
    _x6282 = _abbrv_22810; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(abbrv, _ctx);
    _x6282 = kk_string_empty(); /*string*/
  }
  _x6281 = kk_string_box(_x6282); /*7*/
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_duration__duration_box(delta1, _ctx), _x6281, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_tz_fixed_fun6284__t {
  struct kk_function_s _base;
  kk_std_time_duration__duration delta1;
};
static kk_std_core_types__maybe kk_std_time_calendar_tz_fixed_fun6284(kk_function_t _fself, kk_std_time_instant__instant i01, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_fixed_fun6284(kk_std_time_duration__duration delta1, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun6284__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_tz_fixed_fun6284__t, 1, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_tz_fixed_fun6284, kk_context());
  _self->delta1 = delta1;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_core_types__maybe kk_std_time_calendar_tz_fixed_fun6284(kk_function_t _fself, kk_std_time_instant__instant i01, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun6284__t* _self = kk_function_as(struct kk_std_time_calendar_tz_fixed_fun6284__t*, _fself, _ctx);
  kk_std_time_duration__duration delta1 = _self->delta1; /* std/time/duration/duration */
  kk_drop_match(_self, {kk_std_time_duration__duration_dup(delta1, _ctx);}, {}, _ctx)
  bool _match_5941;
  double _x6285;
  {
    kk_std_num_ddouble__ddouble _x00 = delta1.secs;
    {
      double _x1 = _x00.hi;
      _x6285 = _x1; /*float64*/
    }
  }
  _match_5941 = (_x6285 == (0x0p+0)); /*bool*/
  if (_match_5941) {
    return kk_std_core_types__new_Just(kk_std_time_instant__instant_box(i01, _ctx), _ctx);
  }
  {
    kk_std_time_instant__instant_drop(i01, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
}

kk_std_time_calendar__timezone kk_std_time_calendar_tz_fixed(kk_integer_t hours, kk_std_core_types__optional mins, kk_std_core_types__optional name0, kk_std_core_types__optional abbrv, kk_std_core_types__optional hourwidth, kk_context_t* _ctx) { /* (hours : int, mins : optional<int>, name : optional<string>, abbrv : optional<string>, hourwidth : optional<int>) -> timezone */ 
  kk_integer_t _mins_2272;
  if (kk_std_core_types__is_Optional(mins, _ctx)) {
    kk_box_t _box_x5294 = mins._cons.Optional.value;
    kk_integer_t _mins_2273 = kk_integer_unbox(_box_x5294, _ctx);
    kk_integer_dup(_mins_2273, _ctx);
    kk_std_core_types__optional_drop(mins, _ctx);
    _mins_2272 = _mins_2273; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(mins, _ctx);
    _mins_2272 = kk_integer_from_small(0); /*int*/
  }
  bool _match_5940 = kk_integer_is_zero_borrow(hours); /*bool*/;
  if (_match_5940) {
    bool _match_5953 = kk_integer_is_zero_borrow(_mins_2272); /*bool*/;
    if (_match_5953) {
      kk_std_core_types__optional_drop(name0, _ctx);
      kk_std_core_types__optional_drop(hourwidth, _ctx);
      kk_integer_drop(hours, _ctx);
      kk_std_core_types__optional_drop(abbrv, _ctx);
      kk_integer_drop(_mins_2272, _ctx);
      return kk_std_time_calendar__timezone_dup(kk_std_time_calendar_tz_utc, _ctx);
    }
    {
      kk_integer_t xmins;
      bool _match_5964 = kk_integer_gt_borrow(hours,(kk_integer_from_small(0)),kk_context()); /*bool*/;
      if (_match_5964) {
        kk_integer_t _x6200 = kk_integer_dup(_mins_2272, _ctx); /*int*/
        xmins = kk_integer_abs(_x6200,kk_context()); /*int*/
      }
      else {
        bool _match_5965 = kk_integer_lt_borrow(hours,(kk_integer_from_small(0)),kk_context()); /*bool*/;
        if (_match_5965) {
          kk_integer_t i1_4731;
          kk_integer_t _x6201 = kk_integer_dup(_mins_2272, _ctx); /*int*/
          i1_4731 = kk_integer_abs(_x6201,kk_context()); /*int*/
          xmins = kk_integer_neg(i1_4731,kk_context()); /*int*/
        }
        else {
          xmins = kk_integer_dup(_mins_2272, _ctx); /*int*/
        }
      }
      kk_integer_t x_4732;
      kk_integer_t _x6202 = kk_integer_dup(hours, _ctx); /*int*/
      x_4732 = kk_integer_mul(_x6202,(kk_integer_from_small(3600)),kk_context()); /*int*/
      kk_integer_t y_4733 = kk_integer_mul(xmins,(kk_integer_from_small(60)),kk_context()); /*int*/;
      kk_integer_t secs = kk_integer_add(x_4732,y_4733,kk_context()); /*int*/;
      kk_string_t s_4737;
      kk_integer_t _x6203 = kk_integer_abs(hours,kk_context()); /*int*/
      s_4737 = kk_std_core_show(_x6203, _ctx); /*string*/
      kk_integer_t n;
      kk_string_t _x6204 = kk_string_dup(s_4737, _ctx); /*string*/
      n = kk_string_len_int(_x6204,kk_context()); /*int*/
      kk_string_t y0_4735;
      kk_string_t _x6205;
      kk_define_string_literal(, _s6206, 3, "UTC", _ctx)
      _x6205 = kk_string_dup(_s6206, _ctx); /*string*/
      kk_string_t _x6207;
      kk_string_t _x6208;
      bool _match_5963 = kk_integer_lt_borrow(secs,(kk_integer_from_small(0)),kk_context()); /*bool*/;
      if (_match_5963) {
        kk_define_string_literal(, _s6209, 1, "-", _ctx)
        _x6208 = kk_string_dup(_s6209, _ctx); /*string*/
      }
      else {
        kk_define_string_literal(, _s6210, 1, "+", _ctx)
        _x6208 = kk_string_dup(_s6210, _ctx); /*string*/
      }
      kk_string_t _x6211;
      kk_string_t _x6212;
      bool _match_5960;
      kk_integer_t _brw_5961;
      if (kk_std_core_types__is_Optional(hourwidth, _ctx)) {
        kk_box_t _box_x5295 = hourwidth._cons.Optional.value;
        kk_integer_t _hourwidth_2285 = kk_integer_unbox(_box_x5295, _ctx);
        kk_integer_dup(_hourwidth_2285, _ctx);
        _brw_5961 = _hourwidth_2285; /*int*/
      }
      else {
        _brw_5961 = kk_integer_from_small(1); /*int*/
      }
      bool _brw_5962 = kk_integer_lte_borrow(_brw_5961,n,kk_context()); /*bool*/;
      kk_integer_drop(_brw_5961, _ctx);
      _match_5960 = _brw_5962; /*bool*/
      if (_match_5960) {
        kk_integer_drop(n, _ctx);
        kk_std_core_types__optional_drop(hourwidth, _ctx);
        _x6212 = s_4737; /*string*/
      }
      else {
        kk_string_t s0_17476 = kk_std_core_string('0', _ctx); /*string*/;
        kk_integer_t n0_17477;
        kk_integer_t _x6213;
        if (kk_std_core_types__is_Optional(hourwidth, _ctx)) {
          kk_box_t _box_x5296 = hourwidth._cons.Optional.value;
          kk_integer_t _hourwidth_22850 = kk_integer_unbox(_box_x5296, _ctx);
          kk_integer_dup(_hourwidth_22850, _ctx);
          kk_std_core_types__optional_drop(hourwidth, _ctx);
          _x6213 = _hourwidth_22850; /*int*/
        }
        else {
          kk_std_core_types__optional_drop(hourwidth, _ctx);
          _x6213 = kk_integer_from_small(1); /*int*/
        }
        n0_17477 = kk_integer_sub(_x6213,n,kk_context()); /*int*/
        kk_string_t _x6214;
        kk_ssize_t _x6215 = kk_std_core_ssize__t(n0_17477, _ctx); /*ssize_t*/
        _x6214 = kk_std_core_repeatz(s0_17476, _x6215, _ctx); /*string*/
        _x6212 = kk_std_core__lp__plus__plus__1_rp_(_x6214, s_4737, _ctx); /*string*/
      }
      kk_string_t _x6216;
      bool _match_5958 = kk_integer_is_zero_borrow(_mins_2272); /*bool*/;
      if (_match_5958) {
        kk_integer_drop(_mins_2272, _ctx);
        _x6216 = kk_string_empty(); /*string*/
      }
      else {
        kk_string_t s0_4740;
        kk_integer_t _x6218 = kk_integer_abs(_mins_2272,kk_context()); /*int*/
        s0_4740 = kk_std_core_show(_x6218, _ctx); /*string*/
        kk_integer_t n0;
        kk_string_t _x6219 = kk_string_dup(s0_4740, _ctx); /*string*/
        n0 = kk_string_len_int(_x6219,kk_context()); /*int*/
        kk_string_t _x6220;
        kk_define_string_literal(, _s6221, 1, ":", _ctx)
        _x6220 = kk_string_dup(_s6221, _ctx); /*string*/
        kk_string_t _x6222;
        bool _match_5959 = kk_integer_lte_borrow((kk_integer_from_small(2)),n0,kk_context()); /*bool*/;
        if (_match_5959) {
          kk_integer_drop(n0, _ctx);
          _x6222 = s0_4740; /*string*/
        }
        else {
          kk_string_t s0_174760 = kk_std_core_string('0', _ctx); /*string*/;
          kk_integer_t n0_174770 = kk_integer_sub((kk_integer_from_small(2)),n0,kk_context()); /*int*/;
          kk_string_t _x6223;
          kk_ssize_t _x6224 = kk_std_core_ssize__t(n0_174770, _ctx); /*ssize_t*/
          _x6223 = kk_std_core_repeatz(s0_174760, _x6224, _ctx); /*string*/
          _x6222 = kk_std_core__lp__plus__plus__1_rp_(_x6223, s0_4740, _ctx); /*string*/
        }
        _x6216 = kk_std_core__lp__plus__plus__1_rp_(_x6220, _x6222, _ctx); /*string*/
      }
      _x6211 = kk_std_core__lp__plus__plus__1_rp_(_x6212, _x6216, _ctx); /*string*/
      _x6207 = kk_std_core__lp__plus__plus__1_rp_(_x6208, _x6211, _ctx); /*string*/
      y0_4735 = kk_std_core__lp__plus__plus__1_rp_(_x6205, _x6207, _ctx); /*string*/
      kk_string_t tzonename;
      bool _match_5957;
      kk_string_t _x6225;
      if (kk_std_core_types__is_Optional(name0, _ctx)) {
        kk_box_t _box_x5297 = name0._cons.Optional.value;
        kk_string_t _name_2277 = kk_string_unbox(_box_x5297);
        kk_string_dup(_name_2277, _ctx);
        _x6225 = _name_2277; /*string*/
      }
      else {
        _x6225 = kk_string_empty(); /*string*/
      }
      kk_string_t _x6227 = kk_string_empty(); /*string*/
      _match_5957 = kk_string_is_eq(_x6225,_x6227,kk_context()); /*bool*/
      if (_match_5957) {
        kk_std_core_types__optional_drop(name0, _ctx);
        tzonename = y0_4735; /*string*/
      }
      else {
        kk_string_drop(y0_4735, _ctx);
        if (kk_std_core_types__is_Optional(name0, _ctx)) {
          kk_box_t _box_x5298 = name0._cons.Optional.value;
          kk_string_t _name_22770 = kk_string_unbox(_box_x5298);
          kk_string_dup(_name_22770, _ctx);
          kk_std_core_types__optional_drop(name0, _ctx);
          tzonename = _name_22770; /*string*/
        }
        else {
          kk_std_core_types__optional_drop(name0, _ctx);
          tzonename = kk_string_empty(); /*string*/
        }
      }
      kk_std_core_types__optional frac0_1431;
      kk_box_t _x6230;
      double _x6231;
      kk_std_core_types__optional _match_5956 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
      if (kk_std_core_types__is_Optional(_match_5956, _ctx)) {
        kk_box_t _box_x5299 = _match_5956._cons.Optional.value;
        double _frac_159 = kk_double_unbox(_box_x5299, KK_BORROWED, _ctx);
        kk_std_core_types__optional_drop(_match_5956, _ctx);
        _x6231 = _frac_159; /*float64*/
      }
      else {
        kk_std_core_types__optional_drop(_match_5956, _ctx);
        _x6231 = 0x0p+0; /*float64*/
      }
      _x6230 = kk_double_box(_x6231, _ctx); /*116*/
      frac0_1431 = kk_std_core_types__new_Optional(_x6230, _ctx); /*optional<float64>*/
      kk_std_time_duration__duration delta;
      kk_std_num_ddouble__ddouble _x6232;
      bool _match_5955;
      double _x6233;
      if (kk_std_core_types__is_Optional(frac0_1431, _ctx)) {
        kk_box_t _box_x5302 = frac0_1431._cons.Optional.value;
        double _frac_782 = kk_double_unbox(_box_x5302, KK_BORROWED, _ctx);
        _x6233 = _frac_782; /*float64*/
      }
      else {
        _x6233 = 0x0p+0; /*float64*/
      }
      _match_5955 = (_x6233 == (0x0p+0)); /*bool*/
      if (_match_5955) {
        kk_std_core_types__optional_drop(frac0_1431, _ctx);
        _x6232 = kk_std_num_ddouble_ddouble_int_exp(secs, kk_integer_from_small(0), _ctx); /*std/num/ddouble/ddouble*/
      }
      else {
        kk_std_num_ddouble__ddouble _x6234 = kk_std_num_ddouble_ddouble_int_exp(secs, kk_integer_from_small(0), _ctx); /*std/num/ddouble/ddouble*/
        kk_std_num_ddouble__ddouble _x6235;
        double _x6236;
        if (kk_std_core_types__is_Optional(frac0_1431, _ctx)) {
          kk_box_t _box_x5303 = frac0_1431._cons.Optional.value;
          double _frac_7820 = kk_double_unbox(_box_x5303, KK_BORROWED, _ctx);
          kk_std_core_types__optional_drop(frac0_1431, _ctx);
          _x6236 = _frac_7820; /*float64*/
        }
        else {
          kk_std_core_types__optional_drop(frac0_1431, _ctx);
          _x6236 = 0x0p+0; /*float64*/
        }
        _x6235 = kk_std_num_ddouble__new_Ddouble(_x6236, 0x0p+0, _ctx); /*std/num/ddouble/ddouble*/
        _x6232 = kk_std_num_ddouble__lp__plus__rp_(_x6234, _x6235, _ctx); /*std/num/ddouble/ddouble*/
      }
      delta = kk_std_time_duration__new_Duration(_x6232, _ctx); /*std/time/duration/duration*/
      return kk_std_time_calendar__new_Timezone(kk_reuse_null, 0, tzonename, kk_std_time_calendar_new_tz_fixed_fun6237(abbrv, delta, _ctx), kk_std_time_calendar_new_tz_fixed_fun6241(delta, _ctx), _ctx);
    }
  }
  {
    kk_integer_t xmins0;
    bool _match_5951 = kk_integer_gt_borrow(hours,(kk_integer_from_small(0)),kk_context()); /*bool*/;
    if (_match_5951) {
      kk_integer_t _x6243 = kk_integer_dup(_mins_2272, _ctx); /*int*/
      xmins0 = kk_integer_abs(_x6243,kk_context()); /*int*/
    }
    else {
      bool _match_5952 = kk_integer_lt_borrow(hours,(kk_integer_from_small(0)),kk_context()); /*bool*/;
      if (_match_5952) {
        kk_integer_t i7_4754;
        kk_integer_t _x6244 = kk_integer_dup(_mins_2272, _ctx); /*int*/
        i7_4754 = kk_integer_abs(_x6244,kk_context()); /*int*/
        xmins0 = kk_integer_neg(i7_4754,kk_context()); /*int*/
      }
      else {
        xmins0 = kk_integer_dup(_mins_2272, _ctx); /*int*/
      }
    }
    kk_integer_t x1_4755;
    kk_integer_t _x6245 = kk_integer_dup(hours, _ctx); /*int*/
    x1_4755 = kk_integer_mul(_x6245,(kk_integer_from_small(3600)),kk_context()); /*int*/
    kk_integer_t y1_4756 = kk_integer_mul(xmins0,(kk_integer_from_small(60)),kk_context()); /*int*/;
    kk_integer_t secs1 = kk_integer_add(x1_4755,y1_4756,kk_context()); /*int*/;
    kk_string_t s1_4760;
    kk_integer_t _x6246 = kk_integer_abs(hours,kk_context()); /*int*/
    s1_4760 = kk_std_core_show(_x6246, _ctx); /*string*/
    kk_integer_t n1;
    kk_string_t _x6247 = kk_string_dup(s1_4760, _ctx); /*string*/
    n1 = kk_string_len_int(_x6247,kk_context()); /*int*/
    kk_string_t y2_4758;
    kk_string_t _x6248;
    kk_define_string_literal(, _s6249, 3, "UTC", _ctx)
    _x6248 = kk_string_dup(_s6249, _ctx); /*string*/
    kk_string_t _x6250;
    kk_string_t _x6251;
    bool _match_5950 = kk_integer_lt_borrow(secs1,(kk_integer_from_small(0)),kk_context()); /*bool*/;
    if (_match_5950) {
      kk_define_string_literal(, _s6252, 1, "-", _ctx)
      _x6251 = kk_string_dup(_s6252, _ctx); /*string*/
    }
    else {
      kk_define_string_literal(, _s6253, 1, "+", _ctx)
      _x6251 = kk_string_dup(_s6253, _ctx); /*string*/
    }
    kk_string_t _x6254;
    kk_string_t _x6255;
    bool _match_5947;
    kk_integer_t _brw_5948;
    if (kk_std_core_types__is_Optional(hourwidth, _ctx)) {
      kk_box_t _box_x5308 = hourwidth._cons.Optional.value;
      kk_integer_t _hourwidth_228500 = kk_integer_unbox(_box_x5308, _ctx);
      kk_integer_dup(_hourwidth_228500, _ctx);
      _brw_5948 = _hourwidth_228500; /*int*/
    }
    else {
      _brw_5948 = kk_integer_from_small(1); /*int*/
    }
    bool _brw_5949 = kk_integer_lte_borrow(_brw_5948,n1,kk_context()); /*bool*/;
    kk_integer_drop(_brw_5948, _ctx);
    _match_5947 = _brw_5949; /*bool*/
    if (_match_5947) {
      kk_integer_drop(n1, _ctx);
      kk_std_core_types__optional_drop(hourwidth, _ctx);
      _x6255 = s1_4760; /*string*/
    }
    else {
      kk_string_t s0_174761 = kk_std_core_string('0', _ctx); /*string*/;
      kk_integer_t n0_174771;
      kk_integer_t _x6256;
      if (kk_std_core_types__is_Optional(hourwidth, _ctx)) {
        kk_box_t _box_x5309 = hourwidth._cons.Optional.value;
        kk_integer_t _hourwidth_228501 = kk_integer_unbox(_box_x5309, _ctx);
        kk_integer_dup(_hourwidth_228501, _ctx);
        kk_std_core_types__optional_drop(hourwidth, _ctx);
        _x6256 = _hourwidth_228501; /*int*/
      }
      else {
        kk_std_core_types__optional_drop(hourwidth, _ctx);
        _x6256 = kk_integer_from_small(1); /*int*/
      }
      n0_174771 = kk_integer_sub(_x6256,n1,kk_context()); /*int*/
      kk_string_t _x6257;
      kk_ssize_t _x6258 = kk_std_core_ssize__t(n0_174771, _ctx); /*ssize_t*/
      _x6257 = kk_std_core_repeatz(s0_174761, _x6258, _ctx); /*string*/
      _x6255 = kk_std_core__lp__plus__plus__1_rp_(_x6257, s1_4760, _ctx); /*string*/
    }
    kk_string_t _x6259;
    bool _match_5945 = kk_integer_is_zero_borrow(_mins_2272); /*bool*/;
    if (_match_5945) {
      kk_integer_drop(_mins_2272, _ctx);
      _x6259 = kk_string_empty(); /*string*/
    }
    else {
      kk_string_t s2_4763;
      kk_integer_t _x6261 = kk_integer_abs(_mins_2272,kk_context()); /*int*/
      s2_4763 = kk_std_core_show(_x6261, _ctx); /*string*/
      kk_integer_t n2;
      kk_string_t _x6262 = kk_string_dup(s2_4763, _ctx); /*string*/
      n2 = kk_string_len_int(_x6262,kk_context()); /*int*/
      kk_string_t _x6263;
      kk_define_string_literal(, _s6264, 1, ":", _ctx)
      _x6263 = kk_string_dup(_s6264, _ctx); /*string*/
      kk_string_t _x6265;
      bool _match_5946 = kk_integer_lte_borrow((kk_integer_from_small(2)),n2,kk_context()); /*bool*/;
      if (_match_5946) {
        kk_integer_drop(n2, _ctx);
        _x6265 = s2_4763; /*string*/
      }
      else {
        kk_string_t s0_174762 = kk_std_core_string('0', _ctx); /*string*/;
        kk_integer_t n0_174772 = kk_integer_sub((kk_integer_from_small(2)),n2,kk_context()); /*int*/;
        kk_string_t _x6266;
        kk_ssize_t _x6267 = kk_std_core_ssize__t(n0_174772, _ctx); /*ssize_t*/
        _x6266 = kk_std_core_repeatz(s0_174762, _x6267, _ctx); /*string*/
        _x6265 = kk_std_core__lp__plus__plus__1_rp_(_x6266, s2_4763, _ctx); /*string*/
      }
      _x6259 = kk_std_core__lp__plus__plus__1_rp_(_x6263, _x6265, _ctx); /*string*/
    }
    _x6254 = kk_std_core__lp__plus__plus__1_rp_(_x6255, _x6259, _ctx); /*string*/
    _x6250 = kk_std_core__lp__plus__plus__1_rp_(_x6251, _x6254, _ctx); /*string*/
    y2_4758 = kk_std_core__lp__plus__plus__1_rp_(_x6248, _x6250, _ctx); /*string*/
    kk_string_t tzonename0;
    bool _match_5944;
    kk_string_t _x6268;
    if (kk_std_core_types__is_Optional(name0, _ctx)) {
      kk_box_t _box_x5310 = name0._cons.Optional.value;
      kk_string_t _name_227700 = kk_string_unbox(_box_x5310);
      kk_string_dup(_name_227700, _ctx);
      _x6268 = _name_227700; /*string*/
    }
    else {
      _x6268 = kk_string_empty(); /*string*/
    }
    kk_string_t _x6270 = kk_string_empty(); /*string*/
    _match_5944 = kk_string_is_eq(_x6268,_x6270,kk_context()); /*bool*/
    if (_match_5944) {
      kk_std_core_types__optional_drop(name0, _ctx);
      tzonename0 = y2_4758; /*string*/
    }
    else {
      kk_string_drop(y2_4758, _ctx);
      if (kk_std_core_types__is_Optional(name0, _ctx)) {
        kk_box_t _box_x5311 = name0._cons.Optional.value;
        kk_string_t _name_227701 = kk_string_unbox(_box_x5311);
        kk_string_dup(_name_227701, _ctx);
        kk_std_core_types__optional_drop(name0, _ctx);
        tzonename0 = _name_227701; /*string*/
      }
      else {
        kk_std_core_types__optional_drop(name0, _ctx);
        tzonename0 = kk_string_empty(); /*string*/
      }
    }
    kk_std_core_types__optional frac0_14310;
    kk_box_t _x6273;
    double _x6274;
    kk_std_core_types__optional _match_5943 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_5943, _ctx)) {
      kk_box_t _box_x5312 = _match_5943._cons.Optional.value;
      double _frac_1590 = kk_double_unbox(_box_x5312, KK_BORROWED, _ctx);
      kk_std_core_types__optional_drop(_match_5943, _ctx);
      _x6274 = _frac_1590; /*float64*/
    }
    else {
      kk_std_core_types__optional_drop(_match_5943, _ctx);
      _x6274 = 0x0p+0; /*float64*/
    }
    _x6273 = kk_double_box(_x6274, _ctx); /*116*/
    frac0_14310 = kk_std_core_types__new_Optional(_x6273, _ctx); /*optional<float64>*/
    kk_std_time_duration__duration delta1;
    kk_std_num_ddouble__ddouble _x6275;
    bool _match_5942;
    double _x6276;
    if (kk_std_core_types__is_Optional(frac0_14310, _ctx)) {
      kk_box_t _box_x5315 = frac0_14310._cons.Optional.value;
      double _frac_7821 = kk_double_unbox(_box_x5315, KK_BORROWED, _ctx);
      _x6276 = _frac_7821; /*float64*/
    }
    else {
      _x6276 = 0x0p+0; /*float64*/
    }
    _match_5942 = (_x6276 == (0x0p+0)); /*bool*/
    if (_match_5942) {
      kk_std_core_types__optional_drop(frac0_14310, _ctx);
      _x6275 = kk_std_num_ddouble_ddouble_int_exp(secs1, kk_integer_from_small(0), _ctx); /*std/num/ddouble/ddouble*/
    }
    else {
      kk_std_num_ddouble__ddouble _x6277 = kk_std_num_ddouble_ddouble_int_exp(secs1, kk_integer_from_small(0), _ctx); /*std/num/ddouble/ddouble*/
      kk_std_num_ddouble__ddouble _x6278;
      double _x6279;
      if (kk_std_core_types__is_Optional(frac0_14310, _ctx)) {
        kk_box_t _box_x5316 = frac0_14310._cons.Optional.value;
        double _frac_78200 = kk_double_unbox(_box_x5316, KK_BORROWED, _ctx);
        kk_std_core_types__optional_drop(frac0_14310, _ctx);
        _x6279 = _frac_78200; /*float64*/
      }
      else {
        kk_std_core_types__optional_drop(frac0_14310, _ctx);
        _x6279 = 0x0p+0; /*float64*/
      }
      _x6278 = kk_std_num_ddouble__new_Ddouble(_x6279, 0x0p+0, _ctx); /*std/num/ddouble/ddouble*/
      _x6275 = kk_std_num_ddouble__lp__plus__rp_(_x6277, _x6278, _ctx); /*std/num/ddouble/ddouble*/
    }
    delta1 = kk_std_time_duration__new_Duration(_x6275, _ctx); /*std/time/duration/duration*/
    return kk_std_time_calendar__new_Timezone(kk_reuse_null, 0, tzonename0, kk_std_time_calendar_new_tz_fixed_fun6280(abbrv, delta1, _ctx), kk_std_time_calendar_new_tz_fixed_fun6284(delta1, _ctx), _ctx);
  }
}
 
// Is this the UTC timezone?

bool kk_std_time_calendar_is_tz_utc(kk_std_time_calendar__timezone tz, kk_context_t* _ctx) { /* (tz : timezone) -> bool */ 
  kk_string_t _x6286;
  {
    struct kk_std_time_calendar_Timezone* _con6287 = kk_std_time_calendar__as_Timezone(tz, _ctx);
    kk_string_t _x = _con6287->name;
    kk_function_t _pat0 = _con6287->utc_delta;
    kk_function_t _pat1 = _con6287->utc_inverse;
    if kk_likely(kk_datatype_ptr_is_unique(tz, _ctx)) {
      kk_function_drop(_pat1, _ctx);
      kk_function_drop(_pat0, _ctx);
      kk_datatype_ptr_free(tz, _ctx);
    }
    else {
      kk_string_dup(_x, _ctx);
      kk_datatype_ptr_decref(tz, _ctx);
    }
    _x6286 = _x; /*string*/
  }
  kk_string_t _x6288;
  kk_std_time_calendar__timezone _x6289 = kk_std_time_calendar_tz_utc; /*std/time/calendar/timezone*/
  {
    struct kk_std_time_calendar_Timezone* _con6290 = kk_std_time_calendar__as_Timezone(_x6289, _ctx);
    kk_string_t _x0 = _con6290->name;
    _x6288 = kk_string_dup(_x0, _ctx); /*string*/
  }
  return kk_string_is_eq(_x6286,_x6288,kk_context());
}

kk_std_time_instant__instant kk_std_time_calendar_earth_timestamp_to_instant(kk_std_time_timestamp__timestamp t, kk_std_time_calendar__timezone tz, kk_std_time_instant__timescale ts, kk_context_t* _ctx) { /* (t : std/time/timestamp/timestamp, tz : timezone, ts : std/time/instant/timescale) -> std/time/instant/instant */ 
  kk_std_time_instant__instant i;
  kk_std_time_timestamp__timestamp _x6291 = kk_std_time_timestamp__timestamp_dup(t, _ctx); /*std/time/timestamp/timestamp*/
  kk_std_time_instant__timescale _x6292 = kk_std_time_instant__timescale_dup(ts, _ctx); /*std/time/instant/timescale*/
  i = kk_std_time_instant__new_Instant(_x6291, _x6292, _ctx); /*std/time/instant/instant*/
  bool _match_5937;
  kk_string_t _x6293;
  {
    struct kk_std_time_calendar_Timezone* _con6294 = kk_std_time_calendar__as_Timezone(tz, _ctx);
    kk_string_t _x = _con6294->name;
    kk_string_dup(_x, _ctx);
    _x6293 = _x; /*string*/
  }
  kk_string_t _x6295;
  kk_std_time_calendar__timezone _x6296 = kk_std_time_calendar_tz_utc; /*std/time/calendar/timezone*/
  {
    struct kk_std_time_calendar_Timezone* _con6297 = kk_std_time_calendar__as_Timezone(_x6296, _ctx);
    kk_string_t _x0 = _con6297->name;
    _x6295 = kk_string_dup(_x0, _ctx); /*string*/
  }
  _match_5937 = kk_string_is_eq(_x6293,_x6295,kk_context()); /*bool*/
  if (_match_5937) {
    kk_datatype_ptr_dropn(tz, (KK_I32(3)), _ctx);
    kk_datatype_ptr_dropn(ts, (KK_I32(10)), _ctx);
    kk_datatype_ptr_dropn(t, (KK_I32(0)), _ctx);
    return i;
  }
  {
    struct kk_std_time_calendar_Timezone* _con6298 = kk_std_time_calendar__as_Timezone(tz, _ctx);
    kk_string_t _pat02 = _con6298->name;
    kk_function_t _pat11 = _con6298->utc_delta;
    kk_function_t _x1 = _con6298->utc_inverse;
    kk_function_dup(_x1, _ctx);
    kk_std_core_types__maybe _match_5938;
    kk_std_time_instant__instant _x6299 = kk_std_time_instant__instant_dup(i, _ctx); /*std/time/instant/instant*/
    _match_5938 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x1, (_x1, _x6299, _ctx), _ctx); /*maybe<std/time/instant/instant>*/
    if (kk_std_core_types__is_Just(_match_5938, _ctx)) {
      kk_box_t _box_x5327 = _match_5938._cons.Just.value;
      kk_std_time_instant__instant inv = kk_std_time_instant__instant_unbox(_box_x5327, KK_BORROWED, _ctx);
      if kk_likely(kk_datatype_ptr_is_unique(tz, _ctx)) {
        kk_function_drop(_x1, _ctx);
        kk_function_drop(_pat11, _ctx);
        kk_string_drop(_pat02, _ctx);
        kk_datatype_ptr_free(tz, _ctx);
      }
      else {
        kk_datatype_ptr_decref(tz, _ctx);
      }
      kk_datatype_ptr_dropn(ts, (KK_I32(10)), _ctx);
      kk_datatype_ptr_dropn(t, (KK_I32(0)), _ctx);
      kk_std_time_instant__instant_drop(i, _ctx);
      kk_std_time_instant__instant_dup(inv, _ctx);
      kk_std_core_types__maybe_drop(_match_5938, _ctx);
      return inv;
    }
    {
      kk_std_core_types__tuple2_ _this_4787;
      {
        struct kk_std_time_calendar_Timezone* _con6300 = kk_std_time_calendar__as_Timezone(tz, _ctx);
        kk_function_t _x3 = _con6300->utc_delta;
        kk_function_dup(_x3, _ctx);
        _this_4787 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x3, (_x3, i, _ctx), _ctx); /*(std/time/duration/duration, string)*/
      }
      kk_std_time_timestamp__timestamp t1_4790;
      kk_std_num_ddouble__ddouble _x6301;
      kk_std_num_ddouble__ddouble _x6302;
      {
        struct kk_std_time_timestamp_Timestamp* _con6303 = kk_std_time_timestamp__as_Timestamp(t, _ctx);
        kk_std_num_ddouble__ddouble _x10 = _con6303->since;
        _x6302 = _x10; /*std/time/timestamp/timespan*/
      }
      kk_std_num_ddouble__ddouble _x6304;
      double _x6305;
      double _x6306;
      {
        kk_box_t _box_x5328 = _this_4787.fst;
        kk_box_t _box_x5329 = _this_4787.snd;
        kk_std_time_duration__duration _x2 = kk_std_time_duration__duration_unbox(_box_x5328, KK_BORROWED, _ctx);
        {
          kk_std_num_ddouble__ddouble _x5 = _x2.secs;
          {
            double _x4 = _x5.hi;
            _x6306 = _x4; /*float64*/
          }
        }
      }
      _x6305 = (-_x6306); /*float64*/
      double _x6307;
      double _x6308;
      {
        kk_box_t _box_x5330 = _this_4787.fst;
        kk_box_t _box_x5331 = _this_4787.snd;
        kk_std_time_duration__duration _x20 = kk_std_time_duration__duration_unbox(_box_x5330, KK_BORROWED, _ctx);
        {
          kk_std_num_ddouble__ddouble _x50 = _x20.secs;
          {
            double _x00 = _x50.lo;
            _x6308 = _x00; /*float64*/
          }
        }
      }
      _x6307 = (-_x6308); /*float64*/
      _x6304 = kk_std_num_ddouble__new_Ddouble(_x6305, _x6307, _ctx); /*std/num/ddouble/ddouble*/
      _x6301 = kk_std_num_ddouble__lp__plus__rp_(_x6302, _x6304, _ctx); /*std/num/ddouble/ddouble*/
      int32_t _x6309;
      {
        struct kk_std_time_timestamp_Timestamp* _con6310 = kk_std_time_timestamp__as_Timestamp(t, _ctx);
        kk_std_num_ddouble__ddouble _pat020 = _con6310->since;
        int32_t _x200 = _con6310->leap32;
        _x6309 = _x200; /*int32*/
      }
      t1_4790 = kk_std_time_timestamp__new_Timestamp(kk_reuse_null, 0, _x6301, _x6309, _ctx); /*std/time/timestamp/timestamp*/
      kk_std_time_instant__instant tzi1;
      kk_std_time_instant__timescale _x6311 = kk_std_time_instant__timescale_dup(ts, _ctx); /*std/time/instant/timescale*/
      tzi1 = kk_std_time_instant__new_Instant(t1_4790, _x6311, _ctx); /*std/time/instant/instant*/
      kk_std_core_types__tuple2_ _this0_4794;
      {
        struct kk_std_time_calendar_Timezone* _con6312 = kk_std_time_calendar__as_Timezone(tz, _ctx);
        kk_string_t _pat08 = _con6312->name;
        kk_function_t _x7 = _con6312->utc_delta;
        kk_function_t _pat17 = _con6312->utc_inverse;
        if kk_likely(kk_datatype_ptr_is_unique(tz, _ctx)) {
          kk_function_drop(_pat17, _ctx);
          kk_string_drop(_pat08, _ctx);
          kk_datatype_ptr_free(tz, _ctx);
        }
        else {
          kk_function_dup(_x7, _ctx);
          kk_datatype_ptr_decref(tz, _ctx);
        }
        kk_std_time_instant__instant _x6313 = kk_std_time_instant__instant_dup(tzi1, _ctx); /*std/time/instant/instant*/
        _this0_4794 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x7, (_x7, _x6313, _ctx), _ctx); /*(std/time/duration/duration, string)*/
      }
      bool _match_5939;
      kk_std_time_duration__duration _x6314;
      {
        kk_box_t _box_x5332 = _this_4787.fst;
        kk_box_t _box_x5333 = _this_4787.snd;
        kk_std_time_duration__duration _x21 = kk_std_time_duration__duration_unbox(_box_x5332, KK_BORROWED, _ctx);
        kk_std_core_types__tuple2__drop(_this_4787, _ctx);
        _x6314 = _x21; /*std/time/duration/duration*/
      }
      kk_std_time_duration__duration _x6315;
      {
        kk_box_t _box_x5334 = _this0_4794.fst;
        kk_box_t _box_x5335 = _this0_4794.snd;
        kk_std_time_duration__duration _x6 = kk_std_time_duration__duration_unbox(_box_x5334, KK_BORROWED, _ctx);
        _x6315 = _x6; /*std/time/duration/duration*/
      }
      _match_5939 = kk_std_time_duration__lp__eq__eq__rp_(_x6314, _x6315, _ctx); /*bool*/
      if (_match_5939) {
        kk_datatype_ptr_dropn(ts, (KK_I32(10)), _ctx);
        kk_datatype_ptr_dropn(t, (KK_I32(0)), _ctx);
        kk_std_core_types__tuple2__drop(_this0_4794, _ctx);
        return tzi1;
      }
      {
        kk_std_time_instant__instant_drop(tzi1, _ctx);
        kk_std_time_timestamp__timestamp t3_4797;
        kk_std_num_ddouble__ddouble _x6316;
        kk_std_num_ddouble__ddouble _x6317;
        {
          struct kk_std_time_timestamp_Timestamp* _con6318 = kk_std_time_timestamp__as_Timestamp(t, _ctx);
          kk_std_num_ddouble__ddouble _x11 = _con6318->since;
          _x6317 = _x11; /*std/time/timestamp/timespan*/
        }
        kk_std_num_ddouble__ddouble _x6319;
        double _x6320;
        double _x6321;
        {
          kk_box_t _box_x5336 = _this0_4794.fst;
          kk_box_t _box_x5337 = _this0_4794.snd;
          kk_std_time_duration__duration _x60 = kk_std_time_duration__duration_unbox(_box_x5336, KK_BORROWED, _ctx);
          {
            kk_std_num_ddouble__ddouble _x9 = _x60.secs;
            {
              double _x8 = _x9.hi;
              _x6321 = _x8; /*float64*/
            }
          }
        }
        _x6320 = (-_x6321); /*float64*/
        double _x6322;
        double _x6323;
        {
          kk_box_t _box_x5338 = _this0_4794.fst;
          kk_box_t _box_x5339 = _this0_4794.snd;
          kk_std_time_duration__duration _x61 = kk_std_time_duration__duration_unbox(_box_x5338, KK_BORROWED, _ctx);
          kk_std_core_types__tuple2__drop(_this0_4794, _ctx);
          {
            kk_std_num_ddouble__ddouble _x90 = _x61.secs;
            {
              double _x01 = _x90.lo;
              _x6323 = _x01; /*float64*/
            }
          }
        }
        _x6322 = (-_x6323); /*float64*/
        _x6319 = kk_std_num_ddouble__new_Ddouble(_x6320, _x6322, _ctx); /*std/num/ddouble/ddouble*/
        _x6316 = kk_std_num_ddouble__lp__plus__rp_(_x6317, _x6319, _ctx); /*std/num/ddouble/ddouble*/
        int32_t _x6324;
        {
          struct kk_std_time_timestamp_Timestamp* _con6325 = kk_std_time_timestamp__as_Timestamp(t, _ctx);
          kk_std_num_ddouble__ddouble _pat021 = _con6325->since;
          int32_t _x210 = _con6325->leap32;
          if kk_likely(kk_datatype_ptr_is_unique(t, _ctx)) {
            kk_datatype_ptr_free(t, _ctx);
          }
          else {
            kk_datatype_ptr_decref(t, _ctx);
          }
          _x6324 = _x210; /*int32*/
        }
        t3_4797 = kk_std_time_timestamp__new_Timestamp(kk_reuse_null, 0, _x6316, _x6324, _ctx); /*std/time/timestamp/timestamp*/
        return kk_std_time_instant__new_Instant(t3_4797, ts, _ctx);
      }
    }
  }
}
 
// Create a calendar where we assume that each
// day has 24 hours with 60 minutes, and where minutes are 60 seconds
// (with possibly leap seconds). The function takes care of timezones,
// epoch shifts, and leap seconds, and only needs a calendar `name`,
// a function `days-to-date` to calculate a date given a number of days since the `epoch`,
// a function `date-to-days` to calculate days since the `epoch` for a given date,
// and an optional time scale to be used (=`ts-utc`).
// `month-prefix` (=`""`) is used when displaying numeric months and is set for
// the ISO week dates to `"W"` for example. The `show-era` function can be given
// to display the era of a date.


// lift anonymous function
struct kk_std_time_calendar_earth_calendar_fun6329__t {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x5344;
};
static kk_string_t kk_std_time_calendar_earth_calendar_fun6329(kk_function_t _fself, kk_std_time_date__date _b_5347, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_earth_calendar_fun6329(kk_box_t _fun_unbox_x5344, kk_context_t* _ctx) {
  struct kk_std_time_calendar_earth_calendar_fun6329__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_earth_calendar_fun6329__t, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_earth_calendar_fun6329, kk_context());
  _self->_fun_unbox_x5344 = _fun_unbox_x5344;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_string_t kk_std_time_calendar_earth_calendar_fun6329(kk_function_t _fself, kk_std_time_date__date _b_5347, kk_context_t* _ctx) {
  struct kk_std_time_calendar_earth_calendar_fun6329__t* _self = kk_function_as(struct kk_std_time_calendar_earth_calendar_fun6329__t*, _fself, _ctx);
  kk_box_t _fun_unbox_x5344 = _self->_fun_unbox_x5344; /* 116 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x5344, _ctx);}, {}, _ctx)
  kk_box_t _x6330;
  kk_function_t _x6331 = kk_function_unbox(_fun_unbox_x5344, _ctx); /*(5345) -> 5346*/
  _x6330 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6331, (_x6331, kk_std_time_date__date_box(_b_5347, _ctx), _ctx), _ctx); /*5346*/
  return kk_string_unbox(_x6330);
}


// lift anonymous function
struct kk_std_time_calendar_earth_calendar_fun6332__t {
  struct kk_function_s _base;
};
static kk_string_t kk_std_time_calendar_earth_calendar_fun6332(kk_function_t _fself, kk_std_time_date__date d, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_earth_calendar_fun6332(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_earth_calendar_fun6332, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_string_t kk_std_time_calendar_earth_calendar_fun6332(kk_function_t _fself, kk_std_time_date__date d, kk_context_t* _ctx) {
  kk_unused(_fself);
  kk_std_time_date__date_drop(d, _ctx);
  return kk_string_empty();
}


// lift anonymous function
struct kk_std_time_calendar_earth_calendar_fun6335__t {
  struct kk_function_s _base;
  kk_function_t days_to_date0;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_earth_calendar_fun6335(kk_function_t _fself, kk_std_time_instant__instant i, kk_std_time_duration__duration tzdelta, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_earth_calendar_fun6335(kk_function_t days_to_date0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_earth_calendar_fun6335__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_earth_calendar_fun6335__t, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_earth_calendar_fun6335, kk_context());
  _self->days_to_date0 = days_to_date0;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_earth_calendar_fun6335(kk_function_t _fself, kk_std_time_instant__instant i, kk_std_time_duration__duration tzdelta, kk_context_t* _ctx) {
  struct kk_std_time_calendar_earth_calendar_fun6335__t* _self = kk_function_as(struct kk_std_time_calendar_earth_calendar_fun6335__t*, _fself, _ctx);
  kk_function_t days_to_date0 = _self->days_to_date0; /* (days : int) -> std/time/date/date */
  kk_drop_match(_self, {kk_function_dup(days_to_date0, _ctx);}, {}, _ctx)
  kk_std_core_types__tuple2_ _match_5936;
  kk_std_time_timestamp__timestamp _x6336;
  kk_std_num_ddouble__ddouble _x6337;
  kk_std_num_ddouble__ddouble _x6338;
  {
    kk_std_time_timestamp__timestamp _x1 = i.since;
    kk_std_time_instant__timescale _pat22 = i.ts;
    struct kk_std_time_timestamp_Timestamp* _con6339 = kk_std_time_timestamp__as_Timestamp(_x1, _ctx);
    struct kk_std_time_instant_Timescale* _con6340 = kk_std_time_instant__as_Timescale(_pat22, _ctx);
    kk_std_time_timestamp__timestamp_dup(_x1, _ctx);
    {
      struct kk_std_time_timestamp_Timestamp* _con6341 = kk_std_time_timestamp__as_Timestamp(_x1, _ctx);
      kk_std_num_ddouble__ddouble _x = _con6341->since;
      if kk_likely(kk_datatype_ptr_is_unique(_x1, _ctx)) {
        kk_datatype_ptr_free(_x1, _ctx);
      }
      else {
        kk_datatype_ptr_decref(_x1, _ctx);
      }
      _x6338 = _x; /*std/time/timestamp/timespan*/
    }
  }
  kk_std_num_ddouble__ddouble _x6342;
  {
    kk_std_num_ddouble__ddouble _x2 = tzdelta.secs;
    _x6342 = _x2; /*std/time/timestamp/timespan*/
  }
  _x6337 = kk_std_num_ddouble__lp__plus__rp_(_x6338, _x6342, _ctx); /*std/num/ddouble/ddouble*/
  int32_t _x6343;
  {
    kk_std_time_timestamp__timestamp _x10 = i.since;
    kk_std_time_instant__timescale _pat220 = i.ts;
    struct kk_std_time_timestamp_Timestamp* _con6344 = kk_std_time_timestamp__as_Timestamp(_x10, _ctx);
    struct kk_std_time_instant_Timescale* _con6345 = kk_std_time_instant__as_Timescale(_pat220, _ctx);
    kk_std_time_timestamp__timestamp_dup(_x10, _ctx);
    kk_std_time_instant__instant_drop(i, _ctx);
    {
      struct kk_std_time_timestamp_Timestamp* _con6346 = kk_std_time_timestamp__as_Timestamp(_x10, _ctx);
      kk_std_num_ddouble__ddouble _pat000 = _con6346->since;
      int32_t _x0 = _con6346->leap32;
      if kk_likely(kk_datatype_ptr_is_unique(_x10, _ctx)) {
        kk_datatype_ptr_free(_x10, _ctx);
      }
      else {
        kk_datatype_ptr_decref(_x10, _ctx);
      }
      _x6343 = _x0; /*int32*/
    }
  }
  _x6336 = kk_std_time_timestamp__new_Timestamp(kk_reuse_null, 0, _x6337, _x6343, _ctx); /*std/time/timestamp/timestamp*/
  _match_5936 = kk_std_time_timestamp_days_clock(_x6336, _ctx); /*(int, std/time/date/clock)*/
  {
    kk_box_t _box_x5348 = _match_5936.fst;
    kk_box_t _box_x5349 = _match_5936.snd;
    kk_std_time_date__clock clock = kk_std_time_date__clock_unbox(_box_x5349, KK_BORROWED, _ctx);
    struct kk_std_time_date_Clock* _con6347 = kk_std_time_date__as_Clock(clock, _ctx);
    kk_integer_t days = kk_integer_unbox(_box_x5348, _ctx);
    kk_std_time_date__clock_dup(clock, _ctx);
    kk_integer_dup(days, _ctx);
    kk_std_core_types__tuple2__drop(_match_5936, _ctx);
    kk_std_time_date__date _b_5367_5350 = kk_function_call(kk_std_time_date__date, (kk_function_t, kk_integer_t, kk_context_t*), days_to_date0, (days_to_date0, days, _ctx), _ctx); /*std/time/date/date*/;
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_date__date_box(_b_5367_5350, _ctx), kk_std_time_date__clock_box(clock, _ctx), _ctx);
  }
}


// lift anonymous function
struct kk_std_time_calendar_earth_calendar_fun6349__t {
  struct kk_function_s _base;
  kk_function_t date_to_days0;
};
static kk_std_time_instant__instant kk_std_time_calendar_earth_calendar_fun6349(kk_function_t _fself, kk_std_time_date__date d00, kk_std_time_date__clock c, kk_std_time_calendar__timezone tz, kk_std_time_instant__timescale ts0, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_earth_calendar_fun6349(kk_function_t date_to_days0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_earth_calendar_fun6349__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_earth_calendar_fun6349__t, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_earth_calendar_fun6349, kk_context());
  _self->date_to_days0 = date_to_days0;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_time_instant__instant kk_std_time_calendar_earth_calendar_fun6349(kk_function_t _fself, kk_std_time_date__date d00, kk_std_time_date__clock c, kk_std_time_calendar__timezone tz, kk_std_time_instant__timescale ts0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_earth_calendar_fun6349__t* _self = kk_function_as(struct kk_std_time_calendar_earth_calendar_fun6349__t*, _fself, _ctx);
  kk_function_t date_to_days0 = _self->date_to_days0; /* (std/time/date/date) -> int */
  kk_drop_match(_self, {kk_function_dup(date_to_days0, _ctx);}, {}, _ctx)
  kk_std_time_timestamp__timestamp t_4955;
  kk_std_core_types__optional _x6350;
  kk_box_t _x6351;
  bool _x6352;
  {
    struct kk_std_time_instant_Timescale* _con6353 = kk_std_time_instant__as_Timescale(ts0, _ctx);
    kk_std_core_types__maybe _x31 = _con6353->mb_seconds_in_day;
    kk_std_core_types__maybe_dup(_x31, _ctx);
    if (kk_std_core_types__is_Nothing(_x31, _ctx)) {
      _x6352 = false; /*bool*/
    }
    else {
      kk_std_core_types__maybe_drop(_x31, _ctx);
      _x6352 = true; /*bool*/
    }
  }
  _x6351 = kk_bool_box(_x6352); /*116*/
  _x6350 = kk_std_core_types__new_Optional(_x6351, _ctx); /*optional<116>*/
  t_4955 = kk_std_time_calendar_dc_timestamp(d00, c, date_to_days0, _x6350, _ctx); /*std/time/timestamp/timestamp*/
  kk_std_time_instant__instant i0;
  kk_std_time_timestamp__timestamp _x6354 = kk_std_time_timestamp__timestamp_dup(t_4955, _ctx); /*std/time/timestamp/timestamp*/
  kk_std_time_instant__timescale _x6355 = kk_std_time_instant__timescale_dup(ts0, _ctx); /*std/time/instant/timescale*/
  i0 = kk_std_time_instant__new_Instant(_x6354, _x6355, _ctx); /*std/time/instant/instant*/
  bool _match_5933;
  kk_string_t _x6356;
  {
    struct kk_std_time_calendar_Timezone* _con6357 = kk_std_time_calendar__as_Timezone(tz, _ctx);
    kk_string_t _x3 = _con6357->name;
    kk_string_dup(_x3, _ctx);
    _x6356 = _x3; /*string*/
  }
  kk_string_t _x6358;
  kk_std_time_calendar__timezone _x6359 = kk_std_time_calendar_tz_utc; /*std/time/calendar/timezone*/
  {
    struct kk_std_time_calendar_Timezone* _con6360 = kk_std_time_calendar__as_Timezone(_x6359, _ctx);
    kk_string_t _x00 = _con6360->name;
    _x6358 = kk_string_dup(_x00, _ctx); /*string*/
  }
  _match_5933 = kk_string_is_eq(_x6356,_x6358,kk_context()); /*bool*/
  if (_match_5933) {
    kk_datatype_ptr_dropn(tz, (KK_I32(3)), _ctx);
    kk_datatype_ptr_dropn(ts0, (KK_I32(10)), _ctx);
    kk_datatype_ptr_dropn(t_4955, (KK_I32(0)), _ctx);
    return i0;
  }
  {
    struct kk_std_time_calendar_Timezone* _con6361 = kk_std_time_calendar__as_Timezone(tz, _ctx);
    kk_string_t _pat0200 = _con6361->name;
    kk_function_t _pat110 = _con6361->utc_delta;
    kk_function_t _x100 = _con6361->utc_inverse;
    kk_function_dup(_x100, _ctx);
    kk_std_core_types__maybe _match_5934;
    kk_std_time_instant__instant _x6362 = kk_std_time_instant__instant_dup(i0, _ctx); /*std/time/instant/instant*/
    _match_5934 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x100, (_x100, _x6362, _ctx), _ctx); /*maybe<std/time/instant/instant>*/
    if (kk_std_core_types__is_Just(_match_5934, _ctx)) {
      kk_box_t _box_x5354 = _match_5934._cons.Just.value;
      kk_std_time_instant__instant inv = kk_std_time_instant__instant_unbox(_box_x5354, KK_BORROWED, _ctx);
      if kk_likely(kk_datatype_ptr_is_unique(tz, _ctx)) {
        kk_function_drop(_x100, _ctx);
        kk_function_drop(_pat110, _ctx);
        kk_string_drop(_pat0200, _ctx);
        kk_datatype_ptr_free(tz, _ctx);
      }
      else {
        kk_datatype_ptr_decref(tz, _ctx);
      }
      kk_datatype_ptr_dropn(ts0, (KK_I32(10)), _ctx);
      kk_datatype_ptr_dropn(t_4955, (KK_I32(0)), _ctx);
      kk_std_time_instant__instant_drop(i0, _ctx);
      kk_std_time_instant__instant_dup(inv, _ctx);
      kk_std_core_types__maybe_drop(_match_5934, _ctx);
      return inv;
    }
    {
      kk_std_core_types__tuple2_ _this_4787;
      {
        struct kk_std_time_calendar_Timezone* _con6363 = kk_std_time_calendar__as_Timezone(tz, _ctx);
        kk_function_t _x30 = _con6363->utc_delta;
        kk_function_dup(_x30, _ctx);
        _this_4787 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x30, (_x30, i0, _ctx), _ctx); /*(std/time/duration/duration, string)*/
      }
      kk_std_time_timestamp__timestamp t1_4790;
      kk_std_num_ddouble__ddouble _x6364;
      kk_std_num_ddouble__ddouble _x6365;
      {
        struct kk_std_time_timestamp_Timestamp* _con6366 = kk_std_time_timestamp__as_Timestamp(t_4955, _ctx);
        kk_std_num_ddouble__ddouble _x1000 = _con6366->since;
        _x6365 = _x1000; /*std/time/timestamp/timespan*/
      }
      kk_std_num_ddouble__ddouble _x6367;
      double _x6368;
      double _x6369;
      {
        kk_box_t _box_x5355 = _this_4787.fst;
        kk_box_t _box_x5356 = _this_4787.snd;
        kk_std_time_duration__duration _x20 = kk_std_time_duration__duration_unbox(_box_x5355, KK_BORROWED, _ctx);
        {
          kk_std_num_ddouble__ddouble _x5 = _x20.secs;
          {
            double _x4 = _x5.hi;
            _x6369 = _x4; /*float64*/
          }
        }
      }
      _x6368 = (-_x6369); /*float64*/
      double _x6370;
      double _x6371;
      {
        kk_box_t _box_x5357 = _this_4787.fst;
        kk_box_t _box_x5358 = _this_4787.snd;
        kk_std_time_duration__duration _x200 = kk_std_time_duration__duration_unbox(_box_x5357, KK_BORROWED, _ctx);
        {
          kk_std_num_ddouble__ddouble _x50 = _x200.secs;
          {
            double _x000 = _x50.lo;
            _x6371 = _x000; /*float64*/
          }
        }
      }
      _x6370 = (-_x6371); /*float64*/
      _x6367 = kk_std_num_ddouble__new_Ddouble(_x6368, _x6370, _ctx); /*std/num/ddouble/ddouble*/
      _x6364 = kk_std_num_ddouble__lp__plus__rp_(_x6365, _x6367, _ctx); /*std/num/ddouble/ddouble*/
      int32_t _x6372;
      {
        struct kk_std_time_timestamp_Timestamp* _con6373 = kk_std_time_timestamp__as_Timestamp(t_4955, _ctx);
        kk_std_num_ddouble__ddouble _pat02000 = _con6373->since;
        int32_t _x2000 = _con6373->leap32;
        _x6372 = _x2000; /*int32*/
      }
      t1_4790 = kk_std_time_timestamp__new_Timestamp(kk_reuse_null, 0, _x6364, _x6372, _ctx); /*std/time/timestamp/timestamp*/
      kk_std_time_instant__instant tzi1;
      kk_std_time_instant__timescale _x6374 = kk_std_time_instant__timescale_dup(ts0, _ctx); /*std/time/instant/timescale*/
      tzi1 = kk_std_time_instant__new_Instant(t1_4790, _x6374, _ctx); /*std/time/instant/instant*/
      kk_std_core_types__tuple2_ _this0_4794;
      {
        struct kk_std_time_calendar_Timezone* _con6375 = kk_std_time_calendar__as_Timezone(tz, _ctx);
        kk_string_t _pat08 = _con6375->name;
        kk_function_t _x7 = _con6375->utc_delta;
        kk_function_t _pat17 = _con6375->utc_inverse;
        if kk_likely(kk_datatype_ptr_is_unique(tz, _ctx)) {
          kk_function_drop(_pat17, _ctx);
          kk_string_drop(_pat08, _ctx);
          kk_datatype_ptr_free(tz, _ctx);
        }
        else {
          kk_function_dup(_x7, _ctx);
          kk_datatype_ptr_decref(tz, _ctx);
        }
        kk_std_time_instant__instant _x6376 = kk_std_time_instant__instant_dup(tzi1, _ctx); /*std/time/instant/instant*/
        _this0_4794 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x7, (_x7, _x6376, _ctx), _ctx); /*(std/time/duration/duration, string)*/
      }
      bool _match_5935;
      kk_std_time_duration__duration _x6377;
      {
        kk_box_t _box_x5359 = _this_4787.fst;
        kk_box_t _box_x5360 = _this_4787.snd;
        kk_std_time_duration__duration _x201 = kk_std_time_duration__duration_unbox(_box_x5359, KK_BORROWED, _ctx);
        kk_std_core_types__tuple2__drop(_this_4787, _ctx);
        _x6377 = _x201; /*std/time/duration/duration*/
      }
      kk_std_time_duration__duration _x6378;
      {
        kk_box_t _box_x5361 = _this0_4794.fst;
        kk_box_t _box_x5362 = _this0_4794.snd;
        kk_std_time_duration__duration _x6 = kk_std_time_duration__duration_unbox(_box_x5361, KK_BORROWED, _ctx);
        _x6378 = _x6; /*std/time/duration/duration*/
      }
      _match_5935 = kk_std_time_duration__lp__eq__eq__rp_(_x6377, _x6378, _ctx); /*bool*/
      if (_match_5935) {
        kk_datatype_ptr_dropn(ts0, (KK_I32(10)), _ctx);
        kk_datatype_ptr_dropn(t_4955, (KK_I32(0)), _ctx);
        kk_std_core_types__tuple2__drop(_this0_4794, _ctx);
        return tzi1;
      }
      {
        kk_std_time_instant__instant_drop(tzi1, _ctx);
        kk_std_time_timestamp__timestamp t3_4797;
        kk_std_num_ddouble__ddouble _x6379;
        kk_std_num_ddouble__ddouble _x6380;
        {
          struct kk_std_time_timestamp_Timestamp* _con6381 = kk_std_time_timestamp__as_Timestamp(t_4955, _ctx);
          kk_std_num_ddouble__ddouble _x11 = _con6381->since;
          _x6380 = _x11; /*std/time/timestamp/timespan*/
        }
        kk_std_num_ddouble__ddouble _x6382;
        double _x6383;
        double _x6384;
        {
          kk_box_t _box_x5363 = _this0_4794.fst;
          kk_box_t _box_x5364 = _this0_4794.snd;
          kk_std_time_duration__duration _x60 = kk_std_time_duration__duration_unbox(_box_x5363, KK_BORROWED, _ctx);
          {
            kk_std_num_ddouble__ddouble _x9 = _x60.secs;
            {
              double _x8 = _x9.hi;
              _x6384 = _x8; /*float64*/
            }
          }
        }
        _x6383 = (-_x6384); /*float64*/
        double _x6385;
        double _x6386;
        {
          kk_box_t _box_x5365 = _this0_4794.fst;
          kk_box_t _box_x5366 = _this0_4794.snd;
          kk_std_time_duration__duration _x61 = kk_std_time_duration__duration_unbox(_box_x5365, KK_BORROWED, _ctx);
          kk_std_core_types__tuple2__drop(_this0_4794, _ctx);
          {
            kk_std_num_ddouble__ddouble _x90 = _x61.secs;
            {
              double _x01 = _x90.lo;
              _x6386 = _x01; /*float64*/
            }
          }
        }
        _x6385 = (-_x6386); /*float64*/
        _x6382 = kk_std_num_ddouble__new_Ddouble(_x6383, _x6385, _ctx); /*std/num/ddouble/ddouble*/
        _x6379 = kk_std_num_ddouble__lp__plus__rp_(_x6380, _x6382, _ctx); /*std/num/ddouble/ddouble*/
        int32_t _x6387;
        {
          struct kk_std_time_timestamp_Timestamp* _con6388 = kk_std_time_timestamp__as_Timestamp(t_4955, _ctx);
          kk_std_num_ddouble__ddouble _pat021 = _con6388->since;
          int32_t _x21 = _con6388->leap32;
          if kk_likely(kk_datatype_ptr_is_unique(t_4955, _ctx)) {
            kk_datatype_ptr_free(t_4955, _ctx);
          }
          else {
            kk_datatype_ptr_decref(t_4955, _ctx);
          }
          _x6387 = _x21; /*int32*/
        }
        t3_4797 = kk_std_time_timestamp__new_Timestamp(kk_reuse_null, 0, _x6379, _x6387, _ctx); /*std/time/timestamp/timestamp*/
        return kk_std_time_instant__new_Instant(t3_4797, ts0, _ctx);
      }
    }
  }
}

kk_std_time_calendar__calendar kk_std_time_calendar_earth_calendar(kk_string_t name0, kk_string_t long_name0, kk_function_t days_to_date0, kk_function_t date_to_days0, kk_std_core_types__optional month_prefix0, kk_std_core_types__optional show_era0, kk_context_t* _ctx) { /* (name : string, long-name : string, days-to-date : (days : int) -> std/time/date/date, date-to-days : (std/time/date/date) -> int, month-prefix : optional<string>, show-era : optional<(std/time/date/date) -> string>) -> calendar */ 
  kk_string_t _x6326;
  if (kk_std_core_types__is_Optional(month_prefix0, _ctx)) {
    kk_box_t _box_x5340 = month_prefix0._cons.Optional.value;
    kk_string_t _month_prefix_3624 = kk_string_unbox(_box_x5340);
    kk_string_dup(_month_prefix_3624, _ctx);
    kk_std_core_types__optional_drop(month_prefix0, _ctx);
    _x6326 = _month_prefix_3624; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(month_prefix0, _ctx);
    _x6326 = kk_string_empty(); /*string*/
  }
  kk_function_t _x6328;
  if (kk_std_core_types__is_Optional(show_era0, _ctx)) {
    kk_box_t _fun_unbox_x5344 = show_era0._cons.Optional.value;
    kk_box_dup(_fun_unbox_x5344, _ctx);
    kk_std_core_types__optional_drop(show_era0, _ctx);
    _x6328 = kk_std_time_calendar_new_earth_calendar_fun6329(_fun_unbox_x5344, _ctx); /*(std/time/date/date) -> string*/
  }
  else {
    kk_std_core_types__optional_drop(show_era0, _ctx);
    _x6328 = kk_std_time_calendar_new_earth_calendar_fun6332(_ctx); /*(std/time/date/date) -> string*/
  }
  kk_function_t _x6334;
  kk_function_dup(days_to_date0, _ctx);
  _x6334 = kk_std_time_calendar_new_earth_calendar_fun6335(days_to_date0, _ctx); /*(i : std/time/instant/instant, tzdelta : std/time/duration/duration) -> (6, 7)*/
  kk_function_t _x6348;
  kk_function_dup(date_to_days0, _ctx);
  _x6348 = kk_std_time_calendar_new_earth_calendar_fun6349(date_to_days0, _ctx); /*(d00 : std/time/date/date, c : std/time/date/clock, tz : std/time/calendar/timezone, ts0 : std/time/instant/timescale) -> std/time/instant/instant*/
  return kk_std_time_calendar__new_Calendar(kk_reuse_null, 0, name0, long_name0, _x6326, _x6328, _x6334, _x6348, days_to_date0, date_to_days0, _ctx);
}
 
// An abstraction over solar calendars. Almost all calendars can be defined
// in terms of this function (and `solar-ecalendar`). Given functions to
// convert days to and from a year, and day-of-the-year to and from month
// and month day, this constructs a full calendar.
// Also needs an `epoch-shift` that gives the number of days from the
// calendar epoch to the date ``2000-01-01``. By default this is `730119`
// which corresponds to a ``0001-01-01`` ISO calendar epoch. The
// `has-year-zero` (=`True`) parameter is used when the calendar has a
// year zero. The `month-prefix` (=`""`) is used for displaying numeric
// months (and is `"W"` for `cal-iso-week` for example). The `show-era` function
// can be given to display an era for a given date. Finally the `ts` (=`ts-ti`)
// gives the time scale for the calendar.


// lift anonymous function
struct kk_std_time_calendar_solar_calendar_fun6389__t {
  struct kk_function_s _base;
  kk_function_t days_before_month;
  kk_function_t days_to_month;
  kk_function_t days_to_yeardoy;
  kk_std_core_types__optional epoch_shift;
  kk_std_core_types__optional has_year_zero;
};
static kk_std_time_date__date kk_std_time_calendar_solar_calendar_fun6389(kk_function_t _fself, kk_integer_t days0, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_solar_calendar_fun6389(kk_function_t days_before_month, kk_function_t days_to_month, kk_function_t days_to_yeardoy, kk_std_core_types__optional epoch_shift, kk_std_core_types__optional has_year_zero, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun6389__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_solar_calendar_fun6389__t, 8, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_solar_calendar_fun6389, kk_context());
  _self->days_before_month = days_before_month;
  _self->days_to_month = days_to_month;
  _self->days_to_yeardoy = days_to_yeardoy;
  _self->epoch_shift = epoch_shift;
  _self->has_year_zero = has_year_zero;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_time_date__date kk_std_time_calendar_solar_calendar_fun6389(kk_function_t _fself, kk_integer_t days0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun6389__t* _self = kk_function_as(struct kk_std_time_calendar_solar_calendar_fun6389__t*, _fself, _ctx);
  kk_function_t days_before_month = _self->days_before_month; /* (year : int, month : int) -> int */
  kk_function_t days_to_month = _self->days_to_month; /* (year : int, doy : int) -> int */
  kk_function_t days_to_yeardoy = _self->days_to_yeardoy; /* (days : int) -> (int, int) */
  kk_std_core_types__optional epoch_shift = _self->epoch_shift; /* optional<int> */
  kk_std_core_types__optional has_year_zero = _self->has_year_zero; /* optional<bool> */
  kk_drop_match(_self, {kk_function_dup(days_before_month, _ctx);kk_function_dup(days_to_month, _ctx);kk_function_dup(days_to_yeardoy, _ctx);kk_std_core_types__optional_dup(epoch_shift, _ctx);kk_std_core_types__optional_dup(has_year_zero, _ctx);}, {}, _ctx)
  kk_integer_t days1;
  kk_integer_t _x6390;
  if (kk_std_core_types__is_Optional(epoch_shift, _ctx)) {
    kk_box_t _box_x5369 = epoch_shift._cons.Optional.value;
    kk_integer_t _epoch_shift_3691 = kk_integer_unbox(_box_x5369, _ctx);
    kk_integer_dup(_epoch_shift_3691, _ctx);
    kk_std_core_types__optional_drop(epoch_shift, _ctx);
    _x6390 = _epoch_shift_3691; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(epoch_shift, _ctx);
    _x6390 = kk_integer_from_int(730119, _ctx); /*int*/
  }
  days1 = kk_integer_add(days0,_x6390,kk_context()); /*int*/
  kk_std_core_types__tuple2_ _match_5931 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_integer_t, kk_context_t*), days_to_yeardoy, (days_to_yeardoy, days1, _ctx), _ctx); /*(int, int)*/;
  {
    kk_box_t _box_x5370 = _match_5931.fst;
    kk_box_t _box_x5371 = _match_5931.snd;
    kk_integer_t y0 = kk_integer_unbox(_box_x5370, _ctx);
    kk_integer_t doy = kk_integer_unbox(_box_x5371, _ctx);
    kk_integer_dup(doy, _ctx);
    kk_integer_dup(y0, _ctx);
    kk_std_core_types__tuple2__drop(_match_5931, _ctx);
    kk_integer_t month;
    kk_integer_t _x6391 = kk_integer_dup(y0, _ctx); /*int*/
    kk_integer_t _x6392 = kk_integer_dup(doy, _ctx); /*int*/
    month = kk_function_call(kk_integer_t, (kk_function_t, kk_integer_t, kk_integer_t, kk_context_t*), days_to_month, (days_to_month, _x6391, _x6392, _ctx), _ctx); /*int*/
    kk_integer_t y2_4820;
    kk_integer_t _x6393 = kk_integer_dup(y0, _ctx); /*int*/
    kk_integer_t _x6394 = kk_integer_dup(month, _ctx); /*int*/
    y2_4820 = kk_function_call(kk_integer_t, (kk_function_t, kk_integer_t, kk_integer_t, kk_context_t*), days_before_month, (days_before_month, _x6393, _x6394, _ctx), _ctx); /*int*/
    kk_integer_t x0_4817 = kk_integer_sub(doy,y2_4820,kk_context()); /*int*/;
    kk_integer_t day = kk_integer_add_small_const(x0_4817, 1, _ctx); /*int*/;
    kk_integer_t year;
    if (kk_std_core_types__is_Optional(has_year_zero, _ctx)) {
      kk_box_t _box_x5372 = has_year_zero._cons.Optional.value;
      bool _has_year_zero_3695 = kk_bool_unbox(_box_x5372);
      kk_std_core_types__optional_drop(has_year_zero, _ctx);
      if (_has_year_zero_3695) {
        year = y0; /*int*/
      }
      else {
        bool _match_5932 = kk_integer_gt_borrow(y0,(kk_integer_from_small(0)),kk_context()); /*bool*/;
        if (_match_5932) {
          year = y0; /*int*/
        }
        else {
          year = kk_integer_add_small_const(y0, -1, _ctx); /*int*/
        }
      }
    }
    else {
      kk_std_core_types__optional_drop(has_year_zero, _ctx);
      year = y0; /*int*/
    }
    return kk_std_time_date__new_Date(year, month, day, _ctx);
  }
}


// lift anonymous function
struct kk_std_time_calendar_solar_calendar_fun6395__t {
  struct kk_function_s _base;
  kk_function_t days_before_month;
  kk_function_t days_before_year;
  kk_std_core_types__optional epoch_shift;
  kk_std_core_types__optional has_year_zero;
};
static kk_integer_t kk_std_time_calendar_solar_calendar_fun6395(kk_function_t _fself, kk_std_time_date__date d01, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_solar_calendar_fun6395(kk_function_t days_before_month, kk_function_t days_before_year, kk_std_core_types__optional epoch_shift, kk_std_core_types__optional has_year_zero, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun6395__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_solar_calendar_fun6395__t, 7, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_solar_calendar_fun6395, kk_context());
  _self->days_before_month = days_before_month;
  _self->days_before_year = days_before_year;
  _self->epoch_shift = epoch_shift;
  _self->has_year_zero = has_year_zero;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_integer_t kk_std_time_calendar_solar_calendar_fun6395(kk_function_t _fself, kk_std_time_date__date d01, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun6395__t* _self = kk_function_as(struct kk_std_time_calendar_solar_calendar_fun6395__t*, _fself, _ctx);
  kk_function_t days_before_month = _self->days_before_month; /* (year : int, month : int) -> int */
  kk_function_t days_before_year = _self->days_before_year; /* (year : int) -> int */
  kk_std_core_types__optional epoch_shift = _self->epoch_shift; /* optional<int> */
  kk_std_core_types__optional has_year_zero = _self->has_year_zero; /* optional<bool> */
  kk_drop_match(_self, {kk_function_dup(days_before_month, _ctx);kk_function_dup(days_before_year, _ctx);kk_std_core_types__optional_dup(epoch_shift, _ctx);kk_std_core_types__optional_dup(has_year_zero, _ctx);}, {}, _ctx)
  kk_integer_t y3;
  if (kk_std_core_types__is_Optional(has_year_zero, _ctx)) {
    kk_box_t _box_x5373 = has_year_zero._cons.Optional.value;
    bool _has_year_zero_36950 = kk_bool_unbox(_box_x5373);
    kk_std_core_types__optional_drop(has_year_zero, _ctx);
    if (_has_year_zero_36950) {
      kk_integer_t _x4 = d01.year;
      kk_integer_dup(_x4, _ctx);
      y3 = _x4; /*int*/
    }
    else {
      bool _match_5928;
      kk_integer_t _brw_5929;
      {
        kk_integer_t _x5 = d01.year;
        kk_integer_dup(_x5, _ctx);
        _brw_5929 = _x5; /*int*/
      }
      bool _brw_5930 = kk_integer_gt_borrow(_brw_5929,(kk_integer_from_small(0)),kk_context()); /*bool*/;
      kk_integer_drop(_brw_5929, _ctx);
      _match_5928 = _brw_5930; /*bool*/
      if (_match_5928) {
        kk_integer_t _x6 = d01.year;
        kk_integer_dup(_x6, _ctx);
        y3 = _x6; /*int*/
      }
      else {
        kk_integer_t _x6396;
        {
          kk_integer_t _x7 = d01.year;
          kk_integer_dup(_x7, _ctx);
          _x6396 = _x7; /*int*/
        }
        y3 = kk_integer_add_small_const(_x6396, 1, _ctx); /*int*/
      }
    }
  }
  else {
    kk_std_core_types__optional_drop(has_year_zero, _ctx);
    {
      kk_integer_t _x40 = d01.year;
      kk_integer_dup(_x40, _ctx);
      y3 = _x40; /*int*/
    }
  }
  kk_integer_t x4_4833;
  kk_integer_t _x6397 = kk_integer_dup(y3, _ctx); /*int*/
  x4_4833 = kk_function_call(kk_integer_t, (kk_function_t, kk_integer_t, kk_context_t*), days_before_year, (days_before_year, _x6397, _ctx), _ctx); /*int*/
  kk_integer_t y6_4834;
  kk_integer_t _x6398;
  {
    kk_integer_t _x8 = d01.month;
    kk_integer_dup(_x8, _ctx);
    _x6398 = _x8; /*int*/
  }
  y6_4834 = kk_function_call(kk_integer_t, (kk_function_t, kk_integer_t, kk_integer_t, kk_context_t*), days_before_month, (days_before_month, y3, _x6398, _ctx), _ctx); /*int*/
  kk_integer_t x3_4831 = kk_integer_add(x4_4833,y6_4834,kk_context()); /*int*/;
  kk_integer_t y5_4832;
  kk_integer_t _x6399;
  {
    kk_integer_t _x9 = d01.day;
    kk_integer_dup(_x9, _ctx);
    kk_std_time_date__date_drop(d01, _ctx);
    _x6399 = _x9; /*int*/
  }
  y5_4832 = kk_integer_add_small_const(_x6399, -1, _ctx); /*int*/
  kk_integer_t x2_4829 = kk_integer_add(x3_4831,y5_4832,kk_context()); /*int*/;
  kk_integer_t _x6400;
  if (kk_std_core_types__is_Optional(epoch_shift, _ctx)) {
    kk_box_t _box_x5374 = epoch_shift._cons.Optional.value;
    kk_integer_t _epoch_shift_36910 = kk_integer_unbox(_box_x5374, _ctx);
    kk_integer_dup(_epoch_shift_36910, _ctx);
    kk_std_core_types__optional_drop(epoch_shift, _ctx);
    _x6400 = _epoch_shift_36910; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(epoch_shift, _ctx);
    _x6400 = kk_integer_from_int(730119, _ctx); /*int*/
  }
  return kk_integer_sub(x2_4829,_x6400,kk_context());
}


// lift anonymous function
struct kk_std_time_calendar_solar_calendar_fun6404__t {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x5379;
};
static kk_string_t kk_std_time_calendar_solar_calendar_fun6404(kk_function_t _fself, kk_std_time_date__date _b_5382, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_solar_calendar_fun6404(kk_box_t _fun_unbox_x5379, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun6404__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_solar_calendar_fun6404__t, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_solar_calendar_fun6404, kk_context());
  _self->_fun_unbox_x5379 = _fun_unbox_x5379;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_string_t kk_std_time_calendar_solar_calendar_fun6404(kk_function_t _fself, kk_std_time_date__date _b_5382, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun6404__t* _self = kk_function_as(struct kk_std_time_calendar_solar_calendar_fun6404__t*, _fself, _ctx);
  kk_box_t _fun_unbox_x5379 = _self->_fun_unbox_x5379; /* 116 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x5379, _ctx);}, {}, _ctx)
  kk_box_t _x6405;
  kk_function_t _x6406 = kk_function_unbox(_fun_unbox_x5379, _ctx); /*(5380) -> 5381*/
  _x6405 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6406, (_x6406, kk_std_time_date__date_box(_b_5382, _ctx), _ctx), _ctx); /*5381*/
  return kk_string_unbox(_x6405);
}


// lift anonymous function
struct kk_std_time_calendar_solar_calendar_fun6407__t {
  struct kk_function_s _base;
};
static kk_string_t kk_std_time_calendar_solar_calendar_fun6407(kk_function_t _fself, kk_std_time_date__date d1, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_solar_calendar_fun6407(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_solar_calendar_fun6407, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_string_t kk_std_time_calendar_solar_calendar_fun6407(kk_function_t _fself, kk_std_time_date__date d1, kk_context_t* _ctx) {
  kk_unused(_fself);
  kk_std_time_date__date_drop(d1, _ctx);
  return kk_string_empty();
}


// lift anonymous function
struct kk_std_time_calendar_solar_calendar_fun6410__t {
  struct kk_function_s _base;
  kk_function_t days_to_date0_4811;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_solar_calendar_fun6410(kk_function_t _fself, kk_std_time_instant__instant i, kk_std_time_duration__duration tzdelta, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_solar_calendar_fun6410(kk_function_t days_to_date0_4811, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun6410__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_solar_calendar_fun6410__t, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_solar_calendar_fun6410, kk_context());
  _self->days_to_date0_4811 = days_to_date0_4811;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_solar_calendar_fun6410(kk_function_t _fself, kk_std_time_instant__instant i, kk_std_time_duration__duration tzdelta, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun6410__t* _self = kk_function_as(struct kk_std_time_calendar_solar_calendar_fun6410__t*, _fself, _ctx);
  kk_function_t days_to_date0_4811 = _self->days_to_date0_4811; /* (days : int) -> std/time/date/date */
  kk_drop_match(_self, {kk_function_dup(days_to_date0_4811, _ctx);}, {}, _ctx)
  kk_std_core_types__tuple2_ _match_5927;
  kk_std_time_timestamp__timestamp _x6411;
  kk_std_num_ddouble__ddouble _x6412;
  kk_std_num_ddouble__ddouble _x6413;
  {
    kk_std_time_timestamp__timestamp _x1 = i.since;
    kk_std_time_instant__timescale _pat23 = i.ts;
    struct kk_std_time_timestamp_Timestamp* _con6414 = kk_std_time_timestamp__as_Timestamp(_x1, _ctx);
    struct kk_std_time_instant_Timescale* _con6415 = kk_std_time_instant__as_Timescale(_pat23, _ctx);
    kk_std_time_timestamp__timestamp_dup(_x1, _ctx);
    {
      struct kk_std_time_timestamp_Timestamp* _con6416 = kk_std_time_timestamp__as_Timestamp(_x1, _ctx);
      kk_std_num_ddouble__ddouble _x = _con6416->since;
      if kk_likely(kk_datatype_ptr_is_unique(_x1, _ctx)) {
        kk_datatype_ptr_free(_x1, _ctx);
      }
      else {
        kk_datatype_ptr_decref(_x1, _ctx);
      }
      _x6413 = _x; /*std/time/timestamp/timespan*/
    }
  }
  kk_std_num_ddouble__ddouble _x6417;
  {
    kk_std_num_ddouble__ddouble _x2 = tzdelta.secs;
    _x6417 = _x2; /*std/time/timestamp/timespan*/
  }
  _x6412 = kk_std_num_ddouble__lp__plus__rp_(_x6413, _x6417, _ctx); /*std/num/ddouble/ddouble*/
  int32_t _x6418;
  {
    kk_std_time_timestamp__timestamp _x10 = i.since;
    kk_std_time_instant__timescale _pat230 = i.ts;
    struct kk_std_time_timestamp_Timestamp* _con6419 = kk_std_time_timestamp__as_Timestamp(_x10, _ctx);
    struct kk_std_time_instant_Timescale* _con6420 = kk_std_time_instant__as_Timescale(_pat230, _ctx);
    kk_std_time_timestamp__timestamp_dup(_x10, _ctx);
    kk_std_time_instant__instant_drop(i, _ctx);
    {
      struct kk_std_time_timestamp_Timestamp* _con6421 = kk_std_time_timestamp__as_Timestamp(_x10, _ctx);
      kk_std_num_ddouble__ddouble _pat000 = _con6421->since;
      int32_t _x0 = _con6421->leap32;
      if kk_likely(kk_datatype_ptr_is_unique(_x10, _ctx)) {
        kk_datatype_ptr_free(_x10, _ctx);
      }
      else {
        kk_datatype_ptr_decref(_x10, _ctx);
      }
      _x6418 = _x0; /*int32*/
    }
  }
  _x6411 = kk_std_time_timestamp__new_Timestamp(kk_reuse_null, 0, _x6412, _x6418, _ctx); /*std/time/timestamp/timestamp*/
  _match_5927 = kk_std_time_timestamp_days_clock(_x6411, _ctx); /*(int, std/time/date/clock)*/
  {
    kk_box_t _box_x5383 = _match_5927.fst;
    kk_box_t _box_x5384 = _match_5927.snd;
    kk_std_time_date__clock clock = kk_std_time_date__clock_unbox(_box_x5384, KK_BORROWED, _ctx);
    struct kk_std_time_date_Clock* _con6422 = kk_std_time_date__as_Clock(clock, _ctx);
    kk_integer_t days = kk_integer_unbox(_box_x5383, _ctx);
    kk_std_time_date__clock_dup(clock, _ctx);
    kk_integer_dup(days, _ctx);
    kk_std_core_types__tuple2__drop(_match_5927, _ctx);
    kk_std_time_date__date _b_5402_5385 = kk_function_call(kk_std_time_date__date, (kk_function_t, kk_integer_t, kk_context_t*), days_to_date0_4811, (days_to_date0_4811, days, _ctx), _ctx); /*std/time/date/date*/;
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_date__date_box(_b_5402_5385, _ctx), kk_std_time_date__clock_box(clock, _ctx), _ctx);
  }
}


// lift anonymous function
struct kk_std_time_calendar_solar_calendar_fun6424__t {
  struct kk_function_s _base;
  kk_function_t date_to_days0_4812;
};
static kk_std_time_instant__instant kk_std_time_calendar_solar_calendar_fun6424(kk_function_t _fself, kk_std_time_date__date d00, kk_std_time_date__clock c, kk_std_time_calendar__timezone tz, kk_std_time_instant__timescale ts0, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_solar_calendar_fun6424(kk_function_t date_to_days0_4812, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun6424__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_solar_calendar_fun6424__t, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_solar_calendar_fun6424, kk_context());
  _self->date_to_days0_4812 = date_to_days0_4812;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_time_instant__instant kk_std_time_calendar_solar_calendar_fun6424(kk_function_t _fself, kk_std_time_date__date d00, kk_std_time_date__clock c, kk_std_time_calendar__timezone tz, kk_std_time_instant__timescale ts0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun6424__t* _self = kk_function_as(struct kk_std_time_calendar_solar_calendar_fun6424__t*, _fself, _ctx);
  kk_function_t date_to_days0_4812 = _self->date_to_days0_4812; /* (std/time/date/date) -> int */
  kk_drop_match(_self, {kk_function_dup(date_to_days0_4812, _ctx);}, {}, _ctx)
  kk_std_time_timestamp__timestamp t_4958;
  kk_std_core_types__optional _x6425;
  kk_box_t _x6426;
  bool _x6427;
  {
    struct kk_std_time_instant_Timescale* _con6428 = kk_std_time_instant__as_Timescale(ts0, _ctx);
    kk_std_core_types__maybe _x31 = _con6428->mb_seconds_in_day;
    kk_std_core_types__maybe_dup(_x31, _ctx);
    if (kk_std_core_types__is_Nothing(_x31, _ctx)) {
      _x6427 = false; /*bool*/
    }
    else {
      kk_std_core_types__maybe_drop(_x31, _ctx);
      _x6427 = true; /*bool*/
    }
  }
  _x6426 = kk_bool_box(_x6427); /*116*/
  _x6425 = kk_std_core_types__new_Optional(_x6426, _ctx); /*optional<116>*/
  t_4958 = kk_std_time_calendar_dc_timestamp(d00, c, date_to_days0_4812, _x6425, _ctx); /*std/time/timestamp/timestamp*/
  kk_std_time_instant__instant i0;
  kk_std_time_timestamp__timestamp _x6429 = kk_std_time_timestamp__timestamp_dup(t_4958, _ctx); /*std/time/timestamp/timestamp*/
  kk_std_time_instant__timescale _x6430 = kk_std_time_instant__timescale_dup(ts0, _ctx); /*std/time/instant/timescale*/
  i0 = kk_std_time_instant__new_Instant(_x6429, _x6430, _ctx); /*std/time/instant/instant*/
  bool _match_5924;
  kk_string_t _x6431;
  {
    struct kk_std_time_calendar_Timezone* _con6432 = kk_std_time_calendar__as_Timezone(tz, _ctx);
    kk_string_t _x3 = _con6432->name;
    kk_string_dup(_x3, _ctx);
    _x6431 = _x3; /*string*/
  }
  kk_string_t _x6433;
  kk_std_time_calendar__timezone _x6434 = kk_std_time_calendar_tz_utc; /*std/time/calendar/timezone*/
  {
    struct kk_std_time_calendar_Timezone* _con6435 = kk_std_time_calendar__as_Timezone(_x6434, _ctx);
    kk_string_t _x00 = _con6435->name;
    _x6433 = kk_string_dup(_x00, _ctx); /*string*/
  }
  _match_5924 = kk_string_is_eq(_x6431,_x6433,kk_context()); /*bool*/
  if (_match_5924) {
    kk_datatype_ptr_dropn(tz, (KK_I32(3)), _ctx);
    kk_datatype_ptr_dropn(ts0, (KK_I32(10)), _ctx);
    kk_datatype_ptr_dropn(t_4958, (KK_I32(0)), _ctx);
    return i0;
  }
  {
    struct kk_std_time_calendar_Timezone* _con6436 = kk_std_time_calendar__as_Timezone(tz, _ctx);
    kk_string_t _pat020 = _con6436->name;
    kk_function_t _pat116 = _con6436->utc_delta;
    kk_function_t _x100 = _con6436->utc_inverse;
    kk_function_dup(_x100, _ctx);
    kk_std_core_types__maybe _match_5925;
    kk_std_time_instant__instant _x6437 = kk_std_time_instant__instant_dup(i0, _ctx); /*std/time/instant/instant*/
    _match_5925 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x100, (_x100, _x6437, _ctx), _ctx); /*maybe<std/time/instant/instant>*/
    if (kk_std_core_types__is_Just(_match_5925, _ctx)) {
      kk_box_t _box_x5389 = _match_5925._cons.Just.value;
      kk_std_time_instant__instant inv = kk_std_time_instant__instant_unbox(_box_x5389, KK_BORROWED, _ctx);
      if kk_likely(kk_datatype_ptr_is_unique(tz, _ctx)) {
        kk_function_drop(_x100, _ctx);
        kk_function_drop(_pat116, _ctx);
        kk_string_drop(_pat020, _ctx);
        kk_datatype_ptr_free(tz, _ctx);
      }
      else {
        kk_datatype_ptr_decref(tz, _ctx);
      }
      kk_datatype_ptr_dropn(ts0, (KK_I32(10)), _ctx);
      kk_datatype_ptr_dropn(t_4958, (KK_I32(0)), _ctx);
      kk_std_time_instant__instant_drop(i0, _ctx);
      kk_std_time_instant__instant_dup(inv, _ctx);
      kk_std_core_types__maybe_drop(_match_5925, _ctx);
      return inv;
    }
    {
      kk_std_core_types__tuple2_ _this_4787;
      {
        struct kk_std_time_calendar_Timezone* _con6438 = kk_std_time_calendar__as_Timezone(tz, _ctx);
        kk_function_t _x30 = _con6438->utc_delta;
        kk_function_dup(_x30, _ctx);
        _this_4787 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x30, (_x30, i0, _ctx), _ctx); /*(std/time/duration/duration, string)*/
      }
      kk_std_time_timestamp__timestamp t1_4790;
      kk_std_num_ddouble__ddouble _x6439;
      kk_std_num_ddouble__ddouble _x6440;
      {
        struct kk_std_time_timestamp_Timestamp* _con6441 = kk_std_time_timestamp__as_Timestamp(t_4958, _ctx);
        kk_std_num_ddouble__ddouble _x1000 = _con6441->since;
        _x6440 = _x1000; /*std/time/timestamp/timespan*/
      }
      kk_std_num_ddouble__ddouble _x6442;
      double _x6443;
      double _x6444;
      {
        kk_box_t _box_x5390 = _this_4787.fst;
        kk_box_t _box_x5391 = _this_4787.snd;
        kk_std_time_duration__duration _x20 = kk_std_time_duration__duration_unbox(_box_x5390, KK_BORROWED, _ctx);
        {
          kk_std_num_ddouble__ddouble _x50 = _x20.secs;
          {
            double _x400 = _x50.hi;
            _x6444 = _x400; /*float64*/
          }
        }
      }
      _x6443 = (-_x6444); /*float64*/
      double _x6445;
      double _x6446;
      {
        kk_box_t _box_x5392 = _this_4787.fst;
        kk_box_t _box_x5393 = _this_4787.snd;
        kk_std_time_duration__duration _x200 = kk_std_time_duration__duration_unbox(_box_x5392, KK_BORROWED, _ctx);
        {
          kk_std_num_ddouble__ddouble _x500 = _x200.secs;
          {
            double _x000 = _x500.lo;
            _x6446 = _x000; /*float64*/
          }
        }
      }
      _x6445 = (-_x6446); /*float64*/
      _x6442 = kk_std_num_ddouble__new_Ddouble(_x6443, _x6445, _ctx); /*std/num/ddouble/ddouble*/
      _x6439 = kk_std_num_ddouble__lp__plus__rp_(_x6440, _x6442, _ctx); /*std/num/ddouble/ddouble*/
      int32_t _x6447;
      {
        struct kk_std_time_timestamp_Timestamp* _con6448 = kk_std_time_timestamp__as_Timestamp(t_4958, _ctx);
        kk_std_num_ddouble__ddouble _pat0200 = _con6448->since;
        int32_t _x2000 = _con6448->leap32;
        _x6447 = _x2000; /*int32*/
      }
      t1_4790 = kk_std_time_timestamp__new_Timestamp(kk_reuse_null, 0, _x6439, _x6447, _ctx); /*std/time/timestamp/timestamp*/
      kk_std_time_instant__instant tzi1;
      kk_std_time_instant__timescale _x6449 = kk_std_time_instant__timescale_dup(ts0, _ctx); /*std/time/instant/timescale*/
      tzi1 = kk_std_time_instant__new_Instant(t1_4790, _x6449, _ctx); /*std/time/instant/instant*/
      kk_std_core_types__tuple2_ _this0_4794;
      {
        struct kk_std_time_calendar_Timezone* _con6450 = kk_std_time_calendar__as_Timezone(tz, _ctx);
        kk_string_t _pat080 = _con6450->name;
        kk_function_t _x70 = _con6450->utc_delta;
        kk_function_t _pat170 = _con6450->utc_inverse;
        if kk_likely(kk_datatype_ptr_is_unique(tz, _ctx)) {
          kk_function_drop(_pat170, _ctx);
          kk_string_drop(_pat080, _ctx);
          kk_datatype_ptr_free(tz, _ctx);
        }
        else {
          kk_function_dup(_x70, _ctx);
          kk_datatype_ptr_decref(tz, _ctx);
        }
        kk_std_time_instant__instant _x6451 = kk_std_time_instant__instant_dup(tzi1, _ctx); /*std/time/instant/instant*/
        _this0_4794 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x70, (_x70, _x6451, _ctx), _ctx); /*(std/time/duration/duration, string)*/
      }
      bool _match_5926;
      kk_std_time_duration__duration _x6452;
      {
        kk_box_t _box_x5394 = _this_4787.fst;
        kk_box_t _box_x5395 = _this_4787.snd;
        kk_std_time_duration__duration _x201 = kk_std_time_duration__duration_unbox(_box_x5394, KK_BORROWED, _ctx);
        kk_std_core_types__tuple2__drop(_this_4787, _ctx);
        _x6452 = _x201; /*std/time/duration/duration*/
      }
      kk_std_time_duration__duration _x6453;
      {
        kk_box_t _box_x5396 = _this0_4794.fst;
        kk_box_t _box_x5397 = _this0_4794.snd;
        kk_std_time_duration__duration _x60 = kk_std_time_duration__duration_unbox(_box_x5396, KK_BORROWED, _ctx);
        _x6453 = _x60; /*std/time/duration/duration*/
      }
      _match_5926 = kk_std_time_duration__lp__eq__eq__rp_(_x6452, _x6453, _ctx); /*bool*/
      if (_match_5926) {
        kk_datatype_ptr_dropn(ts0, (KK_I32(10)), _ctx);
        kk_datatype_ptr_dropn(t_4958, (KK_I32(0)), _ctx);
        kk_std_core_types__tuple2__drop(_this0_4794, _ctx);
        return tzi1;
      }
      {
        kk_std_time_instant__instant_drop(tzi1, _ctx);
        kk_std_time_timestamp__timestamp t3_4797;
        kk_std_num_ddouble__ddouble _x6454;
        kk_std_num_ddouble__ddouble _x6455;
        {
          struct kk_std_time_timestamp_Timestamp* _con6456 = kk_std_time_timestamp__as_Timestamp(t_4958, _ctx);
          kk_std_num_ddouble__ddouble _x11 = _con6456->since;
          _x6455 = _x11; /*std/time/timestamp/timespan*/
        }
        kk_std_num_ddouble__ddouble _x6457;
        double _x6458;
        double _x6459;
        {
          kk_box_t _box_x5398 = _this0_4794.fst;
          kk_box_t _box_x5399 = _this0_4794.snd;
          kk_std_time_duration__duration _x600 = kk_std_time_duration__duration_unbox(_box_x5398, KK_BORROWED, _ctx);
          {
            kk_std_num_ddouble__ddouble _x90 = _x600.secs;
            {
              double _x80 = _x90.hi;
              _x6459 = _x80; /*float64*/
            }
          }
        }
        _x6458 = (-_x6459); /*float64*/
        double _x6460;
        double _x6461;
        {
          kk_box_t _box_x5400 = _this0_4794.fst;
          kk_box_t _box_x5401 = _this0_4794.snd;
          kk_std_time_duration__duration _x601 = kk_std_time_duration__duration_unbox(_box_x5400, KK_BORROWED, _ctx);
          kk_std_core_types__tuple2__drop(_this0_4794, _ctx);
          {
            kk_std_num_ddouble__ddouble _x900 = _x601.secs;
            {
              double _x01 = _x900.lo;
              _x6461 = _x01; /*float64*/
            }
          }
        }
        _x6460 = (-_x6461); /*float64*/
        _x6457 = kk_std_num_ddouble__new_Ddouble(_x6458, _x6460, _ctx); /*std/num/ddouble/ddouble*/
        _x6454 = kk_std_num_ddouble__lp__plus__rp_(_x6455, _x6457, _ctx); /*std/num/ddouble/ddouble*/
        int32_t _x6462;
        {
          struct kk_std_time_timestamp_Timestamp* _con6463 = kk_std_time_timestamp__as_Timestamp(t_4958, _ctx);
          kk_std_num_ddouble__ddouble _pat021 = _con6463->since;
          int32_t _x21 = _con6463->leap32;
          if kk_likely(kk_datatype_ptr_is_unique(t_4958, _ctx)) {
            kk_datatype_ptr_free(t_4958, _ctx);
          }
          else {
            kk_datatype_ptr_decref(t_4958, _ctx);
          }
          _x6462 = _x21; /*int32*/
        }
        t3_4797 = kk_std_time_timestamp__new_Timestamp(kk_reuse_null, 0, _x6454, _x6462, _ctx); /*std/time/timestamp/timestamp*/
        return kk_std_time_instant__new_Instant(t3_4797, ts0, _ctx);
      }
    }
  }
}

kk_std_time_calendar__calendar kk_std_time_calendar_solar_calendar(kk_string_t name0, kk_string_t long_name0, kk_function_t days_before_year, kk_function_t days_to_yeardoy, kk_function_t days_before_month, kk_function_t days_to_month, kk_std_core_types__optional epoch_shift, kk_std_core_types__optional has_year_zero, kk_std_core_types__optional month_prefix0, kk_std_core_types__optional show_era0, kk_context_t* _ctx) { /* (name : string, long-name : string, days-before-year : (year : int) -> int, days-to-yeardoy : (days : int) -> (int, int), days-before-month : (year : int, month : int) -> int, days-to-month : (year : int, doy : int) -> int, epoch-shift : optional<int>, has-year-zero : optional<bool>, month-prefix : optional<string>, show-era : optional<(std/time/date/date) -> string>) -> calendar */ 
  kk_function_t days_to_date0_4811;
  kk_function_dup(days_before_month, _ctx);
  kk_std_core_types__optional_dup(epoch_shift, _ctx);
  kk_std_core_types__optional_dup(has_year_zero, _ctx);
  days_to_date0_4811 = kk_std_time_calendar_new_solar_calendar_fun6389(days_before_month, days_to_month, days_to_yeardoy, epoch_shift, has_year_zero, _ctx); /*(days0 : int) -> std/time/date/date*/
  kk_function_t date_to_days0_4812 = kk_std_time_calendar_new_solar_calendar_fun6395(days_before_month, days_before_year, epoch_shift, has_year_zero, _ctx); /*(d01 : std/time/date/date) -> int*/;
  kk_string_t _x6401;
  if (kk_std_core_types__is_Optional(month_prefix0, _ctx)) {
    kk_box_t _box_x5375 = month_prefix0._cons.Optional.value;
    kk_string_t _month_prefix_3699 = kk_string_unbox(_box_x5375);
    kk_string_dup(_month_prefix_3699, _ctx);
    kk_std_core_types__optional_drop(month_prefix0, _ctx);
    _x6401 = _month_prefix_3699; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(month_prefix0, _ctx);
    _x6401 = kk_string_empty(); /*string*/
  }
  kk_function_t _x6403;
  if (kk_std_core_types__is_Optional(show_era0, _ctx)) {
    kk_box_t _fun_unbox_x5379 = show_era0._cons.Optional.value;
    kk_box_dup(_fun_unbox_x5379, _ctx);
    kk_std_core_types__optional_drop(show_era0, _ctx);
    _x6403 = kk_std_time_calendar_new_solar_calendar_fun6404(_fun_unbox_x5379, _ctx); /*(std/time/date/date) -> string*/
  }
  else {
    kk_std_core_types__optional_drop(show_era0, _ctx);
    _x6403 = kk_std_time_calendar_new_solar_calendar_fun6407(_ctx); /*(std/time/date/date) -> string*/
  }
  kk_function_t _x6409;
  kk_function_dup(days_to_date0_4811, _ctx);
  _x6409 = kk_std_time_calendar_new_solar_calendar_fun6410(days_to_date0_4811, _ctx); /*(i : std/time/instant/instant, tzdelta : std/time/duration/duration) -> (6, 7)*/
  kk_function_t _x6423;
  kk_function_dup(date_to_days0_4812, _ctx);
  _x6423 = kk_std_time_calendar_new_solar_calendar_fun6424(date_to_days0_4812, _ctx); /*(d00 : std/time/date/date, c : std/time/date/clock, tz : std/time/calendar/timezone, ts0 : std/time/instant/timescale) -> std/time/instant/instant*/
  return kk_std_time_calendar__new_Calendar(kk_reuse_null, 0, name0, long_name0, _x6401, _x6403, _x6409, _x6423, days_to_date0_4811, date_to_days0_4812, _ctx);
}
 
// An abstraction over solar calendars. Almost all calendars can be defined
// in terms of this function (and `solar-calendar`). Given functions to
// convert days to and from a year, and day-of-the-year to and from month
// and month day, this constructs a full calendar. In contrast to `solar-calendar`
// this function just needs an estimation function from days to a year -- for most
// calendars this is much more convenient. The estimation function returns a
// conservative (i.e. lowest) estimate for the year and a `maybe<int>` that
// is the `safe-day`: if the day of the year is lower or equal to that the estimate
// is surely correct. Otherwise the wrapper will calculate the start day for the next
// year until the correct year is found.
// Also needs an `epoch-shift` that gives the number of days from the
// calendar epoch to the date ``2000-01-01``. By default this is `730119`
// which corresponds to a ``0001-01-01`` ISO calendar epoch. The
// `has-year-zero` (=`True`) parameter is used when the calendar has a
// year zero. The `month-prefix` (=`""`) is used for displaying numeric
// months (and is `"W"` for `cal-iso-week` for example). The `show-era` function
// can be given to display an era for a given date. Finally the `ts` (=`ts-utc`)
// gives the time scale for the calendar.


// lift anonymous function
struct kk_std_time_calendar_solar_ecalendar_fun6465__t {
  struct kk_function_s _base;
  kk_function_t days_before_year;
  kk_function_t estimate_year;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_solar_ecalendar_fun6465(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_solar_ecalendar_fun6465(kk_function_t days_before_year, kk_function_t estimate_year, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_ecalendar_fun6465__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_solar_ecalendar_fun6465__t, 3, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_solar_ecalendar_fun6465, kk_context());
  _self->days_before_year = days_before_year;
  _self->estimate_year = estimate_year;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_solar_ecalendar_fun6465(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_ecalendar_fun6465__t* _self = kk_function_as(struct kk_std_time_calendar_solar_ecalendar_fun6465__t*, _fself, _ctx);
  kk_function_t days_before_year = _self->days_before_year; /* (year : int) -> int */
  kk_function_t estimate_year = _self->estimate_year; /* (days : int) -> (int, maybe<int>) */
  kk_drop_match(_self, {kk_function_dup(days_before_year, _ctx);kk_function_dup(estimate_year, _ctx);}, {}, _ctx)
  kk_std_core_types__tuple2_ _match_5919;
  kk_integer_t _x6466 = kk_integer_dup(days, _ctx); /*int*/
  _match_5919 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_integer_t, kk_context_t*), estimate_year, (estimate_year, _x6466, _ctx), _ctx); /*(int, maybe<int>)*/
  {
    kk_box_t _box_x5404 = _match_5919.fst;
    kk_box_t _box_x5405 = _match_5919.snd;
    kk_integer_t approx = kk_integer_unbox(_box_x5404, _ctx);
    kk_std_core_types__maybe safe = kk_std_core_types__maybe_unbox(_box_x5405, KK_BORROWED, _ctx);
    kk_integer_dup(approx, _ctx);
    kk_std_core_types__maybe_dup(safe, _ctx);
    kk_std_core_types__tuple2__drop(_match_5919, _ctx);
    kk_integer_t y_4847;
    kk_function_t _x6468 = kk_function_dup(days_before_year, _ctx); /*(year : int) -> int*/
    kk_integer_t _x6467 = kk_integer_dup(approx, _ctx); /*int*/
    y_4847 = kk_function_call(kk_integer_t, (kk_function_t, kk_integer_t, kk_context_t*), _x6468, (_x6468, _x6467, _ctx), _ctx); /*int*/
    kk_integer_t doy1;
    kk_integer_t _x6469 = kk_integer_dup(days, _ctx); /*int*/
    doy1 = kk_integer_sub(_x6469,y_4847,kk_context()); /*int*/
    bool _match_5920;
    kk_integer_t _brw_5922;
    if (kk_std_core_types__is_Nothing(safe, _ctx)) {
      _brw_5922 = kk_integer_from_small(0); /*int*/
    }
    else {
      kk_box_t _box_x5406 = safe._cons.Just.value;
      kk_integer_t x0 = kk_integer_unbox(_box_x5406, _ctx);
      kk_integer_dup(x0, _ctx);
      kk_std_core_types__maybe_drop(safe, _ctx);
      _brw_5922 = x0; /*int*/
    }
    bool _brw_5923 = kk_integer_lte_borrow(doy1,_brw_5922,kk_context()); /*bool*/;
    kk_integer_drop(_brw_5922, _ctx);
    _match_5920 = _brw_5923; /*bool*/
    if (_match_5920) {
      kk_function_drop(days_before_year, _ctx);
      kk_integer_drop(days, _ctx);
      return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(approx, _ctx), kk_integer_box(doy1, _ctx), _ctx);
    }
    {
      kk_integer_t y0_4851;
      kk_integer_t _x6470;
      kk_integer_t _x6471 = kk_integer_dup(approx, _ctx); /*int*/
      _x6470 = kk_integer_add_small_const(_x6471, 1, _ctx); /*int*/
      y0_4851 = kk_function_call(kk_integer_t, (kk_function_t, kk_integer_t, kk_context_t*), days_before_year, (days_before_year, _x6470, _ctx), _ctx); /*int*/
      kk_integer_t doy2 = kk_integer_sub(days,y0_4851,kk_context()); /*int*/;
      bool _match_5921 = kk_integer_lt_borrow(doy2,(kk_integer_from_small(0)),kk_context()); /*bool*/;
      if (_match_5921) {
        kk_integer_drop(doy2, _ctx);
        return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(approx, _ctx), kk_integer_box(doy1, _ctx), _ctx);
      }
      {
        kk_integer_drop(doy1, _ctx);
        kk_integer_t _b_5434_5411 = kk_integer_add_small_const(approx, 1, _ctx); /*int*/;
        return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5434_5411, _ctx), kk_integer_box(doy2, _ctx), _ctx);
      }
    }
  }
}


// lift anonymous function
struct kk_std_time_calendar_solar_ecalendar_fun6483__t {
  struct kk_function_s _base;
  kk_std_core_types__optional show_era0;
};
static kk_box_t kk_std_time_calendar_solar_ecalendar_fun6483(kk_function_t _fself, kk_box_t _b_5429, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_solar_ecalendar_fun6483(kk_std_core_types__optional show_era0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_ecalendar_fun6483__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_solar_ecalendar_fun6483__t, 3, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_solar_ecalendar_fun6483, kk_context());
  _self->show_era0 = show_era0;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_box_t kk_std_time_calendar_solar_ecalendar_fun6483(kk_function_t _fself, kk_box_t _b_5429, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_ecalendar_fun6483__t* _self = kk_function_as(struct kk_std_time_calendar_solar_ecalendar_fun6483__t*, _fself, _ctx);
  kk_std_core_types__optional show_era0 = _self->show_era0; /* optional<(std/time/date/date) -> string> */
  kk_drop_match(_self, {kk_std_core_types__optional_dup(show_era0, _ctx);}, {}, _ctx)
  kk_string_t _x6484;
  kk_std_time_date__date _norm_5440 = kk_std_time_date__date_unbox(_b_5429, KK_OWNED, _ctx); /*std/time/date/date*/;
  if (kk_std_core_types__is_Optional(show_era0, _ctx)) {
    kk_box_t _fun_unbox_x5422 = show_era0._cons.Optional.value;
    kk_box_dup(_fun_unbox_x5422, _ctx);
    kk_std_core_types__optional_drop(show_era0, _ctx);
    kk_std_time_date__date _b_5441_5425 = _norm_5440; /*std/time/date/date*/;
    kk_box_t _x6485;
    kk_function_t _x6486 = kk_function_unbox(_fun_unbox_x5422, _ctx); /*(5423) -> 5424*/
    _x6485 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6486, (_x6486, kk_std_time_date__date_box(_b_5441_5425, _ctx), _ctx), _ctx); /*5424*/
    _x6484 = kk_string_unbox(_x6485); /*string*/
  }
  else {
    kk_std_core_types__optional_drop(show_era0, _ctx);
    kk_std_time_date__date d_5442 = _norm_5440; /*std/time/date/date*/;
    kk_std_time_date__date_drop(d_5442, _ctx);
    _x6484 = kk_string_empty(); /*string*/
  }
  return kk_string_box(_x6484);
}

kk_std_time_calendar__calendar kk_std_time_calendar_solar_ecalendar(kk_string_t name0, kk_string_t long_name0, kk_function_t days_before_year, kk_function_t estimate_year, kk_function_t days_before_month, kk_function_t days_to_month, kk_std_core_types__optional epoch_shift, kk_std_core_types__optional has_year_zero, kk_std_core_types__optional month_prefix0, kk_std_core_types__optional show_era0, kk_context_t* _ctx) { /* (name : string, long-name : string, days-before-year : (year : int) -> int, estimate-year : (days : int) -> (int, maybe<int>), days-before-month : (year : int, month : int) -> int, days-to-month : (year : int, doy : int) -> int, epoch-shift : optional<int>, has-year-zero : optional<bool>, month-prefix : optional<string>, show-era : optional<(std/time/date/date) -> string>) -> calendar */ 
  kk_function_t _x6464 = kk_function_dup(days_before_year, _ctx); /*(year : int) -> int*/
  kk_std_core_types__optional _x6472;
  kk_box_t _x6473;
  kk_integer_t _x6474;
  if (kk_std_core_types__is_Optional(epoch_shift, _ctx)) {
    kk_box_t _box_x5413 = epoch_shift._cons.Optional.value;
    kk_integer_t _epoch_shift_2777 = kk_integer_unbox(_box_x5413, _ctx);
    kk_integer_dup(_epoch_shift_2777, _ctx);
    kk_std_core_types__optional_drop(epoch_shift, _ctx);
    _x6474 = _epoch_shift_2777; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(epoch_shift, _ctx);
    _x6474 = kk_integer_from_int(730119, _ctx); /*int*/
  }
  _x6473 = kk_integer_box(_x6474, _ctx); /*116*/
  _x6472 = kk_std_core_types__new_Optional(_x6473, _ctx); /*optional<116>*/
  kk_std_core_types__optional _x6475;
  kk_box_t _x6476;
  bool _x6477;
  if (kk_std_core_types__is_Optional(has_year_zero, _ctx)) {
    kk_box_t _box_x5415 = has_year_zero._cons.Optional.value;
    bool _has_year_zero_2781 = kk_bool_unbox(_box_x5415);
    kk_std_core_types__optional_drop(has_year_zero, _ctx);
    _x6477 = _has_year_zero_2781; /*bool*/
  }
  else {
    kk_std_core_types__optional_drop(has_year_zero, _ctx);
    _x6477 = true; /*bool*/
  }
  _x6476 = kk_bool_box(_x6477); /*116*/
  _x6475 = kk_std_core_types__new_Optional(_x6476, _ctx); /*optional<116>*/
  kk_std_core_types__optional _x6478;
  kk_box_t _x6479;
  kk_string_t _x6480;
  if (kk_std_core_types__is_Optional(month_prefix0, _ctx)) {
    kk_box_t _box_x5417 = month_prefix0._cons.Optional.value;
    kk_string_t _month_prefix_2785 = kk_string_unbox(_box_x5417);
    kk_string_dup(_month_prefix_2785, _ctx);
    kk_std_core_types__optional_drop(month_prefix0, _ctx);
    _x6480 = _month_prefix_2785; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(month_prefix0, _ctx);
    _x6480 = kk_string_empty(); /*string*/
  }
  _x6479 = kk_string_box(_x6480); /*116*/
  _x6478 = kk_std_core_types__new_Optional(_x6479, _ctx); /*optional<116>*/
  kk_std_core_types__optional _x6482 = kk_std_core_types__new_Optional(kk_function_box(kk_std_time_calendar_new_solar_ecalendar_fun6483(show_era0, _ctx), _ctx), _ctx); /*optional<116>*/
  return kk_std_time_calendar_solar_calendar(name0, long_name0, _x6464, kk_std_time_calendar_new_solar_ecalendar_fun6465(days_before_year, estimate_year, _ctx), days_before_month, days_to_month, _x6472, _x6475, _x6478, _x6482, _ctx);
}
 
// Create a standard ISO calendar using a particular time scale
// and calendar `name` (=`ts.name`).


// lift anonymous function
struct kk_std_time_calendar_iso_calendar_fun6491__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_iso_calendar_fun6491(kk_function_t _fself, kk_integer_t _x16490, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_iso_calendar_fun6491(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_iso_calendar_fun6491, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_integer_t kk_std_time_calendar_iso_calendar_fun6491(kk_function_t _fself, kk_integer_t _x16490, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_days_before_year(_x16490, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_iso_calendar_fun6492__t {
  struct kk_function_s _base;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_iso_calendar_fun6492(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_iso_calendar_fun6492(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_iso_calendar_fun6492, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_iso_calendar_fun6492(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx) {
  kk_unused(_fself);
  kk_std_core_types__tuple2_ _match_5914;
  kk_integer_t _x6493 = kk_integer_dup(days, _ctx); /*int*/
  _match_5914 = kk_std_time_calendar_iso_estimate_year(_x6493, _ctx); /*(int, maybe<int>)*/
  {
    kk_box_t _box_x5444 = _match_5914.fst;
    kk_box_t _box_x5445 = _match_5914.snd;
    kk_integer_t approx = kk_integer_unbox(_box_x5444, _ctx);
    kk_std_core_types__maybe safe = kk_std_core_types__maybe_unbox(_box_x5445, KK_BORROWED, _ctx);
    kk_integer_dup(approx, _ctx);
    kk_std_core_types__maybe_dup(safe, _ctx);
    kk_std_core_types__tuple2__drop(_match_5914, _ctx);
    kk_integer_t y_4847;
    kk_integer_t _x6494 = kk_integer_dup(approx, _ctx); /*int*/
    y_4847 = kk_std_time_calendar_iso_days_before_year(_x6494, _ctx); /*int*/
    kk_integer_t doy1;
    kk_integer_t _x6495 = kk_integer_dup(days, _ctx); /*int*/
    doy1 = kk_integer_sub(_x6495,y_4847,kk_context()); /*int*/
    bool _match_5915;
    kk_integer_t _brw_5917;
    if (kk_std_core_types__is_Nothing(safe, _ctx)) {
      _brw_5917 = kk_integer_from_small(0); /*int*/
    }
    else {
      kk_box_t _box_x5446 = safe._cons.Just.value;
      kk_integer_t x0 = kk_integer_unbox(_box_x5446, _ctx);
      kk_integer_dup(x0, _ctx);
      kk_std_core_types__maybe_drop(safe, _ctx);
      _brw_5917 = x0; /*int*/
    }
    bool _brw_5918 = kk_integer_lte_borrow(doy1,_brw_5917,kk_context()); /*bool*/;
    kk_integer_drop(_brw_5917, _ctx);
    _match_5915 = _brw_5918; /*bool*/
    if (_match_5915) {
      kk_integer_drop(days, _ctx);
      return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(approx, _ctx), kk_integer_box(doy1, _ctx), _ctx);
    }
    {
      kk_integer_t year_5145;
      kk_integer_t _x6496 = kk_integer_dup(approx, _ctx); /*int*/
      year_5145 = kk_integer_add_small_const(_x6496, 1, _ctx); /*int*/
      kk_integer_t y = kk_integer_add_small_const(year_5145, -1, _ctx); /*int*/;
      kk_integer_t x0_4671;
      kk_integer_t _x6497 = kk_integer_dup(y, _ctx); /*int*/
      x0_4671 = kk_integer_div(_x6497,(kk_integer_from_small(4)),kk_context()); /*int*/
      kk_integer_t y1_4672;
      kk_integer_t _x6498 = kk_integer_dup(y, _ctx); /*int*/
      y1_4672 = kk_integer_div(_x6498,(kk_integer_from_small(100)),kk_context()); /*int*/
      kk_integer_t x_4669 = kk_integer_sub(x0_4671,y1_4672,kk_context()); /*int*/;
      kk_integer_t y0_4670;
      kk_integer_t _x6499 = kk_integer_dup(y, _ctx); /*int*/
      y0_4670 = kk_integer_div(_x6499,(kk_integer_from_small(400)),kk_context()); /*int*/
      kk_integer_t leapdays = kk_integer_add(x_4669,y0_4670,kk_context()); /*int*/;
      kk_integer_t x1_4673 = kk_integer_mul((kk_integer_from_small(365)),y,kk_context()); /*int*/;
      kk_integer_t y0_4851 = kk_integer_add(x1_4673,leapdays,kk_context()); /*int*/;
      kk_integer_t doy2 = kk_integer_sub(days,y0_4851,kk_context()); /*int*/;
      bool _match_5916 = kk_integer_lt_borrow(doy2,(kk_integer_from_small(0)),kk_context()); /*bool*/;
      if (_match_5916) {
        kk_integer_drop(doy2, _ctx);
        return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(approx, _ctx), kk_integer_box(doy1, _ctx), _ctx);
      }
      {
        kk_integer_drop(doy1, _ctx);
        kk_integer_t _b_5474_5451 = kk_integer_add_small_const(approx, 1, _ctx); /*int*/;
        return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5474_5451, _ctx), kk_integer_box(doy2, _ctx), _ctx);
      }
    }
  }
}


// lift anonymous function
struct kk_std_time_calendar_iso_calendar_fun6502__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_iso_calendar_fun6502(kk_function_t _fself, kk_integer_t _x16500, kk_integer_t _x26501, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_iso_calendar_fun6502(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_iso_calendar_fun6502, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_integer_t kk_std_time_calendar_iso_calendar_fun6502(kk_function_t _fself, kk_integer_t _x16500, kk_integer_t _x26501, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_days_before_month(_x16500, _x26501, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_iso_calendar_fun6505__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_iso_calendar_fun6505(kk_function_t _fself, kk_integer_t _x16503, kk_integer_t _x26504, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_iso_calendar_fun6505(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_iso_calendar_fun6505, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_integer_t kk_std_time_calendar_iso_calendar_fun6505(kk_function_t _fself, kk_integer_t _x16503, kk_integer_t _x26504, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_doy_to_month(_x16503, _x26504, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_iso_calendar_fun6517__t {
  struct kk_function_s _base;
};
static kk_box_t kk_std_time_calendar_iso_calendar_fun6517(kk_function_t _fself, kk_box_t _b_5469, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_iso_calendar_fun6517(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_iso_calendar_fun6517, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_box_t kk_std_time_calendar_iso_calendar_fun6517(kk_function_t _fself, kk_box_t _b_5469, kk_context_t* _ctx) {
  kk_unused(_fself);
  kk_string_t _x6518;
  kk_std_time_date__date _norm_5480 = kk_std_time_date__date_unbox(_b_5469, KK_OWNED, _ctx); /*std/time/date/date*/;
  kk_std_core_types__optional _match_5910 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5910, _ctx)) {
    kk_box_t _fun_unbox_x5462 = _match_5910._cons.Optional.value;
    kk_box_dup(_fun_unbox_x5462, _ctx);
    kk_std_core_types__optional_drop(_match_5910, _ctx);
    kk_std_time_date__date _b_5481_5465 = _norm_5480; /*std/time/date/date*/;
    kk_box_t _x6519;
    kk_function_t _x6520 = kk_function_unbox(_fun_unbox_x5462, _ctx); /*(5463) -> 5464*/
    _x6519 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6520, (_x6520, kk_std_time_date__date_box(_b_5481_5465, _ctx), _ctx), _ctx); /*5464*/
    _x6518 = kk_string_unbox(_x6519); /*string*/
  }
  else {
    kk_std_core_types__optional_drop(_match_5910, _ctx);
    kk_std_time_date__date d_5482 = _norm_5480; /*std/time/date/date*/;
    kk_std_time_date__date_drop(d_5482, _ctx);
    _x6518 = kk_string_empty(); /*string*/
  }
  return kk_string_box(_x6518);
}

kk_std_time_calendar__calendar kk_std_time_calendar_iso_calendar(kk_std_time_instant__timescale ___wildcard__422__23, kk_string_t name0, kk_std_core_types__optional long_name0, kk_context_t* _ctx) { /* (std/time/instant/timescale, name : string, long-name : optional<string>) -> calendar */ 
  kk_datatype_ptr_dropn(___wildcard__422__23, (KK_I32(10)), _ctx);
  kk_string_t _x6488 = kk_string_dup(name0, _ctx); /*string*/
  kk_string_t _x6489;
  if (kk_std_core_types__is_Optional(long_name0, _ctx)) {
    kk_box_t _box_x5443 = long_name0._cons.Optional.value;
    kk_string_t _long_name_2969 = kk_string_unbox(_box_x5443);
    kk_string_drop(name0, _ctx);
    kk_string_dup(_long_name_2969, _ctx);
    kk_std_core_types__optional_drop(long_name0, _ctx);
    _x6489 = _long_name_2969; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(long_name0, _ctx);
    _x6489 = name0; /*string*/
  }
  kk_std_core_types__optional _x6506;
  kk_box_t _x6507;
  kk_integer_t _x6508;
  kk_std_core_types__optional _match_5913 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5913, _ctx)) {
    kk_box_t _box_x5453 = _match_5913._cons.Optional.value;
    kk_integer_t _epoch_shift_2777 = kk_integer_unbox(_box_x5453, _ctx);
    kk_integer_dup(_epoch_shift_2777, _ctx);
    kk_std_core_types__optional_drop(_match_5913, _ctx);
    _x6508 = _epoch_shift_2777; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(_match_5913, _ctx);
    _x6508 = kk_integer_from_int(730119, _ctx); /*int*/
  }
  _x6507 = kk_integer_box(_x6508, _ctx); /*116*/
  _x6506 = kk_std_core_types__new_Optional(_x6507, _ctx); /*optional<116>*/
  kk_std_core_types__optional _x6509;
  kk_box_t _x6510;
  bool _x6511;
  kk_std_core_types__optional _match_5912 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5912, _ctx)) {
    kk_box_t _box_x5455 = _match_5912._cons.Optional.value;
    bool _has_year_zero_2781 = kk_bool_unbox(_box_x5455);
    kk_std_core_types__optional_drop(_match_5912, _ctx);
    _x6511 = _has_year_zero_2781; /*bool*/
  }
  else {
    kk_std_core_types__optional_drop(_match_5912, _ctx);
    _x6511 = true; /*bool*/
  }
  _x6510 = kk_bool_box(_x6511); /*116*/
  _x6509 = kk_std_core_types__new_Optional(_x6510, _ctx); /*optional<116>*/
  kk_std_core_types__optional _x6512;
  kk_box_t _x6513;
  kk_string_t _x6514;
  kk_std_core_types__optional _match_5911 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5911, _ctx)) {
    kk_box_t _box_x5457 = _match_5911._cons.Optional.value;
    kk_string_t _month_prefix_2785 = kk_string_unbox(_box_x5457);
    kk_string_dup(_month_prefix_2785, _ctx);
    kk_std_core_types__optional_drop(_match_5911, _ctx);
    _x6514 = _month_prefix_2785; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(_match_5911, _ctx);
    _x6514 = kk_string_empty(); /*string*/
  }
  _x6513 = kk_string_box(_x6514); /*116*/
  _x6512 = kk_std_core_types__new_Optional(_x6513, _ctx); /*optional<116>*/
  kk_std_core_types__optional _x6516 = kk_std_core_types__new_Optional(kk_function_box(kk_std_time_calendar_new_iso_calendar_fun6517(_ctx), _ctx), _ctx); /*optional<116>*/
  return kk_std_time_calendar_solar_calendar(_x6488, _x6489, kk_std_time_calendar_new_iso_calendar_fun6491(_ctx), kk_std_time_calendar_new_iso_calendar_fun6492(_ctx), kk_std_time_calendar_new_iso_calendar_fun6502(_ctx), kk_std_time_calendar_new_iso_calendar_fun6505(_ctx), _x6506, _x6509, _x6512, _x6516, _ctx);
}
 
// The standard [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) calendar
// using TI time (`ts-ti`). This is a proleptic Gregorian
// calendar except that it uses the year 0 for 1 BCE, -1 for 2 BCE etc.
//
// This is the default calendar used in the library as it guarantees deterministic
// date calculations while still taking historical leap seconds into account.


// lift anonymous function
struct kk_std_time_calendar_cal_iso_fun6527__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_iso_fun6527(kk_function_t _fself, kk_integer_t _x16526, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_fun6527(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_iso_fun6527, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_integer_t kk_std_time_calendar_cal_iso_fun6527(kk_function_t _fself, kk_integer_t _x16526, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_days_before_year(_x16526, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_iso_fun6528__t {
  struct kk_function_s _base;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_cal_iso_fun6528(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_fun6528(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_iso_fun6528, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_cal_iso_fun6528(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx) {
  kk_unused(_fself);
  kk_std_core_types__tuple2_ _match_5905;
  kk_integer_t _x6529 = kk_integer_dup(days, _ctx); /*int*/
  _match_5905 = kk_std_time_calendar_iso_estimate_year(_x6529, _ctx); /*(int, maybe<int>)*/
  {
    kk_box_t _box_x5483 = _match_5905.fst;
    kk_box_t _box_x5484 = _match_5905.snd;
    kk_integer_t approx = kk_integer_unbox(_box_x5483, _ctx);
    kk_std_core_types__maybe safe = kk_std_core_types__maybe_unbox(_box_x5484, KK_BORROWED, _ctx);
    kk_integer_dup(approx, _ctx);
    kk_std_core_types__maybe_dup(safe, _ctx);
    kk_std_core_types__tuple2__drop(_match_5905, _ctx);
    kk_integer_t y_4847;
    kk_integer_t _x6530 = kk_integer_dup(approx, _ctx); /*int*/
    y_4847 = kk_std_time_calendar_iso_days_before_year(_x6530, _ctx); /*int*/
    kk_integer_t doy1;
    kk_integer_t _x6531 = kk_integer_dup(days, _ctx); /*int*/
    doy1 = kk_integer_sub(_x6531,y_4847,kk_context()); /*int*/
    bool _match_5906;
    kk_integer_t _brw_5908;
    if (kk_std_core_types__is_Nothing(safe, _ctx)) {
      _brw_5908 = kk_integer_from_small(0); /*int*/
    }
    else {
      kk_box_t _box_x5485 = safe._cons.Just.value;
      kk_integer_t x0 = kk_integer_unbox(_box_x5485, _ctx);
      kk_integer_dup(x0, _ctx);
      kk_std_core_types__maybe_drop(safe, _ctx);
      _brw_5908 = x0; /*int*/
    }
    bool _brw_5909 = kk_integer_lte_borrow(doy1,_brw_5908,kk_context()); /*bool*/;
    kk_integer_drop(_brw_5908, _ctx);
    _match_5906 = _brw_5909; /*bool*/
    if (_match_5906) {
      kk_integer_drop(days, _ctx);
      return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(approx, _ctx), kk_integer_box(doy1, _ctx), _ctx);
    }
    {
      kk_integer_t year_5146;
      kk_integer_t _x6532 = kk_integer_dup(approx, _ctx); /*int*/
      year_5146 = kk_integer_add_small_const(_x6532, 1, _ctx); /*int*/
      kk_integer_t y = kk_integer_add_small_const(year_5146, -1, _ctx); /*int*/;
      kk_integer_t x0_4671;
      kk_integer_t _x6533 = kk_integer_dup(y, _ctx); /*int*/
      x0_4671 = kk_integer_div(_x6533,(kk_integer_from_small(4)),kk_context()); /*int*/
      kk_integer_t y1_4672;
      kk_integer_t _x6534 = kk_integer_dup(y, _ctx); /*int*/
      y1_4672 = kk_integer_div(_x6534,(kk_integer_from_small(100)),kk_context()); /*int*/
      kk_integer_t x_4669 = kk_integer_sub(x0_4671,y1_4672,kk_context()); /*int*/;
      kk_integer_t y0_4670;
      kk_integer_t _x6535 = kk_integer_dup(y, _ctx); /*int*/
      y0_4670 = kk_integer_div(_x6535,(kk_integer_from_small(400)),kk_context()); /*int*/
      kk_integer_t leapdays = kk_integer_add(x_4669,y0_4670,kk_context()); /*int*/;
      kk_integer_t x1_4673 = kk_integer_mul((kk_integer_from_small(365)),y,kk_context()); /*int*/;
      kk_integer_t y0_4851 = kk_integer_add(x1_4673,leapdays,kk_context()); /*int*/;
      kk_integer_t doy2 = kk_integer_sub(days,y0_4851,kk_context()); /*int*/;
      bool _match_5907 = kk_integer_lt_borrow(doy2,(kk_integer_from_small(0)),kk_context()); /*bool*/;
      if (_match_5907) {
        kk_integer_drop(doy2, _ctx);
        return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(approx, _ctx), kk_integer_box(doy1, _ctx), _ctx);
      }
      {
        kk_integer_drop(doy1, _ctx);
        kk_integer_t _b_5513_5490 = kk_integer_add_small_const(approx, 1, _ctx); /*int*/;
        return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5513_5490, _ctx), kk_integer_box(doy2, _ctx), _ctx);
      }
    }
  }
}


// lift anonymous function
struct kk_std_time_calendar_cal_iso_fun6538__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_iso_fun6538(kk_function_t _fself, kk_integer_t _x16536, kk_integer_t _x26537, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_fun6538(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_iso_fun6538, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_integer_t kk_std_time_calendar_cal_iso_fun6538(kk_function_t _fself, kk_integer_t _x16536, kk_integer_t _x26537, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_days_before_month(_x16536, _x26537, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_iso_fun6541__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_iso_fun6541(kk_function_t _fself, kk_integer_t _x16539, kk_integer_t _x26540, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_fun6541(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_iso_fun6541, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_integer_t kk_std_time_calendar_cal_iso_fun6541(kk_function_t _fself, kk_integer_t _x16539, kk_integer_t _x26540, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_doy_to_month(_x16539, _x26540, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_iso_fun6553__t {
  struct kk_function_s _base;
};
static kk_box_t kk_std_time_calendar_cal_iso_fun6553(kk_function_t _fself, kk_box_t _b_5508, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_fun6553(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_iso_fun6553, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_box_t kk_std_time_calendar_cal_iso_fun6553(kk_function_t _fself, kk_box_t _b_5508, kk_context_t* _ctx) {
  kk_unused(_fself);
  kk_string_t _x6554;
  kk_std_time_date__date _norm_5519 = kk_std_time_date__date_unbox(_b_5508, KK_OWNED, _ctx); /*std/time/date/date*/;
  kk_std_core_types__optional _match_5901 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5901, _ctx)) {
    kk_box_t _fun_unbox_x5501 = _match_5901._cons.Optional.value;
    kk_box_dup(_fun_unbox_x5501, _ctx);
    kk_std_core_types__optional_drop(_match_5901, _ctx);
    kk_std_time_date__date _b_5520_5504 = _norm_5519; /*std/time/date/date*/;
    kk_box_t _x6555;
    kk_function_t _x6556 = kk_function_unbox(_fun_unbox_x5501, _ctx); /*(5502) -> 5503*/
    _x6555 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6556, (_x6556, kk_std_time_date__date_box(_b_5520_5504, _ctx), _ctx), _ctx); /*5503*/
    _x6554 = kk_string_unbox(_x6555); /*string*/
  }
  else {
    kk_std_core_types__optional_drop(_match_5901, _ctx);
    kk_std_time_date__date d_5521 = _norm_5519; /*std/time/date/date*/;
    kk_std_time_date__date_drop(d_5521, _ctx);
    _x6554 = kk_string_empty(); /*string*/
  }
  return kk_string_box(_x6554);
}

kk_std_time_calendar__calendar kk_std_time_calendar_cal_iso;
 
// Return the instant in time for a given `:date` and `:clock` (= `clock0`) interpreted by
// calendar `cal` (=`cal-iso`) in a timezone `tz` (=`tz-utc` by default).

kk_std_time_instant__instant kk_std_time_calendar_instant(kk_std_time_instant__timescale ts, kk_std_time_date__date d, kk_std_core_types__optional c, kk_std_core_types__optional tz, kk_std_core_types__optional cal, kk_context_t* _ctx) { /* (ts : std/time/instant/timescale, d : std/time/date/date, c : optional<std/time/date/clock>, tz : optional<timezone>, cal : optional<calendar>) -> std/time/instant/instant */ 
  kk_std_time_calendar__calendar _match_5900;
  if (kk_std_core_types__is_Optional(cal, _ctx)) {
    kk_box_t _box_x5522 = cal._cons.Optional.value;
    kk_std_time_calendar__calendar _cal_3011 = kk_std_time_calendar__calendar_unbox(_box_x5522, KK_BORROWED, _ctx);
    kk_std_time_calendar__calendar_dup(_cal_3011, _ctx);
    kk_std_core_types__optional_drop(cal, _ctx);
    _match_5900 = _cal_3011; /*std/time/calendar/calendar*/
  }
  else {
    kk_std_core_types__optional_drop(cal, _ctx);
    _match_5900 = kk_std_time_calendar__calendar_dup(kk_std_time_calendar_cal_iso, _ctx); /*std/time/calendar/calendar*/
  }
  {
    struct kk_std_time_calendar_Calendar* _con6558 = kk_std_time_calendar__as_Calendar(_match_5900, _ctx);
    kk_string_t _pat0 = _con6558->name;
    kk_string_t _pat1 = _con6558->long_name;
    kk_string_t _pat2 = _con6558->month_prefix;
    kk_function_t _pat3 = _con6558->show_era;
    kk_function_t _pat4 = _con6558->instant_to_dc;
    kk_function_t _x = _con6558->dc_to_instant;
    kk_function_t _pat5 = _con6558->days_to_date;
    kk_function_t _pat6 = _con6558->date_to_days;
    if kk_likely(kk_datatype_ptr_is_unique(_match_5900, _ctx)) {
      kk_function_drop(_pat6, _ctx);
      kk_function_drop(_pat5, _ctx);
      kk_function_drop(_pat4, _ctx);
      kk_function_drop(_pat3, _ctx);
      kk_string_drop(_pat2, _ctx);
      kk_string_drop(_pat1, _ctx);
      kk_string_drop(_pat0, _ctx);
      kk_datatype_ptr_free(_match_5900, _ctx);
    }
    else {
      kk_function_dup(_x, _ctx);
      kk_datatype_ptr_decref(_match_5900, _ctx);
    }
    kk_std_time_date__clock _x6559;
    if (kk_std_core_types__is_Optional(c, _ctx)) {
      kk_box_t _box_x5523 = c._cons.Optional.value;
      kk_std_time_date__clock _c_3003 = kk_std_time_date__clock_unbox(_box_x5523, KK_BORROWED, _ctx);
      kk_std_time_date__clock_dup(_c_3003, _ctx);
      kk_std_core_types__optional_drop(c, _ctx);
      _x6559 = _c_3003; /*std/time/date/clock*/
    }
    else {
      kk_std_core_types__optional_drop(c, _ctx);
      _x6559 = kk_std_time_date__clock_dup(kk_std_time_date_clock0, _ctx); /*std/time/date/clock*/
    }
    kk_std_time_calendar__timezone _x6560;
    if (kk_std_core_types__is_Optional(tz, _ctx)) {
      kk_box_t _box_x5524 = tz._cons.Optional.value;
      kk_std_time_calendar__timezone _tz_3007 = kk_std_time_calendar__timezone_unbox(_box_x5524, KK_BORROWED, _ctx);
      kk_std_time_calendar__timezone_dup(_tz_3007, _ctx);
      kk_std_core_types__optional_drop(tz, _ctx);
      _x6560 = _tz_3007; /*std/time/calendar/timezone*/
    }
    else {
      kk_std_core_types__optional_drop(tz, _ctx);
      _x6560 = kk_std_time_calendar__timezone_dup(kk_std_time_calendar_tz_utc, _ctx); /*std/time/calendar/timezone*/
    }
    return kk_function_call(kk_std_time_instant__instant, (kk_function_t, kk_std_time_date__date, kk_std_time_date__clock, kk_std_time_calendar__timezone, kk_std_time_instant__timescale, kk_context_t*), _x, (_x, d, _x6559, _x6560, ts, _ctx), _ctx);
  }
}
 
// monadic lift

kk_std_time_instant__instant kk_std_time_calendar__mlift5143_instant_1(kk_std_core_types__optional c, kk_std_core_types__optional cal, kk_std_time_date__date d, kk_std_core_types__optional tz, kk_std_time_instant__timescale _c_5108, kk_context_t* _ctx) { /* (c : optional<std/time/date/clock>, cal : optional<calendar>, d : std/time/date/date, tz : optional<timezone>, std/time/instant/timescale) -> std/time/instant/instant */ 
  kk_evv_t w = kk_evv_swap_create0(kk_context()); /*std/core/hnd/evv<std/time/utc/utc>*/;
  kk_std_time_instant__instant x;
  kk_std_time_calendar__calendar _match_5899;
  if (kk_std_core_types__is_Optional(cal, _ctx)) {
    kk_box_t _box_x5525 = cal._cons.Optional.value;
    kk_std_time_calendar__calendar _cal_3033 = kk_std_time_calendar__calendar_unbox(_box_x5525, KK_BORROWED, _ctx);
    kk_std_time_calendar__calendar_dup(_cal_3033, _ctx);
    kk_std_core_types__optional_drop(cal, _ctx);
    _match_5899 = _cal_3033; /*std/time/calendar/calendar*/
  }
  else {
    kk_std_core_types__optional_drop(cal, _ctx);
    _match_5899 = kk_std_time_calendar__calendar_dup(kk_std_time_calendar_cal_iso, _ctx); /*std/time/calendar/calendar*/
  }
  {
    struct kk_std_time_calendar_Calendar* _con6561 = kk_std_time_calendar__as_Calendar(_match_5899, _ctx);
    kk_string_t _pat0 = _con6561->name;
    kk_string_t _pat1 = _con6561->long_name;
    kk_string_t _pat2 = _con6561->month_prefix;
    kk_function_t _pat3 = _con6561->show_era;
    kk_function_t _pat4 = _con6561->instant_to_dc;
    kk_function_t _x = _con6561->dc_to_instant;
    kk_function_t _pat50 = _con6561->days_to_date;
    kk_function_t _pat60 = _con6561->date_to_days;
    if kk_likely(kk_datatype_ptr_is_unique(_match_5899, _ctx)) {
      kk_function_drop(_pat60, _ctx);
      kk_function_drop(_pat50, _ctx);
      kk_function_drop(_pat4, _ctx);
      kk_function_drop(_pat3, _ctx);
      kk_string_drop(_pat2, _ctx);
      kk_string_drop(_pat1, _ctx);
      kk_string_drop(_pat0, _ctx);
      kk_datatype_ptr_free(_match_5899, _ctx);
    }
    else {
      kk_function_dup(_x, _ctx);
      kk_datatype_ptr_decref(_match_5899, _ctx);
    }
    kk_std_time_date__clock _x6562;
    if (kk_std_core_types__is_Optional(c, _ctx)) {
      kk_box_t _box_x5526 = c._cons.Optional.value;
      kk_std_time_date__clock _c_3025 = kk_std_time_date__clock_unbox(_box_x5526, KK_BORROWED, _ctx);
      kk_std_time_date__clock_dup(_c_3025, _ctx);
      kk_std_core_types__optional_drop(c, _ctx);
      _x6562 = _c_3025; /*std/time/date/clock*/
    }
    else {
      kk_std_core_types__optional_drop(c, _ctx);
      _x6562 = kk_std_time_date__clock_dup(kk_std_time_date_clock0, _ctx); /*std/time/date/clock*/
    }
    kk_std_time_calendar__timezone _x6563;
    if (kk_std_core_types__is_Optional(tz, _ctx)) {
      kk_box_t _box_x5527 = tz._cons.Optional.value;
      kk_std_time_calendar__timezone _tz_3029 = kk_std_time_calendar__timezone_unbox(_box_x5527, KK_BORROWED, _ctx);
      kk_std_time_calendar__timezone_dup(_tz_3029, _ctx);
      kk_std_core_types__optional_drop(tz, _ctx);
      _x6563 = _tz_3029; /*std/time/calendar/timezone*/
    }
    else {
      kk_std_core_types__optional_drop(tz, _ctx);
      _x6563 = kk_std_time_calendar__timezone_dup(kk_std_time_calendar_tz_utc, _ctx); /*std/time/calendar/timezone*/
    }
    x = kk_function_call(kk_std_time_instant__instant, (kk_function_t, kk_std_time_date__date, kk_std_time_date__clock, kk_std_time_calendar__timezone, kk_std_time_instant__timescale, kk_context_t*), _x, (_x, d, _x6562, _x6563, _c_5108, _ctx), _ctx); /*std/time/instant/instant*/
  }
  kk_unit_t keep = kk_Unit;
  kk_evv_set(w,kk_context());
  return x;
}


// lift anonymous function
struct kk_std_time_calendar_instant_fun6568__t_1 {
  struct kk_function_s _base;
  kk_std_core_types__optional c;
  kk_std_core_types__optional cal;
  kk_std_time_date__date d;
  kk_std_core_types__optional tz;
};
static kk_box_t kk_std_time_calendar_instant_fun6568_1(kk_function_t _fself, kk_box_t _b_5536, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_instant_fun6568_1(kk_std_core_types__optional c, kk_std_core_types__optional cal, kk_std_time_date__date d, kk_std_core_types__optional tz, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun6568__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar_instant_fun6568__t_1, 10, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_instant_fun6568_1, kk_context());
  _self->c = c;
  _self->cal = cal;
  _self->d = d;
  _self->tz = tz;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_box_t kk_std_time_calendar_instant_fun6568_1(kk_function_t _fself, kk_box_t _b_5536, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun6568__t_1* _self = kk_function_as(struct kk_std_time_calendar_instant_fun6568__t_1*, _fself, _ctx);
  kk_std_core_types__optional c = _self->c; /* optional<std/time/date/clock> */
  kk_std_core_types__optional cal = _self->cal; /* optional<std/time/calendar/calendar> */
  kk_std_time_date__date d = _self->d; /* std/time/date/date */
  kk_std_core_types__optional tz = _self->tz; /* optional<std/time/calendar/timezone> */
  kk_drop_match(_self, {kk_std_core_types__optional_dup(c, _ctx);kk_std_core_types__optional_dup(cal, _ctx);kk_std_time_date__date_dup(d, _ctx);kk_std_core_types__optional_dup(tz, _ctx);}, {}, _ctx)
  kk_std_time_instant__timescale _c_5541_5108 = kk_std_time_instant__timescale_unbox(_b_5536, KK_OWNED, _ctx); /*std/time/instant/timescale*/;
  kk_std_time_instant__instant _x6569 = kk_std_time_calendar__mlift5143_instant_1(c, cal, d, tz, _c_5541_5108, _ctx); /*std/time/instant/instant*/
  return kk_std_time_instant__instant_box(_x6569, _ctx);
}

kk_std_time_instant__instant kk_std_time_calendar_instant_1(kk_std_time_date__date d, kk_std_core_types__optional c, kk_std_core_types__optional tz, kk_std_core_types__optional cal, kk_std_core_types__optional ts, kk_context_t* _ctx) { /* (d : std/time/date/date, c : optional<std/time/date/clock>, tz : optional<timezone>, cal : optional<calendar>, ts : optional<std/time/instant/timescale>) -> std/time/utc/utc std/time/instant/instant */ 
  kk_std_time_instant__timescale x_5148;
  if (kk_std_core_types__is_Optional(ts, _ctx)) {
    kk_box_t _box_x5528 = ts._cons.Optional.value;
    kk_std_time_instant__timescale _ts_3039 = kk_std_time_instant__timescale_unbox(_box_x5528, KK_BORROWED, _ctx);
    kk_std_time_instant__timescale_dup(_ts_3039, _ctx);
    kk_std_core_types__optional_drop(ts, _ctx);
    x_5148 = _ts_3039; /*std/time/instant/timescale*/
  }
  else {
    kk_std_core_types__optional_drop(ts, _ctx);
    kk_std_core_hnd__ev ev_5151;
    kk_ssize_t _x6564 = (KK_IZ(0)); /*ssize_t*/
    ev_5151 = kk_evv_at(_x6564,kk_context()); /*std/core/hnd/ev<std/time/utc/.hnd-utc>*/
    kk_box_t _x6565;
    {
      struct kk_std_core_hnd_Ev* _con6566 = kk_std_core_hnd__as_Ev(ev_5151, _ctx);
      kk_box_t _box_x5529 = _con6566->hnd;
      kk_std_core_hnd__marker m0 = _con6566->marker;
      kk_std_time_utc__hnd_utc h = kk_std_time_utc__hnd_utc_unbox(_box_x5529, KK_BORROWED, _ctx);
      kk_std_time_utc__hnd_utc_dup(h, _ctx);
      kk_std_core_hnd__clause0 _match_5897;
      kk_std_core_hnd__clause0 _brw_5898 = kk_std_time_utc__select_utc(h, _ctx); /*std/core/hnd/clause0<std/time/instant/timescale,std/time/utc/.hnd-utc,277,278>*/;
      kk_std_time_utc__hnd_utc_drop(h, _ctx);
      _match_5897 = _brw_5898; /*std/core/hnd/clause0<std/time/instant/timescale,std/time/utc/.hnd-utc,277,278>*/
      {
        kk_function_t _fun_unbox_x5532 = _match_5897.clause;
        _x6565 = kk_function_call(kk_box_t, (kk_function_t, kk_std_core_hnd__marker, kk_std_core_hnd__ev, kk_context_t*), _fun_unbox_x5532, (_fun_unbox_x5532, m0, ev_5151, _ctx), _ctx); /*37*/
      }
    }
    x_5148 = kk_std_time_instant__timescale_unbox(_x6565, KK_OWNED, _ctx); /*std/time/instant/timescale*/
  }
  if (kk_yielding(kk_context())) {
    kk_datatype_ptr_dropn(x_5148, (KK_I32(10)), _ctx);
    kk_box_t _x6567 = kk_std_core_hnd_yield_extend(kk_std_time_calendar_new_instant_fun6568_1(c, cal, d, tz, _ctx), _ctx); /*3926*/
    return kk_std_time_instant__instant_unbox(_x6567, KK_OWNED, _ctx);
  }
  {
    kk_evv_t w = kk_evv_swap_create0(kk_context()); /*std/core/hnd/evv<std/time/utc/utc>*/;
    kk_std_time_instant__instant x0;
    kk_std_time_calendar__calendar _match_5896;
    if (kk_std_core_types__is_Optional(cal, _ctx)) {
      kk_box_t _box_x5537 = cal._cons.Optional.value;
      kk_std_time_calendar__calendar _cal_3033 = kk_std_time_calendar__calendar_unbox(_box_x5537, KK_BORROWED, _ctx);
      kk_std_time_calendar__calendar_dup(_cal_3033, _ctx);
      kk_std_core_types__optional_drop(cal, _ctx);
      _match_5896 = _cal_3033; /*std/time/calendar/calendar*/
    }
    else {
      kk_std_core_types__optional_drop(cal, _ctx);
      _match_5896 = kk_std_time_calendar__calendar_dup(kk_std_time_calendar_cal_iso, _ctx); /*std/time/calendar/calendar*/
    }
    {
      struct kk_std_time_calendar_Calendar* _con6570 = kk_std_time_calendar__as_Calendar(_match_5896, _ctx);
      kk_string_t _pat01 = _con6570->name;
      kk_string_t _pat11 = _con6570->long_name;
      kk_string_t _pat20 = _con6570->month_prefix;
      kk_function_t _pat32 = _con6570->show_era;
      kk_function_t _pat40 = _con6570->instant_to_dc;
      kk_function_t _x = _con6570->dc_to_instant;
      kk_function_t _pat50 = _con6570->days_to_date;
      kk_function_t _pat60 = _con6570->date_to_days;
      if kk_likely(kk_datatype_ptr_is_unique(_match_5896, _ctx)) {
        kk_function_drop(_pat60, _ctx);
        kk_function_drop(_pat50, _ctx);
        kk_function_drop(_pat40, _ctx);
        kk_function_drop(_pat32, _ctx);
        kk_string_drop(_pat20, _ctx);
        kk_string_drop(_pat11, _ctx);
        kk_string_drop(_pat01, _ctx);
        kk_datatype_ptr_free(_match_5896, _ctx);
      }
      else {
        kk_function_dup(_x, _ctx);
        kk_datatype_ptr_decref(_match_5896, _ctx);
      }
      kk_std_time_date__clock _x6571;
      if (kk_std_core_types__is_Optional(c, _ctx)) {
        kk_box_t _box_x5538 = c._cons.Optional.value;
        kk_std_time_date__clock _c_3025 = kk_std_time_date__clock_unbox(_box_x5538, KK_BORROWED, _ctx);
        kk_std_time_date__clock_dup(_c_3025, _ctx);
        kk_std_core_types__optional_drop(c, _ctx);
        _x6571 = _c_3025; /*std/time/date/clock*/
      }
      else {
        kk_std_core_types__optional_drop(c, _ctx);
        _x6571 = kk_std_time_date__clock_dup(kk_std_time_date_clock0, _ctx); /*std/time/date/clock*/
      }
      kk_std_time_calendar__timezone _x6572;
      if (kk_std_core_types__is_Optional(tz, _ctx)) {
        kk_box_t _box_x5539 = tz._cons.Optional.value;
        kk_std_time_calendar__timezone _tz_3029 = kk_std_time_calendar__timezone_unbox(_box_x5539, KK_BORROWED, _ctx);
        kk_std_time_calendar__timezone_dup(_tz_3029, _ctx);
        kk_std_core_types__optional_drop(tz, _ctx);
        _x6572 = _tz_3029; /*std/time/calendar/timezone*/
      }
      else {
        kk_std_core_types__optional_drop(tz, _ctx);
        _x6572 = kk_std_time_calendar__timezone_dup(kk_std_time_calendar_tz_utc, _ctx); /*std/time/calendar/timezone*/
      }
      x0 = kk_function_call(kk_std_time_instant__instant, (kk_function_t, kk_std_time_date__date, kk_std_time_date__clock, kk_std_time_calendar__timezone, kk_std_time_instant__timescale, kk_context_t*), _x, (_x, d, _x6571, _x6572, x_5148, _ctx), _ctx); /*std/time/instant/instant*/
    }
    kk_unit_t keep = kk_Unit;
    kk_evv_set(w,kk_context());
    return x0;
  }
}
 
/* Return the instant in time for a given date and clock interpreted by
   calendar `cal` (=`cal-iso`) in a timezone `tz` (=`tz-utc` by default).
  The `month`, `day`, `hour`, `minutes` may be outside their usual ranges
  and will be normalized during the conversion. For example, January 33 converts to February 2.
  This makes it very easy to add- or subtract days or months to an existing time.
  When the `seconds` or fraction of seconds `frac` add up over 60 those extra seconds are
  interpreted as leap seconds.
  Due to timezone transitions, or leap seconds, it is possible to specify dates that never
  happened (as it was skipped by a timezone change), or ambigious times (as a timezone springs back).
  In such cases, the time is always interpreted in the earlier timezone.
*/

kk_std_time_instant__instant kk_std_time_calendar_instant_2(kk_std_time_instant__timescale ts, kk_integer_t year, kk_std_core_types__optional month, kk_std_core_types__optional day, kk_std_core_types__optional hours, kk_std_core_types__optional minutes, kk_std_core_types__optional seconds, kk_std_core_types__optional frac, kk_std_core_types__optional tz, kk_std_core_types__optional cal, kk_context_t* _ctx) { /* (ts : std/time/instant/timescale, year : int, month : optional<int>, day : optional<int>, hours : optional<int>, minutes : optional<int>, seconds : optional<int>, frac : optional<float64>, tz : optional<timezone>, cal : optional<calendar>) -> std/time/instant/instant */ 
  kk_std_num_ddouble__ddouble fsecs;
  kk_std_num_ddouble__ddouble _x6573;
  kk_integer_t _x6574;
  if (kk_std_core_types__is_Optional(seconds, _ctx)) {
    kk_box_t _box_x5542 = seconds._cons.Optional.value;
    kk_integer_t _seconds_3092 = kk_integer_unbox(_box_x5542, _ctx);
    kk_integer_dup(_seconds_3092, _ctx);
    kk_std_core_types__optional_drop(seconds, _ctx);
    _x6574 = _seconds_3092; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(seconds, _ctx);
    _x6574 = kk_integer_from_small(0); /*int*/
  }
  _x6573 = kk_std_num_ddouble_ddouble_int_exp(_x6574, kk_integer_from_small(0), _ctx); /*std/num/ddouble/ddouble*/
  kk_std_num_ddouble__ddouble _x6575;
  double _x6576;
  if (kk_std_core_types__is_Optional(frac, _ctx)) {
    kk_box_t _box_x5543 = frac._cons.Optional.value;
    double _frac_3096 = kk_double_unbox(_box_x5543, KK_BORROWED, _ctx);
    kk_std_core_types__optional_drop(frac, _ctx);
    _x6576 = _frac_3096; /*float64*/
  }
  else {
    kk_std_core_types__optional_drop(frac, _ctx);
    _x6576 = 0x0p+0; /*float64*/
  }
  _x6575 = kk_std_num_ddouble__new_Ddouble(_x6576, 0x0p+0, _ctx); /*std/num/ddouble/ddouble*/
  fsecs = kk_std_num_ddouble__lp__plus__rp_(_x6573, _x6575, _ctx); /*std/num/ddouble/ddouble*/
  kk_std_time_calendar__calendar _match_5894;
  if (kk_std_core_types__is_Optional(cal, _ctx)) {
    kk_box_t _box_x5544 = cal._cons.Optional.value;
    kk_std_time_calendar__calendar _cal_3104 = kk_std_time_calendar__calendar_unbox(_box_x5544, KK_BORROWED, _ctx);
    kk_std_time_calendar__calendar_dup(_cal_3104, _ctx);
    kk_std_core_types__optional_drop(cal, _ctx);
    _match_5894 = _cal_3104; /*std/time/calendar/calendar*/
  }
  else {
    kk_std_core_types__optional_drop(cal, _ctx);
    _match_5894 = kk_std_time_calendar__calendar_dup(kk_std_time_calendar_cal_iso, _ctx); /*std/time/calendar/calendar*/
  }
  {
    struct kk_std_time_calendar_Calendar* _con6577 = kk_std_time_calendar__as_Calendar(_match_5894, _ctx);
    kk_string_t _pat0 = _con6577->name;
    kk_string_t _pat1 = _con6577->long_name;
    kk_string_t _pat2 = _con6577->month_prefix;
    kk_function_t _pat3 = _con6577->show_era;
    kk_function_t _pat4 = _con6577->instant_to_dc;
    kk_function_t _x = _con6577->dc_to_instant;
    kk_function_t _pat5 = _con6577->days_to_date;
    kk_function_t _pat6 = _con6577->date_to_days;
    if kk_likely(kk_datatype_ptr_is_unique(_match_5894, _ctx)) {
      kk_function_drop(_pat6, _ctx);
      kk_function_drop(_pat5, _ctx);
      kk_function_drop(_pat4, _ctx);
      kk_function_drop(_pat3, _ctx);
      kk_string_drop(_pat2, _ctx);
      kk_string_drop(_pat1, _ctx);
      kk_string_drop(_pat0, _ctx);
      kk_datatype_ptr_free(_match_5894, _ctx);
    }
    else {
      kk_function_dup(_x, _ctx);
      kk_datatype_ptr_decref(_match_5894, _ctx);
    }
    kk_std_time_date__date _x6578;
    kk_integer_t _x6579;
    if (kk_std_core_types__is_Optional(month, _ctx)) {
      kk_box_t _box_x5545 = month._cons.Optional.value;
      kk_integer_t _month_3076 = kk_integer_unbox(_box_x5545, _ctx);
      kk_integer_dup(_month_3076, _ctx);
      kk_std_core_types__optional_drop(month, _ctx);
      _x6579 = _month_3076; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(month, _ctx);
      _x6579 = kk_integer_from_small(1); /*int*/
    }
    kk_integer_t _x6580;
    if (kk_std_core_types__is_Optional(day, _ctx)) {
      kk_box_t _box_x5546 = day._cons.Optional.value;
      kk_integer_t _day_3080 = kk_integer_unbox(_box_x5546, _ctx);
      kk_integer_dup(_day_3080, _ctx);
      kk_std_core_types__optional_drop(day, _ctx);
      _x6580 = _day_3080; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(day, _ctx);
      _x6580 = kk_integer_from_small(1); /*int*/
    }
    _x6578 = kk_std_time_date__new_Date(year, _x6579, _x6580, _ctx); /*std/time/date/date*/
    kk_std_time_date__clock _x6581;
    kk_integer_t _x6582;
    if (kk_std_core_types__is_Optional(hours, _ctx)) {
      kk_box_t _box_x5547 = hours._cons.Optional.value;
      kk_integer_t _hours_3084 = kk_integer_unbox(_box_x5547, _ctx);
      kk_integer_dup(_hours_3084, _ctx);
      kk_std_core_types__optional_drop(hours, _ctx);
      _x6582 = _hours_3084; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(hours, _ctx);
      _x6582 = kk_integer_from_small(0); /*int*/
    }
    kk_integer_t _x6583;
    if (kk_std_core_types__is_Optional(minutes, _ctx)) {
      kk_box_t _box_x5548 = minutes._cons.Optional.value;
      kk_integer_t _minutes_3088 = kk_integer_unbox(_box_x5548, _ctx);
      kk_integer_dup(_minutes_3088, _ctx);
      kk_std_core_types__optional_drop(minutes, _ctx);
      _x6583 = _minutes_3088; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(minutes, _ctx);
      _x6583 = kk_integer_from_small(0); /*int*/
    }
    _x6581 = kk_std_time_date__new_Clock(kk_reuse_null, 0, _x6582, _x6583, fsecs, _ctx); /*std/time/date/clock*/
    kk_std_time_calendar__timezone _x6584;
    if (kk_std_core_types__is_Optional(tz, _ctx)) {
      kk_box_t _box_x5549 = tz._cons.Optional.value;
      kk_std_time_calendar__timezone _tz_3100 = kk_std_time_calendar__timezone_unbox(_box_x5549, KK_BORROWED, _ctx);
      kk_std_time_calendar__timezone_dup(_tz_3100, _ctx);
      kk_std_core_types__optional_drop(tz, _ctx);
      _x6584 = _tz_3100; /*std/time/calendar/timezone*/
    }
    else {
      kk_std_core_types__optional_drop(tz, _ctx);
      _x6584 = kk_std_time_calendar__timezone_dup(kk_std_time_calendar_tz_utc, _ctx); /*std/time/calendar/timezone*/
    }
    return kk_function_call(kk_std_time_instant__instant, (kk_function_t, kk_std_time_date__date, kk_std_time_date__clock, kk_std_time_calendar__timezone, kk_std_time_instant__timescale, kk_context_t*), _x, (_x, _x6578, _x6581, _x6584, ts, _ctx), _ctx);
  }
}
 
// monadic lift

kk_std_time_instant__instant kk_std_time_calendar__mlift5144_instant_3(kk_std_core_types__optional cal, kk_std_core_types__optional day, kk_std_core_types__optional frac, kk_std_core_types__optional hours, kk_std_core_types__optional minutes, kk_std_core_types__optional month, kk_std_core_types__optional seconds, kk_std_core_types__optional tz, kk_integer_t year, kk_std_time_instant__timescale _c_5110, kk_context_t* _ctx) { /* (cal : optional<calendar>, day : optional<int>, frac : optional<float64>, hours : optional<int>, minutes : optional<int>, month : optional<int>, seconds : optional<int>, tz : optional<timezone>, year : int, std/time/instant/timescale) -> std/time/instant/instant */ 
  kk_evv_t w = kk_evv_swap_create0(kk_context()); /*std/core/hnd/evv<std/time/utc/utc>*/;
  kk_std_num_ddouble__ddouble fsecs;
  kk_std_num_ddouble__ddouble _x6585;
  kk_integer_t _x6586;
  if (kk_std_core_types__is_Optional(seconds, _ctx)) {
    kk_box_t _box_x5550 = seconds._cons.Optional.value;
    kk_integer_t _seconds_3209 = kk_integer_unbox(_box_x5550, _ctx);
    kk_integer_dup(_seconds_3209, _ctx);
    kk_std_core_types__optional_drop(seconds, _ctx);
    _x6586 = _seconds_3209; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(seconds, _ctx);
    _x6586 = kk_integer_from_small(0); /*int*/
  }
  _x6585 = kk_std_num_ddouble_ddouble_int_exp(_x6586, kk_integer_from_small(0), _ctx); /*std/num/ddouble/ddouble*/
  kk_std_num_ddouble__ddouble _x6587;
  double _x6588;
  if (kk_std_core_types__is_Optional(frac, _ctx)) {
    kk_box_t _box_x5551 = frac._cons.Optional.value;
    double _frac_3213 = kk_double_unbox(_box_x5551, KK_BORROWED, _ctx);
    kk_std_core_types__optional_drop(frac, _ctx);
    _x6588 = _frac_3213; /*float64*/
  }
  else {
    kk_std_core_types__optional_drop(frac, _ctx);
    _x6588 = 0x0p+0; /*float64*/
  }
  _x6587 = kk_std_num_ddouble__new_Ddouble(_x6588, 0x0p+0, _ctx); /*std/num/ddouble/ddouble*/
  fsecs = kk_std_num_ddouble__lp__plus__rp_(_x6585, _x6587, _ctx); /*std/num/ddouble/ddouble*/
  kk_std_time_instant__instant x;
  kk_std_time_calendar__calendar _match_5893;
  if (kk_std_core_types__is_Optional(cal, _ctx)) {
    kk_box_t _box_x5552 = cal._cons.Optional.value;
    kk_std_time_calendar__calendar _cal_3221 = kk_std_time_calendar__calendar_unbox(_box_x5552, KK_BORROWED, _ctx);
    kk_std_time_calendar__calendar_dup(_cal_3221, _ctx);
    kk_std_core_types__optional_drop(cal, _ctx);
    _match_5893 = _cal_3221; /*std/time/calendar/calendar*/
  }
  else {
    kk_std_core_types__optional_drop(cal, _ctx);
    _match_5893 = kk_std_time_calendar__calendar_dup(kk_std_time_calendar_cal_iso, _ctx); /*std/time/calendar/calendar*/
  }
  {
    struct kk_std_time_calendar_Calendar* _con6589 = kk_std_time_calendar__as_Calendar(_match_5893, _ctx);
    kk_string_t _pat0 = _con6589->name;
    kk_string_t _pat1 = _con6589->long_name;
    kk_string_t _pat2 = _con6589->month_prefix;
    kk_function_t _pat3 = _con6589->show_era;
    kk_function_t _pat4 = _con6589->instant_to_dc;
    kk_function_t _x = _con6589->dc_to_instant;
    kk_function_t _pat5 = _con6589->days_to_date;
    kk_function_t _pat6 = _con6589->date_to_days;
    if kk_likely(kk_datatype_ptr_is_unique(_match_5893, _ctx)) {
      kk_function_drop(_pat6, _ctx);
      kk_function_drop(_pat5, _ctx);
      kk_function_drop(_pat4, _ctx);
      kk_function_drop(_pat3, _ctx);
      kk_string_drop(_pat2, _ctx);
      kk_string_drop(_pat1, _ctx);
      kk_string_drop(_pat0, _ctx);
      kk_datatype_ptr_free(_match_5893, _ctx);
    }
    else {
      kk_function_dup(_x, _ctx);
      kk_datatype_ptr_decref(_match_5893, _ctx);
    }
    kk_std_time_date__date _x6590;
    kk_integer_t _x6591;
    if (kk_std_core_types__is_Optional(month, _ctx)) {
      kk_box_t _box_x5553 = month._cons.Optional.value;
      kk_integer_t _month_3193 = kk_integer_unbox(_box_x5553, _ctx);
      kk_integer_dup(_month_3193, _ctx);
      kk_std_core_types__optional_drop(month, _ctx);
      _x6591 = _month_3193; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(month, _ctx);
      _x6591 = kk_integer_from_small(1); /*int*/
    }
    kk_integer_t _x6592;
    if (kk_std_core_types__is_Optional(day, _ctx)) {
      kk_box_t _box_x5554 = day._cons.Optional.value;
      kk_integer_t _day_3197 = kk_integer_unbox(_box_x5554, _ctx);
      kk_integer_dup(_day_3197, _ctx);
      kk_std_core_types__optional_drop(day, _ctx);
      _x6592 = _day_3197; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(day, _ctx);
      _x6592 = kk_integer_from_small(1); /*int*/
    }
    _x6590 = kk_std_time_date__new_Date(year, _x6591, _x6592, _ctx); /*std/time/date/date*/
    kk_std_time_date__clock _x6593;
    kk_integer_t _x6594;
    if (kk_std_core_types__is_Optional(hours, _ctx)) {
      kk_box_t _box_x5555 = hours._cons.Optional.value;
      kk_integer_t _hours_3201 = kk_integer_unbox(_box_x5555, _ctx);
      kk_integer_dup(_hours_3201, _ctx);
      kk_std_core_types__optional_drop(hours, _ctx);
      _x6594 = _hours_3201; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(hours, _ctx);
      _x6594 = kk_integer_from_small(0); /*int*/
    }
    kk_integer_t _x6595;
    if (kk_std_core_types__is_Optional(minutes, _ctx)) {
      kk_box_t _box_x5556 = minutes._cons.Optional.value;
      kk_integer_t _minutes_3205 = kk_integer_unbox(_box_x5556, _ctx);
      kk_integer_dup(_minutes_3205, _ctx);
      kk_std_core_types__optional_drop(minutes, _ctx);
      _x6595 = _minutes_3205; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(minutes, _ctx);
      _x6595 = kk_integer_from_small(0); /*int*/
    }
    _x6593 = kk_std_time_date__new_Clock(kk_reuse_null, 0, _x6594, _x6595, fsecs, _ctx); /*std/time/date/clock*/
    kk_std_time_calendar__timezone _x6596;
    if (kk_std_core_types__is_Optional(tz, _ctx)) {
      kk_box_t _box_x5557 = tz._cons.Optional.value;
      kk_std_time_calendar__timezone _tz_3217 = kk_std_time_calendar__timezone_unbox(_box_x5557, KK_BORROWED, _ctx);
      kk_std_time_calendar__timezone_dup(_tz_3217, _ctx);
      kk_std_core_types__optional_drop(tz, _ctx);
      _x6596 = _tz_3217; /*std/time/calendar/timezone*/
    }
    else {
      kk_std_core_types__optional_drop(tz, _ctx);
      _x6596 = kk_std_time_calendar__timezone_dup(kk_std_time_calendar_tz_utc, _ctx); /*std/time/calendar/timezone*/
    }
    x = kk_function_call(kk_std_time_instant__instant, (kk_function_t, kk_std_time_date__date, kk_std_time_date__clock, kk_std_time_calendar__timezone, kk_std_time_instant__timescale, kk_context_t*), _x, (_x, _x6590, _x6593, _x6596, _c_5110, _ctx), _ctx); /*std/time/instant/instant*/
  }
  kk_unit_t keep = kk_Unit;
  kk_evv_set(w,kk_context());
  return x;
}


// lift anonymous function
struct kk_std_time_calendar_instant_fun6601__t_3 {
  struct kk_function_s _base;
  kk_std_core_types__optional cal;
  kk_std_core_types__optional day;
  kk_std_core_types__optional frac;
  kk_std_core_types__optional hours;
  kk_std_core_types__optional minutes;
  kk_std_core_types__optional month;
  kk_std_core_types__optional seconds;
  kk_std_core_types__optional tz;
  kk_integer_t year;
};
static kk_box_t kk_std_time_calendar_instant_fun6601_3(kk_function_t _fself, kk_box_t _b_5566, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_instant_fun6601_3(kk_std_core_types__optional cal, kk_std_core_types__optional day, kk_std_core_types__optional frac, kk_std_core_types__optional hours, kk_std_core_types__optional minutes, kk_std_core_types__optional month, kk_std_core_types__optional seconds, kk_std_core_types__optional tz, kk_integer_t year, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun6601__t_3* _self = kk_function_alloc_as(struct kk_std_time_calendar_instant_fun6601__t_3, 18, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_instant_fun6601_3, kk_context());
  _self->cal = cal;
  _self->day = day;
  _self->frac = frac;
  _self->hours = hours;
  _self->minutes = minutes;
  _self->month = month;
  _self->seconds = seconds;
  _self->tz = tz;
  _self->year = year;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_box_t kk_std_time_calendar_instant_fun6601_3(kk_function_t _fself, kk_box_t _b_5566, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun6601__t_3* _self = kk_function_as(struct kk_std_time_calendar_instant_fun6601__t_3*, _fself, _ctx);
  kk_std_core_types__optional cal = _self->cal; /* optional<std/time/calendar/calendar> */
  kk_std_core_types__optional day = _self->day; /* optional<int> */
  kk_std_core_types__optional frac = _self->frac; /* optional<float64> */
  kk_std_core_types__optional hours = _self->hours; /* optional<int> */
  kk_std_core_types__optional minutes = _self->minutes; /* optional<int> */
  kk_std_core_types__optional month = _self->month; /* optional<int> */
  kk_std_core_types__optional seconds = _self->seconds; /* optional<int> */
  kk_std_core_types__optional tz = _self->tz; /* optional<std/time/calendar/timezone> */
  kk_integer_t year = _self->year; /* int */
  kk_drop_match(_self, {kk_std_core_types__optional_dup(cal, _ctx);kk_std_core_types__optional_dup(day, _ctx);kk_std_core_types__optional_dup(frac, _ctx);kk_std_core_types__optional_dup(hours, _ctx);kk_std_core_types__optional_dup(minutes, _ctx);kk_std_core_types__optional_dup(month, _ctx);kk_std_core_types__optional_dup(seconds, _ctx);kk_std_core_types__optional_dup(tz, _ctx);kk_integer_dup(year, _ctx);}, {}, _ctx)
  kk_std_time_instant__timescale _c_5576_5110 = kk_std_time_instant__timescale_unbox(_b_5566, KK_OWNED, _ctx); /*std/time/instant/timescale*/;
  kk_std_time_instant__instant _x6602 = kk_std_time_calendar__mlift5144_instant_3(cal, day, frac, hours, minutes, month, seconds, tz, year, _c_5576_5110, _ctx); /*std/time/instant/instant*/
  return kk_std_time_instant__instant_box(_x6602, _ctx);
}

kk_std_time_instant__instant kk_std_time_calendar_instant_3(kk_integer_t year, kk_std_core_types__optional month, kk_std_core_types__optional day, kk_std_core_types__optional hours, kk_std_core_types__optional minutes, kk_std_core_types__optional seconds, kk_std_core_types__optional frac, kk_std_core_types__optional tz, kk_std_core_types__optional cal, kk_std_core_types__optional ts, kk_context_t* _ctx) { /* (year : int, month : optional<int>, day : optional<int>, hours : optional<int>, minutes : optional<int>, seconds : optional<int>, frac : optional<float64>, tz : optional<timezone>, cal : optional<calendar>, ts : optional<std/time/instant/timescale>) -> std/time/utc/utc std/time/instant/instant */ 
  kk_std_time_instant__timescale x_5155;
  if (kk_std_core_types__is_Optional(ts, _ctx)) {
    kk_box_t _box_x5558 = ts._cons.Optional.value;
    kk_std_time_instant__timescale _ts_3227 = kk_std_time_instant__timescale_unbox(_box_x5558, KK_BORROWED, _ctx);
    kk_std_time_instant__timescale_dup(_ts_3227, _ctx);
    kk_std_core_types__optional_drop(ts, _ctx);
    x_5155 = _ts_3227; /*std/time/instant/timescale*/
  }
  else {
    kk_std_core_types__optional_drop(ts, _ctx);
    kk_std_core_hnd__ev ev_5158;
    kk_ssize_t _x6597 = (KK_IZ(0)); /*ssize_t*/
    ev_5158 = kk_evv_at(_x6597,kk_context()); /*std/core/hnd/ev<std/time/utc/.hnd-utc>*/
    kk_box_t _x6598;
    {
      struct kk_std_core_hnd_Ev* _con6599 = kk_std_core_hnd__as_Ev(ev_5158, _ctx);
      kk_box_t _box_x5559 = _con6599->hnd;
      kk_std_core_hnd__marker m0 = _con6599->marker;
      kk_std_time_utc__hnd_utc h = kk_std_time_utc__hnd_utc_unbox(_box_x5559, KK_BORROWED, _ctx);
      kk_std_time_utc__hnd_utc_dup(h, _ctx);
      kk_std_core_hnd__clause0 _match_5891;
      kk_std_core_hnd__clause0 _brw_5892 = kk_std_time_utc__select_utc(h, _ctx); /*std/core/hnd/clause0<std/time/instant/timescale,std/time/utc/.hnd-utc,277,278>*/;
      kk_std_time_utc__hnd_utc_drop(h, _ctx);
      _match_5891 = _brw_5892; /*std/core/hnd/clause0<std/time/instant/timescale,std/time/utc/.hnd-utc,277,278>*/
      {
        kk_function_t _fun_unbox_x5562 = _match_5891.clause;
        _x6598 = kk_function_call(kk_box_t, (kk_function_t, kk_std_core_hnd__marker, kk_std_core_hnd__ev, kk_context_t*), _fun_unbox_x5562, (_fun_unbox_x5562, m0, ev_5158, _ctx), _ctx); /*37*/
      }
    }
    x_5155 = kk_std_time_instant__timescale_unbox(_x6598, KK_OWNED, _ctx); /*std/time/instant/timescale*/
  }
  if (kk_yielding(kk_context())) {
    kk_datatype_ptr_dropn(x_5155, (KK_I32(10)), _ctx);
    kk_box_t _x6600 = kk_std_core_hnd_yield_extend(kk_std_time_calendar_new_instant_fun6601_3(cal, day, frac, hours, minutes, month, seconds, tz, year, _ctx), _ctx); /*3926*/
    return kk_std_time_instant__instant_unbox(_x6600, KK_OWNED, _ctx);
  }
  {
    kk_evv_t w = kk_evv_swap_create0(kk_context()); /*std/core/hnd/evv<std/time/utc/utc>*/;
    kk_std_num_ddouble__ddouble fsecs;
    kk_std_num_ddouble__ddouble _x6603;
    kk_integer_t _x6604;
    if (kk_std_core_types__is_Optional(seconds, _ctx)) {
      kk_box_t _box_x5567 = seconds._cons.Optional.value;
      kk_integer_t _seconds_3209 = kk_integer_unbox(_box_x5567, _ctx);
      kk_integer_dup(_seconds_3209, _ctx);
      kk_std_core_types__optional_drop(seconds, _ctx);
      _x6604 = _seconds_3209; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(seconds, _ctx);
      _x6604 = kk_integer_from_small(0); /*int*/
    }
    _x6603 = kk_std_num_ddouble_ddouble_int_exp(_x6604, kk_integer_from_small(0), _ctx); /*std/num/ddouble/ddouble*/
    kk_std_num_ddouble__ddouble _x6605;
    double _x6606;
    if (kk_std_core_types__is_Optional(frac, _ctx)) {
      kk_box_t _box_x5568 = frac._cons.Optional.value;
      double _frac_3213 = kk_double_unbox(_box_x5568, KK_BORROWED, _ctx);
      kk_std_core_types__optional_drop(frac, _ctx);
      _x6606 = _frac_3213; /*float64*/
    }
    else {
      kk_std_core_types__optional_drop(frac, _ctx);
      _x6606 = 0x0p+0; /*float64*/
    }
    _x6605 = kk_std_num_ddouble__new_Ddouble(_x6606, 0x0p+0, _ctx); /*std/num/ddouble/ddouble*/
    fsecs = kk_std_num_ddouble__lp__plus__rp_(_x6603, _x6605, _ctx); /*std/num/ddouble/ddouble*/
    kk_std_time_instant__instant x0;
    kk_std_time_calendar__calendar _match_5890;
    if (kk_std_core_types__is_Optional(cal, _ctx)) {
      kk_box_t _box_x5569 = cal._cons.Optional.value;
      kk_std_time_calendar__calendar _cal_3221 = kk_std_time_calendar__calendar_unbox(_box_x5569, KK_BORROWED, _ctx);
      kk_std_time_calendar__calendar_dup(_cal_3221, _ctx);
      kk_std_core_types__optional_drop(cal, _ctx);
      _match_5890 = _cal_3221; /*std/time/calendar/calendar*/
    }
    else {
      kk_std_core_types__optional_drop(cal, _ctx);
      _match_5890 = kk_std_time_calendar__calendar_dup(kk_std_time_calendar_cal_iso, _ctx); /*std/time/calendar/calendar*/
    }
    {
      struct kk_std_time_calendar_Calendar* _con6607 = kk_std_time_calendar__as_Calendar(_match_5890, _ctx);
      kk_string_t _pat01 = _con6607->name;
      kk_string_t _pat11 = _con6607->long_name;
      kk_string_t _pat20 = _con6607->month_prefix;
      kk_function_t _pat30 = _con6607->show_era;
      kk_function_t _pat40 = _con6607->instant_to_dc;
      kk_function_t _x = _con6607->dc_to_instant;
      kk_function_t _pat5 = _con6607->days_to_date;
      kk_function_t _pat6 = _con6607->date_to_days;
      if kk_likely(kk_datatype_ptr_is_unique(_match_5890, _ctx)) {
        kk_function_drop(_pat6, _ctx);
        kk_function_drop(_pat5, _ctx);
        kk_function_drop(_pat40, _ctx);
        kk_function_drop(_pat30, _ctx);
        kk_string_drop(_pat20, _ctx);
        kk_string_drop(_pat11, _ctx);
        kk_string_drop(_pat01, _ctx);
        kk_datatype_ptr_free(_match_5890, _ctx);
      }
      else {
        kk_function_dup(_x, _ctx);
        kk_datatype_ptr_decref(_match_5890, _ctx);
      }
      kk_std_time_date__date _x6608;
      kk_integer_t _x6609;
      if (kk_std_core_types__is_Optional(month, _ctx)) {
        kk_box_t _box_x5570 = month._cons.Optional.value;
        kk_integer_t _month_3193 = kk_integer_unbox(_box_x5570, _ctx);
        kk_integer_dup(_month_3193, _ctx);
        kk_std_core_types__optional_drop(month, _ctx);
        _x6609 = _month_3193; /*int*/
      }
      else {
        kk_std_core_types__optional_drop(month, _ctx);
        _x6609 = kk_integer_from_small(1); /*int*/
      }
      kk_integer_t _x6610;
      if (kk_std_core_types__is_Optional(day, _ctx)) {
        kk_box_t _box_x5571 = day._cons.Optional.value;
        kk_integer_t _day_3197 = kk_integer_unbox(_box_x5571, _ctx);
        kk_integer_dup(_day_3197, _ctx);
        kk_std_core_types__optional_drop(day, _ctx);
        _x6610 = _day_3197; /*int*/
      }
      else {
        kk_std_core_types__optional_drop(day, _ctx);
        _x6610 = kk_integer_from_small(1); /*int*/
      }
      _x6608 = kk_std_time_date__new_Date(year, _x6609, _x6610, _ctx); /*std/time/date/date*/
      kk_std_time_date__clock _x6611;
      kk_integer_t _x6612;
      if (kk_std_core_types__is_Optional(hours, _ctx)) {
        kk_box_t _box_x5572 = hours._cons.Optional.value;
        kk_integer_t _hours_3201 = kk_integer_unbox(_box_x5572, _ctx);
        kk_integer_dup(_hours_3201, _ctx);
        kk_std_core_types__optional_drop(hours, _ctx);
        _x6612 = _hours_3201; /*int*/
      }
      else {
        kk_std_core_types__optional_drop(hours, _ctx);
        _x6612 = kk_integer_from_small(0); /*int*/
      }
      kk_integer_t _x6613;
      if (kk_std_core_types__is_Optional(minutes, _ctx)) {
        kk_box_t _box_x5573 = minutes._cons.Optional.value;
        kk_integer_t _minutes_3205 = kk_integer_unbox(_box_x5573, _ctx);
        kk_integer_dup(_minutes_3205, _ctx);
        kk_std_core_types__optional_drop(minutes, _ctx);
        _x6613 = _minutes_3205; /*int*/
      }
      else {
        kk_std_core_types__optional_drop(minutes, _ctx);
        _x6613 = kk_integer_from_small(0); /*int*/
      }
      _x6611 = kk_std_time_date__new_Clock(kk_reuse_null, 0, _x6612, _x6613, fsecs, _ctx); /*std/time/date/clock*/
      kk_std_time_calendar__timezone _x6614;
      if (kk_std_core_types__is_Optional(tz, _ctx)) {
        kk_box_t _box_x5574 = tz._cons.Optional.value;
        kk_std_time_calendar__timezone _tz_3217 = kk_std_time_calendar__timezone_unbox(_box_x5574, KK_BORROWED, _ctx);
        kk_std_time_calendar__timezone_dup(_tz_3217, _ctx);
        kk_std_core_types__optional_drop(tz, _ctx);
        _x6614 = _tz_3217; /*std/time/calendar/timezone*/
      }
      else {
        kk_std_core_types__optional_drop(tz, _ctx);
        _x6614 = kk_std_time_calendar__timezone_dup(kk_std_time_calendar_tz_utc, _ctx); /*std/time/calendar/timezone*/
      }
      x0 = kk_function_call(kk_std_time_instant__instant, (kk_function_t, kk_std_time_date__date, kk_std_time_date__clock, kk_std_time_calendar__timezone, kk_std_time_instant__timescale, kk_context_t*), _x, (_x, _x6608, _x6611, _x6614, x_5155, _ctx), _ctx); /*std/time/instant/instant*/
    }
    kk_unit_t keep = kk_Unit;
    kk_evv_set(w,kk_context());
    return x0;
  }
}
 
// The (proleptic) [Gregorian calendar](https://en.wikipedia.org/wiki/Gregorian_calendar).
// Just like the ISO calendar except that the year 0 does not exist, i.e. after 1 BCE (=`~1`)
// we have 1 CE (=`1`).
// The calendar short name is ``GC``.


// lift anonymous function
struct kk_std_time_calendar_cal_gregorian_fun6620__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_gregorian_fun6620(kk_function_t _fself, kk_integer_t _x16619, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_gregorian_fun6620(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_gregorian_fun6620, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_integer_t kk_std_time_calendar_cal_gregorian_fun6620(kk_function_t _fself, kk_integer_t _x16619, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_days_before_year(_x16619, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_gregorian_fun6621__t {
  struct kk_function_s _base;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_cal_gregorian_fun6621(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_gregorian_fun6621(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_gregorian_fun6621, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_cal_gregorian_fun6621(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx) {
  kk_unused(_fself);
  kk_std_core_types__tuple2_ _match_5884;
  kk_integer_t _x6622 = kk_integer_dup(days, _ctx); /*int*/
  _match_5884 = kk_std_time_calendar_iso_estimate_year(_x6622, _ctx); /*(int, maybe<int>)*/
  {
    kk_box_t _box_x5577 = _match_5884.fst;
    kk_box_t _box_x5578 = _match_5884.snd;
    kk_integer_t approx = kk_integer_unbox(_box_x5577, _ctx);
    kk_std_core_types__maybe safe = kk_std_core_types__maybe_unbox(_box_x5578, KK_BORROWED, _ctx);
    kk_integer_dup(approx, _ctx);
    kk_std_core_types__maybe_dup(safe, _ctx);
    kk_std_core_types__tuple2__drop(_match_5884, _ctx);
    kk_integer_t y_4847;
    kk_integer_t _x6623 = kk_integer_dup(approx, _ctx); /*int*/
    y_4847 = kk_std_time_calendar_iso_days_before_year(_x6623, _ctx); /*int*/
    kk_integer_t doy1;
    kk_integer_t _x6624 = kk_integer_dup(days, _ctx); /*int*/
    doy1 = kk_integer_sub(_x6624,y_4847,kk_context()); /*int*/
    bool _match_5885;
    kk_integer_t _brw_5887;
    if (kk_std_core_types__is_Nothing(safe, _ctx)) {
      _brw_5887 = kk_integer_from_small(0); /*int*/
    }
    else {
      kk_box_t _box_x5579 = safe._cons.Just.value;
      kk_integer_t x0 = kk_integer_unbox(_box_x5579, _ctx);
      kk_integer_dup(x0, _ctx);
      kk_std_core_types__maybe_drop(safe, _ctx);
      _brw_5887 = x0; /*int*/
    }
    bool _brw_5888 = kk_integer_lte_borrow(doy1,_brw_5887,kk_context()); /*bool*/;
    kk_integer_drop(_brw_5887, _ctx);
    _match_5885 = _brw_5888; /*bool*/
    if (_match_5885) {
      kk_integer_drop(days, _ctx);
      return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(approx, _ctx), kk_integer_box(doy1, _ctx), _ctx);
    }
    {
      kk_integer_t year_5161;
      kk_integer_t _x6625 = kk_integer_dup(approx, _ctx); /*int*/
      year_5161 = kk_integer_add_small_const(_x6625, 1, _ctx); /*int*/
      kk_integer_t y = kk_integer_add_small_const(year_5161, -1, _ctx); /*int*/;
      kk_integer_t x0_4671;
      kk_integer_t _x6626 = kk_integer_dup(y, _ctx); /*int*/
      x0_4671 = kk_integer_div(_x6626,(kk_integer_from_small(4)),kk_context()); /*int*/
      kk_integer_t y1_4672;
      kk_integer_t _x6627 = kk_integer_dup(y, _ctx); /*int*/
      y1_4672 = kk_integer_div(_x6627,(kk_integer_from_small(100)),kk_context()); /*int*/
      kk_integer_t x_4669 = kk_integer_sub(x0_4671,y1_4672,kk_context()); /*int*/;
      kk_integer_t y0_4670;
      kk_integer_t _x6628 = kk_integer_dup(y, _ctx); /*int*/
      y0_4670 = kk_integer_div(_x6628,(kk_integer_from_small(400)),kk_context()); /*int*/
      kk_integer_t leapdays = kk_integer_add(x_4669,y0_4670,kk_context()); /*int*/;
      kk_integer_t x1_4673 = kk_integer_mul((kk_integer_from_small(365)),y,kk_context()); /*int*/;
      kk_integer_t y0_4851 = kk_integer_add(x1_4673,leapdays,kk_context()); /*int*/;
      kk_integer_t doy2 = kk_integer_sub(days,y0_4851,kk_context()); /*int*/;
      bool _match_5886 = kk_integer_lt_borrow(doy2,(kk_integer_from_small(0)),kk_context()); /*bool*/;
      if (_match_5886) {
        kk_integer_drop(doy2, _ctx);
        return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(approx, _ctx), kk_integer_box(doy1, _ctx), _ctx);
      }
      {
        kk_integer_drop(doy1, _ctx);
        kk_integer_t _b_5599_5584 = kk_integer_add_small_const(approx, 1, _ctx); /*int*/;
        return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5599_5584, _ctx), kk_integer_box(doy2, _ctx), _ctx);
      }
    }
  }
}


// lift anonymous function
struct kk_std_time_calendar_cal_gregorian_fun6631__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_gregorian_fun6631(kk_function_t _fself, kk_integer_t _x16629, kk_integer_t _x26630, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_gregorian_fun6631(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_gregorian_fun6631, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_integer_t kk_std_time_calendar_cal_gregorian_fun6631(kk_function_t _fself, kk_integer_t _x16629, kk_integer_t _x26630, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_days_before_month(_x16629, _x26630, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_gregorian_fun6634__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_gregorian_fun6634(kk_function_t _fself, kk_integer_t _x16632, kk_integer_t _x26633, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_gregorian_fun6634(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_gregorian_fun6634, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_integer_t kk_std_time_calendar_cal_gregorian_fun6634(kk_function_t _fself, kk_integer_t _x16632, kk_integer_t _x26633, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_doy_to_month(_x16632, _x26633, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_gregorian_fun6644__t {
  struct kk_function_s _base;
};
static kk_box_t kk_std_time_calendar_cal_gregorian_fun6644(kk_function_t _fself, kk_box_t _b_5594, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_gregorian_fun6644(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_gregorian_fun6644, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_box_t kk_std_time_calendar_cal_gregorian_fun6644(kk_function_t _fself, kk_box_t _b_5594, kk_context_t* _ctx) {
  kk_unused(_fself);
  kk_string_t _x6645;
  kk_std_time_date__date d0_5605 = kk_std_time_date__date_unbox(_b_5594, KK_OWNED, _ctx); /*std/time/date/date*/;
  bool _match_5879;
  kk_integer_t _brw_5880;
  {
    kk_integer_t _x = d0_5605.year;
    kk_integer_dup(_x, _ctx);
    kk_std_time_date__date_drop(d0_5605, _ctx);
    _brw_5880 = _x; /*int*/
  }
  bool _brw_5881 = kk_integer_lt_borrow(_brw_5880,(kk_integer_from_small(0)),kk_context()); /*bool*/;
  kk_integer_drop(_brw_5880, _ctx);
  _match_5879 = _brw_5881; /*bool*/
  if (_match_5879) {
    kk_define_string_literal(, _s6646, 3, "BCE", _ctx)
    _x6645 = kk_string_dup(_s6646, _ctx); /*string*/
  }
  else {
    kk_define_string_literal(, _s6647, 2, "CE", _ctx)
    _x6645 = kk_string_dup(_s6647, _ctx); /*string*/
  }
  return kk_string_box(_x6645);
}

kk_std_time_calendar__calendar kk_std_time_calendar_cal_gregorian;
 
/* The standard [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) calendar
but using TI-SLS time (`ts-ti-sls`), i.e. TI with
[smoothed leap seconds](https://www.cl.cam.ac.uk/~mgk25/time/utc-sls/).
This calendar is equivalent to the ISO calendar except for the last 1000 seconds of a day where
a leap second occurs. On such day, the leap second time step (positive or negative)
is distributed over the last 1000 seconds of the day. On the full hour, ISO and ISO-SLS are equal again.
This is a recommended calendar to use for
time stamps or communication with other services since it avoids any potential trouble
with leap seconds while still being quite precise.
Other good properties include:
- All days have 86400 seconds, and the time 23:59:60 never appears
- The time never differs more than one second from UTC
- Time as always equal to UTC on the full and half hour
*/


// lift anonymous function
struct kk_std_time_calendar_cal_iso_sls_fun6653__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_iso_sls_fun6653(kk_function_t _fself, kk_integer_t _x16652, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_sls_fun6653(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_iso_sls_fun6653, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_integer_t kk_std_time_calendar_cal_iso_sls_fun6653(kk_function_t _fself, kk_integer_t _x16652, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_days_before_year(_x16652, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_iso_sls_fun6654__t {
  struct kk_function_s _base;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_cal_iso_sls_fun6654(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_sls_fun6654(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_iso_sls_fun6654, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_cal_iso_sls_fun6654(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx) {
  kk_unused(_fself);
  kk_std_core_types__tuple2_ _match_5874;
  kk_integer_t _x6655 = kk_integer_dup(days, _ctx); /*int*/
  _match_5874 = kk_std_time_calendar_iso_estimate_year(_x6655, _ctx); /*(int, maybe<int>)*/
  {
    kk_box_t _box_x5606 = _match_5874.fst;
    kk_box_t _box_x5607 = _match_5874.snd;
    kk_integer_t approx = kk_integer_unbox(_box_x5606, _ctx);
    kk_std_core_types__maybe safe = kk_std_core_types__maybe_unbox(_box_x5607, KK_BORROWED, _ctx);
    kk_integer_dup(approx, _ctx);
    kk_std_core_types__maybe_dup(safe, _ctx);
    kk_std_core_types__tuple2__drop(_match_5874, _ctx);
    kk_integer_t y_4847;
    kk_integer_t _x6656 = kk_integer_dup(approx, _ctx); /*int*/
    y_4847 = kk_std_time_calendar_iso_days_before_year(_x6656, _ctx); /*int*/
    kk_integer_t doy1;
    kk_integer_t _x6657 = kk_integer_dup(days, _ctx); /*int*/
    doy1 = kk_integer_sub(_x6657,y_4847,kk_context()); /*int*/
    bool _match_5875;
    kk_integer_t _brw_5877;
    if (kk_std_core_types__is_Nothing(safe, _ctx)) {
      _brw_5877 = kk_integer_from_small(0); /*int*/
    }
    else {
      kk_box_t _box_x5608 = safe._cons.Just.value;
      kk_integer_t x0 = kk_integer_unbox(_box_x5608, _ctx);
      kk_integer_dup(x0, _ctx);
      kk_std_core_types__maybe_drop(safe, _ctx);
      _brw_5877 = x0; /*int*/
    }
    bool _brw_5878 = kk_integer_lte_borrow(doy1,_brw_5877,kk_context()); /*bool*/;
    kk_integer_drop(_brw_5877, _ctx);
    _match_5875 = _brw_5878; /*bool*/
    if (_match_5875) {
      kk_integer_drop(days, _ctx);
      return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(approx, _ctx), kk_integer_box(doy1, _ctx), _ctx);
    }
    {
      kk_integer_t year_5162;
      kk_integer_t _x6658 = kk_integer_dup(approx, _ctx); /*int*/
      year_5162 = kk_integer_add_small_const(_x6658, 1, _ctx); /*int*/
      kk_integer_t y = kk_integer_add_small_const(year_5162, -1, _ctx); /*int*/;
      kk_integer_t x0_4671;
      kk_integer_t _x6659 = kk_integer_dup(y, _ctx); /*int*/
      x0_4671 = kk_integer_div(_x6659,(kk_integer_from_small(4)),kk_context()); /*int*/
      kk_integer_t y1_4672;
      kk_integer_t _x6660 = kk_integer_dup(y, _ctx); /*int*/
      y1_4672 = kk_integer_div(_x6660,(kk_integer_from_small(100)),kk_context()); /*int*/
      kk_integer_t x_4669 = kk_integer_sub(x0_4671,y1_4672,kk_context()); /*int*/;
      kk_integer_t y0_4670;
      kk_integer_t _x6661 = kk_integer_dup(y, _ctx); /*int*/
      y0_4670 = kk_integer_div(_x6661,(kk_integer_from_small(400)),kk_context()); /*int*/
      kk_integer_t leapdays = kk_integer_add(x_4669,y0_4670,kk_context()); /*int*/;
      kk_integer_t x1_4673 = kk_integer_mul((kk_integer_from_small(365)),y,kk_context()); /*int*/;
      kk_integer_t y0_4851 = kk_integer_add(x1_4673,leapdays,kk_context()); /*int*/;
      kk_integer_t doy2 = kk_integer_sub(days,y0_4851,kk_context()); /*int*/;
      bool _match_5876 = kk_integer_lt_borrow(doy2,(kk_integer_from_small(0)),kk_context()); /*bool*/;
      if (_match_5876) {
        kk_integer_drop(doy2, _ctx);
        return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(approx, _ctx), kk_integer_box(doy1, _ctx), _ctx);
      }
      {
        kk_integer_drop(doy1, _ctx);
        kk_integer_t _b_5636_5613 = kk_integer_add_small_const(approx, 1, _ctx); /*int*/;
        return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5636_5613, _ctx), kk_integer_box(doy2, _ctx), _ctx);
      }
    }
  }
}


// lift anonymous function
struct kk_std_time_calendar_cal_iso_sls_fun6664__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_iso_sls_fun6664(kk_function_t _fself, kk_integer_t _x16662, kk_integer_t _x26663, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_sls_fun6664(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_iso_sls_fun6664, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_integer_t kk_std_time_calendar_cal_iso_sls_fun6664(kk_function_t _fself, kk_integer_t _x16662, kk_integer_t _x26663, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_days_before_month(_x16662, _x26663, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_iso_sls_fun6667__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_iso_sls_fun6667(kk_function_t _fself, kk_integer_t _x16665, kk_integer_t _x26666, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_sls_fun6667(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_iso_sls_fun6667, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_integer_t kk_std_time_calendar_cal_iso_sls_fun6667(kk_function_t _fself, kk_integer_t _x16665, kk_integer_t _x26666, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_doy_to_month(_x16665, _x26666, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_iso_sls_fun6679__t {
  struct kk_function_s _base;
};
static kk_box_t kk_std_time_calendar_cal_iso_sls_fun6679(kk_function_t _fself, kk_box_t _b_5631, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_sls_fun6679(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_iso_sls_fun6679, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_box_t kk_std_time_calendar_cal_iso_sls_fun6679(kk_function_t _fself, kk_box_t _b_5631, kk_context_t* _ctx) {
  kk_unused(_fself);
  kk_string_t _x6680;
  kk_std_time_date__date _norm_5642 = kk_std_time_date__date_unbox(_b_5631, KK_OWNED, _ctx); /*std/time/date/date*/;
  kk_std_core_types__optional _match_5870 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5870, _ctx)) {
    kk_box_t _fun_unbox_x5624 = _match_5870._cons.Optional.value;
    kk_box_dup(_fun_unbox_x5624, _ctx);
    kk_std_core_types__optional_drop(_match_5870, _ctx);
    kk_std_time_date__date _b_5643_5627 = _norm_5642; /*std/time/date/date*/;
    kk_box_t _x6681;
    kk_function_t _x6682 = kk_function_unbox(_fun_unbox_x5624, _ctx); /*(5625) -> 5626*/
    _x6681 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6682, (_x6682, kk_std_time_date__date_box(_b_5643_5627, _ctx), _ctx), _ctx); /*5626*/
    _x6680 = kk_string_unbox(_x6681); /*string*/
  }
  else {
    kk_std_core_types__optional_drop(_match_5870, _ctx);
    kk_std_time_date__date d_5644 = _norm_5642; /*std/time/date/date*/;
    kk_std_time_date__date_drop(d_5644, _ctx);
    _x6680 = kk_string_empty(); /*string*/
  }
  return kk_string_box(_x6680);
}

kk_std_time_calendar__calendar kk_std_time_calendar_cal_iso_sls;

kk_std_time_timestamp__timestamp kk_std_time_calendar_cal_timestamp(kk_std_time_calendar__calendar cal, kk_std_time_instant__timescale ts, kk_std_time_date__date d, kk_std_time_date__clock c, kk_context_t* _ctx) { /* (cal : calendar, ts : std/time/instant/timescale, d : std/time/date/date, c : std/time/date/clock) -> std/time/timestamp/timestamp */ 
  kk_function_t _x6684;
  {
    struct kk_std_time_calendar_Calendar* _con6685 = kk_std_time_calendar__as_Calendar(cal, _ctx);
    kk_string_t _pat0 = _con6685->name;
    kk_string_t _pat1 = _con6685->long_name;
    kk_string_t _pat2 = _con6685->month_prefix;
    kk_function_t _pat3 = _con6685->show_era;
    kk_function_t _pat4 = _con6685->instant_to_dc;
    kk_function_t _pat5 = _con6685->dc_to_instant;
    kk_function_t _pat6 = _con6685->days_to_date;
    kk_function_t _x = _con6685->date_to_days;
    if kk_likely(kk_datatype_ptr_is_unique(cal, _ctx)) {
      kk_function_drop(_pat6, _ctx);
      kk_function_drop(_pat5, _ctx);
      kk_function_drop(_pat4, _ctx);
      kk_function_drop(_pat3, _ctx);
      kk_string_drop(_pat2, _ctx);
      kk_string_drop(_pat1, _ctx);
      kk_string_drop(_pat0, _ctx);
      kk_datatype_ptr_free(cal, _ctx);
    }
    else {
      kk_function_dup(_x, _ctx);
      kk_datatype_ptr_decref(cal, _ctx);
    }
    _x6684 = _x; /*(date : std/time/date/date) -> int*/
  }
  kk_std_core_types__optional _x6686;
  kk_box_t _x6687;
  bool _x6688;
  {
    struct kk_std_time_instant_Timescale* _con6689 = kk_std_time_instant__as_Timescale(ts, _ctx);
    kk_string_t _pat00 = _con6689->name;
    kk_string_t _pat100 = _con6689->unit;
    kk_function_t _pat20 = _con6689->from_tai;
    kk_function_t _pat30 = _con6689->to_tai;
    kk_std_core_types__maybe _x0 = _con6689->mb_seconds_in_day;
    kk_std_core_types__maybe _pat40 = _con6689->mb_to_mjd2000;
    kk_std_core_types__maybe _pat50 = _con6689->mb_from_mjd2000;
    if kk_likely(kk_datatype_ptr_is_unique(ts, _ctx)) {
      kk_std_core_types__maybe_drop(_pat50, _ctx);
      kk_std_core_types__maybe_drop(_pat40, _ctx);
      kk_function_drop(_pat30, _ctx);
      kk_function_drop(_pat20, _ctx);
      kk_string_drop(_pat100, _ctx);
      kk_string_drop(_pat00, _ctx);
      kk_datatype_ptr_free(ts, _ctx);
    }
    else {
      kk_std_core_types__maybe_dup(_x0, _ctx);
      kk_datatype_ptr_decref(ts, _ctx);
    }
    if (kk_std_core_types__is_Nothing(_x0, _ctx)) {
      _x6688 = false; /*bool*/
    }
    else {
      kk_std_core_types__maybe_drop(_x0, _ctx);
      _x6688 = true; /*bool*/
    }
  }
  _x6687 = kk_bool_box(_x6688); /*116*/
  _x6686 = kk_std_core_types__new_Optional(_x6687, _ctx); /*optional<116>*/
  return kk_std_time_calendar_dc_timestamp(d, c, _x6684, _x6686, _ctx);
}
 
// Create a standard [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) calendar
// using UTC time (`ts-utc-create`) given a provide leap second table (`:leaps-table`).
// This is a proleptic Gregorian
// calendar except that it uses the year 0 for 1 BCE, -1 for 2 BCE etc.
//
// You can create an ISO 8601 UTC calendar using the latest IETF leap
// second data using [`cal-utc-load`](std_time_download.html#cal_utc_load).


// lift anonymous function
struct kk_std_time_calendar_cal_utc_create_fun6695__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_utc_create_fun6695(kk_function_t _fself, kk_integer_t _x16694, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_utc_create_fun6695(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_utc_create_fun6695, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_integer_t kk_std_time_calendar_cal_utc_create_fun6695(kk_function_t _fself, kk_integer_t _x16694, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_days_before_year(_x16694, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_utc_create_fun6696__t {
  struct kk_function_s _base;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_cal_utc_create_fun6696(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_utc_create_fun6696(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_utc_create_fun6696, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_cal_utc_create_fun6696(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx) {
  kk_unused(_fself);
  kk_std_core_types__tuple2_ _match_5865;
  kk_integer_t _x6697 = kk_integer_dup(days, _ctx); /*int*/
  _match_5865 = kk_std_time_calendar_iso_estimate_year(_x6697, _ctx); /*(int, maybe<int>)*/
  {
    kk_box_t _box_x5647 = _match_5865.fst;
    kk_box_t _box_x5648 = _match_5865.snd;
    kk_integer_t approx = kk_integer_unbox(_box_x5647, _ctx);
    kk_std_core_types__maybe safe = kk_std_core_types__maybe_unbox(_box_x5648, KK_BORROWED, _ctx);
    kk_integer_dup(approx, _ctx);
    kk_std_core_types__maybe_dup(safe, _ctx);
    kk_std_core_types__tuple2__drop(_match_5865, _ctx);
    kk_integer_t y_4847;
    kk_integer_t _x6698 = kk_integer_dup(approx, _ctx); /*int*/
    y_4847 = kk_std_time_calendar_iso_days_before_year(_x6698, _ctx); /*int*/
    kk_integer_t doy1;
    kk_integer_t _x6699 = kk_integer_dup(days, _ctx); /*int*/
    doy1 = kk_integer_sub(_x6699,y_4847,kk_context()); /*int*/
    bool _match_5866;
    kk_integer_t _brw_5868;
    if (kk_std_core_types__is_Nothing(safe, _ctx)) {
      _brw_5868 = kk_integer_from_small(0); /*int*/
    }
    else {
      kk_box_t _box_x5649 = safe._cons.Just.value;
      kk_integer_t x0 = kk_integer_unbox(_box_x5649, _ctx);
      kk_integer_dup(x0, _ctx);
      kk_std_core_types__maybe_drop(safe, _ctx);
      _brw_5868 = x0; /*int*/
    }
    bool _brw_5869 = kk_integer_lte_borrow(doy1,_brw_5868,kk_context()); /*bool*/;
    kk_integer_drop(_brw_5868, _ctx);
    _match_5866 = _brw_5869; /*bool*/
    if (_match_5866) {
      kk_integer_drop(days, _ctx);
      return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(approx, _ctx), kk_integer_box(doy1, _ctx), _ctx);
    }
    {
      kk_integer_t year_5163;
      kk_integer_t _x6700 = kk_integer_dup(approx, _ctx); /*int*/
      year_5163 = kk_integer_add_small_const(_x6700, 1, _ctx); /*int*/
      kk_integer_t y = kk_integer_add_small_const(year_5163, -1, _ctx); /*int*/;
      kk_integer_t x0_4671;
      kk_integer_t _x6701 = kk_integer_dup(y, _ctx); /*int*/
      x0_4671 = kk_integer_div(_x6701,(kk_integer_from_small(4)),kk_context()); /*int*/
      kk_integer_t y1_4672;
      kk_integer_t _x6702 = kk_integer_dup(y, _ctx); /*int*/
      y1_4672 = kk_integer_div(_x6702,(kk_integer_from_small(100)),kk_context()); /*int*/
      kk_integer_t x_4669 = kk_integer_sub(x0_4671,y1_4672,kk_context()); /*int*/;
      kk_integer_t y0_4670;
      kk_integer_t _x6703 = kk_integer_dup(y, _ctx); /*int*/
      y0_4670 = kk_integer_div(_x6703,(kk_integer_from_small(400)),kk_context()); /*int*/
      kk_integer_t leapdays = kk_integer_add(x_4669,y0_4670,kk_context()); /*int*/;
      kk_integer_t x1_4673 = kk_integer_mul((kk_integer_from_small(365)),y,kk_context()); /*int*/;
      kk_integer_t y0_4851 = kk_integer_add(x1_4673,leapdays,kk_context()); /*int*/;
      kk_integer_t doy2 = kk_integer_sub(days,y0_4851,kk_context()); /*int*/;
      bool _match_5867 = kk_integer_lt_borrow(doy2,(kk_integer_from_small(0)),kk_context()); /*bool*/;
      if (_match_5867) {
        kk_integer_drop(doy2, _ctx);
        return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(approx, _ctx), kk_integer_box(doy1, _ctx), _ctx);
      }
      {
        kk_integer_drop(doy1, _ctx);
        kk_integer_t _b_5677_5654 = kk_integer_add_small_const(approx, 1, _ctx); /*int*/;
        return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5677_5654, _ctx), kk_integer_box(doy2, _ctx), _ctx);
      }
    }
  }
}


// lift anonymous function
struct kk_std_time_calendar_cal_utc_create_fun6706__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_utc_create_fun6706(kk_function_t _fself, kk_integer_t _x16704, kk_integer_t _x26705, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_utc_create_fun6706(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_utc_create_fun6706, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_integer_t kk_std_time_calendar_cal_utc_create_fun6706(kk_function_t _fself, kk_integer_t _x16704, kk_integer_t _x26705, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_days_before_month(_x16704, _x26705, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_utc_create_fun6709__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_utc_create_fun6709(kk_function_t _fself, kk_integer_t _x16707, kk_integer_t _x26708, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_utc_create_fun6709(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_utc_create_fun6709, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_integer_t kk_std_time_calendar_cal_utc_create_fun6709(kk_function_t _fself, kk_integer_t _x16707, kk_integer_t _x26708, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_doy_to_month(_x16707, _x26708, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_utc_create_fun6721__t {
  struct kk_function_s _base;
};
static kk_box_t kk_std_time_calendar_cal_utc_create_fun6721(kk_function_t _fself, kk_box_t _b_5672, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_utc_create_fun6721(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_utc_create_fun6721, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_box_t kk_std_time_calendar_cal_utc_create_fun6721(kk_function_t _fself, kk_box_t _b_5672, kk_context_t* _ctx) {
  kk_unused(_fself);
  kk_string_t _x6722;
  kk_std_time_date__date _norm_5683 = kk_std_time_date__date_unbox(_b_5672, KK_OWNED, _ctx); /*std/time/date/date*/;
  kk_std_core_types__optional _match_5861 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5861, _ctx)) {
    kk_box_t _fun_unbox_x5665 = _match_5861._cons.Optional.value;
    kk_box_dup(_fun_unbox_x5665, _ctx);
    kk_std_core_types__optional_drop(_match_5861, _ctx);
    kk_std_time_date__date _b_5684_5668 = _norm_5683; /*std/time/date/date*/;
    kk_box_t _x6723;
    kk_function_t _x6724 = kk_function_unbox(_fun_unbox_x5665, _ctx); /*(5666) -> 5667*/
    _x6723 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6724, (_x6724, kk_std_time_date__date_box(_b_5684_5668, _ctx), _ctx), _ctx); /*5667*/
    _x6722 = kk_string_unbox(_x6723); /*string*/
  }
  else {
    kk_std_core_types__optional_drop(_match_5861, _ctx);
    kk_std_time_date__date d_5685 = _norm_5683; /*std/time/date/date*/;
    kk_std_time_date__date_drop(d_5685, _ctx);
    _x6722 = kk_string_empty(); /*string*/
  }
  return kk_string_box(_x6722);
}

kk_std_time_calendar__calendar kk_std_time_calendar_cal_utc_create(kk_std_time_utc__leaps_table leaps, kk_context_t* _ctx) { /* (leaps : std/time/utc/leaps-table) -> calendar */ 
  kk_std_time_instant__timescale ts_utc = kk_std_time_utc_ts_utc_create(leaps, _ctx); /*std/time/instant/timescale*/;
  kk_datatype_ptr_dropn(ts_utc, (KK_I32(10)), _ctx);
  kk_string_t _x6690 = kk_string_empty(); /*string*/
  kk_string_t _x6692;
  kk_define_string_literal(, _s6693, 3, "ISO", _ctx)
  _x6692 = kk_string_dup(_s6693, _ctx); /*string*/
  kk_std_core_types__optional _x6710;
  kk_box_t _x6711;
  kk_integer_t _x6712;
  kk_std_core_types__optional _match_5864 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5864, _ctx)) {
    kk_box_t _box_x5656 = _match_5864._cons.Optional.value;
    kk_integer_t _epoch_shift_2777 = kk_integer_unbox(_box_x5656, _ctx);
    kk_integer_dup(_epoch_shift_2777, _ctx);
    kk_std_core_types__optional_drop(_match_5864, _ctx);
    _x6712 = _epoch_shift_2777; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(_match_5864, _ctx);
    _x6712 = kk_integer_from_int(730119, _ctx); /*int*/
  }
  _x6711 = kk_integer_box(_x6712, _ctx); /*116*/
  _x6710 = kk_std_core_types__new_Optional(_x6711, _ctx); /*optional<116>*/
  kk_std_core_types__optional _x6713;
  kk_box_t _x6714;
  bool _x6715;
  kk_std_core_types__optional _match_5863 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5863, _ctx)) {
    kk_box_t _box_x5658 = _match_5863._cons.Optional.value;
    bool _has_year_zero_2781 = kk_bool_unbox(_box_x5658);
    kk_std_core_types__optional_drop(_match_5863, _ctx);
    _x6715 = _has_year_zero_2781; /*bool*/
  }
  else {
    kk_std_core_types__optional_drop(_match_5863, _ctx);
    _x6715 = true; /*bool*/
  }
  _x6714 = kk_bool_box(_x6715); /*116*/
  _x6713 = kk_std_core_types__new_Optional(_x6714, _ctx); /*optional<116>*/
  kk_std_core_types__optional _x6716;
  kk_box_t _x6717;
  kk_string_t _x6718;
  kk_std_core_types__optional _match_5862 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5862, _ctx)) {
    kk_box_t _box_x5660 = _match_5862._cons.Optional.value;
    kk_string_t _month_prefix_2785 = kk_string_unbox(_box_x5660);
    kk_string_dup(_month_prefix_2785, _ctx);
    kk_std_core_types__optional_drop(_match_5862, _ctx);
    _x6718 = _month_prefix_2785; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(_match_5862, _ctx);
    _x6718 = kk_string_empty(); /*string*/
  }
  _x6717 = kk_string_box(_x6718); /*116*/
  _x6716 = kk_std_core_types__new_Optional(_x6717, _ctx); /*optional<116>*/
  kk_std_core_types__optional _x6720 = kk_std_core_types__new_Optional(kk_function_box(kk_std_time_calendar_new_cal_utc_create_fun6721(_ctx), _ctx), _ctx); /*optional<116>*/
  return kk_std_time_calendar_solar_calendar(_x6690, _x6692, kk_std_time_calendar_new_cal_utc_create_fun6695(_ctx), kk_std_time_calendar_new_cal_utc_create_fun6696(_ctx), kk_std_time_calendar_new_cal_utc_create_fun6706(_ctx), kk_std_time_calendar_new_cal_utc_create_fun6709(_ctx), _x6710, _x6713, _x6716, _x6720, _ctx);
}
 
// Create a new ISO 8601 calendar based on UTC-SLS time, i.e. UTC with smoothed leap
// seconds.


// lift anonymous function
struct kk_std_time_calendar_cal_utc_sls_create_fun6731__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_utc_sls_create_fun6731(kk_function_t _fself, kk_integer_t _x16730, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_utc_sls_create_fun6731(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_utc_sls_create_fun6731, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_integer_t kk_std_time_calendar_cal_utc_sls_create_fun6731(kk_function_t _fself, kk_integer_t _x16730, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_days_before_year(_x16730, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_utc_sls_create_fun6732__t {
  struct kk_function_s _base;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_cal_utc_sls_create_fun6732(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_utc_sls_create_fun6732(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_utc_sls_create_fun6732, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_cal_utc_sls_create_fun6732(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx) {
  kk_unused(_fself);
  kk_std_core_types__tuple2_ _match_5856;
  kk_integer_t _x6733 = kk_integer_dup(days, _ctx); /*int*/
  _match_5856 = kk_std_time_calendar_iso_estimate_year(_x6733, _ctx); /*(int, maybe<int>)*/
  {
    kk_box_t _box_x5686 = _match_5856.fst;
    kk_box_t _box_x5687 = _match_5856.snd;
    kk_integer_t approx = kk_integer_unbox(_box_x5686, _ctx);
    kk_std_core_types__maybe safe = kk_std_core_types__maybe_unbox(_box_x5687, KK_BORROWED, _ctx);
    kk_integer_dup(approx, _ctx);
    kk_std_core_types__maybe_dup(safe, _ctx);
    kk_std_core_types__tuple2__drop(_match_5856, _ctx);
    kk_integer_t y_4847;
    kk_integer_t _x6734 = kk_integer_dup(approx, _ctx); /*int*/
    y_4847 = kk_std_time_calendar_iso_days_before_year(_x6734, _ctx); /*int*/
    kk_integer_t doy1;
    kk_integer_t _x6735 = kk_integer_dup(days, _ctx); /*int*/
    doy1 = kk_integer_sub(_x6735,y_4847,kk_context()); /*int*/
    bool _match_5857;
    kk_integer_t _brw_5859;
    if (kk_std_core_types__is_Nothing(safe, _ctx)) {
      _brw_5859 = kk_integer_from_small(0); /*int*/
    }
    else {
      kk_box_t _box_x5688 = safe._cons.Just.value;
      kk_integer_t x0 = kk_integer_unbox(_box_x5688, _ctx);
      kk_integer_dup(x0, _ctx);
      kk_std_core_types__maybe_drop(safe, _ctx);
      _brw_5859 = x0; /*int*/
    }
    bool _brw_5860 = kk_integer_lte_borrow(doy1,_brw_5859,kk_context()); /*bool*/;
    kk_integer_drop(_brw_5859, _ctx);
    _match_5857 = _brw_5860; /*bool*/
    if (_match_5857) {
      kk_integer_drop(days, _ctx);
      return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(approx, _ctx), kk_integer_box(doy1, _ctx), _ctx);
    }
    {
      kk_integer_t year_5164;
      kk_integer_t _x6736 = kk_integer_dup(approx, _ctx); /*int*/
      year_5164 = kk_integer_add_small_const(_x6736, 1, _ctx); /*int*/
      kk_integer_t y = kk_integer_add_small_const(year_5164, -1, _ctx); /*int*/;
      kk_integer_t x0_4671;
      kk_integer_t _x6737 = kk_integer_dup(y, _ctx); /*int*/
      x0_4671 = kk_integer_div(_x6737,(kk_integer_from_small(4)),kk_context()); /*int*/
      kk_integer_t y1_4672;
      kk_integer_t _x6738 = kk_integer_dup(y, _ctx); /*int*/
      y1_4672 = kk_integer_div(_x6738,(kk_integer_from_small(100)),kk_context()); /*int*/
      kk_integer_t x_4669 = kk_integer_sub(x0_4671,y1_4672,kk_context()); /*int*/;
      kk_integer_t y0_4670;
      kk_integer_t _x6739 = kk_integer_dup(y, _ctx); /*int*/
      y0_4670 = kk_integer_div(_x6739,(kk_integer_from_small(400)),kk_context()); /*int*/
      kk_integer_t leapdays = kk_integer_add(x_4669,y0_4670,kk_context()); /*int*/;
      kk_integer_t x1_4673 = kk_integer_mul((kk_integer_from_small(365)),y,kk_context()); /*int*/;
      kk_integer_t y0_4851 = kk_integer_add(x1_4673,leapdays,kk_context()); /*int*/;
      kk_integer_t doy2 = kk_integer_sub(days,y0_4851,kk_context()); /*int*/;
      bool _match_5858 = kk_integer_lt_borrow(doy2,(kk_integer_from_small(0)),kk_context()); /*bool*/;
      if (_match_5858) {
        kk_integer_drop(doy2, _ctx);
        return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(approx, _ctx), kk_integer_box(doy1, _ctx), _ctx);
      }
      {
        kk_integer_drop(doy1, _ctx);
        kk_integer_t _b_5716_5693 = kk_integer_add_small_const(approx, 1, _ctx); /*int*/;
        return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5716_5693, _ctx), kk_integer_box(doy2, _ctx), _ctx);
      }
    }
  }
}


// lift anonymous function
struct kk_std_time_calendar_cal_utc_sls_create_fun6742__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_utc_sls_create_fun6742(kk_function_t _fself, kk_integer_t _x16740, kk_integer_t _x26741, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_utc_sls_create_fun6742(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_utc_sls_create_fun6742, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_integer_t kk_std_time_calendar_cal_utc_sls_create_fun6742(kk_function_t _fself, kk_integer_t _x16740, kk_integer_t _x26741, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_days_before_month(_x16740, _x26741, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_utc_sls_create_fun6745__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_utc_sls_create_fun6745(kk_function_t _fself, kk_integer_t _x16743, kk_integer_t _x26744, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_utc_sls_create_fun6745(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_utc_sls_create_fun6745, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_integer_t kk_std_time_calendar_cal_utc_sls_create_fun6745(kk_function_t _fself, kk_integer_t _x16743, kk_integer_t _x26744, kk_context_t* _ctx) {
  kk_unused(_fself);
  return kk_std_time_calendar_iso_doy_to_month(_x16743, _x26744, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_utc_sls_create_fun6757__t {
  struct kk_function_s _base;
};
static kk_box_t kk_std_time_calendar_cal_utc_sls_create_fun6757(kk_function_t _fself, kk_box_t _b_5711, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_utc_sls_create_fun6757(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_utc_sls_create_fun6757, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_box_t kk_std_time_calendar_cal_utc_sls_create_fun6757(kk_function_t _fself, kk_box_t _b_5711, kk_context_t* _ctx) {
  kk_unused(_fself);
  kk_string_t _x6758;
  kk_std_time_date__date _norm_5722 = kk_std_time_date__date_unbox(_b_5711, KK_OWNED, _ctx); /*std/time/date/date*/;
  kk_std_core_types__optional _match_5852 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5852, _ctx)) {
    kk_box_t _fun_unbox_x5704 = _match_5852._cons.Optional.value;
    kk_box_dup(_fun_unbox_x5704, _ctx);
    kk_std_core_types__optional_drop(_match_5852, _ctx);
    kk_std_time_date__date _b_5723_5707 = _norm_5722; /*std/time/date/date*/;
    kk_box_t _x6759;
    kk_function_t _x6760 = kk_function_unbox(_fun_unbox_x5704, _ctx); /*(5705) -> 5706*/
    _x6759 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6760, (_x6760, kk_std_time_date__date_box(_b_5723_5707, _ctx), _ctx), _ctx); /*5706*/
    _x6758 = kk_string_unbox(_x6759); /*string*/
  }
  else {
    kk_std_core_types__optional_drop(_match_5852, _ctx);
    kk_std_time_date__date d_5724 = _norm_5722; /*std/time/date/date*/;
    kk_std_time_date__date_drop(d_5724, _ctx);
    _x6758 = kk_string_empty(); /*string*/
  }
  return kk_string_box(_x6758);
}

kk_std_time_calendar__calendar kk_std_time_calendar_cal_utc_sls_create(kk_std_time_utc__leaps_table leaps, kk_context_t* _ctx) { /* (leaps : std/time/utc/leaps-table) -> calendar */ 
  kk_std_time_instant__timescale ts_utc_sls = kk_std_time_utc_ts_utc_sls_create(leaps, _ctx); /*std/time/instant/timescale*/;
  kk_datatype_ptr_dropn(ts_utc_sls, (KK_I32(10)), _ctx);
  kk_string_t _x6726;
  kk_define_string_literal(, _s6727, 3, "SLS", _ctx)
  _x6726 = kk_string_dup(_s6727, _ctx); /*string*/
  kk_string_t _x6728;
  kk_define_string_literal(, _s6729, 7, "ISO-SLS", _ctx)
  _x6728 = kk_string_dup(_s6729, _ctx); /*string*/
  kk_std_core_types__optional _x6746;
  kk_box_t _x6747;
  kk_integer_t _x6748;
  kk_std_core_types__optional _match_5855 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5855, _ctx)) {
    kk_box_t _box_x5695 = _match_5855._cons.Optional.value;
    kk_integer_t _epoch_shift_2777 = kk_integer_unbox(_box_x5695, _ctx);
    kk_integer_dup(_epoch_shift_2777, _ctx);
    kk_std_core_types__optional_drop(_match_5855, _ctx);
    _x6748 = _epoch_shift_2777; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(_match_5855, _ctx);
    _x6748 = kk_integer_from_int(730119, _ctx); /*int*/
  }
  _x6747 = kk_integer_box(_x6748, _ctx); /*116*/
  _x6746 = kk_std_core_types__new_Optional(_x6747, _ctx); /*optional<116>*/
  kk_std_core_types__optional _x6749;
  kk_box_t _x6750;
  bool _x6751;
  kk_std_core_types__optional _match_5854 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5854, _ctx)) {
    kk_box_t _box_x5697 = _match_5854._cons.Optional.value;
    bool _has_year_zero_2781 = kk_bool_unbox(_box_x5697);
    kk_std_core_types__optional_drop(_match_5854, _ctx);
    _x6751 = _has_year_zero_2781; /*bool*/
  }
  else {
    kk_std_core_types__optional_drop(_match_5854, _ctx);
    _x6751 = true; /*bool*/
  }
  _x6750 = kk_bool_box(_x6751); /*116*/
  _x6749 = kk_std_core_types__new_Optional(_x6750, _ctx); /*optional<116>*/
  kk_std_core_types__optional _x6752;
  kk_box_t _x6753;
  kk_string_t _x6754;
  kk_std_core_types__optional _match_5853 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5853, _ctx)) {
    kk_box_t _box_x5699 = _match_5853._cons.Optional.value;
    kk_string_t _month_prefix_2785 = kk_string_unbox(_box_x5699);
    kk_string_dup(_month_prefix_2785, _ctx);
    kk_std_core_types__optional_drop(_match_5853, _ctx);
    _x6754 = _month_prefix_2785; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(_match_5853, _ctx);
    _x6754 = kk_string_empty(); /*string*/
  }
  _x6753 = kk_string_box(_x6754); /*116*/
  _x6752 = kk_std_core_types__new_Optional(_x6753, _ctx); /*optional<116>*/
  kk_std_core_types__optional _x6756 = kk_std_core_types__new_Optional(kk_function_box(kk_std_time_calendar_new_cal_utc_sls_create_fun6757(_ctx), _ctx), _ctx); /*optional<116>*/
  return kk_std_time_calendar_solar_calendar(_x6726, _x6728, kk_std_time_calendar_new_cal_utc_sls_create_fun6731(_ctx), kk_std_time_calendar_new_cal_utc_sls_create_fun6732(_ctx), kk_std_time_calendar_new_cal_utc_sls_create_fun6742(_ctx), kk_std_time_calendar_new_cal_utc_sls_create_fun6745(_ctx), _x6746, _x6749, _x6752, _x6756, _ctx);
}
 
// Create a new calendar from by combining two other calendars. The `switch-date`
// is in terms of the second calendar (`cal2`) and time after (and including) the switch
// date is displayed in `cal2` while times before it in `cal1`. This function is used
// for example for the Julian Gregorian calendar.


// lift anonymous function
struct kk_std_time_calendar_combine_earth_calendars_fun6762__t {
  struct kk_function_s _base;
  kk_std_time_calendar__calendar cal1;
  kk_std_time_calendar__calendar cal2;
  kk_std_time_date__date switch_date;
};
static kk_std_time_date__date kk_std_time_calendar_combine_earth_calendars_fun6762(kk_function_t _fself, kk_integer_t days0, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_combine_earth_calendars_fun6762(kk_std_time_calendar__calendar cal1, kk_std_time_calendar__calendar cal2, kk_std_time_date__date switch_date, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun6762__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_combine_earth_calendars_fun6762__t, 6, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_combine_earth_calendars_fun6762, kk_context());
  _self->cal1 = cal1;
  _self->cal2 = cal2;
  _self->switch_date = switch_date;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_time_date__date kk_std_time_calendar_combine_earth_calendars_fun6762(kk_function_t _fself, kk_integer_t days0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun6762__t* _self = kk_function_as(struct kk_std_time_calendar_combine_earth_calendars_fun6762__t*, _fself, _ctx);
  kk_std_time_calendar__calendar cal1 = _self->cal1; /* std/time/calendar/calendar */
  kk_std_time_calendar__calendar cal2 = _self->cal2; /* std/time/calendar/calendar */
  kk_std_time_date__date switch_date = _self->switch_date; /* std/time/date/date */
  kk_drop_match(_self, {kk_std_time_calendar__calendar_dup(cal1, _ctx);kk_std_time_calendar__calendar_dup(cal2, _ctx);kk_std_time_date__date_dup(switch_date, _ctx);}, {}, _ctx)
  kk_std_time_calendar__calendar calendar0_4896;
  bool _match_5849;
  kk_integer_t _brw_5850;
  {
    struct kk_std_time_calendar_Calendar* _con6763 = kk_std_time_calendar__as_Calendar(cal2, _ctx);
    kk_function_t _x = _con6763->date_to_days;
    kk_function_dup(_x, _ctx);
    _brw_5850 = kk_function_call(kk_integer_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _x, (_x, switch_date, _ctx), _ctx); /*int*/
  }
  bool _brw_5851 = kk_integer_lt_borrow(days0,_brw_5850,kk_context()); /*bool*/;
  kk_integer_drop(_brw_5850, _ctx);
  _match_5849 = _brw_5851; /*bool*/
  if (_match_5849) {
    kk_datatype_ptr_dropn(cal2, (KK_I32(8)), _ctx);
    calendar0_4896 = cal1; /*std/time/calendar/calendar*/
  }
  else {
    kk_datatype_ptr_dropn(cal1, (KK_I32(8)), _ctx);
    calendar0_4896 = cal2; /*std/time/calendar/calendar*/
  }
  {
    struct kk_std_time_calendar_Calendar* _con6764 = kk_std_time_calendar__as_Calendar(calendar0_4896, _ctx);
    kk_string_t _pat06 = _con6764->name;
    kk_string_t _pat110 = _con6764->long_name;
    kk_string_t _pat25 = _con6764->month_prefix;
    kk_function_t _pat33 = _con6764->show_era;
    kk_function_t _pat42 = _con6764->instant_to_dc;
    kk_function_t _pat52 = _con6764->dc_to_instant;
    kk_function_t _x4 = _con6764->days_to_date;
    kk_function_t _pat61 = _con6764->date_to_days;
    if kk_likely(kk_datatype_ptr_is_unique(calendar0_4896, _ctx)) {
      kk_function_drop(_pat61, _ctx);
      kk_function_drop(_pat52, _ctx);
      kk_function_drop(_pat42, _ctx);
      kk_function_drop(_pat33, _ctx);
      kk_string_drop(_pat25, _ctx);
      kk_string_drop(_pat110, _ctx);
      kk_string_drop(_pat06, _ctx);
      kk_datatype_ptr_free(calendar0_4896, _ctx);
    }
    else {
      kk_function_dup(_x4, _ctx);
      kk_datatype_ptr_decref(calendar0_4896, _ctx);
    }
    return kk_function_call(kk_std_time_date__date, (kk_function_t, kk_integer_t, kk_context_t*), _x4, (_x4, days0, _ctx), _ctx);
  }
}


// lift anonymous function
struct kk_std_time_calendar_combine_earth_calendars_fun6765__t {
  struct kk_function_s _base;
  kk_std_time_calendar__calendar cal1;
  kk_std_time_calendar__calendar cal2;
  kk_std_time_date__date switch_date;
};
static kk_integer_t kk_std_time_calendar_combine_earth_calendars_fun6765(kk_function_t _fself, kk_std_time_date__date d1, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_combine_earth_calendars_fun6765(kk_std_time_calendar__calendar cal1, kk_std_time_calendar__calendar cal2, kk_std_time_date__date switch_date, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun6765__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_combine_earth_calendars_fun6765__t, 6, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_combine_earth_calendars_fun6765, kk_context());
  _self->cal1 = cal1;
  _self->cal2 = cal2;
  _self->switch_date = switch_date;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_integer_t kk_std_time_calendar_combine_earth_calendars_fun6765(kk_function_t _fself, kk_std_time_date__date d1, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun6765__t* _self = kk_function_as(struct kk_std_time_calendar_combine_earth_calendars_fun6765__t*, _fself, _ctx);
  kk_std_time_calendar__calendar cal1 = _self->cal1; /* std/time/calendar/calendar */
  kk_std_time_calendar__calendar cal2 = _self->cal2; /* std/time/calendar/calendar */
  kk_std_time_date__date switch_date = _self->switch_date; /* std/time/date/date */
  kk_drop_match(_self, {kk_std_time_calendar__calendar_dup(cal1, _ctx);kk_std_time_calendar__calendar_dup(cal2, _ctx);kk_std_time_date__date_dup(switch_date, _ctx);}, {}, _ctx)
  kk_std_core_types__order x_5815;
  kk_std_time_date__date _x6766 = kk_std_time_date__date_dup(d1, _ctx); /*std/time/date/date*/
  x_5815 = kk_std_time_date_compare(_x6766, switch_date, _ctx); /*order*/
  kk_std_time_calendar__calendar calendar1_4898;
  bool _match_5846;
  kk_integer_t _brw_5847;
  if (kk_std_core_types__is_Lt(x_5815, _ctx)) {
    _brw_5847 = kk_integer_from_small(-1); /*int*/
    goto _match6767;
  }
  if (kk_std_core_types__is_Eq(x_5815, _ctx)) {
    _brw_5847 = kk_integer_from_small(0); /*int*/
    goto _match6767;
  }
  {
    _brw_5847 = kk_integer_from_small(1); /*int*/
  }
  _match6767: ;
  bool _brw_5848 = kk_integer_eq_borrow(_brw_5847,(kk_integer_from_small(-1)),kk_context()); /*bool*/;
  kk_integer_drop(_brw_5847, _ctx);
  _match_5846 = _brw_5848; /*bool*/
  if (_match_5846) {
    kk_datatype_ptr_dropn(cal2, (KK_I32(8)), _ctx);
    calendar1_4898 = cal1; /*std/time/calendar/calendar*/
  }
  else {
    kk_datatype_ptr_dropn(cal1, (KK_I32(8)), _ctx);
    calendar1_4898 = cal2; /*std/time/calendar/calendar*/
  }
  {
    struct kk_std_time_calendar_Calendar* _con6768 = kk_std_time_calendar__as_Calendar(calendar1_4898, _ctx);
    kk_string_t _pat07 = _con6768->name;
    kk_string_t _pat112 = _con6768->long_name;
    kk_string_t _pat28 = _con6768->month_prefix;
    kk_function_t _pat34 = _con6768->show_era;
    kk_function_t _pat43 = _con6768->instant_to_dc;
    kk_function_t _pat53 = _con6768->dc_to_instant;
    kk_function_t _pat62 = _con6768->days_to_date;
    kk_function_t _x5 = _con6768->date_to_days;
    if kk_likely(kk_datatype_ptr_is_unique(calendar1_4898, _ctx)) {
      kk_function_drop(_pat62, _ctx);
      kk_function_drop(_pat53, _ctx);
      kk_function_drop(_pat43, _ctx);
      kk_function_drop(_pat34, _ctx);
      kk_string_drop(_pat28, _ctx);
      kk_string_drop(_pat112, _ctx);
      kk_string_drop(_pat07, _ctx);
      kk_datatype_ptr_free(calendar1_4898, _ctx);
    }
    else {
      kk_function_dup(_x5, _ctx);
      kk_datatype_ptr_decref(calendar1_4898, _ctx);
    }
    return kk_function_call(kk_integer_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _x5, (_x5, d1, _ctx), _ctx);
  }
}


// lift anonymous function
struct kk_std_time_calendar_combine_earth_calendars_fun6771__t {
  struct kk_function_s _base;
  kk_std_time_calendar__calendar cal1;
  kk_std_time_calendar__calendar cal2;
  kk_std_core_types__optional mb_show_era;
  kk_std_time_date__date switch_date;
};
static kk_string_t kk_std_time_calendar_combine_earth_calendars_fun6771(kk_function_t _fself, kk_std_time_date__date d01, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_combine_earth_calendars_fun6771(kk_std_time_calendar__calendar cal1, kk_std_time_calendar__calendar cal2, kk_std_core_types__optional mb_show_era, kk_std_time_date__date switch_date, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun6771__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_combine_earth_calendars_fun6771__t, 8, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_combine_earth_calendars_fun6771, kk_context());
  _self->cal1 = cal1;
  _self->cal2 = cal2;
  _self->mb_show_era = mb_show_era;
  _self->switch_date = switch_date;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_string_t kk_std_time_calendar_combine_earth_calendars_fun6771(kk_function_t _fself, kk_std_time_date__date d01, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun6771__t* _self = kk_function_as(struct kk_std_time_calendar_combine_earth_calendars_fun6771__t*, _fself, _ctx);
  kk_std_time_calendar__calendar cal1 = _self->cal1; /* std/time/calendar/calendar */
  kk_std_time_calendar__calendar cal2 = _self->cal2; /* std/time/calendar/calendar */
  kk_std_core_types__optional mb_show_era = _self->mb_show_era; /* optional<maybe<(std/time/date/date) -> string>> */
  kk_std_time_date__date switch_date = _self->switch_date; /* std/time/date/date */
  kk_drop_match(_self, {kk_std_time_calendar__calendar_dup(cal1, _ctx);kk_std_time_calendar__calendar_dup(cal2, _ctx);kk_std_core_types__optional_dup(mb_show_era, _ctx);kk_std_time_date__date_dup(switch_date, _ctx);}, {}, _ctx)
  kk_std_core_types__maybe _match_5841;
  if (kk_std_core_types__is_Optional(mb_show_era, _ctx)) {
    kk_box_t _box_x5726 = mb_show_era._cons.Optional.value;
    kk_std_core_types__maybe _mb_show_era_4195 = kk_std_core_types__maybe_unbox(_box_x5726, KK_BORROWED, _ctx);
    kk_std_core_types__maybe_dup(_mb_show_era_4195, _ctx);
    kk_std_core_types__optional_drop(mb_show_era, _ctx);
    _match_5841 = _mb_show_era_4195; /*maybe<(std/time/date/date) -> string>*/
  }
  else {
    kk_std_core_types__optional_drop(mb_show_era, _ctx);
    _match_5841 = kk_std_core_types__new_Nothing(_ctx); /*maybe<(std/time/date/date) -> string>*/
  }
  if (kk_std_core_types__is_Just(_match_5841, _ctx)) {
    kk_box_t _fun_unbox_x5730 = _match_5841._cons.Just.value;
    kk_std_time_date__date_drop(switch_date, _ctx);
    kk_datatype_ptr_dropn(cal2, (KK_I32(8)), _ctx);
    kk_datatype_ptr_dropn(cal1, (KK_I32(8)), _ctx);
    kk_box_t _x6772;
    kk_function_t _x6773 = kk_function_unbox(_fun_unbox_x5730, _ctx); /*(5731) -> 5732*/
    _x6772 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6773, (_x6773, kk_std_time_date__date_box(d01, _ctx), _ctx), _ctx); /*5732*/
    return kk_string_unbox(_x6772);
  }
  {
    kk_std_core_types__order x_58150;
    kk_std_time_date__date _x6774 = kk_std_time_date__date_dup(d01, _ctx); /*std/time/date/date*/
    x_58150 = kk_std_time_date_compare(_x6774, switch_date, _ctx); /*order*/
    kk_std_time_calendar__calendar calendar2_4901;
    bool _match_5842;
    kk_integer_t _brw_5843;
    if (kk_std_core_types__is_Lt(x_58150, _ctx)) {
      _brw_5843 = kk_integer_from_small(-1); /*int*/
      goto _match6775;
    }
    if (kk_std_core_types__is_Eq(x_58150, _ctx)) {
      _brw_5843 = kk_integer_from_small(0); /*int*/
      goto _match6775;
    }
    {
      _brw_5843 = kk_integer_from_small(1); /*int*/
    }
    _match6775: ;
    bool _brw_5844 = kk_integer_eq_borrow(_brw_5843,(kk_integer_from_small(-1)),kk_context()); /*bool*/;
    kk_integer_drop(_brw_5843, _ctx);
    _match_5842 = _brw_5844; /*bool*/
    if (_match_5842) {
      kk_datatype_ptr_dropn(cal2, (KK_I32(8)), _ctx);
      calendar2_4901 = cal1; /*std/time/calendar/calendar*/
    }
    else {
      kk_datatype_ptr_dropn(cal1, (KK_I32(8)), _ctx);
      calendar2_4901 = cal2; /*std/time/calendar/calendar*/
    }
    {
      struct kk_std_time_calendar_Calendar* _con6776 = kk_std_time_calendar__as_Calendar(calendar2_4901, _ctx);
      kk_string_t _pat010 = _con6776->name;
      kk_string_t _pat114 = _con6776->long_name;
      kk_string_t _pat210 = _con6776->month_prefix;
      kk_function_t _x6 = _con6776->show_era;
      kk_function_t _pat38 = _con6776->instant_to_dc;
      kk_function_t _pat45 = _con6776->dc_to_instant;
      kk_function_t _pat55 = _con6776->days_to_date;
      kk_function_t _pat64 = _con6776->date_to_days;
      if kk_likely(kk_datatype_ptr_is_unique(calendar2_4901, _ctx)) {
        kk_function_drop(_pat64, _ctx);
        kk_function_drop(_pat55, _ctx);
        kk_function_drop(_pat45, _ctx);
        kk_function_drop(_pat38, _ctx);
        kk_string_drop(_pat210, _ctx);
        kk_string_drop(_pat114, _ctx);
        kk_string_drop(_pat010, _ctx);
        kk_datatype_ptr_free(calendar2_4901, _ctx);
      }
      else {
        kk_function_dup(_x6, _ctx);
        kk_datatype_ptr_decref(calendar2_4901, _ctx);
      }
      return kk_function_call(kk_string_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _x6, (_x6, d01, _ctx), _ctx);
    }
  }
}


// lift anonymous function
struct kk_std_time_calendar_combine_earth_calendars_fun6778__t {
  struct kk_function_s _base;
  kk_function_t days_to_date0;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_combine_earth_calendars_fun6778(kk_function_t _fself, kk_std_time_instant__instant i, kk_std_time_duration__duration tzdelta, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_combine_earth_calendars_fun6778(kk_function_t days_to_date0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun6778__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_combine_earth_calendars_fun6778__t, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_combine_earth_calendars_fun6778, kk_context());
  _self->days_to_date0 = days_to_date0;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_combine_earth_calendars_fun6778(kk_function_t _fself, kk_std_time_instant__instant i, kk_std_time_duration__duration tzdelta, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun6778__t* _self = kk_function_as(struct kk_std_time_calendar_combine_earth_calendars_fun6778__t*, _fself, _ctx);
  kk_function_t days_to_date0 = _self->days_to_date0; /* (days : int) -> std/time/date/date */
  kk_drop_match(_self, {kk_function_dup(days_to_date0, _ctx);}, {}, _ctx)
  kk_std_core_types__tuple2_ _match_5840;
  kk_std_time_timestamp__timestamp _x6779;
  kk_std_num_ddouble__ddouble _x6780;
  kk_std_num_ddouble__ddouble _x6781;
  {
    kk_std_time_timestamp__timestamp _x1 = i.since;
    kk_std_time_instant__timescale _pat23 = i.ts;
    struct kk_std_time_timestamp_Timestamp* _con6782 = kk_std_time_timestamp__as_Timestamp(_x1, _ctx);
    struct kk_std_time_instant_Timescale* _con6783 = kk_std_time_instant__as_Timescale(_pat23, _ctx);
    kk_std_time_timestamp__timestamp_dup(_x1, _ctx);
    {
      struct kk_std_time_timestamp_Timestamp* _con6784 = kk_std_time_timestamp__as_Timestamp(_x1, _ctx);
      kk_std_num_ddouble__ddouble _x0 = _con6784->since;
      if kk_likely(kk_datatype_ptr_is_unique(_x1, _ctx)) {
        kk_datatype_ptr_free(_x1, _ctx);
      }
      else {
        kk_datatype_ptr_decref(_x1, _ctx);
      }
      _x6781 = _x0; /*std/time/timestamp/timespan*/
    }
  }
  kk_std_num_ddouble__ddouble _x6785;
  {
    kk_std_num_ddouble__ddouble _x2 = tzdelta.secs;
    _x6785 = _x2; /*std/time/timestamp/timespan*/
  }
  _x6780 = kk_std_num_ddouble__lp__plus__rp_(_x6781, _x6785, _ctx); /*std/num/ddouble/ddouble*/
  int32_t _x6786;
  {
    kk_std_time_timestamp__timestamp _x10 = i.since;
    kk_std_time_instant__timescale _pat230 = i.ts;
    struct kk_std_time_timestamp_Timestamp* _con6787 = kk_std_time_timestamp__as_Timestamp(_x10, _ctx);
    struct kk_std_time_instant_Timescale* _con6788 = kk_std_time_instant__as_Timescale(_pat230, _ctx);
    kk_std_time_timestamp__timestamp_dup(_x10, _ctx);
    kk_std_time_instant__instant_drop(i, _ctx);
    {
      struct kk_std_time_timestamp_Timestamp* _con6789 = kk_std_time_timestamp__as_Timestamp(_x10, _ctx);
      kk_std_num_ddouble__ddouble _pat000 = _con6789->since;
      int32_t _x00 = _con6789->leap32;
      if kk_likely(kk_datatype_ptr_is_unique(_x10, _ctx)) {
        kk_datatype_ptr_free(_x10, _ctx);
      }
      else {
        kk_datatype_ptr_decref(_x10, _ctx);
      }
      _x6786 = _x00; /*int32*/
    }
  }
  _x6779 = kk_std_time_timestamp__new_Timestamp(kk_reuse_null, 0, _x6780, _x6786, _ctx); /*std/time/timestamp/timestamp*/
  _match_5840 = kk_std_time_timestamp_days_clock(_x6779, _ctx); /*(int, std/time/date/clock)*/
  {
    kk_box_t _box_x5734 = _match_5840.fst;
    kk_box_t _box_x5735 = _match_5840.snd;
    kk_std_time_date__clock clock = kk_std_time_date__clock_unbox(_box_x5735, KK_BORROWED, _ctx);
    struct kk_std_time_date_Clock* _con6790 = kk_std_time_date__as_Clock(clock, _ctx);
    kk_integer_t days = kk_integer_unbox(_box_x5734, _ctx);
    kk_std_time_date__clock_dup(clock, _ctx);
    kk_integer_dup(days, _ctx);
    kk_std_core_types__tuple2__drop(_match_5840, _ctx);
    kk_std_time_date__date _b_5753_5736 = kk_function_call(kk_std_time_date__date, (kk_function_t, kk_integer_t, kk_context_t*), days_to_date0, (days_to_date0, days, _ctx), _ctx); /*std/time/date/date*/;
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_date__date_box(_b_5753_5736, _ctx), kk_std_time_date__clock_box(clock, _ctx), _ctx);
  }
}


// lift anonymous function
struct kk_std_time_calendar_combine_earth_calendars_fun6792__t {
  struct kk_function_s _base;
  kk_function_t date_to_days0;
};
static kk_std_time_instant__instant kk_std_time_calendar_combine_earth_calendars_fun6792(kk_function_t _fself, kk_std_time_date__date d00, kk_std_time_date__clock c, kk_std_time_calendar__timezone tz, kk_std_time_instant__timescale ts0, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_combine_earth_calendars_fun6792(kk_function_t date_to_days0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun6792__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_combine_earth_calendars_fun6792__t, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_combine_earth_calendars_fun6792, kk_context());
  _self->date_to_days0 = date_to_days0;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_time_instant__instant kk_std_time_calendar_combine_earth_calendars_fun6792(kk_function_t _fself, kk_std_time_date__date d00, kk_std_time_date__clock c, kk_std_time_calendar__timezone tz, kk_std_time_instant__timescale ts0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun6792__t* _self = kk_function_as(struct kk_std_time_calendar_combine_earth_calendars_fun6792__t*, _fself, _ctx);
  kk_function_t date_to_days0 = _self->date_to_days0; /* (std/time/date/date) -> int */
  kk_drop_match(_self, {kk_function_dup(date_to_days0, _ctx);}, {}, _ctx)
  kk_std_time_timestamp__timestamp t_5001;
  kk_std_core_types__optional _x6793;
  kk_box_t _x6794;
  bool _x6795;
  {
    struct kk_std_time_instant_Timescale* _con6796 = kk_std_time_instant__as_Timescale(ts0, _ctx);
    kk_std_core_types__maybe _x31 = _con6796->mb_seconds_in_day;
    kk_std_core_types__maybe_dup(_x31, _ctx);
    if (kk_std_core_types__is_Nothing(_x31, _ctx)) {
      _x6795 = false; /*bool*/
    }
    else {
      kk_std_core_types__maybe_drop(_x31, _ctx);
      _x6795 = true; /*bool*/
    }
  }
  _x6794 = kk_bool_box(_x6795); /*116*/
  _x6793 = kk_std_core_types__new_Optional(_x6794, _ctx); /*optional<116>*/
  t_5001 = kk_std_time_calendar_dc_timestamp(d00, c, date_to_days0, _x6793, _ctx); /*std/time/timestamp/timestamp*/
  kk_std_time_instant__instant i0;
  kk_std_time_timestamp__timestamp _x6797 = kk_std_time_timestamp__timestamp_dup(t_5001, _ctx); /*std/time/timestamp/timestamp*/
  kk_std_time_instant__timescale _x6798 = kk_std_time_instant__timescale_dup(ts0, _ctx); /*std/time/instant/timescale*/
  i0 = kk_std_time_instant__new_Instant(_x6797, _x6798, _ctx); /*std/time/instant/instant*/
  bool _match_5837;
  kk_string_t _x6799;
  {
    struct kk_std_time_calendar_Timezone* _con6800 = kk_std_time_calendar__as_Timezone(tz, _ctx);
    kk_string_t _x3 = _con6800->name;
    kk_string_dup(_x3, _ctx);
    _x6799 = _x3; /*string*/
  }
  kk_string_t _x6801;
  kk_std_time_calendar__timezone _x6802 = kk_std_time_calendar_tz_utc; /*std/time/calendar/timezone*/
  {
    struct kk_std_time_calendar_Timezone* _con6803 = kk_std_time_calendar__as_Timezone(_x6802, _ctx);
    kk_string_t _x01 = _con6803->name;
    _x6801 = kk_string_dup(_x01, _ctx); /*string*/
  }
  _match_5837 = kk_string_is_eq(_x6799,_x6801,kk_context()); /*bool*/
  if (_match_5837) {
    kk_datatype_ptr_dropn(tz, (KK_I32(3)), _ctx);
    kk_datatype_ptr_dropn(ts0, (KK_I32(10)), _ctx);
    kk_datatype_ptr_dropn(t_5001, (KK_I32(0)), _ctx);
    return i0;
  }
  {
    struct kk_std_time_calendar_Timezone* _con6804 = kk_std_time_calendar__as_Timezone(tz, _ctx);
    kk_string_t _pat020 = _con6804->name;
    kk_function_t _pat116 = _con6804->utc_delta;
    kk_function_t _x100 = _con6804->utc_inverse;
    kk_function_dup(_x100, _ctx);
    kk_std_core_types__maybe _match_5838;
    kk_std_time_instant__instant _x6805 = kk_std_time_instant__instant_dup(i0, _ctx); /*std/time/instant/instant*/
    _match_5838 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x100, (_x100, _x6805, _ctx), _ctx); /*maybe<std/time/instant/instant>*/
    if (kk_std_core_types__is_Just(_match_5838, _ctx)) {
      kk_box_t _box_x5740 = _match_5838._cons.Just.value;
      kk_std_time_instant__instant inv = kk_std_time_instant__instant_unbox(_box_x5740, KK_BORROWED, _ctx);
      if kk_likely(kk_datatype_ptr_is_unique(tz, _ctx)) {
        kk_function_drop(_x100, _ctx);
        kk_function_drop(_pat116, _ctx);
        kk_string_drop(_pat020, _ctx);
        kk_datatype_ptr_free(tz, _ctx);
      }
      else {
        kk_datatype_ptr_decref(tz, _ctx);
      }
      kk_datatype_ptr_dropn(ts0, (KK_I32(10)), _ctx);
      kk_datatype_ptr_dropn(t_5001, (KK_I32(0)), _ctx);
      kk_std_time_instant__instant_drop(i0, _ctx);
      kk_std_time_instant__instant_dup(inv, _ctx);
      kk_std_core_types__maybe_drop(_match_5838, _ctx);
      return inv;
    }
    {
      kk_std_core_types__tuple2_ _this_4787;
      {
        struct kk_std_time_calendar_Timezone* _con6806 = kk_std_time_calendar__as_Timezone(tz, _ctx);
        kk_function_t _x30 = _con6806->utc_delta;
        kk_function_dup(_x30, _ctx);
        _this_4787 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x30, (_x30, i0, _ctx), _ctx); /*(std/time/duration/duration, string)*/
      }
      kk_std_time_timestamp__timestamp t1_4790;
      kk_std_num_ddouble__ddouble _x6807;
      kk_std_num_ddouble__ddouble _x6808;
      {
        struct kk_std_time_timestamp_Timestamp* _con6809 = kk_std_time_timestamp__as_Timestamp(t_5001, _ctx);
        kk_std_num_ddouble__ddouble _x1000 = _con6809->since;
        _x6808 = _x1000; /*std/time/timestamp/timespan*/
      }
      kk_std_num_ddouble__ddouble _x6810;
      double _x6811;
      double _x6812;
      {
        kk_box_t _box_x5741 = _this_4787.fst;
        kk_box_t _box_x5742 = _this_4787.snd;
        kk_std_time_duration__duration _x20 = kk_std_time_duration__duration_unbox(_box_x5741, KK_BORROWED, _ctx);
        {
          kk_std_num_ddouble__ddouble _x50 = _x20.secs;
          {
            double _x40 = _x50.hi;
            _x6812 = _x40; /*float64*/
          }
        }
      }
      _x6811 = (-_x6812); /*float64*/
      double _x6813;
      double _x6814;
      {
        kk_box_t _box_x5743 = _this_4787.fst;
        kk_box_t _box_x5744 = _this_4787.snd;
        kk_std_time_duration__duration _x200 = kk_std_time_duration__duration_unbox(_box_x5743, KK_BORROWED, _ctx);
        {
          kk_std_num_ddouble__ddouble _x500 = _x200.secs;
          {
            double _x000 = _x500.lo;
            _x6814 = _x000; /*float64*/
          }
        }
      }
      _x6813 = (-_x6814); /*float64*/
      _x6810 = kk_std_num_ddouble__new_Ddouble(_x6811, _x6813, _ctx); /*std/num/ddouble/ddouble*/
      _x6807 = kk_std_num_ddouble__lp__plus__rp_(_x6808, _x6810, _ctx); /*std/num/ddouble/ddouble*/
      int32_t _x6815;
      {
        struct kk_std_time_timestamp_Timestamp* _con6816 = kk_std_time_timestamp__as_Timestamp(t_5001, _ctx);
        kk_std_num_ddouble__ddouble _pat0200 = _con6816->since;
        int32_t _x2000 = _con6816->leap32;
        _x6815 = _x2000; /*int32*/
      }
      t1_4790 = kk_std_time_timestamp__new_Timestamp(kk_reuse_null, 0, _x6807, _x6815, _ctx); /*std/time/timestamp/timestamp*/
      kk_std_time_instant__instant tzi1;
      kk_std_time_instant__timescale _x6817 = kk_std_time_instant__timescale_dup(ts0, _ctx); /*std/time/instant/timescale*/
      tzi1 = kk_std_time_instant__new_Instant(t1_4790, _x6817, _ctx); /*std/time/instant/instant*/
      kk_std_core_types__tuple2_ _this0_4794;
      {
        struct kk_std_time_calendar_Timezone* _con6818 = kk_std_time_calendar__as_Timezone(tz, _ctx);
        kk_string_t _pat080 = _con6818->name;
        kk_function_t _x7 = _con6818->utc_delta;
        kk_function_t _pat170 = _con6818->utc_inverse;
        if kk_likely(kk_datatype_ptr_is_unique(tz, _ctx)) {
          kk_function_drop(_pat170, _ctx);
          kk_string_drop(_pat080, _ctx);
          kk_datatype_ptr_free(tz, _ctx);
        }
        else {
          kk_function_dup(_x7, _ctx);
          kk_datatype_ptr_decref(tz, _ctx);
        }
        kk_std_time_instant__instant _x6819 = kk_std_time_instant__instant_dup(tzi1, _ctx); /*std/time/instant/instant*/
        _this0_4794 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x7, (_x7, _x6819, _ctx), _ctx); /*(std/time/duration/duration, string)*/
      }
      bool _match_5839;
      kk_std_time_duration__duration _x6820;
      {
        kk_box_t _box_x5745 = _this_4787.fst;
        kk_box_t _box_x5746 = _this_4787.snd;
        kk_std_time_duration__duration _x201 = kk_std_time_duration__duration_unbox(_box_x5745, KK_BORROWED, _ctx);
        kk_std_core_types__tuple2__drop(_this_4787, _ctx);
        _x6820 = _x201; /*std/time/duration/duration*/
      }
      kk_std_time_duration__duration _x6821;
      {
        kk_box_t _box_x5747 = _this0_4794.fst;
        kk_box_t _box_x5748 = _this0_4794.snd;
        kk_std_time_duration__duration _x60 = kk_std_time_duration__duration_unbox(_box_x5747, KK_BORROWED, _ctx);
        _x6821 = _x60; /*std/time/duration/duration*/
      }
      _match_5839 = kk_std_time_duration__lp__eq__eq__rp_(_x6820, _x6821, _ctx); /*bool*/
      if (_match_5839) {
        kk_datatype_ptr_dropn(ts0, (KK_I32(10)), _ctx);
        kk_datatype_ptr_dropn(t_5001, (KK_I32(0)), _ctx);
        kk_std_core_types__tuple2__drop(_this0_4794, _ctx);
        return tzi1;
      }
      {
        kk_std_time_instant__instant_drop(tzi1, _ctx);
        kk_std_time_timestamp__timestamp t3_4797;
        kk_std_num_ddouble__ddouble _x6822;
        kk_std_num_ddouble__ddouble _x6823;
        {
          struct kk_std_time_timestamp_Timestamp* _con6824 = kk_std_time_timestamp__as_Timestamp(t_5001, _ctx);
          kk_std_num_ddouble__ddouble _x11 = _con6824->since;
          _x6823 = _x11; /*std/time/timestamp/timespan*/
        }
        kk_std_num_ddouble__ddouble _x6825;
        double _x6826;
        double _x6827;
        {
          kk_box_t _box_x5749 = _this0_4794.fst;
          kk_box_t _box_x5750 = _this0_4794.snd;
          kk_std_time_duration__duration _x600 = kk_std_time_duration__duration_unbox(_box_x5749, KK_BORROWED, _ctx);
          {
            kk_std_num_ddouble__ddouble _x9 = _x600.secs;
            {
              double _x8 = _x9.hi;
              _x6827 = _x8; /*float64*/
            }
          }
        }
        _x6826 = (-_x6827); /*float64*/
        double _x6828;
        double _x6829;
        {
          kk_box_t _box_x5751 = _this0_4794.fst;
          kk_box_t _box_x5752 = _this0_4794.snd;
          kk_std_time_duration__duration _x601 = kk_std_time_duration__duration_unbox(_box_x5751, KK_BORROWED, _ctx);
          kk_std_core_types__tuple2__drop(_this0_4794, _ctx);
          {
            kk_std_num_ddouble__ddouble _x90 = _x601.secs;
            {
              double _x010 = _x90.lo;
              _x6829 = _x010; /*float64*/
            }
          }
        }
        _x6828 = (-_x6829); /*float64*/
        _x6825 = kk_std_num_ddouble__new_Ddouble(_x6826, _x6828, _ctx); /*std/num/ddouble/ddouble*/
        _x6822 = kk_std_num_ddouble__lp__plus__rp_(_x6823, _x6825, _ctx); /*std/num/ddouble/ddouble*/
        int32_t _x6830;
        {
          struct kk_std_time_timestamp_Timestamp* _con6831 = kk_std_time_timestamp__as_Timestamp(t_5001, _ctx);
          kk_std_num_ddouble__ddouble _pat021 = _con6831->since;
          int32_t _x21 = _con6831->leap32;
          if kk_likely(kk_datatype_ptr_is_unique(t_5001, _ctx)) {
            kk_datatype_ptr_free(t_5001, _ctx);
          }
          else {
            kk_datatype_ptr_decref(t_5001, _ctx);
          }
          _x6830 = _x21; /*int32*/
        }
        t3_4797 = kk_std_time_timestamp__new_Timestamp(kk_reuse_null, 0, _x6822, _x6830, _ctx); /*std/time/timestamp/timestamp*/
        return kk_std_time_instant__new_Instant(t3_4797, ts0, _ctx);
      }
    }
  }
}

kk_std_time_calendar__calendar kk_std_time_calendar_combine_earth_calendars(kk_string_t name0, kk_string_t long_name0, kk_std_time_date__date switch_date, kk_std_time_calendar__calendar cal1, kk_std_time_calendar__calendar cal2, kk_std_core_types__optional mb_show_era, kk_context_t* _ctx) { /* (name : string, long-name : string, switch-date : std/time/date/date, cal1 : calendar, cal2 : calendar, mb-show-era : optional<maybe<(std/time/date/date) -> string>>) -> calendar */ 
  kk_function_t days_to_date0;
  kk_std_time_calendar__calendar_dup(cal1, _ctx);
  kk_std_time_calendar__calendar_dup(cal2, _ctx);
  kk_std_time_date__date_dup(switch_date, _ctx);
  days_to_date0 = kk_std_time_calendar_new_combine_earth_calendars_fun6762(cal1, cal2, switch_date, _ctx); /*(days0 : int) -> std/time/date/date*/
  kk_function_t date_to_days0;
  kk_std_time_calendar__calendar_dup(cal1, _ctx);
  kk_std_time_calendar__calendar_dup(cal2, _ctx);
  kk_std_time_date__date_dup(switch_date, _ctx);
  date_to_days0 = kk_std_time_calendar_new_combine_earth_calendars_fun6765(cal1, cal2, switch_date, _ctx); /*(d1 : std/time/date/date) -> int*/
  kk_string_t _x6769;
  kk_std_core_types__optional _match_5845 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5845, _ctx)) {
    kk_box_t _box_x5725 = _match_5845._cons.Optional.value;
    kk_string_t _month_prefix_3624 = kk_string_unbox(_box_x5725);
    kk_string_dup(_month_prefix_3624, _ctx);
    kk_std_core_types__optional_drop(_match_5845, _ctx);
    _x6769 = _month_prefix_3624; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(_match_5845, _ctx);
    _x6769 = kk_string_empty(); /*string*/
  }
  kk_function_t _x6777;
  kk_function_dup(days_to_date0, _ctx);
  _x6777 = kk_std_time_calendar_new_combine_earth_calendars_fun6778(days_to_date0, _ctx); /*(i : std/time/instant/instant, tzdelta : std/time/duration/duration) -> (6, 7)*/
  kk_function_t _x6791;
  kk_function_dup(date_to_days0, _ctx);
  _x6791 = kk_std_time_calendar_new_combine_earth_calendars_fun6792(date_to_days0, _ctx); /*(d00 : std/time/date/date, c : std/time/date/clock, tz : std/time/calendar/timezone, ts0 : std/time/instant/timescale) -> std/time/instant/instant*/
  return kk_std_time_calendar__new_Calendar(kk_reuse_null, 0, name0, long_name0, _x6769, kk_std_time_calendar_new_combine_earth_calendars_fun6771(cal1, cal2, mb_show_era, switch_date, _ctx), _x6777, _x6791, days_to_date0, date_to_days0, _ctx);
}
 
// Convert an `:instant` to a `:date`, `:clock`, timezone delta and abbreviation,
// for a given timezone `tz` (=`tz-utc` by default) and calendar (=`cal-iso` by default).

kk_std_core_types__tuple4_ kk_std_time_calendar_instant_dc(kk_std_time_instant__instant i, kk_std_core_types__optional tz, kk_std_core_types__optional cal, kk_context_t* _ctx) { /* (i : std/time/instant/instant, tz : optional<timezone>, cal : optional<calendar>) -> (std/time/date/date, std/time/date/clock, std/time/duration/duration, string) */ 
  kk_std_time_calendar__timezone _match_5833;
  if (kk_std_core_types__is_Optional(tz, _ctx)) {
    kk_box_t _box_x5755 = tz._cons.Optional.value;
    kk_std_time_calendar__timezone _tz_4331 = kk_std_time_calendar__timezone_unbox(_box_x5755, KK_BORROWED, _ctx);
    kk_std_time_calendar__timezone_dup(_tz_4331, _ctx);
    kk_std_core_types__optional_drop(tz, _ctx);
    _match_5833 = _tz_4331; /*std/time/calendar/timezone*/
  }
  else {
    kk_std_core_types__optional_drop(tz, _ctx);
    _match_5833 = kk_std_time_calendar__timezone_dup(kk_std_time_calendar_tz_utc, _ctx); /*std/time/calendar/timezone*/
  }
  {
    struct kk_std_time_calendar_Timezone* _con6832 = kk_std_time_calendar__as_Timezone(_match_5833, _ctx);
    kk_string_t _pat0 = _con6832->name;
    kk_function_t _x = _con6832->utc_delta;
    kk_function_t _pat1 = _con6832->utc_inverse;
    if kk_likely(kk_datatype_ptr_is_unique(_match_5833, _ctx)) {
      kk_function_drop(_pat1, _ctx);
      kk_string_drop(_pat0, _ctx);
      kk_datatype_ptr_free(_match_5833, _ctx);
    }
    else {
      kk_function_dup(_x, _ctx);
      kk_datatype_ptr_decref(_match_5833, _ctx);
    }
    kk_std_core_types__tuple2_ _match_5834;
    kk_std_time_instant__instant _x6833 = kk_std_time_instant__instant_dup(i, _ctx); /*std/time/instant/instant*/
    _match_5834 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x, (_x, _x6833, _ctx), _ctx); /*(std/time/duration/duration, string)*/
    {
      kk_box_t _box_x5756 = _match_5834.fst;
      kk_box_t _box_x5757 = _match_5834.snd;
      kk_std_time_duration__duration tzdelta = kk_std_time_duration__duration_unbox(_box_x5756, KK_BORROWED, _ctx);
      kk_string_t tzabbrv = kk_string_unbox(_box_x5757);
      kk_string_dup(tzabbrv, _ctx);
      kk_std_core_types__tuple2__drop(_match_5834, _ctx);
      kk_std_time_calendar__calendar _match_5835;
      if (kk_std_core_types__is_Optional(cal, _ctx)) {
        kk_box_t _box_x5758 = cal._cons.Optional.value;
        kk_std_time_calendar__calendar _cal_4335 = kk_std_time_calendar__calendar_unbox(_box_x5758, KK_BORROWED, _ctx);
        kk_std_time_calendar__calendar_dup(_cal_4335, _ctx);
        kk_std_core_types__optional_drop(cal, _ctx);
        _match_5835 = _cal_4335; /*std/time/calendar/calendar*/
      }
      else {
        kk_std_core_types__optional_drop(cal, _ctx);
        _match_5835 = kk_std_time_calendar__calendar_dup(kk_std_time_calendar_cal_iso, _ctx); /*std/time/calendar/calendar*/
      }
      {
        struct kk_std_time_calendar_Calendar* _con6834 = kk_std_time_calendar__as_Calendar(_match_5835, _ctx);
        kk_string_t _pat01 = _con6834->name;
        kk_string_t _pat10 = _con6834->long_name;
        kk_string_t _pat20 = _con6834->month_prefix;
        kk_function_t _pat30 = _con6834->show_era;
        kk_function_t _x0 = _con6834->instant_to_dc;
        kk_function_t _pat40 = _con6834->dc_to_instant;
        kk_function_t _pat50 = _con6834->days_to_date;
        kk_function_t _pat6 = _con6834->date_to_days;
        if kk_likely(kk_datatype_ptr_is_unique(_match_5835, _ctx)) {
          kk_function_drop(_pat6, _ctx);
          kk_function_drop(_pat50, _ctx);
          kk_function_drop(_pat40, _ctx);
          kk_function_drop(_pat30, _ctx);
          kk_string_drop(_pat20, _ctx);
          kk_string_drop(_pat10, _ctx);
          kk_string_drop(_pat01, _ctx);
          kk_datatype_ptr_free(_match_5835, _ctx);
        }
        else {
          kk_function_dup(_x0, _ctx);
          kk_datatype_ptr_decref(_match_5835, _ctx);
        }
        kk_std_core_types__tuple2_ _match_5836 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_std_time_instant__instant, kk_std_time_duration__duration, kk_context_t*), _x0, (_x0, i, tzdelta, _ctx), _ctx); /*(std/time/date/date, std/time/date/clock)*/;
        {
          kk_box_t _box_x5759 = _match_5836.fst;
          kk_box_t _box_x5760 = _match_5836.snd;
          kk_std_time_date__date d = kk_std_time_date__date_unbox(_box_x5759, KK_BORROWED, _ctx);
          kk_std_time_date__clock c = kk_std_time_date__clock_unbox(_box_x5760, KK_BORROWED, _ctx);
          struct kk_std_time_date_Clock* _con6835 = kk_std_time_date__as_Clock(c, _ctx);
          kk_std_time_date__clock_dup(c, _ctx);
          kk_std_time_date__date_dup(d, _ctx);
          kk_std_core_types__tuple2__drop(_match_5836, _ctx);
          return kk_std_core_types__new_dash__lp__comma__comma__comma__rp_(kk_reuse_null, 0, kk_std_time_date__date_box(d, _ctx), kk_std_time_date__clock_box(c, _ctx), kk_std_time_duration__duration_box(tzdelta, _ctx), kk_string_box(tzabbrv), _ctx);
        }
      }
    }
  }
}
 
// Return the local timezone on the current system.


// lift anonymous function
struct kk_std_time_calendar_tz_local_fun6838__t {
  struct kk_function_s _base;
  kk_std_time_calendar__local_timezone tz;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_local_fun6838(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_local_fun6838(kk_std_time_calendar__local_timezone tz, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_local_fun6838__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_tz_local_fun6838__t, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_tz_local_fun6838, kk_context());
  _self->tz = tz;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_local_fun6838(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_local_fun6838__t* _self = kk_function_as(struct kk_std_time_calendar_tz_local_fun6838__t*, _fself, _ctx);
  kk_std_time_calendar__local_timezone tz = _self->tz; /* std/time/calendar/local-timezone */
  kk_drop_match(_self, {kk_std_time_calendar__local_timezone_dup(tz, _ctx);}, {}, _ctx)
  kk_std_num_ddouble__ddouble x_4909 = kk_std_time_utc_unix_timestamp(i0, _ctx); /*std/num/ddouble/ddouble*/;
  kk_std_core_types__tuple2_ _match_5832;
  double _x6839;
  {
    double _x = x_4909.hi;
    _x6839 = _x; /*float64*/
  }
  _match_5832 = kk_std_time_calendar_local_utc_delta(tz, _x6839, _ctx); /*(float64, string)*/
  {
    kk_box_t _box_x5769 = _match_5832.fst;
    kk_box_t _box_x5770 = _match_5832.snd;
    double ofs = kk_double_unbox(_box_x5769, KK_BORROWED, _ctx);
    kk_string_t abbrv = kk_string_unbox(_box_x5770);
    kk_string_dup(abbrv, _ctx);
    kk_std_core_types__tuple2__drop(_match_5832, _ctx);
    kk_box_t _x6840;
    kk_std_time_duration__duration _x6841;
    kk_std_num_ddouble__ddouble _x6842 = kk_std_num_ddouble__new_Ddouble(ofs, 0x0p+0, _ctx); /*std/num/ddouble/ddouble*/
    _x6841 = kk_std_time_duration__new_Duration(_x6842, _ctx); /*std/time/duration/duration*/
    _x6840 = kk_std_time_duration__duration_box(_x6841, _ctx); /*6*/
    return kk_std_core_types__new_dash__lp__comma__rp_(_x6840, kk_string_box(abbrv), _ctx);
  }
}


// lift anonymous function
struct kk_std_time_calendar_tz_local_fun6844__t {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x5776;
};
static kk_std_core_types__maybe kk_std_time_calendar_tz_local_fun6844(kk_function_t _fself, kk_std_time_instant__instant _b_5779, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_local_fun6844(kk_box_t _fun_unbox_x5776, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_local_fun6844__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_tz_local_fun6844__t, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_tz_local_fun6844, kk_context());
  _self->_fun_unbox_x5776 = _fun_unbox_x5776;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_core_types__maybe kk_std_time_calendar_tz_local_fun6844(kk_function_t _fself, kk_std_time_instant__instant _b_5779, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_local_fun6844__t* _self = kk_function_as(struct kk_std_time_calendar_tz_local_fun6844__t*, _fself, _ctx);
  kk_box_t _fun_unbox_x5776 = _self->_fun_unbox_x5776; /* 116 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x5776, _ctx);}, {}, _ctx)
  kk_box_t _x6845;
  kk_function_t _x6846 = kk_function_unbox(_fun_unbox_x5776, _ctx); /*(5777) -> 5778*/
  _x6845 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6846, (_x6846, kk_std_time_instant__instant_box(_b_5779, _ctx), _ctx), _ctx); /*5778*/
  return kk_std_core_types__maybe_unbox(_x6845, KK_OWNED, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_tz_local_fun6847__t {
  struct kk_function_s _base;
};
static kk_std_core_types__maybe kk_std_time_calendar_tz_local_fun6847(kk_function_t _fself, kk_std_time_instant__instant i, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_local_fun6847(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_tz_local_fun6847, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_std_core_types__maybe kk_std_time_calendar_tz_local_fun6847(kk_function_t _fself, kk_std_time_instant__instant i, kk_context_t* _ctx) {
  kk_unused(_fself);
  kk_std_time_instant__instant_drop(i, _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}

kk_std_time_calendar__timezone kk_std_time_calendar_tz_local(kk_context_t* _ctx) { /* () -> ndet timezone */ 
  kk_std_time_calendar__local_timezone tz = kk_std_time_calendar_local_get_timezone(_ctx); /*std/time/calendar/local-timezone*/;
  kk_string_t _x6836 = kk_string_empty(); /*string*/
  kk_function_t _x6843;
  kk_std_core_types__optional _match_5831 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5831, _ctx)) {
    kk_box_t _fun_unbox_x5776 = _match_5831._cons.Optional.value;
    kk_box_dup(_fun_unbox_x5776, _ctx);
    kk_std_core_types__optional_drop(_match_5831, _ctx);
    _x6843 = kk_std_time_calendar_new_tz_local_fun6844(_fun_unbox_x5776, _ctx); /*(std/time/instant/instant) -> maybe<std/time/instant/instant>*/
  }
  else {
    kk_std_core_types__optional_drop(_match_5831, _ctx);
    _x6843 = kk_std_time_calendar_new_tz_local_fun6847(_ctx); /*(std/time/instant/instant) -> maybe<std/time/instant/instant>*/
  }
  return kk_std_time_calendar__new_Timezone(kk_reuse_null, 0, _x6836, kk_std_time_calendar_new_tz_local_fun6838(tz, _ctx), _x6843, _ctx);
}
 
// Return the day of the week for a calendar `cal` (=`cal-iso`).

kk_std_time_date__weekday kk_std_time_calendar_weekday(kk_std_time_date__date d, kk_std_time_calendar__calendar cal, kk_context_t* _ctx) { /* (d : std/time/date/date, cal : calendar) -> std/time/date/weekday */ 
  kk_integer_t x_4911;
  {
    struct kk_std_time_calendar_Calendar* _con6848 = kk_std_time_calendar__as_Calendar(cal, _ctx);
    kk_string_t _pat0 = _con6848->name;
    kk_string_t _pat1 = _con6848->long_name;
    kk_string_t _pat2 = _con6848->month_prefix;
    kk_function_t _pat3 = _con6848->show_era;
    kk_function_t _pat4 = _con6848->instant_to_dc;
    kk_function_t _pat5 = _con6848->dc_to_instant;
    kk_function_t _pat6 = _con6848->days_to_date;
    kk_function_t _x = _con6848->date_to_days;
    if kk_likely(kk_datatype_ptr_is_unique(cal, _ctx)) {
      kk_function_drop(_pat6, _ctx);
      kk_function_drop(_pat5, _ctx);
      kk_function_drop(_pat4, _ctx);
      kk_function_drop(_pat3, _ctx);
      kk_string_drop(_pat2, _ctx);
      kk_string_drop(_pat1, _ctx);
      kk_string_drop(_pat0, _ctx);
      kk_datatype_ptr_free(cal, _ctx);
    }
    else {
      kk_function_dup(_x, _ctx);
      kk_datatype_ptr_decref(cal, _ctx);
    }
    x_4911 = kk_function_call(kk_integer_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _x, (_x, d, _ctx), _ctx); /*int*/
  }
  kk_integer_t dow;
  kk_integer_t _x6849 = kk_integer_add_small_const(x_4911, 6, _ctx); /*int*/
  dow = kk_integer_mod(_x6849,(kk_integer_from_small(7)),kk_context()); /*int*/
  return kk_std_time_date_weekday(dow, _ctx);
}
 
// Create a new calendar from anoter calendar `cal` by adding an offset to the
// years (`year-shift`). This is used for example to create the Ethiopian calendar
// from the Coptic calendar, by using:
// `year-shift-earth-calendar( "EC", "Ethiopian", ~276, cal-coptic )`


// lift anonymous function
struct kk_std_time_calendar_year_shift_earth_calendar_fun6850__t {
  struct kk_function_s _base;
  kk_std_time_calendar__calendar cal;
  kk_integer_t year_shift;
};
static kk_std_time_date__date kk_std_time_calendar_year_shift_earth_calendar_fun6850(kk_function_t _fself, kk_integer_t days0, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_year_shift_earth_calendar_fun6850(kk_std_time_calendar__calendar cal, kk_integer_t year_shift, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6850__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6850__t, 3, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_year_shift_earth_calendar_fun6850, kk_context());
  _self->cal = cal;
  _self->year_shift = year_shift;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_time_date__date kk_std_time_calendar_year_shift_earth_calendar_fun6850(kk_function_t _fself, kk_integer_t days0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6850__t* _self = kk_function_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6850__t*, _fself, _ctx);
  kk_std_time_calendar__calendar cal = _self->cal; /* std/time/calendar/calendar */
  kk_integer_t year_shift = _self->year_shift; /* int */
  kk_drop_match(_self, {kk_std_time_calendar__calendar_dup(cal, _ctx);kk_integer_dup(year_shift, _ctx);}, {}, _ctx)
  kk_std_time_date__date d01;
  {
    struct kk_std_time_calendar_Calendar* _con6851 = kk_std_time_calendar__as_Calendar(cal, _ctx);
    kk_string_t _pat05 = _con6851->name;
    kk_string_t _pat18 = _con6851->long_name;
    kk_string_t _pat24 = _con6851->month_prefix;
    kk_function_t _pat33 = _con6851->show_era;
    kk_function_t _pat42 = _con6851->instant_to_dc;
    kk_function_t _pat52 = _con6851->dc_to_instant;
    kk_function_t _x4 = _con6851->days_to_date;
    kk_function_t _pat60 = _con6851->date_to_days;
    if kk_likely(kk_datatype_ptr_is_unique(cal, _ctx)) {
      kk_function_drop(_pat60, _ctx);
      kk_function_drop(_pat52, _ctx);
      kk_function_drop(_pat42, _ctx);
      kk_function_drop(_pat33, _ctx);
      kk_string_drop(_pat24, _ctx);
      kk_string_drop(_pat18, _ctx);
      kk_string_drop(_pat05, _ctx);
      kk_datatype_ptr_free(cal, _ctx);
    }
    else {
      kk_function_dup(_x4, _ctx);
      kk_datatype_ptr_decref(cal, _ctx);
    }
    d01 = kk_function_call(kk_std_time_date__date, (kk_function_t, kk_integer_t, kk_context_t*), _x4, (_x4, days0, _ctx), _ctx); /*std/time/date/date*/
  }
  kk_integer_t _b_5783_5782;
  kk_integer_t _x6852;
  {
    kk_integer_t _x6 = d01.year;
    kk_integer_dup(_x6, _ctx);
    _x6852 = _x6; /*int*/
  }
  _b_5783_5782 = kk_integer_sub(_x6852,year_shift,kk_context()); /*int*/
  kk_integer_t _x6853;
  kk_std_core_types__optional _match_5830 = kk_std_core_types__new_Optional(kk_integer_box(_b_5783_5782, _ctx), _ctx); /*optional<116>*/;
  if (kk_std_core_types__is_Optional(_match_5830, _ctx)) {
    kk_box_t _box_x5784 = _match_5830._cons.Optional.value;
    kk_integer_t _year_143 = kk_integer_unbox(_box_x5784, _ctx);
    kk_integer_dup(_year_143, _ctx);
    kk_std_core_types__optional_drop(_match_5830, _ctx);
    _x6853 = _year_143; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(_match_5830, _ctx);
    {
      kk_integer_t _x5 = d01.year;
      kk_integer_dup(_x5, _ctx);
      _x6853 = _x5; /*int*/
    }
  }
  kk_integer_t _x6854;
  kk_std_core_types__optional _match_5829 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5829, _ctx)) {
    kk_box_t _box_x5785 = _match_5829._cons.Optional.value;
    kk_integer_t _month_149 = kk_integer_unbox(_box_x5785, _ctx);
    kk_integer_dup(_month_149, _ctx);
    kk_std_core_types__optional_drop(_match_5829, _ctx);
    _x6854 = _month_149; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(_match_5829, _ctx);
    {
      kk_integer_t _x00 = d01.month;
      kk_integer_dup(_x00, _ctx);
      _x6854 = _x00; /*int*/
    }
  }
  kk_integer_t _x6855;
  kk_std_core_types__optional _match_5828 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5828, _ctx)) {
    kk_box_t _box_x5786 = _match_5828._cons.Optional.value;
    kk_integer_t _day_155 = kk_integer_unbox(_box_x5786, _ctx);
    kk_std_time_date__date_drop(d01, _ctx);
    kk_integer_dup(_day_155, _ctx);
    kk_std_core_types__optional_drop(_match_5828, _ctx);
    _x6855 = _day_155; /*int*/
  }
  else {
    kk_std_core_types__optional_drop(_match_5828, _ctx);
    {
      kk_integer_t _x10 = d01.day;
      kk_integer_dup(_x10, _ctx);
      kk_std_time_date__date_drop(d01, _ctx);
      _x6855 = _x10; /*int*/
    }
  }
  return kk_std_time_date__new_Date(_x6853, _x6854, _x6855, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_year_shift_earth_calendar_fun6856__t {
  struct kk_function_s _base;
  kk_std_time_calendar__calendar cal;
  kk_integer_t year_shift;
};
static kk_integer_t kk_std_time_calendar_year_shift_earth_calendar_fun6856(kk_function_t _fself, kk_std_time_date__date d1, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_year_shift_earth_calendar_fun6856(kk_std_time_calendar__calendar cal, kk_integer_t year_shift, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6856__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6856__t, 3, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_year_shift_earth_calendar_fun6856, kk_context());
  _self->cal = cal;
  _self->year_shift = year_shift;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_integer_t kk_std_time_calendar_year_shift_earth_calendar_fun6856(kk_function_t _fself, kk_std_time_date__date d1, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6856__t* _self = kk_function_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6856__t*, _fself, _ctx);
  kk_std_time_calendar__calendar cal = _self->cal; /* std/time/calendar/calendar */
  kk_integer_t year_shift = _self->year_shift; /* int */
  kk_drop_match(_self, {kk_std_time_calendar__calendar_dup(cal, _ctx);kk_integer_dup(year_shift, _ctx);}, {}, _ctx)
  kk_integer_t _b_5788_5787;
  kk_integer_t _x6857;
  {
    kk_integer_t _x9 = d1.year;
    kk_integer_dup(_x9, _ctx);
    _x6857 = _x9; /*int*/
  }
  _b_5788_5787 = kk_integer_add(_x6857,year_shift,kk_context()); /*int*/
  {
    struct kk_std_time_calendar_Calendar* _con6858 = kk_std_time_calendar__as_Calendar(cal, _ctx);
    kk_string_t _pat08 = _con6858->name;
    kk_string_t _pat113 = _con6858->long_name;
    kk_string_t _pat28 = _con6858->month_prefix;
    kk_function_t _pat35 = _con6858->show_era;
    kk_function_t _pat44 = _con6858->instant_to_dc;
    kk_function_t _pat54 = _con6858->dc_to_instant;
    kk_function_t _pat61 = _con6858->days_to_date;
    kk_function_t _x7 = _con6858->date_to_days;
    if kk_likely(kk_datatype_ptr_is_unique(cal, _ctx)) {
      kk_function_drop(_pat61, _ctx);
      kk_function_drop(_pat54, _ctx);
      kk_function_drop(_pat44, _ctx);
      kk_function_drop(_pat35, _ctx);
      kk_string_drop(_pat28, _ctx);
      kk_string_drop(_pat113, _ctx);
      kk_string_drop(_pat08, _ctx);
      kk_datatype_ptr_free(cal, _ctx);
    }
    else {
      kk_function_dup(_x7, _ctx);
      kk_datatype_ptr_decref(cal, _ctx);
    }
    kk_std_time_date__date _x6859;
    kk_integer_t _x6860;
    kk_std_core_types__optional _match_5827 = kk_std_core_types__new_Optional(kk_integer_box(_b_5788_5787, _ctx), _ctx); /*optional<116>*/;
    if (kk_std_core_types__is_Optional(_match_5827, _ctx)) {
      kk_box_t _box_x5789 = _match_5827._cons.Optional.value;
      kk_integer_t _year_1430 = kk_integer_unbox(_box_x5789, _ctx);
      kk_integer_dup(_year_1430, _ctx);
      kk_std_core_types__optional_drop(_match_5827, _ctx);
      _x6860 = _year_1430; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(_match_5827, _ctx);
      {
        kk_integer_t _x8 = d1.year;
        kk_integer_dup(_x8, _ctx);
        _x6860 = _x8; /*int*/
      }
    }
    kk_integer_t _x6861;
    kk_std_core_types__optional _match_5826 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_5826, _ctx)) {
      kk_box_t _box_x5790 = _match_5826._cons.Optional.value;
      kk_integer_t _month_1490 = kk_integer_unbox(_box_x5790, _ctx);
      kk_integer_dup(_month_1490, _ctx);
      kk_std_core_types__optional_drop(_match_5826, _ctx);
      _x6861 = _month_1490; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(_match_5826, _ctx);
      {
        kk_integer_t _x01 = d1.month;
        kk_integer_dup(_x01, _ctx);
        _x6861 = _x01; /*int*/
      }
    }
    kk_integer_t _x6862;
    kk_std_core_types__optional _match_5825 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_5825, _ctx)) {
      kk_box_t _box_x5791 = _match_5825._cons.Optional.value;
      kk_integer_t _day_1550 = kk_integer_unbox(_box_x5791, _ctx);
      kk_std_time_date__date_drop(d1, _ctx);
      kk_integer_dup(_day_1550, _ctx);
      kk_std_core_types__optional_drop(_match_5825, _ctx);
      _x6862 = _day_1550; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(_match_5825, _ctx);
      {
        kk_integer_t _x11 = d1.day;
        kk_integer_dup(_x11, _ctx);
        kk_std_time_date__date_drop(d1, _ctx);
        _x6862 = _x11; /*int*/
      }
    }
    _x6859 = kk_std_time_date__new_Date(_x6860, _x6861, _x6862, _ctx); /*std/time/date/date*/
    return kk_function_call(kk_integer_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _x7, (_x7, _x6859, _ctx), _ctx);
  }
}


// lift anonymous function
struct kk_std_time_calendar_year_shift_earth_calendar_fun6866__t {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x5796;
};
static kk_string_t kk_std_time_calendar_year_shift_earth_calendar_fun6866(kk_function_t _fself, kk_std_time_date__date _b_5799, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_year_shift_earth_calendar_fun6866(kk_box_t _fun_unbox_x5796, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6866__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6866__t, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_year_shift_earth_calendar_fun6866, kk_context());
  _self->_fun_unbox_x5796 = _fun_unbox_x5796;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_string_t kk_std_time_calendar_year_shift_earth_calendar_fun6866(kk_function_t _fself, kk_std_time_date__date _b_5799, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6866__t* _self = kk_function_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6866__t*, _fself, _ctx);
  kk_box_t _fun_unbox_x5796 = _self->_fun_unbox_x5796; /* 116 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x5796, _ctx);}, {}, _ctx)
  kk_box_t _x6867;
  kk_function_t _x6868 = kk_function_unbox(_fun_unbox_x5796, _ctx); /*(5797) -> 5798*/
  _x6867 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6868, (_x6868, kk_std_time_date__date_box(_b_5799, _ctx), _ctx), _ctx); /*5798*/
  return kk_string_unbox(_x6867);
}


// lift anonymous function
struct kk_std_time_calendar_year_shift_earth_calendar_fun6869__t {
  struct kk_function_s _base;
};
static kk_string_t kk_std_time_calendar_year_shift_earth_calendar_fun6869(kk_function_t _fself, kk_std_time_date__date d2, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_year_shift_earth_calendar_fun6869(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_year_shift_earth_calendar_fun6869, _ctx)
  return kk_function_dup(_fself,kk_context());
}

static kk_string_t kk_std_time_calendar_year_shift_earth_calendar_fun6869(kk_function_t _fself, kk_std_time_date__date d2, kk_context_t* _ctx) {
  kk_unused(_fself);
  kk_std_time_date__date_drop(d2, _ctx);
  return kk_string_empty();
}


// lift anonymous function
struct kk_std_time_calendar_year_shift_earth_calendar_fun6872__t {
  struct kk_function_s _base;
  kk_function_t days_to_date0_4916;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_year_shift_earth_calendar_fun6872(kk_function_t _fself, kk_std_time_instant__instant i, kk_std_time_duration__duration tzdelta, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_year_shift_earth_calendar_fun6872(kk_function_t days_to_date0_4916, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6872__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6872__t, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_year_shift_earth_calendar_fun6872, kk_context());
  _self->days_to_date0_4916 = days_to_date0_4916;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_year_shift_earth_calendar_fun6872(kk_function_t _fself, kk_std_time_instant__instant i, kk_std_time_duration__duration tzdelta, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6872__t* _self = kk_function_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6872__t*, _fself, _ctx);
  kk_function_t days_to_date0_4916 = _self->days_to_date0_4916; /* (days : int) -> std/time/date/date */
  kk_drop_match(_self, {kk_function_dup(days_to_date0_4916, _ctx);}, {}, _ctx)
  kk_std_core_types__tuple2_ _match_5824;
  kk_std_time_timestamp__timestamp _x6873;
  kk_std_num_ddouble__ddouble _x6874;
  kk_std_num_ddouble__ddouble _x6875;
  {
    kk_std_time_timestamp__timestamp _x1 = i.since;
    kk_std_time_instant__timescale _pat22 = i.ts;
    struct kk_std_time_timestamp_Timestamp* _con6876 = kk_std_time_timestamp__as_Timestamp(_x1, _ctx);
    struct kk_std_time_instant_Timescale* _con6877 = kk_std_time_instant__as_Timescale(_pat22, _ctx);
    kk_std_time_timestamp__timestamp_dup(_x1, _ctx);
    {
      struct kk_std_time_timestamp_Timestamp* _con6878 = kk_std_time_timestamp__as_Timestamp(_x1, _ctx);
      kk_std_num_ddouble__ddouble _x = _con6878->since;
      if kk_likely(kk_datatype_ptr_is_unique(_x1, _ctx)) {
        kk_datatype_ptr_free(_x1, _ctx);
      }
      else {
        kk_datatype_ptr_decref(_x1, _ctx);
      }
      _x6875 = _x; /*std/time/timestamp/timespan*/
    }
  }
  kk_std_num_ddouble__ddouble _x6879;
  {
    kk_std_num_ddouble__ddouble _x2 = tzdelta.secs;
    _x6879 = _x2; /*std/time/timestamp/timespan*/
  }
  _x6874 = kk_std_num_ddouble__lp__plus__rp_(_x6875, _x6879, _ctx); /*std/num/ddouble/ddouble*/
  int32_t _x6880;
  {
    kk_std_time_timestamp__timestamp _x12 = i.since;
    kk_std_time_instant__timescale _pat220 = i.ts;
    struct kk_std_time_timestamp_Timestamp* _con6881 = kk_std_time_timestamp__as_Timestamp(_x12, _ctx);
    struct kk_std_time_instant_Timescale* _con6882 = kk_std_time_instant__as_Timescale(_pat220, _ctx);
    kk_std_time_timestamp__timestamp_dup(_x12, _ctx);
    kk_std_time_instant__instant_drop(i, _ctx);
    {
      struct kk_std_time_timestamp_Timestamp* _con6883 = kk_std_time_timestamp__as_Timestamp(_x12, _ctx);
      kk_std_num_ddouble__ddouble _pat000 = _con6883->since;
      int32_t _x0 = _con6883->leap32;
      if kk_likely(kk_datatype_ptr_is_unique(_x12, _ctx)) {
        kk_datatype_ptr_free(_x12, _ctx);
      }
      else {
        kk_datatype_ptr_decref(_x12, _ctx);
      }
      _x6880 = _x0; /*int32*/
    }
  }
  _x6873 = kk_std_time_timestamp__new_Timestamp(kk_reuse_null, 0, _x6874, _x6880, _ctx); /*std/time/timestamp/timestamp*/
  _match_5824 = kk_std_time_timestamp_days_clock(_x6873, _ctx); /*(int, std/time/date/clock)*/
  {
    kk_box_t _box_x5800 = _match_5824.fst;
    kk_box_t _box_x5801 = _match_5824.snd;
    kk_std_time_date__clock clock = kk_std_time_date__clock_unbox(_box_x5801, KK_BORROWED, _ctx);
    struct kk_std_time_date_Clock* _con6884 = kk_std_time_date__as_Clock(clock, _ctx);
    kk_integer_t days = kk_integer_unbox(_box_x5800, _ctx);
    kk_std_time_date__clock_dup(clock, _ctx);
    kk_integer_dup(days, _ctx);
    kk_std_core_types__tuple2__drop(_match_5824, _ctx);
    kk_std_time_date__date _b_5819_5802 = kk_function_call(kk_std_time_date__date, (kk_function_t, kk_integer_t, kk_context_t*), days_to_date0_4916, (days_to_date0_4916, days, _ctx), _ctx); /*std/time/date/date*/;
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_date__date_box(_b_5819_5802, _ctx), kk_std_time_date__clock_box(clock, _ctx), _ctx);
  }
}


// lift anonymous function
struct kk_std_time_calendar_year_shift_earth_calendar_fun6886__t {
  struct kk_function_s _base;
  kk_function_t date_to_days0_4917;
};
static kk_std_time_instant__instant kk_std_time_calendar_year_shift_earth_calendar_fun6886(kk_function_t _fself, kk_std_time_date__date d00, kk_std_time_date__clock c, kk_std_time_calendar__timezone tz, kk_std_time_instant__timescale ts0, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_year_shift_earth_calendar_fun6886(kk_function_t date_to_days0_4917, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6886__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6886__t, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_time_calendar_year_shift_earth_calendar_fun6886, kk_context());
  _self->date_to_days0_4917 = date_to_days0_4917;
  return kk_datatype_from_base(&_self->_base, kk_context());
}

static kk_std_time_instant__instant kk_std_time_calendar_year_shift_earth_calendar_fun6886(kk_function_t _fself, kk_std_time_date__date d00, kk_std_time_date__clock c, kk_std_time_calendar__timezone tz, kk_std_time_instant__timescale ts0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6886__t* _self = kk_function_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6886__t*, _fself, _ctx);
  kk_function_t date_to_days0_4917 = _self->date_to_days0_4917; /* (std/time/date/date) -> int */
  kk_drop_match(_self, {kk_function_dup(date_to_days0_4917, _ctx);}, {}, _ctx)
  kk_std_time_timestamp__timestamp t_5004;
  kk_std_core_types__optional _x6887;
  kk_box_t _x6888;
  bool _x6889;
  {
    struct kk_std_time_instant_Timescale* _con6890 = kk_std_time_instant__as_Timescale(ts0, _ctx);
    kk_std_core_types__maybe _x31 = _con6890->mb_seconds_in_day;
    kk_std_core_types__maybe_dup(_x31, _ctx);
    if (kk_std_core_types__is_Nothing(_x31, _ctx)) {
      _x6889 = false; /*bool*/
    }
    else {
      kk_std_core_types__maybe_drop(_x31, _ctx);
      _x6889 = true; /*bool*/
    }
  }
  _x6888 = kk_bool_box(_x6889); /*116*/
  _x6887 = kk_std_core_types__new_Optional(_x6888, _ctx); /*optional<116>*/
  t_5004 = kk_std_time_calendar_dc_timestamp(d00, c, date_to_days0_4917, _x6887, _ctx); /*std/time/timestamp/timestamp*/
  kk_std_time_instant__instant i0;
  kk_std_time_timestamp__timestamp _x6891 = kk_std_time_timestamp__timestamp_dup(t_5004, _ctx); /*std/time/timestamp/timestamp*/
  kk_std_time_instant__timescale _x6892 = kk_std_time_instant__timescale_dup(ts0, _ctx); /*std/time/instant/timescale*/
  i0 = kk_std_time_instant__new_Instant(_x6891, _x6892, _ctx); /*std/time/instant/instant*/
  bool _match_5821;
  kk_string_t _x6893;
  {
    struct kk_std_time_calendar_Timezone* _con6894 = kk_std_time_calendar__as_Timezone(tz, _ctx);
    kk_string_t _x3 = _con6894->name;
    kk_string_dup(_x3, _ctx);
    _x6893 = _x3; /*string*/
  }
  kk_string_t _x6895;
  kk_std_time_calendar__timezone _x6896 = kk_std_time_calendar_tz_utc; /*std/time/calendar/timezone*/
  {
    struct kk_std_time_calendar_Timezone* _con6897 = kk_std_time_calendar__as_Timezone(_x6896, _ctx);
    kk_string_t _x02 = _con6897->name;
    _x6895 = kk_string_dup(_x02, _ctx); /*string*/
  }
  _match_5821 = kk_string_is_eq(_x6893,_x6895,kk_context()); /*bool*/
  if (_match_5821) {
    kk_datatype_ptr_dropn(tz, (KK_I32(3)), _ctx);
    kk_datatype_ptr_dropn(ts0, (KK_I32(10)), _ctx);
    kk_datatype_ptr_dropn(t_5004, (KK_I32(0)), _ctx);
    return i0;
  }
  {
    struct kk_std_time_calendar_Timezone* _con6898 = kk_std_time_calendar__as_Timezone(tz, _ctx);
    kk_string_t _pat0220 = _con6898->name;
    kk_function_t _pat118 = _con6898->utc_delta;
    kk_function_t _x120 = _con6898->utc_inverse;
    kk_function_dup(_x120, _ctx);
    kk_std_core_types__maybe _match_5822;
    kk_std_time_instant__instant _x6899 = kk_std_time_instant__instant_dup(i0, _ctx); /*std/time/instant/instant*/
    _match_5822 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x120, (_x120, _x6899, _ctx), _ctx); /*maybe<std/time/instant/instant>*/
    if (kk_std_core_types__is_Just(_match_5822, _ctx)) {
      kk_box_t _box_x5806 = _match_5822._cons.Just.value;
      kk_std_time_instant__instant inv = kk_std_time_instant__instant_unbox(_box_x5806, KK_BORROWED, _ctx);
      if kk_likely(kk_datatype_ptr_is_unique(tz, _ctx)) {
        kk_function_drop(_x120, _ctx);
        kk_function_drop(_pat118, _ctx);
        kk_string_drop(_pat0220, _ctx);
        kk_datatype_ptr_free(tz, _ctx);
      }
      else {
        kk_datatype_ptr_decref(tz, _ctx);
      }
      kk_datatype_ptr_dropn(ts0, (KK_I32(10)), _ctx);
      kk_datatype_ptr_dropn(t_5004, (KK_I32(0)), _ctx);
      kk_std_time_instant__instant_drop(i0, _ctx);
      kk_std_time_instant__instant_dup(inv, _ctx);
      kk_std_core_types__maybe_drop(_match_5822, _ctx);
      return inv;
    }
    {
      kk_std_core_types__tuple2_ _this_4787;
      {
        struct kk_std_time_calendar_Timezone* _con6900 = kk_std_time_calendar__as_Timezone(tz, _ctx);
        kk_function_t _x30 = _con6900->utc_delta;
        kk_function_dup(_x30, _ctx);
        _this_4787 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x30, (_x30, i0, _ctx), _ctx); /*(std/time/duration/duration, string)*/
      }
      kk_std_time_timestamp__timestamp t1_4790;
      kk_std_num_ddouble__ddouble _x6901;
      kk_std_num_ddouble__ddouble _x6902;
      {
        struct kk_std_time_timestamp_Timestamp* _con6903 = kk_std_time_timestamp__as_Timestamp(t_5004, _ctx);
        kk_std_num_ddouble__ddouble _x100 = _con6903->since;
        _x6902 = _x100; /*std/time/timestamp/timespan*/
      }
      kk_std_num_ddouble__ddouble _x6904;
      double _x6905;
      double _x6906;
      {
        kk_box_t _box_x5807 = _this_4787.fst;
        kk_box_t _box_x5808 = _this_4787.snd;
        kk_std_time_duration__duration _x20 = kk_std_time_duration__duration_unbox(_box_x5807, KK_BORROWED, _ctx);
        {
          kk_std_num_ddouble__ddouble _x50 = _x20.secs;
          {
            double _x40 = _x50.hi;
            _x6906 = _x40; /*float64*/
          }
        }
      }
      _x6905 = (-_x6906); /*float64*/
      double _x6907;
      double _x6908;
      {
        kk_box_t _box_x5809 = _this_4787.fst;
        kk_box_t _box_x5810 = _this_4787.snd;
        kk_std_time_duration__duration _x200 = kk_std_time_duration__duration_unbox(_box_x5809, KK_BORROWED, _ctx);
        {
          kk_std_num_ddouble__ddouble _x500 = _x200.secs;
          {
            double _x000 = _x500.lo;
            _x6908 = _x000; /*float64*/
          }
        }
      }
      _x6907 = (-_x6908); /*float64*/
      _x6904 = kk_std_num_ddouble__new_Ddouble(_x6905, _x6907, _ctx); /*std/num/ddouble/ddouble*/
      _x6901 = kk_std_num_ddouble__lp__plus__rp_(_x6902, _x6904, _ctx); /*std/num/ddouble/ddouble*/
      int32_t _x6909;
      {
        struct kk_std_time_timestamp_Timestamp* _con6910 = kk_std_time_timestamp__as_Timestamp(t_5004, _ctx);
        kk_std_num_ddouble__ddouble _pat0200 = _con6910->since;
        int32_t _x2000 = _con6910->leap32;
        _x6909 = _x2000; /*int32*/
      }
      t1_4790 = kk_std_time_timestamp__new_Timestamp(kk_reuse_null, 0, _x6901, _x6909, _ctx); /*std/time/timestamp/timestamp*/
      kk_std_time_instant__instant tzi1;
      kk_std_time_instant__timescale _x6911 = kk_std_time_instant__timescale_dup(ts0, _ctx); /*std/time/instant/timescale*/
      tzi1 = kk_std_time_instant__new_Instant(t1_4790, _x6911, _ctx); /*std/time/instant/instant*/
      kk_std_core_types__tuple2_ _this0_4794;
      {
        struct kk_std_time_calendar_Timezone* _con6912 = kk_std_time_calendar__as_Timezone(tz, _ctx);
        kk_string_t _pat080 = _con6912->name;
        kk_function_t _x70 = _con6912->utc_delta;
        kk_function_t _pat170 = _con6912->utc_inverse;
        if kk_likely(kk_datatype_ptr_is_unique(tz, _ctx)) {
          kk_function_drop(_pat170, _ctx);
          kk_string_drop(_pat080, _ctx);
          kk_datatype_ptr_free(tz, _ctx);
        }
        else {
          kk_function_dup(_x70, _ctx);
          kk_datatype_ptr_decref(tz, _ctx);
        }
        kk_std_time_instant__instant _x6913 = kk_std_time_instant__instant_dup(tzi1, _ctx); /*std/time/instant/instant*/
        _this0_4794 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x70, (_x70, _x6913, _ctx), _ctx); /*(std/time/duration/duration, string)*/
      }
      bool _match_5823;
      kk_std_time_duration__duration _x6914;
      {
        kk_box_t _box_x5811 = _this_4787.fst;
        kk_box_t _box_x5812 = _this_4787.snd;
        kk_std_time_duration__duration _x201 = kk_std_time_duration__duration_unbox(_box_x5811, KK_BORROWED, _ctx);
        kk_std_core_types__tuple2__drop(_this_4787, _ctx);
        _x6914 = _x201; /*std/time/duration/duration*/
      }
      kk_std_time_duration__duration _x6915;
      {
        kk_box_t _box_x5813 = _this0_4794.fst;
        kk_box_t _box_x5814 = _this0_4794.snd;
        kk_std_time_duration__duration _x60 = kk_std_time_duration__duration_unbox(_box_x5813, KK_BORROWED, _ctx);
        _x6915 = _x60; /*std/time/duration/duration*/
      }
      _match_5823 = kk_std_time_duration__lp__eq__eq__rp_(_x6914, _x6915, _ctx); /*bool*/
      if (_match_5823) {
        kk_datatype_ptr_dropn(ts0, (KK_I32(10)), _ctx);
        kk_datatype_ptr_dropn(t_5004, (KK_I32(0)), _ctx);
        kk_std_core_types__tuple2__drop(_this0_4794, _ctx);
        return tzi1;
      }
      {
        kk_std_time_instant__instant_drop(tzi1, _ctx);
        kk_std_time_timestamp__timestamp t3_4797;
        kk_std_num_ddouble__ddouble _x6916;
        kk_std_num_ddouble__ddouble _x6917;
        {
          struct kk_std_time_timestamp_Timestamp* _con6918 = kk_std_time_timestamp__as_Timestamp(t_5004, _ctx);
          kk_std_num_ddouble__ddouble _x110 = _con6918->since;
          _x6917 = _x110; /*std/time/timestamp/timespan*/
        }
        kk_std_num_ddouble__ddouble _x6919;
        double _x6920;
        double _x6921;
        {
          kk_box_t _box_x5815 = _this0_4794.fst;
          kk_box_t _box_x5816 = _this0_4794.snd;
          kk_std_time_duration__duration _x600 = kk_std_time_duration__duration_unbox(_box_x5815, KK_BORROWED, _ctx);
          {
            kk_std_num_ddouble__ddouble _x90 = _x600.secs;
            {
              double _x80 = _x90.hi;
              _x6921 = _x80; /*float64*/
            }
          }
        }
        _x6920 = (-_x6921); /*float64*/
        double _x6922;
        double _x6923;
        {
          kk_box_t _box_x5817 = _this0_4794.fst;
          kk_box_t _box_x5818 = _this0_4794.snd;
          kk_std_time_duration__duration _x601 = kk_std_time_duration__duration_unbox(_box_x5817, KK_BORROWED, _ctx);
          kk_std_core_types__tuple2__drop(_this0_4794, _ctx);
          {
            kk_std_num_ddouble__ddouble _x900 = _x601.secs;
            {
              double _x010 = _x900.lo;
              _x6923 = _x010; /*float64*/
            }
          }
        }
        _x6922 = (-_x6923); /*float64*/
        _x6919 = kk_std_num_ddouble__new_Ddouble(_x6920, _x6922, _ctx); /*std/num/ddouble/ddouble*/
        _x6916 = kk_std_num_ddouble__lp__plus__rp_(_x6917, _x6919, _ctx); /*std/num/ddouble/ddouble*/
        int32_t _x6924;
        {
          struct kk_std_time_timestamp_Timestamp* _con6925 = kk_std_time_timestamp__as_Timestamp(t_5004, _ctx);
          kk_std_num_ddouble__ddouble _pat0210 = _con6925->since;
          int32_t _x21 = _con6925->leap32;
          if kk_likely(kk_datatype_ptr_is_unique(t_5004, _ctx)) {
            kk_datatype_ptr_free(t_5004, _ctx);
          }
          else {
            kk_datatype_ptr_decref(t_5004, _ctx);
          }
          _x6924 = _x21; /*int32*/
        }
        t3_4797 = kk_std_time_timestamp__new_Timestamp(kk_reuse_null, 0, _x6916, _x6924, _ctx); /*std/time/timestamp/timestamp*/
        return kk_std_time_instant__new_Instant(t3_4797, ts0, _ctx);
      }
    }
  }
}

kk_std_time_calendar__calendar kk_std_time_calendar_year_shift_earth_calendar(kk_string_t name0, kk_string_t long_name0, kk_integer_t year_shift, kk_std_time_calendar__calendar cal, kk_std_core_types__optional month_prefix0, kk_std_core_types__optional show_era0, kk_context_t* _ctx) { /* (name : string, long-name : string, year-shift : int, cal : calendar, month-prefix : optional<string>, show-era : optional<(std/time/date/date) -> string>) -> calendar */ 
  kk_function_t days_to_date0_4916;
  kk_std_time_calendar__calendar_dup(cal, _ctx);
  kk_integer_dup(year_shift, _ctx);
  days_to_date0_4916 = kk_std_time_calendar_new_year_shift_earth_calendar_fun6850(cal, year_shift, _ctx); /*(days0 : int) -> std/time/date/date*/
  kk_function_t date_to_days0_4917 = kk_std_time_calendar_new_year_shift_earth_calendar_fun6856(cal, year_shift, _ctx); /*(d1 : std/time/date/date) -> int*/;
  kk_string_t _x6863;
  if (kk_std_core_types__is_Optional(month_prefix0, _ctx)) {
    kk_box_t _box_x5792 = month_prefix0._cons.Optional.value;
    kk_string_t _month_prefix_4475 = kk_string_unbox(_box_x5792);
    kk_string_dup(_month_prefix_4475, _ctx);
    kk_std_core_types__optional_drop(month_prefix0, _ctx);
    _x6863 = _month_prefix_4475; /*string*/
  }
  else {
    kk_std_core_types__optional_drop(month_prefix0, _ctx);
    _x6863 = kk_string_empty(); /*string*/
  }
  kk_function_t _x6865;
  if (kk_std_core_types__is_Optional(show_era0, _ctx)) {
    kk_box_t _fun_unbox_x5796 = show_era0._cons.Optional.value;
    kk_box_dup(_fun_unbox_x5796, _ctx);
    kk_std_core_types__optional_drop(show_era0, _ctx);
    _x6865 = kk_std_time_calendar_new_year_shift_earth_calendar_fun6866(_fun_unbox_x5796, _ctx); /*(std/time/date/date) -> string*/
  }
  else {
    kk_std_core_types__optional_drop(show_era0, _ctx);
    _x6865 = kk_std_time_calendar_new_year_shift_earth_calendar_fun6869(_ctx); /*(std/time/date/date) -> string*/
  }
  kk_function_t _x6871;
  kk_function_dup(days_to_date0_4916, _ctx);
  _x6871 = kk_std_time_calendar_new_year_shift_earth_calendar_fun6872(days_to_date0_4916, _ctx); /*(i : std/time/instant/instant, tzdelta : std/time/duration/duration) -> (6, 7)*/
  kk_function_t _x6885;
  kk_function_dup(date_to_days0_4917, _ctx);
  _x6885 = kk_std_time_calendar_new_year_shift_earth_calendar_fun6886(date_to_days0_4917, _ctx); /*(d00 : std/time/date/date, c : std/time/date/clock, tz : std/time/calendar/timezone, ts0 : std/time/instant/timescale) -> std/time/instant/instant*/
  return kk_std_time_calendar__new_Calendar(kk_reuse_null, 0, name0, long_name0, _x6863, _x6865, _x6871, _x6885, days_to_date0_4916, date_to_days0_4917, _ctx);
}

// initialization
void kk_std_time_calendar__init(kk_context_t* _ctx){
  static bool _kk_initialized = false;
  if (_kk_initialized) return;
  _kk_initialized = true;
  kk_std_core_types__init(_ctx);
  kk_std_core_hnd__init(_ctx);
  kk_std_core__init(_ctx);
  kk_std_num_float64__init(_ctx);
  kk_std_num_ddouble__init(_ctx);
  kk_std_time_date__init(_ctx);
  kk_std_time_timestamp__init(_ctx);
  kk_std_time_duration__init(_ctx);
  kk_std_time_instant__init(_ctx);
  kk_std_time_utc__init(_ctx);
  #if defined(KK_CUSTOM_INIT)
    KK_CUSTOM_INIT (_ctx);
  #endif
  {
    kk_string_t _x6191;
    kk_define_string_literal(, _s6192, 3, "UTC", _ctx)
    _x6191 = kk_string_dup(_s6192, _ctx); /*string*/
    kk_std_time_calendar_tz_utc = kk_std_time_calendar__new_Timezone(kk_reuse_null, 0, _x6191, kk_std_time_calendar_new_tz_utc_fun6193(_ctx), kk_std_time_calendar_new_tz_utc_fun6197(_ctx), _ctx); /*std/time/calendar/timezone*/
  }
  {
    kk_string_t _x6522 = kk_string_empty(); /*string*/
    kk_string_t _x6524;
    kk_define_string_literal(, _s6525, 3, "ISO", _ctx)
    _x6524 = kk_string_dup(_s6525, _ctx); /*string*/
    kk_std_core_types__optional _x6542;
    kk_box_t _x6543;
    kk_integer_t _x6544;
    kk_std_core_types__optional _match_5904 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_5904, _ctx)) {
      kk_box_t _box_x5492 = _match_5904._cons.Optional.value;
      kk_integer_t _epoch_shift_2777 = kk_integer_unbox(_box_x5492, _ctx);
      kk_integer_dup(_epoch_shift_2777, _ctx);
      kk_std_core_types__optional_drop(_match_5904, _ctx);
      _x6544 = _epoch_shift_2777; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(_match_5904, _ctx);
      _x6544 = kk_integer_from_int(730119, _ctx); /*int*/
    }
    _x6543 = kk_integer_box(_x6544, _ctx); /*116*/
    _x6542 = kk_std_core_types__new_Optional(_x6543, _ctx); /*optional<116>*/
    kk_std_core_types__optional _x6545;
    kk_box_t _x6546;
    bool _x6547;
    kk_std_core_types__optional _match_5903 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_5903, _ctx)) {
      kk_box_t _box_x5494 = _match_5903._cons.Optional.value;
      bool _has_year_zero_2781 = kk_bool_unbox(_box_x5494);
      kk_std_core_types__optional_drop(_match_5903, _ctx);
      _x6547 = _has_year_zero_2781; /*bool*/
    }
    else {
      kk_std_core_types__optional_drop(_match_5903, _ctx);
      _x6547 = true; /*bool*/
    }
    _x6546 = kk_bool_box(_x6547); /*116*/
    _x6545 = kk_std_core_types__new_Optional(_x6546, _ctx); /*optional<116>*/
    kk_std_core_types__optional _x6548;
    kk_box_t _x6549;
    kk_string_t _x6550;
    kk_std_core_types__optional _match_5902 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_5902, _ctx)) {
      kk_box_t _box_x5496 = _match_5902._cons.Optional.value;
      kk_string_t _month_prefix_2785 = kk_string_unbox(_box_x5496);
      kk_string_dup(_month_prefix_2785, _ctx);
      kk_std_core_types__optional_drop(_match_5902, _ctx);
      _x6550 = _month_prefix_2785; /*string*/
    }
    else {
      kk_std_core_types__optional_drop(_match_5902, _ctx);
      _x6550 = kk_string_empty(); /*string*/
    }
    _x6549 = kk_string_box(_x6550); /*116*/
    _x6548 = kk_std_core_types__new_Optional(_x6549, _ctx); /*optional<116>*/
    kk_std_core_types__optional _x6552 = kk_std_core_types__new_Optional(kk_function_box(kk_std_time_calendar_new_cal_iso_fun6553(_ctx), _ctx), _ctx); /*optional<116>*/
    kk_std_time_calendar_cal_iso = kk_std_time_calendar_solar_calendar(_x6522, _x6524, kk_std_time_calendar_new_cal_iso_fun6527(_ctx), kk_std_time_calendar_new_cal_iso_fun6528(_ctx), kk_std_time_calendar_new_cal_iso_fun6538(_ctx), kk_std_time_calendar_new_cal_iso_fun6541(_ctx), _x6542, _x6545, _x6548, _x6552, _ctx); /*std/time/calendar/calendar*/
  }
  {
    kk_string_t _x6615;
    kk_define_string_literal(, _s6616, 2, "GC", _ctx)
    _x6615 = kk_string_dup(_s6616, _ctx); /*string*/
    kk_string_t _x6617;
    kk_define_string_literal(, _s6618, 9, "Gregorian", _ctx)
    _x6617 = kk_string_dup(_s6618, _ctx); /*string*/
    kk_std_core_types__optional _x6635;
    kk_box_t _x6636;
    kk_integer_t _x6637;
    kk_std_core_types__optional _match_5883 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_5883, _ctx)) {
      kk_box_t _box_x5586 = _match_5883._cons.Optional.value;
      kk_integer_t _epoch_shift_2777 = kk_integer_unbox(_box_x5586, _ctx);
      kk_integer_dup(_epoch_shift_2777, _ctx);
      kk_std_core_types__optional_drop(_match_5883, _ctx);
      _x6637 = _epoch_shift_2777; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(_match_5883, _ctx);
      _x6637 = kk_integer_from_int(730119, _ctx); /*int*/
    }
    _x6636 = kk_integer_box(_x6637, _ctx); /*116*/
    _x6635 = kk_std_core_types__new_Optional(_x6636, _ctx); /*optional<116>*/
    kk_std_core_types__optional _x6638 = kk_std_core_types__new_Optional(kk_bool_box(false), _ctx); /*optional<116>*/
    kk_std_core_types__optional _x6639;
    kk_box_t _x6640;
    kk_string_t _x6641;
    kk_std_core_types__optional _match_5882 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_5882, _ctx)) {
      kk_box_t _box_x5589 = _match_5882._cons.Optional.value;
      kk_string_t _month_prefix_2785 = kk_string_unbox(_box_x5589);
      kk_string_dup(_month_prefix_2785, _ctx);
      kk_std_core_types__optional_drop(_match_5882, _ctx);
      _x6641 = _month_prefix_2785; /*string*/
    }
    else {
      kk_std_core_types__optional_drop(_match_5882, _ctx);
      _x6641 = kk_string_empty(); /*string*/
    }
    _x6640 = kk_string_box(_x6641); /*116*/
    _x6639 = kk_std_core_types__new_Optional(_x6640, _ctx); /*optional<116>*/
    kk_std_core_types__optional _x6643 = kk_std_core_types__new_Optional(kk_function_box(kk_std_time_calendar_new_cal_gregorian_fun6644(_ctx), _ctx), _ctx); /*optional<116>*/
    kk_std_time_calendar_cal_gregorian = kk_std_time_calendar_solar_calendar(_x6615, _x6617, kk_std_time_calendar_new_cal_gregorian_fun6620(_ctx), kk_std_time_calendar_new_cal_gregorian_fun6621(_ctx), kk_std_time_calendar_new_cal_gregorian_fun6631(_ctx), kk_std_time_calendar_new_cal_gregorian_fun6634(_ctx), _x6635, _x6638, _x6639, _x6643, _ctx); /*std/time/calendar/calendar*/
  }
  {
    kk_string_t _x6648;
    kk_define_string_literal(, _s6649, 3, "SLS", _ctx)
    _x6648 = kk_string_dup(_s6649, _ctx); /*string*/
    kk_string_t _x6650;
    kk_define_string_literal(, _s6651, 7, "ISO-SLS", _ctx)
    _x6650 = kk_string_dup(_s6651, _ctx); /*string*/
    kk_std_core_types__optional _x6668;
    kk_box_t _x6669;
    kk_integer_t _x6670;
    kk_std_core_types__optional _match_5873 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_5873, _ctx)) {
      kk_box_t _box_x5615 = _match_5873._cons.Optional.value;
      kk_integer_t _epoch_shift_2777 = kk_integer_unbox(_box_x5615, _ctx);
      kk_integer_dup(_epoch_shift_2777, _ctx);
      kk_std_core_types__optional_drop(_match_5873, _ctx);
      _x6670 = _epoch_shift_2777; /*int*/
    }
    else {
      kk_std_core_types__optional_drop(_match_5873, _ctx);
      _x6670 = kk_integer_from_int(730119, _ctx); /*int*/
    }
    _x6669 = kk_integer_box(_x6670, _ctx); /*116*/
    _x6668 = kk_std_core_types__new_Optional(_x6669, _ctx); /*optional<116>*/
    kk_std_core_types__optional _x6671;
    kk_box_t _x6672;
    bool _x6673;
    kk_std_core_types__optional _match_5872 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_5872, _ctx)) {
      kk_box_t _box_x5617 = _match_5872._cons.Optional.value;
      bool _has_year_zero_2781 = kk_bool_unbox(_box_x5617);
      kk_std_core_types__optional_drop(_match_5872, _ctx);
      _x6673 = _has_year_zero_2781; /*bool*/
    }
    else {
      kk_std_core_types__optional_drop(_match_5872, _ctx);
      _x6673 = true; /*bool*/
    }
    _x6672 = kk_bool_box(_x6673); /*116*/
    _x6671 = kk_std_core_types__new_Optional(_x6672, _ctx); /*optional<116>*/
    kk_std_core_types__optional _x6674;
    kk_box_t _x6675;
    kk_string_t _x6676;
    kk_std_core_types__optional _match_5871 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_5871, _ctx)) {
      kk_box_t _box_x5619 = _match_5871._cons.Optional.value;
      kk_string_t _month_prefix_2785 = kk_string_unbox(_box_x5619);
      kk_string_dup(_month_prefix_2785, _ctx);
      kk_std_core_types__optional_drop(_match_5871, _ctx);
      _x6676 = _month_prefix_2785; /*string*/
    }
    else {
      kk_std_core_types__optional_drop(_match_5871, _ctx);
      _x6676 = kk_string_empty(); /*string*/
    }
    _x6675 = kk_string_box(_x6676); /*116*/
    _x6674 = kk_std_core_types__new_Optional(_x6675, _ctx); /*optional<116>*/
    kk_std_core_types__optional _x6678 = kk_std_core_types__new_Optional(kk_function_box(kk_std_time_calendar_new_cal_iso_sls_fun6679(_ctx), _ctx), _ctx); /*optional<116>*/
    kk_std_time_calendar_cal_iso_sls = kk_std_time_calendar_solar_calendar(_x6648, _x6650, kk_std_time_calendar_new_cal_iso_sls_fun6653(_ctx), kk_std_time_calendar_new_cal_iso_sls_fun6654(_ctx), kk_std_time_calendar_new_cal_iso_sls_fun6664(_ctx), kk_std_time_calendar_new_cal_iso_sls_fun6667(_ctx), _x6668, _x6671, _x6674, _x6678, _ctx); /*std/time/calendar/calendar*/
  }
}

// termination
void kk_std_time_calendar__done(kk_context_t* _ctx){
  static bool _kk_done = false;
  if (_kk_done) return;
  _kk_done = true;
  #if defined(KK_CUSTOM_DONE)
    KK_CUSTOM_DONE (_ctx);
  #endif
  kk_std_time_calendar__calendar_drop(kk_std_time_calendar_cal_iso_sls, _ctx);
  kk_std_time_calendar__calendar_drop(kk_std_time_calendar_cal_gregorian, _ctx);
  kk_std_time_calendar__calendar_drop(kk_std_time_calendar_cal_iso, _ctx);
  kk_std_time_calendar__timezone_drop(kk_std_time_calendar_tz_utc, _ctx);
  kk_std_time_utc__done(_ctx);
  kk_std_time_instant__done(_ctx);
  kk_std_time_duration__done(_ctx);
  kk_std_time_timestamp__done(_ctx);
  kk_std_time_date__done(_ctx);
  kk_std_num_ddouble__done(_ctx);
  kk_std_num_float64__done(_ctx);
  kk_std_core__done(_ctx);
  kk_std_core_hnd__done(_ctx);
  kk_std_core_types__done(_ctx);
}
