#pragma once
#ifndef kk_std_core_H
#define kk_std_core_H
// Koka generated module: "std/core", koka version: 2.4.2, platform: 64-bit
#include <kklib.h>
#include "std_core_types.h"
#include "std_core_hnd.h"
/*---------------------------------------------------------------------------
  Copyright 2020-2021, Microsoft Research, Daan Leijen.

  This is free software; you can redistribute it and/or modify it under the
  terms of the Apache License, Version 2.0. A copy of the License can be
  found in the LICENSE file at the root of this distribution.
---------------------------------------------------------------------------*/

kk_box_t kk_std_core_error_pattern(kk_string_t location, kk_string_t definition, kk_context_t* _ctx);

static inline kk_std_core_types__order kk_int_as_order(int i,kk_context_t* ctx) {
  return (i==0 ? kk_std_core_types__new_Eq(ctx) : (i > 0 ? kk_std_core_types__new_Gt(ctx) : kk_std_core_types__new_Lt(ctx)));
}

static inline kk_std_core_types__maybe kk_integer_xparse( kk_string_t s, bool hex, kk_context_t* ctx ) {
  kk_integer_t i;
  bool ok = (hex ? kk_integer_hex_parse(kk_string_cbuf_borrow(s,NULL,ctx),&i,ctx) : kk_integer_parse(kk_string_cbuf_borrow(s,NULL,ctx),&i,ctx) );
  kk_string_drop(s,ctx);
  return (ok ? kk_std_core_types__new_Just(kk_integer_box(i,ctx),ctx) : kk_std_core_types__new_Nothing(ctx));
}

struct kk_std_core_Sslice;

kk_datatype_t kk_string_to_list(kk_string_t s, kk_context_t* ctx);
kk_string_t   kk_string_from_list(kk_datatype_t cs, kk_context_t* ctx);

kk_datatype_t  kk_vector_to_list(kk_vector_t v, kk_datatype_t tail, kk_context_t* ctx);
kk_vector_t    kk_list_to_vector(kk_datatype_t xs, kk_context_t* ctx);

static inline kk_integer_t  kk_string_count_int(kk_string_t s, kk_context_t* ctx) {
  return kk_integer_from_ssize_t( kk_string_count(s,ctx), ctx );
}

static inline kk_integer_t kk_string_cmp_int(kk_string_t s1, kk_string_t s2, kk_context_t* ctx) {
  return kk_integer_from_small( kk_string_cmp(s1,s2,ctx) );
}

kk_string_t  kk_string_join(kk_vector_t v, kk_context_t* ctx);
kk_string_t  kk_string_join_with(kk_vector_t v, kk_string_t sep, kk_context_t* ctx);
kk_string_t  kk_string_replace_all(kk_string_t str, kk_string_t pattern, kk_string_t repl, kk_context_t* ctx);

static inline kk_integer_t kk_string_count_pattern(kk_string_t str, kk_string_t pattern, kk_context_t* ctx) {
  kk_integer_t count = kk_integer_from_ssize_t( kk_string_count_pattern_borrow(str,pattern,ctx), ctx );
  kk_string_drop(str,ctx);
  kk_string_drop(pattern,ctx);
  return count;
}

kk_integer_t kk_slice_count( struct kk_std_core_Sslice sslice, kk_context_t* ctx );
kk_string_t  kk_slice_to_string( struct kk_std_core_Sslice sslice, kk_context_t* ctx );
struct kk_std_core_Sslice kk_slice_first( kk_string_t str, kk_context_t* ctx );
struct kk_std_core_Sslice kk_slice_last( kk_string_t str, kk_context_t* ctx );

struct kk_std_core_Sslice kk_slice_common_prefix_borrow( kk_string_t str1, kk_string_t str2, kk_integer_t upto, kk_context_t* ctx );
struct kk_std_core_Sslice kk_slice_advance_borrow( struct kk_std_core_Sslice slice, kk_integer_t count, kk_context_t* ctx );
struct kk_std_core_Sslice kk_slice_extend_borrow( struct kk_std_core_Sslice slice, kk_integer_t count, kk_context_t* ctx );
kk_std_core_types__maybe kk_slice_next( struct kk_std_core_Sslice slice, kk_context_t* ctx );


static inline kk_unit_t kk_vector_unsafe_assign( kk_vector_t v, kk_ssize_t i, kk_box_t x, kk_context_t* ctx  ) {
  kk_ssize_t len;
  kk_box_t* p = kk_vector_buf_borrow(v,&len,ctx);
  kk_assert(i < len);
  p[i] = x;
  kk_vector_drop(v,ctx); // TODO: use borrowing
  return kk_Unit;
}

kk_vector_t kk_vector_init( kk_ssize_t n, kk_function_t init, kk_context_t* ctx);

static inline kk_box_t kk_vector_at_int_borrow( kk_vector_t v, kk_integer_t n, kk_context_t* ctx) {
  // TODO: check bounds
  kk_box_t b = kk_vector_at_borrow(v,kk_integer_clamp_ssize_t_borrow(n,ctx),ctx);
  return b;
}

static inline double kk_double_abs(double d) {
  return (isfinite(d) && d < 0.0 ? -d : d);
}

static inline kk_std_core_types__tuple2_ kk_integer_div_mod_tuple(kk_integer_t x, kk_integer_t y, kk_context_t* ctx) {
  kk_integer_t mod;
  kk_integer_t div = kk_integer_div_mod(x,y,&mod,ctx);
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(div,ctx),kk_integer_box(mod,ctx),ctx);
}

kk_box_t kk_main_console( kk_function_t action, kk_context_t* ctx );
kk_unit_t kk_assert_fail( kk_string_t msg, kk_context_t* ctx );

struct kk_std_core_error_s;
struct kk_std_core_error_s kk_error_ok( kk_box_t result, kk_context_t* ctx );
struct kk_std_core_error_s kk_error_from_errno( int err, kk_context_t* ctx );




// type declarations

// type std/core/exception-info
struct kk_std_core__exception_info_s {
  kk_block_t _block;
  kk_string_t _tag;
};
typedef kk_datatype_ptr_t kk_std_core__exception_info;
struct kk_std_core_ExnError {
  struct kk_std_core__exception_info_s _base;
};
struct kk_std_core_ExnAssert {
  struct kk_std_core__exception_info_s _base;
};
struct kk_std_core_ExnTodo {
  struct kk_std_core__exception_info_s _base;
};
struct kk_std_core_ExnRange {
  struct kk_std_core__exception_info_s _base;
};
struct kk_std_core_ExnPattern {
  struct kk_std_core__exception_info_s _base;
  kk_string_t location;
  kk_string_t definition;
};
struct kk_std_core_ExnSystem {
  struct kk_std_core__exception_info_s _base;
  kk_integer_t kkloc_errno;
};
struct kk_std_core_ExnInternal {
  struct kk_std_core__exception_info_s _base;
  kk_string_t name;
};
static inline kk_std_core__exception_info kk_std_core__base_ExnError(struct kk_std_core_ExnError* _x, kk_context_t* _ctx) {
  return kk_datatype_from_base(&_x->_base, _ctx);
}
extern kk_string_t kk_std_core__tag_ExnError;
static inline kk_std_core__exception_info kk_std_core__new_ExnError(kk_context_t* _ctx) {
  struct kk_std_core_ExnError* _con = kk_block_alloc_at_as(struct kk_std_core_ExnError, kk_reuse_null, 1 /* scan count */, 0, KK_TAG_OPEN, _ctx);
  _con->_base._tag = kk_string_dup(kk_std_core__tag_ExnError, _ctx);
  return kk_std_core__base_ExnError(_con, _ctx);
}
static inline struct kk_std_core_ExnError* kk_std_core__as_ExnError(kk_std_core__exception_info x, kk_context_t* _ctx) {
  return kk_datatype_as_assert(struct kk_std_core_ExnError*, x, KK_TAG_OPEN, _ctx);
}
static inline kk_std_core__exception_info kk_std_core__base_ExnAssert(struct kk_std_core_ExnAssert* _x, kk_context_t* _ctx) {
  return kk_datatype_from_base(&_x->_base, _ctx);
}
extern kk_string_t kk_std_core__tag_ExnAssert;
static inline kk_std_core__exception_info kk_std_core__new_ExnAssert(kk_context_t* _ctx) {
  struct kk_std_core_ExnAssert* _con = kk_block_alloc_at_as(struct kk_std_core_ExnAssert, kk_reuse_null, 1 /* scan count */, 0, KK_TAG_OPEN, _ctx);
  _con->_base._tag = kk_string_dup(kk_std_core__tag_ExnAssert, _ctx);
  return kk_std_core__base_ExnAssert(_con, _ctx);
}
static inline struct kk_std_core_ExnAssert* kk_std_core__as_ExnAssert(kk_std_core__exception_info x, kk_context_t* _ctx) {
  return kk_datatype_as_assert(struct kk_std_core_ExnAssert*, x, KK_TAG_OPEN, _ctx);
}
static inline kk_std_core__exception_info kk_std_core__base_ExnTodo(struct kk_std_core_ExnTodo* _x, kk_context_t* _ctx) {
  return kk_datatype_from_base(&_x->_base, _ctx);
}
extern kk_string_t kk_std_core__tag_ExnTodo;
static inline kk_std_core__exception_info kk_std_core__new_ExnTodo(kk_context_t* _ctx) {
  struct kk_std_core_ExnTodo* _con = kk_block_alloc_at_as(struct kk_std_core_ExnTodo, kk_reuse_null, 1 /* scan count */, 0, KK_TAG_OPEN, _ctx);
  _con->_base._tag = kk_string_dup(kk_std_core__tag_ExnTodo, _ctx);
  return kk_std_core__base_ExnTodo(_con, _ctx);
}
static inline struct kk_std_core_ExnTodo* kk_std_core__as_ExnTodo(kk_std_core__exception_info x, kk_context_t* _ctx) {
  return kk_datatype_as_assert(struct kk_std_core_ExnTodo*, x, KK_TAG_OPEN, _ctx);
}
static inline kk_std_core__exception_info kk_std_core__base_ExnRange(struct kk_std_core_ExnRange* _x, kk_context_t* _ctx) {
  return kk_datatype_from_base(&_x->_base, _ctx);
}
extern kk_string_t kk_std_core__tag_ExnRange;
static inline kk_std_core__exception_info kk_std_core__new_ExnRange(kk_context_t* _ctx) {
  struct kk_std_core_ExnRange* _con = kk_block_alloc_at_as(struct kk_std_core_ExnRange, kk_reuse_null, 1 /* scan count */, 0, KK_TAG_OPEN, _ctx);
  _con->_base._tag = kk_string_dup(kk_std_core__tag_ExnRange, _ctx);
  return kk_std_core__base_ExnRange(_con, _ctx);
}
static inline struct kk_std_core_ExnRange* kk_std_core__as_ExnRange(kk_std_core__exception_info x, kk_context_t* _ctx) {
  return kk_datatype_as_assert(struct kk_std_core_ExnRange*, x, KK_TAG_OPEN, _ctx);
}
static inline kk_std_core__exception_info kk_std_core__base_ExnSystem(struct kk_std_core_ExnSystem* _x, kk_context_t* _ctx) {
  return kk_datatype_from_base(&_x->_base, _ctx);
}
extern kk_string_t kk_std_core__tag_ExnSystem;
static inline kk_std_core__exception_info kk_std_core__new_ExnSystem(kk_reuse_t _at, int32_t _cpath, kk_integer_t kkloc_errno, kk_context_t* _ctx) {
  struct kk_std_core_ExnSystem* _con = kk_block_alloc_at_as(struct kk_std_core_ExnSystem, _at, 2 /* scan count */, _cpath, KK_TAG_OPEN, _ctx);
  _con->_base._tag = kk_string_dup(kk_std_core__tag_ExnSystem, _ctx);
  _con->kkloc_errno = kkloc_errno;
  return kk_std_core__base_ExnSystem(_con, _ctx);
}
static inline struct kk_std_core_ExnSystem* kk_std_core__as_ExnSystem(kk_std_core__exception_info x, kk_context_t* _ctx) {
  return kk_datatype_as_assert(struct kk_std_core_ExnSystem*, x, KK_TAG_OPEN, _ctx);
}
static inline kk_std_core__exception_info kk_std_core__base_ExnInternal(struct kk_std_core_ExnInternal* _x, kk_context_t* _ctx) {
  return kk_datatype_from_base(&_x->_base, _ctx);
}
extern kk_string_t kk_std_core__tag_ExnInternal;
static inline kk_std_core__exception_info kk_std_core__new_ExnInternal(kk_reuse_t _at, int32_t _cpath, kk_string_t name, kk_context_t* _ctx) {
  struct kk_std_core_ExnInternal* _con = kk_block_alloc_at_as(struct kk_std_core_ExnInternal, _at, 2 /* scan count */, _cpath, KK_TAG_OPEN, _ctx);
  _con->_base._tag = kk_string_dup(kk_std_core__tag_ExnInternal, _ctx);
  _con->name = name;
  return kk_std_core__base_ExnInternal(_con, _ctx);
}
static inline struct kk_std_core_ExnInternal* kk_std_core__as_ExnInternal(kk_std_core__exception_info x, kk_context_t* _ctx) {
  return kk_datatype_as_assert(struct kk_std_core_ExnInternal*, x, KK_TAG_OPEN, _ctx);
}
static inline kk_std_core__exception_info kk_std_core__base_ExnPattern(struct kk_std_core_ExnPattern* _x, kk_context_t* _ctx) {
  return kk_datatype_from_base(&_x->_base, _ctx);
}
extern kk_string_t kk_std_core__tag_ExnPattern;
static inline kk_std_core__exception_info kk_std_core__new_ExnPattern(kk_reuse_t _at, int32_t _cpath, kk_string_t location, kk_string_t definition, kk_context_t* _ctx) {
  struct kk_std_core_ExnPattern* _con = kk_block_alloc_at_as(struct kk_std_core_ExnPattern, _at, 3 /* scan count */, _cpath, KK_TAG_OPEN, _ctx);
  _con->_base._tag = kk_string_dup(kk_std_core__tag_ExnPattern, _ctx);
  _con->location = location;
  _con->definition = definition;
  return kk_std_core__base_ExnPattern(_con, _ctx);
}
static inline struct kk_std_core_ExnPattern* kk_std_core__as_ExnPattern(kk_std_core__exception_info x, kk_context_t* _ctx) {
  return kk_datatype_as_assert(struct kk_std_core_ExnPattern*, x, KK_TAG_OPEN, _ctx);
}
static inline bool kk_std_core__is_ExnError(kk_std_core__exception_info x, kk_context_t* _ctx) {
  return (kk_string_ptr_eq_borrow((kk_datatype_as(struct kk_std_core__exception_info_s*, x, _ctx))->_tag, kk_std_core__tag_ExnError));
}
static inline bool kk_std_core__is_ExnAssert(kk_std_core__exception_info x, kk_context_t* _ctx) {
  return (kk_string_ptr_eq_borrow((kk_datatype_as(struct kk_std_core__exception_info_s*, x, _ctx))->_tag, kk_std_core__tag_ExnAssert));
}
static inline bool kk_std_core__is_ExnTodo(kk_std_core__exception_info x, kk_context_t* _ctx) {
  return (kk_string_ptr_eq_borrow((kk_datatype_as(struct kk_std_core__exception_info_s*, x, _ctx))->_tag, kk_std_core__tag_ExnTodo));
}
static inline bool kk_std_core__is_ExnRange(kk_std_core__exception_info x, kk_context_t* _ctx) {
  return (kk_string_ptr_eq_borrow((kk_datatype_as(struct kk_std_core__exception_info_s*, x, _ctx))->_tag, kk_std_core__tag_ExnRange));
}
static inline bool kk_std_core__is_ExnSystem(kk_std_core__exception_info x, kk_context_t* _ctx) {
  return (kk_string_ptr_eq_borrow((kk_datatype_as(struct kk_std_core__exception_info_s*, x, _ctx))->_tag, kk_std_core__tag_ExnSystem));
}
static inline bool kk_std_core__is_ExnInternal(kk_std_core__exception_info x, kk_context_t* _ctx) {
  return (kk_string_ptr_eq_borrow((kk_datatype_as(struct kk_std_core__exception_info_s*, x, _ctx))->_tag, kk_std_core__tag_ExnInternal));
}
static inline bool kk_std_core__is_ExnPattern(kk_std_core__exception_info x, kk_context_t* _ctx) {
  return (kk_string_ptr_eq_borrow((kk_datatype_as(struct kk_std_core__exception_info_s*, x, _ctx))->_tag, kk_std_core__tag_ExnPattern));
}
static inline kk_std_core__exception_info kk_std_core__exception_info_dup(kk_std_core__exception_info _x, kk_context_t* _ctx) {
  return kk_datatype_ptr_dup(_x, _ctx);
}
static inline void kk_std_core__exception_info_drop(kk_std_core__exception_info _x, kk_context_t* _ctx) {
  kk_datatype_ptr_drop(_x, _ctx);
}
static inline kk_box_t kk_std_core__exception_info_box(kk_std_core__exception_info _x, kk_context_t* _ctx) {
  return kk_datatype_ptr_box(_x);
}
static inline kk_std_core__exception_info kk_std_core__exception_info_unbox(kk_box_t _x, kk_borrow_t _borrow, kk_context_t* _ctx) {
  return kk_datatype_ptr_unbox(_x);
}

// value type std/core/exception
struct kk_std_core_Exception {
  kk_string_t message;
  kk_std_core__exception_info info;
};
typedef struct kk_std_core_Exception kk_std_core__exception;
static inline kk_std_core__exception kk_std_core__new_Exception(kk_string_t message, kk_std_core__exception_info info, kk_context_t* _ctx) {
  kk_std_core__exception _con;
  _con.message = message;
  _con.info = info;
  return _con;
}
static inline bool kk_std_core__is_Exception(kk_std_core__exception x, kk_context_t* _ctx) {
  return (true);
}
static inline kk_std_core__exception kk_std_core__exception_dup(kk_std_core__exception _x, kk_context_t* _ctx) {
  kk_string_dup(_x.message, _ctx);
  kk_std_core__exception_info_dup(_x.info, _ctx);
  return _x;
}
static inline void kk_std_core__exception_drop(kk_std_core__exception _x, kk_context_t* _ctx) {
  kk_string_drop(_x.message, _ctx);
  kk_std_core__exception_info_drop(_x.info, _ctx);
}
static inline kk_box_t kk_std_core__exception_box(kk_std_core__exception _x, kk_context_t* _ctx) {
  kk_box_t _box;
  kk_valuetype_box(kk_std_core__exception, _box, _x, 2 /* scan count */, _ctx);
  return _box;
}
static inline kk_std_core__exception kk_std_core__exception_unbox(kk_box_t _x, kk_borrow_t _borrow, kk_context_t* _ctx) {
  kk_std_core__exception _unbox;
  kk_valuetype_unbox(kk_std_core__exception, _unbox, _x, _borrow, _ctx);
  return _unbox;
}

// type std/core/.hnd-exn
struct kk_std_core__hnd_exn_s {
  kk_block_t _block;
};
typedef kk_datatype_ptr_t kk_std_core__hnd_exn;
struct kk_std_core__Hnd_exn {
  struct kk_std_core__hnd_exn_s _base;
  kk_std_core_hnd__clause1 ctl_throw_exn;
};
static inline kk_std_core__hnd_exn kk_std_core__base_Hnd_exn(struct kk_std_core__Hnd_exn* _x, kk_context_t* _ctx) {
  return kk_datatype_from_base(&_x->_base, _ctx);
}
static inline kk_std_core__hnd_exn kk_std_core__new_Hnd_exn(kk_reuse_t _at, int32_t _cpath, kk_std_core_hnd__clause1 ctl_throw_exn, kk_context_t* _ctx) {
  struct kk_std_core__Hnd_exn* _con = kk_block_alloc_at_as(struct kk_std_core__Hnd_exn, _at, 1 /* scan count */, _cpath, (kk_tag_t)(1), _ctx);
  _con->ctl_throw_exn = ctl_throw_exn;
  return kk_std_core__base_Hnd_exn(_con, _ctx);
}
static inline struct kk_std_core__Hnd_exn* kk_std_core__as_Hnd_exn(kk_std_core__hnd_exn x, kk_context_t* _ctx) {
  return kk_datatype_as_assert(struct kk_std_core__Hnd_exn*, x, (kk_tag_t)(1), _ctx);
}
static inline bool kk_std_core__is_Hnd_exn(kk_std_core__hnd_exn x, kk_context_t* _ctx) {
  return (true);
}
static inline kk_std_core__hnd_exn kk_std_core__hnd_exn_dup(kk_std_core__hnd_exn _x, kk_context_t* _ctx) {
  return kk_datatype_ptr_dup(_x, _ctx);
}
static inline void kk_std_core__hnd_exn_drop(kk_std_core__hnd_exn _x, kk_context_t* _ctx) {
  kk_datatype_ptr_drop(_x, _ctx);
}
static inline kk_box_t kk_std_core__hnd_exn_box(kk_std_core__hnd_exn _x, kk_context_t* _ctx) {
  return kk_datatype_ptr_box(_x);
}
static inline kk_std_core__hnd_exn kk_std_core__hnd_exn_unbox(kk_box_t _x, kk_borrow_t _borrow, kk_context_t* _ctx) {
  return kk_datatype_ptr_unbox(_x);
}

// type std/core/blocking
struct kk_std_core__blocking_s {
  kk_block_t _block;
};
typedef kk_datatype_t kk_std_core__blocking;
static inline kk_std_core__blocking kk_std_core__blocking_dup(kk_std_core__blocking _x, kk_context_t* _ctx) {
  return _x;
}
static inline void kk_std_core__blocking_drop(kk_std_core__blocking _x, kk_context_t* _ctx) {
  
}
static inline kk_box_t kk_std_core__blocking_box(kk_std_core__blocking _x, kk_context_t* _ctx) {
  return kk_datatype_box(_x);
}
static inline kk_std_core__blocking kk_std_core__blocking_unbox(kk_box_t _x, kk_borrow_t _borrow, kk_context_t* _ctx) {
  return kk_datatype_unbox(_x);
}

// type std/core/console
struct kk_std_core__console_s {
  kk_block_t _block;
};
typedef kk_datatype_t kk_std_core__console;
static inline kk_std_core__console kk_std_core__console_dup(kk_std_core__console _x, kk_context_t* _ctx) {
  return _x;
}
static inline void kk_std_core__console_drop(kk_std_core__console _x, kk_context_t* _ctx) {
  
}
static inline kk_box_t kk_std_core__console_box(kk_std_core__console _x, kk_context_t* _ctx) {
  return kk_datatype_box(_x);
}
static inline kk_std_core__console kk_std_core__console_unbox(kk_box_t _x, kk_borrow_t _borrow, kk_context_t* _ctx) {
  return kk_datatype_unbox(_x);
}

// value type std/core/delayed
struct kk_std_core_Delay {
  kk_ref_t dref;
};
typedef struct kk_std_core_Delay kk_std_core__delayed;
static inline kk_std_core__delayed kk_std_core__new_Delay(kk_ref_t dref, kk_context_t* _ctx) {
  kk_std_core__delayed _con = { dref };
  return _con;
}
static inline bool kk_std_core__is_Delay(kk_std_core__delayed x, kk_context_t* _ctx) {
  return (true);
}
static inline kk_std_core__delayed kk_std_core__delayed_dup(kk_std_core__delayed _x, kk_context_t* _ctx) {
  kk_ref_dup(_x.dref, _ctx);
  return _x;
}
static inline void kk_std_core__delayed_drop(kk_std_core__delayed _x, kk_context_t* _ctx) {
  kk_ref_drop(_x.dref, _ctx);
}
static inline kk_box_t kk_std_core__delayed_box(kk_std_core__delayed _x, kk_context_t* _ctx) {
  return kk_ref_box(_x.dref, _ctx);
}
static inline kk_std_core__delayed kk_std_core__delayed_unbox(kk_box_t _x, kk_borrow_t _borrow, kk_context_t* _ctx) {
  return kk_std_core__new_Delay(kk_ref_unbox(_x, _ctx), _ctx);
}

// value type std/core/error
struct kk_std_core_Error {
  kk_std_core__exception exception;
};
struct kk_std_core_Ok {
  kk_box_t result;
};
struct kk_std_core_error_s {
  kk_value_tag_t _tag;
  union {
    struct kk_std_core_Error Error;
    struct kk_std_core_Ok Ok;
    kk_box_t _fields[2];
  } _cons;
};
typedef struct kk_std_core_error_s kk_std_core__error;
static inline kk_std_core__error kk_std_core__new_Error(kk_std_core__exception exception, kk_context_t* _ctx) {
  kk_std_core__error _con;
  _con._tag = kk_value_tag(1);
  _con._cons.Error.exception = exception;
  return _con;
}
static inline kk_std_core__error kk_std_core__new_Ok(kk_box_t result, kk_context_t* _ctx) {
  kk_std_core__error _con;
  _con._tag = kk_value_tag(2);
  _con._cons.Ok.result = result;
  _con._cons._fields[1] = kk_box_null();
  return _con;
}
static inline bool kk_std_core__is_Error(kk_std_core__error x, kk_context_t* _ctx) {
  return (kk_value_tag_eq(x._tag, kk_value_tag(1)));
}
static inline bool kk_std_core__is_Ok(kk_std_core__error x, kk_context_t* _ctx) {
  return (kk_value_tag_eq(x._tag, kk_value_tag(2)));
}
static inline kk_std_core__error kk_std_core__error_dup(kk_std_core__error _x, kk_context_t* _ctx) {
  if (kk_std_core__is_Error(_x, _ctx)) { kk_std_core__exception_dup(_x._cons.Error.exception, _ctx); }
  else { kk_box_dup(_x._cons.Ok.result, _ctx); }
  return _x;
}
static inline void kk_std_core__error_drop(kk_std_core__error _x, kk_context_t* _ctx) {
  if (kk_std_core__is_Error(_x, _ctx)) { kk_std_core__exception_drop(_x._cons.Error.exception, _ctx); }
  else { kk_box_drop(_x._cons.Ok.result, _ctx); }
}
static inline kk_box_t kk_std_core__error_box(kk_std_core__error _x, kk_context_t* _ctx) {
  kk_box_t _box;
  kk_valuetype_box(kk_std_core__error, _box, _x, 3 /* scan count */, _ctx);
  return _box;
}
static inline kk_std_core__error kk_std_core__error_unbox(kk_box_t _x, kk_borrow_t _borrow, kk_context_t* _ctx) {
  kk_std_core__error _unbox;
  kk_valuetype_unbox(kk_std_core__error, _unbox, _x, _borrow, _ctx);
  return _unbox;
}

// type std/core/exn
struct kk_std_core__exn_s {
  kk_block_t _block;
};
typedef kk_datatype_ptr_t kk_std_core__exn;
struct kk_std_core_Exn {
  struct kk_std_core__exn_s _base;
  kk_std_core__hnd_exn _field1;
};
static inline kk_std_core__exn kk_std_core__base_Exn(struct kk_std_core_Exn* _x, kk_context_t* _ctx) {
  return kk_datatype_from_base(&_x->_base, _ctx);
}
static inline kk_std_core__exn kk_std_core__new_Exn(kk_reuse_t _at, int32_t _cpath, kk_std_core__hnd_exn _field1, kk_context_t* _ctx) {
  struct kk_std_core_Exn* _con = kk_block_alloc_at_as(struct kk_std_core_Exn, _at, 1 /* scan count */, _cpath, (kk_tag_t)(1), _ctx);
  _con->_field1 = _field1;
  return kk_std_core__base_Exn(_con, _ctx);
}
static inline struct kk_std_core_Exn* kk_std_core__as_Exn(kk_std_core__exn x, kk_context_t* _ctx) {
  return kk_datatype_as_assert(struct kk_std_core_Exn*, x, (kk_tag_t)(1), _ctx);
}
static inline bool kk_std_core__is_Exn(kk_std_core__exn x, kk_context_t* _ctx) {
  return (true);
}
static inline kk_std_core__exn kk_std_core__exn_dup(kk_std_core__exn _x, kk_context_t* _ctx) {
  return kk_datatype_ptr_dup(_x, _ctx);
}
static inline void kk_std_core__exn_drop(kk_std_core__exn _x, kk_context_t* _ctx) {
  kk_datatype_ptr_drop(_x, _ctx);
}
static inline kk_box_t kk_std_core__exn_box(kk_std_core__exn _x, kk_context_t* _ctx) {
  return kk_datatype_ptr_box(_x);
}
static inline kk_std_core__exn kk_std_core__exn_unbox(kk_box_t _x, kk_borrow_t _borrow, kk_context_t* _ctx) {
  return kk_datatype_ptr_unbox(_x);
}

// type std/core/fsys
struct kk_std_core__fsys_s {
  kk_block_t _block;
};
typedef kk_datatype_t kk_std_core__fsys;
static inline kk_std_core__fsys kk_std_core__fsys_dup(kk_std_core__fsys _x, kk_context_t* _ctx) {
  return _x;
}
static inline void kk_std_core__fsys_drop(kk_std_core__fsys _x, kk_context_t* _ctx) {
  
}
static inline kk_box_t kk_std_core__fsys_box(kk_std_core__fsys _x, kk_context_t* _ctx) {
  return kk_datatype_box(_x);
}
static inline kk_std_core__fsys kk_std_core__fsys_unbox(kk_box_t _x, kk_borrow_t _borrow, kk_context_t* _ctx) {
  return kk_datatype_unbox(_x);
}

// type std/core/global-scope
struct kk_std_core__global_scope_s {
  kk_block_t _block;
};
typedef kk_datatype_t kk_std_core__global_scope;
static inline kk_std_core__global_scope kk_std_core__global_scope_dup(kk_std_core__global_scope _x, kk_context_t* _ctx) {
  return _x;
}
static inline void kk_std_core__global_scope_drop(kk_std_core__global_scope _x, kk_context_t* _ctx) {
  
}
static inline kk_box_t kk_std_core__global_scope_box(kk_std_core__global_scope _x, kk_context_t* _ctx) {
  return kk_datatype_box(_x);
}
static inline kk_std_core__global_scope kk_std_core__global_scope_unbox(kk_box_t _x, kk_borrow_t _borrow, kk_context_t* _ctx) {
  return kk_datatype_unbox(_x);
}

// type std/core/net
struct kk_std_core__net_s {
  kk_block_t _block;
};
typedef kk_datatype_t kk_std_core__net;
static inline kk_std_core__net kk_std_core__net_dup(kk_std_core__net _x, kk_context_t* _ctx) {
  return _x;
}
static inline void kk_std_core__net_drop(kk_std_core__net _x, kk_context_t* _ctx) {
  
}
static inline kk_box_t kk_std_core__net_box(kk_std_core__net _x, kk_context_t* _ctx) {
  return kk_datatype_box(_x);
}
static inline kk_std_core__net kk_std_core__net_unbox(kk_box_t _x, kk_borrow_t _borrow, kk_context_t* _ctx) {
  return kk_datatype_unbox(_x);
}

// type std/core/ui
struct kk_std_core__ui_s {
  kk_block_t _block;
};
typedef kk_datatype_t kk_std_core__ui;
static inline kk_std_core__ui kk_std_core__ui_dup(kk_std_core__ui _x, kk_context_t* _ctx) {
  return _x;
}
static inline void kk_std_core__ui_drop(kk_std_core__ui _x, kk_context_t* _ctx) {
  
}
static inline kk_box_t kk_std_core__ui_box(kk_std_core__ui _x, kk_context_t* _ctx) {
  return kk_datatype_box(_x);
}
static inline kk_std_core__ui kk_std_core__ui_unbox(kk_box_t _x, kk_borrow_t _borrow, kk_context_t* _ctx) {
  return kk_datatype_unbox(_x);
}

// type std/core/list
struct kk_std_core__list_s {
  kk_block_t _block;
};
typedef kk_datatype_t kk_std_core__list;
struct kk_std_core_Cons {
  struct kk_std_core__list_s _base;
  kk_box_t head;
  kk_std_core__list tail;
};
static inline kk_std_core__list kk_std_core__new_Nil(kk_context_t* _ctx) {
  return kk_datatype_from_tag((kk_tag_t)(1));
}
static inline kk_std_core__list kk_std_core__base_Cons(struct kk_std_core_Cons* _x, kk_context_t* _ctx) {
  return kk_datatype_from_base(&_x->_base, _ctx);
}
static inline kk_std_core__list kk_std_core__new_Cons(kk_reuse_t _at, int32_t _cpath, kk_box_t head, kk_std_core__list tail, kk_context_t* _ctx) {
  struct kk_std_core_Cons* _con = kk_block_alloc_at_as(struct kk_std_core_Cons, _at, 2 /* scan count */, _cpath, (kk_tag_t)(2), _ctx);
  _con->head = head;
  _con->tail = tail;
  return kk_std_core__base_Cons(_con, _ctx);
}
static inline struct kk_std_core_Cons* kk_std_core__as_Cons(kk_std_core__list x, kk_context_t* _ctx) {
  return kk_datatype_as_assert(struct kk_std_core_Cons*, x, (kk_tag_t)(2), _ctx);
}
static inline bool kk_std_core__is_Nil(kk_std_core__list x, kk_context_t* _ctx) {
  return (kk_datatype_has_singleton_tag(x, (kk_tag_t)(1)));
}
static inline bool kk_std_core__is_Cons(kk_std_core__list x, kk_context_t* _ctx) {
  return (!kk_std_core__is_Nil(x, _ctx));
}
static inline kk_std_core__list kk_std_core__list_dup(kk_std_core__list _x, kk_context_t* _ctx) {
  return kk_datatype_dup(_x, _ctx);
}
static inline void kk_std_core__list_drop(kk_std_core__list _x, kk_context_t* _ctx) {
  kk_datatype_drop(_x, _ctx);
}
static inline kk_box_t kk_std_core__list_box(kk_std_core__list _x, kk_context_t* _ctx) {
  return kk_datatype_box(_x);
}
static inline kk_std_core__list kk_std_core__list_unbox(kk_box_t _x, kk_borrow_t _borrow, kk_context_t* _ctx) {
  return kk_datatype_unbox(_x);
}

// type std/core/nmd
struct kk_std_core__nmd_s {
  kk_block_t _block;
};
typedef kk_datatype_t kk_std_core__nmd;
static inline kk_std_core__nmd kk_std_core__nmd_dup(kk_std_core__nmd _x, kk_context_t* _ctx) {
  return _x;
}
static inline void kk_std_core__nmd_drop(kk_std_core__nmd _x, kk_context_t* _ctx) {
  
}
static inline kk_box_t kk_std_core__nmd_box(kk_std_core__nmd _x, kk_context_t* _ctx) {
  return kk_datatype_box(_x);
}
static inline kk_std_core__nmd kk_std_core__nmd_unbox(kk_box_t _x, kk_borrow_t _borrow, kk_context_t* _ctx) {
  return kk_datatype_unbox(_x);
}

// type std/core/null
struct kk_std_core__null_s {
  kk_block_t _block;
};
typedef kk_datatype_t kk_std_core__null;
static inline kk_std_core__null kk_std_core__null_dup(kk_std_core__null _x, kk_context_t* _ctx) {
  return _x;
}
static inline void kk_std_core__null_drop(kk_std_core__null _x, kk_context_t* _ctx) {
  
}
static inline kk_box_t kk_std_core__null_box(kk_std_core__null _x, kk_context_t* _ctx) {
  return kk_datatype_box(_x);
}
static inline kk_std_core__null kk_std_core__null_unbox(kk_box_t _x, kk_borrow_t _borrow, kk_context_t* _ctx) {
  return kk_datatype_unbox(_x);
}

// type std/core/scope
struct kk_std_core__scope_s {
  kk_block_t _block;
};
typedef kk_datatype_t kk_std_core__scope;
static inline kk_std_core__scope kk_std_core__scope_dup(kk_std_core__scope _x, kk_context_t* _ctx) {
  return _x;
}
static inline void kk_std_core__scope_drop(kk_std_core__scope _x, kk_context_t* _ctx) {
  
}
static inline kk_box_t kk_std_core__scope_box(kk_std_core__scope _x, kk_context_t* _ctx) {
  return kk_datatype_box(_x);
}
static inline kk_std_core__scope kk_std_core__scope_unbox(kk_box_t _x, kk_borrow_t _borrow, kk_context_t* _ctx) {
  return kk_datatype_unbox(_x);
}

// value type std/core/sslice
struct kk_std_core_Sslice {
  kk_string_t str;
  kk_integer_t start;
  kk_integer_t len;
};
typedef struct kk_std_core_Sslice kk_std_core__sslice;
static inline kk_std_core__sslice kk_std_core__new_Sslice(kk_string_t str, kk_integer_t start, kk_integer_t len, kk_context_t* _ctx) {
  kk_std_core__sslice _con;
  _con.str = str;
  _con.start = start;
  _con.len = len;
  return _con;
}
static inline bool kk_std_core__is_Sslice(kk_std_core__sslice x, kk_context_t* _ctx) {
  return (true);
}
static inline kk_std_core__sslice kk_std_core__sslice_dup(kk_std_core__sslice _x, kk_context_t* _ctx) {
  kk_string_dup(_x.str, _ctx);
  kk_integer_dup(_x.start, _ctx);
  kk_integer_dup(_x.len, _ctx);
  return _x;
}
static inline void kk_std_core__sslice_drop(kk_std_core__sslice _x, kk_context_t* _ctx) {
  kk_string_drop(_x.str, _ctx);
  kk_integer_drop(_x.start, _ctx);
  kk_integer_drop(_x.len, _ctx);
}
static inline kk_box_t kk_std_core__sslice_box(kk_std_core__sslice _x, kk_context_t* _ctx) {
  kk_box_t _box;
  kk_valuetype_box(kk_std_core__sslice, _box, _x, 3 /* scan count */, _ctx);
  return _box;
}
static inline kk_std_core__sslice kk_std_core__sslice_unbox(kk_box_t _x, kk_borrow_t _borrow, kk_context_t* _ctx) {
  kk_std_core__sslice _unbox;
  kk_valuetype_unbox(kk_std_core__sslice, _unbox, _x, _borrow, _ctx);
  return _unbox;
}

// type std/core/stream
struct kk_std_core__stream_s {
  kk_block_t _block;
};
typedef kk_datatype_ptr_t kk_std_core__stream;
struct kk_std_core_Next {
  struct kk_std_core__stream_s _base;
  kk_box_t head;
  kk_std_core__stream tail;
};
static inline kk_std_core__stream kk_std_core__base_Next(struct kk_std_core_Next* _x, kk_context_t* _ctx) {
  return kk_datatype_from_base(&_x->_base, _ctx);
}
static inline kk_std_core__stream kk_std_core__new_Next(kk_reuse_t _at, int32_t _cpath, kk_box_t head, kk_std_core__stream tail, kk_context_t* _ctx) {
  struct kk_std_core_Next* _con = kk_block_alloc_at_as(struct kk_std_core_Next, _at, 2 /* scan count */, _cpath, (kk_tag_t)(1), _ctx);
  _con->head = head;
  _con->tail = tail;
  return kk_std_core__base_Next(_con, _ctx);
}
static inline struct kk_std_core_Next* kk_std_core__as_Next(kk_std_core__stream x, kk_context_t* _ctx) {
  return kk_datatype_as_assert(struct kk_std_core_Next*, x, (kk_tag_t)(1), _ctx);
}
static inline bool kk_std_core__is_Next(kk_std_core__stream x, kk_context_t* _ctx) {
  return (true);
}
static inline kk_std_core__stream kk_std_core__stream_dup(kk_std_core__stream _x, kk_context_t* _ctx) {
  return kk_datatype_ptr_dup(_x, _ctx);
}
static inline void kk_std_core__stream_drop(kk_std_core__stream _x, kk_context_t* _ctx) {
  kk_datatype_ptr_drop(_x, _ctx);
}
static inline kk_box_t kk_std_core__stream_box(kk_std_core__stream _x, kk_context_t* _ctx) {
  return kk_datatype_ptr_box(_x);
}
static inline kk_std_core__stream kk_std_core__stream_unbox(kk_box_t _x, kk_borrow_t _borrow, kk_context_t* _ctx) {
  return kk_datatype_ptr_unbox(_x);
}

// value declarations
 
// Automatically generated. Tests for the `ExnError` constructor of the `:exception-info` type.

static inline bool kk_std_core_is_exnError(kk_std_core__exception_info kkloc_exception_info, kk_context_t* _ctx) { /* (exception-info : exception-info) -> bool */ 
  if (kk_std_core__is_ExnError(kkloc_exception_info, _ctx)) {
    struct kk_std_core_ExnError* _con21106 = kk_std_core__as_ExnError(kkloc_exception_info, _ctx);
    return true;
  }
  {
    return false;
  }
}
 
// Automatically generated. Tests for the `ExnAssert` constructor of the `:exception-info` type.

static inline bool kk_std_core_is_exnAssert(kk_std_core__exception_info kkloc_exception_info, kk_context_t* _ctx) { /* (exception-info : exception-info) -> bool */ 
  if (kk_std_core__is_ExnAssert(kkloc_exception_info, _ctx)) {
    struct kk_std_core_ExnAssert* _con21107 = kk_std_core__as_ExnAssert(kkloc_exception_info, _ctx);
    return true;
  }
  {
    return false;
  }
}
 
// Automatically generated. Tests for the `ExnTodo` constructor of the `:exception-info` type.

static inline bool kk_std_core_is_exnTodo(kk_std_core__exception_info kkloc_exception_info, kk_context_t* _ctx) { /* (exception-info : exception-info) -> bool */ 
  if (kk_std_core__is_ExnTodo(kkloc_exception_info, _ctx)) {
    struct kk_std_core_ExnTodo* _con21108 = kk_std_core__as_ExnTodo(kkloc_exception_info, _ctx);
    return true;
  }
  {
    return false;
  }
}
 
// Automatically generated. Tests for the `ExnRange` constructor of the `:exception-info` type.

static inline bool kk_std_core_is_exnRange(kk_std_core__exception_info kkloc_exception_info, kk_context_t* _ctx) { /* (exception-info : exception-info) -> bool */ 
  if (kk_std_core__is_ExnRange(kkloc_exception_info, _ctx)) {
    struct kk_std_core_ExnRange* _con21109 = kk_std_core__as_ExnRange(kkloc_exception_info, _ctx);
    return true;
  }
  {
    return false;
  }
}
 
// Automatically generated. Tests for the `ExnPattern` constructor of the `:exception-info` type.

static inline bool kk_std_core_is_exnPattern(kk_std_core__exception_info kkloc_exception_info, kk_context_t* _ctx) { /* (exception-info : exception-info) -> bool */ 
  if (kk_std_core__is_ExnPattern(kkloc_exception_info, _ctx)) {
    struct kk_std_core_ExnPattern* _con21110 = kk_std_core__as_ExnPattern(kkloc_exception_info, _ctx);
    return true;
  }
  {
    return false;
  }
}
 
// Automatically generated. Tests for the `ExnSystem` constructor of the `:exception-info` type.

static inline bool kk_std_core_is_exnSystem(kk_std_core__exception_info kkloc_exception_info, kk_context_t* _ctx) { /* (exception-info : exception-info) -> bool */ 
  if (kk_std_core__is_ExnSystem(kkloc_exception_info, _ctx)) {
    struct kk_std_core_ExnSystem* _con21111 = kk_std_core__as_ExnSystem(kkloc_exception_info, _ctx);
    return true;
  }
  {
    return false;
  }
}
 
// Automatically generated. Tests for the `ExnInternal` constructor of the `:exception-info` type.

static inline bool kk_std_core_is_exnInternal(kk_std_core__exception_info kkloc_exception_info, kk_context_t* _ctx) { /* (exception-info : exception-info) -> bool */ 
  if (kk_std_core__is_ExnInternal(kkloc_exception_info, _ctx)) {
    struct kk_std_core_ExnInternal* _con21112 = kk_std_core__as_ExnInternal(kkloc_exception_info, _ctx);
    return true;
  }
  {
    return false;
  }
}

extern kk_string_t kk_std_core__tag_ExnError;

extern kk_string_t kk_std_core__tag_ExnAssert;

extern kk_string_t kk_std_core__tag_ExnTodo;

extern kk_string_t kk_std_core__tag_ExnRange;

extern kk_string_t kk_std_core__tag_ExnPattern;

extern kk_string_t kk_std_core__tag_ExnSystem;

extern kk_string_t kk_std_core__tag_ExnInternal;
 
// Automatically generated. Retrieves the `message` constructor field of the `:exception` type.

static inline kk_string_t kk_std_core_message(kk_std_core__exception exception, kk_context_t* _ctx) { /* (exception : exception) -> string */ 
  {
    kk_string_t _x = exception.message;
    return kk_string_dup(_x, _ctx);
  }
}
 
// Automatically generated. Retrieves the `info` constructor field of the `:exception` type.

static inline kk_std_core__exception_info kk_std_core_info(kk_std_core__exception exception, kk_context_t* _ctx) { /* (exception : exception) -> exception-info */ 
  {
    kk_std_core__exception_info _x = exception.info;
    return kk_std_core__exception_info_dup(_x, _ctx);
  }
}

kk_std_core__exception kk_std_core__copy(kk_std_core__exception _this, kk_std_core_types__optional message0, kk_std_core_types__optional info0, kk_context_t* _ctx); /* (exception, message : optional<string>, info : optional<exception-info>) -> exception */ 
 
// Automatically generated. Retrieves the `dref` constructor field of the `:delayed` type.

static inline kk_ref_t kk_std_core_dref(kk_std_core__delayed delayed, kk_context_t* _ctx) { /* forall<e,a> (delayed : delayed<e,a>) -> ref<global,either<() -> e a,a>> */ 
  {
    kk_ref_t _x = delayed.dref;
    return kk_ref_dup(_x, _ctx);
  }
}

kk_std_core__delayed kk_std_core__copy_1(kk_std_core__delayed _this, kk_std_core_types__optional dref0, kk_context_t* _ctx); /* forall<e,a> (delayed<e,a>, dref : optional<ref<global,either<() -> e a,a>>>) -> delayed<e,a> */ 
 
// Automatically generated. Tests for the `Error` constructor of the `:error` type.

static inline bool kk_std_core_is_error(kk_std_core__error error, kk_context_t* _ctx) { /* forall<a> (error : error<a>) -> bool */ 
  if (kk_std_core__is_Error(error, _ctx)) {
    kk_std_core__exception _pat0 = error._cons.Error.exception;
    return true;
  }
  {
    return false;
  }
}
 
// Automatically generated. Tests for the `Ok` constructor of the `:error` type.

static inline bool kk_std_core_is_ok(kk_std_core__error error, kk_context_t* _ctx) { /* forall<a> (error : error<a>) -> bool */ 
  if (kk_std_core__is_Ok(error, _ctx)) {
    return true;
  }
  {
    return false;
  }
}
 
// Automatically generated. Tests for the `Nil` constructor of the `:list` type.

static inline bool kk_std_core_is_nil(kk_std_core__list list0, kk_context_t* _ctx) { /* forall<a> (list : list<a>) -> bool */ 
  if (kk_std_core__is_Nil(list0, _ctx)) {
    return true;
  }
  {
    return false;
  }
}
 
// Automatically generated. Tests for the `Cons` constructor of the `:list` type.

static inline bool kk_std_core_is_cons(kk_std_core__list list0, kk_context_t* _ctx) { /* forall<a> (list : list<a>) -> bool */ 
  if (kk_std_core__is_Cons(list0, _ctx)) {
    struct kk_std_core_Cons* _con21116 = kk_std_core__as_Cons(list0, _ctx);
    return true;
  }
  {
    return false;
  }
}
 
// Automatically generated. Retrieves the `str` constructor field of the `:sslice` type.

static inline kk_string_t kk_std_core_str(kk_std_core__sslice sslice, kk_context_t* _ctx) { /* (sslice : sslice) -> string */ 
  {
    kk_string_t _x = sslice.str;
    return kk_string_dup(_x, _ctx);
  }
}
 
// Automatically generated. Retrieves the `start` constructor field of the `:sslice` type.

static inline kk_integer_t kk_std_core_start(kk_std_core__sslice sslice, kk_context_t* _ctx) { /* (sslice : sslice) -> int */ 
  {
    kk_integer_t _x = sslice.start;
    return kk_integer_dup(_x, _ctx);
  }
}
 
// Automatically generated. Retrieves the `len` constructor field of the `:sslice` type.

static inline kk_integer_t kk_std_core_len(kk_std_core__sslice sslice, kk_context_t* _ctx) { /* (sslice : sslice) -> int */ 
  {
    kk_integer_t _x = sslice.len;
    return kk_integer_dup(_x, _ctx);
  }
}

kk_std_core__sslice kk_std_core__copy_2(kk_std_core__sslice _this, kk_std_core_types__optional str0, kk_std_core_types__optional start0, kk_std_core_types__optional len0, kk_context_t* _ctx); /* (sslice, str : optional<string>, start : optional<int>, len : optional<int>) -> sslice */ 
 
// Automatically generated. Retrieves the `head` constructor field of the `:stream` type.

static inline kk_box_t kk_std_core_head(kk_std_core__stream stream, kk_context_t* _ctx) { /* forall<a> (stream : stream<a>) -> a */ 
  {
    struct kk_std_core_Next* _con21120 = kk_std_core__as_Next(stream, _ctx);
    kk_box_t _x = _con21120->head;
    return kk_box_dup(_x, _ctx);
  }
}
 
// Automatically generated. Retrieves the `tail` constructor field of the `:stream` type.

static inline kk_std_core__stream kk_std_core_tail(kk_std_core__stream stream, kk_context_t* _ctx) { /* forall<a> (stream : stream<a>) -> stream<a> */ 
  {
    struct kk_std_core_Next* _con21121 = kk_std_core__as_Next(stream, _ctx);
    kk_std_core__stream _x = _con21121->tail;
    return kk_std_core__stream_dup(_x, _ctx);
  }
}

kk_std_core__stream kk_std_core__copy_3(kk_std_core__stream _this, kk_std_core_types__optional head0, kk_std_core_types__optional tail0, kk_context_t* _ctx); /* forall<a> (stream<a>, head : optional<a>, tail : optional<stream<a>>) -> stream<a> */ 
 
// select `throw-exn` operation out of the `:exn` effect handler

static inline kk_std_core_hnd__clause1 kk_std_core__select_throw_exn(kk_std_core__hnd_exn hnd, kk_context_t* _ctx) { /* forall<a,e,b> (hnd : .hnd-exn<e,b>) -> std/core/hnd/clause1<exception,a,.hnd-exn,e,b> */ 
  {
    struct kk_std_core__Hnd_exn* _con21126 = kk_std_core__as_Hnd_exn(hnd, _ctx);
    kk_std_core_hnd__clause1 ctl_throw_exn = _con21126->ctl_throw_exn;
    return kk_std_core_hnd__clause1_dup(ctl_throw_exn, _ctx);
  }
}

extern kk_std_core_hnd__htag kk_std_core__tag_exn;

kk_box_t kk_std_core__handle_exn(int32_t cfc, kk_std_core__hnd_exn hnd, kk_function_t ret, kk_function_t action, kk_context_t* _ctx); /* forall<a,e,b> (cfc : int32, hnd : .hnd-exn<e,b>, ret : (res : a) -> e b, action : () -> <exn|e> a) -> e b */ 
 
// Internal export for the regex module

static inline kk_std_core__sslice kk_std_core__new_sslice(kk_string_t str0, kk_integer_t start0, kk_integer_t len0, kk_context_t* _ctx) { /* (str : string, start : int, len : int) -> sslice */ 
  return kk_std_core__new_Sslice(str0, start0, len0, _ctx);
}

kk_std_core__null kk_std_core__null_any(kk_box_t x, kk_context_t* _ctx); /* forall<a> (x : a) -> null<a> */ 

static inline kk_integer_t kk_std_core_int_8(bool b, kk_context_t* _ctx) { /* (b : bool) -> int */ 
  if (b) {
    return kk_integer_from_small(1);
  }
  {
    return kk_integer_from_small(0);
  }
}

static inline kk_integer_t kk_std_core_int_9(kk_std_core_types__order x, kk_context_t* _ctx) { /* (x : order) -> int */ 
  if (kk_std_core_types__is_Lt(x, _ctx)) {
    return kk_integer_from_small(-1);
  }
  if (kk_std_core_types__is_Eq(x, _ctx)) {
    return kk_integer_from_small(0);
  }
  {
    return kk_integer_from_small(1);
  }
}

extern kk_ref_t kk_std_core_redirect;

kk_unit_t kk_std_core_xprintsln(kk_string_t s, kk_context_t* _ctx); /* (s : string) -> console () */ 

kk_integer_t kk_std_core_string_compare(kk_string_t x, kk_string_t y, kk_context_t* _ctx); /* (x : string, y : string) -> int */ 

int32_t kk_std_core_int32(kk_integer_t i, kk_context_t* _ctx); /* (i : int) -> int32 */ 

kk_ssize_t kk_std_core_ssize__t(kk_integer_t i, kk_context_t* _ctx); /* (i : int) -> ssize_t */ 

kk_string_t kk_std_core_string(kk_char_t c, kk_context_t* _ctx); /* (c : char) -> string */ 

kk_string_t kk_std_core_string_1(kk_vector_t _arg1, kk_context_t* _ctx); /* (vector<char>) -> string */ 

kk_string_t kk_std_core_string_2(kk_std_core__list cs, kk_context_t* _ctx); /* (cs : list<char>) -> total string */ 

kk_string_t kk_std_core_string_3(kk_std_core__sslice slice0, kk_context_t* _ctx); /* (slice : sslice) -> string */ 
 
// Convert a `:maybe` string to a string using the empty sting for `Nothing`

static inline kk_string_t kk_std_core_string_4(kk_std_core_types__maybe ms, kk_context_t* _ctx) { /* (ms : maybe<string>) -> string */ 
  if (kk_std_core_types__is_Nothing(ms, _ctx)) {
    return kk_string_empty();
  }
  {
    kk_box_t _box_x19265 = ms._cons.Just.value;
    kk_string_t s = kk_string_unbox(_box_x19265);
    kk_string_dup(s, _ctx);
    kk_std_core_types__maybe_drop(ms, _ctx);
    return s;
  }
}

kk_std_core__list kk_std_core_vlist(kk_vector_t v, kk_std_core_types__optional tail0, kk_context_t* _ctx); /* forall<a> (v : vector<a>, tail : optional<list<a>>) -> list<a> */ 

kk_string_t kk_std_core_int_show_hex(kk_integer_t i, bool use_capitals, kk_context_t* _ctx); /* (i : int, use-capitals : bool) -> string */ 

kk_string_t kk_std_core_repeatz(kk_string_t s, kk_ssize_t n, kk_context_t* _ctx); /* (s : string, n : ssize_t) -> string */ 

kk_string_t kk_std_core_show_expx(double d, int32_t prec, kk_context_t* _ctx); /* (d : float64, prec : int32) -> string */ 

kk_string_t kk_std_core_show_fixedx(double d, int32_t prec, kk_context_t* _ctx); /* (d : float64, prec : int32) -> string */ 

kk_unit_t kk_std_core_xprints(kk_string_t s, kk_context_t* _ctx); /* (s : string) -> console () */ 

kk_integer_t kk_std_core_pow(kk_integer_t i, kk_integer_t exp, kk_context_t* _ctx); /* (i : int, exp : int) -> int */ 

kk_std_core__sslice kk_std_core_advance(kk_std_core__sslice slice0, kk_integer_t count, kk_context_t* _ctx); /* (slice : sslice, count : int) -> sslice */ 
 
// Apply a function `f` to a specified argument `x`.

static inline kk_box_t kk_std_core_apply(kk_function_t f, kk_box_t x, kk_context_t* _ctx) { /* forall<a,b,e> (f : (a) -> e b, x : a) -> e b */ 
  return kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), f, (f, x, _ctx), _ctx);
}

kk_unit_t kk_std_core_unsafe_assert_fail(kk_string_t msg, kk_context_t* _ctx); /* (msg : string) -> () */ 
 
// O(1). Return the string slice from the start of a string up to the
// start of `slice` argument.

static inline kk_std_core__sslice kk_std_core_before(kk_std_core__sslice slice0, kk_context_t* _ctx) { /* (slice : sslice) -> sslice */ 
  {
    kk_string_t s = slice0.str;
    kk_integer_t start0 = slice0.start;
    kk_string_dup(s, _ctx);
    kk_integer_dup(start0, _ctx);
    kk_std_core__sslice_drop(slice0, _ctx);
    return kk_std_core__new_Sslice(s, kk_integer_from_small(0), start0, _ctx);
  }
}

kk_std_core__sslice kk_std_core_extend(kk_std_core__sslice slice0, kk_integer_t count, kk_context_t* _ctx); /* (slice : sslice, count : int) -> sslice */ 

kk_std_core__sslice kk_std_core_first1(kk_string_t s, kk_context_t* _ctx); /* (s : string) -> sslice */ 

kk_string_t kk_std_core_to_upper(kk_string_t s, kk_context_t* _ctx); /* (s : string) -> string */ 

kk_integer_t kk_std_core_cdiv_exp10(kk_integer_t i, kk_integer_t n, kk_context_t* _ctx); /* (i : int, n : int) -> int */ 

kk_integer_t kk_std_core_mul_exp10(kk_integer_t i, kk_integer_t n, kk_context_t* _ctx); /* (i : int, n : int) -> int */ 

kk_std_core__sslice kk_std_core_common_prefix(kk_string_t s, kk_string_t t, kk_std_core_types__optional upto, kk_context_t* _ctx); /* (s : string, t : string, upto : optional<int>) -> sslice */ 

kk_std_core__list kk_std_core__trmc_lift17311_concat(kk_std_core__list ys, kk_std_core__list zss, kk_std_core_types__cctx _acc, kk_context_t* _ctx); /* forall<a> (ys : list<a>, zss : list<list<a>>, ctx<list<a>>) -> list<a> */ 

kk_std_core__list kk_std_core__lift17311_concat(kk_std_core__list ys0, kk_std_core__list zss0, kk_context_t* _ctx); /* forall<a> (ys : list<a>, zss : list<list<a>>) -> list<a> */ 
 
// Concatenate all lists in a list (e.g. flatten the list). (tail-recursive)

static inline kk_std_core__list kk_std_core_concat(kk_std_core__list xss, kk_context_t* _ctx) { /* forall<a> (xss : list<list<a>>) -> list<a> */ 
  return kk_std_core__lift17311_concat(kk_std_core__new_Nil(_ctx), xss, _ctx);
}
 
// The `const` funs returns its first argument and ignores the second.

static inline kk_box_t kk_std_core_const(kk_box_t x, kk_box_t y, kk_context_t* _ctx) { /* forall<a,b> (x : a, y : b) -> a */ 
  kk_box_drop(y, _ctx);
  return x;
}
 
// Return a 'constant' function that ignores its argument and always returns the same result


// lift anonymous function
struct kk_std_core_const_fun21137__t_1 {
  struct kk_function_s _base;
  kk_box_t default0;
};
extern kk_box_t kk_std_core_const_fun21137_1(kk_function_t _fself, kk_box_t ___wildcard__109__6, kk_context_t* _ctx);
static inline kk_function_t kk_std_core_new_const_fun21137_1(kk_box_t default0, kk_context_t* _ctx) {
  struct kk_std_core_const_fun21137__t_1* _self = kk_function_alloc_as(struct kk_std_core_const_fun21137__t_1, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_core_const_fun21137_1, kk_context());
  _self->default0 = default0;
  return kk_datatype_from_base(&_self->_base, kk_context());
}


static inline kk_function_t kk_std_core_const_1(kk_box_t default0, kk_context_t* _ctx) { /* forall<a,b> (default : a) -> total ((x : b) -> a) */ 
  return kk_std_core_new_const_fun21137_1(default0, _ctx);
}

kk_std_core_types__maybe kk_std_core_next(kk_std_core__sslice slice0, kk_context_t* _ctx); /* (slice : sslice) -> maybe<(char, sslice)> */ 

kk_integer_t kk_std_core_count_digits(kk_integer_t i, kk_context_t* _ctx); /* (i : int) -> int */ 
 
// Convert a `:maybe<a>` value to `:a`, using the `nothing` parameter for `Nothing`.

static inline kk_box_t kk_std_core_default(kk_std_core_types__maybe m, kk_box_t nothing, kk_context_t* _ctx) { /* forall<a> (m : maybe<a>, nothing : a) -> a */ 
  if (kk_std_core_types__is_Nothing(m, _ctx)) {
    return nothing;
  }
  {
    kk_box_t x = m._cons.Just.value;
    kk_box_drop(nothing, _ctx);
    return x;
  }
}
 
// Use default value `def` in case of an error.

static inline kk_box_t kk_std_core_default_1(kk_std_core__error t, kk_box_t def, kk_context_t* _ctx) { /* forall<a> (t : error<a>, def : a) -> a */ 
  if (kk_std_core__is_Error(t, _ctx)) {
    kk_std_core__exception _pat0 = t._cons.Error.exception;
    kk_std_core__error_drop(t, _ctx);
    return def;
  }
  {
    kk_box_t x = t._cons.Ok.result;
    kk_box_drop(def, _ctx);
    return x;
  }
}

kk_std_core_types__either kk_std_core_either(kk_std_core__error t, kk_context_t* _ctx); /* forall<a> (t : error<a>) -> either<exception,a> */ 

extern kk_std_core__sslice kk_std_core_empty;

bool kk_std_core_xends_with(kk_string_t s, kk_string_t post, kk_context_t* _ctx); /* (s : string, post : string) -> bool */ 

kk_std_core__list kk_std_core__mlift18788_trmc_lift17312_flatmap(kk_std_core_types__cctx _acc, kk_function_t f, kk_std_core__list zz, kk_std_core__list ys1_17367, kk_context_t* _ctx); /* forall<a,b,e> (ctx<list<b>>, f : (a) -> e list<b>, zz : list<a>, ys1.17367 : list<b>) -> e list<b> */ 

kk_std_core__list kk_std_core__trmc_lift17312_flatmap(kk_function_t f0, kk_std_core__list ys, kk_std_core__list zs, kk_std_core_types__cctx _acc0, kk_context_t* _ctx); /* forall<a,b,e> (f : (a) -> e list<b>, ys : list<b>, zs : list<a>, ctx<list<b>>) -> e list<b> */ 

kk_std_core__list kk_std_core__lift17312_flatmap(kk_function_t f1, kk_std_core__list ys0, kk_std_core__list zs0, kk_context_t* _ctx); /* forall<a,b,e> (f : (a) -> e list<b>, ys : list<b>, zs : list<a>) -> e list<b> */ 
 
// Concatenate the result lists from applying a function to all elements.

static inline kk_std_core__list kk_std_core_flatmap(kk_std_core__list xs, kk_function_t f, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, f : (a) -> e list<b>) -> e list<b> */ 
  return kk_std_core__lift17312_flatmap(f, kk_std_core__new_Nil(_ctx), xs, _ctx);
}

kk_std_core__list kk_std_core__unroll17327_lift17313_reverse_append(kk_std_core__list acc, kk_std_core__list ys, kk_context_t* _ctx); /* forall<a> (acc : list<a>, ys : list<a>) -> list<a> */ 
 
// unrolling of singleton matches of std/core/.unroll17327-lift17313-reverse-append

static inline kk_std_core__list kk_std_core__lift17313_reverse_append(kk_std_core__list acc, kk_std_core__list ys, kk_context_t* _ctx) { /* forall<a> (acc : list<a>, ys : list<a>) -> list<a> */ 
  if (kk_std_core__is_Nil(ys, _ctx)) {
    return acc;
  }
  {
    return kk_std_core__unroll17327_lift17313_reverse_append(acc, ys, _ctx);
  }
}
 
// Efficiently reverse a list `xs` and append it to `tl`:
// `reverse-append(xs,tl) == reserve(xs) ++ tl

static inline kk_std_core__list kk_std_core_reverse_append(kk_std_core__list xs, kk_std_core__list tl, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, tl : list<a>) -> list<a> */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    return tl;
  }
  {
    return kk_std_core__unroll17327_lift17313_reverse_append(tl, xs, _ctx);
  }
}
 
// monadic lift

static inline kk_box_t kk_std_core__mlift18789_force(kk_ref_t r, kk_box_t x0, kk_context_t* _ctx) { /* forall<a,e> (r : ref<global,either<() -> e a,a>>, x0 : a) -> <st<global>,div|e> a */ 
  kk_unit_t __ = kk_Unit;
  kk_unit_t _brw_21010 = kk_Unit;
  kk_box_t _x21151;
  kk_std_core_types__either _x21152;
  kk_box_t _x21153 = kk_box_dup(x0, _ctx); /*2615*/
  _x21152 = kk_std_core_types__new_Right(_x21153, _ctx); /*either<76,77>*/
  _x21151 = kk_std_core_types__either_box(_x21152, _ctx); /*176*/
  kk_ref_set_borrow(r,_x21151,kk_context());
  kk_ref_drop(r, _ctx);
  _brw_21010;
  return x0;
}

kk_box_t kk_std_core__mlift18790_force(kk_ref_t r, kk_function_t _y_18444, kk_context_t* _ctx); /* forall<a,e> (r : ref<global,either<() -> e a,a>>, () -> <st<global>,div|e> a) -> <alloc<global>,div,read<global>,write<global>|e> a */ 

kk_box_t kk_std_core__mlift18791_force(kk_ref_t r, kk_std_core_types__either _y_18442, kk_context_t* _ctx); /* forall<a,e> (r : ref<global,either<() -> e a,a>>, either<() -> e a,a>) -> <read<global>,div,alloc<global>,write<global>|e> a */ 

kk_box_t kk_std_core_force(kk_std_core__delayed delayed, kk_context_t* _ctx); /* forall<a,e> (delayed : delayed<e,a>) -> e a */ 

kk_string_t kk_std_core_gshow(kk_box_t _arg1, kk_context_t* _ctx); /* forall<a> (a) -> string */ 

kk_string_t kk_std_core_host(kk_context_t* _ctx); /* () -> ndet string */ 

int64_t kk_std_core_int64(kk_integer_t i, kk_context_t* _ctx); /* (i : int) -> int64 */ 
 
// The `ignore` function ignores its argument.

static inline kk_unit_t kk_std_core_ignore(kk_box_t x, kk_context_t* _ctx) { /* forall<a> (x : a) -> () */ 
  kk_box_drop(x, _ctx);
  kk_Unit; return kk_Unit;
}

int16_t kk_std_core_int16(kk_integer_t i, kk_context_t* _ctx); /* (i : int) -> int16 */ 

int8_t kk_std_core_int8(kk_integer_t i, kk_context_t* _ctx); /* (i : int) -> int8 */ 

kk_std_core__list kk_std_core__trmc_unroll17328_lift17314_intersperse(kk_box_t sep, kk_std_core__list ys, kk_std_core_types__cctx _acc, kk_context_t* _ctx); /* forall<a> (sep : a, ys : list<a>, ctx<list<a>>) -> list<a> */ 

kk_std_core__list kk_std_core__unroll17328_lift17314_intersperse(kk_box_t sep0, kk_std_core__list ys0, kk_context_t* _ctx); /* forall<a> (sep : a, ys : list<a>) -> list<a> */ 
 
// unrolling of singleton matches of std/core/.unroll17328-lift17314-intersperse

static inline kk_std_core__list kk_std_core__lift17314_intersperse(kk_box_t sep, kk_std_core__list ys, kk_context_t* _ctx) { /* forall<a> (sep : a, ys : list<a>) -> list<a> */ 
  if (kk_std_core__is_Nil(ys, _ctx)) {
    kk_box_drop(sep, _ctx);
    return kk_std_core__new_Nil(_ctx);
  }
  {
    return kk_std_core__unroll17328_lift17314_intersperse(sep, ys, _ctx);
  }
}

kk_std_core__list kk_std_core_intersperse(kk_std_core__list xs, kk_box_t sep, kk_context_t* _ctx); /* forall<a> (xs : list<a>, sep : a) -> list<a> */ 

intptr_t kk_std_core_intptr__t(kk_integer_t i, kk_context_t* _ctx); /* (i : int) -> intptr_t */ 

extern kk_std_core__sslice kk_std_core_invalid;
 
// Is this an even integer?

static inline bool kk_std_core_is_even(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> bool */ 
  bool b_17373 = kk_integer_is_odd(i,kk_context()); /*bool*/;
  if (b_17373) {
    return false;
  }
  {
    return true;
  }
}

kk_integer_t kk_std_core_is_exp10(kk_integer_t i, kk_context_t* _ctx); /* (i : int) -> int */ 

kk_std_core__sslice kk_std_core_last1(kk_string_t s, kk_context_t* _ctx); /* (s : string) -> sslice */ 

kk_box_t kk_std_core_main_console(kk_function_t main, kk_context_t* _ctx); /* forall<a,e> (main : () -> e a) -> e a */ 

kk_std_core__list kk_std_core__mlift18792_trmc_unroll17329_lift17315_map_peek(kk_std_core_types__cctx _acc, kk_function_t f, kk_std_core__list yy, kk_box_t _trmc_18366, kk_context_t* _ctx); /* forall<a,b,e> (ctx<list<b>>, f : (value : a, rest : list<a>) -> e b, yy : list<a>, b) -> e list<b> */ 

kk_std_core__list kk_std_core__trmc_unroll17329_lift17315_map_peek(kk_function_t f0, kk_std_core__list ys, kk_std_core_types__cctx _acc0, kk_context_t* _ctx); /* forall<a,b,e> (f : (value : a, rest : list<a>) -> e b, ys : list<a>, ctx<list<b>>) -> e list<b> */ 

kk_std_core__list kk_std_core__unroll17329_lift17315_map_peek(kk_function_t f1, kk_std_core__list ys0, kk_context_t* _ctx); /* forall<a,b,e> (f : (value : a, rest : list<a>) -> e b, ys : list<a>) -> e list<b> */ 
 
// unrolling of singleton matches of std/core/.unroll17329-lift17315-map-peek

static inline kk_std_core__list kk_std_core__lift17315_map_peek(kk_function_t f, kk_std_core__list ys, kk_context_t* _ctx) { /* forall<a,b,e> (f : (value : a, rest : list<a>) -> e b, ys : list<a>) -> e list<b> */ 
  if (kk_std_core__is_Nil(ys, _ctx)) {
    kk_function_drop(f, _ctx);
    return kk_std_core__new_Nil(_ctx);
  }
  {
    return kk_std_core__unroll17329_lift17315_map_peek(f, ys, _ctx);
  }
}
 
// Apply a function `f`  to each element of the input list in sequence where `f` takes
// both the current element and the tail list as arguments.

static inline kk_std_core__list kk_std_core_map_peek(kk_std_core__list xs, kk_function_t f, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, f : (value : a, rest : list<a>) -> e b) -> e list<b> */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    kk_function_drop(f, _ctx);
    return kk_std_core__new_Nil(_ctx);
  }
  {
    return kk_std_core__unroll17329_lift17315_map_peek(f, xs, _ctx);
  }
}

static inline kk_integer_t kk_std_core_mbint(kk_std_core_types__maybe m, kk_context_t* _ctx) { /* (m : maybe<int>) -> int */ 
  if (kk_std_core_types__is_Nothing(m, _ctx)) {
    return kk_integer_from_small(0);
  }
  {
    kk_box_t _box_x19378 = m._cons.Just.value;
    kk_integer_t i = kk_integer_unbox(_box_x19378, _ctx);
    kk_integer_dup(i, _ctx);
    kk_std_core_types__maybe_drop(m, _ctx);
    return i;
  }
}

static inline kk_integer_t kk_std_core_negate(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> int */ 
  return kk_integer_neg(i,kk_context());
}

extern kk_ref_t kk_std_core_trace_enabled;

kk_function_t kk_std_core_o(kk_function_t f, kk_function_t g, kk_context_t* _ctx); /* forall<a,b,c,e> (f : (a) -> e b, g : (c) -> e a) -> ((x : c) -> e b) */ 
 
// Set a `hndler` that is always called when the `action` finishes (either normally or with an exception).

static inline kk_box_t kk_std_core_on_exit(kk_function_t hndler, kk_function_t action, kk_context_t* _ctx) { /* forall<a,e> (hndler : () -> e (), action : () -> e a) -> e a */ 
  kk_box_t _x21189 = kk_function_call(kk_box_t, (kk_function_t, kk_context_t*), action, (action, _ctx), _ctx); /*2862*/
  return kk_std_core_hnd_finally_prompt(hndler, _x21189, _ctx);
}

kk_box_t kk_std_core__mlift18793_once(kk_function_t calc, kk_ref_t r, kk_std_core_types__maybe _y_18459, kk_context_t* _ctx); /* forall<_h,_e,a> (calc : () -> a, r : ref<_h,maybe<a>>, maybe<a>) -> <read<_h>,write<_h>,div|_e> a */ 

kk_function_t kk_std_core_once(kk_function_t calc, kk_context_t* _ctx); /* forall<a> (calc : () -> a) -> (() -> a) */ 

kk_std_core_types__maybe kk_std_core_xparse_int(kk_string_t s, bool hex, kk_context_t* _ctx); /* (s : string, hex : bool) -> maybe<int> */ 

kk_box_t kk_std_core_phantom(kk_context_t* _ctx); /* forall<a> () -> a */ 
 
// Returns a singleton list.

static inline kk_std_core__list kk_std_core_single(kk_box_t x, kk_context_t* _ctx) { /* forall<a> (x : a) -> list<a> */ 
  return kk_std_core__new_Cons(kk_reuse_null, 0, x, kk_std_core__new_Nil(_ctx), _ctx);
}

kk_string_t kk_std_core_to_lower(kk_string_t s, kk_context_t* _ctx); /* (s : string) -> string */ 

kk_unit_t kk_std_core_xtrace(kk_string_t message0, kk_context_t* _ctx); /* (message : string) -> () */ 

kk_unit_t kk_std_core_xtrace_any(kk_string_t message0, kk_box_t x, kk_context_t* _ctx); /* forall<a> (message : string, x : a) -> () */ 

int8_t kk_std_core_uint8(kk_integer_t i, kk_context_t* _ctx); /* (i : int) -> int8 */ 

extern kk_ref_t kk_std_core_unique_count;
 
// _Unsafe_. This function removes the exception effect (`:exn`) from the effect of an action

static inline kk_box_t kk_std_core_unsafe_no_exn(kk_function_t action, kk_context_t* _ctx) { /* forall<a,e> (action : () -> <exn|e> a) -> e a */ 
  return kk_function_call(kk_box_t, (kk_function_t, kk_context_t*), action, (action, _ctx), _ctx);
}

kk_vector_t kk_std_core_unvlist(kk_std_core__list xs, kk_context_t* _ctx); /* forall<a> (xs : list<a>) -> vector<a> */ 

kk_vector_t kk_std_core_vector_initz(kk_ssize_t n, kk_function_t f, kk_context_t* _ctx); /* forall<a> (n : ssize_t, f : (ssize_t) -> a) -> vector<a> */ 

bool kk_std_core__lp__excl__eq__4_rp_(kk_std_core_types__order x, kk_std_core_types__order y, kk_context_t* _ctx); /* (x : order, y : order) -> bool */ 

static inline bool kk_std_core__lp__excl__eq__5_rp_(bool x, bool y, kk_context_t* _ctx) { /* (x : bool, y : bool) -> bool */ 
  if (x) {
    if (y) {
      return false;
    }
    {
      return true;
    }
  }
  {
    return y;
  }
}
 
// Add two integers.

static inline kk_integer_t kk_std_core__lp__plus__4_rp_(kk_integer_t x, kk_integer_t y, kk_context_t* _ctx) { /* (x : int, y : int) -> int */ 
  return kk_integer_add(x,y,kk_context());
}
 
// Add two character code points

static inline kk_char_t kk_std_core__lp__plus__3_rp_(kk_char_t c, kk_char_t d, kk_context_t* _ctx) { /* (c : char, d : char) -> total char */ 
  kk_integer_t x_17384 = kk_integer_from_int(c,kk_context()); /*int*/;
  kk_integer_t y_17385 = kk_integer_from_int(d,kk_context()); /*int*/;
  kk_integer_t _x21204 = kk_integer_add(x_17384,y_17385,kk_context()); /*int*/
  return kk_integer_clamp32(_x21204,kk_context());
}

kk_std_core__list kk_std_core__trmc_unroll17330_append(kk_std_core__list xs, kk_std_core__list ys, kk_std_core_types__cctx _acc, kk_context_t* _ctx); /* forall<a> (xs : list<a>, ys : list<a>, ctx<list<a>>) -> list<a> */ 

kk_std_core__list kk_std_core__unroll17330_append(kk_std_core__list xs0, kk_std_core__list ys0, kk_context_t* _ctx); /* forall<a> (xs : list<a>, ys : list<a>) -> list<a> */ 
 
// unrolling of singleton matches of std/core/.unroll17330-append

static inline kk_std_core__list kk_std_core_append(kk_std_core__list xs, kk_std_core__list ys, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, ys : list<a>) -> list<a> */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    return ys;
  }
  {
    return kk_std_core__unroll17330_append(xs, ys, _ctx);
  }
}
 
// Append two lists.

static inline kk_std_core__list kk_std_core__lp__plus__plus__rp_(kk_std_core__list xs, kk_std_core__list ys, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, ys : list<a>) -> list<a> */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    return ys;
  }
  {
    return kk_std_core__unroll17330_append(xs, ys, _ctx);
  }
}

kk_string_t kk_std_core__lp__plus__plus__1_rp_(kk_string_t x, kk_string_t y, kk_context_t* _ctx); /* (x : string, y : string) -> string */ 
 
// Substract two integers.

static inline kk_integer_t kk_std_core__lp__dash__4_rp_(kk_integer_t x, kk_integer_t y, kk_context_t* _ctx) { /* (x : int, y : int) -> int */ 
  return kk_integer_sub(x,y,kk_context());
}
 
// Substract two character codePoints

static inline kk_char_t kk_std_core__lp__dash__3_rp_(kk_char_t c, kk_char_t d, kk_context_t* _ctx) { /* (c : char, d : char) -> total char */ 
  kk_integer_t x_17388 = kk_integer_from_int(c,kk_context()); /*int*/;
  kk_integer_t y_17389 = kk_integer_from_int(d,kk_context()); /*int*/;
  kk_integer_t _x21209 = kk_integer_sub(x_17388,y_17389,kk_context()); /*int*/
  return kk_integer_clamp32(_x21209,kk_context());
}

kk_unit_t kk_std_core_printsln(kk_string_t s, kk_context_t* _ctx); /* (s : string) -> console () */ 

bool kk_std_core__lp__eq__eq__4_rp_(kk_std_core_types__order x, kk_std_core_types__order y, kk_context_t* _ctx); /* (x : order, y : order) -> bool */ 

static inline bool kk_std_core__lp__eq__eq__5_rp_(bool x, bool y, kk_context_t* _ctx) { /* (x : bool, y : bool) -> bool */ 
  if (x) {
    return y;
  }
  if (y) {
    return false;
  }
  {
    return true;
  }
}

bool kk_std_core__lp__lt__5_rp_(kk_std_core_types__order x, kk_std_core_types__order y, kk_context_t* _ctx); /* (x : order, y : order) -> bool */ 

static inline bool kk_std_core__lp__lt__6_rp_(bool x, bool y, kk_context_t* _ctx) { /* (x : bool, y : bool) -> bool */ 
  if (x) {
    return false;
  }
  {
    return y;
  }
}

kk_std_core_types__order kk_std_core_order(kk_integer_t i, kk_context_t* _ctx); /* (i : int) -> order */ 

kk_std_core_types__order kk_std_core_compare_3(kk_string_t x, kk_string_t y, kk_context_t* _ctx); /* (x : string, y : string) -> order */ 

static inline bool kk_std_core__lp__lt__7_rp_(kk_string_t x, kk_string_t y, kk_context_t* _ctx) { /* (x : string, y : string) -> bool */ 
  kk_std_core_types__order x0_17397 = kk_std_core_compare_3(x, y, _ctx); /*order*/;
  kk_integer_t _brw_20983;
  if (kk_std_core_types__is_Lt(x0_17397, _ctx)) {
    _brw_20983 = kk_integer_from_small(-1); /*int*/
    goto _match21220;
  }
  if (kk_std_core_types__is_Eq(x0_17397, _ctx)) {
    _brw_20983 = kk_integer_from_small(0); /*int*/
    goto _match21220;
  }
  {
    _brw_20983 = kk_integer_from_small(1); /*int*/
  }
  _match21220: ;
  bool _brw_20984 = kk_integer_eq_borrow(_brw_20983,(kk_integer_from_small(-1)),kk_context()); /*bool*/;
  kk_integer_drop(_brw_20983, _ctx);
  return _brw_20984;
}

bool kk_std_core__lp__gt__3_rp_(kk_std_core_types__order x, kk_std_core_types__order y, kk_context_t* _ctx); /* (x : order, y : order) -> bool */ 

static inline bool kk_std_core__lp__gt__4_rp_(bool x, bool y, kk_context_t* _ctx) { /* (x : bool, y : bool) -> bool */ 
  if (x) {
    if (y) {
      return false;
    }
    {
      return true;
    }
  }
  {
    return false;
  }
}

static inline bool kk_std_core__lp__gt__5_rp_(kk_string_t x, kk_string_t y, kk_context_t* _ctx) { /* (x : string, y : string) -> bool */ 
  kk_std_core_types__order x0_17404 = kk_std_core_compare_3(x, y, _ctx); /*order*/;
  kk_integer_t _brw_20978;
  if (kk_std_core_types__is_Lt(x0_17404, _ctx)) {
    _brw_20978 = kk_integer_from_small(-1); /*int*/
    goto _match21223;
  }
  if (kk_std_core_types__is_Eq(x0_17404, _ctx)) {
    _brw_20978 = kk_integer_from_small(0); /*int*/
    goto _match21223;
  }
  {
    _brw_20978 = kk_integer_from_small(1); /*int*/
  }
  _match21223: ;
  bool _brw_20979 = kk_integer_eq_borrow(_brw_20978,(kk_integer_from_small(1)),kk_context()); /*bool*/;
  kk_integer_drop(_brw_20978, _ctx);
  return _brw_20979;
}

static inline kk_std_core_types__order kk_std_core_compare_1(kk_char_t x, kk_char_t y, kk_context_t* _ctx) { /* (x : char, y : char) -> order */ 
  bool _match_20976 = (x < y); /*bool*/;
  if (_match_20976) {
    return kk_std_core_types__new_Lt(_ctx);
  }
  {
    bool _match_20977 = (x > y); /*bool*/;
    if (_match_20977) {
      return kk_std_core_types__new_Gt(_ctx);
    }
    {
      return kk_std_core_types__new_Eq(_ctx);
    }
  }
}

kk_std_core_types__order kk_std_core_compare_2(bool x, bool y, kk_context_t* _ctx); /* (x : bool, y : bool) -> order */ 

bool kk_std_core__lp__gt__eq__4_rp_(kk_std_core_types__order x, kk_std_core_types__order y, kk_context_t* _ctx); /* (x : order, y : order) -> bool */ 

static inline bool kk_std_core__lp__gt__eq__5_rp_(bool x, bool y, kk_context_t* _ctx) { /* (x : bool, y : bool) -> bool */ 
  if (x) {
    return true;
  }
  if (y) {
    return false;
  }
  {
    return true;
  }
}

static inline bool kk_std_core__lp__gt__eq__6_rp_(kk_string_t x, kk_string_t y, kk_context_t* _ctx) { /* (x : string, y : string) -> bool */ 
  kk_std_core_types__order x0_17420 = kk_std_core_compare_3(x, y, _ctx); /*order*/;
  kk_integer_t _brw_20971;
  if (kk_std_core_types__is_Lt(x0_17420, _ctx)) {
    _brw_20971 = kk_integer_from_small(-1); /*int*/
    goto _match21226;
  }
  if (kk_std_core_types__is_Eq(x0_17420, _ctx)) {
    _brw_20971 = kk_integer_from_small(0); /*int*/
    goto _match21226;
  }
  {
    _brw_20971 = kk_integer_from_small(1); /*int*/
  }
  _match21226: ;
  bool _brw_20972 = kk_integer_gt_borrow(_brw_20971,(kk_integer_from_small(-1)),kk_context()); /*bool*/;
  kk_integer_drop(_brw_20971, _ctx);
  return _brw_20972;
}

kk_string_t kk_std_core__unroll17331_lift17316_joinsep(kk_string_t sep, kk_std_core__list ys, kk_string_t acc, kk_context_t* _ctx); /* (sep : string, ys : list<string>, acc : string) -> string */ 
 
// unrolling of singleton matches of std/core/.unroll17331-lift17316-joinsep

static inline kk_string_t kk_std_core__lift17316_joinsep(kk_string_t sep, kk_std_core__list ys, kk_string_t acc, kk_context_t* _ctx) { /* (sep : string, ys : list<string>, acc : string) -> string */ 
  if (kk_std_core__is_Nil(ys, _ctx)) {
    kk_string_drop(sep, _ctx);
    return acc;
  }
  {
    return kk_std_core__unroll17331_lift17316_joinsep(sep, ys, acc, _ctx);
  }
}

kk_string_t kk_std_core_joinsep(kk_std_core__list xs, kk_string_t sep, kk_context_t* _ctx); /* (xs : list<string>, sep : string) -> string */ 

kk_string_t kk_std_core_join_2(kk_std_core__list xs, kk_context_t* _ctx); /* (xs : list<string>) -> string */ 

kk_string_t kk_std_core_join_3(kk_std_core__list xs, kk_string_t sep, kk_context_t* _ctx); /* (xs : list<string>, sep : string) -> string */ 

bool kk_std_core__lp__lt__eq__5_rp_(kk_std_core_types__order x, kk_std_core_types__order y, kk_context_t* _ctx); /* (x : order, y : order) -> bool */ 

static inline bool kk_std_core__lp__lt__eq__6_rp_(bool x, bool y, kk_context_t* _ctx) { /* (x : bool, y : bool) -> bool */ 
  if (x) {
    if (y) {
      return true;
    }
    {
      return false;
    }
  }
  {
    return true;
  }
}

static inline bool kk_std_core__lp__lt__eq__7_rp_(kk_string_t x, kk_string_t y, kk_context_t* _ctx) { /* (x : string, y : string) -> bool */ 
  kk_std_core_types__order x0_17442 = kk_std_core_compare_3(x, y, _ctx); /*order*/;
  kk_integer_t _brw_20966;
  if (kk_std_core_types__is_Lt(x0_17442, _ctx)) {
    _brw_20966 = kk_integer_from_small(-1); /*int*/
    goto _match21240;
  }
  if (kk_std_core_types__is_Eq(x0_17442, _ctx)) {
    _brw_20966 = kk_integer_from_small(0); /*int*/
    goto _match21240;
  }
  {
    _brw_20966 = kk_integer_from_small(1); /*int*/
  }
  _match21240: ;
  bool _brw_20967 = kk_integer_lt_borrow(_brw_20966,(kk_integer_from_small(1)),kk_context()); /*bool*/;
  kk_integer_drop(_brw_20966, _ctx);
  return _brw_20967;
}

static inline kk_integer_t kk_std_core_inc(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> int */ 
  return kk_integer_add_small_const(i, 1, _ctx);
}

static inline int32_t kk_std_core_decr(int32_t i, kk_context_t* _ctx) { /* (i : int32) -> int32 */ 
  return (i - ((KK_I32(1))));
}

kk_ssize_t kk_std_core_decr_1(kk_ssize_t i, kk_context_t* _ctx); /* (i : ssize_t) -> ssize_t */ 

static inline int32_t kk_std_core_incr(int32_t i, kk_context_t* _ctx) { /* (i : int32) -> int32 */ 
  return (i + ((KK_I32(1))));
}

kk_ssize_t kk_std_core_incr_1(kk_ssize_t i, kk_context_t* _ctx); /* (i : ssize_t) -> ssize_t */ 

kk_unit_t kk_std_core__mlift18794_lift17317_forz(kk_function_t action, kk_ssize_t end, kk_ssize_t i, kk_unit_t wild__, kk_context_t* _ctx); /* forall<e> (action : (ssize_t) -> e (), end : ssize_t, i : ssize_t, wild_ : ()) -> e () */ 

kk_unit_t kk_std_core__lift17317_forz(kk_function_t action0, kk_ssize_t end0, kk_ssize_t i0, kk_context_t* _ctx); /* forall<e> (action : (ssize_t) -> e (), end : ssize_t, i : ssize_t) -> e () */ 
 
// Executes `action`  for each integer between `start`  upto `end`  (including both `start`  and `end` ).
// If `start > end`  the function returns without any call to `action` .

static inline kk_unit_t kk_std_core_forz(kk_ssize_t start0, kk_ssize_t end, kk_function_t action, kk_context_t* _ctx) { /* forall<e> (start : ssize_t, end : ssize_t, action : (ssize_t) -> e ()) -> e () */ 
  kk_std_core__lift17317_forz(action, end, start0, _ctx); return kk_Unit;
}

kk_unit_t kk_std_core__mlift18795_lift18298_foreach_indexedz(kk_ssize_t end18037, kk_function_t f, kk_ssize_t i18038, kk_vector_t v, kk_unit_t wild__18040, kk_context_t* _ctx); /* forall<a,e> (end18037 : ssize_t, f : (ssize_t, a) -> e (), i18038 : ssize_t, v : vector<a>, wild_18040 : ()) -> e () */ 

kk_unit_t kk_std_core__lift18298_foreach_indexedz(kk_function_t f0, kk_vector_t v0, kk_ssize_t end180370, kk_ssize_t i180380, kk_context_t* _ctx); /* forall<a,e> (f : (ssize_t, a) -> e (), v : vector<a>, end18037 : ssize_t, i18038 : ssize_t) -> e () */ 

static inline kk_unit_t kk_std_core_foreach_indexedz(kk_vector_t v, kk_function_t f, kk_context_t* _ctx) { /* forall<a,e> (v : vector<a>, f : (ssize_t, a) -> e ()) -> e () */ 
  kk_ssize_t start0_17449 = (KK_IZ(0)); /*ssize_t*/;
  kk_ssize_t end_17450;
  kk_ssize_t _x21250 = kk_vector_len_borrow(v,kk_context()); /*ssize_t*/
  end_17450 = kk_std_core_decr_1(_x21250, _ctx); /*ssize_t*/
  kk_std_core__lift18298_foreach_indexedz(f, v, end_17450, start0_17449, _ctx); return kk_Unit;
}

kk_integer_t kk_std_core__unroll17332_lift17318_length_1(kk_std_core__list ys, kk_integer_t acc, kk_context_t* _ctx); /* forall<a> (ys : list<a>, acc : int) -> int */ 
 
// unrolling of singleton matches of std/core/.unroll17332-lift17318-length.1

static inline kk_integer_t kk_std_core__lift17318_length_1(kk_std_core__list ys, kk_integer_t acc, kk_context_t* _ctx) { /* forall<a> (ys : list<a>, acc : int) -> int */ 
  if (kk_std_core__is_Nil(ys, _ctx)) {
    return acc;
  }
  {
    return kk_std_core__unroll17332_lift17318_length_1(ys, acc, _ctx);
  }
}
 
// Returns the length of a list.

static inline kk_integer_t kk_std_core_length_1(kk_std_core__list xs, kk_context_t* _ctx) { /* forall<a> (xs : list<a>) -> int */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    return kk_integer_from_small(0);
  }
  {
    return kk_std_core__unroll17332_lift17318_length_1(xs, kk_integer_from_small(0), _ctx);
  }
}
 
// Return the length of a vector.

static inline kk_integer_t kk_std_core_length_2(kk_vector_t v, kk_context_t* _ctx) { /* forall<a> (v : vector<a>) -> int */ 
  kk_ssize_t _x21253 = kk_vector_len_borrow(v,kk_context()); /*ssize_t*/
  return kk_integer_from_ssize_t(_x21253,kk_context());
}

kk_std_core__list kk_std_core__trmc_list(kk_integer_t lo, kk_integer_t hi, kk_std_core_types__cctx _acc, kk_context_t* _ctx); /* (lo : int, hi : int, ctx<list<int>>) -> total list<int> */ 

kk_std_core__list kk_std_core_list(kk_integer_t lo0, kk_integer_t hi0, kk_context_t* _ctx); /* (lo : int, hi : int) -> total list<int> */ 

kk_std_core__list kk_std_core__trmc_list_1(kk_integer_t lo, kk_integer_t hi, kk_integer_t stride, kk_std_core_types__cctx _acc, kk_context_t* _ctx); /* (lo : int, hi : int, stride : int, ctx<list<int>>) -> total list<int> */ 

kk_std_core__list kk_std_core_list_1(kk_integer_t lo0, kk_integer_t hi0, kk_integer_t stride0, kk_context_t* _ctx); /* (lo : int, hi : int, stride : int) -> total list<int> */ 

kk_std_core__list kk_std_core__mlift18796_trmc_list_2(kk_std_core_types__cctx _acc, kk_function_t f, kk_integer_t hi, kk_integer_t lo, kk_box_t _trmc_18375, kk_context_t* _ctx); /* forall<a,e> (ctx<list<a>>, f : (int) -> e a, hi : int, lo : int, a) -> e list<a> */ 

kk_std_core__list kk_std_core__trmc_list_2(kk_integer_t lo0, kk_integer_t hi0, kk_function_t f0, kk_std_core_types__cctx _acc0, kk_context_t* _ctx); /* forall<a,e> (lo : int, hi : int, f : (int) -> e a, ctx<list<a>>) -> e list<a> */ 

kk_std_core__list kk_std_core_list_2(kk_integer_t lo1, kk_integer_t hi1, kk_function_t f1, kk_context_t* _ctx); /* forall<a,e> (lo : int, hi : int, f : (int) -> e a) -> e list<a> */ 

kk_std_core__list kk_std_core__mlift18797_trmc_list_3(kk_std_core_types__cctx _acc, kk_function_t f, kk_integer_t hi, kk_integer_t lo, kk_integer_t stride, kk_box_t _trmc_18378, kk_context_t* _ctx); /* forall<a,e> (ctx<list<a>>, f : (int) -> e a, hi : int, lo : int, stride : int, a) -> e list<a> */ 

kk_std_core__list kk_std_core__trmc_list_3(kk_integer_t lo0, kk_integer_t hi0, kk_integer_t stride0, kk_function_t f0, kk_std_core_types__cctx _acc0, kk_context_t* _ctx); /* forall<a,e> (lo : int, hi : int, stride : int, f : (int) -> e a, ctx<list<a>>) -> e list<a> */ 

kk_std_core__list kk_std_core_list_3(kk_integer_t lo1, kk_integer_t hi1, kk_integer_t stride1, kk_function_t f1, kk_context_t* _ctx); /* forall<a,e> (lo : int, hi : int, stride : int, f : (int) -> e a) -> e list<a> */ 

kk_std_core__list kk_std_core__mlift18798_trmc_unroll17333_map_5(kk_std_core_types__cctx _acc, kk_function_t f, kk_std_core__list xx, kk_box_t _trmc_18381, kk_context_t* _ctx); /* forall<a,b,e> (ctx<list<b>>, f : (a) -> e b, xx : list<a>, b) -> e list<b> */ 

kk_std_core__list kk_std_core__trmc_unroll17333_map_5(kk_std_core__list xs, kk_function_t f0, kk_std_core_types__cctx _acc0, kk_context_t* _ctx); /* forall<a,b,e> (xs : list<a>, f : (a) -> e b, ctx<list<b>>) -> e list<b> */ 

kk_std_core__list kk_std_core__unroll17333_map_5(kk_std_core__list xs0, kk_function_t f1, kk_context_t* _ctx); /* forall<a,b,e> (xs : list<a>, f : (a) -> e b) -> e list<b> */ 
 
// unrolling of singleton matches of std/core/.unroll17333-map.5

static inline kk_std_core__list kk_std_core_map_5(kk_std_core__list xs, kk_function_t f, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, f : (a) -> e b) -> e list<b> */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    kk_function_drop(f, _ctx);
    return kk_std_core__new_Nil(_ctx);
  }
  {
    return kk_std_core__unroll17333_map_5(xs, f, _ctx);
  }
}

kk_std_core__list kk_std_core__trmc_lift18299_list_4(kk_std_core__list xs18045, kk_std_core_types__cctx _acc, kk_context_t* _ctx); /* (xs18045 : list<int>, ctx<list<char>>) -> total list<char> */ 

kk_std_core__list kk_std_core__lift18299_list_4(kk_std_core__list xs180450, kk_context_t* _ctx); /* (xs18045 : list<int>) -> total list<char> */ 
 
// Create a list of characters from `lo`  to `hi`  (inclusive).

static inline kk_std_core__list kk_std_core_list_4(kk_char_t lo, kk_char_t hi, kk_context_t* _ctx) { /* (lo : char, hi : char) -> total list<char> */ 
  kk_std_core__list xs_17461;
  kk_integer_t _x21304 = kk_integer_from_int(lo,kk_context()); /*int*/
  kk_integer_t _x21305 = kk_integer_from_int(hi,kk_context()); /*int*/
  xs_17461 = kk_std_core_list(_x21304, _x21305, _ctx); /*list<int>*/
  if (kk_std_core__is_Nil(xs_17461, _ctx)) {
    return kk_std_core__new_Nil(_ctx);
  }
  {
    return kk_std_core__lift18299_list_4(xs_17461, _ctx);
  }
}
 
// Convert a `:maybe` type to a list type.

static inline kk_std_core__list kk_std_core_list_5(kk_std_core_types__maybe m, kk_context_t* _ctx) { /* forall<a> (m : maybe<a>) -> list<a> */ 
  if (kk_std_core_types__is_Nothing(m, _ctx)) {
    return kk_std_core__new_Nil(_ctx);
  }
  {
    kk_box_t x = m._cons.Just.value;
    return kk_std_core__new_Cons(kk_reuse_null, 0, x, kk_std_core__new_Nil(_ctx), _ctx);
  }
}

kk_std_core__list kk_std_core_list_6(kk_string_t s, kk_context_t* _ctx); /* (s : string) -> total list<char> */ 
 
// Convert a vector to a list.

static inline kk_std_core__list kk_std_core_list_7(kk_vector_t v, kk_context_t* _ctx) { /* forall<a> (v : vector<a>) -> list<a> */ 
  return kk_std_core_vlist(v, kk_std_core_types__new_None(_ctx), _ctx);
}
 
// monadic lift

static inline kk_std_core_types__maybe kk_std_core__mlift18799_map(kk_box_t _y_18483, kk_context_t* _ctx) { /* forall<a,e> (a) -> e maybe<a> */ 
  return kk_std_core_types__new_Just(_y_18483, _ctx);
}

kk_std_core_types__maybe kk_std_core_map(kk_std_core_types__maybe m, kk_function_t f, kk_context_t* _ctx); /* forall<a,b,e> (m : maybe<a>, f : (a) -> e b) -> e maybe<b> */ 
 
// monadic lift

static inline kk_std_core_types__either kk_std_core__mlift18800_map_1(kk_box_t _y_18485, kk_context_t* _ctx) { /* forall<a,b,e> (b) -> e either<a,b> */ 
  return kk_std_core_types__new_Right(_y_18485, _ctx);
}

kk_std_core_types__either kk_std_core_map_1(kk_std_core_types__either e, kk_function_t f, kk_context_t* _ctx); /* forall<a,b,c,e> (e : either<a,b>, f : (b) -> e c) -> e either<a,c> */ 
 
// monadic lift

static inline kk_std_core_types__tuple2_ kk_std_core__mlift18801_map_2(kk_box_t _y_18487, kk_box_t _y_18488, kk_context_t* _ctx) { /* forall<a,e> (a, a) -> e (a, a) */ 
  return kk_std_core_types__new_dash__lp__comma__rp_(_y_18487, _y_18488, _ctx);
}

kk_std_core_types__tuple2_ kk_std_core__mlift18802_map_2(kk_function_t f, kk_std_core_types__tuple2_ t, kk_box_t _y_18487, kk_context_t* _ctx); /* forall<a,b,e> (f : (a) -> e b, t : (a, a), b) -> e (b, b) */ 

kk_std_core_types__tuple2_ kk_std_core_map_2(kk_std_core_types__tuple2_ t, kk_function_t f, kk_context_t* _ctx); /* forall<a,b,e> (t : (a, a), f : (a) -> e b) -> e (b, b) */ 
 
// monadic lift

static inline kk_std_core_types__tuple3_ kk_std_core__mlift18803_map_3(kk_box_t _y_18489, kk_box_t _y_18490, kk_box_t _y_18491, kk_context_t* _ctx) { /* forall<a,e> (a, a, a) -> e (a, a, a) */ 
  return kk_std_core_types__new_dash__lp__comma__comma__rp_(_y_18489, _y_18490, _y_18491, _ctx);
}

kk_std_core_types__tuple3_ kk_std_core__mlift18804_map_3(kk_box_t _y_18489, kk_function_t f, kk_std_core_types__tuple3_ t, kk_box_t _y_18490, kk_context_t* _ctx); /* forall<a,b,e> (b, f : (a) -> e b, t : (a, a, a), b) -> e (b, b, b) */ 

kk_std_core_types__tuple3_ kk_std_core__mlift18805_map_3(kk_function_t f, kk_std_core_types__tuple3_ t, kk_box_t _y_18489, kk_context_t* _ctx); /* forall<a,b,e> (f : (a) -> e b, t : (a, a, a), b) -> e (b, b, b) */ 

kk_std_core_types__tuple3_ kk_std_core_map_3(kk_std_core_types__tuple3_ t, kk_function_t f, kk_context_t* _ctx); /* forall<a,b,e> (t : (a, a, a), f : (a) -> e b) -> e (b, b, b) */ 
 
// monadic lift

static inline kk_std_core_types__tuple4_ kk_std_core__mlift18806_map_4(kk_box_t _y_18492, kk_box_t _y_18493, kk_box_t _y_18494, kk_box_t _y_18495, kk_context_t* _ctx) { /* forall<a,e> (a, a, a, a) -> e (a, a, a, a) */ 
  return kk_std_core_types__new_dash__lp__comma__comma__comma__rp_(kk_reuse_null, 0, _y_18492, _y_18493, _y_18494, _y_18495, _ctx);
}

kk_std_core_types__tuple4_ kk_std_core__mlift18807_map_4(kk_box_t _y_18492, kk_box_t _y_18493, kk_function_t f, kk_std_core_types__tuple4_ t, kk_box_t _y_18494, kk_context_t* _ctx); /* forall<a,b,e> (b, b, f : (a) -> e b, t : (a, a, a, a), b) -> e (b, b, b, b) */ 

kk_std_core_types__tuple4_ kk_std_core__mlift18808_map_4(kk_box_t _y_18492, kk_function_t f, kk_std_core_types__tuple4_ t, kk_box_t _y_18493, kk_context_t* _ctx); /* forall<a,b,e> (b, f : (a) -> e b, t : (a, a, a, a), b) -> e (b, b, b, b) */ 

kk_std_core_types__tuple4_ kk_std_core__mlift18809_map_4(kk_function_t f, kk_std_core_types__tuple4_ t, kk_box_t _y_18492, kk_context_t* _ctx); /* forall<a,b,e> (f : (a) -> e b, t : (a, a, a, a), b) -> e (b, b, b, b) */ 

kk_std_core_types__tuple4_ kk_std_core_map_4(kk_std_core_types__tuple4_ t, kk_function_t f, kk_context_t* _ctx); /* forall<a,b,e> (t : (a, a, a, a), f : (a) -> e b) -> e (b, b, b, b) */ 

kk_string_t kk_std_core_map_6(kk_string_t s, kk_function_t f, kk_context_t* _ctx); /* forall<e> (s : string, f : (char) -> e char) -> e string */ 

kk_unit_t kk_std_core__mlift18810_lift18300_map_7(kk_ssize_t end18053, kk_function_t f, kk_ssize_t i18054, kk_vector_t v, kk_vector_t w, kk_box_t _y_18498, kk_context_t* _ctx); /* forall<a,b,e> (end18053 : ssize_t, f : (a) -> e b, i18054 : ssize_t, v : vector<a>, w : vector<b>, b) -> e () */ 

kk_unit_t kk_std_core__lift18300_map_7(kk_function_t f0, kk_vector_t v0, kk_vector_t w0, kk_ssize_t end180530, kk_ssize_t i180540, kk_context_t* _ctx); /* forall<a,b,e> (f : (a) -> e b, v : vector<a>, w : vector<b>, end18053 : ssize_t, i18054 : ssize_t) -> e () */ 
 
// monadic lift

static inline kk_vector_t kk_std_core__mlift18811_map_7(kk_vector_t w, kk_unit_t wild__, kk_context_t* _ctx) { /* forall<a,e> (w : vector<a>, wild_ : ()) -> e vector<a> */ 
  return w;
}

kk_vector_t kk_std_core_map_7(kk_vector_t v, kk_function_t f, kk_context_t* _ctx); /* forall<a,b,e> (v : vector<a>, f : (a) -> e b) -> e vector<b> */ 

kk_unit_t kk_std_core__mlift18812_lift17319_for(kk_function_t action, kk_integer_t end, kk_integer_t i, kk_unit_t wild__, kk_context_t* _ctx); /* forall<e> (action : (int) -> e (), end : int, i : int, wild_ : ()) -> e () */ 

kk_unit_t kk_std_core__lift17319_for(kk_function_t action0, kk_integer_t end0, kk_integer_t i0, kk_context_t* _ctx); /* forall<e> (action : (int) -> e (), end : int, i : int) -> e () */ 
 
// Executes `action`  for each integer between `start`  upto `end`  (including both `start`  and `end` ).
// If `start > end`  the function returns without any call to `action` .

static inline kk_unit_t kk_std_core_for(kk_integer_t start0, kk_integer_t end, kk_function_t action, kk_context_t* _ctx) { /* forall<e> (start : int, end : int, action : (int) -> e ()) -> e () */ 
  kk_integer_t _x21413 = kk_integer_dup(start0, _ctx); /*int*/
  kk_std_core__lift17319_for(action, end, _x21413, _ctx); return kk_Unit;
}
 
// Repeat a string `n` times

static inline kk_string_t kk_std_core_repeat(kk_string_t s, kk_integer_t n, kk_context_t* _ctx) { /* (s : string, n : int) -> string */ 
  kk_ssize_t _x21414;
  kk_integer_t _x21415 = kk_integer_dup(n, _ctx); /*int*/
  _x21414 = kk_std_core_ssize__t(_x21415, _ctx); /*ssize_t*/
  return kk_std_core_repeatz(s, _x21414, _ctx);
}

kk_unit_t kk_std_core__mlift18813_lift18301_repeat_1(kk_function_t action, kk_integer_t end18062, kk_integer_t i18063, kk_unit_t wild__18065, kk_context_t* _ctx); /* forall<e> (action : () -> e (), end18062 : int, i18063 : int, wild_18065 : ()) -> e () */ 

kk_unit_t kk_std_core__lift18301_repeat_1(kk_function_t action0, kk_integer_t end180620, kk_integer_t i180630, kk_context_t* _ctx); /* forall<e> (action : () -> e (), end18062 : int, i18063 : int) -> e () */ 
 
// The `repeat` fun executes `action`  `n`  times.

static inline kk_unit_t kk_std_core_repeat_1(kk_integer_t n, kk_function_t action, kk_context_t* _ctx) { /* forall<e> (n : int, action : () -> e ()) -> e () */ 
  kk_integer_t _x21420 = kk_integer_dup(n, _ctx); /*int*/
  kk_std_core__lift18301_repeat_1(action, _x21420, kk_integer_from_small(1), _ctx); return kk_Unit;
}

kk_string_t kk_std_core_pad_left(kk_string_t s, kk_integer_t width, kk_std_core_types__optional fill, kk_context_t* _ctx); /* (s : string, width : int, fill : optional<char>) -> string */ 

kk_string_t kk_std_core_show_hex(kk_integer_t i, kk_std_core_types__optional width, kk_std_core_types__optional use_capitals, kk_std_core_types__optional pre, kk_context_t* _ctx); /* (i : int, width : optional<int>, use-capitals : optional<bool>, pre : optional<string>) -> string */ 
 
// Is the integer positive (stricly greater than zero)

static inline bool kk_std_core_is_pos_2(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> bool */ 
  bool _brw_20923 = kk_integer_gt_borrow(i,(kk_integer_from_small(0)),kk_context()); /*bool*/;
  kk_integer_drop(i, _ctx);
  return _brw_20923;
}
 
// Is the value positive?

static inline bool kk_std_core_is_pos_3(double d, kk_context_t* _ctx) { /* (d : float64) -> bool */ 
  return (d > (0x0p+0));
}
 
// Is the list empty?

static inline bool kk_std_core_is_empty(kk_std_core__list xs, kk_context_t* _ctx) { /* forall<a> (xs : list<a>) -> bool */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    return true;
  }
  {
    kk_std_core__list_drop(xs, _ctx);
    return false;
  }
}
 
// Is a slice empty?

static inline bool kk_std_core_is_empty_1(kk_std_core__sslice slice0, kk_context_t* _ctx) { /* (slice : sslice) -> bool */ 
  bool b_17487;
  kk_integer_t _brw_20921;
  {
    kk_integer_t _x = slice0.len;
    kk_integer_dup(_x, _ctx);
    kk_std_core__sslice_drop(slice0, _ctx);
    _brw_20921 = _x; /*int*/
  }
  bool _brw_20922 = kk_integer_gt_borrow(_brw_20921,(kk_integer_from_small(0)),kk_context()); /*bool*/;
  kk_integer_drop(_brw_20921, _ctx);
  b_17487 = _brw_20922; /*bool*/
  if (b_17487) {
    return false;
  }
  {
    return true;
  }
}
 
// Is a string empty?

static inline bool kk_std_core_is_empty_2(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> bool */ 
  kk_string_t _x21440 = kk_string_empty(); /*string*/
  return kk_string_is_eq(s,_x21440,kk_context());
}

static inline kk_std_core_types__maybe kk_std_core__lp__bar__bar__rp_(kk_std_core_types__maybe m1, kk_std_core_types__maybe m2, kk_context_t* _ctx) { /* forall<a> (m1 : maybe<a>, m2 : maybe<a>) -> maybe<a> */ 
  if (kk_std_core_types__is_Nothing(m1, _ctx)) {
    return m2;
  }
  {
    kk_std_core_types__maybe_drop(m2, _ctx);
    return m1;
  }
}
 
// Choose a non-empty string

static inline kk_string_t kk_std_core__lp__bar__bar__1_rp_(kk_string_t x, kk_string_t y, kk_context_t* _ctx) { /* (x : string, y : string) -> string */ 
  bool _match_20920;
  kk_string_t _x21442 = kk_string_dup(x, _ctx); /*string*/
  kk_string_t _x21443 = kk_string_empty(); /*string*/
  _match_20920 = kk_string_is_eq(_x21442,_x21443,kk_context()); /*bool*/
  if (_match_20920) {
    kk_string_drop(x, _ctx);
    return y;
  }
  {
    kk_string_drop(y, _ctx);
    return x;
  }
}

kk_string_t kk_std_core_show_char(kk_char_t c, kk_context_t* _ctx); /* (c : char) -> string */ 

kk_string_t kk_std_core_show_exp(double d, kk_std_core_types__optional precision, kk_context_t* _ctx); /* (d : float64, precision : optional<int>) -> string */ 

kk_string_t kk_std_core_show_fixed(double d, kk_std_core_types__optional precision, kk_context_t* _ctx); /* (d : float64, precision : optional<int>) -> string */ 

kk_string_t kk_std_core__mlift18814_show_list(kk_std_core__list _c_18511, kk_context_t* _ctx); /* (list<string>) -> string */ 

kk_string_t kk_std_core_show_list(kk_std_core__list xs, kk_function_t show_elem, kk_context_t* _ctx); /* forall<a,e> (xs : list<a>, show-elem : (a) -> e string) -> e string */ 

kk_string_t kk_std_core_show(kk_integer_t i, kk_context_t* _ctx); /* (i : int) -> string */ 

kk_string_t kk_std_core_show_1(double d, kk_std_core_types__optional precision, kk_context_t* _ctx); /* (d : float64, precision : optional<int>) -> string */ 
 
// Show the exception message

static inline kk_string_t kk_std_core_show_10(kk_std_core__exception exn0, kk_context_t* _ctx) { /* (exn : exception) -> string */ 
  {
    kk_string_t _x = exn0.message;
    kk_string_dup(_x, _ctx);
    kk_std_core__exception_drop(exn0, _ctx);
    return _x;
  }
}
 
// Show a `:char` as a character literal

static inline kk_string_t kk_std_core_show_2(kk_char_t c, kk_context_t* _ctx) { /* (c : char) -> string */ 
  kk_string_t _x21594;
  kk_define_string_literal(, _s21595, 1, "\'", _ctx)
  _x21594 = kk_string_dup(_s21595, _ctx); /*string*/
  kk_string_t _x21596;
  kk_string_t _x21597 = kk_std_core_show_char(c, _ctx); /*string*/
  kk_string_t _x21598;
  kk_define_string_literal(, _s21599, 1, "\'", _ctx)
  _x21598 = kk_string_dup(_s21599, _ctx); /*string*/
  _x21596 = kk_std_core__lp__plus__plus__1_rp_(_x21597, _x21598, _ctx); /*string*/
  return kk_std_core__lp__plus__plus__1_rp_(_x21594, _x21596, _ctx);
}

kk_std_core__list kk_std_core__trmc_lift18302_show_3(kk_std_core__list xs18071, kk_std_core_types__cctx _acc, kk_context_t* _ctx); /* (xs18071 : list<char>, ctx<list<string>>) -> list<string> */ 

kk_std_core__list kk_std_core__lift18302_show_3(kk_std_core__list xs180710, kk_context_t* _ctx); /* (xs18071 : list<char>) -> list<string> */ 

kk_string_t kk_std_core_show_3(kk_string_t s, kk_context_t* _ctx); /* (s : string) -> string */ 
 
// Convert a `:bool` to a string

static inline kk_string_t kk_std_core_show_4(bool b, kk_context_t* _ctx) { /* (b : bool) -> string */ 
  if (b) {
    kk_define_string_literal(, _s21614, 4, "True", _ctx)
    return kk_string_dup(_s21614, _ctx);
  }
  {
    kk_define_string_literal(, _s21615, 5, "False", _ctx)
    return kk_string_dup(_s21615, _ctx);
  }
}
 
// Convert a unit value `()` to a string

static inline kk_string_t kk_std_core_show_5(kk_unit_t u, kk_context_t* _ctx) { /* (u : ()) -> string */ 
  kk_define_string_literal(, _s21616, 2, "()", _ctx)
  return kk_string_dup(_s21616, _ctx);
}
 
// Show an `:sslice` as a string literal

static inline kk_string_t kk_std_core_show_6(kk_std_core__sslice s, kk_context_t* _ctx) { /* (s : sslice) -> string */ 
  kk_string_t _x21617 = kk_std_core_string_3(s, _ctx); /*string*/
  return kk_std_core_show_3(_x21617, _ctx);
}

kk_std_core__list kk_std_core__trmc_lift18304_show_7(kk_std_core__list xs18091, kk_std_core_types__cctx _acc, kk_context_t* _ctx); /* (xs18091 : list<string>, ctx<list<string>>) -> list<string> */ 

kk_std_core__list kk_std_core__lift18304_show_7(kk_std_core__list xs180910, kk_context_t* _ctx); /* (xs18091 : list<string>) -> list<string> */ 

kk_string_t kk_std_core__lift18303_show_7(kk_std_core__list xs18078, kk_context_t* _ctx); /* (xs18078 : list<string>) -> string */ 

static inline kk_string_t kk_std_core_show_7(kk_std_core__list xs, kk_context_t* _ctx) { /* (xs : list<string>) -> string */ 
  return kk_std_core__lift18303_show_7(xs, _ctx);
}

kk_std_core__list kk_std_core__trmc_lift18306_show_8(kk_std_core__list xs18111, kk_std_core_types__cctx _acc, kk_context_t* _ctx); /* (xs18111 : list<int>, ctx<list<string>>) -> list<string> */ 

kk_std_core__list kk_std_core__lift18306_show_8(kk_std_core__list xs181110, kk_context_t* _ctx); /* (xs18111 : list<int>) -> list<string> */ 

kk_string_t kk_std_core__lift18305_show_8(kk_std_core__list xs18098, kk_context_t* _ctx); /* (xs18098 : list<int>) -> string */ 

static inline kk_string_t kk_std_core_show_8(kk_std_core__list xs, kk_context_t* _ctx) { /* (xs : list<int>) -> string */ 
  return kk_std_core__lift18305_show_8(xs, _ctx);
}

kk_std_core__list kk_std_core__trmc_lift18308_show_9(kk_std_core__list xs18131, kk_std_core_types__cctx _acc, kk_context_t* _ctx); /* (xs18131 : list<bool>, ctx<list<string>>) -> list<string> */ 

kk_std_core__list kk_std_core__lift18308_show_9(kk_std_core__list xs181310, kk_context_t* _ctx); /* (xs18131 : list<bool>) -> list<string> */ 

kk_string_t kk_std_core__lift18307_show_9(kk_std_core__list xs18118, kk_context_t* _ctx); /* (xs18118 : list<bool>) -> string */ 

static inline kk_string_t kk_std_core_show_9(kk_std_core__list xs, kk_context_t* _ctx) { /* (xs : list<bool>) -> string */ 
  return kk_std_core__lift18307_show_9(xs, _ctx);
}
 
// Print a string to the console, including a final newline character.

static inline kk_unit_t kk_std_core_println(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> console () */ 
  kk_std_core_printsln(s, _ctx); return kk_Unit;
}
 
// Print an integer to the console, including a final newline character.

static inline kk_unit_t kk_std_core_println_1(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> console () */ 
  kk_string_t _x21664 = kk_std_core_show(i, _ctx); /*string*/
  kk_std_core_printsln(_x21664, _ctx); return kk_Unit;
}
 
// Print a float64 to the console, including a final newline character.

static inline kk_unit_t kk_std_core_println_2(double d, kk_context_t* _ctx) { /* (d : float64) -> console () */ 
  kk_string_t _x21665 = kk_std_core_show_1(d, kk_std_core_types__new_None(_ctx), _ctx); /*string*/
  kk_std_core_printsln(_x21665, _ctx); return kk_Unit;
}
 
// Print a boolean to the console, including a final newline character

static inline kk_unit_t kk_std_core_println_3(bool b, kk_context_t* _ctx) { /* (b : bool) -> console () */ 
  kk_string_t _x21666;
  if (b) {
    kk_define_string_literal(, _s21667, 4, "True", _ctx)
    _x21666 = kk_string_dup(_s21667, _ctx); /*string*/
  }
  else {
    kk_define_string_literal(, _s21668, 5, "False", _ctx)
    _x21666 = kk_string_dup(_s21668, _ctx); /*string*/
  }
  kk_std_core_printsln(_x21666, _ctx); return kk_Unit;
}
 
// Print a character to the console, including a final newline character.

static inline kk_unit_t kk_std_core_println_4(kk_char_t c, kk_context_t* _ctx) { /* (c : char) -> console () */ 
  kk_string_t _x21669 = kk_std_core_string(c, _ctx); /*string*/
  kk_std_core_printsln(_x21669, _ctx); return kk_Unit;
}
 
// Print a unit value to the console, including a final newline character

static inline kk_unit_t kk_std_core_println_5(kk_unit_t u, kk_context_t* _ctx) { /* (u : ()) -> console () */ 
  kk_string_t _x21670;
  kk_define_string_literal(, _s21671, 2, "()", _ctx)
  _x21670 = kk_string_dup(_s21671, _ctx); /*string*/
  kk_std_core_printsln(_x21670, _ctx); return kk_Unit;
}

kk_unit_t kk_std_core_prints(kk_string_t s, kk_context_t* _ctx); /* (s : string) -> console () */ 
 
// call `throw-exn` operation of the `:exn` effect

static inline kk_box_t kk_std_core_throw_exn(kk_std_core__exception exn0, kk_context_t* _ctx) { /* forall<a> (exn : exception) -> exn a */ 
  kk_std_core_hnd__ev ev_18981;
  kk_ssize_t _x21676 = (KK_IZ(0)); /*ssize_t*/
  ev_18981 = kk_evv_at(_x21676,kk_context()); /*std/core/hnd/ev<.hnd-exn>*/
  {
    struct kk_std_core_hnd_Ev* _con21677 = kk_std_core_hnd__as_Ev(ev_18981, _ctx);
    kk_box_t _box_x19811 = _con21677->hnd;
    kk_std_core_hnd__marker m0 = _con21677->marker;
    kk_std_core__hnd_exn h = kk_std_core__hnd_exn_unbox(_box_x19811, KK_BORROWED, _ctx);
    kk_std_core__hnd_exn_dup(h, _ctx);
    {
      struct kk_std_core__Hnd_exn* _con21678 = kk_std_core__as_Hnd_exn(h, _ctx);
      kk_std_core_hnd__clause1 ctl_throw_exn = _con21678->ctl_throw_exn;
      if kk_likely(kk_datatype_ptr_is_unique(h, _ctx)) {
        kk_datatype_ptr_free(h, _ctx);
      }
      else {
        kk_std_core_hnd__clause1_dup(ctl_throw_exn, _ctx);
        kk_datatype_ptr_decref(h, _ctx);
      }
      {
        kk_function_t _fun_unbox_x19815 = ctl_throw_exn.clause;
        return kk_function_call(kk_box_t, (kk_function_t, kk_std_core_hnd__marker, kk_std_core_hnd__ev, kk_box_t, kk_context_t*), _fun_unbox_x19815, (_fun_unbox_x19815, m0, ev_18981, kk_std_core__exception_box(exn0, _ctx), _ctx), _ctx);
      }
    }
  }
}

kk_unit_t kk_std_core__default_exn(kk_function_t action, kk_context_t* _ctx); /* forall<e> (action : () -> <console,exn|e> ()) -> <console|e> () */ 

kk_std_core_types__maybe kk_std_core__unroll17334_op(kk_std_core__list xs, kk_integer_t n, kk_context_t* _ctx); /* forall<a> (xs : list<a>, n : int) -> maybe<a> */ 
 
// unrolling of singleton matches of std/core/.unroll17334-op

static inline kk_std_core_types__maybe kk_std_core__lp__lb__rb__2_rp_(kk_std_core__list xs, kk_integer_t n, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, n : int) -> maybe<a> */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    kk_integer_drop(n, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
  {
    return kk_std_core__unroll17334_op(xs, n, _ctx);
  }
}
 
// Raise an integer `i` to the power of `exp`.

static inline kk_integer_t kk_std_core__lp__hat__1_rp_(kk_integer_t i, kk_integer_t exp, kk_context_t* _ctx) { /* (i : int, exp : int) -> int */ 
  return kk_std_core_pow(i, exp, _ctx);
}

kk_std_core__sslice kk_std_core_after(kk_std_core__sslice slice0, kk_context_t* _ctx); /* (slice : sslice) -> sslice */ 

bool kk_std_core__mlift18815_unroll17335_all(kk_function_t predicate, kk_std_core__list xx, bool _y_18514, kk_context_t* _ctx); /* forall<a,e> (predicate : (a) -> e bool, xx : list<a>, bool) -> e bool */ 

bool kk_std_core__unroll17335_all(kk_std_core__list xs, kk_function_t predicate0, kk_context_t* _ctx); /* forall<a,e> (xs : list<a>, predicate : (a) -> e bool) -> e bool */ 
 
// unrolling of singleton matches of std/core/.unroll17335-all

static inline bool kk_std_core_all(kk_std_core__list xs, kk_function_t predicate, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, predicate : (a) -> e bool) -> e bool */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    kk_function_drop(predicate, _ctx);
    return true;
  }
  {
    return kk_std_core__unroll17335_all(xs, predicate, _ctx);
  }
}

bool kk_std_core__mlift18816_unroll17336_any(kk_function_t predicate, kk_std_core__list xx, bool _y_18520, kk_context_t* _ctx); /* forall<a,e> (predicate : (a) -> e bool, xx : list<a>, bool) -> e bool */ 

bool kk_std_core__unroll17336_any(kk_std_core__list xs, kk_function_t predicate0, kk_context_t* _ctx); /* forall<a,e> (xs : list<a>, predicate : (a) -> e bool) -> e bool */ 
 
// unrolling of singleton matches of std/core/.unroll17336-any

static inline bool kk_std_core_any(kk_std_core__list xs, kk_function_t predicate, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, predicate : (a) -> e bool) -> e bool */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    kk_function_drop(predicate, _ctx);
    return false;
  }
  {
    return kk_std_core__unroll17336_any(xs, predicate, _ctx);
  }
}

static inline kk_unit_t kk_std_core_assert(kk_string_t message0, bool condition, kk_context_t* _ctx) { /* (message : string, condition : bool) -> () */ 
  if (condition) {
    kk_string_drop(message0, _ctx);
    kk_Unit; return kk_Unit;
  }
  {
    kk_std_core_unsafe_assert_fail(message0, _ctx); return kk_Unit;
  }
}

kk_std_core_types__maybe kk_std_core_at(kk_vector_t v, kk_integer_t index, kk_context_t* _ctx); /* forall<a> (v : vector<a>, index : int) -> maybe<a> */ 
 
// Convert an int to a boolean, using `False` for 0 and `True` otherwise.

static inline bool kk_std_core_bool(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> bool */ 
  bool _brw_20856 = kk_integer_neq_borrow(i,(kk_integer_from_small(0)),kk_context()); /*bool*/;
  kk_integer_drop(i, _ctx);
  return _brw_20856;
}
 
// Convert a `:maybe` type to a boolean using `False` for `Nothing` and `True` for `Just`.

static inline bool kk_std_core_bool_1(kk_std_core_types__maybe m, kk_context_t* _ctx) { /* forall<a> (m : maybe<a>) -> bool */ 
  if (kk_std_core_types__is_Nothing(m, _ctx)) {
    return false;
  }
  {
    kk_std_core_types__maybe_drop(m, _ctx);
    return true;
  }
}
 
// Convert a string to a boolean, using `False` for the empty string and `True` otherwise.

static inline bool kk_std_core_bool_2(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> bool */ 
  kk_string_t _x21715 = kk_string_empty(); /*string*/
  return kk_string_is_neq(s,_x21715,kk_context());
}

kk_std_core__sslice kk_std_core_first(kk_string_t s, kk_std_core_types__optional n, kk_context_t* _ctx); /* (s : string, n : optional<int>) -> sslice */ 

kk_string_t kk_std_core_capitalize(kk_string_t s, kk_context_t* _ctx); /* (s : string) -> string */ 

kk_box_t kk_std_core_try(kk_function_t action, kk_function_t hndl, kk_context_t* _ctx); /* forall<a,e> (action : () -> <exn|e> a, hndl : (exception) -> e a) -> e a */ 
 
// monadic lift

static inline kk_std_core__error kk_std_core__mlift18817_try_1(kk_box_t _y_18527, kk_context_t* _ctx) { /* forall<a,e> (a) -> <exn|e> error<a> */ 
  return kk_std_core__new_Ok(_y_18527, _ctx);
}

kk_std_core__error kk_std_core_try_1(kk_function_t action, kk_context_t* _ctx); /* forall<a,e> (action : () -> <exn|e> a) -> e error<a> */ 
 
// _Deprecated_; use `try` instead. Catch an exception raised by `throw` and handle it.
// Use `on-exn` or `on-exit` when appropiate.

static inline kk_box_t kk_std_core_catch(kk_function_t action, kk_function_t hndl, kk_context_t* _ctx) { /* forall<a,e> (action : () -> <exn|e> a, hndl : (exception) -> e a) -> e a */ 
  return kk_std_core_try(action, hndl, _ctx);
}

kk_std_core_types__tuple2_ kk_std_core_cdivmod_exp10(kk_integer_t i, kk_integer_t n, kk_context_t* _ctx); /* (i : int, n : int) -> (int, int) */ 

kk_std_core__list kk_std_core__trmc_unroll17337_concat_maybe(kk_std_core__list xs, kk_std_core_types__cctx _acc, kk_context_t* _ctx); /* forall<a> (xs : list<maybe<a>>, ctx<list<a>>) -> list<a> */ 

kk_std_core__list kk_std_core__unroll17337_concat_maybe(kk_std_core__list xs0, kk_context_t* _ctx); /* forall<a> (xs : list<maybe<a>>) -> list<a> */ 
 
// unrolling of singleton matches of std/core/.unroll17337-concat-maybe

static inline kk_std_core__list kk_std_core_concat_maybe(kk_std_core__list xs, kk_context_t* _ctx) { /* forall<a> (xs : list<maybe<a>>) -> list<a> */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    return kk_std_core__new_Nil(_ctx);
  }
  {
    return kk_std_core__unroll17337_concat_maybe(xs, _ctx);
  }
}

kk_std_core_types__maybe kk_std_core__mlift18818_lift17320_for_whilez(kk_function_t action, kk_ssize_t end, kk_ssize_t i, kk_std_core_types__maybe _y_18530, kk_context_t* _ctx); /* forall<a,e> (action : (ssize_t) -> e maybe<a>, end : ssize_t, i : ssize_t, maybe<a>) -> e maybe<a> */ 

kk_std_core_types__maybe kk_std_core__lift17320_for_whilez(kk_function_t action0, kk_ssize_t end0, kk_ssize_t i0, kk_context_t* _ctx); /* forall<a,e> (action : (ssize_t) -> e maybe<a>, end : ssize_t, i : ssize_t) -> e maybe<a> */ 
 
// Executes `action`  for each integer between `start`  upto `end`  (including both `start`  and `end` ).
// If `start > end`  the function returns without any call to `action` .
// If `action` returns `Just`, the iteration is stopped and the result returned

static inline kk_std_core_types__maybe kk_std_core_for_whilez(kk_ssize_t start0, kk_ssize_t end, kk_function_t action, kk_context_t* _ctx) { /* forall<a,e> (start : ssize_t, end : ssize_t, action : (ssize_t) -> e maybe<a>) -> e maybe<a> */ 
  return kk_std_core__lift17320_for_whilez(action, end, start0, _ctx);
}
 
// O(1). The entire string as a slice

static inline kk_std_core__sslice kk_std_core_slice(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> sslice */ 
  kk_string_t _x21765 = kk_string_dup(s, _ctx); /*string*/
  kk_integer_t _x21766 = kk_string_len_int(s,kk_context()); /*int*/
  return kk_std_core__new_Sslice(_x21765, kk_integer_from_small(0), _x21766, _ctx);
}

kk_std_core_types__maybe kk_std_core__mlift18819_unroll17338_foreach_while(kk_function_t action, kk_std_core__list xx, kk_std_core_types__maybe _y_18535, kk_context_t* _ctx); /* forall<a,b,e> (action : (a) -> e maybe<b>, xx : list<a>, maybe<b>) -> e maybe<b> */ 

kk_std_core_types__maybe kk_std_core__unroll17338_foreach_while(kk_std_core__list xs, kk_function_t action0, kk_context_t* _ctx); /* forall<a,b,e> (xs : list<a>, action : (a) -> e maybe<b>) -> e maybe<b> */ 
 
// unrolling of singleton matches of std/core/.unroll17338-foreach-while

static inline kk_std_core_types__maybe kk_std_core_foreach_while(kk_std_core__list xs, kk_function_t action, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, action : (a) -> e maybe<b>) -> e maybe<b> */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    kk_function_drop(action, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
  {
    return kk_std_core__unroll17338_foreach_while(xs, action, _ctx);
  }
}

kk_std_core_types__maybe kk_std_core__mlift18820_foreach_while_1(kk_function_t action, kk_std_core__sslice rest, kk_std_core_types__maybe _y_18541, kk_context_t* _ctx); /* forall<a,e> (action : (c : char) -> e maybe<a>, rest : sslice, maybe<a>) -> e maybe<a> */ 

kk_std_core_types__maybe kk_std_core_foreach_while_1(kk_std_core__sslice slice0, kk_function_t action0, kk_context_t* _ctx); /* forall<a,e> (slice : sslice, action : (c : char) -> e maybe<a>) -> e maybe<a> */ 
 
// Invoke a function for each character in a string.
// If `action` returns `Just`, the function returns immediately with that result.

static inline kk_std_core_types__maybe kk_std_core_foreach_while_2(kk_string_t s, kk_function_t action, kk_context_t* _ctx) { /* forall<a,e> (s : string, action : (c : char) -> e maybe<a>) -> e maybe<a> */ 
  kk_std_core__sslice _x21776;
  kk_string_t _x21777 = kk_string_dup(s, _ctx); /*string*/
  kk_integer_t _x21778 = kk_string_len_int(s,kk_context()); /*int*/
  _x21776 = kk_std_core__new_Sslice(_x21777, kk_integer_from_small(0), _x21778, _ctx); /*sslice*/
  return kk_std_core_foreach_while_1(_x21776, action, _ctx);
}

kk_std_core_types__maybe kk_std_core__mlift18821_lift18309_foreach_while_3(kk_ssize_t end18139, kk_function_t f, kk_ssize_t i18140, kk_vector_t v, kk_std_core_types__maybe _y_18546, kk_context_t* _ctx); /* forall<a,b,e> (end18139 : ssize_t, f : (a) -> e maybe<b>, i18140 : ssize_t, v : vector<a>, maybe<b>) -> e maybe<b> */ 

kk_std_core_types__maybe kk_std_core__lift18309_foreach_while_3(kk_function_t f0, kk_vector_t v0, kk_ssize_t end181390, kk_ssize_t i181400, kk_context_t* _ctx); /* forall<a,b,e> (f : (a) -> e maybe<b>, v : vector<a>, end18139 : ssize_t, i18140 : ssize_t) -> e maybe<b> */ 
 
// Invoke a function `f` for each element in a vector `v`.
// If `f` returns `Just`, the iteration is stopped early and the result is returned.

static inline kk_std_core_types__maybe kk_std_core_foreach_while_3(kk_vector_t v, kk_function_t f, kk_context_t* _ctx) { /* forall<a,b,e> (v : vector<a>, f : (a) -> e maybe<b>) -> e maybe<b> */ 
  kk_ssize_t start0_17616 = (KK_IZ(0)); /*ssize_t*/;
  kk_ssize_t end_17617;
  kk_ssize_t _x21784 = kk_vector_len_borrow(v,kk_context()); /*ssize_t*/
  end_17617 = kk_std_core_decr_1(_x21784, _ctx); /*ssize_t*/
  return kk_std_core__lift18309_foreach_while_3(f, v, end_17617, start0_17616, _ctx);
}

kk_unit_t kk_std_core__mlift18822_unroll17339_foreach(kk_function_t action, kk_std_core__list xx, kk_unit_t wild__, kk_context_t* _ctx); /* forall<a,e> (action : (a) -> e (), xx : list<a>, wild_ : ()) -> e () */ 

kk_unit_t kk_std_core__unroll17339_foreach(kk_std_core__list xs, kk_function_t action0, kk_context_t* _ctx); /* forall<a,e> (xs : list<a>, action : (a) -> e ()) -> e () */ 
 
// unrolling of singleton matches of std/core/.unroll17339-foreach

static inline kk_unit_t kk_std_core_foreach(kk_std_core__list xs, kk_function_t action, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, action : (a) -> e ()) -> e () */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    kk_function_drop(action, _ctx);
    kk_Unit; return kk_Unit;
  }
  {
    kk_std_core__unroll17339_foreach(xs, action, _ctx); return kk_Unit;
  }
}

kk_std_core_types__maybe kk_std_core__mlift18823_lift18310_foreach_1(kk_function_t action, kk_std_core__sslice rest18155, kk_unit_t wild__, kk_context_t* _ctx); /* forall<_a,e> (action : (c : char) -> e (), rest18155 : sslice, wild_ : ()) -> e maybe<_a> */ 

kk_std_core_types__maybe kk_std_core__lift18310_foreach_1(kk_function_t action0, kk_std_core__sslice slice018149, kk_context_t* _ctx); /* forall<_a,e> (action : (c : char) -> e (), slice018149 : sslice) -> e maybe<_a> */ 
 
// monadic lift

static inline kk_unit_t kk_std_core__mlift18824_foreach_1(kk_std_core_types__maybe wild__0, kk_context_t* _ctx) { /* forall<_a,e> (wild_0 : maybe<_a>) -> e () */ 
  kk_std_core_types__maybe_drop(wild__0, _ctx);
  kk_Unit; return kk_Unit;
}

kk_unit_t kk_std_core_foreach_1(kk_std_core__sslice slice0, kk_function_t action, kk_context_t* _ctx); /* forall<e> (slice : sslice, action : (c : char) -> e ()) -> e () */ 

kk_std_core_types__maybe kk_std_core__mlift18825_lift18311_foreach_2(kk_function_t action, kk_std_core__sslice rest18169, kk_unit_t wild__, kk_context_t* _ctx); /* forall<_a,e> (action : (c : char) -> e (), rest18169 : sslice, wild_ : ()) -> e maybe<_a> */ 

kk_std_core_types__maybe kk_std_core__lift18311_foreach_2(kk_function_t action0, kk_std_core__sslice slice018163, kk_context_t* _ctx); /* forall<_a,e> (action : (c : char) -> e (), slice018163 : sslice) -> e maybe<_a> */ 
 
// monadic lift

static inline kk_unit_t kk_std_core__mlift18826_foreach_2(kk_std_core_types__maybe wild__0, kk_context_t* _ctx) { /* forall<_a,e> (wild_0 : maybe<_a>) -> e () */ 
  kk_std_core_types__maybe_drop(wild__0, _ctx);
  kk_Unit; return kk_Unit;
}

kk_unit_t kk_std_core_foreach_2(kk_string_t s, kk_function_t action, kk_context_t* _ctx); /* forall<e> (s : string, action : (c : char) -> e ()) -> e () */ 

kk_unit_t kk_std_core__mlift18827_lift18312_foreach_3(kk_ssize_t end18178, kk_function_t f, kk_ssize_t i18179, kk_vector_t v, kk_unit_t wild__18181, kk_context_t* _ctx); /* forall<a,e> (end18178 : ssize_t, f : (a) -> e (), i18179 : ssize_t, v : vector<a>, wild_18181 : ()) -> e () */ 

kk_unit_t kk_std_core__lift18312_foreach_3(kk_function_t f0, kk_vector_t v0, kk_ssize_t end181780, kk_ssize_t i181790, kk_context_t* _ctx); /* forall<a,e> (f : (a) -> e (), v : vector<a>, end18178 : ssize_t, i18179 : ssize_t) -> e () */ 
 
// Invoke a function `f` for each element in a vector `v`

static inline kk_unit_t kk_std_core_foreach_3(kk_vector_t v, kk_function_t f, kk_context_t* _ctx) { /* forall<a,e> (v : vector<a>, f : (a) -> e ()) -> e () */ 
  kk_ssize_t start0_17623 = (KK_IZ(0)); /*ssize_t*/;
  kk_ssize_t end_17624;
  kk_ssize_t _x21808 = kk_vector_len_borrow(v,kk_context()); /*ssize_t*/
  end_17624 = kk_std_core_decr_1(_x21808, _ctx); /*ssize_t*/
  kk_std_core__lift18312_foreach_3(f, v, end_17624, start0_17623, _ctx); return kk_Unit;
}

kk_integer_t kk_std_core_count_1(kk_string_t s, kk_context_t* _ctx); /* (s : string) -> int */ 

kk_integer_t kk_std_core_count_2(kk_std_core__sslice slice0, kk_context_t* _ctx); /* (slice : sslice) -> int */ 

kk_std_core_types__maybe kk_std_core__lift18313_count_3(kk_ref_t cnt, kk_function_t pred, kk_std_core__sslice slice018186, kk_context_t* _ctx); /* forall<_a,h> (cnt : local-var<h,int>, pred : (char) -> bool, slice018186 : sslice) -> (local<h>) maybe<_a> */ 

kk_integer_t kk_std_core_count_3(kk_string_t s, kk_function_t pred, kk_context_t* _ctx); /* (s : string, pred : (char) -> bool) -> int */ 

static inline kk_integer_t kk_std_core_dec(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> int */ 
  return kk_integer_add_small_const(i, -1, _ctx);
}
 
// Create a new `:delayed` value.

static inline kk_std_core__delayed kk_std_core_delay(kk_function_t action, kk_context_t* _ctx) { /* forall<a,e> (action : () -> e a) -> delayed<e,a> */ 
  kk_ref_t _x21818;
  kk_box_t _x21819;
  kk_std_core_types__either _x21820 = kk_std_core_types__new_Left(kk_function_box(action, _ctx), _ctx); /*either<76,77>*/
  _x21819 = kk_std_core_types__either_box(_x21820, _ctx); /*167*/
  _x21818 = kk_ref_alloc(_x21819,kk_context()); /*ref<168,167>*/
  return kk_std_core__new_Delay(_x21818, _ctx);
}
 
// Calculate `10^exp`

static inline kk_integer_t kk_std_core_exp10(kk_integer_t exp, kk_context_t* _ctx) { /* (exp : int) -> int */ 
  return kk_std_core_mul_exp10(kk_integer_from_small(1), exp, _ctx);
}
 
// Is the integer negative (stricly smaller than zero)

static inline bool kk_std_core_is_neg_2(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> bool */ 
  bool _brw_20816 = kk_integer_lt_borrow(i,(kk_integer_from_small(0)),kk_context()); /*bool*/;
  kk_integer_drop(i, _ctx);
  return _brw_20816;
}
 
// Is the value negative?

static inline bool kk_std_core_is_neg_3(double d, kk_context_t* _ctx) { /* (d : float64) -> bool */ 
  return (d < (0x0p+0));
}

kk_std_core_types__tuple2_ kk_std_core_divmod_exp10(kk_integer_t i, kk_integer_t n, kk_context_t* _ctx); /* (i : int, n : int) -> (int, int) */ 

kk_std_core__list kk_std_core_drop(kk_std_core__list xs, kk_integer_t n, kk_context_t* _ctx); /* forall<a> (xs : list<a>, n : int) -> list<a> */ 

kk_std_core__list kk_std_core__mlift18828_unroll17340_drop_while(kk_function_t predicate, kk_std_core__list xs, kk_std_core__list xx, bool _y_18570, kk_context_t* _ctx); /* forall<a,e> (predicate : (a) -> e bool, xs : list<a>, xx : list<a>, bool) -> e list<a> */ 

kk_std_core__list kk_std_core__unroll17340_drop_while(kk_std_core__list xs0, kk_function_t predicate0, kk_context_t* _ctx); /* forall<a,e> (xs : list<a>, predicate : (a) -> e bool) -> e list<a> */ 
 
// unrolling of singleton matches of std/core/.unroll17340-drop-while

static inline kk_std_core__list kk_std_core_drop_while(kk_std_core__list xs, kk_function_t predicate, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, predicate : (a) -> e bool) -> e list<a> */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    kk_function_drop(predicate, _ctx);
    return kk_std_core__new_Nil(_ctx);
  }
  {
    return kk_std_core__unroll17340_drop_while(xs, predicate, _ctx);
  }
}

kk_std_core_types__maybe kk_std_core_ends_with(kk_string_t s, kk_string_t post, kk_context_t* _ctx); /* (s : string, post : string) -> maybe<sslice> */ 

kk_box_t kk_std_core_throw(kk_string_t message0, kk_std_core_types__optional info0, kk_context_t* _ctx); /* forall<a> (message : string, info : optional<exception-info>) -> exn a */ 

kk_box_t kk_std_core__mlift18829_error_pattern(kk_string_t definition, kk_string_t location, kk_string_t _c_18577, kk_context_t* _ctx); /* forall<a> (definition : string, location : string, string) -> a */ 

kk_box_t kk_std_core_error_pattern(kk_string_t location, kk_string_t definition, kk_context_t* _ctx); /* forall<a> (location : string, definition : string) -> exn a */ 

kk_box_t kk_std_core_untry(kk_std_core__error err, kk_context_t* _ctx); /* forall<a> (err : error<a>) -> exn a */ 
 
// Transform an `:error` type back to an `exn` effect.

static inline kk_box_t kk_std_core_exn(kk_std_core__error err, kk_context_t* _ctx) { /* forall<a> (err : error<a>) -> exn a */ 
  return kk_std_core_untry(err, _ctx);
}
 
// Calculate `2^exp`.

static inline kk_integer_t kk_std_core_exp2(kk_integer_t exp, kk_context_t* _ctx) { /* (exp : int) -> int */ 
  return kk_std_core_pow(kk_integer_from_small(2), exp, _ctx);
}

kk_std_core__list kk_std_core__mlift18830_trmc_unroll17341_filter(kk_std_core_types__cctx _acc, kk_function_t pred, kk_box_t x, kk_std_core__list xx, bool _y_18582, kk_context_t* _ctx); /* forall<a,e> (ctx<list<a>>, pred : (a) -> e bool, x : a, xx : list<a>, bool) -> e list<a> */ 

kk_std_core__list kk_std_core__trmc_unroll17341_filter(kk_std_core__list xs, kk_function_t pred0, kk_std_core_types__cctx _acc0, kk_context_t* _ctx); /* forall<a,e> (xs : list<a>, pred : (a) -> e bool, ctx<list<a>>) -> e list<a> */ 

kk_std_core__list kk_std_core__unroll17341_filter(kk_std_core__list xs0, kk_function_t pred1, kk_context_t* _ctx); /* forall<a,e> (xs : list<a>, pred : (a) -> e bool) -> e list<a> */ 
 
// unrolling of singleton matches of std/core/.unroll17341-filter

static inline kk_std_core__list kk_std_core_filter(kk_std_core__list xs, kk_function_t pred, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, pred : (a) -> e bool) -> e list<a> */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    kk_function_drop(pred, _ctx);
    return kk_std_core__new_Nil(_ctx);
  }
  {
    return kk_std_core__unroll17341_filter(xs, pred, _ctx);
  }
}

kk_std_core__list kk_std_core__mlift18831_trmc_unroll17342_filter_map(kk_std_core_types__cctx _acc, kk_function_t pred, kk_std_core__list xx, kk_std_core_types__maybe _y_18590, kk_context_t* _ctx); /* forall<a,b,e> (ctx<list<b>>, pred : (a) -> e maybe<b>, xx : list<a>, maybe<b>) -> e list<b> */ 

kk_std_core__list kk_std_core__trmc_unroll17342_filter_map(kk_std_core__list xs, kk_function_t pred0, kk_std_core_types__cctx _acc0, kk_context_t* _ctx); /* forall<a,b,e> (xs : list<a>, pred : (a) -> e maybe<b>, ctx<list<b>>) -> e list<b> */ 

kk_std_core__list kk_std_core__unroll17342_filter_map(kk_std_core__list xs0, kk_function_t pred1, kk_context_t* _ctx); /* forall<a,b,e> (xs : list<a>, pred : (a) -> e maybe<b>) -> e list<b> */ 
 
// unrolling of singleton matches of std/core/.unroll17342-filter-map

static inline kk_std_core__list kk_std_core_filter_map(kk_std_core__list xs, kk_function_t pred, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, pred : (a) -> e maybe<b>) -> e list<b> */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    kk_function_drop(pred, _ctx);
    return kk_std_core__new_Nil(_ctx);
  }
  {
    return kk_std_core__unroll17342_filter_map(xs, pred, _ctx);
  }
}

bool kk_std_core_is_zero_1(kk_ssize_t i, kk_context_t* _ctx); /* (i : ssize_t) -> bool */ 
 
// Is the value zero?

static inline bool kk_std_core_is_zero_2(double d, kk_context_t* _ctx) { /* (d : float64) -> bool */ 
  return (d == (0x0p+0));
}

kk_std_core_types__maybe kk_std_core__lift18314_find(kk_function_t pred, kk_std_core__list xs18200, kk_context_t* _ctx); /* forall<a> (pred : (a) -> bool, xs18200 : list<a>) -> maybe<a> */ 
 
// Find the first element satisfying some predicate

static inline kk_std_core_types__maybe kk_std_core_find(kk_std_core__list xs, kk_function_t pred, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, pred : (a) -> bool) -> maybe<a> */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    kk_function_drop(pred, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
  {
    return kk_std_core__lift18314_find(pred, xs, _ctx);
  }
}

kk_std_core_types__maybe kk_std_core_find_1(kk_string_t s, kk_string_t sub, kk_context_t* _ctx); /* (s : string, sub : string) -> maybe<sslice> */ 

kk_std_core_types__maybe kk_std_core_find_last(kk_string_t s, kk_string_t sub, kk_context_t* _ctx); /* (s : string, sub : string) -> maybe<sslice> */ 
 
// Find the first element satisfying some predicate and return it.

static inline kk_std_core_types__maybe kk_std_core_find_maybe(kk_std_core__list xs, kk_function_t pred, kk_context_t* _ctx) { /* forall<a,b> (xs : list<a>, pred : (a) -> maybe<b>) -> maybe<b> */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    kk_function_drop(pred, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
  {
    return kk_std_core__unroll17338_foreach_while(xs, pred, _ctx);
  }
}

kk_std_core__list kk_std_core__mlift18832_trmc_unroll17343_flatmap_maybe(kk_std_core_types__cctx _acc, kk_function_t f, kk_std_core__list xx, kk_std_core_types__maybe _y_18598, kk_context_t* _ctx); /* forall<a,b,e> (ctx<list<b>>, f : (a) -> e maybe<b>, xx : list<a>, maybe<b>) -> e list<b> */ 

kk_std_core__list kk_std_core__trmc_unroll17343_flatmap_maybe(kk_std_core__list xs, kk_function_t f0, kk_std_core_types__cctx _acc0, kk_context_t* _ctx); /* forall<a,b,e> (xs : list<a>, f : (a) -> e maybe<b>, ctx<list<b>>) -> e list<b> */ 

kk_std_core__list kk_std_core__unroll17343_flatmap_maybe(kk_std_core__list xs0, kk_function_t f1, kk_context_t* _ctx); /* forall<a,b,e> (xs : list<a>, f : (a) -> e maybe<b>) -> e list<b> */ 
 
// unrolling of singleton matches of std/core/.unroll17343-flatmap-maybe

static inline kk_std_core__list kk_std_core_flatmap_maybe(kk_std_core__list xs, kk_function_t f, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, f : (a) -> e maybe<b>) -> e list<b> */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    kk_function_drop(f, _ctx);
    return kk_std_core__new_Nil(_ctx);
  }
  {
    return kk_std_core__unroll17343_flatmap_maybe(xs, f, _ctx);
  }
}

kk_box_t kk_std_core__mlift18833_fold_int(kk_integer_t end, kk_function_t f, kk_integer_t start0, kk_box_t x, kk_context_t* _ctx); /* forall<a,e> (end : int, f : (int, a) -> e a, start0 : int, x : a) -> e a */ 

kk_box_t kk_std_core_fold_int(kk_integer_t start00, kk_integer_t end0, kk_box_t init0, kk_function_t f0, kk_context_t* _ctx); /* forall<a,e> (start : int, end : int, init : a, f : (int, a) -> e a) -> e a */ 
 
// fold over the integers between [0,`upto`)  (not including `upto`).

static inline kk_box_t kk_std_core_fold_int_1(kk_integer_t upto, kk_box_t init0, kk_function_t f, kk_context_t* _ctx) { /* forall<a,e> (upto : int, init : a, f : (int, a) -> e a) -> e a */ 
  kk_integer_t _x21917 = kk_integer_add_small_const(upto, -1, _ctx); /*int*/
  return kk_std_core_fold_int(kk_integer_from_small(0), _x21917, init0, f, _ctx);
}

kk_box_t kk_std_core__mlift18834_fold_while_int(kk_integer_t end, kk_function_t f, kk_box_t init0, kk_integer_t start0, kk_std_core_types__maybe _y_18610, kk_context_t* _ctx); /* forall<a,e> (end : int, f : (int, a) -> e maybe<a>, init0 : a, start0 : int, maybe<a>) -> e a */ 

kk_box_t kk_std_core_fold_while_int(kk_integer_t start00, kk_integer_t end0, kk_box_t init00, kk_function_t f0, kk_context_t* _ctx); /* forall<a,e> (start : int, end : int, init : a, f : (int, a) -> e maybe<a>) -> e a */ 

kk_box_t kk_std_core__mlift18835_unroll17344_foldl(kk_function_t f, kk_std_core__list xx, kk_box_t _y_18614, kk_context_t* _ctx); /* forall<a,e,b> (f : (a, b) -> e a, xx : list<b>, a) -> e a */ 

kk_box_t kk_std_core__unroll17344_foldl(kk_std_core__list xs, kk_box_t z, kk_function_t f0, kk_context_t* _ctx); /* forall<a,b,e> (list<a>, b, (b, a) -> e b) -> e b */ 
 
// unrolling of singleton matches of std/core/.unroll17344-foldl

static inline kk_box_t kk_std_core_foldl(kk_std_core__list xs, kk_box_t z, kk_function_t f, kk_context_t* _ctx) { /* forall<a,b,e> (list<a>, b, (b, a) -> e b) -> e b */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    kk_function_drop(f, _ctx);
    return z;
  }
  {
    return kk_std_core__unroll17344_foldl(xs, z, f, _ctx);
  }
}

kk_box_t kk_std_core_foldl1(kk_std_core__list xs, kk_function_t f, kk_context_t* _ctx); /* forall<a,e> (xs : list<a>, f : (a, a) -> <exn|e> a) -> <exn|e> a */ 
 
// Reverse a list.

static inline kk_std_core__list kk_std_core_reverse(kk_std_core__list xs, kk_context_t* _ctx) { /* forall<a> (xs : list<a>) -> list<a> */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    return kk_std_core__new_Nil(_ctx);
  }
  {
    return kk_std_core__unroll17327_lift17313_reverse_append(kk_std_core__new_Nil(_ctx), xs, _ctx);
  }
}

kk_box_t kk_std_core__mlift18836_lift18315_foldr(kk_function_t f, kk_std_core__list xx18215, kk_box_t z182110_18351, kk_context_t* _ctx); /* forall<a,e,b> (f : (b, a) -> e a, xx18215 : list<b>, z182110.18351 : a) -> e a */ 

kk_box_t kk_std_core__lift18315_foldr(kk_function_t f0, kk_std_core__list xs18210, kk_box_t z18211, kk_context_t* _ctx); /* forall<a,e,b> (f : (b, a) -> e a, xs18210 : list<b>, z18211 : a) -> e a */ 
 
// Fold a list from the right, i.e. `foldr([1,2],0,(+)) == 1+(2+0)`
// Note, `foldr` is less efficient than `foldl` as it reverses the list first.

static inline kk_box_t kk_std_core_foldr(kk_std_core__list xs, kk_box_t z, kk_function_t f, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, z : b, f : (a, b) -> e b) -> e b */ 
  kk_std_core__list xs0_17669;
  if (kk_std_core__is_Nil(xs, _ctx)) {
    xs0_17669 = kk_std_core__new_Nil(_ctx); /*list<12774>*/
  }
  else {
    xs0_17669 = kk_std_core__unroll17327_lift17313_reverse_append(kk_std_core__new_Nil(_ctx), xs, _ctx); /*list<12774>*/
  }
  if (kk_std_core__is_Nil(xs0_17669, _ctx)) {
    kk_function_drop(f, _ctx);
    return z;
  }
  {
    return kk_std_core__lift18315_foldr(f, xs0_17669, z, _ctx);
  }
}

kk_box_t kk_std_core_foldr1(kk_std_core__list xs, kk_function_t f, kk_context_t* _ctx); /* forall<a,e> (xs : list<a>, f : (a, a) -> <exn|e> a) -> <exn|e> a */ 

kk_std_core_types__maybe kk_std_core__mlift18837_lift17321_for_while(kk_function_t action, kk_integer_t end, kk_integer_t i, kk_std_core_types__maybe _y_18634, kk_context_t* _ctx); /* forall<a,e> (action : (int) -> e maybe<a>, end : int, i : int, maybe<a>) -> e maybe<a> */ 

kk_std_core_types__maybe kk_std_core__lift17321_for_while(kk_function_t action0, kk_integer_t end0, kk_integer_t i0, kk_context_t* _ctx); /* forall<a,e> (action : (int) -> e maybe<a>, end : int, i : int) -> e maybe<a> */ 
 
// Executes `action`  for each integer between `start`  upto `end`  (including both `start`  and `end` ).
// If `start > end`  the function returns without any call to `action` .
// If `action` returns `Just`, the iteration is stopped and the result returned

static inline kk_std_core_types__maybe kk_std_core_for_while(kk_integer_t start0, kk_integer_t end, kk_function_t action, kk_context_t* _ctx) { /* forall<a,e> (start : int, end : int, action : (int) -> e maybe<a>) -> e maybe<a> */ 
  return kk_std_core__lift17321_for_while(action, end, start0, _ctx);
}

kk_unit_t kk_std_core__mlift18838_lift18316_foreach_indexed(kk_function_t action, kk_ref_t i, kk_std_core__list xx18224, kk_unit_t wild__18225, kk_context_t* _ctx); /* forall<h,a,e> (action : (int, a) -> e (), i : local-var<h,int>, xx18224 : list<a>, wild_18225 : ()) -> <local<h>|e> () */ 

kk_unit_t kk_std_core__mlift18839_lift18316_foreach_indexed(kk_function_t action0, kk_ref_t i0, kk_std_core__list xx182240, kk_integer_t _y_18642, kk_context_t* _ctx); /* forall<h,a,e> (action : (int, a) -> e (), i : local-var<h,int>, xx18224 : list<a>, int) -> <local<h>|e> () */ 

kk_unit_t kk_std_core__mlift18840_lift18316_foreach_indexed(kk_function_t action1, kk_ref_t i1, kk_std_core__list xx182241, kk_unit_t wild__, kk_context_t* _ctx); /* forall<h,a,e> (action : (int, a) -> e (), i : local-var<h,int>, xx18224 : list<a>, wild_ : ()) -> <local<h>|e> () */ 

kk_unit_t kk_std_core__mlift18841_lift18316_foreach_indexed(kk_function_t action2, kk_ref_t i2, kk_box_t x18223, kk_std_core__list xx182242, kk_integer_t j, kk_context_t* _ctx); /* forall<h,a,e> (action : (int, a) -> e (), i : local-var<h,int>, x18223 : a, xx18224 : list<a>, j : int) -> <local<h>|e> () */ 

kk_unit_t kk_std_core__lift18316_foreach_indexed(kk_function_t action4, kk_ref_t i3, kk_std_core__list xs18220, kk_context_t* _ctx); /* forall<h,a,e> (action : (int, a) -> e (), i : local-var<h,int>, xs18220 : list<a>) -> <local<h>|e> () */ 

kk_unit_t kk_std_core_foreach_indexed(kk_std_core__list xs, kk_function_t action, kk_context_t* _ctx); /* forall<a,e> (xs : list<a>, action : (int, a) -> e ()) -> e () */ 

kk_unit_t kk_std_core__mlift18842_lift18317_foreach_indexed_1(kk_ssize_t end18230, kk_function_t f, kk_ssize_t i18231, kk_vector_t v, kk_unit_t wild__18233, kk_context_t* _ctx); /* forall<a,e> (end18230 : ssize_t, f : (int, a) -> e (), i18231 : ssize_t, v : vector<a>, wild_18233 : ()) -> e () */ 

kk_unit_t kk_std_core__lift18317_foreach_indexed_1(kk_function_t f0, kk_vector_t v0, kk_ssize_t end182300, kk_ssize_t i182310, kk_context_t* _ctx); /* forall<a,e> (f : (int, a) -> e (), v : vector<a>, end18230 : ssize_t, i18231 : ssize_t) -> e () */ 
 
// Invoke a function `f` for each element in a vector `v`

static inline kk_unit_t kk_std_core_foreach_indexed_1(kk_vector_t v, kk_function_t f, kk_context_t* _ctx) { /* forall<a,e> (v : vector<a>, f : (int, a) -> e ()) -> e () */ 
  kk_ssize_t start0_17690 = (KK_IZ(0)); /*ssize_t*/;
  kk_ssize_t end_17691;
  kk_ssize_t _x21995 = kk_vector_len_borrow(v,kk_context()); /*ssize_t*/
  end_17691 = kk_std_core_decr_1(_x21995, _ctx); /*ssize_t*/
  kk_std_core__lift18317_foreach_indexed_1(f, v, end_17691, start0_17690, _ctx); return kk_Unit;
}
 
// Print a string to the console.

static inline kk_unit_t kk_std_core_print(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> console () */ 
  kk_std_core_prints(s, _ctx); return kk_Unit;
}
 
// Print an integer to the console.

static inline kk_unit_t kk_std_core_print_1(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> console () */ 
  kk_string_t _x21996 = kk_std_core_show(i, _ctx); /*string*/
  kk_std_core_prints(_x21996, _ctx); return kk_Unit;
}
 
// Print a float64 to the console.

static inline kk_unit_t kk_std_core_print_2(double d, kk_context_t* _ctx) { /* (d : float64) -> console () */ 
  kk_string_t _x21997 = kk_std_core_show_1(d, kk_std_core_types__new_None(_ctx), _ctx); /*string*/
  kk_std_core_prints(_x21997, _ctx); return kk_Unit;
}
 
// Print a boolean to the console

static inline kk_unit_t kk_std_core_print_3(bool b, kk_context_t* _ctx) { /* (b : bool) -> console () */ 
  kk_string_t _x21998;
  if (b) {
    kk_define_string_literal(, _s21999, 4, "True", _ctx)
    _x21998 = kk_string_dup(_s21999, _ctx); /*string*/
  }
  else {
    kk_define_string_literal(, _s22000, 5, "False", _ctx)
    _x21998 = kk_string_dup(_s22000, _ctx); /*string*/
  }
  kk_std_core_prints(_x21998, _ctx); return kk_Unit;
}
 
// Print a character to the console.

static inline kk_unit_t kk_std_core_print_4(kk_char_t c, kk_context_t* _ctx) { /* (c : char) -> console () */ 
  kk_string_t _x22001 = kk_std_core_string(c, _ctx); /*string*/
  kk_std_core_prints(_x22001, _ctx); return kk_Unit;
}
 
// Print a unit value to the console

static inline kk_unit_t kk_std_core_print_5(kk_unit_t u, kk_context_t* _ctx) { /* (u : ()) -> console () */ 
  kk_string_t _x22002;
  kk_define_string_literal(, _s22003, 2, "()", _ctx)
  _x22002 = kk_string_dup(_s22003, _ctx); /*string*/
  kk_std_core_prints(_x22002, _ctx); return kk_Unit;
}
 
// Generic print routine: prints the internal representation as a string to the console,
// including a final newline character.
// Note: this breaks parametricity so it should not be pub

static inline kk_unit_t kk_std_core_gprint(kk_box_t x, kk_context_t* _ctx) { /* forall<a> (x : a) -> console () */ 
  kk_string_t s_17695 = kk_std_core_gshow(x, _ctx); /*string*/;
  kk_std_core_prints(s_17695, _ctx); return kk_Unit;
}
 
// Generic print routine: prints the internal representation as a string to the console, including a final newline character.
// Note: this breaks parametricity so it should not be pub

static inline kk_unit_t kk_std_core_gprintln(kk_box_t x, kk_context_t* _ctx) { /* forall<a> (x : a) -> console () */ 
  kk_string_t s_17696 = kk_std_core_gshow(x, _ctx); /*string*/;
  kk_std_core_printsln(s_17696, _ctx); return kk_Unit;
}

kk_std_core_types__maybe kk_std_core_head_1(kk_std_core__list xs, kk_context_t* _ctx); /* forall<a> (xs : list<a>) -> maybe<a> */ 

kk_box_t kk_std_core_head_2(kk_std_core__list xs, kk_box_t default0, kk_context_t* _ctx); /* forall<a> (xs : list<a>, default : a) -> a */ 

kk_string_t kk_std_core_head_3(kk_string_t s, kk_context_t* _ctx); /* (s : string) -> string */ 

kk_std_core_types__maybe kk_std_core_head_char(kk_string_t s, kk_context_t* _ctx); /* (s : string) -> maybe<char> */ 

static inline int32_t kk_std_core_i32(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> int32 */ 
  return kk_std_core_int32(i, _ctx);
}

static inline int64_t kk_std_core_i64(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> int64 */ 
  return kk_std_core_int64(i, _ctx);
}

kk_integer_t kk_std_core__unroll17345_index_of_acc(kk_std_core__list xs, kk_function_t pred, kk_integer_t idx, kk_context_t* _ctx); /* forall<a> (xs : list<a>, pred : (a) -> bool, idx : int) -> int */ 
 
// unrolling of singleton matches of std/core/.unroll17345-index-of-acc

static inline kk_integer_t kk_std_core_index_of_acc(kk_std_core__list xs, kk_function_t pred, kk_integer_t idx, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, pred : (a) -> bool, idx : int) -> int */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    kk_function_drop(pred, _ctx);
    kk_integer_drop(idx, _ctx);
    return kk_integer_from_small(-1);
  }
  {
    return kk_std_core__unroll17345_index_of_acc(xs, pred, idx, _ctx);
  }
}
 
// Returns the index of the first element where `pred` holds, or `-1` if no such element exists.

static inline kk_integer_t kk_std_core_index_of(kk_std_core__list xs, kk_function_t pred, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, pred : (a) -> bool) -> int */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    kk_function_drop(pred, _ctx);
    return kk_integer_from_small(-1);
  }
  {
    return kk_std_core__unroll17345_index_of_acc(xs, pred, kk_integer_from_small(0), _ctx);
  }
}

kk_std_core__list kk_std_core__trmc_init(kk_std_core__list xs, kk_std_core_types__cctx _acc, kk_context_t* _ctx); /* forall<a> (xs : list<a>, ctx<list<a>>) -> list<a> */ 

kk_std_core__list kk_std_core_init(kk_std_core__list xs0, kk_context_t* _ctx); /* forall<a> (xs : list<a>) -> list<a> */ 
 
// Is the character a lower-case ASCII character ?

static inline bool kk_std_core_is_lower(kk_char_t c, kk_context_t* _ctx) { /* (c : char) -> bool */ 
  bool _match_20775 = (c >= ('a')); /*bool*/;
  if (_match_20775) {
    return (c <= ('z'));
  }
  {
    return false;
  }
}
 
// Is the character an upper-case ASCII character ?

static inline bool kk_std_core_is_upper(kk_char_t c, kk_context_t* _ctx) { /* (c : char) -> bool */ 
  bool _match_20774 = (c >= ('A')); /*bool*/;
  if (_match_20774) {
    return (c <= ('Z'));
  }
  {
    return false;
  }
}

bool kk_std_core_is_alpha(kk_char_t c, kk_context_t* _ctx); /* (c : char) -> bool */ 
 
// Is the character an ASCII digit ?

static inline bool kk_std_core_is_digit(kk_char_t c, kk_context_t* _ctx) { /* (c : char) -> bool */ 
  bool _match_20769 = (c >= ('0')); /*bool*/;
  if (_match_20769) {
    return (c <= ('9'));
  }
  {
    return false;
  }
}
 
// Is the character ASCII letter or digit?

static inline bool kk_std_core_is_alpha_num(kk_char_t c, kk_context_t* _ctx) { /* (c : char) -> bool */ 
  bool _match_20767 = kk_std_core_is_alpha(c, _ctx); /*bool*/;
  if (_match_20767) {
    return true;
  }
  {
    bool _match_20768 = (c >= ('0')); /*bool*/;
    if (_match_20768) {
      return (c <= ('9'));
    }
    {
      return false;
    }
  }
}
 
// Is the character an ASCII character, e.g. `c <= '\x7F'`  ?

static inline bool kk_std_core_is_ascii(kk_char_t c, kk_context_t* _ctx) { /* (c : char) -> bool */ 
  return (c <= 0x007F);
}
 
// Is the character an ASCII control character, e.g. `c < ' '`  ?

static inline bool kk_std_core_is_control(kk_char_t c, kk_context_t* _ctx) { /* (c : char) -> bool */ 
  return (c < (' '));
}

bool kk_std_core_is_hex_digit(kk_char_t c, kk_context_t* _ctx); /* (c : char) -> bool */ 
 
// Is a slice not empty?

static inline bool kk_std_core_is_notempty(kk_std_core__sslice slice0, kk_context_t* _ctx) { /* (slice : sslice) -> bool */ 
  kk_integer_t _brw_20755;
  {
    kk_integer_t _x = slice0.len;
    kk_integer_dup(_x, _ctx);
    kk_std_core__sslice_drop(slice0, _ctx);
    _brw_20755 = _x; /*int*/
  }
  bool _brw_20756 = kk_integer_gt_borrow(_brw_20755,(kk_integer_from_small(0)),kk_context()); /*bool*/;
  kk_integer_drop(_brw_20755, _ctx);
  return _brw_20756;
}
 
// Is a string not empty?

static inline bool kk_std_core_is_notempty_1(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> bool */ 
  kk_string_t _x22020 = kk_string_empty(); /*string*/
  return kk_string_is_neq(s,_x22020,kk_context());
}
 
// Is a slice invalid?

static inline bool kk_std_core_is_valid(kk_std_core__sslice slice0, kk_context_t* _ctx) { /* (slice : sslice) -> bool */ 
  kk_integer_t _brw_20753;
  {
    kk_integer_t _x = slice0.start;
    kk_integer_dup(_x, _ctx);
    kk_std_core__sslice_drop(slice0, _ctx);
    _brw_20753 = _x; /*int*/
  }
  bool _brw_20754 = kk_integer_gte_borrow(_brw_20753,(kk_integer_from_small(0)),kk_context()); /*bool*/;
  kk_integer_drop(_brw_20753, _ctx);
  return _brw_20754;
}

bool kk_std_core_is_white(kk_char_t c, kk_context_t* _ctx); /* (c : char) -> bool */ 

kk_string_t kk_std_core_join_end(kk_std_core__list xs, kk_string_t end, kk_context_t* _ctx); /* (xs : list<string>, end : string) -> string */ 

kk_std_core_types__maybe kk_std_core_last(kk_std_core__list xs, kk_context_t* _ctx); /* forall<a> (xs : list<a>) -> maybe<a> */ 

kk_box_t kk_std_core_last_1(kk_std_core__list xs, kk_box_t default0, kk_context_t* _ctx); /* forall<a> (xs : list<a>, default : a) -> a */ 

kk_std_core__sslice kk_std_core_last_2(kk_string_t s, kk_std_core_types__optional n, kk_context_t* _ctx); /* (s : string, n : optional<int>) -> sslice */ 

kk_std_core__list kk_std_core__trmc_take(kk_std_core__list xs, kk_integer_t n, kk_std_core_types__cctx _acc, kk_context_t* _ctx); /* forall<a> (xs : list<a>, n : int, ctx<list<a>>) -> list<a> */ 

kk_std_core__list kk_std_core_take(kk_std_core__list xs0, kk_integer_t n0, kk_context_t* _ctx); /* forall<a> (xs : list<a>, n : int) -> list<a> */ 

kk_std_core_types__tuple2_ kk_std_core_split(kk_std_core__list xs, kk_integer_t n, kk_context_t* _ctx); /* forall<a> (xs : list<a>, n : int) -> (list<a>, list<a>) */ 
 
// Split a string into parts that were delimited by `sep`. The delimeters are not included in the results.
// For example: `split("1,,2",",") == ["1","","2]`

static inline kk_std_core__list kk_std_core_split_1(kk_string_t s, kk_string_t sep, kk_context_t* _ctx) { /* (s : string, sep : string) -> list<string> */ 
  kk_vector_t v_17721 = kk_string_splitv(s,sep,kk_context()); /*vector<string>*/;
  return kk_std_core_vlist(v_17721, kk_std_core_types__new_None(_ctx), _ctx);
}
 
// Split a string into at most `n` parts that were delimited by a string `sep`. The delimeters are not included in the results (except for possibly the final part).
// For example: `split("1,2,3",",",2) == ["1","2,3"]`

static inline kk_std_core__list kk_std_core_split_2(kk_string_t s, kk_string_t sep, kk_integer_t n, kk_context_t* _ctx) { /* (s : string, sep : string, n : int) -> list<string> */ 
  kk_vector_t v_17722;
  kk_ssize_t _x22040;
  kk_integer_t _x22041 = kk_integer_dup(n, _ctx); /*int*/
  _x22040 = kk_std_core_ssize__t(_x22041, _ctx); /*ssize_t*/
  v_17722 = kk_string_splitv_atmost(s,sep,_x22040,kk_context()); /*vector<string>*/
  return kk_std_core_vlist(v_17722, kk_std_core_types__new_None(_ctx), _ctx);
}
 
// Split a string into a list of lines

static inline kk_std_core__list kk_std_core_lines(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> list<string> */ 
  kk_vector_t v_17725;
  kk_string_t _x22042;
  kk_define_string_literal(, _s22043, 1, "\n", _ctx)
  _x22042 = kk_string_dup(_s22043, _ctx); /*string*/
  v_17725 = kk_string_splitv(s,_x22042,kk_context()); /*vector<string>*/
  return kk_std_core_vlist(v_17725, kk_std_core_types__new_None(_ctx), _ctx);
}

kk_std_core_types__maybe kk_std_core__lift18318_lookup(kk_function_t pred, kk_std_core__list xs18238, kk_context_t* _ctx); /* forall<a,b> (pred : (a) -> bool, xs18238 : list<(a, b)>) -> maybe<b> */ 
 
// Lookup the first element satisfying some predicate

static inline kk_std_core_types__maybe kk_std_core_lookup(kk_std_core__list xs, kk_function_t pred, kk_context_t* _ctx) { /* forall<a,b> (xs : list<(a, b)>, pred : (a) -> bool) -> maybe<b> */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    kk_function_drop(pred, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
  {
    return kk_std_core__lift18318_lookup(pred, xs, _ctx);
  }
}

kk_std_core__list kk_std_core__mlift18843_trmc_unroll17346_lift17322_map_indexed(kk_std_core_types__cctx _acc, kk_function_t f, kk_integer_t i0_17731, kk_std_core__list yy, kk_box_t _trmc_18411, kk_context_t* _ctx); /* forall<a,b,e> (ctx<list<b>>, f : (idx : int, value : a) -> e b, i0.17731 : int, yy : list<a>, b) -> e list<b> */ 

kk_std_core__list kk_std_core__trmc_unroll17346_lift17322_map_indexed(kk_function_t f0, kk_std_core__list ys, kk_integer_t i, kk_std_core_types__cctx _acc0, kk_context_t* _ctx); /* forall<a,b,e> (f : (idx : int, value : a) -> e b, ys : list<a>, i : int, ctx<list<b>>) -> e list<b> */ 

kk_std_core__list kk_std_core__unroll17346_lift17322_map_indexed(kk_function_t f1, kk_std_core__list ys0, kk_integer_t i0, kk_context_t* _ctx); /* forall<a,b,e> (f : (idx : int, value : a) -> e b, ys : list<a>, i : int) -> e list<b> */ 
 
// unrolling of singleton matches of std/core/.unroll17346-lift17322-map-indexed

static inline kk_std_core__list kk_std_core__lift17322_map_indexed(kk_function_t f, kk_std_core__list ys, kk_integer_t i, kk_context_t* _ctx) { /* forall<a,b,e> (f : (idx : int, value : a) -> e b, ys : list<a>, i : int) -> e list<b> */ 
  if (kk_std_core__is_Nil(ys, _ctx)) {
    kk_integer_drop(i, _ctx);
    kk_function_drop(f, _ctx);
    return kk_std_core__new_Nil(_ctx);
  }
  {
    return kk_std_core__unroll17346_lift17322_map_indexed(f, ys, i, _ctx);
  }
}
 
// Apply a function `f`  to each element of the input list in sequence where takes
// both the index of the current element and the element itself as arguments.

static inline kk_std_core__list kk_std_core_map_indexed(kk_std_core__list xs, kk_function_t f, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, f : (idx : int, value : a) -> e b) -> e list<b> */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    kk_function_drop(f, _ctx);
    return kk_std_core__new_Nil(_ctx);
  }
  {
    return kk_std_core__unroll17346_lift17322_map_indexed(f, xs, kk_integer_from_small(0), _ctx);
  }
}

kk_std_core__list kk_std_core__mlift18844_trmc_unroll17347_lift17323_map_indexed_peek(kk_std_core_types__cctx _acc, kk_function_t f, kk_integer_t i0_17736, kk_std_core__list yy, kk_box_t _trmc_18414, kk_context_t* _ctx); /* forall<a,b,e> (ctx<list<b>>, f : (idx : int, value : a, rest : list<a>) -> e b, i0.17736 : int, yy : list<a>, b) -> e list<b> */ 

kk_std_core__list kk_std_core__trmc_unroll17347_lift17323_map_indexed_peek(kk_function_t f0, kk_std_core__list ys, kk_integer_t i, kk_std_core_types__cctx _acc0, kk_context_t* _ctx); /* forall<a,b,e> (f : (idx : int, value : a, rest : list<a>) -> e b, ys : list<a>, i : int, ctx<list<b>>) -> e list<b> */ 

kk_std_core__list kk_std_core__unroll17347_lift17323_map_indexed_peek(kk_function_t f1, kk_std_core__list ys0, kk_integer_t i0, kk_context_t* _ctx); /* forall<a,b,e> (f : (idx : int, value : a, rest : list<a>) -> e b, ys : list<a>, i : int) -> e list<b> */ 
 
// unrolling of singleton matches of std/core/.unroll17347-lift17323-map-indexed-peek

static inline kk_std_core__list kk_std_core__lift17323_map_indexed_peek(kk_function_t f, kk_std_core__list ys, kk_integer_t i, kk_context_t* _ctx) { /* forall<a,b,e> (f : (idx : int, value : a, rest : list<a>) -> e b, ys : list<a>, i : int) -> e list<b> */ 
  if (kk_std_core__is_Nil(ys, _ctx)) {
    kk_integer_drop(i, _ctx);
    kk_function_drop(f, _ctx);
    return kk_std_core__new_Nil(_ctx);
  }
  {
    return kk_std_core__unroll17347_lift17323_map_indexed_peek(f, ys, i, _ctx);
  }
}
 
// Apply a function `f`  to each element of the input list in sequence where takes
// both the index of the current element, the element itself, and the tail list as arguments.

static inline kk_std_core__list kk_std_core_map_indexed_peek(kk_std_core__list xs, kk_function_t f, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, f : (idx : int, value : a, rest : list<a>) -> e b) -> e list<b> */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    kk_function_drop(f, _ctx);
    return kk_std_core__new_Nil(_ctx);
  }
  {
    return kk_std_core__unroll17347_lift17323_map_indexed_peek(f, xs, kk_integer_from_small(0), _ctx);
  }
}

kk_std_core__list kk_std_core__mlift18845_trmc_unroll17348_map_while(kk_std_core_types__cctx _acc, kk_function_t action, kk_std_core__list xx, kk_std_core_types__maybe _y_18670, kk_context_t* _ctx); /* forall<a,b,e> (ctx<list<b>>, action : (a) -> e maybe<b>, xx : list<a>, maybe<b>) -> e list<b> */ 

kk_std_core__list kk_std_core__trmc_unroll17348_map_while(kk_std_core__list xs, kk_function_t action0, kk_std_core_types__cctx _acc0, kk_context_t* _ctx); /* forall<a,b,e> (xs : list<a>, action : (a) -> e maybe<b>, ctx<list<b>>) -> e list<b> */ 

kk_std_core__list kk_std_core__unroll17348_map_while(kk_std_core__list xs0, kk_function_t action1, kk_context_t* _ctx); /* forall<a,b,e> (xs : list<a>, action : (a) -> e maybe<b>) -> e list<b> */ 
 
// unrolling of singleton matches of std/core/.unroll17348-map-while

static inline kk_std_core__list kk_std_core_map_while(kk_std_core__list xs, kk_function_t action, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, action : (a) -> e maybe<b>) -> e list<b> */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    kk_function_drop(action, _ctx);
    return kk_std_core__new_Nil(_ctx);
  }
  {
    return kk_std_core__unroll17348_map_while(xs, action, _ctx);
  }
}
 
// Return the maximum of two integers

static inline kk_integer_t kk_std_core_max(kk_integer_t i, kk_integer_t j, kk_context_t* _ctx) { /* (i : int, j : int) -> int */ 
  bool _match_20737 = kk_integer_gte_borrow(i,j,kk_context()); /*bool*/;
  if (_match_20737) {
    kk_integer_drop(j, _ctx);
    return i;
  }
  {
    kk_integer_drop(i, _ctx);
    return j;
  }
}
 
// Returns the largest of two floats

static inline double kk_std_core_max_1(double x, double y, kk_context_t* _ctx) { /* (x : float64, y : float64) -> float64 */ 
  bool _match_20736 = (x >= y); /*bool*/;
  if (_match_20736) {
    return x;
  }
  {
    return y;
  }
}

kk_integer_t kk_std_core__lift18319_maximum(kk_std_core__list xs18248, kk_integer_t z18249, kk_context_t* _ctx); /* (xs18248 : list<int>, z18249 : int) -> int */ 

kk_integer_t kk_std_core_maximum(kk_std_core__list xs, kk_std_core_types__optional default0, kk_context_t* _ctx); /* (xs : list<int>, default : optional<int>) -> int */ 

double kk_std_core__lift18320_maximum_1(kk_std_core__list xs18256, double z18257, kk_context_t* _ctx); /* (xs18256 : list<float64>, z18257 : float64) -> float64 */ 

double kk_std_core_maximum_1(kk_std_core__list xs, kk_context_t* _ctx); /* (xs : list<float64>) -> float64 */ 
 
// Match a `:maybe` value and either return a default value on `Nothing` or apply a function to the value on `Just`

static inline kk_box_t kk_std_core_maybe(kk_std_core_types__maybe m, kk_box_t onNothing, kk_function_t onJust, kk_context_t* _ctx) { /* forall<a,b,e> (m : maybe<a>, onNothing : b, onJust : (a) -> e b) -> e b */ 
  if (kk_std_core_types__is_Nothing(m, _ctx)) {
    kk_function_drop(onJust, _ctx);
    return onNothing;
  }
  {
    kk_box_t x = m._cons.Just.value;
    kk_box_drop(onNothing, _ctx);
    return kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), onJust, (onJust, x, _ctx), _ctx);
  }
}
 
// Convert a `:maybe<a>` value to `:a`, using the `nothing` parameter for `Nothing`.
// This is an alias for `default`.

static inline kk_box_t kk_std_core_maybe_1(kk_std_core_types__maybe m, kk_box_t nothing, kk_context_t* _ctx) { /* forall<a> (m : maybe<a>, nothing : a) -> a */ 
  if (kk_std_core_types__is_Nothing(m, _ctx)) {
    return nothing;
  }
  {
    kk_box_t x = m._cons.Just.value;
    kk_box_drop(nothing, _ctx);
    return x;
  }
}
 
// Convert a `:either` to a `:maybe` type discarding the value of the `Left` constructor
// and using `Just` for the `Right` constructor.

static inline kk_std_core_types__maybe kk_std_core_maybe_2(kk_std_core_types__either e, kk_context_t* _ctx) { /* forall<a,b> (e : either<a,b>) -> maybe<b> */ 
  if (kk_std_core_types__is_Left(e, _ctx)) {
    kk_std_core_types__either_drop(e, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
  {
    kk_box_t x = e._cons.Right.right;
    return kk_std_core_types__new_Just(x, _ctx);
  }
}

kk_std_core_types__maybe kk_std_core_maybe_3(kk_std_core__list xs, kk_context_t* _ctx); /* forall<a> (xs : list<a>) -> maybe<a> */ 
 
// Transform a boolean to a maybe type, using `Nothing` for `False`

static inline kk_std_core_types__maybe kk_std_core_maybe_4(bool b, kk_context_t* _ctx) { /* (b : bool) -> maybe<()> */ 
  if (b) {
    return kk_std_core_types__new_Just(kk_unit_box(kk_Unit), _ctx);
  }
  {
    return kk_std_core_types__new_Nothing(_ctx);
  }
}
 
// Transform an integer to a maybe type, using `Nothing` for `0`

static inline kk_std_core_types__maybe kk_std_core_maybe_5(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> maybe<int> */ 
  bool _match_20733 = kk_integer_eq_borrow(i,(kk_integer_from_small(0)),kk_context()); /*bool*/;
  if (_match_20733) {
    kk_integer_drop(i, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
  {
    return kk_std_core_types__new_Just(kk_integer_box(i, _ctx), _ctx);
  }
}

kk_std_core_types__maybe kk_std_core_maybe_6(kk_string_t s, kk_context_t* _ctx); /* (s : string) -> maybe<string> */ 
 
// Transform an `:error` type to a `:maybe` value.

static inline kk_std_core_types__maybe kk_std_core_maybe_7(kk_std_core__error t, kk_context_t* _ctx) { /* forall<a> (t : error<a>) -> maybe<a> */ 
  if (kk_std_core__is_Error(t, _ctx)) {
    kk_std_core__exception _pat0 = t._cons.Error.exception;
    kk_std_core__error_drop(t, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
  {
    kk_box_t x = t._cons.Ok.result;
    return kk_std_core_types__new_Just(x, _ctx);
  }
}

kk_std_core_types__maybe kk_std_core_maybe_8(kk_std_core__null n, kk_context_t* _ctx); /* forall<a> (n : null<a>) -> maybe<a> */ 
 
// Return the minimum of two integers

static inline kk_integer_t kk_std_core_min(kk_integer_t i, kk_integer_t j, kk_context_t* _ctx) { /* (i : int, j : int) -> int */ 
  bool _match_20731 = kk_integer_lte_borrow(i,j,kk_context()); /*bool*/;
  if (_match_20731) {
    kk_integer_drop(j, _ctx);
    return i;
  }
  {
    kk_integer_drop(i, _ctx);
    return j;
  }
}
 
// Returns the smallest of two floats

static inline double kk_std_core_min_1(double x, double y, kk_context_t* _ctx) { /* (x : float64, y : float64) -> float64 */ 
  bool _match_20730 = (x <= y); /*bool*/;
  if (_match_20730) {
    return x;
  }
  {
    return y;
  }
}

kk_integer_t kk_std_core__lift18321_minimum(kk_std_core__list xs18264, kk_integer_t z18265, kk_context_t* _ctx); /* (xs18264 : list<int>, z18265 : int) -> int */ 

kk_integer_t kk_std_core_minimum(kk_std_core__list xs, kk_std_core_types__optional default0, kk_context_t* _ctx); /* (xs : list<int>, default : optional<int>) -> int */ 

double kk_std_core__lift18322_minimum_1(kk_std_core__list xs18272, double z18273, kk_context_t* _ctx); /* (xs18272 : list<float64>, z18273 : float64) -> float64 */ 

double kk_std_core_minimum_1(kk_std_core__list xs, kk_context_t* _ctx); /* (xs : list<float64>) -> float64 */ 

kk_unit_t kk_std_core_notrace(kk_context_t* _ctx); /* () -> (st<global>) () */ 

kk_std_core__null kk_std_core_null(kk_std_core_types__maybe x, kk_context_t* _ctx); /* forall<a> (x : maybe<a>) -> null<a> */ 

kk_std_core__null kk_std_core_null_1(kk_integer_t i, kk_context_t* _ctx); /* (i : int) -> null<int> */ 

kk_std_core__null kk_std_core_null_2(kk_string_t s, kk_context_t* _ctx); /* (s : string) -> null<string> */ 
 
// Cast a boolean `False` to null

static inline kk_std_core__null kk_std_core_null_3(bool b, kk_context_t* _ctx) { /* (b : bool) -> null<()> */ 
  kk_std_core_types__maybe _x22101;
  if (b) {
    _x22101 = kk_std_core_types__new_Just(kk_unit_box(kk_Unit), _ctx); /*maybe<111>*/
  }
  else {
    _x22101 = kk_std_core_types__new_Nothing(_ctx); /*maybe<111>*/
  }
  return kk_std_core_null(_x22101, _ctx);
}

kk_string_t kk_std_core_pad_right(kk_string_t s, kk_integer_t width, kk_std_core_types__optional fill, kk_context_t* _ctx); /* (s : string, width : int, fill : optional<char>) -> string */ 

kk_std_core_types__maybe kk_std_core_starts_with(kk_string_t s, kk_string_t pre, kk_context_t* _ctx); /* (s : string, pre : string) -> maybe<sslice> */ 

kk_string_t kk_std_core_trim_left_1(kk_string_t s, kk_string_t sub, kk_context_t* _ctx); /* (s : string, sub : string) -> string */ 

kk_string_t kk_std_core_trim_right_1(kk_string_t s, kk_string_t sub, kk_context_t* _ctx); /* (s : string, sub : string) -> string */ 
 
// Trim whitespace on the left and right side of a string

static inline kk_string_t kk_std_core_trim(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> string */ 
  kk_string_t _x22125 = kk_string_trim_left(s,kk_context()); /*string*/
  return kk_string_trim_right(_x22125,kk_context());
}

kk_std_core_types__maybe kk_std_core_parse_int(kk_string_t s, kk_std_core_types__optional hex, kk_context_t* _ctx); /* (s : string, hex : optional<bool>) -> maybe<int> */ 

kk_integer_t kk_std_core_parse_int_default(kk_string_t s, kk_std_core_types__optional default0, kk_std_core_types__optional hex, kk_context_t* _ctx); /* (s : string, default : optional<int>, hex : optional<bool>) -> int */ 

kk_std_core_types__tuple2_ kk_std_core__mlift18846_unroll17349_partition_acc(kk_std_core__list acc1, kk_std_core__list acc2, kk_function_t pred, kk_box_t x, kk_std_core__list xx, bool _y_18679, kk_context_t* _ctx); /* forall<a,e> (acc1 : list<a>, acc2 : list<a>, pred : (a) -> e bool, x : a, xx : list<a>, bool) -> e (list<a>, list<a>) */ 

kk_std_core_types__tuple2_ kk_std_core__unroll17349_partition_acc(kk_std_core__list xs, kk_function_t pred0, kk_std_core__list acc10, kk_std_core__list acc20, kk_context_t* _ctx); /* forall<a,e> (xs : list<a>, pred : (a) -> e bool, acc1 : list<a>, acc2 : list<a>) -> e (list<a>, list<a>) */ 
 
// unrolling of singleton matches of std/core/.unroll17349-partition-acc

static inline kk_std_core_types__tuple2_ kk_std_core_partition_acc(kk_std_core__list xs, kk_function_t pred, kk_std_core__list acc1, kk_std_core__list acc2, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, pred : (a) -> e bool, acc1 : list<a>, acc2 : list<a>) -> e (list<a>, list<a>) */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    kk_function_drop(pred, _ctx);
    kk_std_core__list _b_20407_20405;
    if (kk_std_core__is_Nil(acc1, _ctx)) {
      _b_20407_20405 = kk_std_core__new_Nil(_ctx); /*list<15996>*/
    }
    else {
      _b_20407_20405 = kk_std_core__unroll17327_lift17313_reverse_append(kk_std_core__new_Nil(_ctx), acc1, _ctx); /*list<15996>*/
    }
    kk_std_core__list _b_20408_20406;
    if (kk_std_core__is_Nil(acc2, _ctx)) {
      _b_20408_20406 = kk_std_core__new_Nil(_ctx); /*list<15996>*/
    }
    else {
      _b_20408_20406 = kk_std_core__unroll17327_lift17313_reverse_append(kk_std_core__new_Nil(_ctx), acc2, _ctx); /*list<15996>*/
    }
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_core__list_box(_b_20407_20405, _ctx), kk_std_core__list_box(_b_20408_20406, _ctx), _ctx);
  }
  {
    return kk_std_core__unroll17349_partition_acc(xs, pred, acc1, acc2, _ctx);
  }
}

kk_std_core_types__tuple2_ kk_std_core_partition(kk_std_core__list xs, kk_function_t pred, kk_context_t* _ctx); /* forall<a,e> (xs : list<a>, pred : (a) -> e bool) -> e (list<a>, list<a>) */ 

kk_unit_t kk_std_core_print_redirect(kk_function_t print0, kk_context_t* _ctx); /* (print : (msg : string) -> console ()) -> io () */ 

kk_std_core__list kk_std_core__trmc_lift18323_remove(kk_function_t pred, kk_std_core__list xs18280, kk_std_core_types__cctx _acc, kk_context_t* _ctx); /* forall<a> (pred : (a) -> bool, xs18280 : list<a>, ctx<list<a>>) -> list<a> */ 

kk_std_core__list kk_std_core__lift18323_remove(kk_function_t pred0, kk_std_core__list xs182800, kk_context_t* _ctx); /* forall<a> (pred : (a) -> bool, xs18280 : list<a>) -> list<a> */ 
 
// Remove those elements of a list that satisfy the given predicate `pred`.
// For example: `remove([1,2,3],odd?) == [2]`

static inline kk_std_core__list kk_std_core_remove(kk_std_core__list xs, kk_function_t pred, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, pred : (a) -> bool) -> list<a> */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    kk_function_drop(pred, _ctx);
    return kk_std_core__new_Nil(_ctx);
  }
  {
    return kk_std_core__lift18323_remove(pred, xs, _ctx);
  }
}

kk_std_core__list kk_std_core__trmc_replicate(kk_box_t x, kk_integer_t n, kk_std_core_types__cctx _acc, kk_context_t* _ctx); /* forall<a> (x : a, n : int, ctx<list<a>>) -> list<a> */ 

kk_std_core__list kk_std_core_replicate(kk_box_t x0, kk_integer_t n0, kk_context_t* _ctx); /* forall<a> (x : a, n : int) -> list<a> */ 

kk_string_t kk_std_core_reverse_join(kk_std_core__list xs, kk_context_t* _ctx); /* (xs : list<string>) -> string */ 

kk_string_t kk_std_core_show_tuple(kk_std_core_types__tuple2_ x, kk_function_t showfst, kk_function_t showsnd, kk_context_t* _ctx); /* forall<a,b> (x : (a, b), showfst : (a) -> string, showsnd : (b) -> string) -> string */ 

static inline kk_std_core_types__order kk_std_core_sign_1(double d, kk_context_t* _ctx) { /* (d : float64) -> order */ 
  bool _match_20715 = (d < (0x0p+0)); /*bool*/;
  if (_match_20715) {
    return kk_std_core_types__new_Lt(_ctx);
  }
  {
    bool _match_20716 = (d > (0x0p+0)); /*bool*/;
    if (_match_20716) {
      return kk_std_core_types__new_Gt(_ctx);
    }
    {
      return kk_std_core_types__new_Eq(_ctx);
    }
  }
}

kk_std_core_types__tuple2_ kk_std_core__mlift18847_unroll17350_lift17324_span(kk_std_core__list acc, kk_function_t predicate, kk_box_t y, kk_std_core__list ys, kk_std_core__list yy, bool _y_18688, kk_context_t* _ctx); /* forall<a,e> (acc : list<a>, predicate : (a) -> e bool, y : a, ys : list<a>, yy : list<a>, bool) -> e (list<a>, list<a>) */ 

kk_std_core_types__tuple2_ kk_std_core__unroll17350_lift17324_span(kk_function_t predicate0, kk_std_core__list ys0, kk_std_core__list acc0, kk_context_t* _ctx); /* forall<a,e> (predicate : (a) -> e bool, ys : list<a>, acc : list<a>) -> e (list<a>, list<a>) */ 
 
// unrolling of singleton matches of std/core/.unroll17350-lift17324-span

static inline kk_std_core_types__tuple2_ kk_std_core__lift17324_span(kk_function_t predicate, kk_std_core__list ys, kk_std_core__list acc, kk_context_t* _ctx) { /* forall<a,e> (predicate : (a) -> e bool, ys : list<a>, acc : list<a>) -> e (list<a>, list<a>) */ 
  if (kk_std_core__is_Nil(ys, _ctx)) {
    kk_function_drop(predicate, _ctx);
    kk_std_core__list _b_20473_20471;
    if (kk_std_core__is_Nil(acc, _ctx)) {
      _b_20473_20471 = kk_std_core__new_Nil(_ctx); /*list<16562>*/
    }
    else {
      _b_20473_20471 = kk_std_core__unroll17327_lift17313_reverse_append(kk_std_core__new_Nil(_ctx), acc, _ctx); /*list<16562>*/
    }
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_core__list_box(_b_20473_20471, _ctx), kk_std_core__list_box(ys, _ctx), _ctx);
  }
  {
    return kk_std_core__unroll17350_lift17324_span(predicate, ys, acc, _ctx);
  }
}

kk_std_core_types__tuple2_ kk_std_core_span(kk_std_core__list xs, kk_function_t predicate, kk_context_t* _ctx); /* forall<a,e> (xs : list<a>, predicate : (a) -> e bool) -> e (list<a>, list<a>) */ 

kk_integer_t kk_std_core__lift18324_sum(kk_std_core__list xs18290, kk_integer_t z18291, kk_context_t* _ctx); /* (xs18290 : list<int>, z18291 : int) -> int */ 
 
// Return the sum of a list of integers

static inline kk_integer_t kk_std_core_sum(kk_std_core__list xs, kk_context_t* _ctx) { /* (xs : list<int>) -> int */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    return kk_integer_from_small(0);
  }
  {
    return kk_std_core__lift18324_sum(xs, kk_integer_from_small(0), _ctx);
  }
}

kk_std_core__list kk_std_core_tail_1(kk_std_core__list xs, kk_context_t* _ctx); /* forall<a> (xs : list<a>) -> list<a> */ 

kk_string_t kk_std_core_tail_2(kk_string_t s, kk_context_t* _ctx); /* (s : string) -> string */ 

kk_std_core__list kk_std_core__mlift18848_trmc_unroll17351_take_while(kk_std_core_types__cctx _acc, kk_function_t predicate, kk_box_t x, kk_std_core__list xx, bool _y_18696, kk_context_t* _ctx); /* forall<a,e> (ctx<list<a>>, predicate : (a) -> e bool, x : a, xx : list<a>, bool) -> e list<a> */ 

kk_std_core__list kk_std_core__trmc_unroll17351_take_while(kk_std_core__list xs, kk_function_t predicate0, kk_std_core_types__cctx _acc0, kk_context_t* _ctx); /* forall<a,e> (xs : list<a>, predicate : (a) -> e bool, ctx<list<a>>) -> e list<a> */ 

kk_std_core__list kk_std_core__unroll17351_take_while(kk_std_core__list xs0, kk_function_t predicate1, kk_context_t* _ctx); /* forall<a,e> (xs : list<a>, predicate : (a) -> e bool) -> e list<a> */ 
 
// unrolling of singleton matches of std/core/.unroll17351-take-while

static inline kk_std_core__list kk_std_core_take_while(kk_std_core__list xs, kk_function_t predicate, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, predicate : (a) -> e bool) -> e list<a> */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    kk_function_drop(predicate, _ctx);
    return kk_std_core__new_Nil(_ctx);
  }
  {
    return kk_std_core__unroll17351_take_while(xs, predicate, _ctx);
  }
}
 
// monadic lift

static inline kk_unit_t kk_std_core__mlift18849_trace(kk_string_t message0, bool _y_18703, kk_context_t* _ctx) { /* forall<_e> (message0 : string, bool) -> <read<global>|_e> () */ 
  if (_y_18703) {
    kk_std_core_xtrace(message0, _ctx); return kk_Unit;
  }
  {
    kk_string_drop(message0, _ctx);
    kk_Unit; return kk_Unit;
  }
}

kk_unit_t kk_std_core_trace(kk_string_t message0, kk_context_t* _ctx); /* (message : string) -> () */ 
 
// monadic lift

static inline kk_unit_t kk_std_core__mlift18850_trace_any(kk_string_t message0, kk_box_t x, bool _y_18705, kk_context_t* _ctx) { /* forall<_e,a> (message0 : string, x : a, bool) -> <read<global>|_e> () */ 
  if (_y_18705) {
    kk_std_core_xtrace_any(message0, x, _ctx); return kk_Unit;
  }
  {
    kk_box_drop(x, _ctx);
    kk_string_drop(message0, _ctx);
    kk_Unit; return kk_Unit;
  }
}

kk_unit_t kk_std_core_trace_any(kk_string_t message0, kk_box_t x, kk_context_t* _ctx); /* forall<a> (message : string, x : a) -> () */ 

kk_string_t kk_std_core_truncate(kk_string_t s, kk_integer_t count, kk_context_t* _ctx); /* (s : string, count : int) -> string */ 

kk_box_t kk_std_core_try_default(kk_box_t value, kk_function_t action, kk_context_t* _ctx); /* forall<a,e> (value : a, action : () -> <exn|e> a) -> e a */ 
 
// monadic lift

static inline kk_integer_t kk_std_core__mlift18851_unique(kk_integer_t u, kk_context_t* _ctx) { /* forall<_e> (u : int) -> <read<global>,write<global>|_e> int */ 
  kk_integer_t _b_20546_20544;
  kk_integer_t _x22227 = kk_integer_dup(u, _ctx); /*int*/
  _b_20546_20544 = kk_integer_add_small_const(_x22227, 1, _ctx); /*int*/
  kk_unit_t __ = kk_Unit;
  kk_ref_set_borrow(kk_std_core_unique_count,(kk_integer_box(_b_20546_20544, _ctx)),kk_context());
  return u;
}

kk_integer_t kk_std_core_unique(kk_context_t* _ctx); /* () -> ndet int */ 

kk_box_t kk_std_core_unjust(kk_std_core_types__maybe m, kk_context_t* _ctx); /* forall<a> (m : maybe<a>) -> exn a */ 

kk_string_t kk_std_core_unlines(kk_std_core__list xs, kk_context_t* _ctx); /* (xs : list<string>) -> string */ 

kk_std_core_types__tuple2_ kk_std_core__unroll17352_lift17325_unzip(kk_std_core__list ys, kk_std_core__list acc1, kk_std_core__list acc2, kk_context_t* _ctx); /* forall<a,b> (ys : list<(a, b)>, acc1 : list<a>, acc2 : list<b>) -> (list<a>, list<b>) */ 
 
// unrolling of singleton matches of std/core/.unroll17352-lift17325-unzip

static inline kk_std_core_types__tuple2_ kk_std_core__lift17325_unzip(kk_std_core__list ys, kk_std_core__list acc1, kk_std_core__list acc2, kk_context_t* _ctx) { /* forall<a,b> (ys : list<(a, b)>, acc1 : list<a>, acc2 : list<b>) -> (list<a>, list<b>) */ 
  if (kk_std_core__is_Nil(ys, _ctx)) {
    kk_std_core__list _b_20573_20571;
    if (kk_std_core__is_Nil(acc1, _ctx)) {
      _b_20573_20571 = kk_std_core__new_Nil(_ctx); /*list<16974>*/
    }
    else {
      _b_20573_20571 = kk_std_core__unroll17327_lift17313_reverse_append(kk_std_core__new_Nil(_ctx), acc1, _ctx); /*list<16974>*/
    }
    kk_std_core__list _b_20574_20572;
    if (kk_std_core__is_Nil(acc2, _ctx)) {
      _b_20574_20572 = kk_std_core__new_Nil(_ctx); /*list<16975>*/
    }
    else {
      _b_20574_20572 = kk_std_core__unroll17327_lift17313_reverse_append(kk_std_core__new_Nil(_ctx), acc2, _ctx); /*list<16975>*/
    }
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_core__list_box(_b_20573_20571, _ctx), kk_std_core__list_box(_b_20574_20572, _ctx), _ctx);
  }
  {
    return kk_std_core__unroll17352_lift17325_unzip(ys, acc1, acc2, _ctx);
  }
}
 
// Unzip a list of pairs into two lists

static inline kk_std_core_types__tuple2_ kk_std_core_unzip(kk_std_core__list xs, kk_context_t* _ctx) { /* forall<a,b> (xs : list<(a, b)>) -> (list<a>, list<b>) */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx), kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx), _ctx);
  }
  {
    return kk_std_core__unroll17352_lift17325_unzip(xs, kk_std_core__new_Nil(_ctx), kk_std_core__new_Nil(_ctx), _ctx);
  }
}

kk_vector_t kk_std_core_vector_1(kk_string_t s, kk_context_t* _ctx); /* (s : string) -> vector<char> */ 
 
// Create a new vector of length `n`  with initial elements `default` .


// lift anonymous function
struct kk_std_core_vector_fun22253__t_2 {
  struct kk_function_s _base;
  kk_box_t default0;
};
extern kk_box_t kk_std_core_vector_fun22253_2(kk_function_t _fself, kk_ssize_t ___wildcard__1859__30, kk_context_t* _ctx);
static inline kk_function_t kk_std_core_new_vector_fun22253_2(kk_box_t default0, kk_context_t* _ctx) {
  struct kk_std_core_vector_fun22253__t_2* _self = kk_function_alloc_as(struct kk_std_core_vector_fun22253__t_2, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_core_vector_fun22253_2, kk_context());
  _self->default0 = default0;
  return kk_datatype_from_base(&_self->_base, kk_context());
}


static inline kk_vector_t kk_std_core_vector_2(kk_integer_t n, kk_box_t default0, kk_context_t* _ctx) { /* forall<a> (n : int, default : a) -> vector<a> */ 
  kk_ssize_t _x22251;
  kk_integer_t _x22252 = kk_integer_dup(n, _ctx); /*int*/
  _x22251 = kk_std_core_ssize__t(_x22252, _ctx); /*ssize_t*/
  return kk_std_core_vector_initz(_x22251, kk_std_core_new_vector_fun22253_2(default0, _ctx), _ctx);
}
 
// Convert a list to a vector.

static inline kk_vector_t kk_std_core_vector_3(kk_std_core__list xs, kk_context_t* _ctx) { /* forall<a> (xs : list<a>) -> vector<a> */ 
  return kk_std_core_unvlist(xs, _ctx);
}
 
// Create a new vector of length `n`  with initial elements given by function `f` .


// lift anonymous function
struct kk_std_core_vector_init_fun22256__t {
  struct kk_function_s _base;
  kk_function_t f;
};
extern kk_box_t kk_std_core_vector_init_fun22256(kk_function_t _fself, kk_ssize_t i, kk_context_t* _ctx);
static inline kk_function_t kk_std_core_new_vector_init_fun22256(kk_function_t f, kk_context_t* _ctx) {
  struct kk_std_core_vector_init_fun22256__t* _self = kk_function_alloc_as(struct kk_std_core_vector_init_fun22256__t, 2, _ctx);
  _self->_base.fun = kk_kkfun_ptr_box(&kk_std_core_vector_init_fun22256, kk_context());
  _self->f = f;
  return kk_datatype_from_base(&_self->_base, kk_context());
}


static inline kk_vector_t kk_std_core_vector_init(kk_integer_t n, kk_function_t f, kk_context_t* _ctx) { /* forall<a> (n : int, f : (int) -> a) -> vector<a> */ 
  kk_ssize_t _x22254;
  kk_integer_t _x22255 = kk_integer_dup(n, _ctx); /*int*/
  _x22254 = kk_std_core_ssize__t(_x22255, _ctx); /*ssize_t*/
  return kk_std_core_vector_initz(_x22254, kk_std_core_new_vector_init_fun22256(f, _ctx), _ctx);
}

kk_unit_t kk_std_core__mlift18852_while(kk_function_t action, kk_function_t predicate, kk_unit_t wild__, kk_context_t* _ctx); /* forall<e> (action : () -> <div|e> (), predicate : () -> <div|e> bool, wild_ : ()) -> <div|e> () */ 

kk_unit_t kk_std_core__mlift18853_while(kk_function_t action0, kk_function_t predicate0, bool _y_18711, kk_context_t* _ctx); /* forall<e> (action : () -> <div|e> (), predicate : () -> <div|e> bool, bool) -> <div|e> () */ 

kk_unit_t kk_std_core_while(kk_function_t predicate1, kk_function_t action1, kk_context_t* _ctx); /* forall<e> (predicate : () -> <div|e> bool, action : () -> <div|e> ()) -> <div|e> () */ 

kk_std_core__list kk_std_core__trmc_unroll17353_zip(kk_std_core__list xs, kk_std_core__list ys, kk_std_core_types__cctx _acc, kk_context_t* _ctx); /* forall<a,b> (xs : list<a>, ys : list<b>, ctx<list<(a, b)>>) -> list<(a, b)> */ 

kk_std_core__list kk_std_core__unroll17353_zip(kk_std_core__list xs0, kk_std_core__list ys0, kk_context_t* _ctx); /* forall<a,b> (xs : list<a>, ys : list<b>) -> list<(a, b)> */ 
 
// unrolling of singleton matches of std/core/.unroll17353-zip

static inline kk_std_core__list kk_std_core_zip(kk_std_core__list xs, kk_std_core__list ys, kk_context_t* _ctx) { /* forall<a,b> (xs : list<a>, ys : list<b>) -> list<(a, b)> */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    kk_std_core__list_drop(ys, _ctx);
    return kk_std_core__new_Nil(_ctx);
  }
  {
    return kk_std_core__unroll17353_zip(xs, ys, _ctx);
  }
}

kk_std_core__list kk_std_core__mlift18854_trmc_unroll17354_zipwith(kk_std_core_types__cctx _acc, kk_function_t f, kk_std_core__list xx, kk_std_core__list yy, kk_box_t _trmc_18427, kk_context_t* _ctx); /* forall<a,b,c,e> (ctx<list<c>>, f : (a, b) -> e c, xx : list<a>, yy : list<b>, c) -> e list<c> */ 

kk_std_core__list kk_std_core__trmc_unroll17354_zipwith(kk_std_core__list xs, kk_std_core__list ys, kk_function_t f0, kk_std_core_types__cctx _acc0, kk_context_t* _ctx); /* forall<a,b,c,e> (xs : list<a>, ys : list<b>, f : (a, b) -> e c, ctx<list<c>>) -> e list<c> */ 

kk_std_core__list kk_std_core__unroll17354_zipwith(kk_std_core__list xs0, kk_std_core__list ys0, kk_function_t f1, kk_context_t* _ctx); /* forall<a,b,c,e> (xs : list<a>, ys : list<b>, f : (a, b) -> e c) -> e list<c> */ 
 
// unrolling of singleton matches of std/core/.unroll17354-zipwith

static inline kk_std_core__list kk_std_core_zipwith(kk_std_core__list xs, kk_std_core__list ys, kk_function_t f, kk_context_t* _ctx) { /* forall<a,b,c,e> (xs : list<a>, ys : list<b>, f : (a, b) -> e c) -> e list<c> */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    kk_std_core__list_drop(ys, _ctx);
    kk_function_drop(f, _ctx);
    return kk_std_core__new_Nil(_ctx);
  }
  {
    return kk_std_core__unroll17354_zipwith(xs, ys, f, _ctx);
  }
}

kk_std_core__list kk_std_core__mlift18855_trmc_unroll17355_lift17326_zipwith_indexed(kk_std_core_types__cctx _acc, kk_function_t f, kk_integer_t i0_17863, kk_std_core__list xx, kk_std_core__list yy, kk_box_t _trmc_18430, kk_context_t* _ctx); /* forall<a,b,c,e> (ctx<list<c>>, f : (int, a, b) -> e c, i0.17863 : int, xx : list<a>, yy : list<b>, c) -> e list<c> */ 

kk_std_core__list kk_std_core__trmc_unroll17355_lift17326_zipwith_indexed(kk_function_t f0, kk_integer_t i, kk_std_core__list xs, kk_std_core__list ys, kk_std_core_types__cctx _acc0, kk_context_t* _ctx); /* forall<a,b,c,e> (f : (int, a, b) -> e c, i : int, xs : list<a>, ys : list<b>, ctx<list<c>>) -> e list<c> */ 

kk_std_core__list kk_std_core__unroll17355_lift17326_zipwith_indexed(kk_function_t f1, kk_integer_t i0, kk_std_core__list xs0, kk_std_core__list ys0, kk_context_t* _ctx); /* forall<a,b,c,e> (f : (int, a, b) -> e c, i : int, xs : list<a>, ys : list<b>) -> e list<c> */ 
 
// unrolling of singleton matches of std/core/.unroll17355-lift17326-zipwith-indexed

static inline kk_std_core__list kk_std_core__lift17326_zipwith_indexed(kk_function_t f, kk_integer_t i, kk_std_core__list xs, kk_std_core__list ys, kk_context_t* _ctx) { /* forall<a,b,c,e> (f : (int, a, b) -> e c, i : int, xs : list<a>, ys : list<b>) -> e list<c> */ 
  if (kk_std_core__is_Nil(xs, _ctx)) {
    kk_std_core__list_drop(ys, _ctx);
    kk_integer_drop(i, _ctx);
    kk_function_drop(f, _ctx);
    return kk_std_core__new_Nil(_ctx);
  }
  {
    return kk_std_core__unroll17355_lift17326_zipwith_indexed(f, i, xs, ys, _ctx);
  }
}
 
// Zip two lists together by apply a function `f` to all corresponding elements
// and their index in the list.
// The returned list is only as long as the smallest input list.

static inline kk_std_core__list kk_std_core_zipwith_indexed(kk_std_core__list xs0, kk_std_core__list ys0, kk_function_t f, kk_context_t* _ctx) { /* forall<a,b,c,e> (xs0 : list<a>, ys0 : list<b>, f : (int, a, b) -> e c) -> e list<c> */ 
  if (kk_std_core__is_Nil(xs0, _ctx)) {
    kk_std_core__list_drop(ys0, _ctx);
    kk_function_drop(f, _ctx);
    return kk_std_core__new_Nil(_ctx);
  }
  {
    return kk_std_core__unroll17355_lift17326_zipwith_indexed(f, kk_integer_from_small(0), xs0, ys0, _ctx);
  }
}

void kk_std_core__init(kk_context_t* _ctx);


void kk_std_core__done(kk_context_t* _ctx);

#endif // header
